@@ -29,9 +29,9 @@
 #include <gpac/network.h>
 #include <gpac/utf.h>
 
-
-
-
+#ifndef _WIN32_WCE
+#include <mshtml.h>
+#endif
 
 /////////////////////////////////////////////////////////////////////////////
 // CGPAXPlugin
@@ -54,7 +54,7 @@ void CGPAXPlugin::SetStatusText(char *msg)
 		if (msg) {
 			u16 w_msg[1024];
 			gf_utf8_mbstowcs(w_msg, 1024, (const char **)&msg);
-			m_pBrowser->put_StatusText(w_msg);
+			m_pBrowser->put_StatusText((BSTR) w_msg);
 		} else {
 			m_pBrowser->put_StatusText(L"");
 		}
@@ -101,7 +101,7 @@ Bool CGPAXPlugin::EventProc(GF_Event *evt)
 		m_height = evt->size.height;
         gf_term_set_size(m_term, m_width, m_height);
         break;
-	case GF_EVENT_MOUSEDOUBLECLICK:
+	case GF_EVENT_DBLCLICK:
 		gf_term_set_option(m_term, GF_OPT_FULLSCREEN, !gf_term_get_option(m_term, GF_OPT_FULLSCREEN));
 		break;
 	case GF_EVENT_KEYDOWN:
@@ -143,12 +143,12 @@ Bool CGPAXPlugin::EventProc(GF_Event *evt)
 				else if (!strnicmp(evt->navigate.parameters[i], "_target=", 8)) {
 					sz_ptr = & evt->navigate.parameters[i]+8;
 					gf_utf8_mbstowcs(w_szTar, 1024, (const char **)sz_ptr);
-					target.bstrVal = w_szTar;
+					target.bstrVal = (BSTR) w_szTar;
 				}
 			}
 			sz_ptr = & evt->navigate.to_url;
 			gf_utf8_mbstowcs(w_szURL, 1024, (const char **)sz_ptr);
-			m_pBrowser->Navigate(w_szURL, &flags, &target, NULL, NULL);;
+			m_pBrowser->Navigate((BSTR) w_szURL, &flags, &target, NULL, NULL);;
 			return 1;
 		}
 #endif
@@ -180,7 +180,7 @@ Bool CGPAXPlugin::ReadParamString(LPPROPERTYBAG pPropBag, LPERRORLOG pErrorLog,
         {
 //            USES_CONVERSION;
 //            lstrcpyn(buf,OLE2T(v.bstrVal),bufsize);
-			const u16 *srcp = v.bstrVal;
+			const u16 *srcp = (const u16 *) v.bstrVal;
 			u32 len = gf_utf8_wcstombs(buf, bufsize, &srcp);
 			if (len>=0) {
 				buf[len] = 0;
@@ -192,6 +192,70 @@ Bool CGPAXPlugin::ReadParamString(LPPROPERTYBAG pPropBag, LPERRORLOG pErrorLog,
     return retval;
 }
 
+void CGPAXPlugin::LoadDATAUrl()
+{
+#ifndef _WIN32_WCE
+	HRESULT hr;
+
+	if (m_url[0]) return;
+    /*get parent doc*/
+	CComPtr<IOleContainer> spContainer;
+	if (m_spClientSite->GetContainer(&spContainer) != S_OK)
+		return;
+    CComPtr<IHTMLDocument2> spDoc = CComQIPtr<IHTMLDocument2>(spContainer);
+    CComPtr<IHTMLElementCollection> spColl;
+	if (spDoc->get_all(&spColl) != S_OK)
+		return;
+	/*get HTML <object> in the doc*/
+    CComPtr<IDispatch> spDisp;
+	CComPtr<IHTMLElementCollection> sphtmlObjects;
+
+    CComPtr<IDispatch> spDispObjects;
+	if (spColl->tags(CComVariant("OBJECT"), &spDispObjects) != S_OK)
+		return;
+    CComPtr<IHTMLElementCollection> spObjs = CComQIPtr<IHTMLElementCollection>(spDispObjects);
+	
+	/*browse all objects and find us*/
+	long lCount = 0;
+	spObjs->get_length(&lCount);
+	for (long lCnt = 0; lCnt < lCount; lCnt++) {   
+		IDispatch *an_obj= NULL;
+		CComVariant varEmpty;
+		CComVariant varName;
+		varName.vt = VT_I4;
+		varName.lVal = lCnt;
+		hr = spObjs->item(varName, varEmpty, &an_obj);
+		varName.Clear();
+		varEmpty.Clear();
+		if (hr != S_OK) continue;
+
+		/*get the IHTMLObject*/
+		IHTMLObjectElement* pObjectElem=NULL;
+		an_obj->QueryInterface(IID_IHTMLObjectElement, (void**)&pObjectElem);
+		if (!pObjectElem) continue;
+
+		/*get its parent owner - it MUST be us*/
+		IDispatch *disp= NULL;
+		pObjectElem->get_object(&disp);
+		if (disp != this) continue;
+
+		BSTR data = NULL;
+		if ((pObjectElem->get_data(&data) == S_OK) && data) {
+			const u16 *srcp = (const u16 *) data;
+			u32 len = gf_utf8_wcstombs(m_url, MAXLEN_URL, &srcp);
+			if (len>=0) m_url[len] = 0;
+		}
+		SysFreeString(data);
+		break;
+	}
+
+	if (m_url) {
+		UpdateURL();
+	}
+#endif
+
+}
+
 //Create window message fuction. when the window is created, also initialize a instance of
 //GPAC player instance.
 LRESULT CGPAXPlugin::OnCreate(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
@@ -203,7 +267,9 @@ LRESULT CGPAXPlugin::OnCreate(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHa
     const char *str;
 
     if (m_hWnd==NULL) return 0;
-    gpac_cfg = "GPAC.cfg";
+
+	
+	gpac_cfg = "GPAC.cfg";
 
 #if defined(_DEBUG) && !defined(_WIN32_WCE)
 	strcpy((char *) config_path, "D:\\cvs\\gpac\\bin\\w32_deb\\");
@@ -252,11 +318,10 @@ LRESULT CGPAXPlugin::OnCreate(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHa
 			goto err_exit;
 		}
 		gf_cfg_set_key(m_user.config, "General", "ModulesDirectory", (const char *) config_path);
-		gf_cfg_set_key(m_user.config, "Rendering", "Raster2D", "GPAC 2D Raster");
+		gf_cfg_set_key(m_user.config, "Compositor", "Raster2D", "GPAC 2D Raster");
 		sprintf((char *) cfg_file, "%s\\cache", config_path);
 		gf_cfg_set_key(m_user.config, "General", "CacheDirectory", (const char *) cfg_file);
 		gf_cfg_set_key(m_user.config, "Network", "AutoReconfigUDP", "no");
-		gf_cfg_set_key(m_user.config, "Network", "UDPNotAvailable", "no");
 		gf_cfg_set_key(m_user.config, "Network", "BufferLength", "0");
 		gf_cfg_set_key(m_user.config, "Audio", "ForceConfig", "yes");
 		gf_cfg_set_key(m_user.config, "Audio", "NumBuffers", "2");
@@ -282,12 +347,6 @@ LRESULT CGPAXPlugin::OnCreate(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHa
     m_user.opaque = this;
     m_user.EventProc = GPAX_EventProc;
 
-	if (strstr(m_url, ".wrl") || strstr(m_url, ".x3d") || strstr(m_url, ".x3dv")) {
-		gf_cfg_set_key(m_user.config, "Rendering", "RendererName", "GPAC 3D Renderer");
-	} else {
-		gf_cfg_set_key(m_user.config, "Rendering", "RendererName", m_bUse3D ? "GPAC 3D Renderer" : "GPAC 2D Renderer");
-	}
-
 	//create a terminal
     m_term = gf_term_new(&m_user);
 
@@ -295,6 +354,8 @@ LRESULT CGPAXPlugin::OnCreate(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHa
 	
 	gf_term_set_option(m_term, GF_OPT_AUDIO_VOLUME, 100);
     
+	LoadDATAUrl();
+
 	RECT rc;
     ::GetWindowRect(m_hWnd, &rc);
     m_width = rc.right-rc.left;
@@ -355,7 +416,10 @@ STDMETHODIMP CGPAXPlugin::Load(LPPROPERTYBAG pPropBag, LPERRORLOG pErrorLog)
 	char szOpt[1024];
     // examine the <embed>/<param> tag arguments
 
+	m_url[0] = 0;
     ReadParamString(pPropBag,pErrorLog,L"src", m_url, MAXLEN_URL);
+	if (!m_url[0])
+	    ReadParamString(pPropBag,pErrorLog,L"data", m_url, MAXLEN_URL);
 
     if (ReadParamString(pPropBag,pErrorLog,L"autostart", szOpt, 1024))
 		m_bAutoStart = (!stricmp(szOpt, "false") || !stricmp(szOpt, "no")) ? 0 : 1;
@@ -485,15 +549,15 @@ STDMETHODIMP CGPAXPlugin::Update(BSTR _mtype, BSTR _updates)
 		u32 len;
 		char mtype[1024], *updates;
 
-		srcp = _mtype;
+		srcp = (u16 *) _mtype;
 		len = gf_utf8_wcstombs(mtype, 1024, (const u16 **)&srcp);
 		mtype[len] = 0;
 
-		srcp = _updates;
+		srcp = (u16 *)_updates;
 		len = gf_utf8_wcstombs(NULL, 0, (const u16 **)&srcp);
 		if (len) {
 			updates = (char *) malloc(sizeof(char) * (len+1));
-			srcp = _updates;
+			srcp = (u16 *)_updates;
 			len = gf_utf8_wcstombs(updates, len, (const u16 **)&srcp);
 			updates[len] = 0;
 			gf_term_scene_update(m_term, mtype, updates);
@@ -519,7 +583,7 @@ STDMETHODIMP CGPAXPlugin::get_src(BSTR *url)
 }
 STDMETHODIMP CGPAXPlugin::put_src(BSTR url)
 {
-	const u16 *srcp = url;
+	const u16 *srcp = (const u16 *)url;
 	u32 len = gf_utf8_wcstombs(m_url, MAXLEN_URL, &srcp);
 	m_url[len] = 0;
 	UpdateURL();
