@@ -804,6 +804,7 @@ void WriteNodeCode(GF_List *BNodes)
 	fprintf(f, "\n#include <gpac/internal/scenegraph_dev.h>\n");
 
 	for (k=0; k<gf_list_count(BNodes); k++) {
+		Bool is_parent = 0;
 		n = gf_list_get(BNodes, k);
 
 		if (n->skip_impl) continue;
@@ -819,7 +820,7 @@ void WriteNodeCode(GF_List *BNodes)
 			
 			//delete all children node
 			if (!strcmp(bf->name, "children") && stricmp(n->name, "audioBuffer")) {
-				fprintf(f, "\tgf_sg_vrml_parent_destroy((GF_Node *) p);\t\n");
+				is_parent = 1;
 				continue;
 			}
 
@@ -863,6 +864,7 @@ void WriteNodeCode(GF_List *BNodes)
 				}
 			}
 		}
+		if (is_parent) fprintf(f, "\tgf_sg_vrml_parent_destroy((GF_Node *) p);\t\n");
 		fprintf(f, "\tgf_node_free((GF_Node *) p);\n}\n\n");
 
 		//node fields
@@ -899,14 +901,6 @@ void WriteNodeCode(GF_List *BNodes)
 			}
 		}
 
-		//setup pointers
-		fprintf(f, "\n#ifdef GF_NODE_USE_POINTERS\n");
-		fprintf(f, "\t((GF_Node *)p)->sgprivate->name = \"%s\";\n", n->name);
-		fprintf(f, "\t((GF_Node *)p)->sgprivate->node_del = %s_Del;\n", n->name);
-		fprintf(f, "\t((GF_Node *)p)->sgprivate->get_field_count = %s_get_field_count;\n", n->name);
-		fprintf(f, "\t((GF_Node *)p)->sgprivate->get_field = %s_get_field;\n", n->name);
-		fprintf(f, "#endif\n\n");
-
 		//check if we have a child node
 		for (i=0; i<gf_list_count(n->Fields); i++) {
 			bf = gf_list_get(n->Fields, i);
@@ -1237,8 +1231,6 @@ void WriteNodeCode(GF_List *BNodes)
 	}
 	fprintf(f, "\tdefault:\n\t\treturn NULL;\n\t}\n}\n\n");
 
-	fprintf(f, "#ifndef GF_NODE_USE_POINTERS\n");
-
 	fprintf(f, "const char *gf_sg_mpeg4_node_get_class_name(u32 NodeTag)\n{\n\tswitch (NodeTag) {\n");
 	for (i=0; i<gf_list_count(BNodes); i++) {
 		n = gf_list_get(BNodes, i);
@@ -1273,8 +1265,6 @@ void WriteNodeCode(GF_List *BNodes)
 	}
 	fprintf(f, "\tdefault:\n\t\treturn GF_BAD_PARAM;\n\t}\n}\n\n");
 
-	fprintf(f, "\n\n#endif\n\n");
-
 	fprintf(f, "GF_Err gf_sg_mpeg4_node_get_field_index(GF_Node *node, u32 inField, u8 code_mode, u32 *fieldIndex)\n{\n\tswitch (node->sgprivate->tag) {\n");
 	for (i=0; i<gf_list_count(BNodes); i++) {
 		n = gf_list_get(BNodes, i);
