@@ -352,6 +352,7 @@ void WriteNodeCode(GF_List *BNodes, FILE *vrml_code)
 	fprintf(vrml_code, "\n/*for NDT tag definitions*/\n#include <gpac/nodes_mpeg4.h>\n");
 
 	for (k=0; k<gf_list_count(BNodes); k++) {
+		Bool is_parent = 0;
 		n = gf_list_get(BNodes, k);
 		if (n->skip_impl) continue;
 		fprintf(vrml_code, "\n/*\n\t%s Node deletion\n*/\n\n", n->name);
@@ -365,7 +366,7 @@ void WriteNodeCode(GF_List *BNodes, FILE *vrml_code)
 			
 			//delete all children node
 			if (strcmp(bf->type, "eventOut") && !strcmp(bf->name, "children")) {
-				fprintf(vrml_code, "\tgf_sg_vrml_parent_destroy(node);\t\n");
+				is_parent = 1;
 				continue;
 			}
 
@@ -406,6 +407,8 @@ void WriteNodeCode(GF_List *BNodes, FILE *vrml_code)
 				}
 			}
 		}
+		if (is_parent)
+			fprintf(vrml_code, "\tgf_sg_vrml_parent_destroy(node);\t\n");
 		/*avoids gcc warnings in case no field to delete*/
 		fprintf(vrml_code, "\tgf_node_free((GF_Node *)p);\n}\n\n");
 
@@ -440,25 +443,6 @@ void WriteNodeCode(GF_List *BNodes, FILE *vrml_code)
 			}
 		}
 
-		//setup pointers
-		fprintf(vrml_code, "\n#ifdef GF_NODE_USE_POINTERS\n\n");
-		fprintf(vrml_code, "\t((GF_Node *)p)->sgprivate->name = \"%s\";\n", n->name);
-		fprintf(vrml_code, "\t((GF_Node *)p)->sgprivate->node_del = %s_Del;\n", n->name);
-		fprintf(vrml_code, "\t((GF_Node *)p)->sgprivate->get_field_count = %s_get_field_count;\n", n->name);
-		fprintf(vrml_code, "\t((GF_Node *)p)->sgprivate->get_field = %s_get_field;\n", n->name);
-
-		//check if we have a child node
-		for (i=0; i<gf_list_count(n->Fields); i++) {
-			bf = gf_list_get(n->Fields, i);
-			if ( !strcmp(bf->name, "children") || 
-					( !strstr(bf->type, "event") && strstr(bf->familly, "MF") && strstr(bf->familly, "Node")) ) {
-
-				sprintf(n->Child_NDT_Name, "NDT_SF%s", bf->familly+2);
-				break;
-			}
-		}
-		fprintf(vrml_code, "\n#endif\n\n");
-
 		fprintf(vrml_code, "\n\t/*default field values*/\n");
 		
 		for (i=0; i<gf_list_count(n->Fields); i++) {
@@ -873,8 +857,6 @@ void WriteNodeCode(GF_List *BNodes, FILE *vrml_code)
 	}
 	fprintf(vrml_code, "\tdefault:\n\t\treturn NULL;\n\t}\n}\n\n");
 
-	fprintf(vrml_code, "#ifndef GF_NODE_USE_POINTERS\n");
-
 	fprintf(vrml_code, "const char *gf_sg_x3d_node_get_class_name(u32 NodeTag)\n{\n\tswitch (NodeTag) {\n");
 	for (i=0; i<gf_list_count(BNodes); i++) {
 		n = gf_list_get(BNodes, i);
@@ -903,8 +885,6 @@ void WriteNodeCode(GF_List *BNodes, FILE *vrml_code)
 	}
 	fprintf(vrml_code, "\tdefault:\n\t\treturn GF_BAD_PARAM;\n\t}\n}\n\n");
 
-	fprintf(vrml_code, "\n#endif\n\n");
-
 	fprintf(vrml_code, "\nu32 gf_node_x3d_type_by_class_name(const char *node_name)\n{\n\tif(!node_name) return 0;\n");
 	for (i=0; i<gf_list_count(BNodes); i++) {
 		n = gf_list_get(BNodes, i);
@@ -983,7 +963,7 @@ void WriteNDT(FILE *f, GF_List *XNodes, GF_List *NDTs)
 	}
 
 	//NodeTag complete translation
-	fprintf(f, "\n\n\nBool X3D_IsNodeInTable(u32 NDT_Tag, u32 NodeTag)\n{\n\tconst u32 *types;\n\tu32 count, i;\n\tif (!NodeTag) return 0;\n\ttypes = NULL; count = 0;\n");
+	fprintf(f, "\n\n\nBool gf_x3d_get_node_type(u32 NDT_Tag, u32 NodeTag)\n{\n\tconst u32 *types;\n\tu32 count, i;\n\tif (!NodeTag) return 0;\n\ttypes = NULL; count = 0;\n");
 
 	fprintf(f, "\tswitch (NDT_Tag) {\n");
 	for (i=0; i<gf_list_count(NDTs); i++) {
