@@ -24,7 +24,6 @@
 
 #include <gpac/scene_manager.h>
 #include <gpac/nodes_x3d.h>
-#include <gpac/nodes_svg_sa.h>
 #include <gpac/internal/bifs_dev.h>
 #include <gpac/constants.h>
 #include <gpac/avparse.h>
@@ -266,7 +265,7 @@ static void dump_stats(FILE *dump, GF_SceneStatistics *stats)
 		fprintf(dump, "</NodeStat>\n");
 	}
 	if (i) {
-		fprintf(dump, "<CumulatedStat TotalNumberOfNodes=\"%d\" ReallyAllocatedNodes=\"%d\" DeletedNodes=\"%d\"/>\n", count, created, deleted);
+		fprintf(dump, "<CumulatedStat TotalNumberOfNodes=\"%d\" ReallyAllocatedNodes=\"%d\" DeletedNodes=\"%d\" NumberOfAttributes=\"%d\"/>\n", count, created, deleted, stats->nb_svg_attributes);
 		fprintf(dump, "<DrawableNodesCumulatedStat TotalNumberOfNodes=\"%d\" ReallyAllocatedNodes=\"%d\" DeletedNodes=\"%d\"/>\n", draw_count, draw_created, draw_deleted);
 	}
 	fprintf(dump, "</NodeStatistics>\n");
@@ -290,7 +289,7 @@ static void dump_stats(FILE *dump, GF_SceneStatistics *stats)
 	fprintf(dump, "<Resolutions scaleIntegerPart=\"%d\" scaleFracPart=\"%d\" coordIntegerPart=\"%d\" coordFracPart=\"%d\"/>\n", stats->scale_int_res_2d, stats->scale_frac_res_2d, stats->int_res_2d, stats->frac_res_2d);
 	fprintf(dump, "</FixedValues>\n");
 	fprintf(dump, "<FieldStatistic FieldType=\"MFVec2f\">\n");
-	fprintf(dump, "<ParsingInfo NumParsed=\"%d\" NumRemoved=\"%d\"/>", stats->count_2d, stats->rem_2d);
+	fprintf(dump, "<ParsingInfo NumParsed=\"%d\" NumRemoved=\"%d\"/>\n", stats->count_2d, stats->rem_2d);
 	if (stats->count_2d) {
 		fprintf(dump, "<ExtendInfo MinVec2f=\"%f %f\" MaxVec2f=\"%f %f\"/>\n", FIX2FLT( stats->min_2d.x) , FIX2FLT( stats->min_2d.y ), FIX2FLT( stats->max_2d.x ), FIX2FLT( stats->max_2d.y ) );
 	}
@@ -391,7 +390,7 @@ void dump_scene_stats(char *file, char *inName, u32 stat_level)
 	fprintf(stdout, "Analysing Scene\n");
 
 	fprintf(dump, "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n");
-	fprintf(dump, "<!-- Scene Graph Statistics Generated by MP4Box - GPAC " GPAC_VERSION" -->\n");
+	fprintf(dump, "<!-- Scene Graph Statistics Generated by MP4Box - GPAC " GPAC_FULL_VERSION" -->\n");
 	fprintf(dump, "<SceneStatistics file=\"%s\" DumpType=\"%s\">\n", file, (stat_level==1) ? "full scene" : ((stat_level==2) ? "AccessUnit based" : "SceneGraph after each AU"));
 
 	sm = gf_sm_stats_new();
@@ -519,9 +518,28 @@ void PrintNodeSFField(u32 type, void *far_ptr)
 		break;
 	}
 }
+
+static Bool node_in_table_by_tag(u32 tag, u32 NDTType)
+{
+	if (!tag) return 0;
+	if (tag==TAG_ProtoNode) return 1;
+	else if (tag<=GF_NODE_RANGE_LAST_MPEG4) {
+		u32 i;
+
+		for (i=0;i<GF_BIFS_LAST_VERSION; i++) {
+			if (gf_bifs_get_node_type(NDTType, tag, i+1)) return 1;
+		}
+		return 0;
+	} else if (tag<=GF_NODE_RANGE_LAST_X3D) {
+		return gf_x3d_get_node_type(NDTType, tag);
+	}
+	return 0;
+}
+
 void PrintNode(const char *name, u32 graph_type)
 {
 	const char *nname, *std_name;
+	char szField[1024];
 	GF_Node *node;
 	GF_SceneGraph *sg;
 	u32 tag, nbF, i;
@@ -529,13 +547,19 @@ void PrintNode(const char *name, u32 graph_type)
 	u8 qt, at;
 	Fixed bmin, bmax;
 	u32 nbBits;
+	Bool is_nodefield = 0;
+
+	char *sep = strchr(name, '.');
+	if (sep) {
+		strcpy(szField, sep+1);
+		sep[0] = 0;
+		is_nodefield = 1;
+	}
 
 	tag = 0;
 	if (graph_type==2) {
-#ifndef GPAC_DISABLE_SVG
-		tag = gf_svg_get_element_tag(name);
-#endif
-		std_name = "SVG";
+		fprintf(stdout, "SVG node printing is not supported\n");
+		return;
 	} else if (graph_type==1) {
 		tag = gf_node_x3d_type_by_class_name(name);
 		std_name = "X3D";
@@ -558,12 +582,36 @@ void PrintNode(const char *name, u32 graph_type)
 	}
 	nbF = gf_node_get_field_count(node);
 
-	if (graph_type==2) {
-		fprintf(stdout, "SVG Element Syntax:\n");
-		fprintf(stdout, "<%s ", nname);
-	} else {
-		fprintf(stdout, "Node Syntax:\n%s {\n", nname);
+	if (is_nodefield) {
+		u32 tfirst, tlast;
+		if (gf_node_get_field_by_name(node, szField, &f) != GF_OK) {
+			fprintf(stdout, "Field %s is not a member of node %s\n", szField, name);
+			return;
+		}
+		fprintf(stdout, "Allowed nodes in %s.%s:\n", name, szField);
+		if (graph_type==1) {
+			tfirst = GF_NODE_RANGE_FIRST_X3D;
+			tlast = GF_NODE_RANGE_LAST_X3D;
+		} else {
+			tfirst = GF_NODE_RANGE_FIRST_MPEG4;
+			tlast = GF_NODE_RANGE_LAST_MPEG4;
+		}
+		for (i=tfirst; i<tlast;i++) {
+			GF_Node *tmp = gf_node_new(sg, i);
+			gf_node_register(tmp, NULL);
+			if (node_in_table_by_tag(i, f.NDTtype)) {
+				const char *nname = gf_node_get_class_name(tmp);
+				if (nname && strcmp(nname, "Unknown Node")) {
+					fprintf(stdout, "\t%s\n", nname); 
+				}
+			}
+			gf_node_unregister(tmp, NULL);
+		}
+		return;
 	}
+
+	fprintf(stdout, "Node Syntax:\n%s {\n", nname);
+
 	for (i=0; i<nbF; i++) {
 		gf_node_get_field(node, i, &f);
 		if (graph_type==2) {
@@ -605,11 +653,7 @@ void PrintNode(const char *name, u32 graph_type)
 		}
 		fprintf(stdout, "\n");
 	}
-	if (graph_type==2) {
-		fprintf(stdout, "/>\n%d possible attributes\n",nbF);
-	} else {
-		fprintf(stdout, "}\n\n");
-	}
+	fprintf(stdout, "}\n\n");
 
 	gf_node_unregister(node, NULL);
 	gf_sg_del(sg);
@@ -945,7 +989,7 @@ static void DumpMetaItem(GF_ISOFile *file, Bool root_meta, u32 tk_num, char *nam
 void DumpTrackInfo(GF_ISOFile *file, u32 trackID, Bool full_dump)
 {
 	Float scale;
-	u32 trackNum, j, size, max_rate, rate, ts, mtype, msub_type, timescale, sr, nb_ch, count;
+	u32 trackNum, i, j, size, max_rate, rate, ts, mtype, msub_type, timescale, sr, nb_ch, count, alt_group, nb_groups;
 	u64 time_slice, dur;
 	u8 bps;
 	GF_ESD *esd;
@@ -960,6 +1004,7 @@ void DumpTrackInfo(GF_ISOFile *file, u32 trackID, Bool full_dump)
 	timescale = gf_isom_get_media_timescale(file, trackNum);
 	fprintf(stdout, "Track # %d Info - TrackID %d - TimeScale %d - Duration %s\n", trackNum, trackID, timescale, format_duration(gf_isom_get_media_duration(file, trackNum), timescale, szDur));
 	if (gf_isom_is_track_in_root_od(file, trackNum) ) fprintf(stdout, "Track is present in Root OD\n");
+	if (!gf_isom_is_track_enabled(file, trackNum))  fprintf(stdout, "Track is disabled\n");
 	gf_isom_get_media_language(file, trackNum, sType);
 	fprintf(stdout, "Media Info: Language \"%s\" - ", GetLanguage(sType) );
 	mtype = gf_isom_get_media_type(file, trackNum);
@@ -1041,11 +1086,11 @@ void DumpTrackInfo(GF_ISOFile *file, u32 trackID, Bool full_dump)
 					}
 				} 
 				/*OGG media*/
-				else if (esd->decoderConfig->objectTypeIndication==GPAC_OGG_MEDIA_OTI) {
+				else if (esd->decoderConfig->objectTypeIndication==GPAC_OTI_MEDIA_OGG) {
 					char *szName;
 					gf_isom_get_visual_info(file, trackNum, 1, &w, &h);
 					if (full_dump) fprintf(stdout, "\t");
-					if (!strnicmp(&esd->decoderConfig->decoderSpecificInfo->data[1], "theora", 6)) szName = "Theora";
+					if (!strnicmp(&esd->decoderConfig->decoderSpecificInfo->data[3], "theora", 6)) szName = "Theora";
 					else szName = "Unknown";
 					fprintf(stdout, "Ogg/%s video / GPAC Mux  - Visual Size %d x %d\n", szName, w, h);
 				}
@@ -1091,13 +1136,13 @@ void DumpTrackInfo(GF_ISOFile *file, u32 trackID, Bool full_dump)
 					}
 					break;
 				/*OGG media*/
-				case GPAC_OGG_MEDIA_OTI:
+				case GPAC_OTI_MEDIA_OGG:
 				{
 					char *szName;
 					if (full_dump) fprintf(stdout, "\t");
-					if (!strnicmp(&esd->decoderConfig->decoderSpecificInfo->data[1], "vorbis", 6)) szName = "Vorbis";
-					else if (!strnicmp(&esd->decoderConfig->decoderSpecificInfo->data[0], "Speex", 5)) szName = "Speex";
-					else if (!strnicmp(&esd->decoderConfig->decoderSpecificInfo->data[0], "Flac", 4)) szName = "Flac";
+					if (!strnicmp(&esd->decoderConfig->decoderSpecificInfo->data[3], "vorbis", 6)) szName = "Vorbis";
+					else if (!strnicmp(&esd->decoderConfig->decoderSpecificInfo->data[2], "Speex", 5)) szName = "Speex";
+					else if (!strnicmp(&esd->decoderConfig->decoderSpecificInfo->data[2], "Flac", 4)) szName = "Flac";
 					else szName = "Unknown";
 					fprintf(stdout, "Ogg/%s audio / GPAC Mux - Sample Rate %d - %d channel(s)\n", szName, sr, nb_ch);
 				}
@@ -1213,8 +1258,22 @@ void DumpTrackInfo(GF_ISOFile *file, u32 trackID, Bool full_dump)
 		fprintf(stdout, "\t3GPP EVRC stream - Sample Rate %d - %d channel(s) %d bits per samples\n", sr, nb_ch, (u32) bps);
 	} else if (msub_type == GF_ISOM_SUBTYPE_3GP_QCELP) {
 		fprintf(stdout, "\t3GPP QCELP stream - Sample Rate %d - %d channel(s) %d bits per samples\n", sr, nb_ch, (u32) bps);
+	} else if (msub_type == GF_ISOM_SUBTYPE_AC3) {
+		fprintf(stdout, "\tAC3 stream - Sample Rate %d - %d channel(s) %d bits per samples\n", sr, nb_ch, (u32) bps);
 	} else if (msub_type == GF_ISOM_SUBTYPE_3GP_SMV) {
 		fprintf(stdout, "\t3GPP SMV stream - Sample Rate %d - %d channel(s) %d bits per samples\n", sr, nb_ch, (u32) bps);
+	} else if (msub_type == GF_ISOM_SUBTYPE_3GP_DIMS) {
+		u32 w, h;
+		GF_DIMSDescription dims;
+		gf_isom_get_visual_info(file, trackNum, 1, &w, &h);
+
+		gf_isom_get_dims_description(file, trackNum, 1, &dims);
+		fprintf(stdout, "\t3GPP DIMS stream - size %d x %d - Profile %d - Level %d\n", w, h, dims.profile, dims.level);
+		fprintf(stdout, "\tpathComponents: %d - useFullRequestHost: %s\n", dims.pathComponents, dims.fullRequestHost ? "yes" : "no");
+		fprintf(stdout, "\tstream type: %s - redundant: %s\n", dims.streamType ? "primary" : "secondary", (dims.containsRedundant==1) ? "main" : ((dims.containsRedundant==2) ? "redundant" : "main+redundant") );
+		if (dims.textEncoding[0]) fprintf(stdout, "\ttext encoding %s\n", dims.textEncoding);
+		if (dims.contentEncoding[0]) fprintf(stdout, "\tcontent encoding %s\n", dims.contentEncoding);
+		if (dims.content_script_types) fprintf(stdout, "\tscript languages %s\n", dims.content_script_types);
 	} else if (mtype==GF_ISOM_MEDIA_HINT) {
 		u32 refTrack;
 		s32 i, refCount = gf_isom_get_reference_count(file, trackNum, GF_ISOM_REF_HINT);
@@ -1264,6 +1323,26 @@ void DumpTrackInfo(GF_ISOFile *file, u32 trackID, Bool full_dump)
 	}
 
 	DumpMetaItem(file, 0, trackNum, "Track Meta");
+
+	gf_isom_get_track_switch_group_count(file, trackNum, &alt_group, &nb_groups);
+	if (alt_group) {
+		fprintf(stdout, "Alternate Group ID %d\n", alt_group);
+		for (i=0; i<nb_groups; i++) {
+			u32 nb_crit, switchGroupID; 
+			const u32 *criterias = gf_isom_get_track_switch_parameter(file, trackNum, i+1, &switchGroupID, &nb_crit);
+			if (switchGroupID) {
+				fprintf(stdout, "\tSwitchGroup ID %d criterias: ", switchGroupID);
+			} else {
+				fprintf(stdout, "\tAlternate Group criterias: ");
+			}
+			for (j=0; j<nb_crit; j++) {
+				if (j) fprintf(stdout, " ");
+				fprintf(stdout, "%s", gf_4cc_to_str(criterias[j]) );
+			}
+			if (j) fprintf(stdout, "\n");
+		}
+	}
+
 	if (!full_dump) {
 		fprintf(stdout, "\n");
 		return;
