@@ -45,6 +45,7 @@ void convert_file_info(char *inName, u32 trackID)
 	Bool found;
 	GF_MediaImporter import;
 	memset(&import, 0, sizeof(GF_MediaImporter));
+	import.trackID = trackID;
 	import.in_name = inName;
 	import.flags = GF_IMPORT_PROBE_ONLY;
 	e = gf_media_import(&import);
@@ -73,6 +74,7 @@ void convert_file_info(char *inName, u32 trackID)
 		case GF_ISOM_MEDIA_AUDIO: fprintf(stdout, "Audio (%s)", gf_4cc_to_str(import.tk_info[i].media_type)); break;
 		case GF_ISOM_MEDIA_TEXT: fprintf(stdout, "Text (%s)", gf_4cc_to_str(import.tk_info[i].media_type)); break;
 		case GF_ISOM_MEDIA_SCENE: fprintf(stdout, "Scene (%s)", gf_4cc_to_str(import.tk_info[i].media_type)); break;
+		case GF_ISOM_MEDIA_OD: fprintf(stdout, "ObjectDescriptor (%s)", gf_4cc_to_str(import.tk_info[i].media_type)); break;
 		default: fprintf(stdout, "Other (4CC: %s)", gf_4cc_to_str(import.tk_info[i].type)); break;
 		}
 
@@ -115,7 +117,7 @@ void convert_file_info(char *inName, u32 trackID)
 		if (import.tk_info[i].flags & GF_IMPORT_FORCE_PACKED) fprintf(stdout, "\tCan force packed bitstream import\n");
 		if (import.tk_info[i].flags & GF_IMPORT_OVERRIDE_FPS) fprintf(stdout, "\tCan override source frame rate\n");
 		if (import.tk_info[i].flags & (GF_IMPORT_SBR_IMPLICIT|GF_IMPORT_SBR_EXPLICIT)) fprintf(stdout, "\tCan use AAC-SBR signaling\n");
-		if (import.tk_info[i].flags & GF_IMPORT_FORCE_MPEG4) fprintf(stdout, "\tCan force MPEG-4 Systems stream description\n");
+		if (import.tk_info[i].flags & GF_IMPORT_FORCE_MPEG4) fprintf(stdout, "\tCan force MPEG-4 Systems signaling\n");
 		if (import.tk_info[i].flags & GF_IMPORT_3GPP_AGGREGATION) fprintf(stdout, "\tCan use 3GPP frame aggregation\n");
 		if (import.tk_info[i].flags & GF_IMPORT_NO_DURATION) fprintf(stdout, "\tCannot use duration-based import\n");
 
@@ -130,7 +132,8 @@ GF_Err import_file(GF_ISOFile *dest, char *inName, u32 import_flags, Double forc
 {
 	u32 track_id, i, timescale, track;
 	s32 par_d, par_n, prog_id, delay;
-	Bool do_audio, do_video, do_all;
+	Bool do_audio, do_video, do_all, disable;
+	u32 group;
 	const char *szLan;
 	GF_Err e;
 	GF_MediaImporter import;
@@ -145,8 +148,10 @@ GF_Err import_file(GF_ISOFile *dest, char *inName, u32 import_flags, Double forc
 		return GF_BAD_PARAM;
 	}
 
+	disable = 0;
 	szLan = NULL;
 	delay = 0;
+	group = 0;
 	par_d = par_n = -2;
 	/*use ':' as separator, but beware DOS paths...*/
 	ext = strchr(szName, ':');
@@ -188,6 +193,13 @@ GF_Err import_file(GF_ISOFile *dest, char *inName, u32 import_flags, Double forc
 		else if (!strnicmp(ext+1, "font=", 5)) import.fontName = strdup(ext+6);
 		else if (!strnicmp(ext+1, "size=", 5)) import.fontSize = atoi(ext+6);
 		else if (!strnicmp(ext+1, "fmt=", 4)) import.streamFormat = strdup(ext+5);
+		else if (!strnicmp(ext+1, "disable", 7)) disable = 1;
+		else if (!strnicmp(ext+1, "group=", 6)) {
+			group = atoi(ext+7);
+			if (!group) group = gf_isom_get_next_alternate_group_id(dest);
+		}
+		
+
 		/*unrecognized, assume name has colon in it*/
 		else {
 		 ext = ext2;
@@ -300,6 +312,8 @@ GF_Err import_file(GF_ISOFile *dest, char *inName, u32 import_flags, Double forc
 			timescale = gf_isom_get_timescale(dest);
 			track = gf_isom_get_track_by_id(import.dest, import.final_trackID);
 			if (szLan) gf_isom_set_media_language(import.dest, track, (char *) szLan);
+			if (disable) gf_isom_set_track_enabled(import.dest, track, 0);
+
 			if (delay) {
 				u64 tk_dur;
 				gf_isom_remove_edit_segments(import.dest, track);
@@ -321,6 +335,10 @@ GF_Err import_file(GF_ISOFile *dest, char *inName, u32 import_flags, Double forc
 				e = gf_media_change_par(import.dest, track, par_n, par_d);
 			}
 			if (handler_name) gf_isom_set_handler_name(import.dest, track, handler_name);
+
+			if (group) {
+				gf_isom_set_alternate_group_id(import.dest, track, group);
+			}
 		}
 		if (track_id) fprintf(stdout, "WARNING: Track ID %d not found in file\n", track_id);
 		else if (do_video) fprintf(stdout, "WARNING: Video track not found\n");
@@ -571,6 +589,7 @@ GF_Err split_isomedia_file(GF_ISOFile *mp4, Double split_dur, u32 split_size_kb,
 			if (gf_isom_has_time_offset(mp4, tki->tk)) {
 				gf_isom_set_cts_packing(dest, tki->dst_tk, 1);
 			}
+			gf_isom_remove_edit_segments(dest, tki->dst_tk);
 		}
 		do_add = 1;
 		is_last = 0;
@@ -790,6 +809,8 @@ GF_Err split_isomedia_file(GF_ISOFile *mp4, Double split_dur, u32 split_size_kb,
 
 		/*repack CTSs*/
 		for (i=0; i<nb_tk; i++) {
+			u32 j;
+			u64 new_track_dur;
 			tki = &tks[i];
 			if (tki->stop_state == 2) continue;
 			if (!gf_isom_get_sample_count(dest, tki->dst_tk)) {
@@ -802,6 +823,28 @@ GF_Err split_isomedia_file(GF_ISOFile *mp4, Double split_dur, u32 split_size_kb,
 			if (is_last && tki->can_duplicate) {
 				gf_isom_set_last_sample_duration(dest, tki->dst_tk, gf_isom_get_sample_duration(mp4, tki->tk, tki->sample_count));
 			}
+
+			/*rewrite edit list*/
+			new_track_dur = gf_isom_get_track_duration(dest, tki->dst_tk);
+			count = gf_isom_get_edit_segment_count(mp4, tki->tk);
+			if (count>2) {
+				fprintf(stdout, "Warning: %d edit segments - not supported while splitting (max 2) - ignoring extra\n", count);
+				count=2;
+			}
+			for (j=0; j<count; j++) {
+				u64 editTime, segDur, MediaTime;
+				u8 mode;
+			
+				gf_isom_get_edit_segment(mp4, tki->tk, j+1, &editTime, &segDur, &MediaTime, &mode);
+				if (!j && (mode!=GF_ISOM_EDIT_EMPTY) ) {
+					fprintf(stdout, "Warning: Edit list doesn't look like a track delay scheme - ignoring\n");
+					break;
+				}
+				if (mode==GF_ISOM_EDIT_NORMAL) {
+					segDur = new_track_dur;
+				} 
+				gf_isom_set_edit_segment(dest, tki->dst_tk, editTime, segDur, MediaTime, mode);
+			}
 		}
 		/*check chapters*/
 		do_add = 1;
@@ -847,9 +890,9 @@ err_exit:
 	return e;
 }
 
-GF_Err cat_multiple_files(GF_ISOFile *dest, char *fileName, u32 import_flags, Double force_fps, u32 frames_per_sample, char *tmp_dir);
+GF_Err cat_multiple_files(GF_ISOFile *dest, char *fileName, u32 import_flags, Double force_fps, u32 frames_per_sample, char *tmp_dir, Bool force_cat);
 
-GF_Err cat_isomedia_file(GF_ISOFile *dest, char *fileName, u32 import_flags, Double force_fps, u32 frames_per_sample, char *tmp_dir)
+GF_Err cat_isomedia_file(GF_ISOFile *dest, char *fileName, u32 import_flags, Double force_fps, u32 frames_per_sample, char *tmp_dir, Bool force_cat)
 {
 	u32 i, j, count, nb_tracks, nb_samp, nb_done;
 	GF_ISOFile *orig;
@@ -857,10 +900,10 @@ GF_Err cat_isomedia_file(GF_ISOFile *dest, char *fileName, u32 import_flags, Dou
 	Float ts_scale;
 	Double dest_orig_dur;
 	u32 dst_tk, tk_id, mtype;
-	u64 insert_dts, skip_until_dts;
+	u64 insert_dts;
 	GF_ISOSample *samp;
 
-	if (strchr(fileName, '*')) return cat_multiple_files(dest, fileName, import_flags, force_fps, frames_per_sample, tmp_dir);
+	if (strchr(fileName, '*')) return cat_multiple_files(dest, fileName, import_flags, force_fps, frames_per_sample, tmp_dir, force_cat);
 
 	e = GF_OK;
 	if (!gf_isom_probe_file(fileName)) {
@@ -905,13 +948,19 @@ GF_Err cat_isomedia_file(GF_ISOFile *dest, char *fileName, u32 import_flags, Dou
 	}
 	
 	dest_orig_dur = (Double) (s64) gf_isom_get_duration(dest);
+	if (!gf_isom_get_timescale(dest)) {
+		gf_isom_set_timescale(dest, gf_isom_get_timescale(orig));
+	}
 	dest_orig_dur /= gf_isom_get_timescale(dest);
 
 	fprintf(stdout, "Appending file %s\n", fileName);
 	nb_done = 0;
 	for (i=0; i<nb_tracks; i++) {
-		u64 last_DTS;
+		u64 last_DTS, dest_track_dur_before_cat;
+		u32 nb_edits = 0;
+		Bool new_track = 0;
 		Bool use_ts_dur = 1;
+		Bool merge_edits = 0;
 		mtype = gf_isom_get_media_type(orig, i+1);
 		switch (mtype) {
 		case GF_ISOM_MEDIA_HINT:
@@ -936,7 +985,13 @@ GF_Err cat_isomedia_file(GF_ISOFile *dest, char *fileName, u32 import_flags, Dou
 		tk_id = gf_isom_get_track_id(orig, i+1);
 		dst_tk = gf_isom_get_track_by_id(dest, tk_id);
 		if (dst_tk) {
-			if (!gf_isom_is_same_sample_description(orig, i+1, dest, dst_tk)) dst_tk = 0;
+			/*we MUST have the same codec*/
+			if (gf_isom_get_media_subtype(orig, i+1, 1) != gf_isom_get_media_subtype(dest, dst_tk, 1)) dst_tk = 0;
+			/*we only support cat with the same number of sample descriptions*/
+			if (gf_isom_get_sample_description_count(orig, i+1) != gf_isom_get_sample_description_count(dest, dst_tk)) dst_tk = 0;
+			/*if not forcing cat, check the media codec config is the same*/
+			if (!force_cat && !gf_isom_is_same_sample_description(orig, i+1, dest, dst_tk)) dst_tk = 0;
+			/*we force the same visual resolution*/
 			else if (mtype==GF_ISOM_MEDIA_VISUAL) {
 				u32 w, h, ow, oh;
 				gf_isom_get_visual_info(orig, i+1, 1, &ow, &oh);
@@ -957,6 +1012,20 @@ GF_Err cat_isomedia_file(GF_ISOFile *dest, char *fileName, u32 import_flags, Dou
 							dst_tk = j+1;
 							break;
 						}
+					}
+					/*check language code*/
+					else if (mtype==GF_ISOM_MEDIA_AUDIO) {
+						u32 lang_src, lang_dst;
+						char lang[4];
+						lang[3]=0;
+						gf_isom_get_media_language(orig, i+1, lang);
+						lang_src = GF_4CC(lang[0], lang[1], lang[2], lang[3]);
+						gf_isom_get_media_language(dest, j+1, lang);
+						lang_dst = GF_4CC(lang[0], lang[1], lang[2], lang[3]);
+						if (lang_dst==lang_src) {
+							dst_tk = j+1;
+							break;
+						}
 					} else {
 						dst_tk = j+1;
 						break;
@@ -970,31 +1039,39 @@ GF_Err cat_isomedia_file(GF_ISOFile *dest, char *fileName, u32 import_flags, Dou
 			e = gf_isom_clone_track(orig, i+1, dest, 1, &dst_tk);
 			if (e) goto err_exit;
 			gf_isom_clone_pl_indications(orig, dest);
-			/*remove any edit stuff that could have been copied over*/
-			gf_isom_remove_edit_segments(dest, dst_tk);
+			new_track = 1;
+		} else {
+			nb_edits = gf_isom_get_edit_segment_count(orig, i+1);
 		}
+
+		dest_track_dur_before_cat = gf_isom_get_media_duration(dest, dst_tk);
+
 		count = gf_isom_get_sample_count(dest, dst_tk);
 		if (use_ts_dur && (count>1)) {
 			insert_dts = 2*gf_isom_get_sample_dts(dest, dst_tk, count) - gf_isom_get_sample_dts(dest, dst_tk, count-1);
 		} else {
-			insert_dts = gf_isom_get_media_duration(dest, dst_tk);
+			insert_dts = dest_track_dur_before_cat;
 			if (!count) insert_dts = 0;
 		}
 
 		ts_scale = (Float) gf_isom_get_media_timescale(dest, dst_tk);
 		ts_scale /= gf_isom_get_media_timescale(orig, i+1);
-		skip_until_dts = 0;
-		if (gf_isom_get_edit_segment_count(orig, i+1)) { 
-			u64 editTime, segmentDuration, mediaTime;
-			u8 editMode;
-			gf_isom_get_edit_segment(orig, i+1, 1, &editTime, &segmentDuration, &mediaTime, &editMode);
-			if (editMode == GF_ISOM_EDIT_EMPTY) {
-				Double offset = (Double) (s64)segmentDuration * gf_isom_get_media_timescale(orig, i+1);
-				offset /= gf_isom_get_timescale(orig);
-				insert_dts += (u64) (offset*ts_scale);
-			}
-			else if (editMode == GF_ISOM_EDIT_NORMAL) {
-				skip_until_dts = mediaTime;
+
+		/*if not a new track, see if we can merge the edit list - this is a crude test that only checks
+		we have the same edit types*/
+		if (nb_edits && (nb_edits == gf_isom_get_edit_segment_count(dest, dst_tk)) ) { 
+			u64 editTime, segmentDuration, mediaTime, dst_editTime, dst_segmentDuration, dst_mediaTime;
+			u8 dst_editMode, editMode;
+			u32 j;
+			merge_edits = 1;
+			for (j=0; j<nb_edits; j++) {
+				gf_isom_get_edit_segment(orig, i+1, j+1, &editTime, &segmentDuration, &mediaTime, &editMode);
+				gf_isom_get_edit_segment(dest, dst_tk, j+1, &dst_editTime, &dst_segmentDuration, &dst_mediaTime, &dst_editMode);
+
+				if (dst_editMode!=editMode) { 
+					merge_edits=0; 
+					break; 
+				}
 			}
 		}
 
@@ -1003,14 +1080,6 @@ GF_Err cat_isomedia_file(GF_ISOFile *dest, char *fileName, u32 import_flags, Dou
 		for (j=0; j<count; j++) {
 			u32 di;
 			samp = gf_isom_get_sample(orig, i+1, j+1, &di);
-			if (skip_until_dts) {
-				if (skip_until_dts > samp->DTS) {
-					gf_isom_sample_del(&samp);
-					continue;
-				}
-				samp->DTS -= skip_until_dts;
-			}
-
 			last_DTS = samp->DTS;
 			samp->DTS =  (u64) (ts_scale * (s64)samp->DTS) + insert_dts;
 			samp->CTS_Offset =  (u32) (samp->CTS_Offset * ts_scale);
@@ -1033,6 +1102,55 @@ GF_Err cat_isomedia_file(GF_ISOFile *dest, char *fileName, u32 import_flags, Dou
 			insert_dts = gf_isom_get_media_duration(orig, i+1) - last_DTS;
 			gf_isom_set_last_sample_duration(dest, dst_tk, (u32) insert_dts);
 		}
+
+		if (merge_edits) {
+			/*get the first edit normal mode and add the new track dur*/
+			for (j=nb_edits; j>0; j--) {
+				u64 editTime, segmentDuration, mediaTime;
+				u8 editMode;
+				gf_isom_get_edit_segment(dest, dst_tk, j, &editTime, &segmentDuration, &mediaTime, &editMode);
+
+				if (editMode==GF_ISOM_EDIT_NORMAL) {
+					Double dur = (Double) (s64) gf_isom_get_media_duration(orig, i+1);
+					/*convert to dst time scale*/
+					dur *= ts_scale;
+
+					/*convert to track time scale*/
+					ts_scale = (Float) gf_isom_get_timescale(dest);
+					ts_scale /= (Float) gf_isom_get_media_timescale(dest, dst_tk);
+					dur *= ts_scale;
+
+					segmentDuration += (u64) (s64) dur;
+					gf_isom_modify_edit_segment(dest, dst_tk, j, segmentDuration, mediaTime, editMode);
+					break;
+				}
+			}
+		} else {
+			u64 editTime, segmentDuration, mediaTime, edit_offset;
+			Double t;
+			u8 editMode;
+			u32 j, count;
+			
+			count = gf_isom_get_edit_segment_count(dest, dst_tk);
+			gf_isom_get_edit_segment(dest, dst_tk, count, &editTime, &segmentDuration, &mediaTime, &editMode);
+
+
+			/*convert to dst time scale*/
+			ts_scale = (Float) gf_isom_get_timescale(dest);
+			ts_scale /= (Float) gf_isom_get_timescale(orig);
+
+			edit_offset = editTime + segmentDuration;
+			count = gf_isom_get_edit_segment_count(orig, i+1);
+			for (j=0; j<count; j++) {
+				gf_isom_get_edit_segment(orig, i+1, j+1, &editTime, &segmentDuration, &mediaTime, &editMode);
+				t = (Double) (s64) editTime; t *= ts_scale; t += (s64) edit_offset; editTime = (s64) t;
+				t = (Double) (s64) segmentDuration; t *= ts_scale; segmentDuration = (s64) t;
+				t = (Double) (s64) mediaTime; t *= ts_scale; t+= (s64) dest_track_dur_before_cat; mediaTime = (s64) t;
+
+				gf_isom_set_edit_segment(dest, dst_tk, editTime, segmentDuration, mediaTime, editMode);
+			}
+		}
+
 	}
 	gf_set_progress("Appending", nb_samp, nb_samp);
 
@@ -1074,6 +1192,7 @@ typedef struct
 	Double force_fps;
 	u32 frames_per_sample;
 	char *tmp_dir;
+	Bool force_cat;
 } CATEnum;
 
 Bool cat_enumerate(void *cbk, char *szName, char *szPath)
@@ -1089,12 +1208,12 @@ Bool cat_enumerate(void *cbk, char *szName, char *szPath)
 	strcpy(szFileName, szName);
 	strcat(szFileName, cat_enum->szOpt);
 
-	e = cat_isomedia_file(cat_enum->dest, szFileName, cat_enum->import_flags, cat_enum->force_fps, cat_enum->frames_per_sample, cat_enum->tmp_dir);
+	e = cat_isomedia_file(cat_enum->dest, szFileName, cat_enum->import_flags, cat_enum->force_fps, cat_enum->frames_per_sample, cat_enum->tmp_dir, cat_enum->force_cat);
 	if (e) return 1;
 	return 0;
 }
 
-GF_Err cat_multiple_files(GF_ISOFile *dest, char *fileName, u32 import_flags, Double force_fps, u32 frames_per_sample, char *tmp_dir)
+GF_Err cat_multiple_files(GF_ISOFile *dest, char *fileName, u32 import_flags, Double force_fps, u32 frames_per_sample, char *tmp_dir, Bool force_cat)
 {
 	CATEnum cat_enum;
 	char *sep;
@@ -1104,6 +1223,7 @@ GF_Err cat_multiple_files(GF_ISOFile *dest, char *fileName, u32 import_flags, Do
 	cat_enum.force_fps = force_fps;
 	cat_enum.frames_per_sample = frames_per_sample;
 	cat_enum.tmp_dir = tmp_dir;
+	cat_enum.force_cat = force_cat;
 
 	strcpy(cat_enum.szPath, fileName);
 	sep = strrchr(cat_enum.szPath, GF_PATH_SEPARATOR);
@@ -1154,37 +1274,77 @@ GF_Err EncodeFile(char *in, GF_ISOFile *mp4, GF_SMEncodeOptions *opts, FILE *log
 	e = gf_sm_load_init(&load);
 	if (e<0) {
 		gf_sm_load_done(&load);
-		fprintf(stdout, "Cannot load context - %s\n", gf_error_to_string(e));
+		fprintf(stdout, "Cannot load context %s - %s\n", in, gf_error_to_string(e));
 		goto err_exit;
 	}
 	e = gf_sm_load_run(&load);
 	gf_sm_load_done(&load);
 
-	if (opts->auto_qant) {
+	if (opts->auto_quant) {
 		fprintf(stdout, "Analysing Scene for Automatic Quantization\n");
 		statsman = gf_sm_stats_new();
 		e = gf_sm_stats_for_scene(statsman, ctx);
 		if (!e) {
 			GF_SceneStatistics *stats = gf_sm_stats_get(statsman);
-			if (opts->resolution > (s32)stats->frac_res_2d) {
-				fprintf(stdout, " Given resolution %d is (unnecessarily) too high, using %d instead.\n", opts->resolution, stats->frac_res_2d);
-				opts->resolution = stats->frac_res_2d;
-			} else if (stats->int_res_2d + opts->resolution <= 0) {
-				fprintf(stdout, " Given resolution %d is too low, using %d instead.\n", opts->resolution, stats->int_res_2d - 1);
-				opts->resolution = 1 - stats->int_res_2d;
-			}				
-			opts->coord_bits = stats->int_res_2d + opts->resolution;
-			fprintf(stdout, " Coordinates & Lengths encoded using ");
-			if (opts->resolution < 0) fprintf(stdout, "only the %d most significant bits (of %d).\n", opts->coord_bits, stats->int_res_2d);
-			else fprintf(stdout, "a %d.%d representation\n", stats->int_res_2d, opts->resolution);
+			/*LASeR*/
+			if (opts->auto_quant==1) {
+				if (opts->resolution > (s32)stats->frac_res_2d) {
+					fprintf(stdout, " Given resolution %d is (unnecessarily) too high, using %d instead.\n", opts->resolution, stats->frac_res_2d);
+					opts->resolution = stats->frac_res_2d;
+				} else if (stats->int_res_2d + opts->resolution <= 0) {
+					fprintf(stdout, " Given resolution %d is too low, using %d instead.\n", opts->resolution, stats->int_res_2d - 1);
+					opts->resolution = 1 - stats->int_res_2d;
+				}				
+				opts->coord_bits = stats->int_res_2d + opts->resolution;
+				fprintf(stdout, " Coordinates & Lengths encoded using ");
+				if (opts->resolution < 0) fprintf(stdout, "only the %d most significant bits (of %d).\n", opts->coord_bits, stats->int_res_2d);
+				else fprintf(stdout, "a %d.%d representation\n", stats->int_res_2d, opts->resolution);
 
-			fprintf(stdout, " Matrix Scale & Skew Coefficients ");
-			if (opts->coord_bits < stats->scale_int_res_2d) {
-				opts->scale_bits = stats->scale_int_res_2d - opts->coord_bits;
-				fprintf(stdout, "encoded using a %d.8 representation\n", stats->scale_int_res_2d);
-			} else  {
-				opts->scale_bits = 0;
-				fprintf(stdout, "not encoded.\n");
+				fprintf(stdout, " Matrix Scale & Skew Coefficients ");
+				if (opts->coord_bits - 8 < stats->scale_int_res_2d) {
+					opts->scale_bits = stats->scale_int_res_2d - opts->coord_bits + 8;
+					fprintf(stdout, "encoded using a %d.8 representation\n", stats->scale_int_res_2d);
+				} else  {
+					opts->scale_bits = 0;
+					fprintf(stdout, "encoded using a %d.8 representation\n", opts->coord_bits - 8);
+				}
+			} 
+			/*BIFS*/
+			else if (stats->base_layer) {
+				GF_AUContext *au;
+				GF_CommandField *inf;
+				M_QuantizationParameter *qp;
+				GF_Command *com = gf_sg_command_new(ctx->scene_graph, GF_SG_GLOBAL_QUANTIZER);
+				qp = (M_QuantizationParameter *) gf_node_new(ctx->scene_graph, TAG_MPEG4_QuantizationParameter);
+	
+				inf = gf_sg_command_field_new(com);
+				inf->new_node = (GF_Node *)qp;
+				inf->field_ptr = &inf->new_node;
+				inf->fieldType = GF_SG_VRML_SFNODE;
+				gf_node_register(inf->new_node, NULL);
+				au = gf_list_get(stats->base_layer->AUs, 0);
+				gf_list_insert(au->commands, com, 0);
+				qp->useEfficientCoding = 1;
+				qp->textureCoordinateQuant = 0;
+				if ((stats->count_2f+stats->count_2d) && opts->resolution) {
+					qp->position2DMin = stats->min_2d;
+					qp->position2DMax = stats->max_2d;
+					qp->position2DNbBits = opts->resolution;
+					qp->position2DQuant = 1;
+				}
+				if ((stats->count_3f+stats->count_3d) &&  opts->resolution) {
+					qp->position3DMin = stats->min_3d;
+					qp->position3DMax = stats->max_3d;
+					qp->position3DQuant = opts->resolution;
+					qp->position3DQuant = 1;
+					qp->textureCoordinateQuant = 1;
+				}
+				if (0 && stats->count_float && opts->resolution) {
+					qp->scaleMin = stats->min_fixed;
+					qp->scaleMax = stats->max_fixed;
+					qp->scaleNbBits = 2*opts->resolution;
+					qp->scaleQuant = 1;
+				}
 			}
 		}
 	}
