@@ -36,7 +36,7 @@
 void convert_file_info(char *inName, u32 trackID);
 GF_Err import_file(GF_ISOFile *dest, char *inName, u32 import_flags, Double force_fps, u32 frames_per_sample);
 GF_Err split_isomedia_file(GF_ISOFile *mp4, Double split_dur, u32 split_size_kb, char *inName, Double InterleavingTime, Double chunk_start, const char *tmpdir, char *outfile);
-GF_Err cat_isomedia_file(GF_ISOFile *mp4, char *fileName, u32 import_flags, Double force_fps, u32 frames_per_sample, char *tmp_dir);
+GF_Err cat_isomedia_file(GF_ISOFile *mp4, char *fileName, u32 import_flags, Double force_fps, u32 frames_per_sample, char *tmp_dir, Bool force_cat);
 
 GF_Err EncodeFile(char *in, GF_ISOFile *mp4, GF_SMEncodeOptions *opts, FILE *logs);
 GF_Err EncodeFileChunk(char *chunkFile, char *bifs, char *inputContext, char *outputContext, const char *tmpdir);
@@ -102,7 +102,7 @@ u32 nb_itunes_tags = sizeof(itags) / sizeof(itunes_tag);
 
 void PrintVersion()
 {
-	fprintf(stdout, "MP4Box - GPAC version " GPAC_VERSION "\n"
+	fprintf(stdout, "MP4Box - GPAC version " GPAC_FULL_VERSION "\n"
 #ifdef GPAC_FIXED_POINT
 				"GPAC compiled in fixed-point version\n"
 #endif
@@ -131,7 +131,7 @@ void PrintGeneralUsage()
 			"                       * Note: Default temp dir is OS-dependent\n"
 			" -no-sys              removes all MPEG-4 Systems info except IOD (profiles)\n"
 			"                       * Note: Set by default whith '-add' and '-cat'\n"
-			" -no-iod              removes InitialObjkectDescriptor from file\n"
+			" -no-iod              removes InitialObjectDescriptor from file\n"
 			" -isma                rewrites the file as an ISMA 1.0 AV file\n"
 			" -ismax               same as \'-isma\' and removes all clock references\n"
 			" -3gp                 rewrites as 3GPP(2) file (no more MPEG-4 Systems Info)\n"
@@ -144,6 +144,8 @@ void PrintGeneralUsage()
 			" -cprt string         adds copyright string to movie\n"
 			" -chap file           adds chapter information contained in file\n"
 			" -rem trackID:        removes track from file\n"
+			" -enable trackID:     enables track\n"
+			" -disable trackID:    disables track\n"
 			" -new:                forces creation of a new destination file\n"
 			" -rem trackID:        removes track from file\n"
 			" -lang [tkID=]LAN:    sets track language. LAN is the ISO 639-2 code (eng, und)\n"
@@ -158,8 +160,29 @@ void PrintGeneralUsage()
 			"                       * Note: this removes all MPEG-4 Systems media\n"
 			" -split-chunk S:E     extracts a new file from Start to End (in seconds)\n"
 			"                       * Note: this removes all MPEG-4 Systems media\n"
+			" -group-add fmt       creates a new grouping information in the file. Format is\n"
+			"                      a colon-separated list of following options:\n"
+			"                      refTrack=ID: ID of the track used as a group reference.\n"
+			"                       If not set, the track will belong to the same group as the previous trackID specified.\n"
+			"                       If 0 or no previous track specified, a new alternate group will be created\n"
+			"                      switchID=ID: ID of the switch group to create.\n"
+			"                       If 0, a new ID will be computed for you\n"
+			"                       If <0, disables SwitchGroup\n"
+			"                      criteria=string: list of space-separated 4CCs.\n"
+			"                      trackID=ID: ID of the track to add to this group.\n"
+			"\n"
+			"                       *WARNING* Options modify state as they are parsed:\n"
+			"                        trackID=1:criteria=lang:trackID=2\n"
+			"                       is different from:\n"
+			"                        criteria=lang:trackID=1:trackID=2\n"
+			"\n"
+			" -group-rem-track ID  removes track from its group\n"
+			" -group-rem ID        removes the track's group\n"
+			" -group-clean         removes all group information from all tracks\n"
+			" -ref id:XXXX:refID   adds a reference of type 4CC from track ID to track refID\n"
 			"\n");
 }
+
 void PrintFormats()
 {
 	fprintf(stdout, "Suppported raw formats and file extensions:\n"
@@ -214,6 +237,9 @@ void PrintImportUsage()
 			" \":delay=delay_ms\":   sets imported media initial delay in ms\n"
 			" \":par=PAR\":          sets visual pixel aspect ratio (PAR=Num:Den)\n"
 			" \":name=NAME\":        sets track handler name\n"
+			" \":disable\":          imported track(s) will be disabled\n"
+			" \":group=G\":          adds the track as part of the G alternate group.\n"
+			"                        * If G is 0, the first available GroupID will be picked.\n"
 			" \":fps=VAL\":          same as -fps option\n"
 			" \":agg=VAL\":          same as -agg option\n"
 			" \":par=VAL\":          same as -par option\n"
@@ -229,6 +255,8 @@ void PrintImportUsage()
 			" -add file:           add file tracks to (new) output file\n"
 			" -cat file:           concatenates file samples to (new) output file\n"
 			"                       * Note: creates tracks if needed\n"
+			" -force-cat:          skips media configuration check when concatenating file\n"
+			"                       !!! THIS MAY BREAK THE CONCATENATED TRACK(S) !!!\n"
 			" -keep-sys:           keeps all MPEG-4 Systems info when using '-add' / 'cat'\n"
 			" -keep-all:           keeps all existing tracks when using '-add'\n"
 			"                       * Note: only used when adding IsoMedia files\n"
@@ -246,6 +274,7 @@ void PrintImportUsage()
 			"                       * For all other imports, default FPS is 25\n"
 			"                      -- THIS IS IGNORED FOR IsoMedia IMPORT --\n"
 			" -mpeg4:              forces MPEG-4 sample descriptions when possible (3GPP2)\n"
+			"                       For AAC, forces MPEG-4 AAC signaling even if MPEG-2\n"
 			" -agg N:              aggregates N audio frames in 1 sample (3GP media only)\n"
 			"                       * Note: Maximum value is 15 - Disabled by default\n"
 			"\n"
@@ -269,14 +298,14 @@ void PrintEncodeUsage()
 			" -ctx-out file:       specifies storage of updated context (MP4/BT/XMT)\n"
 			"\n"
 			"LASeR Encoding options\n"
-			" -auto_quant res:     resolution is given as if using -resolution\n"
-			"                       but coord-bits and scale-bits are infered\n"
 			" -resolution res:     resolution factor (-8 to 7, default 0)\n"
 			"                       all coords are multiplied by 2^res before truncation\n"
 			" -coord-bits bits:    bits used for encoding truncated coordinates\n"
 			"                       (0 to 31, default 12)\n"
 			" -scale-bits bits:    extra bits used for encoding truncated scales\n"
 			"                       (0 to 4, default 0)\n"
+			" -auto-quant res:     resolution is given as if using -resolution\n"
+			"                       but coord-bits and scale-bits are infered\n"
 			);
 }
 
@@ -346,7 +375,6 @@ void PrintHintUsage()
 			"\n"
 			" -add-sdp string:     adds sdp string to (hint) track (\"-add-sdp tkID:string\")\n"
 			"                      or movie. This will take care of SDP lines ordering\n"
-			"                       * WARNING: You cannot add anything to SDP, cf rfc2327.\n"
 			" -unhint:             removes all hinting information.\n"
 			"\n");
 }
@@ -440,8 +468,8 @@ void PrintSWFUsage()
 			"\n"
 			" -global:             all SWF defines are placed in first scene replace\n"
 			"                       * Note: By default SWF defines are sent when needed\n"
-			" -ctrl:               uses a dedicated stream for movie control\n"
-			"                       * Note: Forces \'-global\'\n"
+			" -no-ctrl:            uses a single stream for movie control and dictionary\n"
+			"                       * Note: this will disable ActionScript\n"
 			" -no-text:            removes all SWF text\n"
 			" -no-font:            removes all embedded SWF Fonts (terminal fonts used)\n"
 			" -no-line:            removes all lines from SWF shapes\n"
@@ -574,6 +602,10 @@ GF_Err HintFile(GF_ISOFile *file, u32 MTUSize, u32 max_ptime, u32 rtp_rate, u32
 		copy = copy_data;
 		/*skip emty tracks (mainly MPEG-4 interaction streams...*/
 		if (!gf_isom_get_sample_count(file, i+1)) continue;
+		if (!gf_isom_is_track_enabled(file, i+1)) {
+			fprintf(stdout, "Track ID %d disabled - skipping hint\n", gf_isom_get_track_id(file, i+1) );
+			continue;
+		}
 
 		mtype = gf_isom_get_media_type(file, i+1);
 		switch (mtype) {
@@ -741,6 +773,13 @@ void remove_systems_tracks(GF_ISOFile *file)
 		/*only remove real systems tracks (eg, delaing with scene description & presentation)
 		but keep meta & all unknown tracks*/
 		case GF_ISOM_MEDIA_SCENE:
+			switch (gf_isom_get_media_subtype(file, i+1, 1)) {
+			case GF_ISOM_MEDIA_DIMS:
+				gf_isom_remove_track_from_root_od(file, i+1);
+				continue;
+			default:
+				break;
+			}
 		case GF_ISOM_MEDIA_OD:
 		case GF_ISOM_MEDIA_OCR:
 		case GF_ISOM_MEDIA_MPEGJ:
@@ -908,6 +947,97 @@ static Bool parse_meta_args(MetaAction *meta, char *opts)
 }
 #endif
 
+
+
+
+typedef struct
+{	
+	/*0: set tsel param - 1 remove tsel - 2 remove all tsel info in alternate group - 3 remove all tsel info in file*/
+	u32 act_type;
+	u32 trackID;
+
+	u32 refTrackID;
+	u32 criteria[30];
+	u32 nb_criteria;
+	Bool is_switchGroup;
+	u32 switchGroupID;
+} TSELAction;
+
+static Bool parse_tsel_args(TSELAction *tsel_list, char *opts, u32 *nb_tsel_act)
+{
+	u32 act;
+	u32 refTrackID = 0;
+	Bool has_switch_id;
+	u32 switch_id = 0;
+	u32 criteria[30];
+	u32 nb_criteria = 0;
+	TSELAction *tsel_act;
+	char szSlot[1024], *next;
+
+	has_switch_id = 0;
+	act = tsel_list[*nb_tsel_act].act_type;
+			
+
+	if (!opts) return 0;
+	while (1) {
+		if (!opts || !opts[0]) return 1;
+		if (opts[0]==':') opts += 1;
+		strcpy(szSlot, opts);
+		next = strchr(szSlot, ':');
+		/*use ':' as separator, but beware DOS paths...*/
+		if (next && next[1]=='\\') next = strchr(szSlot+2, ':');
+		if (next) next[0] = 0;
+
+
+		if (!strnicmp(szSlot, "ref=", 4)) refTrackID = atoi(szSlot+4);  
+		else if (!strnicmp(szSlot, "switchID=", 9)) {
+			if (atoi(szSlot+9)<0) {
+				switch_id = 0; 
+				has_switch_id = 0;
+			} else {
+				switch_id = atoi(szSlot+9); 
+				has_switch_id = 1;
+			}
+		}
+		else if (!strnicmp(szSlot, "switchID", 8)) {
+				switch_id = 0; 
+				has_switch_id = 1;
+		}
+		else if (!strnicmp(szSlot, "criteria=", 9)) {
+			u32 j=9;
+			nb_criteria = 0;
+			while (j+3<strlen(szSlot)) {
+				criteria[nb_criteria] = GF_4CC(szSlot[j], szSlot[j+1], szSlot[j+2], szSlot[j+3]);
+				j+=5;
+				nb_criteria++;
+			}
+		}
+		else if (!strnicmp(szSlot, "trackID=", 8) || !strchr(szSlot, '=') ) {
+			if (*nb_tsel_act>=MAX_CUMUL_OPS) {
+				fprintf(stdout, "Sorry - no more than %d track selection operations allowed\n", MAX_CUMUL_OPS); 
+				return 0; 
+			}
+			tsel_act = &tsel_list[*nb_tsel_act];
+			memset(tsel_act, 0, sizeof(TSELAction));
+			tsel_act->act_type = act;
+			tsel_act->trackID = strchr(szSlot, '=') ? atoi(szSlot+8) : atoi(szSlot);
+			tsel_act->refTrackID = refTrackID;
+			tsel_act->switchGroupID = switch_id;
+			tsel_act->is_switchGroup = has_switch_id;
+			tsel_act->nb_criteria = nb_criteria;
+			memcpy(tsel_act->criteria, criteria, sizeof(u32)*nb_criteria);
+
+			if (!refTrackID)
+				refTrackID = tsel_act->trackID;
+
+			(*nb_tsel_act) ++;
+		}		
+		opts += strlen(szSlot);
+	}
+	return 1;
+}
+
+
 #define CHECK_NEXT_ARG	if (i+1==(u32)argc) { fprintf(stdout, "Missing arg - please check usage\n"); return 1; }
 
 #define CHECK_META_OPS	CHECK_NEXT_ARG if (nb_meta_act>=MAX_CUMUL_OPS) { fprintf(stdout, "Sorry - no more than %d meta operations allowed\n", MAX_CUMUL_OPS); return 1; }
@@ -922,6 +1052,9 @@ typedef struct
 	3: set track KMS URI
 	4: set visual track PAR if possible
 	5: set track handler name
+	6: enables track
+	7: disables track
+	8: referenceTrack
 	*/
 	u32 act_type;
 	/*track ID*/
@@ -953,10 +1086,11 @@ int main(int argc, char **argv)
 	char *szFilesToCat[MAX_CUMUL_OPS];
 	char *szTracksToAdd[MAX_CUMUL_OPS];
 	TrackAction tracks[MAX_CUMUL_OPS];
+	TSELAction tsel_acts[MAX_CUMUL_OPS];
 	u32 brand_add[MAX_CUMUL_OPS], brand_rem[MAX_CUMUL_OPS];
-	u32 i, MTUSize, stat_level, hint_flags, info_track_id, import_flags, nb_add, nb_cat, ismaCrypt, agg_samples, nb_sdp_ex, max_ptime, raw_sample_num, split_size, nb_meta_act, nb_track_act, rtp_rate, major_brand, nb_alt_brand_add, nb_alt_brand_rem, old_interleave, car_dur, minor_version, conv_type;
+	u32 i, MTUSize, stat_level, hint_flags, info_track_id, import_flags, nb_add, nb_cat, ismaCrypt, agg_samples, nb_sdp_ex, max_ptime, raw_sample_num, split_size, nb_meta_act, nb_track_act, rtp_rate, major_brand, nb_alt_brand_add, nb_alt_brand_rem, old_interleave, car_dur, minor_version, conv_type, nb_tsel_acts;
 	Bool HintIt, needSave, FullInter, Frag, HintInter, dump_std, dump_rtp, dump_mode, regular_iod, trackID, HintCopy, remove_sys_tracks, remove_hint, force_new, keep_sys_tracks, remove_root_od, import_subtitle;
-	Bool print_sdp, print_info, open_edit, track_dump_type, dump_isom, dump_cr, force_ocr, encode, do_log, do_flat, dump_srt, dump_ttxt, x3d_info, chunk_mode, dump_ts, do_saf, dump_m2ts, dump_cart, do_hash, verbose;
+	Bool print_sdp, print_info, open_edit, track_dump_type, dump_isom, dump_cr, force_ocr, encode, do_log, do_flat, dump_srt, dump_ttxt, x3d_info, chunk_mode, dump_ts, do_saf, dump_m2ts, dump_cart, do_hash, verbose, force_cat;
 	char *inName, *outName, *arg, *mediaSource, *tmpdir, *input_ctx, *output_ctx, *drm_file, *avi2raw, *cprt, *chap_file, *pes_dump, *itunes_tags, *pack_file, *raw_cat;
 	GF_ISOFile *file;
 
@@ -966,7 +1100,7 @@ int main(int argc, char **argv)
 		return 1;
 	}
 
-	nb_add = nb_cat = nb_track_act = nb_sdp_ex = max_ptime = raw_sample_num = nb_meta_act = rtp_rate = major_brand = nb_alt_brand_add = nb_alt_brand_rem = car_dur = minor_version = 0;
+	nb_tsel_acts = nb_add = nb_cat = nb_track_act = nb_sdp_ex = max_ptime = raw_sample_num = nb_meta_act = rtp_rate = major_brand = nb_alt_brand_add = nb_alt_brand_rem = car_dur = minor_version = 0;
 	e = GF_OK;
 	split_duration = 0.0;
 	split_start = -1.0;
@@ -977,7 +1111,7 @@ int main(int argc, char **argv)
 	MTUSize = 1450;
 	HintCopy = FullInter = HintInter = encode = do_log = old_interleave = do_saf = do_hash = verbose = 0;
 	chunk_mode = dump_mode = Frag = force_ocr = remove_sys_tracks = agg_samples = remove_hint = keep_sys_tracks = remove_root_od = 0;
-	x3d_info = conv_type = HintIt = needSave = print_sdp = print_info = regular_iod = dump_std = open_edit = dump_isom = dump_rtp = dump_cr = dump_srt = dump_ttxt = force_new = dump_ts = dump_m2ts = dump_cart = import_subtitle = 0;
+	x3d_info = conv_type = HintIt = needSave = print_sdp = print_info = regular_iod = dump_std = open_edit = dump_isom = dump_rtp = dump_cr = dump_srt = dump_ttxt = force_new = dump_ts = dump_m2ts = dump_cart = import_subtitle = force_cat = 0;
 	track_dump_type = 0;
 	ismaCrypt = 0;
 	file = NULL;
@@ -989,7 +1123,7 @@ int main(int argc, char **argv)
 	do_flat = 0;
 	inName = outName = mediaSource = input_ctx = output_ctx = drm_file = avi2raw = cprt = chap_file = pack_file = raw_cat = NULL;
 
-	swf_flags = 0;
+	swf_flags = GF_SM_SWF_SPLIT_TIMELINE;
 	swf_flatten_angle = 0.0f;
 	tmpdir = NULL;
 	
@@ -997,7 +1131,8 @@ int main(int argc, char **argv)
 	for (i = 1; i < (u32) argc ; i++) {
 		arg = argv[i];
 		/*main file*/
-		if (isalnum(arg[0]) || (arg[0]=='/') || (arg[0]=='.') || (arg[0]=='\\') ) {
+//		if (isalnum(arg[0]) || (arg[0]=='/') || (arg[0]=='.') || (arg[0]=='\\') ) {
+		if (arg[0] != '-') {
 			if (inName) { fprintf(stdout, "Error - 2 input names specified, please check usage\n"); return 1; }
 			inName = arg;
 		}
@@ -1126,13 +1261,15 @@ int main(int argc, char **argv)
 #ifndef GPAC_READ_ONLY
 		/*SWF importer options*/
 		else if (!stricmp(arg, "-global")) swf_flags |= GF_SM_SWF_STATIC_DICT;
-		else if (!stricmp(arg, "-ctrl")) swf_flags |= GF_SM_SWF_SPLIT_TIMELINE;
+		else if (!stricmp(arg, "-no-ctrl")) swf_flags &= ~GF_SM_SWF_SPLIT_TIMELINE;
 		else if (!stricmp(arg, "-no-text")) swf_flags |= GF_SM_SWF_NO_TEXT;
 		else if (!stricmp(arg, "-no-font")) swf_flags |= GF_SM_SWF_NO_FONT;
 		else if (!stricmp(arg, "-no-line")) swf_flags |= GF_SM_SWF_NO_LINE;
 		else if (!stricmp(arg, "-no-grad")) swf_flags |= GF_SM_SWF_NO_GRADIENT;
 		else if (!stricmp(arg, "-quad")) swf_flags |= GF_SM_SWF_QUAD_CURVE;
 		else if (!stricmp(arg, "-xlp")) swf_flags |= GF_SM_SWF_SCALABLE_LINE;
+		else if (!stricmp(arg, "-ic2d")) swf_flags |= GF_SM_SWF_USE_IC2D;
+		else if (!stricmp(arg, "-same-app")) swf_flags |= GF_SM_SWF_REUSE_APPEARANCE;
 		else if (!stricmp(arg, "-flatten")) {
 			CHECK_NEXT_ARG
 			swf_flatten_angle = (Float) atof(argv[i+1]);
@@ -1146,7 +1283,9 @@ int main(int argc, char **argv)
 		else if (!stricmp(arg, "-no-sys") || !stricmp(arg, "-nosys")) { remove_sys_tracks = 1; open_edit = 1; }
 		else if (!stricmp(arg, "-no-iod")) { remove_root_od = 1; open_edit = 1; }
 		else if (!stricmp(arg, "-out")) { CHECK_NEXT_ARG outName = argv[i+1]; i++; }
-		else if (!stricmp(arg, "-tmp")) { CHECK_NEXT_ARG tmpdir = argv[i+1]; i++; }
+		else if (!stricmp(arg, "-tmp")) {
+			CHECK_NEXT_ARG tmpdir = argv[i+1]; i++; 
+		}
 		else if (!stricmp(arg, "-cprt")) { CHECK_NEXT_ARG cprt = argv[i+1]; i++; open_edit = 1; }
 		else if (!stricmp(arg, "-chap")) { CHECK_NEXT_ARG chap_file = argv[i+1]; i++; open_edit = 1; }
 		else if (!stricmp(arg, "-inter") || !stricmp(arg, "-old-inter")) {
@@ -1217,6 +1356,7 @@ int main(int argc, char **argv)
 				sdp_lines[nb_sdp_ex].line = argv[i+1];
 				sdp_lines[nb_sdp_ex].trackID = 0;
 			}
+			open_edit = 1;
 			nb_sdp_ex++;
 			i++;
 		}
@@ -1251,18 +1391,21 @@ int main(int argc, char **argv)
 			nb_cat++;
 			i++;
 		}
+		else if (!stricmp(arg, "-force-cat")) force_cat = 1;
 		else if (!stricmp(arg, "-raw-cat")) {
 			CHECK_NEXT_ARG
 			raw_cat = argv[i+1];
 			i++;
 		}
-		else if (!stricmp(arg, "-rem")) {
+		else if (!stricmp(arg, "-rem") || !stricmp(arg, "-disable") || !stricmp(arg, "-enable")) {
 			CHECK_NEXT_ARG
 			if (nb_track_act>=MAX_CUMUL_OPS) {
 				fprintf(stdout, "Sorry - no more than %d track operations allowed\n", MAX_CUMUL_OPS);
 				return 1;
 			}
-			tracks[nb_track_act].act_type = 0;
+			if (!stricmp(arg, "-enable")) tracks[nb_track_act].act_type = 6;
+			else if (!stricmp(arg, "-disable")) tracks[nb_track_act].act_type = 7;
+			else tracks[nb_track_act].act_type = 0;
 			tracks[nb_track_act].trackID = atoi(argv[i+1]);
 			open_edit = 1;
 			nb_track_act++;
@@ -1340,6 +1483,34 @@ int main(int argc, char **argv)
 			nb_track_act++;
 			i++;
 		}
+		else if (!stricmp(arg, "-ref")) {
+			char *szTK, *ext;
+			CHECK_NEXT_ARG
+			if (nb_track_act>=MAX_CUMUL_OPS) {
+				fprintf(stdout, "Sorry - no more than %d track operations allowed\n", MAX_CUMUL_OPS);
+				return 1;
+			}
+			szTK = argv[i+1];
+			ext = strchr(szTK, ':');
+			if (!ext) {
+				fprintf(stdout, "Bad format for track reference - expecting ID:XXXX:refID got %s\n", argv[i+1]);
+				return 1;
+			}
+			tracks[nb_track_act].act_type = 8;
+			ext[0] = 0; tracks[nb_track_act].trackID = atoi(szTK); ext[0] = ':'; szTK = ext+1;
+			ext = strchr(szTK, ':');
+			if (!ext) {
+				fprintf(stdout, "Bad format for track reference - expecting ID:XXXX:refID got %s\n", argv[i+1]);
+				return 1;
+			}
+			ext[0] = 0;
+			strncpy(tracks[nb_track_act].lang, szTK, 4);
+			ext[0] = ':';
+			tracks[nb_track_act].delay_ms = (s32) atoi(ext+1);
+			open_edit = 1;
+			nb_track_act++;
+			i++;
+		}
 		else if (!stricmp(arg, "-name")) {
 			char szTK[GF_MAX_PATH], *ext;
 			CHECK_NEXT_ARG
@@ -1404,10 +1575,10 @@ int main(int argc, char **argv)
 			opts.resolution = atoi(argv[i+1]);
 			i++;
 		}
-		else if (!stricmp(arg, "-auto_quant")) {
+		else if (!stricmp(arg, "-auto-quant")) {
 			CHECK_NEXT_ARG
 			opts.resolution = atoi(argv[i+1]);
-			opts.auto_qant = 1;
+			opts.auto_quant = 1;
 			i++;
 		}
 		else if (!stricmp(arg, "-coord-bits")) {
@@ -1420,6 +1591,12 @@ int main(int argc, char **argv)
 			opts.scale_bits = atoi(argv[i+1]);
 			i++;
 		}
+		else if (!stricmp(arg, "-global-quant")) {
+			CHECK_NEXT_ARG
+			opts.resolution = atoi(argv[i+1]);
+			opts.auto_quant = 2;
+			i++;
+		}
 		/*chunk encoding*/
 		else if (!stricmp(arg, "-ctx-out") || !stricmp(arg, "-outctx")) { CHECK_NEXT_ARG output_ctx = argv[i+1]; i++; }
 		else if (!stricmp(arg, "-ctx-in") || !stricmp(arg, "-inctx")) {
@@ -1544,6 +1721,21 @@ int main(int argc, char **argv)
 			nb_meta_act++;
 			i++;
 		}
+		else if (!stricmp(arg, "-group-add") || !stricmp(arg, "-group-rem-track") || !stricmp(arg, "-group-rem")) { 
+			tsel_acts[nb_tsel_acts].act_type = !stricmp(arg, "-group-rem") ? 2 : ( !stricmp(arg, "-group-rem-track") ? 1 : 0 );
+			if (parse_tsel_args(tsel_acts, argv[i+1], &nb_tsel_acts)==0) {
+				fprintf(stdout, "Invalid group syntax - check usage\n");
+				return 1;
+			}
+			open_edit=1;
+			i++;
+		}
+		else if (!stricmp(arg, "-group-clean")) { 
+			tsel_acts[nb_tsel_acts].act_type = 3;
+			nb_tsel_acts++;
+			open_edit=1;
+		}
+
 		else if (!stricmp(arg, "-package")) {
 			CHECK_NEXT_ARG 
 			pack_file  = argv[i+1];
@@ -1618,7 +1810,7 @@ int main(int argc, char **argv)
 			else PrintUsage();
 			return 0;
 		}
-		else if (!stricmp(arg, "-v")) verbose = 1;
+		else if (!stricmp(arg, "-v")) verbose++;
 		else if (!stricmp(arg, "-tag-list")) {
 			fprintf(stdout, "Supported iTunes tag modifiers:\n");
 			for (i=0; i<nb_itunes_tags; i++) {
@@ -1662,6 +1854,7 @@ int main(int argc, char **argv)
 		return 0;
 	}
 
+//	gf_log_set_level((verbose>1) ? GF_LOG_DEBUG : (verbose ? GF_LOG_INFO : GF_LOG_WARNING) );
 	gf_log_set_level(verbose ? GF_LOG_DEBUG : GF_LOG_INFO);
 	gf_log_set_tools(GF_LOG_CONTAINER|GF_LOG_SCENE|GF_LOG_PARSER|GF_LOG_AUTHOR|GF_LOG_CODING);
 	if (quiet) {
@@ -1773,7 +1966,7 @@ int main(int argc, char **argv)
 			}
 		}
 		for (i=0; i<nb_cat; i++) {
-			e = cat_isomedia_file(file, szFilesToCat[i], import_flags, import_fps, agg_samples, tmpdir);
+			e = cat_isomedia_file(file, szFilesToCat[i], import_flags, import_fps, agg_samples, tmpdir, force_cat);
 			if (e) {
 				fprintf(stdout, "Error appending %s: %s\n", szFilesToCat[i], gf_error_to_string(e));
 				gf_isom_delete(file);
@@ -1857,7 +2050,7 @@ int main(int argc, char **argv)
 		case 1:
 			file = gf_isom_open(inName, (u8) (open_edit ? GF_ISOM_OPEN_EDIT : ( (dump_isom>0) ? GF_ISOM_OPEN_READ_DUMP : GF_ISOM_OPEN_READ) ), tmpdir);
 			if (!file) {
-				if (open_edit) {
+				if (open_edit && nb_meta_act) {
 					file = gf_isom_open(inName, GF_ISOM_WRITE_EDIT, tmpdir);
 					if (!outName && file) outName = inName;
 				}
@@ -1909,18 +2102,17 @@ int main(int argc, char **argv)
 	}
 
 	strcpy(outfile, outName ? outName : inName);
-	if (strchr(outfile, '.')) {
-		char szExt[20];
-		strcpy(szExt, strchr(outfile, '.'));
-		strlwr(szExt);
-		/*turn on 3GP saving*/
-		if (!strcmp(szExt, ".3gp") || !strcmp(szExt, ".3gpp") || !strcmp(szExt, ".3g2")) 
-			conv_type = GF_ISOM_CONV_TYPE_3GPP;
-		else if (!strcmp(szExt, ".m4a") || !strcmp(szExt, ".m4v"))
-			conv_type = GF_ISOM_CONV_TYPE_IPOD;
-		else if (!strcmp(szExt, ".psp"))
-			conv_type = GF_ISOM_CONV_TYPE_PSP;
+	if (strrchr(outfile, '.')) {
+		char *szExt = strrchr(outfile, '.');
 
+		/*turn on 3GP saving*/
+		if (!stricmp(szExt, ".3gp") || !stricmp(szExt, ".3gpp") || !stricmp(szExt, ".3g2")) 
+			conv_type = GF_ISOM_CONV_TYPE_3GPP;
+		else if (!stricmp(szExt, ".m4a") || !stricmp(szExt, ".m4v"))
+			conv_type = GF_ISOM_CONV_TYPE_IPOD;
+		else if (!stricmp(szExt, ".psp"))
+			conv_type = GF_ISOM_CONV_TYPE_PSP;
+		
 		while (outfile[strlen(outfile)-1] != '.') outfile[strlen(outfile)-1] = 0;
 		outfile[strlen(outfile)-1] = 0;
 	}
@@ -2069,6 +2261,37 @@ int main(int argc, char **argv)
 	}
 
 #ifndef GPAC_READ_ONLY
+
+	for (i=0; i<nb_tsel_acts; i++) {
+		switch (tsel_acts[i].act_type) {
+		case 0:
+			e = gf_isom_set_track_switch_parameter(file, 
+				gf_isom_get_track_by_id(file, tsel_acts[i].trackID),
+				tsel_acts[i].refTrackID ? gf_isom_get_track_by_id(file, tsel_acts[i].refTrackID) : 0,
+				tsel_acts[i].is_switchGroup ? 1 : 0,
+				&tsel_acts[i].switchGroupID,
+				tsel_acts[i].criteria, tsel_acts[i].nb_criteria);
+			if (e) goto err_exit;
+			needSave = 1;
+			break;
+		case 1:
+			e = gf_isom_reset_track_switch_parameter(file, gf_isom_get_track_by_id(file, tsel_acts[i].trackID), 0);
+			if (e) goto err_exit;
+			needSave = 1;
+			break;
+		case 2:
+			e = gf_isom_reset_track_switch_parameter(file, gf_isom_get_track_by_id(file, tsel_acts[i].trackID), 1);
+			if (e) goto err_exit;
+			needSave = 1;
+			break;
+		case 3:
+			e = gf_isom_reset_switch_parameters(file);
+			if (e) goto err_exit;
+			needSave = 1;
+			break;
+		}
+	}
+
 	if (remove_sys_tracks) {
 		remove_systems_tracks(file);
 		needSave = 1;
@@ -2092,18 +2315,30 @@ int main(int argc, char **argv)
 
 
 	if (!encode) {
+		if (!file) {
+			fprintf(stdout, "Nothing to do - exiting\n");
+			gf_sys_close();
+			return 0;
+		}
 		if (outName) {
 			strcpy(outfile, outName);
 		} else {
 			char *rel_name = strrchr(inName, GF_PATH_SEPARATOR);
 			if (!rel_name) rel_name = strrchr(inName, '/');
+
+			strcpy(outfile, "");
+			if (tmpdir) {
+				strcpy(outfile, tmpdir);
+				if (!strchr("\\/", tmpdir[strlen(tmpdir)-1])) strcat(outfile, "/");
+			}
+			if (!pack_file) strcat(outfile, "out_");
+			strcat(outfile, rel_name ? rel_name + 1 : inName);
+
 			if (pack_file) {
 				strcpy(outfile, rel_name ? rel_name + 1 : inName);
 				rel_name = strrchr(outfile, '.');
 				if (rel_name) rel_name[0] = 0;
 				strcat(outfile, ".m21");
-			} else {
-				sprintf(outfile, "out_%s", rel_name ? rel_name + 1 : inName);
 			}
 		}
 		if ((conv_type == GF_ISOM_CONV_TYPE_ISMA) || (conv_type == GF_ISOM_CONV_TYPE_ISMA_EX)) {
@@ -2234,6 +2469,22 @@ int main(int argc, char **argv)
 			e = gf_isom_set_handler_name(file, track, tka->hdl_name);
 			needSave = 1;
 			break;
+		case 6:
+			if (!gf_isom_is_track_enabled(file, track)) {
+				e = gf_isom_set_track_enabled(file, track, 1);
+				needSave = 1;
+			}
+			break;
+		case 7:
+			if (gf_isom_is_track_enabled(file, track)) {
+				e = gf_isom_set_track_enabled(file, track, 0);
+				needSave = 1;
+			}
+			break;
+		case 8:
+			e = gf_isom_set_track_reference(file, track, GF_4CC(tka->lang[0], tka->lang[1], tka->lang[2], tka->lang[3]), (u32) tka->delay_ms);
+			needSave = 1;
+			break;
 		}
 		if (e) goto err_exit;
 	}
@@ -2359,7 +2610,7 @@ int main(int argc, char **argv)
 		gf_isom_delete(file);
 		if (!e && !outName && !force_new) {
 			if (remove(inName)) fprintf(stdout, "Error removing file %s\n", inName);
-			else if (rename(outfile, inName)) fprintf(stdout, "Error renaming file %s\n", outfile);
+			else if (rename(outfile, inName)) fprintf(stdout, "Error renaming file %s to %s\n", outfile, inName);
 		}
 		gf_sys_close();
 		return (e!=GF_OK) ? 1 : 0;
@@ -2459,10 +2710,10 @@ int main(int argc, char **argv)
 		else fprintf(stdout, "%.3f secs Interleaving%s\n", InterleavingTime, old_interleave ? " - no drift control" : "");
 
 		e = gf_isom_close(file);
-		if (e) goto err_exit;
-		if (!outName && !encode && !force_new && !pack_file) {
+
+		if (!e && !outName && !encode && !force_new && !pack_file) {
 			if (remove(inName)) fprintf(stdout, "Error removing file %s\n", inName);
-			else if (rename(outfile, inName)) fprintf(stdout, "Error renaming file %s\n", outfile);
+			else if (rename(outfile, inName)) fprintf(stdout, "Error renaming file %s to %s\n", outfile, inName);
 		}
 	} else {
 		gf_isom_delete(file);
