@@ -42,12 +42,14 @@
 
 /*local prototypes*/
 void PrintWorldInfo(GF_Terminal *term);
-void ViewOD(GF_Terminal *term, u32 OD_ID);
+void ViewOD(GF_Terminal *term, u32 OD_ID, u32 number);
 void PrintODList(GF_Terminal *term);
 void ViewODs(GF_Terminal *term, Bool show_timing);
 void PrintGPACConfig();
 
 static Bool not_threaded = 0;
+static Bool no_audio = 0;
+static Bool no_regulation = 0;
 Bool is_connected = 0;
 Bool startup_file = 0;
 GF_User user;
@@ -55,14 +57,20 @@ GF_Terminal *term;
 u64 Duration;
 GF_Err last_error = GF_OK;
 
+static Bool request_next_playlist_item = 0;
+
 static GF_Config *cfg_file;
 static Bool display_rti = 0;
 static Bool Run;
 static Bool CanSeek = 0;
 static u32 Volume=100;
 static char the_url[GF_MAX_PATH];
-static Bool no_mime_check = 0;
+static Bool no_mime_check = 1;
 static Bool be_quiet = 0;
+static u32 log_time_start = 0;
+
+static u32 forced_width=0;
+static u32 forced_height=0;
 
 /*windowless options*/
 u32 align_mode = 0;
@@ -73,7 +81,7 @@ Bool right_down = 0;
 
 #ifndef GPAC_READ_ONLY
 void dump_frame(GF_Terminal *term, char *rad_path, u32 dump_type, u32 frameNum);
-Bool dump_file(char *the_url, u32 dump_mode, Double fps, u32 width, u32 height, u32 *times, u32 nb_times);
+Bool dump_file(char *the_url, u32 dump_mode, Double fps, u32 width, u32 height, Float scale, u32 *times, u32 nb_times);
 #endif
 
 void PrintUsage()
@@ -81,6 +89,7 @@ void PrintUsage()
 	fprintf(stdout, "Usage MP4Client [options] [filename]\n"
 		"\t-c fileName:    user-defined configuration file\n"
 		"\t-rti fileName:  logs run-time info (FPS, CPU, Mem usage) to file\n"
+		"\t-rtix fileName:  same as -rti but driven by GPAC logs\n"
 		"\t-quiet:         removes script message, buffering and downloading status\n"
 		"\t-log-file file: sets output log file.\n"
 		"\t-log-level lev: sets log level. Possible values are:\n"
@@ -102,15 +111,17 @@ void PrintUsage()
 		"\t        \"media\"      : terminal media object management\n"
 		"\t        \"scene\"      : scene graph and scene manager\n"
 		"\t        \"script\"     : scripting engine messages\n"
-		"\t        \"compose\"    : composition engine (events, etc)\n"
-		"\t        \"render\"     : renderng engine (2D, 3D, etc)\n"
+		"\t        \"interact\"   : interaction engine (events, scripts, etc)\n"
+		"\t        \"compose\"    : composition engine (2D, 3D, etc)\n"
 		"\t        \"service\"    : network service management\n"
 		"\t        \"mmio\"       : Audio/Video HW I/O management\n"
 		"\t        \"none\"       : no tool logged\n"
 		"\t        \"all\"        : all tools logged\n"
 		"\n"
 		"\t-size WxH:      specifies visual size (default: scene size)\n"
+		"\t-scale s:      scales the visual size (default: 1)\n"
 		"\t-no-thread:     disables thread usage (except for audio)\n"
+		"\t-no-audio:	   disables audio \n"
 		"\t-no-wnd:        uses windowless mode (Win32 only)\n"
 		"\t-align vh:      specifies v and h alignment for windowless mode\n"
 		"                   possible v values: t(op), m(iddle), b(ottom)\n"
@@ -120,15 +131,22 @@ void PrintUsage()
 		"\t-bmp [times]:   dumps given frames to bmp\n"
 		"\t-raw [times]:   dumps given frames to bmp\n"
 		"\t-avi [times]:   dumps given file to raw avi\n"
+		"\t-rgbds:         dumps the RGBDS pixel format texture\n"
+		"                   with -avi [times]: dumps an rgbds-format .avi\n"
+		"\t-rgbd:          dumps the RGBD pixel format texture\n"
+		"					with -avi [times]: dumps an rgbd-format .avi\n"		
+		"\t-depth:         dumps depthmap (z-buffer) frames\n"
+		"                   with -avi [times]: dumps depthmap in grayscale .avi\n"		
+		"                   with -bmp: dumps depthmap in grayscale .bmp\n"		
 		"\t-fps FPS:       specifies frame rate for AVI dumping (default: 25.0)\n"
-		"\t-2d:            uses 2D renderer\n"
-		"\t-3d:            uses 3D renderer\n"
+		"\t-2d:            uses 2D compositor\n"
+		"\t-3d:            uses 3D compositor\n"
 		"\t-fill:          uses fill aspect ratio for dumping (default: none)\n"
 		"\t-show:          show window while dumping (default: no)\n"
 		"MP4Client - GPAC command line player and dumper - version %s\n"
 		"GPAC Written by Jean Le Feuvre (c) 2001-2005 - ENST (c) 2005-200X\n",
 
-		GPAC_VERSION
+		GPAC_FULL_VERSION
 		);
 }
 
@@ -137,7 +155,7 @@ void PrintHelp()
 	fprintf(stdout, "MP4Client command keys:\n"
 		"\to: connect to the specified URL\n"
 		"\tO: connect to the specified URL in playlist mode\n"
-		"\tN: switch to the next URL in the playlist\n"
+		"\tN: switch to the next URL in the playlist (works with return key as well)\n"
 		"\tr: restart current presentation\n"
 		"\tp: play/pause the presentation\n"
 		"\ts: step one frame ahead\n"
@@ -146,7 +164,8 @@ void PrintHelp()
 		"\n"
 		"\tw: view world info\n"
 		"\tv: view Object Descriptor list\n"
-		"\ti: view Object Descriptor info\n"
+		"\ti: view Object Descriptor info (by ID)\n"
+		"\tj: view Object Descriptor info (by number)\n"
 		"\tb: view media objects timing and buffering info\n"
 		"\tm: view media objects buffering and memory info\n"
 		"\td: dumps scene graph\n"
@@ -159,8 +178,8 @@ void PrintHelp()
 		"\tn: changes navigation mode\n"
 		"\tx: reset to last active viewpoint\n"
 		"\n"
-		"\t2: restart using 2D renderer\n"
-		"\t3: restart using 3D renderer\n"
+		"\t2: restart using 2D compositor\n"
+		"\t3: restart using 3D compositor\n"
 		"\n"
 		"\t4: forces 4/3 Aspect Ratio\n"
 		"\t5: forces 16/9 Aspect Ratio\n"
@@ -168,7 +187,7 @@ void PrintHelp()
 		"\t7: forces original Aspect Ratio (default)\n"
 		"\n"
 		"\tL: changes to new log level. CF MP4Client usage for possible values\n"
-		"\tM: select new tools to log. CF MP4Client usage for possible values\n"
+		"\tT: select new tools to log. CF MP4Client usage for possible values\n"
 		"\n"
 		"\tl: list available modules\n"
 		"\tc: prints some GPAC configuration info\n"
@@ -179,7 +198,7 @@ void PrintHelp()
 		"MP4Client - GPAC command line player - version %s\n"
 		"GPAC Written by Jean Le Feuvre (c) 2001-2005 - ENST (c) 2005-200X\n",
 
-		GPAC_VERSION
+		GPAC_FULL_VERSION
 		);
 }
 
@@ -202,7 +221,9 @@ GF_Config *create_default_config(char *file_path, char *file_name)
 #ifdef GPAC_MODULES_PATH
 	fprintf(stdout, "Using module directory %s \n", GPAC_MODULES_PATH);
 	strcpy(szPath, GPAC_MODULES_PATH);
-#else
+#elif defined(WIN32)
+	strcpy(szPath, file_path);	
+#else 
 	fprintf(stdout, "Please enter full path to GPAC modules directory:\n");
 	scanf("%s", szPath);
 #endif
@@ -211,8 +232,7 @@ GF_Config *create_default_config(char *file_path, char *file_name)
 	gf_cfg_set_key(cfg, "Audio", "NumBuffers", "2");
 	gf_cfg_set_key(cfg, "Audio", "TotalDuration", "120");
 	gf_cfg_set_key(cfg, "Audio", "DisableNotification", "no");
-	gf_cfg_set_key(cfg, "Rendering", "RendererName", "GPAC 2D Renderer");
-	gf_cfg_set_key(cfg, "FontEngine", "DriverName", "ft_font");
+	gf_cfg_set_key(cfg, "FontEngine", "FontReader", "ft_font");
 
 #ifdef WIN32
 	GetWindowsDirectory((char*)szPath, MAX_PATH);
@@ -232,39 +252,47 @@ GF_Config *create_default_config(char *file_path, char *file_name)
 	gf_cfg_set_key(cfg, "FontEngine", "FontDirectory", szPath);
 
 #ifdef WIN32
-	fprintf(stdout, "Please enter full path to a cache directory for HTTP downloads:\n");
+/*	fprintf(stdout, "Please enter full path to a cache directory for HTTP downloads:\n");
 	scanf("%s", szPath);
+*/
+	GetWindowsDirectory((char*)szPath, MAX_PATH);
+	if (szPath[strlen((char*)szPath)-1] != '\\') strcat((char*)szPath, "\\");
+	strcat((char *)szPath, "Temp");
+
 	gf_cfg_set_key(cfg, "General", "CacheDirectory", szPath);
+	fprintf(stdout, "Using default cache directory %s\n", szPath);
 #else
 	fprintf(stdout, "Using /tmp as a cache directory for HTTP downloads:\n");
 	gf_cfg_set_key(cfg, "General", "CacheDirectory", "/tmp");
 #endif
 
 	gf_cfg_set_key(cfg, "Downloader", "CleanCache", "yes");
-	gf_cfg_set_key(cfg, "Rendering", "AntiAlias", "All");
-	gf_cfg_set_key(cfg, "Rendering", "Framerate", "30");
+	gf_cfg_set_key(cfg, "Compositor", "AntiAlias", "All");
+	gf_cfg_set_key(cfg, "Compositor", "FrameRate", "30");
 	/*use power-of-2 emulation*/
-	gf_cfg_set_key(cfg, "Render3D", "EmulatePOW2", "yes");
+	gf_cfg_set_key(cfg, "Compositor", "EmulatePOW2", "yes");
 #ifdef WIN32
-	gf_cfg_set_key(cfg, "Render2D", "ScalableZoom", "yes");
+	gf_cfg_set_key(cfg, "Compositor", "ScalableZoom", "yes");
 	gf_cfg_set_key(cfg, "Video", "DriverName", "DirectX Video Output");
 #else
 #ifdef __DARWIN__
 	gf_cfg_set_key(cfg, "Video", "DriverName", "SDL Video Output");
 	/*SDL not so fast with scalable zoom*/
-	gf_cfg_set_key(cfg, "Render2D", "ScalableZoom", "no");
+	gf_cfg_set_key(cfg, "Compositor", "ScalableZoom", "no");
 #else
 	gf_cfg_set_key(cfg, "Video", "DriverName", "X11 Video Output");
 	/*x11 only supports scalable zoom*/
-	gf_cfg_set_key(cfg, "Render2D", "ScalableZoom", "yes");
+	gf_cfg_set_key(cfg, "Compositor", "ScalableZoom", "yes");
 	gf_cfg_set_key(cfg, "Audio", "DriverName", "SDL Audio Output");
 #endif
 #endif
 	gf_cfg_set_key(cfg, "Video", "SwitchResolution", "no");
 	gf_cfg_set_key(cfg, "Network", "AutoReconfigUDP", "yes");
-	gf_cfg_set_key(cfg, "Network", "UDPNotAvailable", "no");
 	gf_cfg_set_key(cfg, "Network", "UDPTimeout", "10000");
 	gf_cfg_set_key(cfg, "Network", "BufferLength", "3000");
+#ifdef GPAC_TRISCOPE_MODE
+	gf_cfg_set_key(cfg, "Compositor", "OGLDepthBuffGain", "5");
+#endif
 
 	/*store and reload*/
 	gf_cfg_del(cfg);
@@ -281,30 +309,19 @@ static void PrintTime(u64 time)
 	fprintf(stdout, "%02d:%02d:%02d.%02d", h, m, s, ms);
 }
 
-#define RTI_UPDATE_TIME_MS	200
+
+static u32 rti_update_time_ms = 200;
 static FILE *rti_logs = NULL;
 static u64 memory_at_gpac_startup = 0;
-static u64 memory_at_gpac_load = 0;
-
-static void init_rti_logs(char *rti_file, char *url)
-{
-	if (rti_logs) fclose(rti_logs);
-	rti_logs = fopen(rti_file, "wt");
-	if (rti_logs) {
-		fprintf(rti_logs, "!! GPAC RunTime Info ");
-		if (url) fprintf(rti_logs, "for file %s", url);
-		fprintf(rti_logs, " !!\n");
-		fprintf(rti_logs, "SysTime(ms)\tSceneTime(ms)\tCPU\tFPS\tMemory(kB)\tObservation\n");
-		memory_at_gpac_load = 0;
-	}
-}
 
 static void UpdateRTInfo(const char *legend)
 {
 	GF_SystemRTInfo rti;
 
 	/*refresh every second*/
-	if ((!display_rti && !rti_logs) || !gf_sys_get_rti(RTI_UPDATE_TIME_MS, &rti, 0 /*GF_RTI_ALL_PROCESSES_TIMES | GF_RTI_PROCESS_MEMORY*/) /*|| !rti.sampling_period_duration*/) return;
+	if (!display_rti && !rti_logs) return;
+	if (!gf_sys_get_rti(rti_update_time_ms, &rti, 0) && !legend) 
+		return;
 
 	if (display_rti) {
 		char szMsg[1024];
@@ -320,15 +337,15 @@ static void UpdateRTInfo(const char *legend)
 		gf_term_user_event(term, &evt);
 	}
 	if (rti_logs) {
-		if (!memory_at_gpac_load) memory_at_gpac_load = rti.gpac_memory;
-		fprintf(rti_logs, "%d\t%d\t%d\t%d\t%d\t%s\n", 
+		fprintf(rti_logs, "% 8d\t% 8d\t% 8d\t% 4d\t% 8d\t%s", 
 			gf_sys_clock(),
 			gf_term_get_time_in_ms(term),
 			rti.total_cpu_usage,
 			(u32) gf_term_get_framerate(term, 0),
-			(u32) ((rti.gpac_memory - memory_at_gpac_load) / 1024), 
-			legend
+			(u32) (rti.gpac_memory / 1024), 
+			legend ? legend : ""
 			);
+		if (!legend) fprintf(rti_logs, "\n");
 	}
 }
 
@@ -362,7 +379,7 @@ static void ResetCaption()
 			event.caption.caption = str ? str+1 : the_url;
 		}
 	} else {
-		event.caption.caption = "GPAC MP4Client " GPAC_VERSION;
+		event.caption.caption = "GPAC MP4Client " GPAC_FULL_VERSION;
 	}
 	gf_term_user_event(term, &event);
 }
@@ -384,13 +401,6 @@ Bool GPAC_EventProc(void *ptr, GF_Event *evt)
 	if (!term) return 0;
 
 	switch (evt->type) {
-	case GF_EVENT_UPDATE_RTI:
-		UpdateRTInfo(evt->caption.caption);
-		break;
-	case GF_EVENT_RESET_RTI:
-		memory_at_gpac_load = 0;
-		UpdateRTInfo(evt->caption.caption);
-		break;
 	case GF_EVENT_DURATION:
 		Duration = 1000;
 		Duration = (u64) (((s64) Duration) * evt->duration.duration);
@@ -425,7 +435,7 @@ Bool GPAC_EventProc(void *ptr, GF_Event *evt)
 		break;
 	
 
-	case GF_EVENT_MOUSEDOUBLECLICK:
+	case GF_EVENT_DBLCLICK:
 		gf_term_set_option(term, GF_OPT_FULLSCREEN, !gf_term_get_option(term, GF_OPT_FULLSCREEN));
 		return 0;
 
@@ -485,6 +495,9 @@ Bool GPAC_EventProc(void *ptr, GF_Event *evt)
 			case GF_KEY_DOWN: 
 				if (Volume) { Volume = (Volume > 5) ? (Volume-5) : 0; gf_term_set_option(term, GF_OPT_AUDIO_VOLUME, Volume); }
 				break;
+			case GF_KEY_PAGEDOWN:
+				if (Volume!=100) { Volume = MIN(Volume + 5, 100); gf_term_set_option(term, GF_OPT_AUDIO_VOLUME, Volume); } 
+				break;
 			}
 		} else {
 			switch (evt->key.key_code) {
@@ -494,6 +507,9 @@ Bool GPAC_EventProc(void *ptr, GF_Event *evt)
 			case GF_KEY_ESCAPE:
 				gf_term_set_option(term, GF_OPT_FULLSCREEN, !gf_term_get_option(term, GF_OPT_FULLSCREEN));
 				break;
+			case GF_KEY_PAGEDOWN:
+				request_next_playlist_item = 1;
+				break;
 			}
 		}
 
@@ -506,7 +522,7 @@ Bool GPAC_EventProc(void *ptr, GF_Event *evt)
 			fprintf(stdout, "Scene Time: %f \n", gf_term_get_time_in_ms(term)/1000.0);
 			break;
 		case GF_KEY_D:
-			gf_term_set_option(term, GF_OPT_DIRECT_RENDER, !gf_term_get_option(term, GF_OPT_DIRECT_RENDER) );
+			gf_term_set_option(term, GF_OPT_DIRECT_DRAW, !gf_term_get_option(term, GF_OPT_DIRECT_DRAW) );
 			break;
 		case GF_KEY_4: gf_term_set_option(term, GF_OPT_ASPECT_RATIO, GF_ASPECT_RATIO_4_3); break;
 		case GF_KEY_5: gf_term_set_option(term, GF_OPT_ASPECT_RATIO, GF_ASPECT_RATIO_16_9); break;
@@ -531,7 +547,6 @@ Bool GPAC_EventProc(void *ptr, GF_Event *evt)
 		if (evt->connect.is_connected) {
 			is_connected = 1;
 			fprintf(stdout, "Service Connected\n");
-			/* memory_at_gpac_load = 0;*/
 		} else if (is_connected) {
 			fprintf(stdout, "Service %s\n", is_connected ? "Disconnected" : "Connection Failed");
 			is_connected = 0;
@@ -550,6 +565,15 @@ Bool GPAC_EventProc(void *ptr, GF_Event *evt)
 			move.move.align_y = (align_mode>>8) & 0xFF;
 			move.move.relative = 2;
 			gf_term_user_event(term, &move);
+		} 
+		break;
+	case GF_EVENT_SCENE_SIZE:
+		if (forced_width && forced_height) {
+			GF_Event size;
+			size.type = GF_EVENT_SIZE;
+			size.size.width = forced_width;
+			size.size.height = forced_height;
+			gf_term_user_event(term, &size);
 		}
 		break;
 
@@ -560,6 +584,19 @@ Bool GPAC_EventProc(void *ptr, GF_Event *evt)
 	case GF_EVENT_QUIT:
 		Run = 0;
 		break;
+	case GF_EVENT_DISCONNECT:
+		gf_term_disconnect(term);
+		break;
+	case GF_EVENT_MIGRATE:
+	{
+		const char *str = gf_cfg_get_key(cfg_file, "Network", "SessionMigration");
+		if (!str || !strcmp(str, "no"))
+			gf_cfg_set_key(cfg_file, "Network", "SessionMigration", "yes");
+
+		fprintf(stdout, "Migrating session %s\n", the_url);
+		gf_term_disconnect(term);
+	}
+		break;
 	case GF_EVENT_NAVIGATE_INFO:
 		if (evt->navigate.to_url) fprintf(stdout, "Go to URL: \"%s\"\r", evt->navigate.to_url);
 		break;
@@ -573,6 +610,9 @@ Bool GPAC_EventProc(void *ptr, GF_Event *evt)
 			fprintf(stdout, "Navigation destination not supported\nGo to URL: %s\n", evt->navigate.to_url);
 		}
 		break;
+	case GF_EVENT_SET_CAPTION:
+		gf_term_user_event(term, evt);
+		break;
 	case GF_EVENT_AUTHORIZATION:
 		if (!strlen(evt->auth.user)) {
 			fprintf(stdout, "Authorization required for site %s\n", evt->auth.site_url);
@@ -654,9 +694,6 @@ GF_Config *loadconfigfile(char *filepath)
 	strcpy(szPath, ".");
 	cfg = gf_cfg_new(szPath, "GPAC.cfg");
 	if (cfg) goto success;
-	strcpy(szPath, "C:\\Program Files\\GPAC");
-	cfg = gf_cfg_new(szPath, "GPAC.cfg");
-	if (cfg) goto success;
 	strcpy(szPath, ".");
 	cfg = gf_cfg_new(szPath, "GPAC.cfg");
 	if (cfg) goto success;
@@ -706,7 +743,7 @@ void set_navigation()
 	u32 type = gf_term_get_option(term, GF_OPT_NAVIGATION_TYPE);
 	e = GF_OK;
 	if (!type) {
-		fprintf(stdout, "Content/renderer doesn't allow user-selectable navigation\n");
+		fprintf(stdout, "Content/compositor doesn't allow user-selectable navigation\n");
 	} else if (type==1) {
 		fprintf(stdout, "Select Navigation (\'N\'one, \'E\'xamine, \'S\'lide): ");
 		scanf("%s", navstr);
@@ -752,12 +789,14 @@ static u32 parse_log_tools(char *val)
 		else if (!stricmp(val, "media")) flags |= GF_LOG_MEDIA;
 		else if (!stricmp(val, "scene")) flags |= GF_LOG_SCENE;
 		else if (!stricmp(val, "script")) flags |= GF_LOG_SCRIPT;
+		else if (!stricmp(val, "interact")) flags |= GF_LOG_INTERACT;
+		else if (!stricmp(val, "smil")) flags |= GF_LOG_SMIL;
 		else if (!stricmp(val, "compose")) flags |= GF_LOG_COMPOSE;
-		else if (!stricmp(val, "render")) flags |= GF_LOG_RENDER;
-		else if (!stricmp(val, "service")) flags |= GF_LOG_SERVICE;
 		else if (!stricmp(val, "mmio")) flags |= GF_LOG_MMIO;
 		else if (!stricmp(val, "none")) flags = 0;
 		else if (!stricmp(val, "all")) flags = 0xFFFFFFFF;
+		else if (!stricmp(val, "rti")) flags |= GF_LOG_RTI;
+		else if (!stricmp(val, "cache")) flags |= GF_LOG_CACHE;
 		if (!sep) break;
 		sep[0] = ':';
 		val = sep+1;
@@ -788,7 +827,7 @@ static Bool get_time_list(char *arg, u32 *times, u32 *nb_times)
 			if (fps) sec = ((Double)f) / fps;
 			sec += 3600*h + 60*m + s;
 			times[*nb_times] = (u32) (1000*sec);
-			*nb_times ++;
+			(*nb_times) ++;
 			return 1;
 		}
 	}
@@ -817,37 +856,92 @@ static Bool get_time_list(char *arg, u32 *times, u32 *nb_times)
 static void on_gpac_log(void *cbk, u32 ll, u32 lm, const char *fmt, va_list list)
 {
 	FILE *logs = cbk;
-    vfprintf(logs, fmt, list);
-	fflush(logs);
+
+	if (rti_logs && (lm & GF_LOG_RTI)) {
+		char szMsg[2048];
+		vsprintf(szMsg, fmt, list);
+		UpdateRTInfo(szMsg + 6 /*"[RTI] "*/);
+	} else {
+		if (log_time_start) fprintf(logs, "[At %d]", gf_sys_clock() - log_time_start);
+		vfprintf(logs, fmt, list);
+		fflush(logs);
+	}
+}
+
+static void init_rti_logs(char *rti_file, char *url, Bool use_rtix)
+{
+	if (rti_logs) fclose(rti_logs);
+	rti_logs = fopen(rti_file, "wt");
+	if (rti_logs) {
+		fprintf(rti_logs, "!! GPAC RunTime Info ");
+		if (url) fprintf(rti_logs, "for file %s", url);
+		fprintf(rti_logs, " !!\n");
+		fprintf(rti_logs, "SysTime(ms)\tSceneTime(ms)\tCPU\tFPS\tMemory(kB)\tObservation\n");
+
+		/*turn on RTI loging*/
+		if (use_rtix) {
+			gf_log_set_callback(NULL, on_gpac_log);
+			gf_log_set_level(GF_LOG_DEBUG);
+			gf_log_set_tools(GF_LOG_RTI);
+
+			GF_LOG(GF_LOG_DEBUG, GF_LOG_RTI, ("[RTI] System state when enabling log"));
+		} else if (log_time_start) {
+			log_time_start = gf_sys_clock();
+		}
+	}
 }
 
 int main (int argc, char **argv)
 {
 	const char *str;
-	u32 i, width, height, times[100], nb_times, rend_mode, dump_mode;
+	u32 i, times[100], nb_times, dump_mode;
+	u32 simulation_time = 0;
 	Bool start_fs = 0;
+	Bool use_rtix = 0;
+	Bool rgbds_dump = 0;
+	Bool rgbd_dump = 0;
+	Bool depth_dump = 0;
 	Double fps = 25.0;
 	Bool ret, fill_ar, visible;
 	char *url_arg, *the_cfg, *rti_file;
 	GF_SystemRTInfo rti;
 	FILE *playlist = NULL;
 	FILE *logfile = NULL;
+	Float scale = 1;
 
 	/*by default use current dir*/
 	strcpy(the_url, ".");
 
 	memset(&user, 0, sizeof(GF_User));
 
-	dump_mode = rend_mode = 0;
+	dump_mode = 0;
 	fill_ar = visible = 0;
 	url_arg = the_cfg = rti_file = NULL;
-	width = height = 0;
 	nb_times = 0;
 	times[0] = 0;
 
+	/*first locate config file if specified*/
 	for (i=1; i<(u32) argc; i++) {
 		char *arg = argv[i];
-		if (isalnum(arg[0]) || (arg[0]=='/') || (arg[0]=='.') || (arg[0]=='\\') ) {
+		if (!strcmp(arg, "-c") || !strcmp(arg, "-cfg")) {
+			the_cfg = argv[i+1];
+			i++;
+			break;
+		}
+	}
+	cfg_file = loadconfigfile(the_cfg);
+	if (!cfg_file) {
+		fprintf(stdout, "Error: Configuration File \"GPAC.cfg\" not found\n");
+		if (logfile) fclose(logfile);
+		return 1;
+	}
+
+
+
+	for (i=1; i<(u32) argc; i++) {
+		char *arg = argv[i];
+//		if (isalnum(arg[0]) || (arg[0]=='/') || (arg[0]=='.') || (arg[0]=='\\') ) {
+		if (arg[0] != '-') {
 			url_arg = arg;
 		} else if (!strcmp(arg, "-c") || !strcmp(arg, "-cfg")) {
 			the_cfg = argv[i+1];
@@ -855,43 +949,66 @@ int main (int argc, char **argv)
 		} else if (!strcmp(arg, "-rti")) {
 			rti_file = argv[i+1];
 			i++;
+		} else if (!strcmp(arg, "-rtix")) {
+			rti_file = argv[i+1];
+			i++;
+			use_rtix = 1;
 		} else if (!strcmp(arg, "-fill")) {
 			fill_ar = 1;
 		} else if (!strcmp(arg, "-show")) {
 			visible = 1;
 		} else if (!strcmp(arg, "-avi")) {
-			dump_mode = 1;
+			if (rgbds_dump) dump_mode = 5;
+			else if (depth_dump) dump_mode = 8;
+			else if (rgbd_dump) dump_mode = 10;
+			else dump_mode=1;
 			if ((url_arg || (i+2<(u32)argc)) && get_time_list(argv[i+1], times, &nb_times)) i++;
+		} else if (!strcmp(arg, "-rgbds")) { /*get dump in rgbds pixel format*/
+			rgbds_dump = 1;
+			dump_mode=6;                    /* rgbds texture directly*/
+			if (dump_mode==1) dump_mode = 5;    /* .avi rgbds dump*/
+		} else if (!strcmp(arg, "-rgbd")) { /*get dump in rgbd pixel format*/
+			rgbd_dump = 1;
+			dump_mode=9;  /* rgbd texture directly*/
+			if (dump_mode==1) dump_mode = 10;    /* .avi rgbds dump*/
+		} else if (!strcmp(arg, "-depth")) {
+			depth_dump = 1;        
+			if (dump_mode==2) dump_mode=7; /* grayscale .bmp depth dump*/
+			else if (dump_mode==1) dump_mode=8; /* .avi depth dump*/
+			else dump_mode=4;   /*depth dump*/
 		} else if (!strcmp(arg, "-bmp")) {
-			dump_mode = 2;
+			if(depth_dump) dump_mode=7; /*grayscale depth .bmp dump*/
+			else dump_mode=2;
 			if ((url_arg || (i+2<(u32)argc)) && get_time_list(argv[i+1], times, &nb_times)) i++;
 		} else if (!strcmp(arg, "-raw")) {
 			dump_mode = 3;
 			if ((url_arg || (i+2<(u32)argc)) && get_time_list(argv[i+1], times, &nb_times)) i++;
+
 		} else if (!stricmp(arg, "-size")) {
-			if (sscanf(argv[i+1], "%dx%d", &width, &height) != 2) {
-				width = height = 0;
+			if (sscanf(argv[i+1], "%dx%d", &forced_width, &forced_height) != 2) {
+				forced_width = forced_height = 0;
 			}
 			i++;
+		} else if (!stricmp(arg, "-scale")) {
+			sscanf(argv[i+1], "%f", &scale);
+			i++;
 		} else if (!stricmp(arg, "-fps")) {
 			fps = atof(argv[i+1]);
 			i++;
-		} else if (!stricmp(arg, "-2d")) {
-			rend_mode = 1;
-		} else if (!stricmp(arg, "-3d")) {
-			rend_mode = 2;
 		} else if (!strcmp(arg, "-quiet")) {
 			be_quiet = 1;
-		} else if (!strcmp(arg, "-log-file")) {
+		} else if (!strcmp(arg, "-log-file") || !strcmp(arg, "-lf")) {
 			logfile = fopen(argv[i+1], "wt");
 			gf_log_set_callback(logfile, on_gpac_log);
 			i++;
-		} else if (!strcmp(arg, "-log-level")) {
+		} else if (!strcmp(arg, "-log-level") || !strcmp(arg, "-ll")) {
 			gf_log_set_level(parse_log_level(argv[i+1]));
 			i++;
-		} else if (!strcmp(arg, "-log-tools")) {
+		} else if (!strcmp(arg, "-log-tools") || !strcmp(arg, "-lt")) {
 			gf_log_set_tools(parse_log_tools(argv[i+1]));
 			i++;
+		} else if (!strcmp(arg, "-log-clock") || !strcmp(arg, "-lc")) {
+			log_time_start = 1;
 		} else if (!strcmp(arg, "-align")) {
 			if (argv[i+1][0]=='m') align_mode = 1;
 			else if (argv[i+1][0]=='b') align_mode = 2;
@@ -902,7 +1019,30 @@ int main (int argc, char **argv)
 		}
 		else if (!strcmp(arg, "-no-wnd")) user.init_flags |= GF_TERM_WINDOWLESS;
 		else if (!strcmp(arg, "-no-thread")) not_threaded = 1;
+		else if (!strcmp(arg, "-no-audio")) no_audio = 1;
+		else if (!strcmp(arg, "-no-regulation")) no_regulation = 1;
 		else if (!strcmp(arg, "-fs")) start_fs = 1;
+		else if (!strcmp(arg, "-opt")) {
+			char *sep, *sep2, szSec[1024], szKey[1024], szVal[1024];
+			sep = strchr(argv[i+1], ':');
+			if (sep) {
+				sep[0] = 0;
+				strcpy(szSec, argv[i+1]);
+				sep[0] = ':'; sep ++;
+				sep2 = strchr(sep, '=');
+				if (sep2) {
+					sep2[0] = 0;
+					strcpy(szKey, sep);
+					strcpy(szVal, sep2+1);
+					sep2[0] = '='; 
+					if (!stricmp(szVal, "null")) szVal[0]=0;
+					gf_cfg_set_key(cfg_file, szSec, szKey, szVal[0] ? szVal : NULL);
+				}
+			}
+			gf_cfg_set_key(cfg_file, szSec, szKey, szVal);
+			i++;
+		}
+		else if (!strncmp(arg, "-run-for=", 9)) simulation_time = atoi(arg+9);
 		else {
 			PrintUsage();
 			return 1;
@@ -915,55 +1055,48 @@ int main (int argc, char **argv)
 		return 1;
 	}
 	if (dump_mode) rti_file = NULL;
-	cfg_file = loadconfigfile(the_cfg);
-	if (!cfg_file) {
-		fprintf(stdout, "Error: Configuration File \"GPAC.cfg\" not found\n");
-		if (logfile) fclose(logfile);
-		return 1;
-	}
-
 	gf_sys_init();
 	
 	gf_sys_get_rti(500, &rti, GF_RTI_SYSTEM_MEMORY_ONLY);
 	memory_at_gpac_startup = rti.physical_memory_avail;
-	if (rti_file) init_rti_logs(rti_file, url_arg);
+	if (rti_file) init_rti_logs(rti_file, url_arg, use_rtix);
 
 	/*setup dumping options*/
 	if (dump_mode) {
-		if (rend_mode==2) user.init_flags |= GF_TERM_FORCE_3D;
-		else if (rend_mode==1) user.init_flags |= GF_TERM_FORCE_2D;
 		user.init_flags |= GF_TERM_NO_AUDIO | GF_TERM_NO_VISUAL_THREAD | GF_TERM_NO_REGULATION /*| GF_TERM_INIT_HIDE*/;
-		if (!visible) user.init_flags |= GF_TERM_INIT_HIDE;
+		if (visible || dump_mode==8) user.init_flags |= GF_TERM_INIT_HIDE;
 	} else {
-		init_w = width;
-		init_h = height;
+		init_w = forced_width;
+		init_h = forced_height;
 	}
 
-	fprintf(stdout, "Loading modules ... ");
+	fprintf(stdout, "Loading modules\n");
 	str = gf_cfg_get_key(cfg_file, "General", "ModulesDirectory");
 
 	user.modules = gf_modules_new((const unsigned char *) str, cfg_file);
-	i = gf_modules_get_count(user.modules);
-	if (!i) {
+	if (user.modules) i = gf_modules_get_count(user.modules);
+	if (!i || !user.modules) {
 		fprintf(stdout, "Error: no modules found in %s - exiting\n", str);
-		gf_modules_del(user.modules);
+		if (user.modules) gf_modules_del(user.modules);
 		gf_cfg_del(cfg_file);
 		gf_sys_close();
 		if (logfile) fclose(logfile);
 		return 1;
 	}
-	fprintf(stdout, "OK (%d found in %s)\n", i, str);
+	fprintf(stdout, "Modules Loaded (%d found in %s)\n", i, str);
 
 	user.config = cfg_file;
 	user.EventProc = GPAC_EventProc;
 	/*dummy in this case (global vars) but MUST be non-NULL*/
 	user.opaque = user.modules;
 	if (not_threaded) user.init_flags |= GF_TERM_NO_VISUAL_THREAD;
+	if (no_audio) user.init_flags |= GF_TERM_NO_AUDIO;
+	if (no_regulation) user.init_flags |= GF_TERM_NO_REGULATION;
 
-	fprintf(stdout, "Loading GPAC Terminal ... ");	
+	fprintf(stdout, "Loading GPAC Terminal\n");	
 	term = gf_term_new(&user);
 	if (!term) {
-		fprintf(stdout, "\nInit error - check you have at least one video out...\nFound modules:\n");
+		fprintf(stdout, "\nInit error - check you have at least one video out and one rasterizer...\nFound modules:\n");
 		list_modules(user.modules);
 		gf_modules_del(user.modules);
 		gf_cfg_del(cfg_file);
@@ -971,7 +1104,7 @@ int main (int argc, char **argv)
 		if (logfile) fclose(logfile);
 		return 1;
 	}
-	fprintf(stdout, "OK\n");
+	fprintf(stdout, "Terminal Loaded\n");
 
 
 	if (dump_mode) {
@@ -986,14 +1119,7 @@ int main (int argc, char **argv)
 		if (!strcmp(str, "No Audio Output Available")) fprintf(stdout, "WARNING: no audio output availble - make sure no other program is locking the sound card\n");
 
 		str = gf_cfg_get_key(cfg_file, "General", "NoMIMETypeFetch");
-		no_mime_check = (str && !stricmp(str, "yes")) ? 1 : 0;
-	}
-
-
-	if (rend_mode) {
-		fprintf(stdout, "Using %dD renderer\n", (rend_mode==2) ? 3 : 2);
-	} else {
-		fprintf(stdout, "Using %s\n", gf_cfg_get_key(cfg_file, "Rendering", "RendererName"));
+		no_mime_check = (!str || !stricmp(str, "yes")) ? 1 : 0;
 	}
 
 	str = gf_cfg_get_key(cfg_file, "HTTPProxy", "Enabled");
@@ -1003,8 +1129,13 @@ int main (int argc, char **argv)
 	}
 
 	if (rti_file) {
-		memory_at_gpac_load = 0;
-		UpdateRTInfo("Before connecting ...");
+		str = gf_cfg_get_key(cfg_file, "General", "RTIRefreshPeriod");
+		if (str) {
+			rti_update_time_ms = atoi(str);
+		} else {
+			gf_cfg_set_key(cfg_file, "General", "RTIRefreshPeriod", "200");
+		}
+		UpdateRTInfo("At GPAC load time\n");
 	}
 
 	Run = 1;
@@ -1015,7 +1146,7 @@ int main (int argc, char **argv)
 			times[0] = 0;
 			nb_times++;
 		}
-		ret = dump_file(url_arg, dump_mode, fps, width, height, times, nb_times);
+		ret = dump_file(url_arg, dump_mode, fps, forced_width, forced_height, scale, times, nb_times);
 		Run = 0;
 	} else
 #endif
@@ -1025,7 +1156,7 @@ int main (int argc, char **argv)
 		char *ext;
 		strcpy(the_url, url_arg);
 		ext = strrchr(the_url, '.');
-		if (ext && !stricmp(ext, ".m3u")) {
+		if (ext && (!stricmp(ext, ".m3u") || !stricmp(ext, ".pls"))) {
 			fprintf(stdout, "Opening Playlist %s\n", the_url);
 			playlist = fopen(the_url, "rt");
 			if (playlist) {
@@ -1043,11 +1174,15 @@ int main (int argc, char **argv)
 		fprintf(stdout, "Hit 'h' for help\n\n");
 		str = gf_cfg_get_key(cfg_file, "General", "StartupFile");
 		if (str) {
-			strcpy(the_url, "MP4Client "GPAC_VERSION);
+			strcpy(the_url, "MP4Client "GPAC_FULL_VERSION);
 			gf_term_connect(term, str);
 			startup_file = 1;
 		}
 	}
+	/*reset session migration*/
+	str = gf_cfg_get_key(cfg_file, "Network", "SessionMigration");
+	if (str && !strcmp(str, "yes"))
+		gf_cfg_set_key(cfg_file, "Network", "SessionMigration", "no");
 
 	if (start_fs) gf_term_set_option(term, GF_OPT_FULLSCREEN, 1);
 
@@ -1056,26 +1191,43 @@ int main (int argc, char **argv)
 		
 		/*we don't want getchar to block*/
 		if (!gf_prompt_has_input()) {
-//			UpdateRTInfo("");
+			if (request_next_playlist_item) {
+				c = '\n';
+				request_next_playlist_item = 0;
+				goto force_input;
+			}
+			if (!use_rtix || display_rti) UpdateRTInfo(NULL);
 			if (not_threaded) {
 				gf_term_process_step(term);
 			} else {
-				gf_sleep(RTI_UPDATE_TIME_MS);
+				gf_sleep(rti_update_time_ms);
+			}
+			/*sim time*/
+			if (simulation_time && (gf_term_get_time_in_ms(term)>simulation_time)) {
+				Run = 0;
 			}
 			continue;
 		}
 		c = gf_prompt_get_char();
 
+force_input:
 		switch (c) {
 		case 'q':
 			Run = 0;
 			break;
+		case 'Q':
+			str = gf_cfg_get_key(cfg_file, "Network", "SessionMigration");
+			if (!str || !strcmp(str, "no"))
+				gf_cfg_set_key(cfg_file, "Network", "SessionMigration", "yes");
+			fprintf(stdout, "Migrating session %s\n", the_url);
+			gf_term_disconnect(term);
+			break;
 		case 'o':
 			startup_file = 0;
 			gf_term_disconnect(term);
 			fprintf(stdout, "Enter the absolute URL\n");
 			scanf("%s", the_url);
-			if (rti_file) init_rti_logs(rti_file, the_url);
+			if (rti_file) init_rti_logs(rti_file, the_url, use_rtix);
 			gf_term_connect(term, the_url);
 			break;
 		case 'O':
@@ -1184,7 +1336,15 @@ int main (int argc, char **argv)
 				u32 ID;
 				fprintf(stdout, "Enter OD ID (0 for main OD): ");
 				scanf("%d", &ID);
-				ViewOD(term, ID);
+				ViewOD(term, ID, (u32)-1);
+			}
+			break;
+		case 'j':
+			if (is_connected) {
+				u32 num;
+				fprintf(stdout, "Enter OD number (0 for main OD): ");
+				scanf("%d", &num);
+				ViewOD(term, (u32)-1, num);
 			}
 			break;
 		case 'b':
@@ -1228,29 +1388,11 @@ int main (int argc, char **argv)
 		case 'c':
 			PrintGPACConfig();
 			break;
-		case '2':
 		case '3':
 		{
-			GF_Terminal *a_term;
-			u32 now = gf_term_get_time_in_ms(term);
-			Bool reconnect = (is_connected && !startup_file) ? 1 : 0;
-			str = gf_cfg_get_key(cfg_file, "Rendering", "RendererName");
-			if (strstr(str, "2D") && (c=='2')) { fprintf(stdout, "Already using 2D Renderer\n"); break; }
-			if (strstr(str, "3D") && (c=='3')) { fprintf(stdout, "Already using 3D Renderer\n"); break; }
-			if (is_connected) gf_term_disconnect(term);
-			a_term = term;
-			term = NULL;
-			gf_term_del(a_term);
-			gf_cfg_set_key(cfg_file, "Rendering", "RendererName", (c=='2') ? "GPAC 2D Renderer" : "GPAC 3D Renderer");
-			term = gf_term_new(&user);
-			if (!term) {
-				fprintf(stdout, "Error reloading renderer - aborting\n");
-				goto exit;
-			}
-			fprintf(stdout, "Using %s\n", gf_cfg_get_key(cfg_file, "Rendering", "RendererName"));
-			if (reconnect) gf_term_connect_from_time(term, the_url, now, 0);
-			else if (startup_file) {
-				gf_term_connect(term, gf_cfg_get_key(cfg_file, "General", "StartupFile"));
+			Bool use_3d = !gf_term_get_option(term, GF_OPT_USE_OPENGL);
+			if (gf_term_set_option(term, GF_OPT_USE_OPENGL, use_3d)==GF_OK) {
+				fprintf(stdout, "Using %s for 2D drawing\n", use_3d ? "OpenGL" : "2D rasterizer");
 			}
 		}
 			break;
@@ -1314,7 +1456,7 @@ int main (int argc, char **argv)
 			gf_log_set_level(parse_log_level(szLog));
 		}
 			break;
-		case 'M':
+		case 'T':
 		{
 			char szLog[1024];
 			fprintf(stdout, "Enter new log tools:\n");
@@ -1325,10 +1467,22 @@ int main (int argc, char **argv)
 		case 'g':
 		{
 			GF_SystemRTInfo rti;
-			gf_sys_get_rti(RTI_UPDATE_TIME_MS, &rti, 0);
+			gf_sys_get_rti(rti_update_time_ms, &rti, 0);
 			fprintf(stdout, "GPAC allocated memory "LLD"\n", rti.gpac_memory);
 		}
 			break;
+		case 'M':
+		{
+			u32 size;
+			fprintf(stdout, "Enter new video cache memory in kBytes (current %d):\n", gf_term_get_option(term, GF_OPT_VIDEO_CACHE_SIZE));
+			scanf("%d", &size);
+			gf_term_set_option(term, GF_OPT_VIDEO_CACHE_SIZE, size);
+		}
+			break;
+
+		case 'E':
+			gf_term_set_option(term, GF_OPT_RELOAD_CONFIG, 1); 
+			break;
 
 		case 'h':
 			PrintHelp();
@@ -1339,15 +1493,13 @@ int main (int argc, char **argv)
 	}
 
 	gf_term_disconnect(term);
-	if (rti_file) UpdateRTInfo("disconnected");
+	if (rti_file) UpdateRTInfo("Disconnected\n");
 
 	fprintf(stdout, "Deleting terminal... ");
 	if (playlist) fclose(playlist);
 	gf_term_del(term);
 	fprintf(stdout, "OK\n");
 
-exit:
-
 	fprintf(stdout, "Unloading modules... ");
 	gf_modules_del(user.modules);
 	fprintf(stdout, "OK\n");
@@ -1404,7 +1556,7 @@ void PrintODList(GF_Terminal *term)
 	}
 }
 
-void ViewOD(GF_Terminal *term, u32 OD_ID)
+void ViewOD(GF_Terminal *term, u32 OD_ID, u32 number)
 {
 	ODInfo odi;
 	u32 i, j, count, d_enum,id;
@@ -1415,7 +1567,8 @@ void ViewOD(GF_Terminal *term, u32 OD_ID)
 	if (!root_odm) return;
 
 	odm = NULL;
-	if (!OD_ID) {
+	if ((!OD_ID && (number == (u32)(-1))) ||
+		((OD_ID == (u32)(-1)) && !number)) {
 		odm = root_odm;
 		if ((gf_term_get_object_info(term, odm, &odi) != GF_OK)) odm=NULL;
 	} else {
@@ -1423,16 +1576,21 @@ void ViewOD(GF_Terminal *term, u32 OD_ID)
 		for (i=0; i<count; i++) {
 			odm = gf_term_get_object(term, root_odm, i);
 			if (!odm) break;
-			if ((gf_term_get_object_info(term, odm, &odi) == GF_OK) && (odi.od->objectDescriptorID == OD_ID)) break;
+			if (gf_term_get_object_info(term, odm, &odi) == GF_OK) {
+				if ((number == (u32)(-1)) && (odi.od->objectDescriptorID == OD_ID)) break;
+				else if (i == (u32)(number-1)) break;
+			}
 			odm = NULL;
 		}
 	}
 	if (!odm) {
-		fprintf(stdout, "cannot find OD with ID %d\n", OD_ID);
+		if (number == (u32)-1) fprintf(stdout, "cannot find OD with ID %d\n", OD_ID);
+		else fprintf(stdout, "cannot find OD with number %d\n", number);
 		return;
 	}
 	if (!odi.od) {
-		fprintf(stdout, "Object %d not attached yet\n", OD_ID);
+		if (number == (u32)-1) fprintf(stdout, "Object %d not attached yet\n", OD_ID);
+		else fprintf(stdout, "Object #%d not attached yet\n", number);		
 		return;
 	}
 
