@@ -143,6 +143,11 @@ BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
 	ON_COMMAND(ID_FILE_EXIT, OnFileExit)
 	ON_COMMAND(ID_VIEW_CPU, OnViewCPU)
 	ON_UPDATE_COMMAND_UI(ID_VIEW_CPU, OnUpdateViewCPU)
+
+	ON_COMMAND(ID_FILE_COPY, OnFileCopy)
+	ON_UPDATE_COMMAND_UI(ID_FILE_COPY, OnUpdateFileCopy)
+	ON_COMMAND(ID_FILE_PASTE, OnFilePaste)
+	ON_UPDATE_COMMAND_UI(ID_FILE_PASTE, OnUpdateFilePaste)
 	
 	//}}AFX_MSG_MAP
 END_MESSAGE_MAP()
@@ -191,7 +196,7 @@ void CALLBACK EXPORT RTInfoTimer(HWND , UINT , UINT nID , DWORD )
 	char szMsg[100];
 	GF_SystemRTInfo rti;
 	if (nID != RTI_TIMER) return;
-	WinGPAC *app = GetApp();
+	Osmo4 *app = GetApp();
 	CMainFrame *pFrame = (CMainFrame *) app->m_pMainWnd;
 	/*shutdown*/
 	if (!pFrame) return;
@@ -485,7 +490,7 @@ void CALLBACK EXPORT ProgressTimer(HWND , UINT , UINT nID , DWORD )
 {
 	u32 now;
 	if (nID != PROGRESS_TIMER) return;
-	WinGPAC *app = GetApp();
+	Osmo4 *app = GetApp();
 	CMainFrame *pFrame = (CMainFrame *) app->m_pMainWnd;
 	/*shutdown*/
 	if (!pFrame) return;
@@ -518,7 +523,7 @@ void CMainFrame::SetProgTimer(Bool bOn)
 LONG CMainFrame::Open(WPARAM wParam, LPARAM lParam)
 {
 	Bool do_pause;
-	WinGPAC *app = GetApp();
+	Osmo4 *app = GetApp();
 	CString txt, url;
 	m_bStartupFile = FALSE;
 	txt = "Osmo4 - ";
@@ -570,7 +575,7 @@ void CMainFrame::OnLButtonUp(UINT , CPoint ) { ForwardMessage(); }
 void CMainFrame::OnDropFiles(HDROP hDropInfo) 
 {
 	u32 i, count;
-	WinGPAC *app = GetApp();
+	Osmo4 *app = GetApp();
 	char fileName[MAX_PATH];
 
 	count = ::DragQueryFile(hDropInfo, 0xFFFFFFFF, NULL, 0);
@@ -586,10 +591,10 @@ void CMainFrame::OnDropFiles(HDROP hDropInfo)
 		}
 	}
 	
-	if (count==1) 
+/*	if (count==1) 
 		m_pPlayList->Truncate();
 	else
-		m_pPlayList->Clear();
+*/		m_pPlayList->Clear();
 
 	for (i=0; i<count; i++) {
 		::DragQueryFile (hDropInfo, i, fileName, MAX_PATH);
@@ -640,14 +645,14 @@ BOOL CMainFrame::DestroyWindow()
 
 void CMainFrame::OnViewOriginal() 
 {
-	WinGPAC *gpac = GetApp();
+	Osmo4 *gpac = GetApp();
 	gf_term_set_option(gpac->m_term, GF_OPT_ORIGINAL_VIEW, 1);	
 	OnSetSize(gpac->orig_width, gpac->orig_height);
 }
 
 void CMainFrame::SetFullscreen() 
 {
-	WinGPAC *gpac = GetApp();
+	Osmo4 *gpac = GetApp();
 	if (!m_bFullScreen) {
 //		GetWindowRect(&backup_wnd_rc);
 		if (gf_term_set_option(gpac->m_term, GF_OPT_FULLSCREEN, 1) == GF_OK) 
@@ -711,7 +716,7 @@ void CMainFrame::OnUpdateArKeep(CCmdUI* pCmdUI)
 void CMainFrame::OnUpdateNavigate(CCmdUI* pCmdUI)
 {
 	BOOL enable;
-	WinGPAC *app = GetApp();
+	Osmo4 *app = GetApp();
 	pCmdUI->Enable(FALSE);
 	if (!app->m_isopen) return;
 	
@@ -741,7 +746,7 @@ void CMainFrame::OnUpdateNavigate(CCmdUI* pCmdUI)
 
 void CMainFrame::SetNavigate(u32 mode)
 {
-	WinGPAC *app = GetApp();
+	Osmo4 *app = GetApp();
 	gf_term_set_option(app->m_term, GF_OPT_NAVIGATION, mode);
 }
 void CMainFrame::OnNavigateNone() { SetNavigate(GF_NAVIGATE_NONE); }
@@ -756,14 +761,20 @@ void CMainFrame::OnNavigateGame() { SetNavigate(GF_NAVIGATE_GAME); }
 
 void CMainFrame::OnNavigateReset()
 {
-	WinGPAC *app = GetApp();
+	Osmo4 *app = GetApp();
 	gf_term_set_option(app->m_term, GF_OPT_NAVIGATION_TYPE, 0);
 }
 
 
 LONG CMainFrame::OnNavigate(WPARAM /*wParam*/, LPARAM /*lParam*/)
 {
-	WinGPAC *gpac = GetApp();
+	Osmo4 *gpac = GetApp();
+
+	/*this is a migrate instruction, just disconnect the player*/
+	if (gpac->m_navigate_url.IsEmpty() ) {
+		gf_term_disconnect(gpac->m_term);
+		return 0;
+	}
 
 	if (gf_term_is_supported_url(gpac->m_term, gpac->m_navigate_url, 1, gpac->m_NoMimeFetch)) {
 		char *str = gf_url_concatenate(m_pPlayList->GetURL(), gpac->m_navigate_url);
@@ -884,7 +895,7 @@ void CMainFrame::OnNavInfo()
 
 void CMainFrame::BuildViewList()
 {
-	WinGPAC *app = GetApp();
+	Osmo4 *app = GetApp();
 	if (!app->m_isopen) return;
 
 	/*THIS IS HARCODED FROM THE MENU LAYOUT */
@@ -894,7 +905,7 @@ void CMainFrame::BuildViewList()
 	s32 id = ID_VP_0;
 	nb_viewpoints = 0;
 	while (1) {
-		const char *szName;
+		const char *szName = NULL;
 		Bool bound;
 		GF_Err e = gf_term_get_viewpoint(app->m_term, nb_viewpoints+1, &szName, &bound);
 		if (e) break;
@@ -915,7 +926,7 @@ void CMainFrame::BuildStreamList(Bool reset_only)
 {
 	u32 nb_subs;
 	CMenu *pSelect;
-	WinGPAC *app = GetApp();
+	Osmo4 *app = GetApp();
 
 	pSelect = GetMenu()->GetSubMenu(2)->GetSubMenu(0);
 	/*THIS IS HARCODED FROM THE MENU LAYOUT */
@@ -945,8 +956,8 @@ void CMainFrame::BuildStreamList(Bool reset_only)
 
 		if (gf_term_get_object_info(app->m_term, odm, &info) != GF_OK) break;
 		if (info.owns_service) {
-			char *szName = strrchr(info.service_url, '\\');
-			if (!szName) szName = strrchr(info.service_url, '/');
+			char *szName = (char *)strrchr(info.service_url, '\\');
+			if (!szName) szName = (char *)strrchr(info.service_url, '/');
 			if (!szName) szName = (char *) info.service_url;
 			else szName += 1;
 			strcpy(szLabel, szName);
@@ -957,18 +968,30 @@ void CMainFrame::BuildStreamList(Bool reset_only)
 		switch (info.od_type) {
 		case GF_STREAM_AUDIO:
 			pMenu = pSelect->GetSubMenu(0);
-			if (!info.owns_service) sprintf(szLabel, "Audio #%d", pMenu->GetMenuItemCount() + 1);
+			if (!info.owns_service) {
+				if (info.lang) {
+					sprintf(szLabel, "Language %s (ID %d)", gf_4cc_to_str(info.lang), info.od->objectDescriptorID);
+				} else {
+					sprintf(szLabel, "ID %d", info.od->objectDescriptorID);
+				}
+			}
 			pMenu->AppendMenu(MF_ENABLED, ID_SELOBJ_0 + i, szLabel);
 			break;
 		case GF_STREAM_VISUAL:
 			pMenu = pSelect->GetSubMenu(1);
-			if (!info.owns_service) sprintf(szLabel, "Video #%d", pMenu->GetMenuItemCount() + 1);
+			if (!info.owns_service) sprintf(szLabel, "ID %d", info.od->objectDescriptorID);
 			pMenu->AppendMenu(MF_ENABLED, ID_SELOBJ_0 + i, szLabel);
 			break;
 		case GF_STREAM_TEXT:
 			nb_subs ++;
 			pMenu = pSelect->GetSubMenu(2);
-			if (!info.owns_service) sprintf(szLabel, "Subtitle #%d", pMenu->GetMenuItemCount() + 1);
+			if (!info.owns_service) {
+				if (info.lang) {
+					sprintf(szLabel, "Language %s (ID %d)", gf_4cc_to_str(info.lang), info.od->objectDescriptorID);
+				} else {
+					sprintf(szLabel, "ID %d", info.od->objectDescriptorID);
+				}
+			}
 			pMenu->AppendMenu(MF_ENABLED, ID_SELOBJ_0 + i, szLabel);
 			break;
 		}
@@ -983,7 +1006,7 @@ void CMainFrame::BuildStreamList(Bool reset_only)
 BOOL CMainFrame::OnCommand(WPARAM wParam, LPARAM lParam)
 {
 	int ID = LOWORD(wParam);
-	WinGPAC *app = GetApp();
+	Osmo4 *app = GetApp();
 
 	if ( (ID>=ID_VP_0) && (ID<=ID_VP_0+nb_viewpoints)) {
 		ID -= ID_VP_0;
@@ -1008,7 +1031,7 @@ BOOL CMainFrame::OnCommand(WPARAM wParam, LPARAM lParam)
 		}
 		return TRUE;
 	}
-	if ( (ID>=ID_SELOBJ_0) && (ID<=ID_SELOBJ_9)) {
+	if ( (ID>=ID_SELOBJ_0) && (ID<=ID_SELOBJ_29)) {
 		ID -= ID_SELOBJ_0;
 		GF_ObjectManager *root_od = gf_term_get_root_object(app->m_term);
 		if (!root_od) return TRUE;
@@ -1026,7 +1049,7 @@ BOOL CMainFrame::OnCommand(WPARAM wParam, LPARAM lParam)
 
 void CMainFrame::OnInitMenuPopup(CMenu* pPopupMenu, UINT ID, BOOL bSys)
 {
-	WinGPAC *app = GetApp();
+	Osmo4 *app = GetApp();
 	/*viewport list*/
 	if (pPopupMenu->GetMenuItemID(0)==ID_VP_0) {
 		for (int i=0; i<nb_viewpoints; i++) {
@@ -1054,7 +1077,7 @@ void CMainFrame::OnInitMenuPopup(CMenu* pPopupMenu, UINT ID, BOOL bSys)
 			pPopupMenu->EnableMenuItem(0, MF_BYPOSITION | MF_ENABLED);
 		}
 	}
-	if ((pPopupMenu->GetMenuItemID(0)>=ID_SELOBJ_0) && (pPopupMenu->GetMenuItemID(0)<=ID_SELOBJ_9)) {
+	if ((pPopupMenu->GetMenuItemID(0)>=ID_SELOBJ_0) && (pPopupMenu->GetMenuItemID(0)<=ID_SELOBJ_29)) {
 		GF_ObjectManager *root_od = gf_term_get_root_object(app->m_term);
 		if (!root_od) return;
 
@@ -1114,7 +1137,7 @@ void CMainFrame::OnCollideDisp()
 
 void CMainFrame::OnUpdateCollideDisp(CCmdUI* pCmdUI) 
 {
-	WinGPAC *gpac = GetApp(); 
+	Osmo4 *gpac = GetApp(); 
 	pCmdUI->Enable(gpac->m_isopen);	
 	pCmdUI->SetCheck( (gf_term_get_option(gpac->m_term, GF_OPT_COLLISION) == GF_COLLISION_DISPLACEMENT) ? 1 : 0);
 }
@@ -1126,7 +1149,7 @@ void CMainFrame::OnCollideNone()
 
 void CMainFrame::OnUpdateCollideNone(CCmdUI* pCmdUI) 
 {
-	WinGPAC *gpac = GetApp(); 
+	Osmo4 *gpac = GetApp(); 
 	pCmdUI->Enable(gpac->m_isopen);	
 	pCmdUI->SetCheck( (gf_term_get_option(gpac->m_term, GF_OPT_COLLISION) == GF_COLLISION_NONE) ? 1 : 0);
 }
@@ -1138,21 +1161,21 @@ void CMainFrame::OnCollideReg()
 
 void CMainFrame::OnUpdateCollideReg(CCmdUI* pCmdUI) 
 {
-	WinGPAC *gpac = GetApp(); 
+	Osmo4 *gpac = GetApp(); 
 	pCmdUI->Enable(gpac->m_isopen);	
 	pCmdUI->SetCheck( (gf_term_get_option(gpac->m_term, GF_OPT_COLLISION) == GF_COLLISION_NORMAL) ? 1 : 0);
 }
 
 void CMainFrame::OnHeadlight() 
 {
-	WinGPAC *app = GetApp();
+	Osmo4 *app = GetApp();
 	Bool val = !gf_term_get_option(app->m_term, GF_OPT_HEADLIGHT);
 	gf_term_set_option(app->m_term, GF_OPT_HEADLIGHT, val);
 }
 
 void CMainFrame::OnUpdateHeadlight(CCmdUI* pCmdUI) 
 {
-	WinGPAC *app = GetApp();
+	Osmo4 *app = GetApp();
 	pCmdUI->Enable(FALSE);
 	if (!app->m_isopen) return;
 	u32 type = gf_term_get_option(app->m_term, GF_OPT_NAVIGATION_TYPE);
@@ -1164,14 +1187,14 @@ void CMainFrame::OnUpdateHeadlight(CCmdUI* pCmdUI)
 
 void CMainFrame::OnGravity() 
 {
-	WinGPAC *app = GetApp();
+	Osmo4 *app = GetApp();
 	Bool val = gf_term_get_option(app->m_term, GF_OPT_GRAVITY) ? 0 : 1;
 	gf_term_set_option(app->m_term, GF_OPT_GRAVITY, val);
 }
 
 void CMainFrame::OnUpdateGravity(CCmdUI* pCmdUI) 
 {
-	WinGPAC *app = GetApp();
+	Osmo4 *app = GetApp();
 	pCmdUI->Enable(FALSE);
 	if (!app->m_isopen) return;
 	u32 type = gf_term_get_option(app->m_term, GF_OPT_NAVIGATION_TYPE);
@@ -1230,7 +1253,7 @@ BOOL CMainFrame::OnNotify(WPARAM wParam, LPARAM lParam, LRESULT* pResult)
 
 void CMainFrame::OnNavNext() 
 {
-	WinGPAC *app = GetApp();
+	Osmo4 *app = GetApp();
 	/*don't play if last could trigger playlist loop*/
 	if ((m_pPlayList->m_cur_entry<0) || (gf_list_count(m_pPlayList->m_entries) == 1 + (u32) m_pPlayList->m_cur_entry)) return;
 	m_pPlayList->PlayNext();
@@ -1245,7 +1268,7 @@ void CMainFrame::OnUpdateNavNext(CCmdUI* pCmdUI)
 
 void CMainFrame::OnNavPrev() 
 {
-	WinGPAC *app = GetApp();
+	Osmo4 *app = GetApp();
 	if (m_pPlayList->m_cur_entry<=0) return;
 	m_pPlayList->PlayPrev();
 }
@@ -1315,7 +1338,7 @@ void CMainFrame::LookForSubtitles()
 
 void CMainFrame::OnCacheEnable()
 {
-	WinGPAC *app = GetApp();
+	Osmo4 *app = GetApp();
 	u32 state = gf_term_get_option(app->m_term, GF_OPT_MEDIA_CACHE);
 	if (state==GF_MEDIA_CACHE_DISABLED) {
 		gf_term_set_option(app->m_term, GF_OPT_MEDIA_CACHE, GF_MEDIA_CACHE_ENABLED);
@@ -1326,7 +1349,7 @@ void CMainFrame::OnCacheEnable()
 
 void CMainFrame::OnUpdateCacheEnable(CCmdUI* pCmdUI) 
 {
-	WinGPAC *app = GetApp();
+	Osmo4 *app = GetApp();
 	u32 state = gf_term_get_option(app->m_term, GF_OPT_MEDIA_CACHE);
 	switch (state) {
 	case GF_MEDIA_CACHE_ENABLED:
@@ -1345,19 +1368,19 @@ void CMainFrame::OnUpdateCacheEnable(CCmdUI* pCmdUI)
 
 void CMainFrame::OnUpdateCacheStop(CCmdUI* pCmdUI) 
 {
-	WinGPAC *app = GetApp();
+	Osmo4 *app = GetApp();
 	u32 state = gf_term_get_option(app->m_term, GF_OPT_MEDIA_CACHE);
 	pCmdUI->Enable( (state==GF_MEDIA_CACHE_RUNNING) ? TRUE : FALSE);
 }
 
 void CMainFrame::OnCacheStop()
 {
-	WinGPAC *app = GetApp();
+	Osmo4 *app = GetApp();
 	gf_term_set_option(app->m_term, GF_OPT_MEDIA_CACHE, GF_MEDIA_CACHE_DISABLED);
 }
 void CMainFrame::OnCacheAbort()
 {
-	WinGPAC *app = GetApp();
+	Osmo4 *app = GetApp();
 	gf_term_set_option(app->m_term, GF_OPT_MEDIA_CACHE, GF_MEDIA_CACHE_DISCARD);
 }
 
@@ -1372,7 +1395,7 @@ void CMainFrame::BuildChapterList(Bool reset_only)
 	CMenu *pChaps;
 	ODInfo odi;
 	NetInfoCommand com;
-	WinGPAC *app = GetApp();
+	Osmo4 *app = GetApp();
 
 	/*THIS IS HARCODED FROM THE MENU LAYOUT */
 	pChaps = GetMenu()->GetSubMenu(2)->GetSubMenu(1);
@@ -1429,3 +1452,68 @@ void CMainFrame::OnUpdateViewCPU(CCmdUI* pCmdUI)
 	pCmdUI->SetCheck(m_show_rti);
 }
 
+
+void CMainFrame::OnFileCopy()
+{
+	u32 len;
+	const char *text = gf_term_get_text_selection(GetApp()->m_term, 0);
+	if (!text) return;
+
+	if (!IsClipboardFormatAvailable(CF_TEXT)) return;
+	if (!OpenClipboard()) return;
+	EmptyClipboard();
+	
+	len = strlen(text);
+	if (!len) return;
+
+	HGLOBAL hglbCopy = GlobalAlloc(GMEM_MOVEABLE, (len + 1) * sizeof(char)); 
+	LPTSTR lptstrCopy = (char *) GlobalLock(hglbCopy);
+	memcpy(lptstrCopy, text, len * sizeof(char)); 
+	lptstrCopy[len] = 0;
+	GlobalUnlock(hglbCopy); 
+	SetClipboardData(CF_TEXT, hglbCopy);
+	CloseClipboard(); 
+}
+
+void CMainFrame::OnUpdateFileCopy(CCmdUI* pCmdUI)
+{
+	Osmo4 *app = GetApp();
+	if (IsClipboardFormatAvailable(CF_TEXT)
+		&& app->m_term 
+		&& (gf_term_get_text_selection(app->m_term, 1)!=NULL)
+	) {
+		pCmdUI->Enable(TRUE);
+	} else {
+		pCmdUI->Enable(FALSE);
+	}
+}
+
+
+void CMainFrame::OnFilePaste()
+{
+	if (!IsClipboardFormatAvailable(CF_TEXT)) return;
+	if (!OpenClipboard()) return;
+
+	HGLOBAL hglbCopy = GetClipboardData(CF_TEXT);
+	if (hglbCopy) {
+		LPTSTR lptstrCopy = (char *) GlobalLock(hglbCopy);
+		gf_term_paste_text(GetApp()->m_term, lptstrCopy, 0);
+		GlobalUnlock(hglbCopy); 
+	}
+	CloseClipboard(); 
+}
+
+void CMainFrame::OnUpdateFilePaste(CCmdUI* pCmdUI)
+{
+	Osmo4 *app = GetApp();
+	if (IsClipboardFormatAvailable(CF_TEXT)
+		&& app->m_term 
+		&& (gf_term_paste_text(app->m_term, NULL, 1)==GF_OK)
+	) {
+		pCmdUI->Enable(TRUE);
+	} else {
+		pCmdUI->Enable(FALSE);
+	}
+}
+
+
