@@ -15,37 +15,38 @@ static char THIS_FILE[] = __FILE__;
 #endif
 
 /////////////////////////////////////////////////////////////////////////////
-// WinGPAC
+// Osmo4
 
-BEGIN_MESSAGE_MAP(WinGPAC, CWinApp)
-	//{{AFX_MSG_MAP(WinGPAC)
+BEGIN_MESSAGE_MAP(Osmo4, CWinApp)
+	//{{AFX_MSG_MAP(Osmo4)
 	ON_COMMAND(ID_FILEOPEN, OnOpenFile)
 	ON_COMMAND(ID_FILE_STEP, OnFileStep)
 	ON_COMMAND(ID_OPEN_URL, OnOpenUrl)
 	ON_COMMAND(ID_FILE_RELOAD, OnFileReload)
+	ON_COMMAND(ID_CONFIG_RELOAD, OnConfigReload)
 	ON_COMMAND(ID_FILE_PLAY, OnFilePlay)
 	ON_UPDATE_COMMAND_UI(ID_FILE_PLAY, OnUpdateFilePlay)
 	ON_UPDATE_COMMAND_UI(ID_FILE_STEP, OnUpdateFileStep)
 	ON_COMMAND(ID_FILE_STOP, OnFileStop)
 	ON_UPDATE_COMMAND_UI(ID_FILE_STOP, OnUpdateFileStop)
 	ON_COMMAND(ID_SWITCH_RENDER, OnSwitchRender)
-	ON_COMMAND(ID_RELOAD_TERMINAL, OnReloadTerminal)
 	ON_UPDATE_COMMAND_UI(ID_FILE_RELOAD, OnUpdateFileStop)
 	ON_COMMAND(ID_H_ABOUT, OnAbout)
+	ON_COMMAND(ID_FILE_MIGRATE, OnFileMigrate)
 	//}}AFX_MSG_MAP
 END_MESSAGE_MAP()
 
 /////////////////////////////////////////////////////////////////////////////
-// WinGPAC construction
+// Osmo4 construction
 
-WinGPAC::WinGPAC()
+Osmo4::Osmo4()
 {
 }
 
 /////////////////////////////////////////////////////////////////////////////
-// The one and only WinGPAC object
+// The one and only Osmo4 object
 
-WinGPAC theApp;
+Osmo4 theApp;
 
 
 
@@ -141,7 +142,7 @@ u32 get_sys_col(int idx)
 static void Osmo4_progress_cbk(void *usr, char *title, u32 done, u32 total)
 {
 	if (!total) return;
-	CMainFrame *pFrame = (CMainFrame *) ((WinGPAC *) usr)->m_pMainWnd;
+	CMainFrame *pFrame = (CMainFrame *) ((Osmo4 *) usr)->m_pMainWnd;
 	s32 prog = (s32) ( (100 * (u64)done) / total);
 	if (pFrame->m_last_prog < prog) {
 		pFrame->console_err = GF_OK;
@@ -154,14 +155,14 @@ static void Osmo4_progress_cbk(void *usr, char *title, u32 done, u32 total)
 
 #define W32_MIN_WIDTH 120
 
-static log_msg(char *msg)
+static void log_msg(char *msg)
 {
 	::MessageBox(NULL, msg, "GPAC", MB_OK);
 }
 Bool Osmo4_EventProc(void *priv, GF_Event *evt)
 {
 	u32 dur;
-	WinGPAC *gpac = (WinGPAC *) priv;
+	Osmo4 *gpac = (Osmo4 *) priv;
 	CMainFrame *pFrame = (CMainFrame *) gpac->m_pMainWnd;
 	/*shutdown*/
 	if (!pFrame) return 0;
@@ -256,6 +257,16 @@ Bool Osmo4_EventProc(void *priv, GF_Event *evt)
 	case GF_EVENT_QUIT:
 		pFrame->PostMessage(WM_CLOSE, 0L, 0L);
 		break;
+	case GF_EVENT_MIGRATE:
+	{
+		const char *str = gf_cfg_get_key(gpac->m_user.config, "Network", "SessionMigration");
+		if (!str || !strcmp(str, "no"))
+			gf_cfg_set_key(gpac->m_user.config, "Network", "SessionMigration", "yes");
+
+		gpac->m_navigate_url = "";
+		pFrame->PostMessage(WM_NAVIGATE, NULL, NULL);
+	}
+		break;
 	case GF_EVENT_KEYDOWN:
 		if (gpac->can_seek && evt->key.flags & GF_KEY_MOD_ALT) {
 			s32 res;
@@ -313,7 +324,10 @@ Bool Osmo4_EventProc(void *priv, GF_Event *evt)
 	case GF_EVENT_STREAMLIST:
 		pFrame->BuildStreamList(0);
 		return 0;
-	case GF_EVENT_MOUSEDOUBLECLICK:
+	case GF_EVENT_SET_CAPTION:
+		pFrame->SetWindowText(evt->caption.caption);
+		break;
+	case GF_EVENT_DBLCLICK:
 		pFrame->PostMessage(WM_COMMAND, ID_VIEW_FULLSCREEN);
 		return 0;
 	case GF_EVENT_AUTHORIZATION:
@@ -377,7 +391,7 @@ static void osmo4_do_log(void *cbk, u32 level, u32 tool, const char *fmt, va_lis
 	fflush(logs);
 }
 
-BOOL WinGPAC::InitInstance()
+BOOL Osmo4::InitInstance()
 {
 	CCommandLineInfo cmdInfo;
 
@@ -461,12 +475,16 @@ BOOL WinGPAC::InitInstance()
 		}
 	}
 
+#if 0
 	// Standard initialization
 #ifdef _AFXDLL
 	Enable3dControls();			// Call this when using MFC in a shared DLL
 #else
 	Enable3dControlsStatic();	// Call this when linking to MFC statically
 #endif
+
+#endif
+
 	SetRegistryKey(_T("GPAC"));
 	CMainFrame* pFrame = new CMainFrame;
 	m_pMainWnd = pFrame;
@@ -485,8 +503,8 @@ BOOL WinGPAC::InitInstance()
 			unsigned char str_path[MAX_PATH];
 			gf_cfg_set_key(m_user.config, "General", "ModulesDirectory", (const char *) szAppPath);
 
-			sOpt = gf_cfg_get_key(m_user.config, "Rendering", "Raster2D");
-			if (!sOpt) gf_cfg_set_key(m_user.config, "Rendering", "Raster2D", "GPAC 2D Raster");
+			sOpt = gf_cfg_get_key(m_user.config, "Compositor", "Raster2D");
+			if (!sOpt) gf_cfg_set_key(m_user.config, "Compositor", "Raster2D", "GPAC 2D Raster");
 
 			sOpt = gf_cfg_get_key(m_user.config, "General", "CacheDirectory");
 			if (!sOpt) {
@@ -495,7 +513,6 @@ BOOL WinGPAC::InitInstance()
 			}
 			/*setup UDP traffic autodetect*/
 			gf_cfg_set_key(m_user.config, "Network", "AutoReconfigUDP", "yes");
-			gf_cfg_set_key(m_user.config, "Network", "UDPNotAvailable", "no");
 			gf_cfg_set_key(m_user.config, "Network", "UDPTimeout", "10000");
 			gf_cfg_set_key(m_user.config, "Network", "BufferLength", "3000");
 
@@ -526,7 +543,7 @@ BOOL WinGPAC::InitInstance()
 		if (!sOpt) gf_cfg_set_key(m_user.config, "Video", "UseHardwareMemory", "yes");
 
 		/*by default use GDIplus, much faster than freetype on font loading*/
-		gf_cfg_set_key(m_user.config, "FontEngine", "DriverName", "gdip_rend");
+		gf_cfg_set_key(m_user.config, "FontEngine", "FontReader", "gdip_rend");
 
 	}	
 	if (! gf_modules_get_count(m_user.modules) ) {
@@ -550,6 +567,11 @@ BOOL WinGPAC::InitInstance()
 		gf_cfg_set_key(m_user.config, "Video", "DriverName", "dx_hw");
 	}
 
+	/*reset session migration*/
+	str = gf_cfg_get_key(m_user.config, "Network", "SessionMigration");
+	if (str && !strcmp(str, "yes"))
+		gf_cfg_set_key(m_user.config, "Network", "SessionMigration", "no");
+
 	/*check log file*/
 	str = gf_cfg_get_key(m_user.config, "General", "LogFile");
 	if (str) {
@@ -590,9 +612,8 @@ BOOL WinGPAC::InitInstance()
 			else if (!stricmp(val, "media")) m_log_tools |= GF_LOG_MEDIA;
 			else if (!stricmp(val, "scene")) m_log_tools |= GF_LOG_SCENE;
 			else if (!stricmp(val, "script")) m_log_tools |= GF_LOG_SCRIPT;
+			else if (!stricmp(val, "interact")) m_log_tools |= GF_LOG_INTERACT;
 			else if (!stricmp(val, "compose")) m_log_tools |= GF_LOG_COMPOSE;
-			else if (!stricmp(val, "render")) m_log_tools |= GF_LOG_RENDER;
-			else if (!stricmp(val, "service")) m_log_tools |= GF_LOG_SERVICE;
 			else if (!stricmp(val, "mmio")) m_log_tools |= GF_LOG_MMIO;
 			else if (!stricmp(val, "none")) m_log_tools = 0;
 			else if (!stricmp(val, "all")) m_log_tools = 0xFFFFFFFF;
@@ -669,7 +690,7 @@ BOOL WinGPAC::InitInstance()
 	return TRUE;
 }
 
-int WinGPAC::ExitInstance() 
+int Osmo4::ExitInstance() 
 {
 	if (m_term) gf_term_del(m_term);
 	if (m_user.modules) gf_modules_del(m_user.modules);
@@ -685,42 +706,9 @@ int WinGPAC::ExitInstance()
 }
 
 
-void WinGPAC::ReloadTerminal()
-{
-	CMainFrame *pFrame = (CMainFrame *) m_pMainWnd;
-	Bool reconnect = (m_isopen && pFrame->m_bStartupFile) ? 1 : 0;
-	pFrame->console_err = GF_OK;
-	pFrame->console_message = "Reloading GPAC Terminal";
-	m_pMainWnd->PostMessage(WM_CONSOLEMSG, 0, 0);
-
-	m_reconnect_time = 0;
-	if (can_seek) m_reconnect_time = gf_term_get_time_in_ms(m_term);
-
-	gf_term_del(m_term);
-	m_term = gf_term_new(&m_user);
-	if (!m_term) {
-		MessageBox(NULL, "Fatal Error !!", "Couldn't change renderer", MB_OK);
-		m_pMainWnd->PostMessage(WM_DESTROY);
-		return;
-	}
-	pFrame->console_message = "GPAC Terminal reloaded";
-	m_pMainWnd->PostMessage(WM_CONSOLEMSG, 0, 0);
-	UpdateRenderSwitch();
-
-	RECT rc;
-	((CMainFrame *) m_pMainWnd)->m_pWndView->GetClientRect(&rc);
-	gf_term_set_size(m_term, rc.right - rc.left, rc.bottom - rc.top);
-
-	if (reconnect) m_pMainWnd->PostMessage(WM_OPENURL);
-	else {
-		const char *sOpt = gf_cfg_get_key(GetApp()->m_user.config, "General", "StartupFile");
-		if (sOpt) gf_term_connect(m_term, sOpt);
-	}
-}
-
 
 /////////////////////////////////////////////////////////////////////////////
-// WinGPAC message handlers
+// Osmo4 message handlers
 
 
 
@@ -773,7 +761,7 @@ BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
 	//}}AFX_MSG_MAP
 END_MESSAGE_MAP()
 
-void WinGPAC::OnAbout() 
+void Osmo4::OnAbout() 
 {
 	CAboutDlg aboutDlg;
 	aboutDlg.DoModal();
@@ -782,7 +770,7 @@ void WinGPAC::OnAbout()
 BOOL CAboutDlg::OnInitDialog() 
 {
 	CDialog::OnInitDialog();
-	CString str = "GPAC/Osmo4 - version " GPAC_VERSION;
+	CString str = "GPAC/Osmo4 - version " GPAC_FULL_VERSION;
 	SetWindowText(str);
 	return TRUE;  
 }
@@ -793,10 +781,10 @@ void CAboutDlg::OnGogpac()
 }
 
 /////////////////////////////////////////////////////////////////////////////
-// WinGPAC message handlers
+// Osmo4 message handlers
 
 
-void WinGPAC::SetOptions()
+void Osmo4::SetOptions()
 {
 	const char *sOpt = gf_cfg_get_key(m_user.config, "General", "Loop");
 	m_Loop = (sOpt && !stricmp(sOpt, "yes")) ? 1 : 0;
@@ -807,11 +795,11 @@ void WinGPAC::SetOptions()
 	sOpt = gf_cfg_get_key(m_user.config, "General", "ViewXMT");
 	m_ViewXMTA  = (sOpt && !stricmp(sOpt, "yes")) ? 1 : 0;
 	sOpt = gf_cfg_get_key(m_user.config, "General", "NoMIMETypeFetch");
-	m_NoMimeFetch = (sOpt && !stricmp(sOpt, "yes")) ? 1 : 0;
+	m_NoMimeFetch = (!sOpt || !stricmp(sOpt, "yes")) ? 1 : 0;
 }
 
 
-void WinGPAC::OnOpenUrl() 
+void Osmo4::OnOpenUrl() 
 {
 	COpenUrl url;
 	if (url.DoModal() != IDOK) return;
@@ -824,7 +812,7 @@ void WinGPAC::OnOpenUrl()
 }
 
 
-CString WinGPAC::GetFileFilter()
+CString Osmo4::GetFileFilter()
 {
 	u32 keyCount, i;
 	CString sFiles;
@@ -909,7 +897,7 @@ CString WinGPAC::GetFileFilter()
 	return supportedFiles;
 }
 
-void WinGPAC::OnOpenFile() 
+void Osmo4::OnOpenFile() 
 {
 	CString sFiles = GetFileFilter();
 	u32 nb_items;
@@ -950,28 +938,28 @@ void WinGPAC::OnOpenFile()
 }
 
 
-void WinGPAC::Pause()
+void Osmo4::Pause()
 {
 	if (!m_isopen) return;
 	gf_term_set_option(m_term, GF_OPT_PLAY_STATE, (gf_term_get_option(m_term, GF_OPT_PLAY_STATE)==GF_STATE_PLAYING) ? GF_STATE_PAUSED : GF_STATE_PLAYING);
 }
 
-void WinGPAC::OnMainPause() 
+void Osmo4::OnMainPause() 
 {
 	Pause();	
 }
 
-void WinGPAC::OnFileStep() 
+void Osmo4::OnFileStep() 
 {
 	gf_term_set_option(m_term, GF_OPT_PLAY_STATE, GF_STATE_STEP_PAUSE);
 	((CMainFrame *) m_pMainWnd)->m_wndToolBar.SetButtonInfo(5, ID_FILE_PLAY, TBBS_BUTTON, 3);
 }
-void WinGPAC::OnUpdateFileStep(CCmdUI* pCmdUI) 
+void Osmo4::OnUpdateFileStep(CCmdUI* pCmdUI) 
 {
 	pCmdUI->Enable(m_isopen && !m_reset);	
 }
 
-void WinGPAC::PlayFromTime(u32 time)
+void Osmo4::PlayFromTime(u32 time)
 {
 	Bool do_pause;
 	if (start_mode==1) do_pause = 1;
@@ -982,13 +970,28 @@ void WinGPAC::PlayFromTime(u32 time)
 }
 
 
-void WinGPAC::OnFileReload() 
+void Osmo4::OnFileReload() 
 {
 	gf_term_disconnect(m_term);
 	m_pMainWnd->PostMessage(WM_OPENURL);
 }
 
-void WinGPAC::UpdatePlayButton(Bool force_play)
+void Osmo4::OnFileMigrate() 
+{
+	const char *str = gf_cfg_get_key(m_user.config, "Network", "SessionMigration");
+	if (!str || !strcmp(str, "no"))
+		gf_cfg_set_key(m_user.config, "Network", "SessionMigration", "yes");
+
+	m_navigate_url = "";
+	m_pMainWnd->PostMessage(WM_NAVIGATE, NULL, NULL);
+}
+
+void Osmo4::OnConfigReload() 
+{
+	gf_term_set_option(m_term, GF_OPT_RELOAD_CONFIG, 1); 
+}
+
+void Osmo4::UpdatePlayButton(Bool force_play)
 {
 	if (!force_play && gf_term_get_option(m_term, GF_OPT_PLAY_STATE)==GF_STATE_PLAYING) {
 		((CMainFrame *) m_pMainWnd)->m_wndToolBar.SetButtonInfo(5, ID_FILE_PLAY, TBBS_BUTTON, 4);
@@ -997,7 +1000,7 @@ void WinGPAC::UpdatePlayButton(Bool force_play)
 	}
 }
 
-void WinGPAC::OnFilePlay() 
+void Osmo4::OnFilePlay() 
 {
 	if (m_isopen) {
 		if (m_reset) {
@@ -1013,7 +1016,7 @@ void WinGPAC::OnFilePlay()
 	}
 }
 
-void WinGPAC::OnUpdateFilePlay(CCmdUI* pCmdUI) 
+void Osmo4::OnUpdateFilePlay(CCmdUI* pCmdUI) 
 {
 	if (m_isopen) {
 		pCmdUI->Enable(TRUE);	
@@ -1032,7 +1035,7 @@ void WinGPAC::OnUpdateFilePlay(CCmdUI* pCmdUI)
 	}
 }
 
-void WinGPAC::OnFileStop() 
+void Osmo4::OnFileStop() 
 {
 	CMainFrame *pFrame = (CMainFrame *) m_pMainWnd;
 	if (m_reset) return;
@@ -1044,33 +1047,27 @@ void WinGPAC::OnFileStop()
 	start_mode = 2;
 }
 
-void WinGPAC::OnUpdateFileStop(CCmdUI* pCmdUI) 
+void Osmo4::OnUpdateFileStop(CCmdUI* pCmdUI) 
 {
 	pCmdUI->Enable(m_isopen);	
 }
 
-void WinGPAC::OnSwitchRender() 
+void Osmo4::OnSwitchRender() 
 {
-	const char *opt = gf_cfg_get_key(m_user.config, "Rendering", "RendererName");
-	if (!stricmp(opt, "GPAC 2D Renderer"))
-		gf_cfg_set_key(m_user.config, "Rendering", "RendererName", "GPAC 3D Renderer");
-	else
-		gf_cfg_set_key(m_user.config, "Rendering", "RendererName", "GPAC 2D Renderer");
+	const char *opt = gf_cfg_get_key(m_user.config, "Compositor", "ForceOpenGL");
+	Bool use_gl = (opt && !stricmp(opt, "yes")) ? 1 : 0;
+	gf_cfg_set_key(m_user.config, "Compositor", "ForceOpenGL", use_gl ? "no" : "yes");
 
-	ReloadTerminal();
+	gf_term_set_option(m_term, GF_OPT_USE_OPENGL, !use_gl);
+
+	UpdateRenderSwitch();
 }
 
-void WinGPAC::UpdateRenderSwitch()
+void Osmo4::UpdateRenderSwitch()
 {
-	const char *opt = gf_cfg_get_key(m_user.config, "Rendering", "RendererName");
-	if (!stricmp(opt, "GPAC 3D Renderer"))
+	const char *opt = gf_cfg_get_key(m_user.config, "Compositor", "ForceOpenGL");
+	if (opt && !stricmp(opt, "no"))
 		((CMainFrame *) m_pMainWnd)->m_wndToolBar.SetButtonInfo(12, ID_SWITCH_RENDER, TBBS_BUTTON, 10);
 	else
 		((CMainFrame *) m_pMainWnd)->m_wndToolBar.SetButtonInfo(12, ID_SWITCH_RENDER, TBBS_BUTTON, 9);
 }
-
-void WinGPAC::OnReloadTerminal() 
-{
-	ReloadTerminal();
-}
-
