@@ -37,8 +37,10 @@
 IMPLEMENT_APP(wxOsmo4App)
 
 #include "osmo4.xpm"
+
 #include <wx/dnd.h>
 #include <wx/filename.h>
+#include <wx/clipbrd.h>
 
 #include "toolbar.xpm"
 
@@ -228,7 +230,6 @@ Bool GPAC_EventProc(void *ptr, GF_Event *evt)
 		gf_set_progress(sTitle, evt->progress.done, evt->progress.total);
 	}
 		break;
-	
 	case GF_EVENT_KEYDOWN:
 		if (app->m_can_seek && (evt->key.flags & GF_KEY_MOD_ALT)) {
 			s32 res;
@@ -300,6 +301,14 @@ Bool GPAC_EventProc(void *ptr, GF_Event *evt)
 		app->AddPendingEvent(wxevt);
 	}
 		return 1;
+	case GF_EVENT_SET_CAPTION:
+	{
+		wxGPACEvent wxevt(app);
+		wxevt.to_url = wxString(evt->caption.caption, wxConvUTF8);
+		wxevt.gpac_evt.type = evt->type;
+		app->AddPendingEvent(wxevt);
+	}
+		return 1;
 
 	case GF_EVENT_QUIT:
 	case GF_EVENT_VIEWPOINTS:
@@ -312,7 +321,7 @@ Bool GPAC_EventProc(void *ptr, GF_Event *evt)
 		app->AddPendingEvent(wxevt);
 	}
 		break;
-	case GF_EVENT_MOUSEDOUBLECLICK:
+	case GF_EVENT_DBLCLICK:
 		gf_term_set_option(app->m_term, GF_OPT_FULLSCREEN, !gf_term_get_option(app->m_term, GF_OPT_FULLSCREEN));
 		return 0;
 	case GF_EVENT_MOUSEDOWN:
@@ -390,7 +399,7 @@ bool myDropfiles::OnDropFiles(wxCoord x, wxCoord y, const wxArrayString& filenam
 	u32 count = filenames.GetCount();
 
 	if (count==1) {
-		char *ext = strrchr(filenames.Item(0).mb_str(wxConvUTF8) , '.');
+		const char *ext = strrchr(filenames.Item(0).mb_str(wxConvUTF8) , '.');
 		/*if playing and sub d&d, open sub in current presentation*/
 		if (m_pMain->m_connected && ext && ( !stricmp(ext, ".srt") || !stricmp(ext, ".sub") || !stricmp(ext, ".ttxt") || !stricmp(ext, ".xml") ) ) {
 			m_pMain->AddSubtitle(filenames.Item(0).mb_str(wxConvUTF8) , 1);
@@ -437,7 +446,7 @@ void wxOsmo4Frame::CheckVideoOut()
 	void *os_handle = NULL;
 	void *os_display = NULL;
 	/*build a child window for embed display*/
-	if (stricmp(sOpt, "SDL Video Output")) {
+	if (sOpt && stricmp(sOpt, "SDL Video Output")) {
 		if (m_user.os_window_handler) return;
 		m_bExternalView = 0;
 
@@ -614,9 +623,8 @@ Bool wxOsmo4Frame::LoadTerminal()
 			else if (!stricmp(val, "media")) m_log_tools |= GF_LOG_MEDIA;
 			else if (!stricmp(val, "scene")) m_log_tools |= GF_LOG_SCENE;
 			else if (!stricmp(val, "script")) m_log_tools |= GF_LOG_SCRIPT;
+			else if (!stricmp(val, "interact")) m_log_tools |= GF_LOG_INTERACT;
 			else if (!stricmp(val, "compose")) m_log_tools |= GF_LOG_COMPOSE;
-			else if (!stricmp(val, "render")) m_log_tools |= GF_LOG_RENDER;
-			else if (!stricmp(val, "service")) m_log_tools |= GF_LOG_SERVICE;
 			else if (!stricmp(val, "mmio")) m_log_tools |= GF_LOG_MMIO;
 			else if (!stricmp(val, "none")) m_log_tools = 0;
 			else if (!stricmp(val, "all")) m_log_tools = 0xFFFFFFFF;
@@ -648,7 +656,7 @@ Bool wxOsmo4Frame::LoadTerminal()
 		const char *sOpt;
 		wxDirDialog dlg(NULL, wxT("Locate GPAC modules directory"));
 		if  (!gf_modules_get_count(m_user.modules)) {
-		  gf_modules_del(m_user.modules);
+		  if (m_user.modules) gf_modules_del(m_user.modules);
 		  m_user.modules = NULL;
 			if ( dlg.ShowModal() != wxID_OK ) return false;
 			str = dlg.GetPath().mb_str(wxConvUTF8);
@@ -657,6 +665,7 @@ Bool wxOsmo4Frame::LoadTerminal()
 			if (!m_user.modules || !gf_modules_get_count(m_user.modules) ) {
 				wxMessageDialog(NULL, wxT("Cannot find any modules for GPAC"), wxT("Init error"), wxOK);
 				gf_cfg_del(m_user.config);
+				m_user.config = NULL;
 				return 0;
 			}
 		}
@@ -675,7 +684,6 @@ Bool wxOsmo4Frame::LoadTerminal()
 
 		/*setup UDP traffic autodetect*/
 		gf_cfg_set_key(m_user.config, "Network", "AutoReconfigUDP", "yes");
-		gf_cfg_set_key(m_user.config, "Network", "UDPNotAvailable", "no");
 		gf_cfg_set_key(m_user.config, "Network", "UDPTimeout", "10000");
 		gf_cfg_set_key(m_user.config, "Network", "BufferLength", "3000");
 
@@ -686,19 +694,18 @@ Bool wxOsmo4Frame::LoadTerminal()
 			gf_cfg_set_key(m_user.config, "Audio", "NumBuffers", "2");
 			gf_cfg_set_key(m_user.config, "Audio", "TotalDuration", "120");
 		}
-		gf_cfg_set_key(m_user.config, "Rendering", "RendererName", "GPAC 2D Renderer");
 
 #ifdef WIN32
 		unsigned char str_path[MAX_PATH];
-		sOpt = gf_cfg_get_key(m_user.config, "Rendering", "Raster2D");
-		if (!sOpt) gf_cfg_set_key(m_user.config, "Rendering", "Raster2D", "gdip_rend");
+		sOpt = gf_cfg_get_key(m_user.config, "Compositor", "Raster2D");
+		if (!sOpt) gf_cfg_set_key(m_user.config, "Compositor", "Raster2D", "gdip_rend");
 		sOpt = gf_cfg_get_key(m_user.config, "General", "CacheDirectory");
 		if (!sOpt) {
 			sprintf((char *) str_path, "%scache", abs_gpac_path.mb_str(wxConvUTF8));
 			gf_cfg_set_key(m_user.config, "General", "CacheDirectory", (const char *) str_path);
 		}
 		/*by default use GDIplus, much faster than freetype on font loading*/
-		gf_cfg_set_key(m_user.config, "FontEngine", "DriverName", "gdip_rend");
+		gf_cfg_set_key(m_user.config, "FontEngine", "FontReader", "gdip_rend");
 		gf_cfg_set_key(m_user.config, "Video", "DriverName", "DirectX Video Output");
 
 		sOpt = gf_cfg_get_key(m_user.config, "General", "StartupFile");
@@ -720,7 +727,7 @@ Bool wxOsmo4Frame::LoadTerminal()
 			gf_cfg_set_key(m_user.config, "FontEngine", "FontDirectory", (const char *) dlg2.GetPath().mb_str(wxConvUTF8) );
 
 		gf_cfg_set_key(m_user.config, "Video", "DriverName", "SDL Video Output");
-		gf_cfg_set_key(m_user.config, "Render2D", "ScalableZoom", "no");
+		gf_cfg_set_key(m_user.config, "Compositor", "ScalableZoom", "no");
 #else
 		gf_cfg_set_key(m_user.config, "FontEngine", "FontDirectory", "/usr/share/fonts/truetype/");
 		/*these fonts seems installed by default on many systems...*/
@@ -731,7 +738,7 @@ Bool wxOsmo4Frame::LoadTerminal()
 
 		gf_cfg_set_key(m_user.config, "General", "CacheDirectory", "/tmp");
 		gf_cfg_set_key(m_user.config, "Video", "DriverName", "X11 Video Output");
-		gf_cfg_set_key(m_user.config, "Render2D", "ScalableZoom", "yes");
+		gf_cfg_set_key(m_user.config, "Compositor", "ScalableZoom", "yes");
 		gf_cfg_set_key(m_user.config, "Audio", "DriverName", "SDL Audio Output");
 #endif
 
@@ -757,6 +764,7 @@ Bool wxOsmo4Frame::LoadTerminal()
 		wxMessageDialog(NULL, wxT("No modules available - system cannot work"), wxT("Fatal Error"), wxOK).ShowModal();
 		gf_modules_del(m_user.modules);
 		gf_cfg_del(m_user.config);
+		m_user.config = NULL;
 		return 0;
 	}
 
@@ -777,7 +785,7 @@ Bool wxOsmo4Frame::LoadTerminal()
 		wxMessageDialog(NULL, wxT("Fatal Error"), wxT("Cannot load GPAC Terminal"), wxOK).ShowModal();
 		return 0;
 	} else {
-		::wxLogMessage(wxT("GPAC Terminal started - using ") + wxString(gf_cfg_get_key(m_user.config, "Rendering", "RendererName"), wxConvUTF8) );
+		::wxLogMessage(wxT("GPAC Terminal started") );
 	}
 	return 1;
 }
@@ -818,14 +826,15 @@ wxDEFAULT_FRAME_STYLE
 	menu->Append(FILE_OPEN_URL, wxT("&Open URL\tCtrl+U"), wxT("Open remote presentation"));
 	menu->AppendSeparator();
 	menu->Append(FILE_PROPERTIES, wxT("&Properties\tCtrl+I"), wxT("Show presentation properties"));
+	menu->Enable(FILE_PROPERTIES, 0);
 	wxMenu *smenu = new wxMenu();
 	smenu->Append(ID_MCACHE_ENABLE, wxT("&Enable"), wxT("Turns Recorder On/Off"));
 	smenu->Append(ID_MCACHE_STOP, wxT("&Stop"), wxT("Stops recording and saves"));
 	smenu->Append(ID_MCACHE_ABORT, wxT("&Abort"), wxT("Stops recording and discards"));
 	menu->Append(0, wxT("&Streaming Cache"), smenu);
-	menu->Enable(FILE_PROPERTIES, 0);
 	menu->AppendSeparator();
-	menu->Append(TERM_RELOAD, wxT("Reload &Player"), wxT("Reload GPAC Terminal"));
+	menu->Append(FILE_COPY, wxT("&Copy\tCtrl+C"), wxT("Copy selected text"));
+	menu->Append(FILE_PASTE, wxT("&Paste\tCtrl+V"), wxT("Copy selected text"));
 	menu->AppendSeparator();
 	menu->Append(FILE_QUIT, wxT("E&xit"), wxT("Quit the application"));
 	b->Append(menu, wxT("&File"));
@@ -892,6 +901,7 @@ wxDEFAULT_FRAME_STYLE
 	menu->Append(FILE_STEP, wxT("&Step-by-Step\tCtrl+S"), wxT("Play/Pause/Resume Presentation"));
 	menu->Append(FILE_STOP, wxT("&Stop"), wxT("Stop Presentation"));
 	menu->AppendSeparator();
+	menu->Append(FILE_RELOAD_CONFIG, wxT("&Reload Config\tCtrl+R"), wxT("Reload Configuration File"));
 	menu->Append(FILE_RELOAD, wxT("&Reload File\tCtrl+R"), wxT("Reload Presentation"));
 	b->Append(menu, wxT("&Play"));
 
@@ -1086,9 +1096,9 @@ BEGIN_EVENT_TABLE(wxOsmo4Frame, wxFrame)
 	EVT_CLOSE(wxOsmo4Frame::OnCloseApp)
 	EVT_MENU(FILE_OPEN, wxOsmo4Frame::OnFileOpen)
 	EVT_MENU(FILE_OPEN_URL, wxOsmo4Frame::OnFileOpenURL)
+	EVT_MENU(FILE_RELOAD_CONFIG, wxOsmo4Frame::OnFileReloadConfig)
 	EVT_MENU(FILE_RELOAD, wxOsmo4Frame::OnFileReload)
 	EVT_MENU(FILE_PROPERTIES, wxOsmo4Frame::OnFileProperties)
-	EVT_MENU(TERM_RELOAD, wxOsmo4Frame::OnTermReload)
 	EVT_MENU(FILE_QUIT, wxOsmo4Frame::OnFileQuit)
 	EVT_MENU(VIEW_FULLSCREEN, wxOsmo4Frame::OnFullScreen)
 	EVT_MENU(VIEW_OPTIONS, wxOsmo4Frame::OnOptions)
@@ -1099,6 +1109,10 @@ BEGIN_EVENT_TABLE(wxOsmo4Frame, wxFrame)
 	EVT_MENU(VIEW_ORIGINAL, wxOsmo4Frame::OnViewOriginal)
 	EVT_MENU(VIEW_PLAYLIST, wxOsmo4Frame::OnPlaylist)
 	EVT_UPDATE_UI(VIEW_PLAYLIST, wxOsmo4Frame::OnUpdatePlayList)
+	EVT_MENU(FILE_COPY, wxOsmo4Frame::OnFileCopy)
+	EVT_UPDATE_UI(FILE_COPY, wxOsmo4Frame::OnUpdateFileCopy)
+	EVT_MENU(FILE_PASTE, wxOsmo4Frame::OnFilePaste)
+	EVT_UPDATE_UI(FILE_PASTE, wxOsmo4Frame::OnUpdateFilePaste)
 
 	EVT_MENU(ID_CLEAR_NAV, wxOsmo4Frame::OnClearNav)
 	EVT_UPDATE_UI(ID_STREAM_MENU, wxOsmo4Frame::OnUpdateStreamMenu)
@@ -1343,6 +1357,11 @@ void wxOsmo4Frame::OnFileReload(wxCommandEvent & WXUNUSED(event))
 	DoConnect();
 }
 
+void wxOsmo4Frame::OnFileReloadConfig(wxCommandEvent & WXUNUSED(event))
+{
+	gf_term_set_option(m_term, GF_OPT_RELOAD_CONFIG, 1);
+}
+
 void wxOsmo4Frame::OnFileQuit(wxCommandEvent & WXUNUSED(event))
 {
 	Close(FALSE);
@@ -1536,7 +1555,7 @@ BEGIN_EVENT_TABLE(AboutDlg, wxDialog)
 END_EVENT_TABLE()
 
 AboutDlg::AboutDlg(wxWindow *parent)
-             : wxDialog(parent, -1, wxString(wxT("GPAC/Osmo4 V ")wxT(GPAC_VERSION)))
+             : wxDialog(parent, -1, wxString(wxT("GPAC/Osmo4 V ")wxT(GPAC_FULL_VERSION)))
 {
 	SetSize(220, 320);
 	Centre();
@@ -1610,6 +1629,9 @@ void wxOsmo4Frame::OnGPACEvent(wxGPACEvent &event)
 	case GF_EVENT_QUIT:
 		Close(TRUE);
 		break;
+	case GF_EVENT_SET_CAPTION:
+		SetTitle(event.to_url);
+		break;
 	case GF_EVENT_CONNECT:
 		BuildStreamList(0);
 		ConnectAcknowledged(event.gpac_evt.connect.is_connected);
@@ -1852,40 +1874,6 @@ void wxOsmo4Frame::OnSlide(wxScrollEvent &event)
 }
 
 
-void wxOsmo4Frame::ReloadTerminal()
-{
-	Bool reconnect = (m_connected && !m_bStartupFile) ? 1 : 0;
-	::wxLogMessage(wxT("Reloading GPAC Terminal"));
-
-	u32 reconnect_time = 0;
-	if (m_duration) reconnect_time = gf_term_get_time_in_ms(m_term);
-
-	gf_term_del(m_term);
-	/*SET IT TO NULL to make sure we don't process callbacks during setup!!!*/
-	m_term = NULL;
-
-	CheckVideoOut();
-	m_term = gf_term_new(&m_user);
-	if (!m_term) {
-		wxMessageDialog(this, wxT("Fatal Error !!"), wxT("Couldn't change renderer"), wxOK);
-		Destroy();
-		return;
-	}
-	if (reconnect) {
-		if (reconnect_time) {
-			gf_term_connect_from_time(m_term, m_pPlayList->GetURL().mb_str(wxConvUTF8), reconnect_time, 0);
-		} else {
-			gf_term_connect(m_term, m_pPlayList->GetURL().mb_str(wxConvUTF8));
-		}
-	} else if (m_bStartupFile) {
-		gf_term_connect(m_term, gf_cfg_get_key(m_user.config, "General", "StartupFile") );
-	}
-	::wxLogMessage(wxT("GPAC Terminal reloaded"));
-
-	UpdateRenderSwitch();
-}
-
-
 void wxOsmo4Frame::BuildViewList()
 {
 	if (!vp_list || !m_connected) return;
@@ -1969,23 +1957,23 @@ void wxOsmo4Frame::OnUpdateNavigation(wxUpdateUIEvent & event)
 
 void wxOsmo4Frame::OnRenderSwitch(wxCommandEvent &WXUNUSED(event))
 {
-	const char *opt = gf_cfg_get_key(m_user.config, "Rendering", "RendererName");
-	if (!stricmp(opt, "GPAC 2D Renderer"))
-		gf_cfg_set_key(m_user.config, "Rendering", "RendererName", "GPAC 3D Renderer");
-	else
-		gf_cfg_set_key(m_user.config, "Rendering", "RendererName", "GPAC 2D Renderer");
+	const char *opt = gf_cfg_get_key(m_user.config, "Compositor", "ForceOpenGL");
+	Bool use_gl = (opt && !stricmp(opt, "yes")) ? 1 : 0;
+	gf_cfg_set_key(m_user.config, "Compositor", "ForceOpenGL", use_gl ? "no" : "yes");
 
-	ReloadTerminal();
+	gf_term_set_option(m_term, GF_OPT_USE_OPENGL, !use_gl);
+
+	UpdateRenderSwitch();
 }
 
 void wxOsmo4Frame::UpdateRenderSwitch()
 {
-	const char *opt = gf_cfg_get_key(m_user.config, "Rendering", "RendererName");
+	const char *opt = gf_cfg_get_key(m_user.config, "Compositor", "ForceOpenGL");
 	m_pToolBar->RemoveTool(SWITCH_RENDER);
-	if (!stricmp(opt, "GPAC 3D Renderer")) 
-		m_pToolBar->InsertTool(12, SWITCH_RENDER, *m_pSW3D, wxNullBitmap, FALSE, NULL, wxT("Switch to 2D Renderer"));
+	if (opt && !stricmp(opt, "yes")) 
+		m_pToolBar->InsertTool(12, SWITCH_RENDER, *m_pSW2D, wxNullBitmap, FALSE, NULL, wxT("2D Rasterizer"));
 	else
-		m_pToolBar->InsertTool(12, SWITCH_RENDER, *m_pSW2D, wxNullBitmap, FALSE, NULL, wxT("Switch to 3D Renderer"));
+		m_pToolBar->InsertTool(12, SWITCH_RENDER, *m_pSW3D, wxNullBitmap, FALSE, NULL, wxT("OpenGL Rendering"));
 
 #ifdef WIN32
 	/*there's a display bug with the menubtn, remove and reinsert*/
@@ -1997,11 +1985,6 @@ void wxOsmo4Frame::UpdateRenderSwitch()
 	m_pToolBar->Realize();
 }
 
-void wxOsmo4Frame::OnTermReload(wxCommandEvent &WXUNUSED(event))
-{
-	ReloadTerminal();
-}
-
 void wxOsmo4Frame::UpdatePlay()
 {
 	m_pToolBar->RemoveTool(FILE_PLAY);
@@ -2255,13 +2238,13 @@ void wxOsmo4Frame::BuildStreamList(Bool reset_only)
 		if (gf_term_get_object_info(m_term, odm, &info) != GF_OK) break;
 
 		if (info.owns_service) {
-			char *szName = strrchr(info.service_url, '\\');
-			if (!szName) szName = strrchr(info.service_url, '/');
+			char *szName = (char *) strrchr(info.service_url, '\\');
+			if (!szName) szName = (char *) strrchr(info.service_url, '/');
 			if (!szName) szName = (char *) info.service_url;
 			else szName += 1;
 			strcpy(szLabel, szName);
 			szName = strrchr(szLabel, '.');
-			if (szName) szName[0] = 0;
+			if (szName) szName[0] = '\0';
 		}
 		switch (info.od_type) {
 		case GF_STREAM_AUDIO:
@@ -2488,3 +2471,48 @@ void wxOsmo4Frame::OnUpdateChapterMenu(wxUpdateUIEvent & event)
 		event.Enable(1);
 	}
 }
+
+void wxOsmo4Frame::OnFileCopy(wxCommandEvent &event)
+{
+	const char *text = gf_term_get_text_selection(m_term, 0);
+	if (!text) return;
+	if (!wxTheClipboard->Open()) return;
+
+	wxTheClipboard->SetData( new wxTextDataObject( wxString(text, wxConvUTF8)) );
+	wxTheClipboard->Close();
+}
+
+void wxOsmo4Frame::OnUpdateFileCopy(wxUpdateUIEvent &event)
+{
+	if (gf_term_get_text_selection(m_term, 1)!=NULL) {
+		event.Enable(1);
+	} else {
+		event.Enable(0);
+	}
+}
+
+void wxOsmo4Frame::OnFilePaste(wxCommandEvent &event)
+{
+	if (!wxTheClipboard->Open()) return;
+    if (wxTheClipboard->IsSupported( wxDF_TEXT )) {
+		wxTextDataObject data;
+		wxTheClipboard->GetData(data);
+		gf_term_paste_text(m_term, data.GetText().mb_str(wxConvUTF8), 0);
+	}
+	wxTheClipboard->Close();
+}
+
+void wxOsmo4Frame::OnUpdateFilePaste(wxUpdateUIEvent &event)
+{
+	Bool ok = 0;
+	if (wxTheClipboard->Open()) {
+	    if (wxTheClipboard->IsSupported( wxDF_TEXT )) {
+			if (gf_term_paste_text(m_term, NULL, 1)==GF_OK) {
+				ok = 1;
+			}
+		}
+		wxTheClipboard->Close();
+	}
+	event.Enable(ok ? 1 : 0);
+}
+
