@@ -78,12 +78,15 @@ enum {
 	FILE_OPEN = wxID_HIGHEST+1,
 	FILE_OPEN_URL,
 	FILE_RELOAD,
+	FILE_RELOAD_CONFIG,
 	FILE_PLAY,
 	FILE_STEP,
 	FILE_STOP,
 	FILE_PREV,
 	FILE_NEXT,
 	FILE_PROPERTIES,
+	FILE_COPY,
+	FILE_PASTE,
 	TERM_RELOAD,
 	FILE_QUIT,
 	VIEW_FULLSCREEN,
@@ -236,7 +239,6 @@ public:
 	void DoConnect();
 
 	void ConnectAcknowledged(Bool bOk); 
-	void ReloadTerminal(); 
 	void SetStatus(wxString str);
 
 	void OnFilePlay(wxCommandEvent &event);
@@ -277,8 +279,8 @@ private:
 	void OnFileOpen(wxCommandEvent &event);
 	void OnFileOpenURL(wxCommandEvent &event);
 	void OnFileReload(wxCommandEvent &event);
+	void OnFileReloadConfig(wxCommandEvent & event);
 	void OnFileProperties(wxCommandEvent &event);
-	void OnTermReload(wxCommandEvent &event);
 	void OnFileQuit(wxCommandEvent &event);
 	void OnFullScreen(wxCommandEvent &event);
 	void OnOptions(wxCommandEvent &event);
@@ -343,6 +345,11 @@ private:
 	void OnUpdateCacheEnable(wxUpdateUIEvent & event);
 	void OnUpdateCacheAbort(wxUpdateUIEvent & event);
 
+	void OnFileCopy(wxCommandEvent &event);
+	void OnUpdateFileCopy(wxUpdateUIEvent &event);
+	void OnFilePaste(wxCommandEvent &event);
+	void OnUpdateFilePaste(wxUpdateUIEvent &event);
+
 
 	void CheckVideoOut();
 
