@@ -39,6 +39,8 @@
 #define WM_LOADTERM	WM_USER + 1
 #define STATE_TIMER_ID		20
 #define STATE_TIMER_DUR		1000
+#define GPAC_TIMER_ID		21
+#define GPAC_TIMER_DUR		33
 
 
 Bool gf_file_dialog(HINSTANCE inst, HWND parent, char *url, const char *ext_list, GF_Config *cfg);
@@ -52,6 +54,7 @@ static HWND g_hwnd_menu1 = NULL;
 static HWND g_hwnd_menu2 = NULL;
 static HWND g_hwnd_status = NULL;
 static HINSTANCE g_hinst = NULL;
+static Bool is_ppc = 0;
 
 static Bool is_connected = 0;
 static Bool navigation_on = 0;
@@ -78,18 +81,24 @@ static Bool reset_status = 1;
 static u32 last_state_time = 0;
 static Bool loop = 0;
 static Bool full_screen = 0;
-static Bool use_3D_renderer = 0;
+static Bool force_2d_gl = 0;
 static Bool ctrl_mod_down = 0;
 static Bool view_cpu = 0;
 static Bool menu_switched = 0;
 static Bool use_low_fps = 0;
 static Bool use_svg_prog = 0;
 
+static Bool log_rti = 0;
+static FILE *rti_file = NULL;
+static u32 rti_update_time_ms = 200;
+
+static u32 playlist_act = 0;
+
 void set_status(char *state)
 {
 	if (show_status && g_hwnd_status) {
 		TCHAR wstate[1024];
-		CE_CharToWide(state, wstate);
+		CE_CharToWide(state, (u16 *) wstate);
 		SendMessage(g_hwnd_status, WM_SETTEXT, 0, (LPARAM) wstate);
 		last_state_time = GetTickCount();
 	}
@@ -133,6 +142,7 @@ void update_state_info()
 static u32 prev_pos = 0;
 void cbk_on_progress(void *_title, u32 done, u32 total)
 {
+#if 0
 	char szMsg[1024];
 	u32 pos = (u32) ((u64) done * 100)/total;
 	if (pos<prev_pos) prev_pos = 0;
@@ -141,6 +151,7 @@ void cbk_on_progress(void *_title, u32 done, u32 total)
 		sprintf(szMsg, "%s: (%02d/100)", _title ? (char *)_title : "", pos);
 		set_status(szMsg);
 	}
+#endif
 }
 
 void set_full_screen()
@@ -156,6 +167,140 @@ void set_full_screen()
 }
 
 
+static void on_gpac_rti_log(void *cbk, u32 ll, u32 lm, const char *fmt, va_list list)
+{
+	GF_SystemRTInfo rti;
+
+	if (lm != GF_LOG_RTI) return;
+
+	gf_sys_get_rti(rti_update_time_ms, &rti, 0);
+	
+	fprintf(rti_file, "% 8d\t% 8d\t% 8d\t% 4d\t% 8d\t\t", 
+		gf_sys_clock(),
+		gf_term_get_time_in_ms(term),
+		rti.total_cpu_usage,
+		(u32) gf_term_get_framerate(term, 0),
+		(u32) ( rti.gpac_memory / 1024)
+	);
+	if (fmt) vfprintf(rti_file, fmt+6 /*[RTI] "*/, list);
+}
+
+static void on_gpac_log(void *cbk, u32 ll, u32 lm, const char *fmt, va_list list)
+{
+	if (rti_file && fmt) vfprintf(rti_file, fmt, list);	
+}
+
+static void setup_logs()
+{
+	if (rti_file) fclose(rti_file);
+	rti_file = NULL;
+
+	gf_log_set_level(GF_LOG_ERROR);
+	gf_log_set_tools(0);
+	gf_log_set_callback(NULL, NULL);
+
+	if (log_rti) {
+		rti_file = fopen("\\gpac_rti.txt", "a+t");
+
+		fprintf(rti_file, "!! GPAC RunTime Info for file %s !!\n", the_url);
+		fprintf(rti_file, "SysTime(ms)\tSceneTime(ms)\tCPU\tFPS\tMemory(kB)\tObservation\n");
+
+		gf_log_set_level(GF_LOG_DEBUG);
+		gf_log_set_tools(GF_LOG_RTI);
+		gf_log_set_callback(rti_file, on_gpac_rti_log);
+		GF_LOG(GF_LOG_DEBUG, GF_LOG_RTI, ("[RTI] System state when enabling log\n"));
+	} else {
+		u32 lt, ll;
+		const char *str = gf_cfg_get_key(user.config, "General", "LogLevel");
+		ll=0;
+		if (str && stricmp(str, "none")) {
+			if (!stricmp(str, "debug")) ll = GF_LOG_DEBUG;
+			else if (!stricmp(str, "info")) ll = GF_LOG_INFO;
+			else if (!stricmp(str, "warning")) ll = GF_LOG_WARNING;
+			else if (!stricmp(str, "error")) ll = GF_LOG_ERROR;
+			gf_log_set_level(ll);
+		}
+		lt = 0;
+		str = gf_cfg_get_key(user.config, "General", "LogTools");
+		if (str) {
+			char *sep;
+			char *val = (char *) str;
+			while (val) {
+				sep = strchr(val, ':');
+				if (sep) sep[0] = 0;
+				if (!stricmp(val, "core")) lt |= GF_LOG_CODING;
+				else if (!stricmp(val, "coding")) lt |= GF_LOG_CODING;
+				else if (!stricmp(val, "container")) lt |= GF_LOG_CONTAINER;
+				else if (!stricmp(val, "network")) lt |= GF_LOG_NETWORK;
+				else if (!stricmp(val, "rtp")) lt |= GF_LOG_RTP;
+				else if (!stricmp(val, "author")) lt |= GF_LOG_AUTHOR;
+				else if (!stricmp(val, "sync")) lt |= GF_LOG_SYNC;
+				else if (!stricmp(val, "codec")) lt |= GF_LOG_CODEC;
+				else if (!stricmp(val, "parser")) lt |= GF_LOG_PARSER;
+				else if (!stricmp(val, "media")) lt |= GF_LOG_MEDIA;
+				else if (!stricmp(val, "scene")) lt |= GF_LOG_SCENE;
+				else if (!stricmp(val, "script")) lt |= GF_LOG_SCRIPT;
+				else if (!stricmp(val, "interact")) lt |= GF_LOG_INTERACT;
+				else if (!stricmp(val, "compose")) lt |= GF_LOG_COMPOSE;
+				else if (!stricmp(val, "mmio")) lt |= GF_LOG_MMIO;
+				else if (!stricmp(val, "none")) ll = 0;
+				else if (!stricmp(val, "all")) lt = 0xFFFFFFFF;
+				if (!sep) break;
+				sep[0] = ':';
+				val = sep+1;
+			}
+			gf_log_set_tools(lt);
+		}
+		if (ll && (rti_file = fopen("\\gpac_logs.txt", "wt"))) {
+			gf_log_set_callback(rti_file, on_gpac_log);
+		}
+
+	}
+}
+
+void do_open_file()
+{
+	gf_cfg_set_key(user.config, "RecentFiles", the_url, NULL);
+	gf_cfg_insert_key(user.config, "RecentFiles", the_url, "", 0);
+	u32 count = gf_cfg_get_key_count(user.config, "RecentFiles");
+	if (count > 10) gf_cfg_set_key(user.config, "RecentFiles", gf_cfg_get_key_name(user.config, "RecentFiles", count-1), NULL);
+
+	setup_logs();
+
+	gf_term_connect(term, the_url);
+}
+
+void switch_playlist(Bool play_prev)
+{
+	char szPLE[20];
+	u32 idx = 0;
+	u32 count;
+	const char *ple = gf_cfg_get_key(user.config, "General", "PLEntry");
+	if (ple) idx = atoi(ple);
+	
+	count = gf_cfg_get_key_count(user.config, "Playlist");
+	if (!count) return;
+	/*not the first launch*/
+	if (strlen(the_url)) {
+		if (!idx && play_prev) return;
+		if (play_prev) idx--;
+		else idx++;
+		if (idx>=count) return;
+	} else {
+		if (idx>=count) idx=0;
+	}
+
+	ple = gf_cfg_get_key_name(user.config, "Playlist", idx);
+	if (!ple) return;
+
+	sprintf(szPLE, "%d", idx);
+	gf_cfg_set_key(user.config, "General", "PLEntry", szPLE);
+
+	strcpy(the_url, ple);
+	do_open_file();
+}
+
+
 Bool GPAC_EventProc(void *ptr, GF_Event *evt)
 {
 	switch (evt->type) {
@@ -209,27 +354,59 @@ Bool GPAC_EventProc(void *ptr, GF_Event *evt)
 			evt->type = ctrl_mod_down ? GF_EVENT_KEYDOWN : GF_EVENT_KEYUP;
 			gf_term_user_event(term, evt);
 			break;
+		case GF_KEY_MEDIAPREVIOUSTRACK:
+			playlist_act = 2;
+			break;
+		case GF_KEY_MEDIANEXTTRACK:
+			playlist_act = 1;
+			break;
 		}
 		break;
-	case GF_EVENT_MOUSEDOUBLECLICK:
-		set_full_screen();
-		return 0;
+	case GF_EVENT_NAVIGATE:
+		if (gf_term_is_supported_url(term, evt->navigate.to_url, 1, 1)) {
+			gf_term_navigate_to(term, evt->navigate.to_url);
+			return 1;
+		} else {
+			u16 dst[1024];
+			SHELLEXECUTEINFO info;
+
+/*
+			if (full_screen) gf_term_set_option(term, GF_OPT_FULLSCREEN, 0);
+			full_screen = 0;
+*/
+			memset(&info, 0, sizeof(SHELLEXECUTEINFO));
+			info.cbSize = sizeof(SHELLEXECUTEINFO);
+			info.lpVerb = L"open";
+			info.fMask = SEE_MASK_NOCLOSEPROCESS;
+			info.lpFile = L"iexplore";
+			CE_CharToWide((char *) evt->navigate.to_url, dst);
+			info.lpParameters = (LPCTSTR) dst;
+			info.nShow = SW_SHOWNORMAL;
+			ShellExecuteEx(&info);
+		}
+		return 1;
 	}
 	return 0;
 }
 
+#define TERM_NOT_THREADED
 
 Bool LoadTerminal()
 {
 	/*by default use current dir*/
 	strcpy(the_url, ".");
 
+	setup_logs();
+
 	g_hwnd_disp = CreateWindow(TEXT("STATIC"), NULL, WS_CHILD | WS_VISIBLE , 0, 0, disp_w, disp_h, g_hwnd, NULL, g_hinst, NULL);
 
 	user.EventProc = GPAC_EventProc;
 	/*dummy in this case (global vars) but MUST be non-NULL*/
 	user.opaque = user.modules;
 	user.os_window_handler = g_hwnd_disp;
+#ifdef TERM_NOT_THREADED
+	user.init_flags = GF_TERM_NO_VISUAL_THREAD | GF_TERM_NO_REGULATION;
+#endif
 
 	term = gf_term_new(&user);
 	if (!term) {
@@ -239,13 +416,16 @@ Bool LoadTerminal()
 		return 0;
 	}
 
+#ifdef TERM_NOT_THREADED
+	::SetTimer(g_hwnd, GPAC_TIMER_ID, GPAC_TIMER_DUR, NULL);
+#endif
+
 	const char *str = gf_cfg_get_key(user.config, "General", "StartupFile");
 	if (str) {
 		do_layout(1);
 		strcpy(the_url, str);
 		gf_term_connect(term, str);
 	}
-
 	return 1;
 }
 
@@ -258,15 +438,18 @@ void do_layout(Bool notif_size)
 		::ShowWindow(g_hwnd_status, SW_HIDE);
 		::ShowWindow(g_hwnd_menu1, SW_HIDE);
 		::ShowWindow(g_hwnd_menu2, SW_HIDE);
+
+		SHFullScreen(g_hwnd, SHFS_HIDESIPBUTTON);
 		::MoveWindow(g_hwnd, 0, 0, screen_w, screen_h, 1);
 		::MoveWindow(g_hwnd_disp, 0, 0, screen_w, screen_h, 1);
+		SetForegroundWindow(g_hwnd_disp);
 	} else {
 		::ShowWindow(g_hwnd_menu1, menu_switched ? SW_HIDE : SW_SHOW);
 		::ShowWindow(g_hwnd_menu2, menu_switched ? SW_SHOW : SW_HIDE);
 		if (show_status) {
-			::MoveWindow(g_hwnd, 0, caption_h, disp_w, disp_h, 1);
+			::MoveWindow(g_hwnd, 0, 0, disp_w, disp_h, 1);
 			::ShowWindow(g_hwnd_status, SW_SHOW);
-			::MoveWindow(g_hwnd_status, 0, 0, disp_w, caption_h-2, 1);
+			::MoveWindow(g_hwnd_status, 0, 0, disp_w, caption_h, 1);
 			::MoveWindow(g_hwnd_disp, 0, caption_h, disp_w, disp_h - caption_h, 1);
 			w = disp_w;
 			h = disp_h - caption_h;
@@ -336,11 +519,13 @@ void freeze_display(Bool do_freeze)
 		}
 		/*freeze display*/
 		gf_term_set_option(term, GF_OPT_FREEZE_DISPLAY, 1);
+		
 		set_backlight_state(0);
 		gf_sleep(100);
 	} else {
 		if (prev_backlight_state) set_backlight_state(1);
 		gf_term_set_option(term, GF_OPT_FREEZE_DISPLAY, 0);
+
 		if (do_resume) {
 			gf_term_set_option(term, GF_OPT_PLAY_STATE, GF_STATE_PLAYING);
 			set_backlight_state(1);
@@ -348,14 +533,20 @@ void freeze_display(Bool do_freeze)
 	}
 }
 
-void do_open_file()
+static void show_taskbar(Bool show_it)
 {
-	gf_cfg_set_key(user.config, "RecentFiles", the_url, NULL);
-	gf_cfg_insert_key(user.config, "RecentFiles", the_url, "", 0);
-	u32 count = gf_cfg_get_key_count(user.config, "RecentFiles");
-	if (count > 10) gf_cfg_set_key(user.config, "RecentFiles", gf_cfg_get_key_name(user.config, "RecentFiles", count-1), NULL);
+	HWND wnd;
+	if (!is_ppc) return;
 
-	gf_term_connect(term, the_url);
+	wnd = GetForegroundWindow();
+	wnd = g_hwnd;
+	if (show_it) {
+		SHFullScreen(wnd, SHFS_SHOWSTARTICON | SHFS_SHOWTASKBAR| SHFS_SHOWSIPBUTTON);
+		::ShowWindow(::FindWindow(_T("HHTaskbar"),NULL), SW_SHOWNA);
+	} else {
+		::ShowWindow(::FindWindow(_T("HHTaskbar"),NULL), SW_HIDE);
+		SHFullScreen(wnd, SHFS_HIDESTARTICON | SHFS_HIDETASKBAR| SHFS_HIDESIPBUTTON);
+	}
 }
 
 void refresh_recent_files()
@@ -377,40 +568,11 @@ void refresh_recent_files()
 		if (!name) name = (char *) sOpt;
 		else name += 1;
 
-		CE_CharToWide(name, txt);
+		CE_CharToWide(name, (u16 *) txt);
 		AppendMenu(hMenu, MF_STRING, IDM_OPEN_FILE1+i, txt);
 	}
 }
 
-void switch_playlist(Bool play_prev)
-{
-	char szPLE[20];
-	u32 idx = 0;
-	u32 count;
-	const char *ple = gf_cfg_get_key(user.config, "General", "PLEntry");
-	if (ple) idx = atoi(ple);
-	
-	count = gf_cfg_get_key_count(user.config, "Playlist");
-	if (!count) return;
-	/*not the first launch*/
-	if (strlen(the_url)) {
-		if (!idx && play_prev) return;
-		if (play_prev) idx--;
-		else idx++;
-		if (idx>=count) return;
-	} else {
-		if (idx>=count) idx=0;
-	}
-
-	ple = gf_cfg_get_key_name(user.config, "Playlist", idx);
-	if (!ple) return;
-
-	sprintf(szPLE, "%d", idx);
-	gf_cfg_set_key(user.config, "General", "PLEntry", szPLE);
-
-	strcpy(the_url, ple);
-	do_open_file();
-}
 
 void open_file(HWND hwnd)
 {
@@ -493,21 +655,6 @@ void view_about(HWND hwnd)
 	freeze_display(0);
 }
 
-void reload_terminal()
-{
-	HCURSOR hcur = SetCursor(LoadCursor(NULL, IDC_WAIT));
-	gf_term_del(term);
-	term = gf_term_new(&user);
-	if (!term) {
-		MessageBox(NULL, _T("Fatal Error !!"), _T("Couldn't change renderer"), MB_OK);
-		PostQuitMessage(0);
-	} else {
-		const char *str = gf_cfg_get_key(user.config, "Rendering", "RendererName");
-		use_3D_renderer = (str && strstr(str, "3D")) ? 1 : 0;
-	}
-	SetCursor(hcur);
-}
-
 void pause_file()
 {
 	if (!is_connected) return;
@@ -538,17 +685,48 @@ void set_svg_progressive()
 	}
 }
 
+void do_copy_paste()
+{
+	if (!OpenClipboard(g_hwnd)) return;
+
+	/*or we are editing text and clipboard is not empty*/
+	if (IsClipboardFormatAvailable(CF_TEXT) && (gf_term_paste_text(term, NULL, 1)==GF_OK)) {
+		HGLOBAL hglbCopy = GetClipboardData(CF_TEXT);
+		if (hglbCopy) {
+			char szString[1024];
+			LPCTSTR paste_string = (LPCTSTR) GlobalLock(hglbCopy);
+			CE_WideToChar((u16 *) paste_string, szString);
+			gf_term_paste_text(term, szString, 0);
+			GlobalUnlock(hglbCopy); 
+		}
+	}
+	/*we have something to copy*/
+	else if (gf_term_get_text_selection(term, 1)!=NULL) {
+		u32 len;
+		const char *text = gf_term_get_text_selection(term, 0);
+		if (text && strlen(text)) {
+			EmptyClipboard();
+			len = strlen(text);
+
+			HGLOBAL hglbCopy = GlobalAlloc(GMEM_MOVEABLE, (len + 1) * sizeof(u16)); 
+			LPCTSTR new_string = (LPCTSTR) GlobalLock(hglbCopy);
+			CE_CharToWide((char*)text, (u16*)new_string); 
+			GlobalUnlock(hglbCopy); 
+			SetClipboardData(CF_TEXT, hglbCopy);
+		}
+	}
+	CloseClipboard(); 
+}
+
 BOOL HandleCommand(HWND hwnd, WPARAM wParam, LPARAM lParam)
 {
 	switch (LOWORD(wParam)) {
 	case IDM_FILE_OPEN:
 		open_file(hwnd);
 		break;
-	case IDM_FILE_RELOAD:
-		/*doesn't work with SVG files yet...*/
-		//gf_term_play_from_time(term, 0);
-		gf_term_disconnect(term);
-		gf_term_connect(term, the_url);
+	case IDM_FILE_LOG_RTI:
+		log_rti = !log_rti;
+		setup_logs();
 		break;
 	case IDM_FILE_PAUSE:
 		pause_file();
@@ -566,10 +744,10 @@ BOOL HandleCommand(HWND hwnd, WPARAM wParam, LPARAM lParam)
 	case IDM_VIEW_CPU:
 		view_cpu = !view_cpu;
 		break;
-	case IDM_VIEW_3DREND:
-		use_3D_renderer = !use_3D_renderer;
-		gf_cfg_set_key(user.config, "Rendering", "RendererName", use_3D_renderer ? "GPAC 3D Renderer" : "GPAC 2D Renderer");
-		reload_terminal();
+	case IDM_VIEW_FORCEGL:
+		force_2d_gl = !force_2d_gl;
+		gf_cfg_set_key(user.config, "Compositor", "ForceOpenGL", force_2d_gl ? "yes" : "no");
+		gf_term_set_option(term, GF_OPT_USE_OPENGL, force_2d_gl);
 		break;
 	case IDM_NAV_RESET:
 		gf_term_set_option(term, GF_OPT_NAVIGATION_TYPE, 0);
@@ -624,9 +802,9 @@ BOOL HandleCommand(HWND hwnd, WPARAM wParam, LPARAM lParam)
 		break;
 	case IDM_VIEW_DIRECT:
 	{
-		Bool drend = gf_term_get_option(term, GF_OPT_DIRECT_RENDER) ? 0 : 1;
-		gf_term_set_option(term, GF_OPT_DIRECT_RENDER, drend);
-		gf_cfg_set_key(user.config, "Render2D", "DirectRender", drend ? "yes" : "no");
+		Bool drend = gf_term_get_option(term, GF_OPT_DIRECT_DRAW) ? 0 : 1;
+		gf_term_set_option(term, GF_OPT_DIRECT_DRAW, drend);
+		gf_cfg_set_key(user.config, "Compositor", "DirectDraw", drend ? "yes" : "no");
 	}
 		break;
 
@@ -635,6 +813,9 @@ BOOL HandleCommand(HWND hwnd, WPARAM wParam, LPARAM lParam)
 		menu_switched = !menu_switched;
 		do_layout(0);
 		break;
+	case ID_FILE_CUT_PASTE:
+		do_copy_paste();
+		break;
 
 	case IDM_OPEN_FILE1: 
 	case IDM_OPEN_FILE2: 
@@ -666,13 +847,13 @@ static BOOL OnMenuPopup(const HWND hWnd, const WPARAM wParam)
 {
 	if (menu_switched) {
 		CheckMenuItem((HMENU)wParam, IDM_VIEW_STATUS, MF_BYCOMMAND| (show_status ? MF_CHECKED : MF_UNCHECKED) );
-		CheckMenuItem((HMENU)wParam, IDM_VIEW_3DREND, MF_BYCOMMAND| (use_3D_renderer ? MF_CHECKED : MF_UNCHECKED) );
+		CheckMenuItem((HMENU)wParam, IDM_VIEW_FORCEGL, MF_BYCOMMAND| (force_2d_gl ? MF_CHECKED : MF_UNCHECKED) );
 		EnableMenuItem((HMENU)wParam, IDM_VIEW_CPU, MF_BYCOMMAND| (show_status ? MF_ENABLED : MF_GRAYED) );
 		if (show_status) CheckMenuItem((HMENU)wParam, IDM_VIEW_CPU, MF_BYCOMMAND| (view_cpu ? MF_CHECKED : MF_UNCHECKED) );
 		CheckMenuItem((HMENU)wParam, IDM_VIEW_LOW_RATE, MF_BYCOMMAND| (use_low_fps ? MF_CHECKED : MF_UNCHECKED) );
 
-		EnableMenuItem((HMENU)wParam, IDM_VIEW_DIRECT, MF_BYCOMMAND| (!use_3D_renderer ? MF_ENABLED : MF_GRAYED) );
-		CheckMenuItem((HMENU)wParam, IDM_VIEW_DIRECT, MF_BYCOMMAND| (!use_3D_renderer && gf_term_get_option(term, GF_OPT_DIRECT_RENDER) ? MF_CHECKED : MF_UNCHECKED) );
+		EnableMenuItem((HMENU)wParam, IDM_VIEW_DIRECT, MF_BYCOMMAND| (!force_2d_gl ? MF_ENABLED : MF_GRAYED) );
+		CheckMenuItem((HMENU)wParam, IDM_VIEW_DIRECT, MF_BYCOMMAND| (!force_2d_gl && gf_term_get_option(term, GF_OPT_DIRECT_DRAW) ? MF_CHECKED : MF_UNCHECKED) );
 
 		CheckMenuItem((HMENU)wParam, IDM_VIEW_SVG_LOAD, MF_BYCOMMAND| (use_svg_prog ? MF_CHECKED : MF_UNCHECKED) );
 		return TRUE;
@@ -685,9 +866,20 @@ static BOOL OnMenuPopup(const HWND hWnd, const WPARAM wParam)
 	CheckMenuItem((HMENU)wParam, IDM_VIEW_AR_4_3, MF_BYCOMMAND| (opt==GF_ASPECT_RATIO_4_3) ? MF_CHECKED : MF_UNCHECKED);
 	CheckMenuItem((HMENU)wParam, IDM_VIEW_AR_16_9, MF_BYCOMMAND| (opt==GF_ASPECT_RATIO_16_9) ? MF_CHECKED : MF_UNCHECKED);
 
-	EnableMenuItem((HMENU)wParam, IDM_FILE_RELOAD, MF_BYCOMMAND| (is_connected ? MF_ENABLED : MF_GRAYED) );
 	EnableMenuItem((HMENU)wParam, IDM_FILE_PAUSE, MF_BYCOMMAND| (is_connected ? MF_ENABLED : MF_GRAYED) );
 
+	if (/*we have something to copy*/
+		(gf_term_get_text_selection(term, 1)!=NULL) 
+		/*or we are editing text and clipboard is not empty*/
+		|| (IsClipboardFormatAvailable(CF_TEXT) && (gf_term_paste_text(term, NULL, 1)==GF_OK))
+	) {
+		EnableMenuItem((HMENU)wParam, ID_FILE_CUT_PASTE, MF_BYCOMMAND| MF_ENABLED);
+	} else {
+		EnableMenuItem((HMENU)wParam, ID_FILE_CUT_PASTE, MF_BYCOMMAND| MF_GRAYED);
+	}
+	
+	CheckMenuItem((HMENU)wParam, IDM_FILE_LOG_RTI, MF_BYCOMMAND| (log_rti) ? MF_CHECKED : MF_UNCHECKED);
+	
 	u32 type;
 	if (!is_connected) type = 0;
 	else type = gf_term_get_option(term, GF_OPT_NAVIGATION_TYPE);
@@ -759,7 +951,10 @@ BOOL CALLBACK MainWndProc (HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
 			return DefWindowProc(hwnd, msg, wParam, lParam);
 		break;
 	case WM_TIMER:
-		update_state_info();
+		if (wParam==STATE_TIMER_ID) update_state_info();
+#ifdef TERM_NOT_THREADED
+		else gf_term_process_step(term);
+#endif
 		break;
 	case WM_HOTKEY:
 		break;
@@ -839,12 +1034,13 @@ BOOL InitInstance (int CmdShow)
 	if (!g_hwnd) return FALSE;
 
 	ShowWindow(g_hwnd, CmdShow);
-	::SetTimer(g_hwnd, STATE_TIMER_ID, STATE_TIMER_DUR, NULL);
 
 	refresh_recent_files();
-	set_status("Loading Terminal");
+
+	::SetTimer(g_hwnd, STATE_TIMER_ID, STATE_TIMER_DUR, NULL);
 	do_layout(1);
 
+	set_status("Loading Terminal");
 	PostMessage(g_hwnd, WM_LOADTERM, 0, 0);
 	set_status("Ready");
 	return TRUE;
@@ -866,13 +1062,12 @@ Bool initial_setup(const char *szExePath)
 
 	strcpy(szPath, szExePath);
 	strcat(szPath, "cache");
-	CE_CharToWide(szPath, wzPath);
+	CE_CharToWide(szPath, (u16 *) wzPath);
 	CreateDirectory(wzPath, NULL);
 	gf_cfg_set_key(user.config, "General", "CacheDirectory", szPath);
 
 	/*base rendering options*/
-	gf_cfg_set_key(user.config, "Rendering", "RendererName", "GPAC 2D Renderer");
-	gf_cfg_set_key(user.config, "Rendering", "Raster2D", "GPAC 2D Raster");
+	gf_cfg_set_key(user.config, "Compositor", "Raster2D", "GPAC 2D Raster");
 	/*enable UDP traffic autodetect*/
 	gf_cfg_set_key(user.config, "Network", "AutoReconfigUDP", "yes");
 	gf_cfg_set_key(user.config, "Network", "UDPTimeout", "10000");
@@ -914,12 +1109,18 @@ int WINAPI WinMain(HINSTANCE hInstance,
 	HWND 	hwndOld = NULL;	
 	const char *str;
 	Bool initial_launch = 0;
+
+	if (hwndOld = FindWindow(_T("Osmophone"), NULL))
+	{
+		SetForegroundWindow(hwndOld);    
+		return 0;
+	}
 	
 	memset(&user, 0, sizeof(GF_User));
 	term = NULL;
 
 	GetModuleFileName(NULL, wzExePath, GF_MAX_PATH);
-	CE_WideToChar(wzExePath, szExePath);
+	CE_WideToChar((u16 *) wzExePath, szExePath);
 	char *sep = strrchr(szExePath, '\\');
 	sep[1] = 0;
 
@@ -930,14 +1131,15 @@ int WINAPI WinMain(HINSTANCE hInstance,
 	screen_w = GetSystemMetrics(SM_CXSCREEN);
 	screen_h = GetSystemMetrics(SM_CYSCREEN);
 	disp_w = screen_w;
-	disp_h = screen_h - menu_h - caption_h;
+	disp_h = screen_h - menu_h /*- caption_h*/;
 
 	
-	if (hwndOld = FindWindow(_T("Osmophone"), NULL))
-	{
-		SetForegroundWindow(hwndOld);    
-		return 0;
-	}
+	TCHAR      szBuf[MAX_PATH];
+	SystemParametersInfo(SPI_GETPLATFORMTYPE, MAX_PATH, szBuf, 0);
+
+	if (! lstrcmp(szBuf, __TEXT("PocketPC"))) is_ppc = 1;
+	else if (! lstrcmp(szBuf, __TEXT("Palm PC2"))) is_ppc = 1;
+
 
 	user.config = gf_cfg_new(szExePath, "GPAC.cfg");
 	if (!user.config) {
@@ -983,19 +1185,37 @@ int WINAPI WinMain(HINSTANCE hInstance,
 	str = gf_cfg_get_key(user.config, "General", "Loop");
 	loop = (str && !stricmp(str, "yes")) ? 1 : 0;
 
-	str = gf_cfg_get_key(user.config, "Rendering", "RendererName");
-	use_3D_renderer = (str && strstr(str, "3D")) ? 1 : 0;
-
 	str = gf_cfg_get_key(user.config, "SAXLoader", "Progressive");
 	use_svg_prog = (str && !strcmp(str, "yes")) ? 1 : 0;
 
+	str = gf_cfg_get_key(user.config, "General", "RTIRefreshPeriod");
+	if (str) {
+		rti_update_time_ms = atoi(str);
+	} else {
+		gf_cfg_set_key(user.config, "General", "RTIRefreshPeriod", "200");
+	}
+
+
+	if (is_ppc) GXOpenInput();
+
 	if (InitInstance(nShowCmd)) {
 		SetForegroundWindow(g_hwnd);
+		show_taskbar(0);
+
+		force_2d_gl = gf_term_get_option(term, GF_OPT_USE_OPENGL);
+
 		while (GetMessage(&msg, NULL, 0,0) == TRUE) {
 			TranslateMessage (&msg);
 			DispatchMessage (&msg);
+
+			if (playlist_act) {
+				switch_playlist(playlist_act-1);
+				playlist_act = 0;
+			}
 		}
+		show_taskbar(1);
 	}
+	if (is_ppc) GXCloseInput();
 
 	/*and destroy*/
 	if (term) gf_term_del(term);
@@ -1005,5 +1225,6 @@ int WINAPI WinMain(HINSTANCE hInstance,
 	if (backlight_off) set_backlight_state(0);
 
 	gf_sys_close();
+	if (rti_file) fclose(rti_file);
 	return 0;
 }
