@@ -35,6 +35,7 @@ void usage()
 					"                * option can be used several times, once for each program\n"
 					"-rate=R        specifies target rate in kbits/sec of the multiplex\n"
 					"                If not set, transport stream will be of variable bitrate\n"
+					"-mpeg4			forces usage of MPEG-4 signaling (use of IOD and SL Config)\n"
 					"\n"
 					"dst can be a file, an RTP or a UDP destination (unicast/multicast)\n"
 		);
@@ -206,7 +207,147 @@ void m2ts_mux_table_update(M2TS_Mux_Stream *stream, u8 table_id, u16 table_id_ex
 
 		if (use_syntax_indicator) {
 			u32 CRC;
-			CRC = gf_m2ts_crc32(section->data,section->length-CRC_LENGTH); 
+			CRC = gf_crc_32(section->data,section->length-CRC_LENGTH); 
+			section->data[section->length-4] = (CRC >> 24) & 0xFF;
+			section->data[section->length-3] = (CRC >> 16) & 0xFF;
+			section->data[section->length-2] = (CRC >> 8) & 0xFF;
+			section->data[section->length-1] = CRC & 0xFF;
+		}
+	
+		if (prev_sec) prev_sec->next = section;
+		else table->section = section;
+		prev_sec = section;
+	}
+	stream->current_table = stream->tables;
+	stream->current_section = stream->current_table->section;
+	stream->current_section_offset = 0;
+}
+
+void m2ts_mux_table_update_mpeg4(M2TS_Mux_Stream *stream, u8 table_id, u16 table_id_extension,
+						   u8 *table_payload, u32 table_payload_length, 
+						   Bool use_syntax_indicator, Bool private_indicator,
+						   Bool use_checksum) 
+{
+	GF_SLHeader hdr;
+	u32 overhead_size;
+	u32 offset, sl_size;
+	u32 section_number, nb_sections;
+	M2TS_Mux_Table *table, *prev_table;
+	/*max section length for MPEG-4 BIFS and OD*/
+	u32 maxSectionLength = 4096;
+	M2TS_Mux_Section *section, *prev_sec;
+	GF_BitStream *bs;
+
+	/* check if there is already a table with that id */
+	prev_table = NULL;
+	table = stream->tables;
+	while (table) {
+		if (table->table_id == table_id) {
+			/* if yes, we need to flush the table and increase the version number */
+			M2TS_Mux_Section *sec = table->section;
+			while (sec) {
+				M2TS_Mux_Section *sec2 = sec->next;
+				free(sec->data);
+				free(sec);
+				sec = sec2;
+			}
+			table->version_number = (table->version_number + 1)%0x1F;
+			break;
+		}
+		prev_table = table;
+		table = table->next;
+	}
+
+	if (!table) {
+		/* if no, the table is created */
+		GF_SAFEALLOC(table, M2TS_Mux_Table);
+		table->table_id = table_id;
+		if (prev_table) prev_table->next = table;
+		else stream->tables = table;
+	}
+
+	if (!table_payload_length) return;
+
+	overhead_size = SECTION_HEADER_LENGTH;
+	if (use_syntax_indicator) overhead_size += SECTION_ADDITIONAL_HEADER_LENGTH + CRC_LENGTH;
+	
+	section_number = 0;
+	nb_sections = 1;
+	hdr = stream->sl_header;
+	sl_size = gf_sl_get_header_size(&stream->ifce->sl_config, &hdr);
+	/*SL-packetized data doesn't fit in one section, we must repacketize*/
+	if (sl_size + table_payload_length > maxSectionLength - overhead_size) {
+		nb_sections = 0;
+		offset = 0;
+		hdr.accessUnitEndFlag = 0;
+		while (offset<table_payload_length) {
+			sl_size = gf_sl_get_header_size(&stream->ifce->sl_config, &hdr);
+			/*remove start flag*/
+			hdr.accessUnitStartFlag = 0;
+			/*fill each section but beware of last packet*/
+			offset += maxSectionLength - overhead_size - sl_size;
+			nb_sections++;
+		}
+	}	
+	prev_sec = NULL;
+	offset = 0;
+	hdr = stream->sl_header;
+	while (offset < table_payload_length) {
+		u32 remain;
+		char *slhdr;
+		u32 slhdr_size;		
+		GF_SAFEALLOC(section, M2TS_Mux_Section);
+
+		hdr.accessUnitEndFlag = (section_number+1==nb_sections) ? stream->sl_header.accessUnitEndFlag : 0;
+		gf_sl_packetize(&stream->ifce->sl_config, &hdr, NULL, 0, &slhdr, &slhdr_size);
+		hdr.accessUnitStartFlag = 0;
+
+		remain = table_payload_length - offset;
+		if (remain > maxSectionLength - overhead_size - slhdr_size) {
+			section->length = maxSectionLength;
+		} else {
+			section->length = remain + overhead_size + slhdr_size;
+		}
+		sl_size = section->length - overhead_size - slhdr_size;
+
+		bs = gf_bs_new(NULL,0,GF_BITSTREAM_WRITE);
+
+		/* first header (not included in section length */
+		gf_bs_write_int(bs,	table_id, 8);
+		gf_bs_write_int(bs,	use_syntax_indicator, 1);
+		gf_bs_write_int(bs,	private_indicator, 1); 
+		gf_bs_write_int(bs,	3, 2);  /* reserved bits are all set */
+		gf_bs_write_int(bs,	section->length - SECTION_HEADER_LENGTH, 12);
+	
+		if (use_syntax_indicator) {
+			/* second header */
+			gf_bs_write_int(bs,	table_id_extension, 16);
+			gf_bs_write_int(bs,	3, 2); /* reserved bits are all set */
+			gf_bs_write_int(bs,	table->version_number, 5);
+			gf_bs_write_int(bs,	1, 1); /* current_next_indicator = 1: we don't send version in advance */
+			gf_bs_write_int(bs,	section_number, 8);
+			section_number++;
+			gf_bs_write_int(bs,	nb_sections-1, 8);
+		}
+
+		/*write sl header*/
+		gf_bs_write_data(bs, slhdr, slhdr_size);
+		free(slhdr);
+		/*write sl data*/
+		gf_bs_write_data(bs, table_payload + offset, sl_size);
+		offset += sl_size;
+	
+		if (use_syntax_indicator) {
+			/* place holder for CRC */
+			gf_bs_write_u32(bs, 0);
+		}
+
+		gf_bs_get_content(bs, (char**) &section->data, &section->length); 
+		gf_bs_del(bs);
+
+		if (use_syntax_indicator) {
+			u32 CRC;
+			CRC = gf_crc_32(section->data,section->length-CRC_LENGTH); 
 			section->data[section->length-4] = (CRC >> 24) & 0xFF;
 			section->data[section->length-3] = (CRC >> 16) & 0xFF;
 			section->data[section->length-2] = (CRC >> 8) & 0xFF;
@@ -326,25 +467,27 @@ void m2ts_mux_table_get_next_packet(M2TS_Mux_Stream *stream, u8 *packet)
 		if (section->length - stream->current_section_offset >= payload_length) {
 			padding_length = 0;
 		} else {
-			padding_length = payload_length - section->length - stream->current_section_offset; 
+			padding_length = payload_length - section->length + stream->current_section_offset; 
 			payload_length -= padding_length;
 		}
 	}
 	assert(payload_length + stream->current_section_offset <= section->length);
 	
-	gf_bs_write_int(bs,	0, 1);    // priority indicator
-	gf_bs_write_int(bs,	stream->pid, 13); // section pid
-	gf_bs_write_int(bs,	0, 2);    // scrambling indicator
-	gf_bs_write_int(bs,	adaptation_field_control, 2);    // we do not use adaptation field for sections 
-	gf_bs_write_int(bs,	stream->continuity_counter, 4);   // continuity counter
+	gf_bs_write_int(bs,	0, 1);    /*priority indicator*/
+	gf_bs_write_int(bs,	stream->pid, 13); /*pid*/
+	gf_bs_write_int(bs,	0, 2);    /*scrambling indicator*/
+	gf_bs_write_int(bs,	adaptation_field_control, 2);    /*we do not use adaptation field for sections */
+	gf_bs_write_int(bs,	stream->continuity_counter, 4);   /*continuity counter*/
 	if (stream->continuity_counter < 15) stream->continuity_counter++;
 	else stream->continuity_counter=0;
 
 	if (adaptation_field_control != M2TS_ADAPTATION_NONE)
 		m2ts_add_adaptation(bs, 0, 0, 0, padding_length);
 
-	if (!stream->current_section_offset) { //write pointer field
-		gf_bs_write_u8(bs, 0); /* no concatenations of sections in ts packets, so start address is 0 */
+	/*pointer field*/
+	if (!stream->current_section_offset) {
+		/* no concatenations of sections in ts packets, so start address is 0 */
+		gf_bs_write_u8(bs, 0);
 	}
 	gf_bs_del(bs);
 
@@ -358,12 +501,13 @@ void m2ts_mux_table_get_next_packet(M2TS_Mux_Stream *stream, u8 *packet)
 		stream->current_section = stream->current_section->next;
 		if (!stream->current_section) {
 			stream->current_table = stream->current_table->next;
-			if (!stream->current_table) {
+			/*carousel table*/
+			if (!stream->current_table && stream->refresh_rate_ms) {
 				stream->current_table = stream->tables;
-				/*update time*/
+				/*update ES time*/
 				m2ts_time_inc(&stream->time, stream->refresh_rate_ms, 1000);
 			}
-			stream->current_section = stream->current_table->section;
+			if (stream->current_table) stream->current_section = stream->current_table->section;
 		}
 	}
 
@@ -409,27 +553,35 @@ Bool m2ts_stream_process_pmt(M2TS_Mux *muxer, M2TS_Mux_Stream *stream)
 		gf_bs_write_int(bs,	stream->program->pcr->pid, 13);
 		gf_bs_write_int(bs,	0xF, 4); // reserved
 	
-		if (1 /* !iod */) {
+		if (!stream->program->iod) {
 			gf_bs_write_int(bs,	0, 12); // program info length =0
 		} else {
-#if 0
 			u32 len;
-			len = iod->length + 4;
-			gf_bs_write_int(bs,	len, 12); // program info length = iod len
-			/*for (i =0; i< gf_list_count(program->desc); i++)
-			{
-				// program descriptors
-				// IOD
-			}*/
-			gf_bs_write_int(bs,	29, 8); // tag
-			len = iod->length + 2;
-			gf_bs_write_int(bs,	len, 8); // length
-			gf_bs_write_int(bs,	2, 8);  // Scope_of_IOD_label : 0x10 iod unique a l'intrieur de programme
-													// 0x11 iod unoque dans la flux ts
-			gf_bs_write_int(bs,	2, 8);  // IOD_label
+			GF_BitStream *bs_iod;
+			char *iod_data;
+			u32 iod_data_len;
 
-			gf_bs_write_data(bs, iod->data, iod->length);  // IOD
-#endif
+			bs_iod = gf_bs_new(NULL,0,GF_BITSTREAM_WRITE);
+			gf_odf_write_descriptor(bs_iod, stream->program->iod);
+			gf_bs_get_content(bs_iod, &iod_data, &iod_data_len); 
+			gf_bs_del(bs_iod);
+
+			len = iod_data_len + 4;
+			gf_bs_write_int(bs,	len, 12); // program info length 
+			
+			gf_bs_write_int(bs,	GF_M2TS_MPEG4_IOD_DESCRIPTOR, 8);
+			len = iod_data_len + 2;
+			gf_bs_write_int(bs,	len, 8);
+			
+			/* Scope_of_IOD_label : 
+				0x10 iod unique a l'intrieur de programme
+				0x11 iod unoque dans le flux ts */
+			gf_bs_write_int(bs,	2, 8);  
+
+			gf_bs_write_int(bs,	2, 8);  // IOD_label
+			
+			gf_bs_write_data(bs, iod_data, iod_data_len);
+			free(iod_data);
 		}	
 		es = stream->program->streams;
 		while (es) {
@@ -439,13 +591,11 @@ Bool m2ts_stream_process_pmt(M2TS_Mux *muxer, M2TS_Mux_Stream *stream)
 			gf_bs_write_int(bs,	0xF, 4); // reserved
 			
 			/* Second Loop Descriptor */
-			if (0 /*iod*/) {
-#if 0
+			if (stream->program->iod) {
 				gf_bs_write_int(bs,	4, 12); // ES info length = 4 :only SL Descriptor
-				gf_bs_write_int(bs,	30, 8); // tag
-				gf_bs_write_int(bs,	1, 8); // length
-				gf_bs_write_int(bs,	es->mpeg4_es_id, 16);  // mpeg4_esid
-#endif
+				gf_bs_write_int(bs,	GF_M2TS_MPEG4_SL_DESCRIPTOR, 8); 
+				gf_bs_write_int(bs,	2, 8); 
+				gf_bs_write_int(bs,	es->ifce->stream_id, 16);  // mpeg4_esid
 			} else {
 				gf_bs_write_int(bs,	0, 12);
 			}
@@ -462,106 +612,177 @@ Bool m2ts_stream_process_pmt(M2TS_Mux *muxer, M2TS_Mux_Stream *stream)
 	return 1;
 }
 
-
 Bool m2ts_stream_process_stream(M2TS_Mux *muxer, M2TS_Mux_Stream *stream)
 {
-	if (stream->pck_offset == stream->pck.data_len) {
-		if (stream->ifce->caps & GF_ESI_AU_PULL_CAP) {
-			if (stream->pck_offset) stream->ifce->input_ctrl(stream->ifce, GF_ESI_INPUT_DATA_RELEASE, NULL);
+	if (stream->mpeg2_stream_type==GF_M2TS_SYSTEMS_MPEG4_SECTIONS) {
+		/*section not completely sent yet*/
+		if (stream->current_section) return 1;
+	} 
+	else if (stream->pck_offset < stream->pck.data_len) {
+		/*PES packet not completely sent yet*/
+		return 1;
+	}
 
-			stream->pck_offset = 0;
-			stream->pck.data_len = 0;
+	/*PULL mode*/
+	if (stream->ifce->caps & GF_ESI_AU_PULL_CAP) {
+		if (stream->pck.data_len) {
+			/*discard packet data if we use SL over PES*/
+			if (stream->mpeg2_stream_type==GF_M2TS_SYSTEMS_MPEG4_PES) free(stream->pck.data);
+			/*release data*/
+			stream->ifce->input_ctrl(stream->ifce, GF_ESI_INPUT_DATA_RELEASE, NULL);
+		}
+		stream->pck_offset = 0;
+		stream->pck.data_len = 0;
 
-			/*EOF*/
-			if (stream->ifce->caps & GF_ESI_STREAM_IS_OVER) return 0;
-			stream->ifce->input_ctrl(stream->ifce, GF_ESI_INPUT_DATA_PULL, &stream->pck);
-		} else {
-			M2TS_Packet *pck;
-			/*flush input pipe*/
-			stream->ifce->input_ctrl(stream->ifce, GF_ESI_INPUT_DATA_FLUSH, NULL);
-			gf_mx_p(stream->mx);
-			/*discard first packet*/
-			if (stream->pck_offset) {
-				assert(stream->pck_first);
-				pck = stream->pck_first;
-				stream->pck_first = pck->next;
-				free(pck->data);
-				free(pck);
-			}
-			stream->pck_offset = 0;
-			stream->pck.data_len = 0;
-
-			/*fill pck*/
+		/*EOS*/
+		if (stream->ifce->caps & GF_ESI_STREAM_IS_OVER) return 0;
+		stream->ifce->input_ctrl(stream->ifce, GF_ESI_INPUT_DATA_PULL, &stream->pck);
+	} else {
+		M2TS_Packet *pck;
+		/*flush input pipe*/
+		stream->ifce->input_ctrl(stream->ifce, GF_ESI_INPUT_DATA_FLUSH, NULL);
+		gf_mx_p(stream->mx);
+		/*discard first packet*/
+		if (stream->pck_offset) {
+			assert(stream->pck_first);
 			pck = stream->pck_first;
-			if (!pck) {
-				gf_mx_v(stream->mx);
-				return 0;
-			}
-			stream->pck.cts = pck->cts;
-			stream->pck.data = pck->data;
-			stream->pck.data_len = pck->data_len;
-			stream->pck.dts = pck->dts;
-			stream->pck.flags = pck->flags;
+			stream->pck_first = pck->next;
+			free(pck->data);
+			free(pck);
+		}
+		stream->pck_offset = 0;
+		stream->pck.data_len = 0;
+
+		/*fill pck*/
+		pck = stream->pck_first;
+		if (!pck) {
 			gf_mx_v(stream->mx);
+			return 0;
 		}
-		/*!! watchout !!*/
-		if (stream->ts_scale) {
-			stream->pck.cts = (u64) (stream->ts_scale * (s64) stream->pck.cts);
-			stream->pck.dts = (u64) (stream->ts_scale * (s64) stream->pck.dts);
-		}
+		stream->pck.cts = pck->cts;
+		stream->pck.data = pck->data;
+		stream->pck.data_len = pck->data_len;
+		stream->pck.dts = pck->dts;
+		stream->pck.flags = pck->flags;
+		gf_mx_v(stream->mx);
+	}
+	if (!(stream->pck.flags & GF_ESI_DATA_HAS_DTS))
+		stream->pck.dts = stream->pck.cts;
 
-		/*initializing the PCR*/
-		if (!stream->program->pcr_init) {
-			if (stream==stream->program->pcr) {
-				stream->program->pcr_init_ts_time = muxer->time;
-				stream->program->pcr_init_time = stream->pck.dts;
-				stream->program->pcr_init = 1;
-			} else {
-				/*don't send until PCR is initialized*/
-				return 0;
-			}
-		}
-		/*move to current time in TS unit*/
-		stream->time = stream->program->pcr_init_ts_time;
-		m2ts_time_inc(&stream->time, (u32) (stream->pck.dts - stream->program->pcr_init_time), 90000);
+	/*!! watchout !!*/
+	if (stream->ts_scale) {
+		stream->pck.cts = (u64) (stream->ts_scale * (s64) stream->pck.cts);
+		stream->pck.dts = (u64) (stream->ts_scale * (s64) stream->pck.dts);
+	}
 
-		/*compute bitrate if needed*/
-		if (!stream->bit_rate) {
-			if (!stream->last_br_time) {
-				stream->last_br_time = stream->pck.dts + 1;
-				stream->bytes_since_last_time = stream->pck.data_len;
+	/*SL-encapsultaion*/
+	switch (stream->mpeg2_stream_type) {
+	case GF_M2TS_SYSTEMS_MPEG4_SECTIONS:
+		/*update SL config*/
+		stream->sl_header.accessUnitStartFlag = (stream->pck.flags & GF_ESI_DATA_AU_START) ? 1 : 0;
+		stream->sl_header.accessUnitEndFlag = (stream->pck.flags & GF_ESI_DATA_AU_END) ? 1 : 0;
+		stream->sl_header.accessUnitLength += stream->pck.data_len;
+		stream->sl_header.randomAccessPointFlag = (stream->pck.flags & GF_ESI_DATA_AU_RAP) ? 1: 0;
+		stream->sl_header.compositionTimeStampFlag = (stream->pck.flags & GF_ESI_DATA_HAS_CTS) ? 1 : 0;
+		stream->sl_header.compositionTimeStamp = stream->pck.cts;
+		stream->sl_header.decodingTimeStampFlag = (stream->pck.flags & GF_ESI_DATA_HAS_DTS) ? 1: 0;
+		stream->sl_header.decodingTimeStamp = stream->pck.dts;
+
+		m2ts_mux_table_update_mpeg4(stream, stream->table_id, muxer->ts_id, stream->pck.data, stream->pck.data_len, 1, 0, 0);
+		break;
+	case GF_M2TS_SYSTEMS_MPEG4_PES:
+	{
+		char *src_data;
+		u32 src_data_len;
+
+		/*update SL config*/
+		stream->sl_header.accessUnitStartFlag = (stream->pck.flags & GF_ESI_DATA_AU_START) ? 1 : 0;
+		stream->sl_header.accessUnitEndFlag = (stream->pck.flags & GF_ESI_DATA_AU_END) ? 1 : 0;
+		stream->sl_header.accessUnitLength += stream->pck.data_len;
+		stream->sl_header.randomAccessPointFlag = (stream->pck.flags & GF_ESI_DATA_AU_RAP) ? 1: 0;
+		stream->sl_header.compositionTimeStampFlag = (stream->pck.flags & GF_ESI_DATA_HAS_CTS) ? 1 : 0;
+		stream->sl_header.compositionTimeStamp = stream->pck.cts;
+		stream->sl_header.decodingTimeStampFlag = (stream->pck.flags & GF_ESI_DATA_HAS_DTS) ? 1: 0;
+		stream->sl_header.decodingTimeStamp = stream->pck.dts;
+
+		src_data = stream->pck.data;
+		src_data_len = stream->pck.data_len;
+		stream->pck.data_len = 0;
+		stream->pck.data = NULL;
+
+		gf_sl_packetize(&stream->ifce->sl_config, &stream->sl_header, src_data, src_data_len, &stream->pck.data, &stream->pck.data_len);
+
+		/*discard src data*/
+		if (!(stream->ifce->caps & GF_ESI_AU_PULL_CAP)) {
+			free(src_data);
+			stream->pck_first->data = stream->pck.data;
+			stream->pck_first->data_len = stream->pck.data_len;
+		}
+	}
+		break;
+	}
+
+
+	/*initializing the PCR*/
+	if (!stream->program->pcr_init) {
+		if (stream==stream->program->pcr) {
+			stream->program->pcr_init_ts_time = muxer->time;
+			stream->program->pcr_init_time = stream->pck.dts;
+			stream->program->pcr_init = 1;
+		} else {
+			/*don't send until PCR is initialized*/
+			return 0;
+		}
+	}
+
+	/*move to current time in TS unit*/
+	stream->time = stream->program->pcr_init_ts_time;
+	m2ts_time_inc(&stream->time, (u32) (stream->pck.dts - stream->program->pcr_init_time), 90000);
+
+	/*compute bitrate if needed*/
+	if (!stream->bit_rate) {
+		if (!stream->last_br_time) {
+			stream->last_br_time = stream->pck.dts + 1;
+			stream->bytes_since_last_time = stream->pck.data_len;
+		} else {
+			if (stream->pck.dts - stream->last_br_time - 1 >= 90000) {
+				u64 r = 8*stream->bytes_since_last_time;
+				r*=90000;
+				stream->bit_rate = (u32) (r / (stream->pck.dts - stream->last_br_time - 1));
+				stream->program->mux->needs_reconfig = 1;
 			} else {
-				if (stream->pck.dts - stream->last_br_time - 1 >= 90000) {
-					u64 r = 8*stream->bytes_since_last_time;
-					r*=90000;
-					stream->bit_rate = (u32) (r / (stream->pck.dts - stream->last_br_time - 1));
-					stream->program->mux->needs_reconfig = 1;
-				} else {
-					stream->bytes_since_last_time += stream->pck.data_len;
-				}
+				stream->bytes_since_last_time += stream->pck.data_len;
 			}
 		}
 	}
 	return 1;
 }
 
-#define PES_HEADER_LENGTH 19
-u32 m2ts_stream_add_pes_header(GF_BitStream *bs, M2TS_Mux_Stream *stream, 
-								Bool use_dts, u64 dts, 
-								Bool use_pts, u64 cts, 
-								u32 au_length)
+static u32 m2ts_stream_get_pes_header_length(M2TS_Mux_Stream *stream)
+{
+	u32 hdr_len;
+	/*not the AU start*/
+	if (stream->pck_offset || !(stream->pck.flags & GF_ESI_DATA_AU_START) ) return 0;
+	hdr_len = 9;
+	if (stream->pck.flags & GF_ESI_DATA_HAS_CTS) hdr_len += 5;
+	if (stream->pck.flags & GF_ESI_DATA_HAS_DTS) hdr_len += 5;
+	return hdr_len;
+}
+
+u32 m2ts_stream_add_pes_header(GF_BitStream *bs, M2TS_Mux_Stream *stream)
 {
 	u32 pes_len;
-	u32 pes_header_data_length;
-
-	pes_header_data_length = 10;
+	Bool use_pts, use_dts;
 
 	gf_bs_write_int(bs,	0x1, 24);//packet start code
 	gf_bs_write_u8(bs,	stream->mpeg2_stream_id);// stream id 
 
-	pes_len = au_length + 3; // 3 = header size
-	if (use_dts) pes_len += 5;
+	use_pts = (stream->pck.flags & GF_ESI_DATA_HAS_CTS) ? 1 : 0;
+	use_dts = (stream->pck.flags & GF_ESI_DATA_HAS_DTS) ? 1 : 0;
+
+	pes_len = stream->pck.data_len + 3; // 3 = header size
 	if (use_pts) pes_len += 5;
+	if (use_dts) pes_len += 5;
 	gf_bs_write_int(bs, pes_len, 16); // pes packet length
 	
 	gf_bs_write_int(bs, 0x2, 2); // reserved
@@ -575,18 +796,18 @@ u32 m2ts_stream_add_pes_header(GF_BitStream *bs, M2TS_Mux_Stream *stream,
 	gf_bs_write_int(bs, use_dts, 1);
 	gf_bs_write_int(bs, 0x0, 6); //6 flags = 0 (ESCR, ES_rate, DSM_trick, additional_copy, PES_CRC, PES_extension)
 
-	gf_bs_write_int(bs, pes_header_data_length, 8);
+	gf_bs_write_int(bs, use_dts*5+use_pts*5, 8);
 
 	if (use_pts) {
 		u64 t;
 		gf_bs_write_int(bs, use_dts ? 0x3 : 0x2, 4); // reserved '0011' || '0010'
-		t = ((cts >> 30) & 0x7);
+		t = ((stream->pck.cts >> 30) & 0x7);
 		gf_bs_write_long_int(bs, t, 3);
 		gf_bs_write_int(bs, 1, 1); // marker bit
-		t = ((cts >> 15) & 0x7fff);
+		t = ((stream->pck.cts >> 15) & 0x7fff);
 		gf_bs_write_long_int(bs, t, 15);
 		gf_bs_write_int(bs, 1, 1); // marker bit
-		t = cts & 0x7fff;
+		t = stream->pck.cts & 0x7fff;
 		gf_bs_write_long_int(bs, t, 15);
 		gf_bs_write_int(bs, 1, 1); // marker bit
 	}
@@ -594,13 +815,13 @@ u32 m2ts_stream_add_pes_header(GF_BitStream *bs, M2TS_Mux_Stream *stream,
 	if (use_dts) {
 		u64 t;
 		gf_bs_write_int(bs, 0x1, 4); // reserved '0001'
-		t = ((dts >> 30) & 0x7);
+		t = ((stream->pck.dts >> 30) & 0x7);
 		gf_bs_write_long_int(bs, t, 3);
 		gf_bs_write_int(bs, 1, 1); // marker bit
-		t = ((dts >> 15) & 0x7fff);
+		t = ((stream->pck.dts >> 15) & 0x7fff);
 		gf_bs_write_long_int(bs, t, 15);
 		gf_bs_write_int(bs, 1, 1); // marker bit
-		t = dts & 0x7fff;
+		t = stream->pck.dts & 0x7fff;
 		gf_bs_write_long_int(bs, t, 15);
 		gf_bs_write_int(bs, 1, 1); // marker bit
 	}
@@ -610,21 +831,20 @@ u32 m2ts_stream_add_pes_header(GF_BitStream *bs, M2TS_Mux_Stream *stream,
 void m2ts_mux_pes_get_next_packet(M2TS_Mux_Stream *stream, u8 *packet)
 {
 	GF_BitStream *bs;
-	Bool au_start, is_rap, needs_pcr;
-	u32 remain, adaptation_field_control, payload_length, padding_length;
+	Bool is_rap, needs_pcr;
+	u32 remain, adaptation_field_control, payload_length, padding_length, hdr_len;
 
 	assert(stream->pid);
 	bs = gf_bs_new(packet, 188, GF_BITSTREAM_WRITE);
 
-	au_start = 0;
-	if (!stream->pck_offset && (stream->pck.flags & GF_ESI_DATA_AU_START) ) au_start = 1;
+	hdr_len = m2ts_stream_get_pes_header_length(stream);
 	remain = stream->pck.data_len - stream->pck_offset;
 
+	needs_pcr = (hdr_len && (stream==stream->program->pcr) ) ? 1 : 0;
 	adaptation_field_control = M2TS_ADAPTATION_NONE;
-	payload_length = 184 - PES_HEADER_LENGTH*au_start;
-	needs_pcr = (au_start && (stream==stream->program->pcr) ) ? 1 : 0;
-
+	payload_length = 184 - hdr_len;
 	padding_length = 0;
+
 	if (needs_pcr) {
 		adaptation_field_control = M2TS_ADAPTATION_AND_PAYLOAD;
 		/*AF headers + PCR*/
@@ -643,7 +863,7 @@ void m2ts_mux_pes_get_next_packet(M2TS_Mux_Stream *stream, u8 *packet)
 
 	gf_bs_write_int(bs,	0x47, 8); // sync byte
 	gf_bs_write_int(bs,	0, 1);    // error indicator
-	gf_bs_write_int(bs,	au_start, 1); // start ind
+	gf_bs_write_int(bs,	hdr_len ? 1 : 0, 1); // start ind
 	gf_bs_write_int(bs,	0, 1);	  // transport priority
 	gf_bs_write_int(bs,	stream->pid, 13); // pid
 	gf_bs_write_int(bs,	0, 2);    // scrambling
@@ -652,8 +872,7 @@ void m2ts_mux_pes_get_next_packet(M2TS_Mux_Stream *stream, u8 *packet)
 	if (stream->continuity_counter < 15) stream->continuity_counter++;
 	else stream->continuity_counter=0;
 
-	if (au_start && (stream->pck.flags & GF_ESI_DATA_AU_RAP) ) is_rap = 1;
-	else is_rap = 0;
+	is_rap = (hdr_len && (stream->pck.flags & GF_ESI_DATA_AU_RAP) ) ? 1 : 0;
 
 	if (adaptation_field_control != M2TS_ADAPTATION_NONE) {
 		/*FIXME - WE NEED A REAL PCR, NOT THE DTS*/
@@ -661,16 +880,13 @@ void m2ts_mux_pes_get_next_packet(M2TS_Mux_Stream *stream, u8 *packet)
 	}
 
 	/*FIXME - we need proper packetization here in case we're not fed with full AUs*/
-	if (au_start) {
-		m2ts_stream_add_pes_header(bs, stream, 
-							(stream->ifce->caps & GF_ESI_SIGNAL_DTS) ? 1 : 0, stream->pck.dts,
-							1, stream->pck.cts, stream->pck.data_len);
-	}
+	if (hdr_len) m2ts_stream_add_pes_header(bs, stream);
 
 	gf_bs_del(bs);
+
 	memcpy(packet+188-payload_length, stream->pck.data + stream->pck_offset, payload_length);
 	stream->pck_offset += payload_length;
-
+	
 	assert(stream->pck_offset <= stream->pck.data_len);
 	m2ts_time_inc(&stream->time, payload_length, stream->bit_rate);
 }
@@ -734,8 +950,11 @@ M2TS_Mux_Stream *m2ts_program_stream_add(M2TS_Mux_Program *program, struct __ele
 	}
 	if (program->pmt) program->pmt->table_needs_update = 1;
 	stream->bit_rate = ifce->bit_rate;
+
 	switch (ifce->stream_type) {
 	case GF_STREAM_VISUAL:
+		/*just pick first valid stream_id in visual range*/
+		stream->mpeg2_stream_id = 0xE0;
 		switch (ifce->object_type_indication) {
 		case 0x20:
 			stream->mpeg2_stream_type = GF_M2TS_VIDEO_MPEG4;
@@ -754,11 +973,15 @@ M2TS_Mux_Stream *m2ts_program_stream_add(M2TS_Mux_Program *program, struct __ele
 		case 0x65:
 			stream->mpeg2_stream_type = GF_M2TS_VIDEO_MPEG2;
 			break;
+		/*JPEG/PNG carried in MPEG-4 PES*/
+		case 0x6C:
+		case 0x6D:
+			stream->mpeg2_stream_type = GF_M2TS_SYSTEMS_MPEG4_PES;
+			stream->mpeg2_stream_id = 0xFA;
+			break;
 		default:
 			break;
 		}
-		/*just pick first valid stream_id in visual range*/
-		stream->mpeg2_stream_id = 0xE0;
 		break;
 	case GF_STREAM_AUDIO:
 		switch (ifce->object_type_indication) {
@@ -775,10 +998,25 @@ M2TS_Mux_Stream *m2ts_program_stream_add(M2TS_Mux_Program *program, struct __ele
 		/*just pick first valid stream_id in audio range*/
 		stream->mpeg2_stream_id = 0xC0;
 		break;
+	case GF_STREAM_SCENE:
+	case GF_STREAM_OD:
+		stream->mpeg2_stream_type = GF_M2TS_SYSTEMS_MPEG4_SECTIONS;
+		stream->mpeg2_stream_id = 0xFA;
+		stream->table_id = (ifce->stream_type==GF_STREAM_OD) ? GF_M2TS_TABLE_ID_MPEG4_OD : GF_M2TS_TABLE_ID_MPEG4_BIFS;
+		break;
 	}
+
+	/*override signaling for all streams except BIFS/OD, to use MPEG-4 PES*/
+	if (program->mux->mpeg4_signaling) {
+		if (stream->mpeg2_stream_type != GF_M2TS_SYSTEMS_MPEG4_SECTIONS) {
+			stream->mpeg2_stream_type = GF_M2TS_SYSTEMS_MPEG4_PES;
+			stream->mpeg2_stream_id = 0xFA;/*ISO/IEC14496-1_SL-packetized_stream*/
+		}
+	}
+
 	stream->ifce->output_ctrl = m2ts_output_ctrl;
 	stream->ifce->output_udta = stream;
-	stream->mx = gf_mx_new();
+	stream->mx = gf_mx_new("M2TS PID");
 	if (ifce->timescale != 90000) stream->ts_scale = 90000.0 / ifce->timescale;
 	return stream;
 }
@@ -812,7 +1050,7 @@ M2TS_Mux *m2ts_mux_new(u32 mux_rate, Bool real_time)
 	GF_BitStream *bs;
 	M2TS_Mux *muxer;
 	GF_SAFEALLOC(muxer, M2TS_Mux);
-	muxer->pat = m2ts_stream_new(0); /* 0 = PAT_PID */
+	muxer->pat = m2ts_stream_new(GF_M2TS_PID_PAT);
 	muxer->pat->process = m2ts_stream_process_pat;
 	muxer->pat->refresh_rate_ms = M2TS_PSI_REFRESH_RATE;
 	muxer->real_time = real_time;
@@ -1009,17 +1247,18 @@ send_pck:
 		if (nb_streams && (nb_streams==nb_streams_done)) {
 			*status = GF_M2TS_STATE_EOS;
 		} else {
-			/* padding packets ?? */
-			if (muxer->fixed_rate) {
-				GF_LOG(GF_LOG_DEBUG, GF_LOG_CONTAINER, ("[MPEG2-TS Muxer] Inserting empty packet at %d:%d\n", time.sec, time.nanosec));
-				ret = muxer->null_pck;
-			}
-			/*we still need to increase the mux time, even though we're not fixed-rate*/
-			else {
-				m2ts_time_inc(&muxer->time, 1504/*188*8*/, muxer->bit_rate);
-			}
 			*status = GF_M2TS_STATE_PADDING;
 		}
+		/* padding packets ?? */
+		if (muxer->fixed_rate) {
+			GF_LOG(GF_LOG_DEBUG, GF_LOG_CONTAINER, ("[MPEG2-TS Muxer] Inserting empty packet at %d:%d\n", time.sec, time.nanosec));
+			ret = muxer->null_pck;
+			muxer->tot_pad_sent++;
+		}
+		/*we still need to increase the mux time, even though we're not fixed-rate*/
+		else {
+			m2ts_time_inc(&muxer->time, 1504/*188*8*/, muxer->bit_rate);
+		}
 	} else {
 		if (stream_to_process->tables) {
 			m2ts_mux_table_get_next_packet(stream_to_process, muxer->dst_pck);
@@ -1036,14 +1275,15 @@ send_pck:
 		/*increment time*/
 		m2ts_time_inc(&muxer->time, 1504/*188*8*/, muxer->bit_rate);
 
-		muxer->pck_sent++;
-		if (now - muxer->last_br_time > 500) {
-			u64 size = 8*188*muxer->pck_sent*1000;
-			muxer->avg_br = (u32) (size/(now - muxer->last_br_time));
-			muxer->last_br_time = now;
-			muxer->pck_sent=0;
+		if (muxer->real_time) {
+			muxer->pck_sent++;
+			if (now - muxer->last_br_time > 500) {
+				u64 size = 8*188*muxer->pck_sent*1000;
+				muxer->avg_br = (u32) (size/(now - muxer->last_br_time));
+				muxer->last_br_time = now;
+				muxer->pck_sent=0;
+			}
 		}
-
 	}
 	return ret;
 }
@@ -1053,6 +1293,8 @@ typedef struct
 	GF_ISOFile *mp4;
 	u32 track, sample_number, sample_count;
 	GF_ISOSample *sample;
+	/*refresh rate for images*/
+	u32 refresh_rate_ms, nb_repeat_last;
 } GF_ESIMP4;
 
 static GF_Err mp4_input_ctrl(GF_ESInterface *ifce, u32 act_type, void *param)
@@ -1070,11 +1312,20 @@ static GF_Err mp4_input_ctrl(GF_ESInterface *ifce, u32 act_type, void *param)
 			priv->sample = gf_isom_get_sample(priv->mp4, priv->track, priv->sample_number+1, NULL);
 		}
 		if (!priv->sample) return GF_IO_ERR;
-		pck->dts = priv->sample->DTS;
-		pck->cts = priv->sample->DTS + priv->sample->CTS_Offset;
 		pck->data_len = priv->sample->dataLength;
 		pck->data = priv->sample->data;
-		pck->flags = GF_ESI_DATA_AU_START | GF_ESI_DATA_AU_END;
+		pck->flags = GF_ESI_DATA_AU_START | GF_ESI_DATA_AU_END | GF_ESI_DATA_HAS_CTS;
+		pck->cts = priv->sample->DTS;
+		
+		if (priv->nb_repeat_last) {
+			pck->cts += priv->nb_repeat_last*ifce->timescale * priv->refresh_rate_ms / 1000;
+		}
+
+		if (priv->sample->CTS_Offset) {
+			pck->dts = pck->cts;
+			pck->cts += priv->sample->CTS_Offset;
+			pck->flags |= GF_ESI_DATA_HAS_DTS;
+		}
 		if (priv->sample->IsRAP) pck->flags |= GF_ESI_DATA_AU_RAP;
 	}
 		return GF_OK;
@@ -1082,8 +1333,14 @@ static GF_Err mp4_input_ctrl(GF_ESInterface *ifce, u32 act_type, void *param)
 		if (priv->sample) {
 			gf_isom_sample_del(&priv->sample);
 			priv->sample_number++;
-			if (priv->sample_number==priv->sample_count) 
-				ifce->caps |= GF_ESI_STREAM_IS_OVER;
+			if (priv->sample_number==priv->sample_count) {
+				if (priv->refresh_rate_ms) {
+					priv->nb_repeat_last++;
+					priv->sample_number--;
+				} else {
+					ifce->caps |= GF_ESI_STREAM_IS_OVER;
+				}
+			}
 		}
 		return GF_OK;
 	case GF_ESI_INPUT_DESTROY:
@@ -1126,6 +1383,13 @@ static void fill_isom_es_ifce(GF_ESInterface *ifce, GF_ISOFile *mp4, u32 track_n
 
 	if (gf_isom_has_time_offset(mp4, track_num)) ifce->caps |= GF_ESI_SIGNAL_DTS;
 
+	/*turn on image repeat*/
+	switch (ifce->object_type_indication) {
+	case 0x6C:
+	case 0x6D:
+		//priv->refresh_rate_ms = 500;
+		break;
+	}
 	ifce->bit_rate = (u32) avg_rate;
 	ifce->duration = (Double) (s64) gf_isom_get_media_duration(mp4, track_num);
 	ifce->duration /= ifce->timescale;
@@ -1280,6 +1544,7 @@ typedef struct
 	GF_ISOFile *mp4;
 	u32 nb_streams, pcr_idx;
 	GF_ESInterface streams[40];
+	GF_Descriptor *iod;
 } M2TSProgram;
 
 Bool open_program(M2TSProgram *prog, const char *src)
@@ -1292,15 +1557,41 @@ Bool open_program(M2TSProgram *prog, const char *src)
 
 	/* Open ISO file  */
 	if (gf_isom_probe_file(src)) {
+		u32 nb_tracks;
+		u32 first_audio = 0;
 		prog->mp4 = gf_isom_open(src, GF_ISOM_OPEN_READ, 0);
-		prog->nb_streams = gf_isom_get_track_count(prog->mp4); 
-		for (i=0; i<prog->nb_streams; i++) {
+		prog->nb_streams = 0;
+		/*on MPEG-2 TS, carry 3GPP timed text as MPEG-4 Part17*/
+		gf_isom_text_set_streaming_mode(prog->mp4, 1);
+		nb_tracks = gf_isom_get_track_count(prog->mp4); 
+		for (i=0; i<nb_tracks; i++) {
+			if (gf_isom_get_media_type(prog->mp4, i+1) == GF_ISOM_MEDIA_HINT) 
+				continue; 
 			fill_isom_es_ifce(&prog->streams[i], prog->mp4, i+1);
+			prog->nb_streams++;
 			/*get first visual stream as PCR*/
-			if (!prog->pcr_idx && (gf_isom_get_media_type(prog->mp4, i+1) == GF_ISOM_MEDIA_VISUAL) && (gf_isom_get_sample_count(prog->mp4, i+1)>1) ) {
+			if (!prog->pcr_idx && 
+				(gf_isom_get_media_type(prog->mp4, i+1) == GF_ISOM_MEDIA_VISUAL) && 
+				(gf_isom_get_sample_count(prog->mp4, i+1)>1) ) {
 				prog->pcr_idx = i+1;
 			}
+			if (!first_audio && (gf_isom_get_media_type(prog->mp4, i+1) == GF_ISOM_MEDIA_AUDIO) ) {
+				first_audio = i+1;
+			}
+			prog->streams[i].sl_config.timestampResolution = 90000; //prog->streams[i].timescale;
+			prog->streams[i].sl_config.useRandomAccessPointFlag = 1;
+			prog->streams[i].sl_config.useAccessUnitStartFlag = 1;
+			prog->streams[i].sl_config.useAccessUnitEndFlag = 1;
+			prog->streams[i].sl_config.useTimestampsFlag = 1;
+			prog->streams[i].sl_config.timestampLength = 33;
+			prog->streams[i].sl_config.tag = GF_ODF_SLC_TAG;
+			gf_isom_set_extraction_slc(prog->mp4, i+1, 1, &prog->streams[i].sl_config);
 		}
+		/* WARNING: the returned IOD may be different from the one actually in the file
+		 because it rewrites the SL config according to SL extraction settings. */
+		prog->iod = gf_isom_get_root_od(prog->mp4);
+		/*if no visual PCR found, use first audio*/
+		if (!prog->pcr_idx) prog->pcr_idx = first_audio;
 		if (prog->pcr_idx) prog->pcr_idx-=1;
 		return 1;
 	}
@@ -1351,12 +1642,12 @@ int main(int argc, char **argv)
 {
 	const char *ts_pck;
 	GF_Err e;
-	u32 res;
-	Bool real_time=0;
+	u32 res, run_time;
+	Bool real_time, mpeg4_signaling;
 	M2TS_Mux *muxer;
 	u32 i, j, mux_rate, nb_progs, cur_pid;
 	char *ts_out = NULL;
-	FILE *ts_file;
+FILE *ts_file;
 	GF_Socket *ts_udp;
 	GF_RTPChannel *ts_rtp;
 	GF_RTSPTransport tr;
@@ -1365,7 +1656,7 @@ int main(int argc, char **argv)
 	u32 output_type;
 	M2TSProgram progs[MAX_MUX_SRC_PROG];
 
-
+	real_time=0;	
 	output_type = 0;
 	ts_file = NULL;
 	ts_udp = NULL;
@@ -1373,19 +1664,26 @@ int main(int argc, char **argv)
 	ts_out = NULL;
 	nb_progs = 0;
 	mux_rate = 0;
+	run_time = 0;
+	mpeg4_signaling = 0;
 	for (i=1; i<argc; i++) {
 		char *arg = argv[i];
 		if (arg[0]=='-') {
-			if (!strnicmp(arg, "-rate=", 6)) mux_rate = 1024*atoi(arg+6);
-			else if (!strnicmp(arg, "-prog=", 6)) {
+			if (!strnicmp(arg, "-rate=", 6)) {
+				mux_rate = 1024*atoi(arg+6);
+			} else if (!strnicmp(arg, "-prog=", 6)) {
 				memset(&progs[nb_progs], 0, sizeof(M2TSProgram));
 				res = open_program(&progs[nb_progs], arg+6);
 				if (res) {
 					nb_progs++;
 					if (res==2) real_time=1;
 				}
-			}
-		}
+			} else if (!strnicmp(arg, "-mpeg4", 6)) {
+				mpeg4_signaling = 1;
+			} else if (!strnicmp(arg, "-time=", 6)) {
+				run_time = atoi(arg+6);
+			} 
+		} 
 		/*output*/
 		else {
 			if (!strnicmp(arg, "rtp://", 6) || !strnicmp(arg, "udp://", 6)) {
@@ -1415,6 +1713,11 @@ int main(int argc, char **argv)
 		usage();
 		return 0;
 	}
+	
+	if (run_time && !mux_rate) {
+		fprintf(stdout, "Cannot specify TS run time for VBR multiplex - disabling run time\n");
+		run_time = 0;
+	}
 
 	gf_log_set_level(GF_LOG_DEBUG);
 	gf_log_set_tools(GF_LOG_RTP);
@@ -1422,9 +1725,8 @@ int main(int argc, char **argv)
 
 	gf_sys_init();
 
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_RTP, ("What a FGucking test!!\n"));
-
 	muxer = m2ts_mux_new(mux_rate, real_time);
+	muxer->mpeg4_signaling = mpeg4_signaling;
 	/* Open mpeg2ts file*/
 	switch(output_type) {
 	case 0:
@@ -1439,7 +1741,7 @@ int main(int argc, char **argv)
 		if (gf_sk_is_multicast_address((char *)ts_out)) {
 			e = gf_sk_setup_multicast(ts_udp, (char *)ts_out, port, 0, 0, NULL);
 		} else {
-			e = gf_sk_bind(ts_udp, port, (char *)ts_out, port, GF_SOCK_REUSE_PORT);
+			e = gf_sk_bind(ts_udp, NULL, port, (char *)ts_out, port, GF_SOCK_REUSE_PORT);
 		}
 		if (e) {
 			fprintf(stdout, "Error inhitializing UDP socket: %s\n", gf_error_to_string(e));
@@ -1485,6 +1787,7 @@ int main(int argc, char **argv)
 	cur_pid = 100;
 	for (i=0; i<nb_progs; i++) {
 		M2TS_Mux_Program *program = m2ts_mux_program_add(muxer, i+1, cur_pid);
+		if (muxer->mpeg4_signaling) program->iod = progs[i].iod;
 		for (j=0; j<progs[i].nb_streams; j++) {
 			m2ts_program_stream_add(program, &progs[i].streams[j], cur_pid+j+1, (progs[i].pcr_idx==j) ? 1 : 0);
 		}
@@ -1536,12 +1839,17 @@ int main(int argc, char **argv)
 				if (c == 'q') break;
 			}
 			fprintf(stdout, "M2TS: time %d - TS time %d - avg bitrate %d\r", gf_m2ts_get_sys_clock(muxer), gf_m2ts_get_ts_clock(muxer), muxer->avg_br);
-		} else {
-			if (status==GF_M2TS_STATE_EOS) 
+		} else if (run_time) {
+			if (gf_m2ts_get_ts_clock(muxer) > run_time) {
+				fprintf(stdout, "Stoping multiplex at %d ms (requested runtime %d ms)\n", gf_m2ts_get_ts_clock(muxer), run_time);
 				break;
+			}
+		} else if (status==GF_M2TS_STATE_EOS) {
+			break;
 		}
 	}
 
+
 exit:
 	if (ts_file) fclose(ts_file);
 	if (ts_udp) gf_sk_del(ts_udp);
@@ -1553,6 +1861,7 @@ exit:
 		for (j=0; j<progs[i].nb_streams; j++) {
 			progs[i].streams[j].input_ctrl(&progs[i].streams[j], GF_ESI_INPUT_DESTROY, NULL);
 		}
+		if (progs[i].iod) gf_odf_desc_del((GF_Descriptor*)progs[i].iod);
 		if (progs[i].mp4) gf_isom_close(progs[i].mp4);
 	}
 	gf_sys_close();
