@@ -24,7 +24,7 @@
 
 #include <gpac/isomedia.h>
 #include <gpac/ietf.h>
-#include <gpac/config.h>
+#include <gpac/config_file.h>
 #include <gpac/base_coding.h>
 
 GP_RTPPacketizer *gf_rtp_packetizer_create_and_init_from_file(GF_ISOFile *file, 
@@ -117,6 +117,8 @@ typedef struct __tag_rtp_stream
 
 	Double ts_scale, microsec_ts_scale;
 	Bool process_burst;
+	/*NALU size for H264/AVC*/
+	u32 avc_nalu_size;
 
 	struct __tag_rtp_session *session;
 } RTP_Stream;
@@ -243,7 +245,7 @@ static void burst_on_pck_done(void *cbk, GF_RTPHeader *header)
 /*
  * The RTP packetiser has added data to the current RTP packet
  */
-static void burst_on_pck_data(void *cbk, char *data, u32 data_size, Bool is_head) 
+static void on_pck_data(void *cbk, char *data, u32 data_size, Bool is_head) 
 {
 	RTP_Stream *rtp = cbk;
 	if (!data ||!data_size) return;
@@ -287,10 +289,12 @@ static void on_pck_done(void *cbk, GF_RTPHeader *header)
 	if (e) 
 		fprintf(stdout, "Error %s sending RTP packet\n", gf_error_to_string(e));
 	free(rtp->packet.payload);				
+
+	if (rtp->session->streamer->log_level == LOG_PACKET) 
+		fprintf(stdout, "  RTP SN %u - TS %u - M %u - Size %u\n", rtp->packet.header.SequenceNumber, rtp->packet.header.TimeStamp, rtp->packet.header.Marker, rtp->packet.payload_len + RTP_HEADER_SIZE);
+
 	rtp->packet.payload = NULL;
 	rtp->packet.payload_len = 0;
-	
-	if (rtp->session->streamer->log_level == LOG_PACKET) fprintf(stdout, "  RTP SN %u - TS %u - M %u - Size %u\n", rtp->packet.header.SequenceNumber, rtp->packet.header.TimeStamp, rtp->packet.header.Marker, rtp->packet.payload_len + RTP_HEADER_SIZE);
 }
 
 
@@ -302,12 +306,12 @@ GF_Err rtp_init_packetizer(RTP_Stream *rtp, char *dest_ip)
 
 	if (rtp->session->streamer->burst_mode) {
 		rtp->packetizer = gf_rtp_packetizer_create_and_init_from_file(rtp->session->mp4File, rtp->track, rtp,
-												burst_on_pck_new, burst_on_pck_done, NULL, burst_on_pck_data,
+												burst_on_pck_new, burst_on_pck_done, NULL, on_pck_data,
 												rtp->session->streamer->path_mtu, 0, 0, flags, 
 												rtp->session->streamer->payt, 0, 0, 0);
 	} else {
 		rtp->packetizer = gf_rtp_packetizer_create_and_init_from_file(rtp->session->mp4File, rtp->track, rtp,
-												on_pck_new, on_pck_done, NULL, burst_on_pck_data,
+												on_pck_new, on_pck_done, NULL, on_pck_data,
 												rtp->session->streamer->path_mtu, 0, 0, flags, 
 												rtp->session->streamer->payt, 0, 0, 0);
 	}
@@ -415,6 +419,7 @@ GF_Err rtp_setup_sdp(RTP_Session *session, char *dest_ip)
 				}
 			}
 			fprintf(sdp_out, "%s\n", sdpLine);
+			rtp->avc_nalu_size = avcc->nal_unit_size;
 			gf_odf_avc_cfg_del(avcc);
 		}
 		/*MPEG-4 decoder config*/
@@ -627,7 +632,33 @@ void burst_process_session(RTP_Session *session)
 
 		if (session->streamer->log_level >= LOG_AU) fprintf(stdout, "Sess %d - stream %d - Processing AU %d - DTS "LLD" - CTS "LLD"\n", session->id, rtp->track, rtp->current_au, rtp->packetizer->sl_header.decodingTimeStamp, rtp->packetizer->sl_header.compositionTimeStamp);
 
-		gf_rtp_builder_process(rtp->packetizer, rtp->au->data, rtp->au->dataLength, (u8) 1, rtp->au->dataLength, rtp->sample_duration, (u8) rtp->sample_desc_index);
+		/*unpack nal units*/
+		if (rtp->avc_nalu_size) {
+			u32 v, size;
+			u32 remain = rtp->au->dataLength;
+			char *ptr = rtp->au->data;
+
+			rtp->packetizer->sl_header.accessUnitStartFlag = 1;
+			rtp->packetizer->sl_header.accessUnitEndFlag = 0;
+			while (remain) {
+				size = 0;
+				v = rtp->avc_nalu_size;
+				while (v) {
+					size |= (u8) *ptr;
+					ptr++;
+					remain--;
+					v-=1;
+					if (v) size<<=8;
+				}
+				remain -= size;
+				rtp->packetizer->sl_header.accessUnitEndFlag = remain ? 0 : 1;
+				gf_rtp_builder_process(rtp->packetizer, ptr, size, (u8) !remain, rtp->au->dataLength, rtp->sample_duration, (u8) rtp->sample_desc_index );
+				ptr += size;
+				rtp->packetizer->sl_header.accessUnitStartFlag = 0;
+			}
+		} else {
+			gf_rtp_builder_process(rtp->packetizer, rtp->au->data, rtp->au->dataLength, (u8) 1, rtp->au->dataLength, rtp->sample_duration, (u8) rtp->sample_desc_index);
+		}
 			
 		rtp->next_ts = (u32)(rtp->packetizer->sl_header.decodingTimeStamp + rtp->sample_duration);
 		/*OK delete sample*/
@@ -727,7 +758,34 @@ void process_sessions(Streamer *streamer)
 	}
 
 	/*send packets*/
-	gf_rtp_builder_process(to_send->packetizer, to_send->au->data, to_send->au->dataLength, (u8) 1, to_send->au->dataLength, to_send->sample_duration, (u8) to_send->sample_desc_index);
+
+	/*unpack nal units*/
+	if (to_send->avc_nalu_size) {
+		u32 v, size;
+		u32 remain = to_send->au->dataLength;
+		char *ptr = to_send->au->data;
+
+		to_send->packetizer->sl_header.accessUnitStartFlag = 1;
+		to_send->packetizer->sl_header.accessUnitEndFlag = 0;
+		while (remain) {
+			size = 0;
+			v = to_send->avc_nalu_size;
+			while (v) {
+				size |= (u8) *ptr;
+				ptr++;
+				remain--;
+				v-=1;
+				if (v) size<<=8;
+			}
+			remain -= size;
+			to_send->packetizer->sl_header.accessUnitEndFlag = remain ? 0 : 1;
+			gf_rtp_builder_process(to_send->packetizer, ptr, size, (u8) !remain, to_send->au->dataLength, to_send->sample_duration, (u8) to_send->sample_desc_index );
+			ptr += size;
+			to_send->packetizer->sl_header.accessUnitStartFlag = 0;
+		}
+	} else {
+		gf_rtp_builder_process(to_send->packetizer, to_send->au->data, to_send->au->dataLength, (u8) 1, to_send->au->dataLength, to_send->sample_duration, (u8) to_send->sample_desc_index);
+	}
 	/*delete sample*/
 	gf_isom_sample_del(&to_send->au);
 
@@ -762,12 +820,12 @@ u16 check_next_port(Streamer *streamer, u16 first_port)
 
 GF_Err configuration(Streamer *streamer, char *cfg_file, char *src_file, char *ip_dest, u16 port, Bool loop, Bool force_mpeg4)
 {
-	GF_Err e;
+	GF_Err e = GF_OK;
 	RTP_Session *session, *last_sess;
 	u32 nb_sessions;
-	const char *opt;
+	const char *opt = NULL;
 	const char *dest_ip;
-	GF_Config *configFile;	
+	GF_Config *configFile = NULL;	
 	u32 i, j;					
 
 	if (cfg_file) {
@@ -794,6 +852,8 @@ GF_Err configuration(Streamer *streamer, char *cfg_file, char *src_file, char *i
 		fprintf(stdout, " Path MTU: %u bytes \n", streamer->path_mtu);
 
 		/*configure burst mode*/
+		opt = gf_cfg_get_key(configFile, "GLOBAL", "burst_mode");
+		streamer->burst_mode = (opt && !strcmp(opt, "yes")) ? 1 : 0;
 		if (streamer->burst_mode) {
 			opt = gf_cfg_get_key(configFile, "GLOBAL", "off_duration");
 			streamer->offDuration = opt ? atoi(opt) : 0;
