@@ -31,6 +31,7 @@
 #include <gpac/bitstream.h>
 #include <gpac/bifs.h>
 #include <gpac/thread.h>
+#include <gpac/internal/scenegraph_dev.h>
 
 typedef struct {
 	/*node this mask is for*/
@@ -70,7 +71,6 @@ typedef struct
 typedef struct 
 {
 	BIFSConfig config;
-	Bool UseName;
 	u16 ESID;
 } BIFSStreamInfo;
 
@@ -90,6 +90,8 @@ struct __tag_bifs_dec
 	/*active stream*/
 	BIFSStreamInfo *info;
 
+	Bool UseName;
+
 	GF_SceneGraph *scenegraph;
 	/*modified during conditional execution / proto parsing*/
 	GF_SceneGraph *current_graph;
@@ -100,16 +102,12 @@ struct __tag_bifs_dec
 	/*active QP*/
 	M_QuantizationParameter *ActiveQP;
 
-	/*QP 14 stuff: we need to store the last numb of fields in the last recieved Coord //field (!!!)*/
+	/*QP 14 stuff: we need to store the last numb of fields in the last received Coord //field (!!!)*/
 	
 	/*number of iten in the Coord field*/
 	u32 NumCoord;
 	Bool coord_stored, storing_coord;
 
-	/*active QP*/
-	M_QuantizationParameter *GlobalQP;
-
-
 	/*only set at SceneReplace during proto parsing, NULL otherwise*/
 	GF_Proto *pCurrentProto;
 
@@ -145,12 +143,13 @@ GF_Err gf_bifs_dec_command(GF_BifsDecoder * codec, GF_BitStream *bs);
 and added to the list*/
 GF_Err gf_bifs_dec_proto_list(GF_BifsDecoder * codec, GF_BitStream *bs, GF_List *proto_list);
 /*decodes field(s) of a node - exported for MultipleReplace*/
-GF_Err gf_bifs_dec_node_list(GF_BifsDecoder * codec, GF_BitStream *bs, GF_Node *node);
-GF_Err gf_bifs_dec_node_mask(GF_BifsDecoder * codec, GF_BitStream *bs, GF_Node *node);
+GF_Err gf_bifs_dec_node_list(GF_BifsDecoder * codec, GF_BitStream *bs, GF_Node *node, Bool is_proto);
+GF_Err gf_bifs_dec_node_mask(GF_BifsDecoder * codec, GF_BitStream *bs, GF_Node *node, Bool is_proto);
 
 /*called once a field has been modified through a command, send eventOut or propagate eventIn if needed*/
 void gf_bifs_check_field_change(GF_Node *node, GF_FieldInfo *field);
 
+GF_Err gf_bifs_flush_command_list(GF_BifsDecoder *codec);
 
 struct __tag_bifs_enc
 {
@@ -160,6 +159,8 @@ struct __tag_bifs_enc
 	/*active stream*/
 	BIFSStreamInfo *info;
 
+	Bool UseName;
+
 	/*the scene graph the codec is encoding (set htrough ReplaceScene or manually)*/
 	GF_SceneGraph *scene_graph;
 	/*current proto graph for DEF/USE*/
@@ -171,9 +172,6 @@ struct __tag_bifs_enc
 	/*active QP*/
 	M_QuantizationParameter *ActiveQP;
 
-	/*active QP*/
-	M_QuantizationParameter *GlobalQP;
-
 	u32 NumCoord;
 	Bool coord_stored, storing_coord;
 
@@ -221,8 +219,5 @@ u32 gf_bifs_get_ndt_bits(u32 NDT_Tag, u32 Version);
 /*return absolute node tag given its type in the NDT and the NDT version number*/
 u32 gf_bifs_ndt_get_node_type(u32 NDT_Tag, u32 NodeType, u32 Version);
 
-/*set QP and anim info for a proto field (BIFS allows for that in proto coding)*/
-GF_Err gf_bifs_proto_field_set_aq_info(GF_ProtoFieldInterface *field, u32 QP_Type, u32 hasMinMax, u32 QPSFType, void *qp_min_value, void *qp_max_value, u32 QP13_NumBits);
-
 #endif	//_GF_BIFS_DEV_H_
 
