@@ -56,7 +56,7 @@ struct _net_service
 	struct _tag_terminal *term;
 	/*service url*/
 	char *url;
-	/*struct _od_manager owning service, NULL for services created for remote channels*/
+	/*od_manager owning service, NULL for services created for remote channels*/
 	struct _od_manager *owner;
 	/*number of attached remote channels ODM (ESD URLs)*/
 	u32 nb_ch_users;
@@ -92,6 +92,8 @@ GF_Err gf_term_service_cache_close(GF_ClientService *ns, Bool no_save);
 this is mainly designed for movie dumping in MP4Client*/
 GF_Err gf_term_step_clocks(GF_Terminal * term, u32 ms_diff);
 
+void gf_term_sample_clocks(GF_Terminal *term);
+
 /*
 		Inline scene stuff
 */
@@ -145,54 +147,73 @@ struct _inline_scene
 	OD shutdown/startup when seeking. This will also remove unneeded net traffic for AddChannel/RemoveChannel
 	like RTSP TEARDOWN/SETUP*/
 	Bool static_media_ressources;
+
+	/*current simulation time of the compositor*/
+	Double simulation_time;
+
+	/*current IRI fragment if any*/
+	char *fragment_uri;
+
+	/*secondary resource scene*/
+	Bool secondary_resource;
+
+	char *redirect_xml_base;
 };
 
-GF_InlineScene *gf_is_new(GF_InlineScene *parentScene);
-void gf_is_del(GF_InlineScene *is);
-struct _od_manager *gf_is_find_odm(GF_InlineScene *is, u16 OD_ID);
-void gf_is_disconnect(GF_InlineScene *is, Bool for_shutdown);
-void gf_is_remove_object(GF_InlineScene *is, GF_ObjectManager *odm, Bool for_shutdown);
+GF_InlineScene *gf_inline_new(GF_InlineScene *parentScene);
+void gf_inline_del(GF_InlineScene *is);
+struct _od_manager *gf_inline_find_odm(GF_InlineScene *is, u16 OD_ID);
+void gf_inline_disconnect(GF_InlineScene *is, Bool for_shutdown);
+void gf_inline_remove_object(GF_InlineScene *is, GF_ObjectManager *odm, Bool for_shutdown);
 /*browse all (media) channels and send buffering info to the app*/
-void gf_is_buffering_info(GF_InlineScene *is);
-void gf_is_attach_to_renderer(GF_InlineScene *is);
-struct _mediaobj *gf_is_get_media_object(GF_InlineScene *is, MFURL *url, u32 obj_type_hint, Bool lock_timelines);
-struct _mediaobj *gf_is_get_media_object_ex(GF_InlineScene *is, MFURL *url, u32 obj_type_hint, Bool lock_timelines, struct _mediaobj *sync_ref);
-
-void gf_is_setup_object(GF_InlineScene *is, GF_ObjectManager *odm);
+void gf_inline_buffering_info(GF_InlineScene *is);
+void gf_inline_attach_to_compositor(GF_InlineScene *is);
+struct _mediaobj *gf_inline_get_media_object(GF_InlineScene *is, MFURL *url, u32 obj_type_hint, Bool lock_timelines);
+struct _mediaobj *gf_inline_get_media_object_ex(GF_InlineScene *is, MFURL *url, u32 obj_type_hint, Bool lock_timelines, struct _mediaobj *sync_ref, Bool always_load_new, GF_Node *node_ptr);
+void gf_inline_setup_object(GF_InlineScene *is, GF_ObjectManager *odm);
 /*restarts inline scene - care has to be taken not to remove the scene while it is traversed*/
-void gf_is_restart(GF_InlineScene *is);
+void gf_inline_restart(GF_InlineScene *is);
 /*updates scene duration based on settings*/
-void gf_is_set_duration(GF_InlineScene *is);
+void gf_inline_set_duration(GF_InlineScene *is);
 /*locate media object by ODID (non dynamic ODs) or URL (dynamic ODs)*/
-struct _mediaobj *gf_is_find_object(GF_InlineScene *is, u16 ODID, char *url);
+struct _mediaobj *gf_inline_find_object(GF_InlineScene *is, u16 ODID, char *url);
 /*returns scene time in sec - exact meaning of time depends on standard used*/
-Double gf_is_get_time(void *_is);
+Double gf_inline_get_time(void *_is);
 /*returns true if the given node DEF name is the url target view (eg blabla#myview)*/
-Bool gf_is_default_scene_viewpoint(GF_Node *node);
+Bool gf_inline_default_scene_viewpoint(GF_Node *node);
 /*register extra scene graph for on-screen display*/
-void gf_is_register_extra_graph(GF_InlineScene *is, GF_SceneGraph *extra_scene, Bool do_remove);
+void gf_inline_register_extra_graph(GF_InlineScene *is, GF_SceneGraph *extra_scene, Bool do_remove);
 /*forces scene size info (without changing pixel metrics) - this may be needed by modules using extra graphs (like timedtext)*/
-void gf_is_force_scene_size(GF_InlineScene *is, u32 width, u32 height);
+void gf_inline_force_scene_size(GF_InlineScene *is, u32 width, u32 height);
 /*regenerate a scene graph based on available objects - can only be called for dynamic OD streams*/
-void gf_is_regenerate(GF_InlineScene *is);
+void gf_inline_regenerate(GF_InlineScene *is);
 /*selects given ODM for dynamic scenes*/
-void gf_is_select_object(GF_InlineScene *is, GF_ObjectManager *odm);
+void gf_inline_select_object(GF_InlineScene *is, GF_ObjectManager *odm);
 /*restarts dynamic scene from given time: scene graph is not reseted, objects are just restarted
 instead of closed and reopened. If a media control is present on inline, from_time is overriden by MC range*/
-void gf_is_restart_dynamic(GF_InlineScene *is, u64 from_time);
+void gf_inline_restart_dynamic(GF_InlineScene *is, u64 from_time);
 /*owner inline node has been modified*/
-void gf_is_on_modified(GF_Node *node);
+void gf_inline_on_modified(GF_Node *node);
 /*returns scene graph associated with an externProto lib - exported for VRML/X3D loaded*/
-GF_SceneGraph *gf_is_get_proto_lib(void *_is, MFURL *lib_url);
-/*exported for renderers: handles filtering of "self" parameter indicating anchor only acts on container inline scene
+GF_SceneGraph *gf_inline_get_proto_lib(void *_is, MFURL *lib_url);
+/*exported for compositor: handles filtering of "self" parameter indicating anchor only acts on container inline scene
 not root one. Returns 1 if handled (cf user.h, navigate event)*/
-Bool gf_is_process_anchor(GF_Node *caller, GF_Event *evt);
+Bool gf_inline_process_anchor(GF_Node *caller, GF_Event *evt);
 /*extern proto fetcher*/
-GF_SceneGraph *gf_is_get_proto_lib(void *SceneCallback, MFURL *lib_url);
-void gf_is_force_scene_size_video(GF_InlineScene *is, GF_MediaObject *mo);
+GF_SceneGraph *gf_inline_get_proto_lib(void *SceneCallback, MFURL *lib_url);
+void gf_inline_force_scene_size_video(GF_InlineScene *is, GF_MediaObject *mo);
+void gf_inline_sample_time(GF_InlineScene *is);
 /*compares object URL with another URL - ONLY USE THIS WITH DYNAMIC ODs*/
 Bool gf_mo_is_same_url(GF_MediaObject *obj, MFURL *inline_url);
 
+void gf_mo_update_caps(GF_MediaObject *mo);
+
+
+GF_Node *gf_inline_get_subscene_root(GF_Node *inline_node);
+GF_Node *gf_inline_get_parent_node(GF_Node *node, u32 idx);
+
+const char *gf_inline_get_fragment_uri(GF_Node *node);
+void gf_inline_set_fragment_uri(GF_Node *node, const char *uri);
 
 enum
 {
@@ -212,7 +233,7 @@ enum
 	GF_TERM_MULTI_THREAD = 1<<3,
 	GF_TERM_SYSDEC_RESYNC = 1<<4,
 	GF_TERM_SINGLE_CLOCK = 1<<5,
-	GF_TERM_RENDER_FRAME = 1<<6
+	GF_TERM_DRAW_FRAME = 1<<6
 };
 
 
@@ -223,8 +244,8 @@ struct _tag_terminal
 
 	/*callback to user application*/	
 	GF_User *user;
-	/*scene renderer*/
-	struct __tag_base_renderer *renderer;
+	/*scene compositor*/
+	struct __tag_compositor *compositor;
 	/*file downloader*/
 	GF_DownloadManager *downloader;
 	/*top level scene*/
@@ -236,6 +257,8 @@ struct _tag_terminal
 	GF_Mutex *mm_mx;
 	/*decoding thread*/
 	GF_Thread *mm_thread;
+	/*last codec used in mm loop*/
+	u32 last_codec;
 	/*thread priority*/
 	s32 priority;
 	u32 cumulated_priority;
@@ -259,7 +282,7 @@ struct _tag_terminal
 	
 	/*options (cf config doc)*/
 	Bool enable_cache;
-	/*data timeout for network buffering in ms - if no data is recieved within this timeout
+	/*data timeout for network buffering in ms - if no data is received within this timeout
 	the initial buffering aborts. */
 	u32 net_data_timeout;
 
@@ -271,6 +294,12 @@ struct _tag_terminal
 	/*special list used by nodes needing a call to RenderNode but not in the traverese scene graph (VRML/MPEG-4 protos only). 
 	For these nodes, the traverse effect passed will be NULL. This is only used by InputSensor node at the moment*/
 	GF_List *nodes_pending;
+
+	/*root node of the user prefs*/
+	GF_SceneGraph *dcci_doc;
+
+	GF_List *extensions;
+	GF_List *unthreaded_extensions;
 };
 
 
@@ -305,20 +334,20 @@ void gf_term_close_services(GF_Terminal *app, GF_ClientService *service);
 void gf_term_lock_net(GF_Terminal *app, Bool LockIt);
 
 
-/*locks scene renderer*/
-void gf_term_lock_renderer(GF_Terminal *app, Bool LockIt);
-/*get scene renderer time*/
+/*locks scene compositor*/
+void gf_term_lock_compositor(GF_Terminal *app, Bool LockIt);
+/*get scene compositor time - FIXME this is not flexible enough for SMIL/Multiple time containers*/
 u32 gf_term_get_time(GF_Terminal *term);
-/*forces scene retraversal*/
-void gf_term_invalidate_renderer(GF_Terminal *term);
+/*forces scene composition*/
+void gf_term_invalidate_compositor(GF_Terminal *term);
 
 /*callbacks for scene graph library so that all related ESM nodes are properly instanciated*/
 void gf_term_node_callback(void *_is, u32 type, GF_Node *node, void *param);
 
 /*add/rem node requiring a call to render without being present in traversed graph (VRML/MPEG-4 protos). 
 For these nodes, the traverse effect passed will be NULL.*/
-void gf_term_add_render_node(GF_Terminal *term, GF_Node *node);
-void gf_term_rem_render_node(GF_Terminal *term, GF_Node *node);
+void gf_term_queue_node_traverse(GF_Terminal *term, GF_Node *node);
+void gf_term_unqueue_node_traverse(GF_Terminal *term, GF_Node *node);
 
 
 
@@ -358,6 +387,8 @@ void gf_clock_reset(GF_Clock *ck);
 void gf_clock_stop(GF_Clock *ck);
 /*return clock time in ms*/
 u32 gf_clock_time(GF_Clock *ck);
+/*return ellapsed time in ms since start of the clock*/
+u32 gf_clock_ellapse_time(GF_Clock *ck);
 /*sets clock time - FIXME: drift updates for OCRs*/
 void gf_clock_set_time(GF_Clock *ck, u32 TS);
 /*return clock time in ms without drift adjustment - used by audio objects only*/
@@ -418,10 +449,10 @@ struct _es_channel
 	Bool BufferOn;
 	/*min level to trigger buffering on, max to trigger it off. */
 	u32 MinBuffer, MaxBuffer;
-	/*amount of buffered media - this is the DTS of the last recieved AU minus the onject clock time, to make sure
+	/*amount of buffered media - this is the DTS of the last received AU minus the onject clock time, to make sure
 	we always have MaxBuffer ms ready for composition when resuming the clock*/
 	s32 BufferTime;
-	/*last recieved AU time - if exceeding a certain time and buffering is on, buffering is turned off.
+	/*last received AU time - if exceeding a certain time and buffering is on, buffering is turned off.
 	This is needed for streams with very short duration (less than buffer time) and stream with only one AU (BIFS/OD)*/
 	u32 last_au_time;
 	/*Current reassemnbling buffer - currently packets are NOT reordered, only AUs are*/
@@ -450,6 +481,10 @@ struct _es_channel
 	BIFS and OD to be decoded and first frame render, in order to detect media objects that would also need
 	buffering - note this doesn't affect the clock, it is still paused if buffering*/
 	Bool first_au_fetched;
+
+	/* used in Carousel, to skip packets until the end of AU */ 
+	Bool skip_carousel_au;
+	
 	/* TimeStamp to Media Time mapping*/
 	/*TS (in TSResolution) corresponding to the SeedTime of the decoder. Delivered by net, otherwise 0*/
 	u64 seed_ts;
@@ -463,15 +498,27 @@ struct _es_channel
 	/*flag for clock init. Only a channel owning the clock will set this flag on clock init*/
 	Bool IsClockInit;
 
-	/*duration of last recieved AU if any, 0 if not known (most of the time)*/
+	/*duration of last received AU if any, 0 if not known (most of the time)*/
 	u32 au_duration;
+	/*A channel with this flag set considers each incoming packet as a complete AU and assigns timestamps 
+	upon reception matching the reception time, then dispatching it into the decoding buffer (only tested
+	with audi video). This flag is turned on by setting esd->slconfig->predefined to 'SLPredef_SkipSL' */
 	Bool skip_sl;
 
-	Bool is_protected;
+	/*indicates that decoding can be called directly when receiving a complete AU on this channel
+	This is used by systems streams in non-seekable (eg broadcast/multicast, MPEG-2 TS multiplexes) to 
+	make sure resources are setup as fast as possible. If the AU is too early, it will be kept in the 
+	decoding buffer*/
+	Bool dispatch_after_db;
+
+	/*indicates that decoding is called directly when receiving a packet on this channel
+	This is used to bypass SL defragmenting and decoding buffer for EIT internal streams*/
+	Bool bypass_sl_and_db;
 
 	GF_IPMPTool *ipmp_tool;
+	Bool is_protected;
 
-	/*TSs as recieved from network - these are used for cache storage*/
+	/*TSs as received from network - these are used for cache storage*/
 	u64 net_dts, net_cts;
 };
 
@@ -497,7 +544,7 @@ void gf_es_drop_au(GF_Channel *ch);
 void gf_es_on_connect(GF_Channel *ch);
 /*reconfigure SL for this channel*/
 void gf_es_reconfig_sl(GF_Channel *ch, GF_SLConfig *slc);
-/*hack for streaming: whenever a time map (media time <-> TS time) event is recieved on the channel reset decoding buffer
+/*hack for streaming: whenever a time map (media time <-> TS time) event is received on the channel reset decoding buffer
 this is needed because all server tested resend packets on already running channel*/
 void gf_es_map_time(GF_Channel *ch, Bool reset);
 /*dummy channels are used by scene decoders which don't use ESM but load directly the scene graph themselves
@@ -534,10 +581,8 @@ enum
 	/*the codec uses the interface from another codec (only used by private scene streams to handle
 	any intern sprite/animation streams)*/
 	GF_ESM_CODEC_IS_USE = 1<<1,
-	/*set for OD codec when all ressources are static and scene graph is generated on the fly*/
-	GF_ESM_CODEC_IS_SCENE_OD = 1<<2,
 	/*set for OD codec when static (ressources are declared in OD stream esd a la ISMA*/
-	GF_ESM_CODEC_IS_STATIC_OD = 1<<3,
+	GF_ESM_CODEC_IS_STATIC_OD = 1<<2,
 };
 
 struct _generic_codec 
@@ -615,6 +660,11 @@ enum
 	GF_ODM_INLINE_PROFILES = (1<<5),
 	/*flag set if object declared by network service, not from OD stream*/
 	GF_ODM_NOT_IN_OD_STREAM = (1<<6),
+
+	/*dynamic flags*/
+	
+	/*flag set if associated subscene must be regenerated*/
+	GF_ODM_REGENERATE_SCENE = (1<<10),
 };
 
 enum
@@ -666,7 +716,7 @@ struct _od_manager
 	/*full object duration 0 if unknown*/
 	u64 duration;
 	/*
-	upon start: media start time as requested by scene renderer (eg not media control)
+	upon start: media start time as requested by scene compositor (eg not media control)
 	set to -1 upon stop to postpone stop request
 	*/
 	u64 media_start_time, media_stop_time;
@@ -720,8 +770,6 @@ struct _object_clock *gf_odm_get_media_clock(GF_ObjectManager *odm);
 void gf_odm_init_segments(GF_ObjectManager *odm, GF_List *list, MFURL *url);
 /*returns true if this OD depends on the given clock*/
 Bool gf_odm_shares_clock(GF_ObjectManager *odm, struct _object_clock *ock);
-/*refresh all ODs when an non-interactive stream is found*/
-void gf_odm_refresh_uninteractives(GF_ObjectManager *odm);
 
 GF_Segment *gf_odm_find_segment(GF_ObjectManager *odm, char *descName);
 /*locks ODM with destruction check - returns 0 if object manager is not attached to object*/
@@ -761,6 +809,13 @@ struct _mediaobj
 	u32 framesize;
 	/*pointer to data frame */
 	char *frame;
+	/*nodes currently registered with the media object - used to dispatch MediaAccessEvents*/
+	GF_List *nodes;
+	/*pointer to the node responsible for the creation of this media object
+	ONLY used for scene media type (animationStreams) 
+	Reset upon creation of the decoder.
+	*/
+	void *node_ptr;
 };
 
 GF_MediaObject *gf_mo_new();
@@ -781,7 +836,8 @@ GF_Err gf_odm_post_es_setup(struct _es_channel *ch, struct _generic_codec *dec,
 */
 void gf_term_attach_service(GF_Terminal *term, GF_InputService *service_hdl);
 
-
+/*media access events */
+void gf_term_service_media_event(GF_ObjectManager *odm, u32 event_type);
 
 u32 URL_GetODID(MFURL *url);
 
