@@ -49,27 +49,29 @@ extern "C" {
 		Video hardware output module
 */
 
-/*opengl configuration*/
-typedef struct
-{
-	Bool double_buffered;
-} GF_GLConfig;
-
 enum
 {
 	/*HW supports YUV->backbuffer blitting*/
 	GF_VIDEO_HW_HAS_YUV = (1<<1),
 	/*HW supports keying*/
 	GF_VIDEO_HW_HAS_COLOR_KEY = (1<<2),
-	/*HW supports OpenGL rendering. Whether this is OpenGL or OpenGL-ES depends on compilation settings and cannot be 
-	changed at runtime*/
-	GF_VIDEO_HW_HAS_OPENGL = (1<<4),
-	/*HW supports 90 degres rotation of display (Mobile Phones & PDAs)*/
-	GF_VIDEO_HW_CAN_ROTATE = (1<<5),
+	/*HW supports 90 degres rotation of display in 2D mode (Mobile Phones & PDAs)*/
+	GF_VIDEO_HW_CAN_ROTATE = (1<<3),
+	/*HW supports OpenGL rendering. Whether this is OpenGL or OpenGL-ES depends on compilation settings
+	and cannot be changed at runtime*/
+	GF_VIDEO_HW_OPENGL = (1<<4),
+	/*HW supports OpenGL offscreen rendering. Whether this is OpenGL or OpenGL-ES depends on compilation settings
+	and cannot be changed at runtime*/
+	GF_VIDEO_HW_OPENGL_OFFSCREEN = (1<<5),
+	/*HW supports OpenGL offscreen rendering with alpha. Whether this is OpenGL or OpenGL-ES depends on compilation settings
+	and cannot be changed at runtime*/
+	GF_VIDEO_HW_OPENGL_OFFSCREEN_ALPHA = (1<<6),
+	/*HW supports YUV overlays*/
+	GF_VIDEO_HW_HAS_YUV_OVERLAY = (1<<7),
 };
 
 /*interface name and version for video output*/
-#define GF_VIDEO_OUTPUT_INTERFACE	GF_4CC('G','V','O',0x03) 
+#define GF_VIDEO_OUTPUT_INTERFACE	GF_4CC('G','V','O',0x04) 
 
 /*
 			video output interface
@@ -84,7 +86,10 @@ enum
 	The context shall be setup in Resize and SetFullScreen calls which are always happening in the main 
 	rendering thread. This will take care of openGL context issues with multithreading
 
-	Except Setup and Shutdown functions, all interface functions are called through the main renderer thread
+	By default all modules are required to be setup in 2D. If 3D is needed, a GF_EVENT_VIDEO_SETUP will
+	be sent with the desired configuration.
+
+	Except Setup and Shutdown functions, all interface functions are called through the main compositor thread
 	or its user to avoid multithreading issues. Care must still be taken when handling events
 */
 typedef struct _video_out
@@ -94,9 +99,8 @@ typedef struct _video_out
 
 	/*setup system - if os_handle is NULL the driver shall create the output display (common case)
 	the other case is currently only used by child windows on win32 and winCE
-	@init_flags: a list of initialization flags as specified in user.h
-	if cfg is specified, the output is 3D, otherwise 2D*/
-	GF_Err (*Setup)(struct _video_out *vout, void *os_handle, void *os_display, u32 init_flags, GF_GLConfig *cfg);
+	@init_flags: a list of initialization flags as specified in user.h*/
+	GF_Err (*Setup)(struct _video_out *vout, void *os_handle, void *os_display, u32 init_flags);
 	/*shutdown system */
 	void (*Shutdown) (struct _video_out *vout);
 
@@ -114,17 +118,23 @@ typedef struct _video_out
 	GF_EVENT_SIZE:  inital window resize upon scene load
 	GF_EVENT_VIDEO_SETUP: all HW related setup:
 		* for 2D output, this means resizing the backbuffer if needed (depending on HW constraints)
-		* for 3D output, this means re-setup of OpenGL context (depending on HW constraints). Depending on windowing systems 
-			and implementations, it could be possible to resize a window without destroying the GL context.
+		* for 3D output, this means re-setup of OpenGL context (depending on HW constraints).
+			* This can be a request for an offscreen rendering surface. If supported, this surface SHALL
+			be readable through glReadPixels. If not supported, just return an error.
+			Note that GPAC never uses more than one GL context (offscreen or main video)
+			* Depending on windowing systems and implementations, it could be possible to resize a window 
+		without destroying the GL context. If the GL context is destroyed, the module should send an event
+		of the same type to the player.
 	
 	This function is also called with a NULL event at the begining of each rendering cycle, in order to allow event 
 	handling for modules uncapable of safe multithreading (eg X11)
 	*/
 	GF_Err (*ProcessEvent)(struct _video_out *vout, GF_Event *event);
 
-	/*pass events to user (assigned before setup)*/
+	/*pass events to user (assigned before setup) - return 1 if the event has been processed by GPAC 
+	(eiher scene or navigation), 0 otherwise*/
 	void *evt_cbk_hdl;
-	void (*on_event)(void *hdl, GF_Event *event);
+	Bool (*on_event)(void *hdl, GF_Event *event);
 
 	/*
 			All the following are 2D specific and are NEVER called in 3D mode
@@ -141,8 +151,12 @@ typedef struct _video_out
 	/*blit surface src to backbuffer - if a window is not specified, the full surface is used
 	the blitter MUST support stretching and RGB24 sources. Support for YUV is indicated in the hw caps
 	of the driver. If none is supported, just set this function to NULL and let gpac performs software blitting.
-	Whenever this function fails, the blit will be performed in software mode*/
-	GF_Err (*Blit)(struct _video_out *vout, GF_VideoSurface *video_src, GF_Window *src_wnd, GF_Window *dst_wnd, u32 *src_rgb_key);
+	Whenever this function fails, the blit will be performed in software mode
+	if is_overlay is set, this is an overlay on the video memory (Flush would have been called before)
+		overlay_type 1: this is regular overlay without color keying
+		overlay_type 2: this is overlay with color keying
+	*/
+	GF_Err (*Blit)(struct _video_out *vout, GF_VideoSurface *video_src, GF_Window *src_wnd, GF_Window *dst_wnd, u32 overlay_type);
 
 	/*set of above HW flags*/
 	u32 hw_caps;
@@ -153,6 +167,12 @@ typedef struct _video_out
 	/*maximum resolution of the screen*/
 	u32 max_screen_width, max_screen_height;
 
+	/*overlay color key used by the hardware bliter - if not set, only top-level overlay can be used*/
+	u32 overlay_color_key;
+#ifdef ENABLE_JOYSTICK
+	u32 centered_mode;
+#endif
+
 	/*driver private*/
 	void *opaque;
 } GF_VideoOutput;
