@@ -33,26 +33,14 @@ extern "C" {
 #include <gpac/list.h>
 #include <gpac/math.h>
 
-
-/*define to enable SVG test graph with static attribute allocation but no inheritance*/
-//#define	GPAC_ENABLE_SVG_SA
-//#define	GPAC_ENABLE_SVG_SANI
-
-#if defined(GPAC_ENABLE_SVG_SA) || defined(GPAC_ENABLE_SVG_SANI)
-#define	GPAC_ENABLE_SVG_SA_BASE
-#endif
-
-
 /*
 	TAG definitions are static, in order to be able to mix nodes from different standard
 	in a single scenegraph. These TAGs are only used internally (they do not match any
 	binary encoding)
 */
 enum {
-	/*no node shall use this tag*/
-	TAG_ForbiddenZero = 0,
 	/*undefined node: just the base node class, used for parsing*/
-	TAG_UndefinedNode = 1,
+	TAG_UndefinedNode = 0,
 	/*all MPEG-4/VRML/X3D proto instances have this tag*/
 	TAG_ProtoNode,
 
@@ -75,18 +63,6 @@ enum {
 	/*all nodes below MUST be parent nodes*/
 	GF_NODE_FIRST_PARENT_NODE_TAG,
 
-#ifdef GPAC_ENABLE_SVG_SA
-	/*range for SVG / static alloc mode*/
-	GF_NODE_RANGE_FIRST_SVG_SA, 
-	GF_NODE_RANGE_LAST_SVG_SA = GF_NODE_RANGE_FIRST_SVG_SA+100,
-#endif
-
-#ifdef GPAC_ENABLE_SVG_SANI
-	/*range for SVG static alloc and no inheritance mode*/
-	GF_NODE_RANGE_FIRST_SVG_SANI, 
-	GF_NODE_RANGE_LAST_SVG_SANI = GF_NODE_RANGE_FIRST_SVG_SANI+100,
-#endif
-
 	/*DOM text node*/
 	TAG_DOMText,
 	/*all nodes below MUST use the base DOM structure (with dyn attribute list)*/
@@ -94,10 +70,33 @@ enum {
 	
 	/*full node*/
 	TAG_DOMFullNode = GF_NODE_FIRST_DOM_NODE_TAG,
+
 	/*range for SVG*/
 	GF_NODE_RANGE_FIRST_SVG, 
 	GF_NODE_RANGE_LAST_SVG = GF_NODE_RANGE_FIRST_SVG+100,
 
+	/*range for XBL*/
+	GF_NODE_RANGE_FIRST_XBL, 
+	TAG_XBL_bindings = GF_NODE_RANGE_FIRST_XBL,
+	TAG_XBL_binding,
+	TAG_XBL_content,
+	TAG_XBL_children,
+	TAG_XBL_implementation,
+	TAG_XBL_constructor,
+	TAG_XBL_destructor,
+	TAG_XBL_field,
+	TAG_XBL_property,
+	TAG_XBL_getter,
+	TAG_XBL_setter,
+	TAG_XBL_method,
+	TAG_XBL_parameter,
+	TAG_XBL_body,
+	TAG_XBL_handlers,
+	TAG_XBL_handler,
+	TAG_XBL_resources,
+	TAG_XBL_stylesheet,
+	TAG_XBL_image,
+	GF_NODE_RANGE_LAST_XBL,
 };
 
 
@@ -163,11 +162,15 @@ by calling the gf_sg_get_next_available_node_id function
 GF_Err gf_node_set_id(GF_Node*n, u32 nodeID, const char *nodeDEFName);
 /*get def name of the node , NULL if not set*/
 const char *gf_node_get_name(GF_Node*);
+/*get def name of the node , or the string representation of the node pointer if not set*/
+const char *gf_node_get_log_name(GF_Node*);
 /*get def ID of the node, 0 if node not def*/
 u32 gf_node_get_id(GF_Node*);
 /* gets node built-in name (eg 'Appearance', ..) */
 const char *gf_node_get_class_name(GF_Node *Node);
 
+u32 gf_sg_node_get_tag_by_class_name(const char *name, u32 xmlns);
+
 /*unset the node ID*/
 GF_Err gf_node_remove_id(GF_Node *p);
 
@@ -175,13 +178,12 @@ GF_Err gf_node_remove_id(GF_Node *p);
 void *gf_node_get_private(GF_Node*);
 void gf_node_set_private(GF_Node*, void *);
 
-/*set rendering function. When rendering a scene graph, the render stack is passed
-through the graph without being touched. If a node has no associated RenderNode(), the traversing of the 
+/*set traversal callback function. If a node has no associated callback, the traversing of the 
 graph won't propagate below it. It is the app responsability to setup traversing functions as needed
 VRML/MPEG4:  Instanciated Protos are handled internally as well as interpolators, valuators and scripts
 @is_destroy: set when the node is about to be destroyed
 */
-GF_Err gf_node_set_callback_function(GF_Node *, void (*NodeFunction)(GF_Node *node, void *render_stack, Bool is_destroy) );
+GF_Err gf_node_set_callback_function(GF_Node *, void (*NodeFunction)(GF_Node *node, void *traverse_state, Bool is_destroy) );
 
 /*register a node (DEFed or not), specifying parent if any.
 A node must be registered whenever used by something (a parent node, a command, whatever) to prevent its 
@@ -208,14 +210,14 @@ GF_Err gf_node_replace(GF_Node *old_node, GF_Node *new_node, Bool updateOrderedG
 u32 gf_node_get_num_instances(GF_Node *node);
 
 
-/*calls RenderNode on this node*/
-void gf_node_render(GF_Node *node, void *renderStack);
+/*calls node traverse callback routine on this node*/
+void gf_node_traverse(GF_Node *node, void *udta);
 /*allows a node to be re-rendered - by default a node in its render phase will never be rendered a second time. 
 Use this function to enable a second render for this node - this must be called while node is being rendered*/
-void gf_node_allow_cyclic_render(GF_Node *node);
+void gf_node_allow_cyclic_traverse(GF_Node *node);
 
-/*blindly calls RenderNode on all nodes in the "children" list*/
-void gf_node_render_children(GF_Node *node, void *renderStack);;
+/*blindly calls traverse callback on all children nodes */
+void gf_node_traverse_children(GF_Node *node, void *renderStack);
 /*returns number of parent for this node (parent are kept regardless of DEF state)*/
 u32 gf_node_get_parent_count(GF_Node *node);
 /*returns desired parent for this node (parent are kept regardless of DEF state)
@@ -233,10 +235,18 @@ enum
 	if relying on this flag for sub-tree discarding (eg, culling or similar)*/
 	GF_SG_CHILD_DIRTY = 1<<1,
 
+	/*flag set by bindable nodes to indicate a modification of the bindable stack. This is 
+	only used for offscreen rendering of Layer3D*/
+	GF_SG_VRML_BINDABLE_DIRTY = 1<<2,
+
+	/*flag set whenever a ColorTransform node is removed from a parent node*/
+	GF_SG_VRML_COLOR_DIRTY = 1<<3,
+
+
 	/*SVG-specific flags due to mix of geometry and appearance & co attributes*/
 	/*SVG geometry changed is the same as base flag*/
 	GF_SG_SVG_GEOMETRY_DIRTY		= GF_SG_NODE_DIRTY,
-	GF_SG_SVG_COLOR_DIRTY			= 1<<2,
+	GF_SG_SVG_COLOR_DIRTY			= GF_SG_VRML_BINDABLE_DIRTY,
 	GF_SG_SVG_DISPLAYALIGN_DIRTY	= 1<<3,
 	GF_SG_SVG_FILL_DIRTY			= 1<<4,
 	GF_SG_SVG_FILLOPACITY_DIRTY		= 1<<5,
@@ -248,21 +258,20 @@ enum
 	GF_SG_SVG_FONTWEIGHT_DIRTY		= 1<<11,
 	GF_SG_SVG_LINEINCREMENT_DIRTY	= 1<<12,
 	GF_SG_SVG_OPACITY_DIRTY			= 1<<13,
-	GF_SG_SVG_SOLIDCOLOR_DIRTY		= 1<<14,
-	GF_SG_SVG_SOLIDOPACITY_DIRTY	= 1<<15,
-	GF_SG_SVG_STOPCOLOR_DIRTY		= 1<<16,
-	GF_SG_SVG_STOPOPACITY_DIRTY		= 1<<17,
-	GF_SG_SVG_STROKE_DIRTY			= 1<<18,
-	GF_SG_SVG_STROKEDASHARRAY_DIRTY	= 1<<19,
-	GF_SG_SVG_STROKEDASHOFFSET_DIRTY= 1<<20,
-	GF_SG_SVG_STROKELINECAP_DIRTY	= 1<<21,
-	GF_SG_SVG_STROKELINEJOIN_DIRTY	= 1<<22,
-	GF_SG_SVG_STROKEMITERLIMIT_DIRTY= 1<<23,
-	GF_SG_SVG_STROKEOPACITY_DIRTY	= 1<<24,
-	GF_SG_SVG_STROKEWIDTH_DIRTY		= 1<<25,
-	GF_SG_SVG_TEXTALIGN_DIRTY		= 1<<26,
-	GF_SG_SVG_TEXTANCHOR_DIRTY		= 1<<27,
-	GF_SG_SVG_VECTOREFFECT_DIRTY	= 1<<28,
+	GF_SG_SVG_SOLIDCOLOR_OR_OPACITY_DIRTY		= 1<<14,
+	GF_SG_SVG_STOPCOLOR_OR_OPACITY_DIRTY		= 1<<15,
+	GF_SG_SVG_STROKE_DIRTY			= 1<<16,
+	GF_SG_SVG_STROKEDASHARRAY_DIRTY	= 1<<17,
+	GF_SG_SVG_STROKEDASHOFFSET_DIRTY= 1<<18,
+	GF_SG_SVG_STROKELINECAP_DIRTY	= 1<<19,
+	GF_SG_SVG_STROKELINEJOIN_DIRTY	= 1<<20,
+	GF_SG_SVG_STROKEMITERLIMIT_DIRTY= 1<<21,
+	GF_SG_SVG_STROKEOPACITY_DIRTY	= 1<<22,
+	GF_SG_SVG_STROKEWIDTH_DIRTY		= 1<<23,
+	GF_SG_SVG_TEXTPOSITION_DIRTY	= 1<<24,
+	GF_SG_SVG_DISPLAY_DIRTY			= 1<<25,
+	GF_SG_SVG_VECTOREFFECT_DIRTY	= 1<<26,
+	GF_SG_SVG_XLINK_HREF_DIRTY		= 1<<27,
 };
 
 /*set dirty flags.
@@ -275,6 +284,11 @@ which means tat if you never clean the dirty flags, no propagation will take pla
 */
 void gf_node_dirty_set(GF_Node *node, u32 flags, Bool dirty_parents);
 
+/*mark all parent subtrees for this node as GF_SG_CHILD_DIRTY
+Note: parent subtree marking aborts if a node in the subtree is already marked with GF_SG_CHILD_DIRTY
+which means that if you never clean the dirty flags, no propagation will take place
+*/
+void gf_node_dirty_parents(GF_Node *node);
 
 /*set dirty flag off. It is the user responsability to clear dirty flags
 if @flags is 0, all flags are set off
@@ -342,6 +356,9 @@ void gf_sg_del(GF_SceneGraph *sg);
 /*reset the full graph - all nodes, routes and protos are destroyed*/
 void gf_sg_reset(GF_SceneGraph *sg);
 
+/*parses the given XML document and returns a scene graph composed of GF_DOMFullNode*/
+GF_Err gf_sg_new_from_xml_doc(const char *src, GF_SceneGraph **scene);
+
 /*set/get user private data*/
 void gf_sg_set_private(GF_SceneGraph *sg, void *user_priv);
 void *gf_sg_get_private(GF_SceneGraph *sg);
@@ -397,10 +414,17 @@ Note:
 	- this doesn't perform application setup for the node, this must be done by the caller
 */
 GF_Node *gf_node_new(GF_SceneGraph *sg, u32 tag);
-/*clones a node in the given graph and register with parent cloned. The cloning respects DEF/USE nodes*/
-GF_Node *gf_node_clone(GF_SceneGraph *inScene, GF_Node *orig, GF_Node *cloned_parent);
 /*inits node (either internal stack or user-defined) - usually called once the node has been fully loaded*/
 void gf_node_init(GF_Node *node);
+
+/*clones a node in the given graph and register with parent cloned. The cloning handles ID based on id_suffix:
+ id_suffix = NULL: all IDs are removed from the cloned subtree, (each node instance will become a hard copy)
+ id_suffix = "": ID will be kept exactly as they where in the original subtree - this may lead to errors due to 
+		the presence of the same ID depending on the standard (DOM, ...).
+ id_suffix = anything: all IDs are translated ($(name) -> $(name)id_suffix) and bynary IDs are generated on the fly
+*/
+GF_Node *gf_node_clone(GF_SceneGraph *inScene, GF_Node *orig, GF_Node *cloned_parent, char *id_suffix, Bool deep);
+
 /*gets scene time for scene this node belongs too, 0 if timeline not specified*/
 Double gf_node_get_scene_time(GF_Node *node);
 
@@ -462,6 +486,8 @@ typedef union
 	GF_JSAPIOPT gpac_cfg;
 	GF_Node *node;
 	struct __gf_download_manager *dnld_man;
+	GF_SceneGraph *scene;
+	void *term;
 	GF_JSAPIINFO info;
 } GF_JSAPIParam;
 
@@ -501,8 +527,6 @@ enum
 	GF_JSAPI_OP_GET_FOCUS,
 	/*!set focus to given node.*/
 	GF_JSAPI_OP_SET_FOCUS,
-	/*!get target scene URL*/
-	GF_JSAPI_OP_GET_URL,
 	/*!replace target scene URL*/
 	GF_JSAPI_OP_LOAD_URL,
 	/*!get option by section and key*/
@@ -511,6 +535,29 @@ enum
 	GF_JSAPI_OP_SET_OPT,
 	/*!retrieve download manager*/
 	GF_JSAPI_OP_GET_DOWNLOAD_MANAGER,
+	/*!get navigation speed if any*/
+	GF_JSAPI_OP_GET_SPEED,
+	/*!get current frame rate*/
+	GF_JSAPI_OP_GET_FPS,
+	/*!set current title*/
+	GF_JSAPI_OP_SET_TITLE,
+	/*!gets DCCI scenegraph if any*/
+	GF_JSAPI_OP_GET_DCCI,
+	/*!gets subscene for current node if any*/
+	GF_JSAPI_OP_GET_SUBSCENE,
+	/*!resolves relative Xlink based on xml:base*/
+	GF_JSAPI_OP_RESOLVE_XLINK,
+	/*!evaluates if the given IRI is available for playback (returns 1) or not. If the IRI is
+	NULL, this evaluates whether the scene is ready for composition (canvas setup) or not.*/
+	GF_JSAPI_OP_EVAL_IRI,
+
+	/*!gets GPAC terminal*/
+	GF_JSAPI_OP_GET_TERM,
+
+	/*!pauses an SVG element*/
+	GF_JSAPI_OP_PAUSE_SVG,
+	/*!resumes an SVG ELEMENT*/
+	GF_JSAPI_OP_RESUME_SVG,
 };
 /*
 interface to various get/set options:
@@ -566,6 +613,10 @@ enum
 	/*same as NodeDelete, and also updates OrderedGroup.order when deleting a child*/
 	GF_SG_NODE_DELETE_EX,
 
+	/*BIFS*/
+	GF_SG_FIELD_REPLACE_OP, 
+	GF_SG_INDEXED_REPLACE_OP,
+
 	GF_SG_LAST_BIFS_COMMAND,
 
 
@@ -640,19 +691,42 @@ typedef struct
 	*/
 	Bool use_names;
 
-	/*route insert, replace and delete (BIFS only)
-	fromNodeID is also used to identify operandElementId in LASeR Add/Replace
-	*/
-	u32 RouteID;
-	char *def_name;
-	u32 fromNodeID, fromFieldIndex;
-	u32 toNodeID, toFieldIndex;
-
 	/*proto list to insert - BIFS only*/
 	GF_List *new_proto_list;
 	/*proto ID list to delete - BIFS only*/
 	u32 *del_proto_list;
 	u32 del_proto_list_size;
+
+
+	/*route insert, replace and delete (BIFS only)
+		fromNodeID is also used to identify operandElementId in LASeR Add/Replace
+
+	OR
+		sendEvent
+	*/
+//	union {
+		u32 RouteID;
+		u32 send_event_name;
+//	};
+//	union {
+		char *def_name;
+		char *send_event_string;
+//	};
+//	union {
+		u32 fromNodeID;
+		s32 send_event_integer;
+//	};
+	u32 fromFieldIndex;
+
+//	union {
+		u32 toNodeID;
+		s32 send_event_x;
+//	};
+//	union {
+		u32 toFieldIndex;
+		s32 send_event_y;
+//	};
+	Bool aggregated;
 } GF_Command;
 
 
@@ -668,9 +742,10 @@ GF_Err gf_sg_command_apply(GF_SceneGraph *inScene, GF_Command *com, Double time_
 GF_Err gf_sg_command_apply_list(GF_SceneGraph *graph, GF_List *comList, Double time_offset);
 /*returns new commandFieldInfo structure and registers it with command*/
 GF_CommandField *gf_sg_command_field_new(GF_Command *com);
-/*clones the command in another graph - needed for uncompressed conditional in protos*/
-GF_Command *gf_sg_command_clone(GF_Command *com, GF_SceneGraph *inGraph);
-
+/*clones the command in another graph - needed for uncompressed conditional in protos
+if force_clone is not set and the target graph is the same as the command graph, nodes are just registered
+with the new commands rather than cloned*/
+GF_Command *gf_sg_command_clone(GF_Command *com, GF_SceneGraph *inGraph, Bool force_clone);
 
 #ifdef __cplusplus
 }
