@@ -69,18 +69,13 @@ GF_Err BIFS_ReleaseScene(GF_SceneDecoder *plug)
 	return GF_OK;
 }
 
-static GF_Err BIFS_AttachStream(GF_BaseDecoder *plug, 
-									 u16 ES_ID, 
-									 char *decSpecInfo, 
-									 u32 decSpecInfoSize, 
-									 u16 DependsOnES_ID,
-									 u32 objectTypeIndication, 
-									 Bool Upstream)
+static GF_Err BIFS_AttachStream(GF_BaseDecoder *plug, GF_ESD *esd)
 {
 	BIFSPriv *priv = (BIFSPriv *)plug->privateStack;
 	GF_Err e;
-	if (Upstream) return GF_NOT_SUPPORTED;
-	e = gf_bifs_decoder_configure_stream(priv->codec, ES_ID, decSpecInfo, decSpecInfoSize, objectTypeIndication);
+	if (esd->decoderConfig->upstream) return GF_NOT_SUPPORTED;
+	if (!esd->decoderConfig->decoderSpecificInfo) return GF_BAD_PARAM;
+	e = gf_bifs_decoder_configure_stream(priv->codec, esd->ESID, esd->decoderConfig->decoderSpecificInfo->data, esd->decoderConfig->decoderSpecificInfo->dataLength, esd->decoderConfig->objectTypeIndication);
 	if (!e) priv->nb_streams++;
 	return e;
 }
@@ -104,7 +99,7 @@ static GF_Err BIFS_ProcessData(GF_SceneDecoder*plug, char *inBuffer, u32 inBuffe
 	e = gf_bifs_decode_au(priv->codec, ES_ID, inBuffer, inBufferLength, ((Double)AU_time)/1000.0);
 
 	/*if scene not attached do it*/
-	gf_is_attach_to_renderer(priv->pScene);
+	gf_inline_attach_to_compositor(priv->pScene);
 	return e;
 }
 
@@ -114,10 +109,12 @@ Bool BIFS_CanHandleStream(GF_BaseDecoder *ifce, u32 StreamType, u32 ObjectType,
 	if (StreamType!=GF_STREAM_SCENE) return 0;
 	switch (ObjectType) {
 	case 0x00:
-	case 0x01:
-	case 0x02:
-	case 0xFE:
-	case 0xFF:
+		return 1;
+	case GPAC_OTI_SCENE_BIFS:
+	case GPAC_OTI_SCENE_BIFS_V2:
+	/*Streams with this value with a StreamType indicating a systems stream (values 1,2,3, 6, 7, 8, 9) 
+		shall be treated as if the ObjectTypeIndication had been set to 0x01*/
+	case 0xFF:	
 		priv->PL = PL;
 		return 1;
 	default:
