@@ -64,7 +64,7 @@ static GF_Err CTXLoad_SetCapabilities(GF_BaseDecoder *plug, const GF_CodecCapabi
 static void ODS_SetupOD(GF_InlineScene *is, GF_ObjectDescriptor *od)
 {
 	GF_ObjectManager *odm;
-	odm = gf_is_find_odm(is, od->objectDescriptorID);
+	odm = gf_inline_find_odm(is, od->objectDescriptorID);
 	/*remove the old OD*/
 	if (odm) gf_odm_disconnect(odm, 1);
 	odm = gf_odm_new();
@@ -97,7 +97,7 @@ void CTXLoad_OnActivate(GF_Node *node)
 	GF_InlineScene *is = (GF_InlineScene *) gf_node_get_private(node);
 	M_Conditional*c = (M_Conditional*)node;
 	/*always apply in parent graph to handle protos correctly*/
-	if (c->activate) gf_sg_command_apply_list(gf_node_get_graph(node), c->buffer.commandList, gf_is_get_time(is));
+	if (c->activate) gf_sg_command_apply_list(gf_node_get_graph(node), c->buffer.commandList, gf_inline_get_time(is));
 }
 void CTXLoad_OnReverseActivate(GF_Node *node)
 {
@@ -105,7 +105,7 @@ void CTXLoad_OnReverseActivate(GF_Node *node)
 	M_Conditional*c = (M_Conditional*)node;
 	/*always apply in parent graph to handle protos correctly*/
 	if (!c->reverseActivate) 
-		gf_sg_command_apply_list(gf_node_get_graph(node), c->buffer.commandList, gf_is_get_time(is));
+		gf_sg_command_apply_list(gf_node_get_graph(node), c->buffer.commandList, gf_inline_get_time(is));
 }
 
 void CTXLoad_NodeCallback(void *cbk, u32 type, GF_Node *node, void *param)
@@ -153,34 +153,29 @@ static GF_Err CTXLoad_Setup(GF_BaseDecoder *plug)
 	priv->ctx = gf_sm_new(priv->inline_scene->graph);
 	memset(&priv->load, 0, sizeof(GF_SceneLoader));
 	priv->load.ctx = priv->ctx;
+	priv->load.is = priv->inline_scene;
 	priv->load.scene_graph = priv->inline_scene->graph;
 	priv->load.fileName = priv->file_name;
 	priv->load.flags = GF_SM_LOAD_FOR_PLAYBACK;
 	priv->load.localPath = gf_modules_get_option((GF_BaseInterface *)plug, "General", "CacheDirectory");
-	priv->load.swf_import_flags = GF_SM_SWF_STATIC_DICT | GF_SM_SWF_QUAD_CURVE | GF_SM_SWF_SCALABLE_LINE;
+	priv->load.swf_import_flags = GF_SM_SWF_STATIC_DICT | GF_SM_SWF_QUAD_CURVE | GF_SM_SWF_SCALABLE_LINE | GF_SM_SWF_SPLIT_TIMELINE;
 	return GF_OK;
 }
 
-static GF_Err CTXLoad_AttachStream(GF_BaseDecoder *plug, 
-									 u16 ES_ID, 
-									 char *decSpecInfo, 
-									 u32 decSpecInfoSize, 
-									 u16 DependsOnES_ID,
-									 u32 objectTypeIndication, 
-									 Bool Upstream)
+static GF_Err CTXLoad_AttachStream(GF_BaseDecoder *plug, GF_ESD *esd)
 {
 	const char *ext;
 	GF_BitStream *bs;
 	u32 size;
 	CTXLoadPriv *priv = (CTXLoadPriv *)plug->privateStack;
-	if (Upstream) return GF_NOT_SUPPORTED;
+	if (esd->decoderConfig->upstream) return GF_NOT_SUPPORTED;
 
 	/*animation stream like*/
 	if (priv->ctx) {
 		GF_StreamContext *sc;
 		u32 i = 0;
 		while ((sc = (GF_StreamContext *)gf_list_enum(priv->ctx->streams, &i))) {
-			if (ES_ID == sc->ESID) {
+			if (esd->ESID == sc->ESID) {
 				priv->nb_streams++;
 				return GF_OK;
 			}
@@ -188,18 +183,18 @@ static GF_Err CTXLoad_AttachStream(GF_BaseDecoder *plug,
 		return GF_NON_COMPLIANT_BITSTREAM;
 	}
 	/*main dummy stream we need a dsi*/
-	if (!decSpecInfo) 
+	if (!esd->decoderConfig->decoderSpecificInfo) 
 		return GF_NON_COMPLIANT_BITSTREAM;
-	bs = gf_bs_new(decSpecInfo, decSpecInfoSize, GF_BITSTREAM_READ);
+	bs = gf_bs_new(esd->decoderConfig->decoderSpecificInfo->data, esd->decoderConfig->decoderSpecificInfo->dataLength, GF_BITSTREAM_READ);
 	priv->file_size = gf_bs_read_u32(bs);
 	gf_bs_del(bs);
-	size = decSpecInfoSize - sizeof(u32);
+	size = esd->decoderConfig->decoderSpecificInfo->dataLength - sizeof(u32);
 	priv->file_name = (char *) malloc(sizeof(char)*(1 + size) );
-	memcpy(priv->file_name, decSpecInfo + sizeof(u32),  sizeof(char)*(decSpecInfoSize - sizeof(u32)) );
+	memcpy(priv->file_name, esd->decoderConfig->decoderSpecificInfo->data + sizeof(u32),  sizeof(char)*(esd->decoderConfig->decoderSpecificInfo->dataLength - sizeof(u32)) );
 	priv->file_name[size] = 0;
 	priv->nb_streams = 1;
 	priv->load_flags = 0;
-	priv->base_stream_id = ES_ID;
+	priv->base_stream_id = esd->ESID;
 
 	
 	CTXLoad_Setup(plug);
@@ -265,12 +260,6 @@ static Bool CTXLoad_StreamInRootOD(GF_ObjectDescriptor *od, u32 ESID)
 }
 
 
-static GF_SceneGraph *CTXLoad_GetProtoLib(void *cbk, MFURL *lib_url)
-{
-	CTXLoadPriv *priv = (CTXLoadPriv *)cbk;
-	return gf_is_get_proto_lib(priv->inline_scene, lib_url);
-}
-
 Double CTXLoad_GetVRMLTime(void *cbk)
 {
 	u32 secs, msecs;
@@ -291,18 +280,18 @@ static void CTXLoad_CheckStreams(CTXLoadPriv *priv )
 	i=0;
 	while ((sc = (GF_StreamContext *)gf_list_enum(priv->ctx->streams, &i))) {
 		/*all streams in root OD are handled with ESID 0 to differentiate with any animation streams*/
-		if (CTXLoad_StreamInRootOD(priv->ctx->root_od, sc->ESID)) sc->ESID = 0;
+		if (CTXLoad_StreamInRootOD(priv->ctx->root_od, sc->ESID)) sc->in_root_od = 1;
 		if (!sc->timeScale) sc->timeScale = 1000;
 
 		j=0;
 		while ((au = (GF_AUContext *)gf_list_enum(sc->AUs, &j))) {
 			if (!au->timing) au->timing = (u64) (sc->timeScale*au->timing_sec);
 		}
-		if (au && !sc->ESID && (au->timing>max_dur)) max_dur = (u32) (au->timing * 1000 / sc->timeScale);
+		if (au && sc->in_root_od && (au->timing>max_dur)) max_dur = (u32) (au->timing * 1000 / sc->timeScale);
 	}
 	if (max_dur) {
 		priv->inline_scene->root_od->duration = max_dur;
-		gf_is_set_duration(priv->inline_scene);
+		gf_inline_set_duration(priv->inline_scene);
 	}
 }
 
@@ -310,7 +299,7 @@ static GF_Err CTXLoad_ProcessData(GF_SceneDecoder *plug, char *inBuffer, u32 inB
 								u16 ES_ID, u32 stream_time, u32 mmlevel)
 {
 	GF_Err e = GF_OK;
-	u32 i, j, k, nb_updates;
+	u32 i, j, k, nb_updates, last_rap=0;
 	GF_AUContext *au;
 	Bool can_delete_com;
 	GF_StreamContext *sc;
@@ -326,6 +315,32 @@ static GF_Err CTXLoad_ProcessData(GF_SceneDecoder *plug, char *inBuffer, u32 inB
 		if (e) return e;
 	}
 
+
+	if (stream_time==(u32)-1) {
+		/*seek on root stream: destroy the context manager and reload it. We cannot seek on the main stream
+		because commands may have changed node attributes/children and we d'ont track the initial value*/
+		if (priv->base_stream_id == ES_ID) {
+			if (priv->src) fclose(priv->src);
+			priv->src = NULL;
+			gf_sm_load_done(&priv->load);
+			priv->file_pos = 0;
+			/*this will call detach scene*/
+			gf_inline_disconnect(priv->inline_scene, 1);
+			return CTXLoad_Setup((GF_BaseDecoder *)plug);
+		}
+		i=0;
+		while ((sc = (GF_StreamContext *)gf_list_enum(priv->ctx->streams, &i))) {
+			/*not our stream*/
+			if (!sc->in_root_od && (sc->ESID != ES_ID)) continue;
+			/*not the base stream*/
+			if (sc->in_root_od && (priv->base_stream_id != ES_ID)) continue;
+			/*handle SWF media extraction*/
+			if ((sc->streamType == GF_STREAM_OD) && (priv->load_flags==1)) continue;
+			sc->last_au_time = 0;
+		}
+		return GF_OK;
+	}
+
 	if (priv->load_flags != 2) {
 
 		if (priv->progressive_support) {
@@ -363,7 +378,7 @@ static GF_Err CTXLoad_ProcessData(GF_SceneDecoder *plug, char *inBuffer, u32 inB
 			}
 			if (!priv->inline_scene->graph_attached) {
 				gf_sg_set_scene_size_info(priv->inline_scene->graph, priv->ctx->scene_width, priv->ctx->scene_height, priv->ctx->is_pixel_metrics);
-				gf_is_attach_to_renderer(priv->inline_scene);
+				gf_inline_attach_to_compositor(priv->inline_scene);
 
 				CTXLoad_CheckStreams(priv);
 			}
@@ -376,11 +391,12 @@ static GF_Err CTXLoad_ProcessData(GF_SceneDecoder *plug, char *inBuffer, u32 inB
 			priv->load_flags = 1;
 			e = gf_sm_load_init(&priv->load);
 			if (!e) {
+				CTXLoad_CheckStreams(priv);
 				gf_sg_set_scene_size_info(priv->inline_scene->graph, priv->ctx->scene_width, priv->ctx->scene_height, priv->ctx->is_pixel_metrics);
 				/*VRML, override base clock*/
 				if ((priv->load.type==GF_SM_LOAD_VRML) || (priv->load.type==GF_SM_LOAD_X3DV) || (priv->load.type==GF_SM_LOAD_X3D)) {
+					/*override clock callback*/
 					gf_sg_set_scene_time_callback(priv->inline_scene->graph, CTXLoad_GetVRMLTime);
-					gf_sg_set_proto_loader(priv->inline_scene->graph, CTXLoad_GetProtoLib);
 				}
 			}
 		} 
@@ -403,7 +419,7 @@ static GF_Err CTXLoad_ProcessData(GF_SceneDecoder *plug, char *inBuffer, u32 inB
 		if (priv->load_flags==2) {
 			CTXLoad_CheckStreams(priv);
 			if (!gf_list_count(priv->ctx->streams)) {
-				gf_is_attach_to_renderer(priv->inline_scene);
+				gf_inline_attach_to_compositor(priv->inline_scene);
 			}
 		}
 	}
@@ -413,29 +429,38 @@ static GF_Err CTXLoad_ProcessData(GF_SceneDecoder *plug, char *inBuffer, u32 inB
 	i=0;
 	while ((sc = (GF_StreamContext *)gf_list_enum(priv->ctx->streams, &i))) {
 		/*not our stream*/
-		if (sc->ESID && (sc->ESID != ES_ID)) continue;
+		if (!sc->in_root_od && (sc->ESID != ES_ID)) continue;
 		/*not the base stream*/
-		if (!sc->ESID && (priv->base_stream_id != ES_ID)) continue;
+		if (sc->in_root_od && (priv->base_stream_id != ES_ID)) continue;
 		/*handle SWF media extraction*/
 		if ((sc->streamType == GF_STREAM_OD) && (priv->load_flags==1)) continue;
 
 		/*check for seek*/
 		if (sc->last_au_time > 1 + stream_time) {
-			/*root streams shall not seek this way*/
-			if (!sc->ESID) assert(0);
-			/*on other streams, we assume the stream is 100% RAP wrt to the base stream*/
-			else {
-				sc->last_au_time = 0;
-			}
+			sc->last_au_time = 0;
 		}
 
 		can_delete_com = 0;
-		if (!sc->ESID && (priv->load_flags==2)) can_delete_com = 1;
+		if (sc->in_root_od && (priv->load_flags==2)) can_delete_com = 1;
 
 		/*we're in the right stream, apply update*/
 		j=0;
+
+		/*seek*/
+		if (!sc->last_au_time) {
+			while ((au = (GF_AUContext *)gf_list_enum(sc->AUs, &j))) {
+				u32 au_time = (u32) (au->timing*1000/sc->timeScale);
+
+				if (au_time > stream_time) 
+					break;
+				if (au->is_rap) last_rap = j-1;
+			}
+			j = last_rap;
+		}
+
 		while ((au = (GF_AUContext *)gf_list_enum(sc->AUs, &j))) {
 			u32 au_time = (u32) (au->timing*1000/sc->timeScale);
+
 			if (au_time + 1 <= sc->last_au_time) {
 				/*remove first replace command*/
 				if (can_delete_com && (sc->streamType==GF_STREAM_SCENE)) {
@@ -451,6 +476,7 @@ static GF_Err CTXLoad_ProcessData(GF_SceneDecoder *plug, char *inBuffer, u32 inB
 				}
 				continue;
 			}
+
 			if (au_time > stream_time) {
 				nb_updates++;
 				break;
@@ -580,7 +606,7 @@ static GF_Err CTXLoad_ProcessData(GF_SceneDecoder *plug, char *inBuffer, u32 inB
 					{
 						GF_ODRemove *odR = (GF_ODRemove*)com;
 						for (k=0; k<odR->NbODs; k++) {
-							GF_ObjectManager *odm = gf_is_find_odm(priv->inline_scene, odR->OD_ID[k]);
+							GF_ObjectManager *odm = gf_inline_find_odm(priv->inline_scene, odR->OD_ID[k]);
 							if (odm) gf_odm_disconnect(odm, 1);
 						}
 					}
@@ -600,11 +626,12 @@ static GF_Err CTXLoad_ProcessData(GF_SceneDecoder *plug, char *inBuffer, u32 inB
 			}
 			sc->last_au_time = au_time + 1;
 			/*attach graph to renderer*/
-			gf_is_attach_to_renderer(priv->inline_scene);
+			if (!priv->inline_scene->graph_attached)
+				gf_inline_attach_to_compositor(priv->inline_scene);
 			if (e) return e;
 
 			/*for root streams remove completed AUs (no longer needed)*/
-			if (!sc->ESID && !gf_list_count(au->commands) ) {
+			if (sc->in_root_od && !gf_list_count(au->commands) ) {
 				j--;
 				gf_list_rem(sc->AUs, j);
 				gf_list_del(au->commands);
@@ -630,6 +657,7 @@ const char *CTXLoad_GetName(struct _basedecoder *plug)
 	case GF_SM_LOAD_SWF: return "Flash (SWF) Emulator";
 	case GF_SM_LOAD_XSR: return "LASeRML Loader";
 	case GF_SM_LOAD_MP4: return "MP4 Memory Loader";
+	case GF_SM_LOAD_XBL: return "XBL Parser";
 
 	default: return "Undetermined";
 	}
@@ -638,9 +666,16 @@ const char *CTXLoad_GetName(struct _basedecoder *plug)
 Bool CTXLoad_CanHandleStream(GF_BaseDecoder *ifce, u32 StreamType, u32 ObjectType, char *decSpecInfo, u32 decSpecInfoSize, u32 PL)
 {
 	if (StreamType==GF_STREAM_PRIVATE_SCENE) {
-		if (ObjectType==1) return 1;
+		switch (ObjectType) {
+		case GPAC_OTI_PRIVATE_SCENE_GENERIC:
+			return 1;
 		/*LASeR ML: we use this plugin since it has command handling*/
-		if (ObjectType==3) return 1;
+		case GPAC_OTI_PRIVATE_SCENE_LASER:
+			return 1;
+		/* XBL */
+		case GPAC_OTI_PRIVATE_SCENE_XBL:
+			return 1;
+		}
 	}
 	/*SVG*/
 	//if ((StreamType==GF_STREAM_PRIVATE_SCENE) && (ObjectType==2)) return 1;
