@@ -76,24 +76,25 @@ Bool DC_RemoveChannel(DCReader *read, LPNETCHANNEL ch)
 Bool DC_CanHandleURL(GF_InputService *plug, const char *url)
 {
 	char *sExt = strrchr(url, '.');
-	if (!sExt) return 0;
-	if (!strnicmp(sExt, ".gz", 3)) sExt = strrchr(sExt, '.');
-	if (!strnicmp(url, "rtsp://", 7)) return 0;
+	if (sExt) {
+		if (!strnicmp(sExt, ".gz", 3)) sExt = strrchr(sExt, '.');
+		if (!strnicmp(url, "rtsp://", 7)) return 0;
 
 
-	/*the mpeg-4 mime types for bt/xmt are NOT registered at all :)*/
-	if (gf_term_check_extension(plug, "application/x-bt", "bt bt.gz btz", "MPEG-4 Text (BT)", sExt)) return 1;
-	if (gf_term_check_extension(plug, "application/x-xmt", "xmt xmt.gz xmtz", "MPEG-4 Text (XMT)", sExt)) return 1;
-	//if (gf_term_check_extension(plug, "application/x-xmta", "xmta xmta.gz xmtaz", "MPEG-4 Text (XMT)", sExt)) return 1;
-	/*but all these ones are*/
-	if (gf_term_check_extension(plug, "model/vrml", "wrl wrl.gz", "VRML World", sExt)) return 1;
-	if (gf_term_check_extension(plug, "x-model/x-vrml", "wrl wrl.gz", "VRML World", sExt)) return 1;
-	if (gf_term_check_extension(plug, "model/x3d+vrml", "x3dv x3dv.gz x3dvz", "X3D/VRML World", sExt)) return 1;
-	if (gf_term_check_extension(plug, "model/x3d+xml", "x3d x3d.gz x3dz", "X3D/XML World", sExt)) return 1;
-	if (gf_term_check_extension(plug, "application/x-shockwave-flash", "swf", "Macromedia Flash Movie", sExt)) return 1;
-	if (gf_term_check_extension(plug, "image/svg+xml", "svg svg.gz svgz", "SVG Document", sExt)) return 1;
-	if (gf_term_check_extension(plug, "image/x-svgm", "svgm", "SVGM Document", sExt)) return 1;
-	if (gf_term_check_extension(plug, "application/x-LASeR+xml", "xsr", "LASeR Document", sExt)) return 1;
+		/*the mpeg-4 mime types for bt/xmt are NOT registered at all :)*/
+		if (gf_term_check_extension(plug, "application/x-bt", "bt bt.gz btz", "MPEG-4 Text (BT)", sExt)) return 1;
+		if (gf_term_check_extension(plug, "application/x-xmt", "xmt xmt.gz xmtz", "MPEG-4 Text (XMT)", sExt)) return 1;
+		//if (gf_term_check_extension(plug, "application/x-xmta", "xmta xmta.gz xmtaz", "MPEG-4 Text (XMT)", sExt)) return 1;
+		/*but all these ones are*/
+		if (gf_term_check_extension(plug, "model/vrml", "wrl wrl.gz", "VRML World", sExt)) return 1;
+		if (gf_term_check_extension(plug, "x-model/x-vrml", "wrl wrl.gz", "VRML World", sExt)) return 1;
+		if (gf_term_check_extension(plug, "model/x3d+vrml", "x3dv x3dv.gz x3dvz", "X3D/VRML World", sExt)) return 1;
+		if (gf_term_check_extension(plug, "model/x3d+xml", "x3d x3d.gz x3dz", "X3D/XML World", sExt)) return 1;
+		if (gf_term_check_extension(plug, "application/x-shockwave-flash", "swf", "Macromedia Flash Movie", sExt)) return 1;
+		if (gf_term_check_extension(plug, "image/svg+xml", "svg svg.gz svgz", "SVG Document", sExt)) return 1;
+		if (gf_term_check_extension(plug, "image/x-svgm", "svgm", "SVGM Document", sExt)) return 1;
+		if (gf_term_check_extension(plug, "application/x-LASeR+xml", "xsr", "LASeR Document", sExt)) return 1;
+	}
 
 	if (!strnicmp(url, "file://", 7) || !strstr(url, "://")) {
 		char *rtype = gf_xml_get_root_type(url, NULL);
@@ -103,6 +104,7 @@ Bool DC_CanHandleURL(GF_InputService *plug, const char *url)
 			else if (!strcmp(rtype, "XMT-A")) handled = 1;
 			else if (!strcmp(rtype, "X3D")) handled = 1;
 			else if (!strcmp(rtype, "svg")) handled = 1;
+			else if (!strcmp(rtype, "bindings")) handled = 1;
 			free(rtype);
 			return handled;
 		}
@@ -121,8 +123,22 @@ void DC_NetIO(void *cbk, GF_NETIO_Parameter *param)
 	e = param->error;
 
 	if (param->msg_type==GF_NETIO_DATA_TRANSFERED) {
-	}
-	else if (!e && (param->msg_type!=GF_NETIO_DATA_EXCHANGE)) return;
+	} else if (param->msg_type==GF_NETIO_PARSE_HEADER) {
+		if (!strcmp(param->name, "Content-Type")) {
+			if (strstr(param->value, "application/x-bt")) read->oti = GPAC_OTI_PRIVATE_SCENE_GENERIC;
+			if (strstr(param->value, "application/x-xmt")) read->oti = GPAC_OTI_PRIVATE_SCENE_GENERIC;
+			if (strstr(param->value, "model/vrml")) read->oti = GPAC_OTI_PRIVATE_SCENE_GENERIC;
+			if (strstr(param->value, "model/x3d+vrml")) read->oti = GPAC_OTI_PRIVATE_SCENE_GENERIC;
+			if (strstr(param->value, "application/x-shockwave-flash")) read->oti = GPAC_OTI_PRIVATE_SCENE_GENERIC;
+			if (strstr(param->value, "image/svg+xml")) read->oti = GPAC_OTI_PRIVATE_SCENE_SVG;
+			if (strstr(param->value, "image/x-svgm")) read->oti = GPAC_OTI_PRIVATE_SCENE_SVG;
+			if (strstr(param->value, "application/x-LASeR+xml")) read->oti = GPAC_OTI_PRIVATE_SCENE_GENERIC;
+		}
+		return;
+	} else if (!e && (param->msg_type!=GF_NETIO_DATA_EXCHANGE)) return;
+
+	if (!e && !read->oti)
+		return;
 
 	/*OK confirm*/
 	if (!read->is_service_connected) {
@@ -153,7 +169,16 @@ GF_Err DC_ConnectService(GF_InputService *plug, GF_ClientService *serv, const ch
 	read->dnload = NULL;
 
 	strcpy(read->szURL, url);
-	ext = strrchr(read->szURL, '.');
+	ext = strchr(read->szURL, '#');
+	if (ext) {
+		char *anext;
+		ext[0] = 0;
+		anext = strrchr(read->szURL, '.');
+		ext[0] = '#';
+		ext = anext;
+	} else {
+		ext = strrchr(read->szURL, '.');
+	}
 	if (ext && !stricmp(ext, ".gz")) {
 		char *anext;
 		ext[0] = 0;
@@ -161,14 +186,17 @@ GF_Err DC_ConnectService(GF_InputService *plug, GF_ClientService *serv, const ch
 		ext[0] = '.';
 		ext = anext;
 	}
-	ext += 1;
-	if (ext) {
-		tmp = strchr(ext, '#');
-		if (tmp) tmp[0] = 0;
-	}
 	read->service = serv;
 	
 	if (ext) {
+		char *cgi_par = NULL;
+		ext += 1;
+		if (ext) {
+			tmp = strchr(ext, '#'); if (tmp) tmp[0] = 0;
+			/* Warning the '?' sign should not be present in local files but it is convenient to have it 
+			   to test web content locally */
+			cgi_par = strchr(ext, '?'); if (cgi_par) cgi_par[0] = 0;
+		}
 		if (!stricmp(ext, "bt") || !stricmp(ext, "btz") || !stricmp(ext, "bt.gz") 
 			|| !stricmp(ext, "xmta") 
 			|| !stricmp(ext, "xmt") || !stricmp(ext, "xmt.gz") || !stricmp(ext, "xmtz") 
@@ -177,29 +205,37 @@ GF_Err DC_ConnectService(GF_InputService *plug, GF_ClientService *serv, const ch
 			|| !stricmp(ext, "x3dv") || !stricmp(ext, "x3dv.gz") || !stricmp(ext, "x3dvz") 
 			|| !stricmp(ext, "swf")
 			) 
-			read->oti = 0x01;
+			read->oti = GPAC_OTI_PRIVATE_SCENE_GENERIC;
 
 		else if (!stricmp(ext, "svg") || !stricmp(ext, "svgz") || !stricmp(ext, "svg.gz")) {
-			read->oti = 0x02;
+			read->oti = GPAC_OTI_PRIVATE_SCENE_SVG;
 		}
 		/*XML LASeR*/
 		else if (!stricmp(ext, "xsr"))
-			read->oti = 0x03;
+			read->oti = GPAC_OTI_PRIVATE_SCENE_LASER;
+		else if (!stricmp(ext, "xbl"))
+			read->oti = GPAC_OTI_PRIVATE_SCENE_XBL;
+
+		if (cgi_par) cgi_par[0] = '?';
 	}
 
 	if (!read->oti && (!strnicmp(url, "file://", 7) || !strstr(url, "://"))) {
 		char *rtype = gf_xml_get_root_type(url, NULL);
 		if (rtype) {
-			if (!strcmp(rtype, "SAFSession")) read->oti = 0x03;
-			else if (!strcmp(rtype, "svg")) read->oti = 0x02;
-			else if (!strcmp(rtype, "XMT-A")) read->oti = 0x01;
-			else if (!strcmp(rtype, "X3D")) read->oti = 0x01;
+			if (!strcmp(rtype, "SAFSession")) read->oti = GPAC_OTI_PRIVATE_SCENE_LASER;
+			else if (!strcmp(rtype, "svg")) read->oti = GPAC_OTI_PRIVATE_SCENE_SVG;
+			else if (!strcmp(rtype, "XMT-A")) read->oti = GPAC_OTI_PRIVATE_SCENE_GENERIC;
+			else if (!strcmp(rtype, "X3D")) read->oti = GPAC_OTI_PRIVATE_SCENE_GENERIC;
+			else if (!strcmp(rtype, "bindings")) read->oti = GPAC_OTI_PRIVATE_SCENE_XBL;
 			free(rtype);
 		}
 	}
 
 	/*remote fetch*/
-	if (strnicmp(url, "file://", 7) && strstr(url, "://")) {
+	if (!strnicmp(url, "file://", 7)) {
+		url += 7;
+	}
+	else if (strstr(url, "://")) {
 		DC_DownloadFile(plug, read->szURL);
 		return GF_OK;
 	}
