@@ -28,6 +28,60 @@
 #define DDCONTEXT	DDContext *dd = (DDContext *)dr->opaque;
 #define DDBACK		DDSurface *pBack = (DDSurface *) gf_list_get(dd->surfaces, 0);
 
+
+
+
+static Bool pixelformat_yuv(u32 pixel_format)
+{
+	switch (pixel_format) {
+	case GF_PIXEL_YUY2:
+	case GF_PIXEL_YVYU:
+	case GF_PIXEL_UYVY:
+	case GF_PIXEL_VYUY:
+	case GF_PIXEL_Y422:
+	case GF_PIXEL_UYNV:
+	case GF_PIXEL_YUNV:
+	case GF_PIXEL_V422:
+	case GF_PIXEL_YV12:
+	case GF_PIXEL_IYUV:
+	case GF_PIXEL_I420:
+		return 1;
+	default:
+		return 0;
+	}
+}
+
+
+static u32 get_win_4CC(u32 pixel_format)
+{
+	switch (pixel_format) {
+	case GF_PIXEL_YUY2:
+		return mmioFOURCC('Y', 'U', 'Y', '2');
+	case GF_PIXEL_YVYU:
+		return mmioFOURCC('Y', 'V', 'Y', 'U');
+	case GF_PIXEL_UYVY:
+		return mmioFOURCC('U', 'Y', 'V', 'Y');
+	case GF_PIXEL_VYUY:
+		return mmioFOURCC('V', 'Y', 'U', 'Y');
+	case GF_PIXEL_Y422:
+		return mmioFOURCC('Y', '4', '2', '2');
+	case GF_PIXEL_UYNV:
+		return mmioFOURCC('U', 'Y', 'N', 'V');
+	case GF_PIXEL_YUNV:
+		return mmioFOURCC('Y', 'U', 'N', 'V');
+	case GF_PIXEL_V422:
+		return mmioFOURCC('V', '4', '2', '2');
+	case GF_PIXEL_YV12:
+		return mmioFOURCC('Y', 'V', '1', '2');
+	case GF_PIXEL_IYUV:
+		return mmioFOURCC('I', 'Y', 'U', 'V');
+	case GF_PIXEL_I420:
+		return mmioFOURCC('I', '4', '2', '0');
+	default:
+		return 0;
+	}
+}
+
 static GF_Err DD_ClearBackBuffer(GF_VideoOutput *dr, u32 color)
 {
 	HRESULT hr;
@@ -60,8 +114,9 @@ static GF_Err DD_ClearBackBuffer(GF_VideoOutput *dr, u32 color)
 	return FAILED(hr) ? GF_IO_ERR : GF_OK;
 }
 
-GF_Err CreateBackBuffer(GF_VideoOutput *dr, u32 Width, u32 Height)
+GF_Err CreateBackBuffer(GF_VideoOutput *dr, u32 Width, u32 Height, Bool use_system_memory)
 {
+	Bool force_reinit;
 	HRESULT hr;
 	const char *opt; 
 #ifdef USE_DX_3
@@ -73,7 +128,10 @@ GF_Err CreateBackBuffer(GF_VideoOutput *dr, u32 Width, u32 Height)
 	DDCONTEXT;
 
 
-	if (dd->pBack && !dd->fullscreen && (dd->width == Width) && (dd->height == Height) ) {
+	force_reinit = 0;
+	if (use_system_memory && !dd->systems_memory) force_reinit = 1;
+	else if (!use_system_memory && dd->systems_memory) force_reinit = 1;
+	if (dd->pBack && !force_reinit&& !dd->fullscreen && (dd->width == Width) && (dd->height == Height) ) {
 		return GF_OK;
 	}
 
@@ -88,9 +146,14 @@ GF_Err CreateBackBuffer(GF_VideoOutput *dr, u32 Width, u32 Height)
 	if (dd->systems_memory==2) {
 		ddsd.ddsCaps.dwCaps |= DDSCAPS_SYSTEMMEMORY;
 	} else {
-		opt = gf_modules_get_option((GF_BaseInterface *)dr, "Video", "UseHardwareMemory");
-		if (opt && !strcmp(opt, "yes")) {
+		opt = NULL;
+		if (use_system_memory) {
+			opt = gf_modules_get_option((GF_BaseInterface *)dr, "Video", "UseHardwareMemory");
+			if (opt && !strcmp(opt, "yes")) use_system_memory = 0;
+		}
+		if (!use_system_memory) {
 			dd->systems_memory = 0;
+			ddsd.ddsCaps.dwCaps |= DDSCAPS_VIDEOMEMORY;
 		} else {
 			ddsd.ddsCaps.dwCaps |= DDSCAPS_SYSTEMMEMORY;
 			dd->systems_memory = 1;
@@ -101,12 +164,28 @@ GF_Err CreateBackBuffer(GF_VideoOutput *dr, u32 Width, u32 Height)
 	ddsd.dwHeight = Height;
 
 #ifdef USE_DX_3
-    if( FAILED( hr = IDirectDraw_CreateSurface(dd->pDD, &ddsd, &dd->pBack, NULL ) ) )
-        return GF_IO_ERR;
+    hr = IDirectDraw_CreateSurface(dd->pDD, &ddsd, &dd->pBack, NULL );
 #else
-    if( FAILED( hr = IDirectDraw7_CreateSurface(dd->pDD, &ddsd, &dd->pBack, NULL ) ) )
-        return GF_IO_ERR;
+    hr = IDirectDraw7_CreateSurface(dd->pDD, &ddsd, &dd->pBack, NULL );
 #endif
+	if (FAILED(hr)) {
+		if (!dd->systems_memory) {
+			ddsd.ddsCaps.dwCaps &= ~DDSCAPS_VIDEOMEMORY;
+			ddsd.ddsCaps.dwCaps |= DDSCAPS_SYSTEMMEMORY;
+			dd->systems_memory = 1;
+			if (opt && !strcmp(opt, "yes")) {
+				gf_modules_set_option((GF_BaseInterface *)dr, "Video", "UseHardwareMemory", "no");
+
+				GF_LOG(GF_LOG_INFO, GF_LOG_MMIO, ("[DX Out] Hardware Video not available for backbuffer)\n"));
+			}
+#ifdef USE_DX_3
+		    hr = IDirectDraw_CreateSurface(dd->pDD, &ddsd, &dd->pBack, NULL );
+#else
+		    hr = IDirectDraw7_CreateSurface(dd->pDD, &ddsd, &dd->pBack, NULL );
+#endif
+		}
+		if (FAILED(hr)) return GF_IO_ERR;
+	}
 
 	/*store size*/
 	if (!dd->fullscreen) {
@@ -119,6 +198,7 @@ GF_Err CreateBackBuffer(GF_VideoOutput *dr, u32 Width, u32 Height)
 	DD_ClearBackBuffer(dr, 0xFF000000);
 
 	if (!dd->yuv_init) DD_InitYUV(dr);
+	GF_LOG(GF_LOG_INFO, GF_LOG_MMIO, ("[DX Out] BackBuffer %d x %d created on %s memory\n", Width, Height, dd->systems_memory ? "System" : "Video"));
 	return GF_OK;
 }
 
@@ -244,7 +324,8 @@ GF_Err InitDirectDraw(GF_VideoOutput *dr, u32 Width, u32 Height)
     }
 	IDirectDrawClipper_Release(pcClipper);
 	dd->ddraw_init = 1;
-	return CreateBackBuffer(dr, Width, Height);
+	/*if YUV not initialize, init using HW video memory to setup HW caps*/
+	return CreateBackBuffer(dr, Width, Height, dd->yuv_init);
 }
 
 static GF_Err DD_LockSurface(DDContext *dd, GF_VideoSurface *vi, void *surface)
@@ -275,7 +356,7 @@ static GF_Err DD_LockSurface(DDContext *dd, GF_VideoSurface *vi, void *surface)
 	vi->width = desc.dwWidth;
 	vi->height = desc.dwHeight;
 	vi->pitch = desc.lPitch;
-	vi->is_hardware_memory = !dd->systems_memory;
+	vi->is_hardware_memory = dd->systems_memory ? 0 : 1;
 	return GF_OK;
 }
 
@@ -307,6 +388,8 @@ static void *LockOSContext(GF_VideoOutput *dr, Bool do_lock)
 {
 	DDCONTEXT;
 
+	if (!dd->pBack) return NULL;
+
 	if (do_lock) {
 		if (!dd->lock_hdc && ! IDirectDrawSurface_IsLost(dd->pBack)) {
 			if (FAILED(IDirectDrawSurface_GetDC(dd->pBack, &dd->lock_hdc)) ) 
@@ -320,7 +403,7 @@ static void *LockOSContext(GF_VideoOutput *dr, Bool do_lock)
 }
 
 
-static GF_Err DD_BlitSurface(DDContext *dd, DDSurface *src, GF_Window *src_wnd, GF_Window *dst_wnd, u32 *key)
+static GF_Err DD_BlitSurface(DDContext *dd, DDSurface *src, GF_Window *src_wnd, GF_Window *dst_wnd, GF_ColorKey *key)
 {
 	HRESULT hr;
 	u32 dst_w, dst_h, src_w, src_h, flags;
@@ -335,8 +418,10 @@ static GF_Err DD_BlitSurface(DDContext *dd, DDSurface *src, GF_Window *src_wnd,
 	if (dst_wnd != NULL) MAKERECT(r_dst, dst_wnd);
 
 	if (key) {
+		u32 col;
 		DDCOLORKEY ck;
-		ck.dwColorSpaceHighValue = ck.dwColorSpaceLowValue = *key;
+		col = GF_COL_ARGB(0xFF, key->r, key->g, key->b);
+		ck.dwColorSpaceHighValue = ck.dwColorSpaceLowValue = col;
 		hr = IDirectDrawSurface_SetColorKey(src->pSurface, DDCKEY_SRCBLT, &ck);
 		if (FAILED(hr)) return GF_IO_ERR;
 	}
@@ -354,59 +439,6 @@ static GF_Err DD_BlitSurface(DDContext *dd, DDSurface *src, GF_Window *src_wnd,
 	return FAILED(hr) ? GF_IO_ERR : GF_OK;
 }
 
-
-
-static Bool pixelformat_yuv(u32 pixel_format)
-{
-	switch (pixel_format) {
-	case GF_PIXEL_YUY2:
-	case GF_PIXEL_YVYU:
-	case GF_PIXEL_UYVY:
-	case GF_PIXEL_VYUY:
-	case GF_PIXEL_Y422:
-	case GF_PIXEL_UYNV:
-	case GF_PIXEL_YUNV:
-	case GF_PIXEL_V422:
-	case GF_PIXEL_YV12:
-	case GF_PIXEL_IYUV:
-	case GF_PIXEL_I420:
-		return 1;
-	default:
-		return 0;
-	}
-}
-
-
-static u32 get_win_4CC(u32 pixel_format)
-{
-	switch (pixel_format) {
-	case GF_PIXEL_YUY2:
-		return mmioFOURCC('Y', 'U', 'Y', '2');
-	case GF_PIXEL_YVYU:
-		return mmioFOURCC('Y', 'V', 'Y', 'U');
-	case GF_PIXEL_UYVY:
-		return mmioFOURCC('U', 'Y', 'V', 'Y');
-	case GF_PIXEL_VYUY:
-		return mmioFOURCC('V', 'Y', 'U', 'Y');
-	case GF_PIXEL_Y422:
-		return mmioFOURCC('Y', '4', '2', '2');
-	case GF_PIXEL_UYNV:
-		return mmioFOURCC('U', 'Y', 'N', 'V');
-	case GF_PIXEL_YUNV:
-		return mmioFOURCC('Y', 'U', 'N', 'V');
-	case GF_PIXEL_V422:
-		return mmioFOURCC('V', '4', '2', '2');
-	case GF_PIXEL_YV12:
-		return mmioFOURCC('Y', 'V', '1', '2');
-	case GF_PIXEL_IYUV:
-		return mmioFOURCC('I', 'Y', 'U', 'V');
-	case GF_PIXEL_I420:
-		return mmioFOURCC('I', '4', '2', '0');
-	default:
-		return 0;
-	}
-}
-
 static DDSurface *DD_GetSurface(GF_VideoOutput *dr, u32 width, u32 height, u32 pixel_format)
 {
 #ifdef USE_DX_3
@@ -420,14 +452,16 @@ static DDSurface *DD_GetSurface(GF_VideoOutput *dr, u32 width, u32 height, u32 p
 	/*yuv format*/
 	if (pixelformat_yuv(pixel_format)) {
 		if (dr->yuv_pixel_format) {
+			DDSurface *yuvp = &dd->yuv_pool;		
+
 			/*don't recreate a surface if not needed*/
-			if (dd->yuv_pool.pSurface && (dd->yuv_pool.width >= width) && (dd->yuv_pool.height >= height) ) return &dd->yuv_pool;
+			if (yuvp->pSurface && (yuvp->width >= width) && (yuvp->height >= height) ) return yuvp;
 
-			width = MAX(dd->yuv_pool.width, width);
-			height = MAX(dd->yuv_pool.height, height);
+			width = MAX(yuvp->width, width);
+			height = MAX(yuvp->height, height);
 
-			SAFE_DD_RELEASE(dd->yuv_pool.pSurface);
-			memset(&dd->yuv_pool, 0, sizeof(DDSurface));
+			SAFE_DD_RELEASE(yuvp->pSurface);
+			memset(yuvp, 0, sizeof(DDSurface));
 
 			memset (&ddsd, 0, sizeof(ddsd));
 			ddsd.dwSize = sizeof(ddsd);
@@ -440,16 +474,18 @@ static DDSurface *DD_GetSurface(GF_VideoOutput *dr, u32 width, u32 height, u32 p
 			ddsd.ddpfPixelFormat.dwFourCC = get_win_4CC(dr->yuv_pixel_format);
 
 #ifdef USE_DX_3
-			if( FAILED( hr = IDirectDraw_CreateSurface(dd->pDD, &ddsd, &dd->yuv_pool.pSurface, NULL ) ) )
+			if( FAILED( hr = IDirectDraw_CreateSurface(dd->pDD, &ddsd, &yuvp->pSurface, NULL ) ) )
 				return NULL;
 #else
-			if( FAILED( hr = IDirectDraw7_CreateSurface(dd->pDD, &ddsd, &dd->yuv_pool.pSurface, NULL ) ) )
+			if( FAILED( hr = IDirectDraw7_CreateSurface(dd->pDD, &ddsd, &yuvp->pSurface, NULL ) ) )
 				return NULL;
 #endif
-			dd->yuv_pool.format = dr->yuv_pixel_format;
-			dd->yuv_pool.width = width;
-			dd->yuv_pool.height = height;
-			return &dd->yuv_pool;
+			yuvp->format = dr->yuv_pixel_format;
+			yuvp->width = width;
+			yuvp->height = height;
+
+
+			return yuvp;
 		}
 	}
 
@@ -481,7 +517,7 @@ static DDSurface *DD_GetSurface(GF_VideoOutput *dr, u32 width, u32 height, u32 p
 	return &dd->rgb_pool;
 }
 
-static GF_Err DD_Blit(GF_VideoOutput *dr, GF_VideoSurface *video_src, GF_Window *src_wnd, GF_Window *dst_wnd, u32 *key)
+static GF_Err DD_Blit(GF_VideoOutput *dr, GF_VideoSurface *video_src, GF_Window *src_wnd, GF_Window *dst_wnd, u32 overlay_type)
 {
 	GF_VideoSurface temp_surf;
 	GF_Err e;
@@ -498,7 +534,7 @@ static GF_Err DD_Blit(GF_VideoOutput *dr, GF_VideoSurface *video_src, GF_Window
 		h = video_src->height;
 	}
 	/*get RGB or YUV pool surface*/
-	pool = DD_GetSurface(dr, w, h, key ? dd->pixelFormat : video_src->pixel_format);
+	pool = DD_GetSurface(dr, w, h, video_src->pixel_format);
 	if (!pool) return GF_IO_ERR;
 
 	temp_surf.pixel_format = pool->format;
@@ -511,6 +547,52 @@ static GF_Err DD_Blit(GF_VideoOutput *dr, GF_VideoSurface *video_src, GF_Window
 	e = DD_UnlockSurface(dd, pool->pSurface);
 	if (e) return e;
 
+	if (overlay_type) {
+		HRESULT hr;
+		RECT dst_rc, src_rc;
+		POINT pt;
+		GF_Window dst = *dst_wnd;
+		GF_Window src = *src_wnd;
+
+		src.x = src.y = 0;
+		MAKERECT(src_rc, (&src));
+		pt.x = dst.x;
+		pt.y = dst.y;
+		ClientToScreen(dd->cur_hwnd, &pt);
+		dst.x = pt.x;
+		dst.y = pt.y;
+		MAKERECT(dst_rc, (&dst));
+
+#if 1
+		if (overlay_type==1) {
+			hr = IDirectDrawSurface2_UpdateOverlay(pool->pSurface, &src_rc, dd->pPrimary, &dst_rc, DDOVER_SHOW, NULL);
+		} else {
+			DDOVERLAYFX ddofx;
+			memset(&ddofx, 0, sizeof(DDOVERLAYFX));
+			ddofx.dwSize = sizeof(DDOVERLAYFX);
+			ddofx.dckDestColorkey.dwColorSpaceLowValue = dr->overlay_color_key;
+			ddofx.dckDestColorkey.dwColorSpaceHighValue = dr->overlay_color_key;
+			hr = IDirectDrawSurface2_UpdateOverlay(pool->pSurface, &src_rc, dd->pPrimary, &dst_rc, DDOVER_SHOW | DDOVER_KEYDESTOVERRIDE, &ddofx);
+		}
+		if (FAILED(hr)) {
+			IDirectDrawSurface2_UpdateOverlay(pool->pSurface, NULL, dd->pPrimary, NULL, DDOVER_HIDE, NULL);
+		}
+#else
+		if (overlay_type==1) {
+			hr = IDirectDrawSurface_Blt(dd->pPrimary, &dst_rc, pool->pSurface, &src_rc, 0, NULL);
+		} else {
+			DDBLTFX ddfx;
+			memset(&ddfx, 0, sizeof(DDBLTFX));
+			ddfx.dwSize = sizeof(DDBLTFX);
+			ddfx.ddckDestColorkey.dwColorSpaceLowValue = dr->overlay_color_key;
+			ddfx.ddckDestColorkey.dwColorSpaceHighValue = dr->overlay_color_key;
+
+			hr = IDirectDrawSurface_Blt(dd->pPrimary, &dst_rc, pool->pSurface, &src_rc, DDBLT_WAIT | DDBLT_KEYDESTOVERRIDE, &ddfx);
+		}
+#endif
+		return FAILED(hr) ? GF_IO_ERR : GF_OK;
+	}
+
 	src_wnd_2.x = src_wnd_2.y = 0;
 	if (src_wnd) {
 		src_wnd_2.w = src_wnd->w;
@@ -520,7 +602,7 @@ static GF_Err DD_Blit(GF_VideoOutput *dr, GF_VideoSurface *video_src, GF_Window
 		src_wnd_2.h = video_src->height;
 	}
 	/*and blit surface*/
-	return DD_BlitSurface(dd, pool, &src_wnd_2, dst_wnd, key);
+	return DD_BlitSurface(dd, pool, &src_wnd_2, dst_wnd, NULL);
 }
 
 
@@ -564,11 +646,12 @@ void DD_InitYUV(GF_VideoOutput *dr)
 	u32 now, min_packed = 0xFFFFFFFF, min_planar = 0xFFFFFFFF;
 	u32 best_packed = 0, best_planar = 0;
 	Bool checkPacked = TRUE;
-	
+	const char *opt;
+
 	DDCONTEXT;
 
-	w = 320;
-	h = 240;
+	w = 720;
+	h = 576;
 
 	dd->yuv_init = 1;
 
@@ -592,7 +675,7 @@ void DD_InitYUV(GF_VideoOutput *dr)
 	free(codes);
 	/*too bad*/
 	if (!num_yuv) {
-		dr->hw_caps &= ~GF_VIDEO_HW_HAS_YUV;
+		dr->hw_caps &= ~(GF_VIDEO_HW_HAS_YUV | GF_VIDEO_HW_HAS_YUV_OVERLAY);
 		dr->yuv_pixel_format = 0;
 		return;
 	}
@@ -653,25 +736,53 @@ rem_fmt:
 	}
 	/*too bad*/
 	if (!num_yuv) {
-		dr->hw_caps &= ~GF_VIDEO_HW_HAS_YUV;
+		dr->hw_caps &= ~(GF_VIDEO_HW_HAS_YUV | GF_VIDEO_HW_HAS_YUV_OVERLAY);
 		dr->yuv_pixel_format = 0;
+		GF_LOG(GF_LOG_INFO, GF_LOG_MMIO, ("[DX Out] YUV hardware not available\n"));
 		return;
 	}
 
 	if (best_planar && (min_planar < min_packed )) {
 		dr->yuv_pixel_format = best_planar;
 	} else {
+		min_planar = min_packed;
 		dr->yuv_pixel_format = best_packed;
 	} 
-	dr->hw_caps |= GF_VIDEO_HW_HAS_YUV;
+	dr->yuv_pixel_format = GF_PIXEL_YV12;
+
+	GF_LOG(GF_LOG_INFO, GF_LOG_MMIO, ("[DX Out] Picked YUV format %s - drawn in %d ms\n", gf_4cc_to_str(dr->yuv_pixel_format), min_planar));
+	dr->hw_caps |= GF_VIDEO_HW_HAS_YUV_OVERLAY;
+
+	/*enable YUV->RGB on the backbuffer ?*/
+	opt = gf_modules_get_option((GF_BaseInterface *)dr, "Video", "EnableOffscreenYUV");
+	/*no set is the default*/
+	if (!opt) {
+		opt = "yes";
+		gf_modules_set_option((GF_BaseInterface *)dr, "Video", "EnableOffscreenYUV", "yes");
+	}
+	if (!strcmp(opt, "yes")) dr->hw_caps |= GF_VIDEO_HW_HAS_YUV;
+	
+	/*get YUV overlay key*/
+	opt = gf_modules_get_option((GF_BaseInterface *)dr, "Video", "OverlayColorKey");
+	/*no set is the default*/
+	if (!opt) {
+		opt = "0101FE";
+		gf_modules_set_option((GF_BaseInterface *)dr, "Video", "OverlayColorKey", "0101FE");
+	}
+	sscanf(opt, "%06x", &dr->overlay_color_key);
+	if (dr->overlay_color_key) dr->overlay_color_key |= 0xFF000000;
+	GF_LOG(GF_LOG_INFO, GF_LOG_MMIO, ("[DX Out] YUV->RGB enabled: %s - ColorKey enabled: %s (key %x)\n", 
+									(dr->hw_caps & GF_VIDEO_HW_HAS_YUV) ? "Yes" : "No",
+									dr->overlay_color_key ? "Yes" : "No", dr->overlay_color_key
+							));
 }
 
-GF_Err DD_SetBackBufferSize(GF_VideoOutput *dr, u32 width, u32 height)
+GF_Err DD_SetBackBufferSize(GF_VideoOutput *dr, u32 width, u32 height, Bool use_system_memory)
 {
 	DDCONTEXT;
-	if (dd->is_3D_out) return GF_BAD_PARAM;
+	if (dd->output_3d_type) return GF_BAD_PARAM;
 	if (!dd->ddraw_init) return InitDirectDraw(dr, width, height);
-	return CreateBackBuffer(dr, width, height);
+	return CreateBackBuffer(dr, width, height, use_system_memory);
 }
 
 
