@@ -27,6 +27,13 @@
 
 #include <gpac/user.h>
 
+#ifdef _WIN32_WCE
+#ifdef GPAC_USE_OGL_ES
+#endif
+#else
+#include <GL/gl.h>
+#endif
+
 #define DDCONTEXT	DDContext *dd = (DDContext *)dr->opaque;
 
 
@@ -34,12 +41,11 @@ static void RestoreWindow(DDContext *dd)
 {
 	if (!dd->NeedRestore) return;
 
-
 	dd->NeedRestore = 0;
-	if (dd->is_3D_out) {
+	if (dd->output_3d_type==1) {
+#ifndef _WIN32_WCE
 		ChangeDisplaySettings(NULL,0);
-		SetForegroundWindow(GetDesktopWindow());
-		SetForegroundWindow(dd->cur_hwnd);
+#endif
 	} else {
 #ifdef USE_DX_3
 		IDirectDraw_SetCooperativeLevel(dd->pDD, dd->cur_hwnd, DDSCL_NORMAL);
@@ -49,23 +55,28 @@ static void RestoreWindow(DDContext *dd)
 		dd->NeedRestore = 0;
 	}
 
-	SetForegroundWindow(dd->cur_hwnd);
+//	SetForegroundWindow(dd->cur_hwnd);
 	SetFocus(dd->cur_hwnd);
 }
 
-void DestroyObjects(DDContext *dd)
+void DestroyObjectsEx(DDContext *dd, Bool only_3d)
 {
-	RestoreWindow(dd);
+	if (!only_3d) {
+		RestoreWindow(dd);
 
-	SAFE_DD_RELEASE(dd->rgb_pool.pSurface);
-	memset(&dd->rgb_pool, 0, sizeof(DDSurface));
-	SAFE_DD_RELEASE(dd->yuv_pool.pSurface);
-	memset(&dd->yuv_pool, 0, sizeof(DDSurface));
+		SAFE_DD_RELEASE(dd->rgb_pool.pSurface);
+		memset(&dd->rgb_pool, 0, sizeof(DDSurface));
+		SAFE_DD_RELEASE(dd->yuv_pool.pSurface);
+		memset(&dd->yuv_pool, 0, sizeof(DDSurface));
 
-	SAFE_DD_RELEASE(dd->pPrimary);
-	SAFE_DD_RELEASE(dd->pBack);
-	SAFE_DD_RELEASE(dd->pDD);
-	dd->ddraw_init = 0;
+		SAFE_DD_RELEASE(dd->pPrimary);
+		SAFE_DD_RELEASE(dd->pBack);
+		SAFE_DD_RELEASE(dd->pDD);
+		dd->ddraw_init = 0;
+
+		/*do not destroy associated GL context*/
+		if (dd->output_3d_type==2) return;
+	}
 
 	/*delete openGL context*/
 #ifdef GPAC_USE_OGL_ES
@@ -79,7 +90,7 @@ void DestroyObjects(DDContext *dd)
 		dd->gl_HDC = 0L;
 		dd->egldpy = NULL;
 	}
-#else
+#elif !defined(_WIN32_WCE) 
 	if (dd->gl_HRC) {
 		wglMakeCurrent(dd->gl_HDC, NULL);
 		wglDeleteContext(dd->gl_HRC);
@@ -92,8 +103,14 @@ void DestroyObjects(DDContext *dd)
 #endif
 }
 
+void DestroyObjects(DDContext *dd)
+{
+	DestroyObjectsEx(dd, 0);
+}
+
 GF_Err DD_SetupOpenGL(GF_VideoOutput *dr) 
 {
+	const char *sOpt;
 	GF_Event evt;
 	DDCONTEXT
 
@@ -101,12 +118,23 @@ GF_Err DD_SetupOpenGL(GF_VideoOutput *dr)
 	EGLint major, minor;
 	EGLint n;
 	EGLConfig configs[1];
-	static int egl_atts[] = {EGL_RED_SIZE, 5, EGL_GREEN_SIZE, 5, EGL_BLUE_SIZE, 5, 
-				/*alpha for compositeTexture*/
-				EGL_ALPHA_SIZE,     1,
-                EGL_DEPTH_SIZE,     16,
-                EGL_STENCIL_SIZE,   EGL_DONT_CARE,
-                EGL_NONE};
+	u32 nb_bits;
+	u32 i=0;
+	static int egl_atts[20];
+
+	sOpt = gf_modules_get_option((GF_BaseInterface *)dr, "Video", "GLNbBitsPerComponent");
+	nb_bits = sOpt ? atoi(sOpt) : 5;
+
+	egl_atts[i++] = EGL_RED_SIZE; egl_atts[i++] = nb_bits;
+	egl_atts[i++] = EGL_GREEN_SIZE; egl_atts[i++] = nb_bits;
+	egl_atts[i++] = EGL_BLUE_SIZE; egl_atts[i++] = nb_bits;
+	/*alpha for compositeTexture*/
+	egl_atts[i++] = EGL_ALPHA_SIZE; egl_atts[i++] = 1;
+	sOpt = gf_modules_get_option((GF_BaseInterface *)dr, "Video", "GLNbBitsDepth");
+	nb_bits = sOpt ? atoi(sOpt) : 5;
+	egl_atts[i++] = EGL_DEPTH_SIZE; egl_atts[i++] = nb_bits;
+	egl_atts[i++] = EGL_STENCIL_SIZE; egl_atts[i++] = EGL_DONT_CARE;
+	egl_atts[i++] = EGL_NONE;
 
 	/*already setup*/
 	DestroyObjects(dd);
@@ -130,25 +158,37 @@ GF_Err DD_SetupOpenGL(GF_VideoOutput *dr)
 		dd->surface = 0L;
 		return GF_IO_ERR;
 	}
-#else
+#elif !defined(_WIN32_WCE)
     PIXELFORMATDESCRIPTOR pfd; 
     s32 pixelformat; 
 
 	/*already setup*/
 //	if (dd->gl_HRC) return GF_OK;
-	DestroyObjects(dd);
+	DestroyObjectsEx(dd, (dd->output_3d_type==1) ? 0 : 1);
 
-	dd->gl_HDC = GetDC(dd->cur_hwnd);
+	if (dd->output_3d_type==2) {
+		dd->gl_HDC = GetDC(dd->gl_hwnd);
+	} else {
+		dd->gl_HDC = GetDC(dd->cur_hwnd);
+	}
 	if (!dd->gl_HDC) return GF_IO_ERR;
 
     memset(&pfd, 0, sizeof(pfd));
     pfd.nSize = sizeof(PIXELFORMATDESCRIPTOR);
     pfd.nVersion = 1;
-    pfd.dwFlags = PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER;
+    pfd.dwFlags = PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL;
+	if (dd->gl_double_buffer) 
+		pfd.dwFlags |= PFD_DOUBLEBUFFER;
+	else {
+		sOpt = gf_modules_get_option((GF_BaseInterface *)dr, "Video", "UseGLDoubleBuffering");
+		if (sOpt && !strcmp(sOpt, "yes")) pfd.dwFlags |= PFD_DOUBLEBUFFER;
+	}
+
     pfd.dwLayerMask = PFD_MAIN_PLANE;
     pfd.iPixelType = PFD_TYPE_RGBA;
     pfd.cColorBits = 32;
-	pfd.cDepthBits = 32;
+	sOpt = gf_modules_get_option((GF_BaseInterface *)dr, "Video", "GLNbBitsDepth");
+	pfd.cDepthBits = sOpt ? atoi(sOpt) : 16;
 	/*we need alpha support for composite textures...*/
 	pfd.cAlphaBits = 8;
     if ( (pixelformat = ChoosePixelFormat(dd->gl_HDC, &pfd)) == FALSE ) return GF_IO_ERR; 
@@ -158,31 +198,30 @@ GF_Err DD_SetupOpenGL(GF_VideoOutput *dr)
 	if (!dd->gl_HRC) return GF_IO_ERR;
 	if (!wglMakeCurrent(dd->gl_HDC, dd->gl_HRC)) return GF_IO_ERR;
 #endif
-	evt.type = GF_EVENT_VIDEO_SETUP;
-	dr->on_event(dr->evt_cbk_hdl, &evt);	
+	if (dd->output_3d_type==1) {
+		evt.type = GF_EVENT_VIDEO_SETUP;
+		evt.setup.opengl_mode = 1;
+		dr->on_event(dr->evt_cbk_hdl, &evt);	
+	}
 	return GF_OK;
 }
 
 
 
 
-GF_Err DD_Setup(GF_VideoOutput *dr, void *os_handle, void *os_display, u32 init_flags, GF_GLConfig *cfg)
+GF_Err DD_Setup(GF_VideoOutput *dr, void *os_handle, void *os_display, u32 init_flags)
 {
 	RECT rc;
 	DDCONTEXT
 	dd->os_hwnd = (HWND) os_handle;
 	
-	if (init_flags & (GF_TERM_NO_VISUAL_THREAD | GF_TERM_NO_REGULATION) ) dd->systems_memory = 2;
+	//if (init_flags & (GF_TERM_NO_VISUAL_THREAD | GF_TERM_NO_REGULATION) ) dd->systems_memory = 2;
 	DD_SetupWindow(dr, init_flags);
 	/*fatal error*/
 	if (!dd->os_hwnd) return GF_IO_ERR;
 	dd->cur_hwnd = dd->os_hwnd;
 
-	if (cfg) {
-		dd->is_3D_out = 1;
-		return GF_OK;
-	}
-	dd->is_3D_out = 0;
+	dd->output_3d_type = 0;
 	GetWindowRect(dd->cur_hwnd, &rc);
 	return InitDirectDraw(dr, rc.right - rc.left, rc.bottom - rc.top);
 }
@@ -190,6 +229,9 @@ GF_Err DD_Setup(GF_VideoOutput *dr, void *os_handle, void *os_display, u32 init_
 static void DD_Shutdown(GF_VideoOutput *dr)
 {
 	DDCONTEXT
+
+	/*force destroy of opengl*/
+	if (dd->output_3d_type) dd->output_3d_type = 1;
 	DestroyObjects(dd);
 
 	DD_ShutdownWindow(dr);
@@ -227,11 +269,9 @@ static GF_Err DD_SetFullScreen(GF_VideoOutput *dr, Bool bOn, u32 *outWidth, u32
 	dd->timer = 0;
 	ShowWindow(dd->cur_hwnd, SW_HIDE);
 	dd->cur_hwnd = dd->fullscreen ? dd->fs_hwnd : dd->os_hwnd;
-	ShowWindow(dd->cur_hwnd, SW_SHOWNORMAL);
-	SetForegroundWindow(dd->cur_hwnd);
+	ShowWindow(dd->cur_hwnd, SW_SHOW);
 
-
-	if (dd->is_3D_out) {
+	if (dd->output_3d_type==1) {
 		DEVMODE settings;
 		e = GF_OK;
 		/*Setup FS*/
@@ -241,11 +281,9 @@ static GF_Err DD_SetFullScreen(GF_VideoOutput *dr, Bool bOn, u32 *outWidth, u32
 				dd->fs_width = MaxWidth;
 				dd->fs_height = MaxHeight;
 			}
+			SetWindowPos(dd->cur_hwnd, NULL, 0, 0, dd->fs_width, dd->fs_height, SWP_NOZORDER | SWP_SHOWWINDOW | SWP_ASYNCWINDOWPOS);
 
-			/*force size change (we do it whether we own or not the window)*/
-			SetWindowPos(dd->cur_hwnd, NULL, 0, 0, dd->fs_width, dd->fs_height, SWP_NOZORDER | SWP_SHOWWINDOW);
-			SetForegroundWindow(dd->cur_hwnd);
-
+#ifndef _WIN32_WCE
 			memset(&settings, 0, sizeof(DEVMODE));
 			settings.dmSize = sizeof(DEVMODE);
 			settings.dmPelsWidth = dd->fs_width;
@@ -255,8 +293,9 @@ static GF_Err DD_SetFullScreen(GF_VideoOutput *dr, Bool bOn, u32 *outWidth, u32
 				GF_LOG(GF_LOG_ERROR, GF_LOG_MMIO, ("[DirectDraw] cannot change display settings\n"));
 				e = GF_IO_ERR;
 			} 
-
 			dd->NeedRestore = 1;
+#endif
+
 			dd->fs_store_width = dd->fs_width;
 			dd->fs_store_height = dd->fs_height;
 		}
@@ -275,19 +314,20 @@ static GF_Err DD_SetFullScreen(GF_VideoOutput *dr, Bool bOn, u32 *outWidth, u32
 		*outWidth = dd->store_width;
 		*outHeight = dd->store_height;
 	}
+	SetForegroundWindow(dd->cur_hwnd);
+	SetFocus(dd->cur_hwnd);
 	return e;
 }
 
 
-static GF_Err DD_Flush(GF_VideoOutput *dr, GF_Window *dest)
+GF_Err DD_Flush(GF_VideoOutput *dr, GF_Window *dest)
 {
 	RECT rc;
 	HRESULT hr;
 	DDCONTEXT;
 
-
 	if (!dd) return GF_BAD_PARAM;
-	if (dd->is_3D_out) {
+	if (dd->output_3d_type==1) {
 #ifdef GPAC_USE_OGL_ES
 		if (dd->surface) eglSwapBuffers(dd->egldpy, dd->surface);
 #else
@@ -295,6 +335,7 @@ static GF_Err DD_Flush(GF_VideoOutput *dr, GF_Window *dest)
 #endif
 		return GF_OK;
 	}
+
 	if (!dd->ddraw_init) return GF_BAD_PARAM;
 
 	if (!dd->fullscreen && dd->windowless) {
@@ -309,6 +350,7 @@ static GF_Err DD_Flush(GF_VideoOutput *dr, GF_Window *dest)
 		dr->LockOSContext(dr, 0);
 		return GF_OK;
 	}
+	
 	if (dest) {
 		POINT pt;
 		pt.x = dest->x;
@@ -317,9 +359,9 @@ static GF_Err DD_Flush(GF_VideoOutput *dr, GF_Window *dest)
 		dest->x = pt.x;
 		dest->y = pt.y;
 		MAKERECT(rc, dest);
-		hr = IDirectDrawSurface_Blt(dd->pPrimary, &rc, dd->pBack, NULL, DDBLT_WAIT, NULL );
+		hr = IDirectDrawSurface_Blt(dd->pPrimary, &rc, dd->pBack, NULL, DDBLT_WAIT, NULL);
 	} else {
-		hr = IDirectDrawSurface_Blt(dd->pPrimary, NULL, dd->pBack, NULL, DDBLT_WAIT, NULL );
+		hr = IDirectDrawSurface_Blt(dd->pPrimary, NULL, dd->pBack, NULL, DDBLT_WAIT, NULL);
 	}
 	if (hr == DDERR_SURFACELOST) {
 		IDirectDrawSurface_Restore(dd->pPrimary);
@@ -406,7 +448,7 @@ static void *NewDXVideoOutput()
 
     driv->max_screen_width = GetSystemMetrics(SM_CXSCREEN);
     driv->max_screen_height = GetSystemMetrics(SM_CYSCREEN);
-	driv->hw_caps = GF_VIDEO_HW_HAS_OPENGL;
+	driv->hw_caps = GF_VIDEO_HW_OPENGL | GF_VIDEO_HW_OPENGL_OFFSCREEN | GF_VIDEO_HW_OPENGL_OFFSCREEN_ALPHA;
 
 	DD_SetupDDraw(driv);
 
