@@ -40,12 +40,22 @@ static const GF_VideoOutput *the_video_output = NULL;
 
 void DD_SetCursor(GF_VideoOutput *dr, u32 cursor_type);
 
+
+#ifdef _WIN32_WCE
+static void DD_GetCoordinates(DWORD lParam, GF_Event *evt)
+{
+	evt->mouse.x = LOWORD(lParam);
+	evt->mouse.y = HIWORD(lParam);
+}
+#else
+
 static void DD_GetCoordinates(DWORD lParam, GF_Event *evt)
 {
 	POINTS pt = MAKEPOINTS(lParam);
 	evt->mouse.x = pt.x;
 	evt->mouse.y = pt.y;
 }
+#endif
 
 static void w32_translate_key(u32 wParam, u32 lParam, GF_EventKey *evt)
 {
@@ -67,9 +77,6 @@ static void w32_translate_key(u32 wParam, u32 lParam, GF_EventKey *evt)
 	case VK_KANJI: evt->key_code = GF_KEY_KANJIMODE; break;
 	case VK_ESCAPE: evt->key_code = GF_KEY_ESCAPE; break;
 	case VK_CONVERT: evt->key_code = GF_KEY_CONVERT; break;
-	case VK_NONCONVERT: evt->key_code = GF_KEY_NONCONVERT; break;
-	case VK_ACCEPT: evt->key_code = GF_KEY_ACCEPT; break;
-	case VK_MODECHANGE: evt->key_code = GF_KEY_MODECHANGE; break;
 	case VK_SPACE: evt->key_code = GF_KEY_SPACE; break;
 	case VK_PRIOR: evt->key_code = GF_KEY_PAGEUP; break;
 	case VK_NEXT: evt->key_code = GF_KEY_PAGEDOWN; break;
@@ -88,6 +95,12 @@ static void w32_translate_key(u32 wParam, u32 lParam, GF_EventKey *evt)
 	case VK_DELETE: evt->key_code = GF_KEY_DEL; break;
 	case VK_HELP: evt->key_code = GF_KEY_HELP; break;
 
+#ifndef _WIN32_WCE
+	case VK_NONCONVERT: evt->key_code = GF_KEY_NONCONVERT; break;
+	case VK_ACCEPT: evt->key_code = GF_KEY_ACCEPT; break;
+	case VK_MODECHANGE: evt->key_code = GF_KEY_MODECHANGE; break;
+#endif
+
 /*	case '!': evt->key_code = GF_KEY_EXCLAMATION; break;
 	case '"': evt->key_code = GF_KEY_QUOTATION; break;
 	case '#': evt->key_code = GF_KEY_NUMBER; break;
@@ -304,6 +317,7 @@ void release_mouse(DDContext *ctx, HWND hWnd, GF_VideoOutput *vout)
 
 LRESULT APIENTRY DD_WindowProc(HWND hWnd, UINT msg, UINT wParam, LONG lParam)
 {
+	Bool ret = 0;
 	GF_Event evt;
 	DDContext *ctx;
 	GF_VideoOutput *vout = (GF_VideoOutput *) GetWindowLong(hWnd, GWL_USERDATA);
@@ -317,14 +331,20 @@ LRESULT APIENTRY DD_WindowProc(HWND hWnd, UINT msg, UINT wParam, LONG lParam)
 		evt.type = GF_EVENT_SIZE;
 		evt.size.width = LOWORD(lParam);
 		evt.size.height = HIWORD(lParam);
-		vout->on_event(vout->evt_cbk_hdl, &evt);
+		ret = vout->on_event(vout->evt_cbk_hdl, &evt);
+		break;
+	case WM_MOVE:
+		evt.type = GF_EVENT_MOVE;
+		evt.move.x = LOWORD(lParam);
+		evt.move.y = HIWORD(lParam);
+		ret = vout->on_event(vout->evt_cbk_hdl, &evt);
 		break;
 	case WM_CLOSE:
 		if (hWnd==ctx->os_hwnd) {
 			evt.type = GF_EVENT_QUIT;
-			vout->on_event(vout->evt_cbk_hdl, &evt);
+			ret = vout->on_event(vout->evt_cbk_hdl, &evt);
 		}
-		return 1;
+		break;
 	case WM_DESTROY:
 		if (ctx->owns_hwnd || (hWnd==ctx->fs_hwnd)) {
 			PostQuitMessage (0);
@@ -335,14 +355,11 @@ LRESULT APIENTRY DD_WindowProc(HWND hWnd, UINT msg, UINT wParam, LONG lParam)
 		}
 		break;
 	case WM_ACTIVATE:
-#if 1
-		if (ctx->fullscreen && (LOWORD(wParam)==WA_INACTIVE) && (hWnd==ctx->fs_hwnd)) {
+		if ((ctx->output_3d_type!=2) && ctx->fullscreen && (LOWORD(wParam)==WA_INACTIVE) && (hWnd==ctx->fs_hwnd)) {
 			evt.type = GF_EVENT_SHOWHIDE;
-			vout->on_event(vout->evt_cbk_hdl, &evt);
+			ret = vout->on_event(vout->evt_cbk_hdl, &evt);
 		}
-#endif
 		break;
-
 	case WM_SETCURSOR:
 		if (ctx->cur_hwnd==hWnd) DD_SetCursor(vout, ctx->cursor_type);
 		return 1;
@@ -366,7 +383,7 @@ LRESULT APIENTRY DD_WindowProc(HWND hWnd, UINT msg, UINT wParam, LONG lParam)
 			DD_SetCursor(vout, (ctx->cursor_type==GF_CURSOR_HIDE) ? ctx->cursor_type_backup : ctx->cursor_type);
 			evt.type = GF_EVENT_MOUSEMOVE;
 			DD_GetCoordinates(lParam, &evt);
-			vout->on_event(vout->evt_cbk_hdl, &evt);
+			ret = vout->on_event(vout->evt_cbk_hdl, &evt);
 			mouse_start_timer(ctx, hWnd, vout);
 		}
 		break;
@@ -389,7 +406,7 @@ LRESULT APIENTRY DD_WindowProc(HWND hWnd, UINT msg, UINT wParam, LONG lParam)
 		evt.type = GF_EVENT_MOUSEDOWN;
 		DD_GetCoordinates(lParam, &evt);
 		evt.mouse.button = GF_MOUSE_LEFT;
-		vout->on_event(vout->evt_cbk_hdl, &evt);
+		ret = vout->on_event(vout->evt_cbk_hdl, &evt);
 		if (!ctx->has_focus && (hWnd==ctx->os_hwnd)) {
 			ctx->has_focus = 1;
 			SetFocus(ctx->os_hwnd);
@@ -400,7 +417,7 @@ LRESULT APIENTRY DD_WindowProc(HWND hWnd, UINT msg, UINT wParam, LONG lParam)
 		evt.type = GF_EVENT_MOUSEUP;
 		DD_GetCoordinates(lParam, &evt);
 		evt.mouse.button = GF_MOUSE_LEFT;
-		vout->on_event(vout->evt_cbk_hdl, &evt);
+		ret = vout->on_event(vout->evt_cbk_hdl, &evt);
 		break;
 	case WM_RBUTTONDOWN:
 	case WM_RBUTTONDBLCLK:
@@ -408,7 +425,7 @@ LRESULT APIENTRY DD_WindowProc(HWND hWnd, UINT msg, UINT wParam, LONG lParam)
 		evt.type = GF_EVENT_MOUSEDOWN;
 		DD_GetCoordinates(lParam, &evt);
 		evt.mouse.button = GF_MOUSE_RIGHT;
-		vout->on_event(vout->evt_cbk_hdl, &evt);
+		ret = vout->on_event(vout->evt_cbk_hdl, &evt);
 		if (!ctx->has_focus && (hWnd==ctx->os_hwnd)) {
 			ctx->has_focus = 1;
 			SetFocus(ctx->os_hwnd);
@@ -419,7 +436,7 @@ LRESULT APIENTRY DD_WindowProc(HWND hWnd, UINT msg, UINT wParam, LONG lParam)
 		evt.type = GF_EVENT_MOUSEUP;
 		DD_GetCoordinates(lParam, &evt);
 		evt.mouse.button = GF_MOUSE_RIGHT;
-		vout->on_event(vout->evt_cbk_hdl, &evt);
+		ret = vout->on_event(vout->evt_cbk_hdl, &evt);
 		mouse_start_timer(ctx, hWnd, vout);
 		break;
 	case WM_MBUTTONDOWN:
@@ -428,7 +445,7 @@ LRESULT APIENTRY DD_WindowProc(HWND hWnd, UINT msg, UINT wParam, LONG lParam)
 		evt.type = GF_EVENT_MOUSEDOWN;
 		evt.mouse.button = GF_MOUSE_MIDDLE;
 		DD_GetCoordinates(lParam, &evt);
-		vout->on_event(vout->evt_cbk_hdl, &evt);
+		ret = vout->on_event(vout->evt_cbk_hdl, &evt);
 		if (!ctx->has_focus && (hWnd==ctx->os_hwnd)) {
 			ctx->has_focus = 1;
 			SetFocus(ctx->os_hwnd);
@@ -439,7 +456,7 @@ LRESULT APIENTRY DD_WindowProc(HWND hWnd, UINT msg, UINT wParam, LONG lParam)
 		evt.type = GF_EVENT_MOUSEUP;
 		DD_GetCoordinates(lParam, &evt);
 		evt.mouse.button = GF_MOUSE_MIDDLE;
-		vout->on_event(vout->evt_cbk_hdl, &evt);
+		ret = vout->on_event(vout->evt_cbk_hdl, &evt);
 		mouse_start_timer(ctx, hWnd, vout);
 		break;
 	case WM_MOUSEWHEEL: 
@@ -448,10 +465,10 @@ LRESULT APIENTRY DD_WindowProc(HWND hWnd, UINT msg, UINT wParam, LONG lParam)
 			evt.type = GF_EVENT_MOUSEWHEEL;
 			DD_GetCoordinates(lParam, &evt);
 			evt.mouse.wheel_pos = FLT2FIX( ((Float) (s16) HIWORD(wParam)) / WHEEL_DELTA );
-			vout->on_event(vout->evt_cbk_hdl, &evt);
+			ret = vout->on_event(vout->evt_cbk_hdl, &evt);
 			mouse_start_timer(ctx, hWnd, vout);
 		}
-		return 1;
+		break;
 
 	/*there's a bug on alt state (we miss one event)*/
 	case WM_SYSKEYDOWN:
@@ -460,15 +477,23 @@ LRESULT APIENTRY DD_WindowProc(HWND hWnd, UINT msg, UINT wParam, LONG lParam)
 	case WM_KEYDOWN:
 		w32_translate_key(wParam, lParam, &evt.key);
 		evt.type = ((msg==WM_SYSKEYDOWN) || (msg==WM_KEYDOWN)) ? GF_EVENT_KEYDOWN : GF_EVENT_KEYUP;
-		vout->on_event(vout->evt_cbk_hdl, &evt);
+		ret = vout->on_event(vout->evt_cbk_hdl, &evt);
 		break;
 
 	case WM_CHAR:
 		evt.type = GF_EVENT_TEXTINPUT;
 		evt.character.unicode_char = wParam;
-		vout->on_event(vout->evt_cbk_hdl, &evt);
+		ret = vout->on_event(vout->evt_cbk_hdl, &evt);
 		break;
 	}
+
+	if (ret) return 1;
+
+/*
+	if (ctx->parent_wnd && (hWnd==ctx->os_hwnd) ) {
+		return SendMessage(ctx->parent_wnd, msg, wParam, lParam);
+	}
+*/
 	return DefWindowProc (hWnd, msg, wParam, lParam);
 }
 
@@ -486,6 +511,10 @@ typedef BOOL (WINAPI* typSetLayeredWindowAttributes)(HWND,COLORREF,BYTE,DWORD);
 
 static void SetWindowless(GF_VideoOutput *vout, HWND hWnd)
 {
+#ifdef _WIN32_WCE
+	GF_LOG(GF_LOG_ERROR, GF_LOG_MMIO, ("[DX Out] Windowloess mode not supported on Window CE\n"));
+	return;
+#else
 	const char *opt;
 	u32 a, r, g, b;
 	COLORREF ckey;
@@ -510,9 +539,9 @@ static void SetWindowless(GF_VideoOutput *vout, HWND hWnd)
 	SetWindowLong(hWnd, GWL_EXSTYLE, GetWindowLong(hWnd, GWL_EXSTYLE) | WS_EX_LAYERED);
 
 	/*get background ckey*/
-	opt = gf_modules_get_option((GF_BaseInterface *)vout, "Rendering", "ColorKey");
+	opt = gf_modules_get_option((GF_BaseInterface *)vout, "Compositor", "ColorKey");
 	if (!opt) {
-		gf_modules_set_option((GF_BaseInterface *)vout, "Rendering", "ColorKey", "FFFEFEFE");
+		gf_modules_set_option((GF_BaseInterface *)vout, "Compositor", "ColorKey", "FFFEFEFE");
 		opt = "FFFEFEFE";
 	}
 
@@ -524,31 +553,37 @@ static void SetWindowless(GF_VideoOutput *vout, HWND hWnd)
 		_SetLayeredWindowAttributes(hWnd, ckey, 0, LWA_COLORKEY);
 
 	GF_LOG(GF_LOG_INFO, GF_LOG_MMIO, ("[DX Out] Using color key %s\n", opt));
+#endif
 }
 
- 
 
-u32 DD_WindowThread(void *par)
+Bool DD_InitWindows(GF_VideoOutput *vout, DDContext *ctx)
 {
 	u32 flags;
+#ifndef _WIN32_WCE
 	RECT rc;
-	MSG msg;
+#endif
 	WNDCLASS wc;
 	HINSTANCE hInst;
-	GF_VideoOutput *vout = par;
-	DDContext *ctx = (DDContext *)vout->opaque;
-
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_CORE, ("[DirectXOutput] Entering thread ID %d\n", gf_th_id() ));
 
+#ifndef _WIN32_WCE
 	hInst = GetModuleHandle("gm_dx_hw.dll");
+#else
+	hInst = GetModuleHandle(_T("gm_dx_hw.dll"));
+#endif
+
 	memset(&wc, 0, sizeof(WNDCLASS));
+#ifndef _WIN32_WCE
 	wc.style = CS_BYTEALIGNWINDOW;
+	wc.hIcon = LoadIcon (NULL, IDI_APPLICATION);
+	wc.lpszClassName = "GPAC DirectDraw Output";
+#else
+	wc.lpszClassName = _T("GPAC DirectDraw Output");
+#endif
 	wc.hInstance = hInst;
 	wc.lpfnWndProc = DD_WindowProc;
-	wc.hIcon = LoadIcon (NULL, IDI_APPLICATION);
 	wc.hCursor = LoadCursor (NULL, IDC_ARROW);
 	wc.hbrBackground = (HBRUSH)GetStockObject (BLACK_BRUSH);
-	wc.lpszClassName = "GPAC DirectDraw Output";
 	RegisterClass (&wc);
 
 	flags = ctx->switch_res;
@@ -557,11 +592,13 @@ u32 DD_WindowThread(void *par)
 	if (!ctx->os_hwnd) {
 		if (flags & GF_TERM_WINDOWLESS) ctx->windowless = 1;
 
+#ifdef _WIN32_WCE
+		ctx->os_hwnd = CreateWindow(_T("GPAC DirectDraw Output"), _T("GPAC DirectDraw Output"), WS_POPUP, 0, 0, 120, 100, NULL, NULL, hInst, NULL);
+#else
 		ctx->os_hwnd = CreateWindow("GPAC DirectDraw Output", "GPAC DirectDraw Output", ctx->windowless ? WS_POPUP : WS_OVERLAPPEDWINDOW, 0, 0, 120, 100, NULL, NULL, hInst, NULL);
-
+#endif
 		if (ctx->os_hwnd == NULL) {
-			ctx->th_state = 2;
-			return 1;
+			return 0;
 		}
 		if (flags & GF_TERM_INIT_HIDE) {
 			ShowWindow(ctx->os_hwnd, SW_HIDE);
@@ -571,23 +608,40 @@ u32 DD_WindowThread(void *par)
 		}
 
 		/*get border & title bar sizes*/
+#ifdef _WIN32_WCE
+		ctx->off_w = 0;
+		ctx->off_h = 0;
+#else
 		rc.left = rc.top = 0;
 		rc.right = rc.bottom = 100;
 		AdjustWindowRect(&rc, WS_OVERLAPPEDWINDOW, 0);
 		ctx->off_w = rc.right - rc.left - 100;
 		ctx->off_h = rc.bottom - rc.top - 100;
+#endif
 		ctx->owns_hwnd = 1;
 
 		if (ctx->windowless) SetWindowless(vout, ctx->os_hwnd);
 	}
 
+#ifdef _WIN32_WCE
+	ctx->fs_hwnd = CreateWindow(_T("GPAC DirectDraw Output"), _T("GPAC DirectDraw FS Output"), WS_POPUP, 0, 0, 120, 100, NULL, NULL, hInst, NULL);
+#else
 	ctx->fs_hwnd = CreateWindow("GPAC DirectDraw Output", "GPAC DirectDraw FS Output", WS_POPUP, 0, 0, 120, 100, NULL, NULL, hInst, NULL);
+#endif
 	if (!ctx->fs_hwnd) {
-		ctx->th_state = 2;
-		return 1;
+		return 0;
 	}
 	ShowWindow(ctx->fs_hwnd, SW_HIDE);
-	ctx->th_state = 1;
+
+#ifdef _WIN32_WCE
+	ctx->gl_hwnd = CreateWindow(_T("GPAC DirectDraw Output"), _T("GPAC OpenGL Offscreen"), WS_POPUP, 0, 0, 120, 100, NULL, NULL, hInst, NULL);
+#else
+	ctx->gl_hwnd = CreateWindow("GPAC DirectDraw Output", "GPAC OpenGL Offscreen", WS_POPUP, 0, 0, 120, 100, NULL, NULL, hInst, NULL);
+#endif
+	if (!ctx->gl_hwnd) {
+		return 0;
+	}
+	ShowWindow(ctx->gl_hwnd, SW_HIDE);
 
 	/*if visible set focus*/
 	if (!ctx->switch_res) SetFocus(ctx->os_hwnd);
@@ -603,9 +657,24 @@ u32 DD_WindowThread(void *par)
 	ctx->curs_collide = LoadCursor(hInst, MAKEINTRESOURCE(IDC_COLLIDE));
 	ctx->cursor_type = GF_CURSOR_NORMAL;
 
-	while (GetMessage (&(msg), NULL, 0, 0)) {
-		TranslateMessage (&(msg));
-		DispatchMessage (&(msg));
+	return 1;
+}
+
+u32 DD_WindowThread(void *par)
+{
+	MSG msg;
+
+	GF_VideoOutput *vout = par;
+	DDContext *ctx = (DDContext *)vout->opaque;
+
+	GF_LOG(GF_LOG_DEBUG, GF_LOG_CORE, ("[DirectXOutput] Entering thread ID %d\n", gf_th_id() ));
+
+	if (DD_InitWindows(vout, ctx)) {
+		ctx->th_state = 1;
+		while (GetMessage (&(msg), NULL, 0, 0)) {
+			TranslateMessage (&(msg));
+			DispatchMessage (&(msg));
+		}
 	}
 	ctx->th_state = 2;
 	return 0;
@@ -622,35 +691,51 @@ void DD_SetupWindow(GF_VideoOutput *dr, u32 flags)
 			ctx->orig_wnd_proc = GetWindowLong(ctx->os_hwnd, GWL_WNDPROC);
 			SetWindowLong(ctx->os_hwnd, GWL_WNDPROC, (DWORD) DD_WindowProc);
 		}
+		ctx->parent_wnd = GetParent(ctx->os_hwnd);
 	}
 	ctx->switch_res = flags;
-	/*create our event thread - since we always have a dedicated window for fullscreen, we need that
-	even when a window is passed to us*/
-	ctx->th = gf_th_new();
+
+	/*create event thread*/
+	ctx->th = gf_th_new("DirectX Video");
 	gf_th_run(ctx->th, DD_WindowThread, dr);
 	while (!ctx->th_state) gf_sleep(2);
 
 	if (!the_video_output) the_video_output = dr;
 }
 
+static void dd_closewindow(HWND hWnd)
+{
+	PostMessage(hWnd, WM_DESTROY, 0, 0);
+}
+
 void DD_ShutdownWindow(GF_VideoOutput *dr)
 {
 	DDContext *ctx = (DDContext *)dr->opaque;
 
 	if (ctx->owns_hwnd) {
-		PostMessage(ctx->os_hwnd, WM_DESTROY, 0, 0);
+		dd_closewindow(ctx->os_hwnd);
 	} else if (ctx->orig_wnd_proc) {
 		/*restore window proc*/
-		//SetWindowLong(ctx->os_hwnd, GWL_WNDPROC, ctx->orig_wnd_proc);
+		SetWindowLong(ctx->os_hwnd, GWL_WNDPROC, ctx->orig_wnd_proc);
 		ctx->orig_wnd_proc = 0L;
 	}
-	PostMessage(ctx->fs_hwnd, WM_DESTROY, 0, 0);
-	while (ctx->th_state!=2) gf_sleep(10);
-	UnregisterClass("GPAC DirectDraw Output", GetModuleHandle("gm_dx_hw.dll"));
+	dd_closewindow(ctx->fs_hwnd);
+	dd_closewindow(ctx->gl_hwnd);
+
+	while (ctx->th_state!=2) 
+		gf_sleep(10);
+
 	gf_th_del(ctx->th);
 	ctx->th = NULL;
+
+#ifdef _WIN32_WCE
+	UnregisterClass(_T("GPAC DirectDraw Output"), GetModuleHandle(_T("gm_dx_hw.dll")) );
+#else
+	UnregisterClass("GPAC DirectDraw Output", GetModuleHandle("gm_dx_hw.dll"));
+#endif
 	ctx->os_hwnd = NULL;
 	ctx->fs_hwnd = NULL;
+	ctx->gl_hwnd = NULL;
 	the_video_output = NULL;
 }
 
@@ -690,19 +775,25 @@ HWND DD_GetGlobalHWND()
 	return ((DDContext*)the_video_output->opaque)->os_hwnd;
 }
 
-
-
+/*Note: all calls to SetWindowPos are made in a non-blocking way using SWP_ASYNCWINDOWPOS. This avoids deadlocks
+when the compositor request a size change and the DX window thread has grabbed the main compositor mutex. 
+This typically happens when switching playlist items as fast as possible*/
 GF_Err DD_ProcessEvent(GF_VideoOutput*dr, GF_Event *evt)
 {
 	DDContext *ctx = (DDContext *)dr->opaque;
-	if (!evt) return GF_OK;
+
+	if (!evt) {
+		return GF_OK;
+	}
 
 	switch (evt->type) {
 	case GF_EVENT_SET_CURSOR:
 		DD_SetCursor(dr, evt->cursor.cursor_type);
 		break;
 	case GF_EVENT_SET_CAPTION:
+#ifndef _WIN32_WCE
 		if (evt->caption.caption) SetWindowText(ctx->os_hwnd, evt->caption.caption);
+#endif
 		break;
 	case GF_EVENT_MOVE:
 		if (evt->move.relative == 2) {
@@ -717,15 +808,15 @@ GF_Err DD_ProcessEvent(GF_VideoOutput*dr, GF_Event *evt)
 			if (evt->move.align_y==1) y = (fsh - ctx->height) / 2;
 			else if (evt->move.align_y==2) y = fsh - ctx->height;
 
-			SetWindowPos(ctx->os_hwnd, NULL, x, y, 0, 0, SWP_NOZORDER | SWP_NOSIZE);
+			SetWindowPos(ctx->os_hwnd, NULL, x, y, 0, 0, SWP_NOZORDER | SWP_NOSIZE | SWP_ASYNCWINDOWPOS);
 		}
 		else if (evt->move.relative) {
 			POINT pt;
 			pt.x = pt.y = 0;
 			MapWindowPoints(ctx->os_hwnd, NULL, &pt, 1);
-			SetWindowPos(ctx->os_hwnd, NULL, evt->move.x + pt.x, evt->move.y + pt.y, 0, 0, SWP_NOZORDER | SWP_NOSIZE);
+			SetWindowPos(ctx->os_hwnd, NULL, evt->move.x + pt.x, evt->move.y + pt.y, 0, 0, SWP_NOZORDER | SWP_NOSIZE | SWP_ASYNCWINDOWPOS);
 		} else {
-			SetWindowPos(ctx->os_hwnd, NULL, evt->move.x, evt->move.y, 0, 0, SWP_NOZORDER | SWP_NOSIZE);
+			SetWindowPos(ctx->os_hwnd, NULL, evt->move.x, evt->move.y, 0, 0, SWP_NOZORDER | SWP_NOSIZE | SWP_ASYNCWINDOWPOS);
 		}
 		break;
 	case GF_EVENT_SHOWHIDE:
@@ -735,9 +826,9 @@ GF_Err DD_ProcessEvent(GF_VideoOutput*dr, GF_Event *evt)
 	case GF_EVENT_SIZE:
 		if (ctx->owns_hwnd) {
 			if (ctx->windowless)
-				SetWindowPos(ctx->os_hwnd, NULL, 0, 0, evt->size.width, evt->size.height, SWP_NOZORDER | SWP_NOMOVE);
+				SetWindowPos(ctx->os_hwnd, NULL, 0, 0, evt->size.width, evt->size.height, SWP_NOZORDER | SWP_NOMOVE | SWP_ASYNCWINDOWPOS);
 			else 
-				SetWindowPos(ctx->os_hwnd, NULL, 0, 0, evt->size.width + ctx->off_w, evt->size.height + ctx->off_h, SWP_NOZORDER | SWP_NOMOVE);
+				SetWindowPos(ctx->os_hwnd, NULL, 0, 0, evt->size.width + ctx->off_w, evt->size.height + ctx->off_h, SWP_NOZORDER | SWP_NOMOVE | SWP_ASYNCWINDOWPOS);
 
 			if (ctx->fullscreen) {
 				ctx->store_width = evt->size.width;
@@ -747,12 +838,25 @@ GF_Err DD_ProcessEvent(GF_VideoOutput*dr, GF_Event *evt)
 		break;
 	/*HW setup*/
 	case GF_EVENT_VIDEO_SETUP:
-		if (ctx->is_3D_out) {
-			ctx->width = evt->size.width;
-			ctx->height = evt->size.height;
+		switch (evt->setup.opengl_mode) {
+		case 0:
+			ctx->output_3d_type = 0;
+			return DD_SetBackBufferSize(dr, evt->setup.width, evt->setup.height, evt->setup.system_memory);
+		case 1:
+			ctx->output_3d_type = 1;
+			ctx->width = evt->setup.width;
+			ctx->height = evt->setup.height;
+			ctx->gl_double_buffer = evt->setup.back_buffer;
+			return DD_SetupOpenGL(dr);
+		case 2:
+			ctx->output_3d_type = 2;
+			GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("[DX Out] Attempting to resize Offscreen OpenGL window to %d x %d\n", evt->size.width, evt->size.height));
+			SetWindowPos(ctx->gl_hwnd, NULL, 0, 0, evt->size.width, evt->size.height, SWP_NOZORDER | SWP_NOMOVE | SWP_ASYNCWINDOWPOS);
+			GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("[DX Out] Resizing Offscreen OpenGL window to %d x %d\n", evt->size.width, evt->size.height));
+			SetForegroundWindow(ctx->cur_hwnd);
+			ctx->gl_double_buffer = evt->setup.back_buffer;
 			return DD_SetupOpenGL(dr);
 		}
-		return DD_SetBackBufferSize(dr, evt->size.width, evt->size.height);
 	}
 	return GF_OK;
 }
