@@ -25,14 +25,6 @@
 #include "ffmpeg_in.h"
 #include <gpac/avparse.h>
 
-static void gf_av_vlog(void* avcl, int level, const char *fmt, va_list vl)
-{
-	char szMsg[1024];
-	vsprintf(szMsg, fmt, vl);
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_CODEC, (szMsg));
-}
-
-
 static AVCodec *ffmpeg_get_codec(u32 codec_4cc)
 {
 	char name[5];
@@ -100,7 +92,7 @@ static void FFDEC_LoadDSI(FFDec *ffd, GF_BitStream *bs, Bool from_ff_demux)
 	}
 }
 
-static GF_Err FFDEC_AttachStream(GF_BaseDecoder *plug, u16 ES_ID, char *decSpecInfo, u32 decSpecInfoSize, u16 DependsOnES_ID, u32 objectTypeIndication, Bool UpStream)
+static GF_Err FFDEC_AttachStream(GF_BaseDecoder *plug, GF_ESD *esd)
 {
 	u32 codec_id;
 	int gotpic;
@@ -108,53 +100,54 @@ static GF_Err FFDEC_AttachStream(GF_BaseDecoder *plug, u16 ES_ID, char *decSpecI
 	GF_M4VDecSpecInfo dsi;
 	GF_Err e;
 	FFDec *ffd = (FFDec *)plug->privateStack;
-	if (ffd->ES_ID || DependsOnES_ID || UpStream) return GF_NOT_SUPPORTED;
+	if (ffd->ES_ID || esd->dependsOnESID || esd->decoderConfig->upstream) return GF_NOT_SUPPORTED;
 	if (!ffd->oti) return GF_NOT_SUPPORTED;
-	ffd->ES_ID = ES_ID;
+	ffd->ES_ID = esd->ESID;
 
 	ffd->ctx = avcodec_alloc_context();
 	
 	/*private FFMPEG DSI*/
-	if (ffd->oti == GPAC_FFMPEG_CODECS_OTI) {
-		bs = gf_bs_new(decSpecInfo, decSpecInfoSize, GF_BITSTREAM_READ);
+	if (ffd->oti == GPAC_OTI_MEDIA_FFMPEG) {
+		bs = gf_bs_new(esd->decoderConfig->decoderSpecificInfo->data, esd->decoderConfig->decoderSpecificInfo->dataLength, GF_BITSTREAM_READ);
 		codec_id = gf_bs_read_u32(bs);
 		if (ffd->st==GF_STREAM_AUDIO) {
 			ffd->ctx->codec_type = CODEC_TYPE_AUDIO;
 			ffd->ctx->sample_rate = gf_bs_read_u32(bs);
 			ffd->ctx->channels = gf_bs_read_u16(bs);
-			ffd->ctx->bits_per_sample = gf_bs_read_u16(bs);
-			/*force 16 bits output*/
-			ffd->ctx->bits_per_sample = 16;
 			ffd->ctx->frame_size = gf_bs_read_u16(bs);
+			/*bits_per_sample */gf_bs_read_u8(bs);
+			/*num_frames_per_au*/ gf_bs_read_u8(bs);
+
+			/*ffmpeg specific*/
 			ffd->ctx->block_align = gf_bs_read_u16(bs);
 		} else if (ffd->st==GF_STREAM_VISUAL) {
 			ffd->ctx->codec_type = CODEC_TYPE_VIDEO;
-			ffd->ctx->width = gf_bs_read_u32(bs);
-			ffd->ctx->height = gf_bs_read_u32(bs);
+			ffd->ctx->width = gf_bs_read_u16(bs);
+			ffd->ctx->height = gf_bs_read_u16(bs);
 		}
-		ffd->ctx->codec_tag = gf_bs_read_u32(bs);
 		ffd->ctx->bit_rate = gf_bs_read_u32(bs);
 
+		ffd->ctx->codec_tag = gf_bs_read_u32(bs);
+
 		ffd->codec = avcodec_find_decoder(codec_id);
 		FFDEC_LoadDSI(ffd, bs, 1);
 		gf_bs_del(bs);
 	} 
 	/*private QT DSI*/
-	else if (ffd->oti == GPAC_EXTRA_CODECS_OTI) {
-		bs = gf_bs_new(decSpecInfo, decSpecInfoSize, GF_BITSTREAM_READ);
+	else if (ffd->oti == GPAC_OTI_MEDIA_GENERIC) {
+		bs = gf_bs_new(esd->decoderConfig->decoderSpecificInfo->data, esd->decoderConfig->decoderSpecificInfo->dataLength, GF_BITSTREAM_READ);
 		codec_id = gf_bs_read_u32(bs);
 		if (ffd->st==GF_STREAM_AUDIO) {
 			ffd->ctx->codec_type = CODEC_TYPE_AUDIO;
-			ffd->ctx->sample_rate = gf_bs_read_u16(bs);
+			ffd->ctx->sample_rate = gf_bs_read_u32(bs);
+			ffd->ctx->channels = gf_bs_read_u16(bs);
 			ffd->ctx->frame_size = gf_bs_read_u16(bs);
-			ffd->ctx->channels = gf_bs_read_u8(bs);
-			ffd->ctx->bits_per_sample = gf_bs_read_u8(bs);
-			/*packed mode*/ gf_bs_read_u8(bs);
+			/*bits_per_sample */ gf_bs_read_u8(bs);
+			/*num_frames_per_au*/ gf_bs_read_u8(bs);
 			/*just in case...*/
 			if (codec_id == GF_4CC('a', 'm', 'r', ' ')) {
 			  ffd->ctx->sample_rate = 8000;
 			  ffd->ctx->channels = 1;
-			  ffd->ctx->bits_per_sample = 16;
 			  ffd->ctx->frame_size = 160;
 			}
 		} else if (ffd->st==GF_STREAM_VISUAL) {
@@ -178,7 +171,7 @@ static GF_Err FFDEC_AttachStream(GF_BaseDecoder *plug, u16 ES_ID, char *decSpecI
 			case 0x21:
 				codec_id = CODEC_ID_H264;
 				/*ffmpeg H264/AVC needs that*/
-				ffd->ctx->codec_tag = 0x31637661;
+				//ffd->ctx->codec_tag = 0x31637661;
 				break;
 			case 0x6A:
 			case 0x60:
@@ -218,25 +211,28 @@ static GF_Err FFDEC_AttachStream(GF_BaseDecoder *plug, u16 ES_ID, char *decSpecI
 	if ((ffd->st==GF_STREAM_VISUAL)) {
 		/*for all MPEG-4 variants get size*/
 		if ((ffd->oti==0x20) || (ffd->oti == 0x21)) {
-			if (!decSpecInfoSize || !decSpecInfo) return GF_NON_COMPLIANT_BITSTREAM;
+			/*if not set this may be a remap of non-mpeg4 transport (eg, transport on MPEG-TS) where
+			the DSI is carried in-band*/
+			if (esd->decoderConfig->decoderSpecificInfo->data) {
 
-			/*for regular MPEG-4, try to decode and if this fails try H263 decoder at first frame*/
-			if (ffd->oti==0x20) {
-				e = gf_m4v_get_config(decSpecInfo, decSpecInfoSize, &dsi);
-				if (e) return e;
-				ffd->ctx->width = dsi.width;
-				ffd->ctx->height = dsi.height;
-				if (!dsi.width && !dsi.height) ffd->check_short_header = 1;
-				ffd->previous_par = (dsi.par_num<<16) | dsi.par_den;
-				ffd->no_par_update = 1;
-			} else if (ffd->oti==0x21) {
-				ffd->check_h264_isma = 1;
+				/*for regular MPEG-4, try to decode and if this fails try H263 decoder at first frame*/
+				if (ffd->oti==0x20) {
+					e = gf_m4v_get_config(esd->decoderConfig->decoderSpecificInfo->data, esd->decoderConfig->decoderSpecificInfo->dataLength, &dsi);
+					if (e) return e;
+					ffd->ctx->width = dsi.width;
+					ffd->ctx->height = dsi.height;
+					if (!dsi.width && !dsi.height) ffd->check_short_header = 1;
+					ffd->previous_par = (dsi.par_num<<16) | dsi.par_den;
+					ffd->no_par_update = 1;
+				} else if (ffd->oti==0x21) {
+					ffd->check_h264_isma = 1;
+				}
+
+				/*setup dsi for FFMPEG context BEFORE attaching decoder (otherwise not proper init)*/
+				ffd->ctx->extradata = malloc(sizeof(char)*esd->decoderConfig->decoderSpecificInfo->dataLength);
+				memcpy(ffd->ctx->extradata, esd->decoderConfig->decoderSpecificInfo->data, esd->decoderConfig->decoderSpecificInfo->dataLength);
+				ffd->ctx->extradata_size = esd->decoderConfig->decoderSpecificInfo->dataLength;
 			}
-
-			/*setup dsi for FFMPEG context BEFORE attaching decoder (otherwise not proper init)*/
-			ffd->ctx->extradata = malloc(sizeof(char)*decSpecInfoSize);
-			memcpy(ffd->ctx->extradata, decSpecInfo, decSpecInfoSize);
-			ffd->ctx->extradata_size = decSpecInfoSize;
 		}
 		ffd->frame = avcodec_alloc_frame();
 	}
@@ -249,8 +245,7 @@ static GF_Err FFDEC_AttachStream(GF_BaseDecoder *plug, u16 ES_ID, char *decSpecI
 			ffd->ctx->frame_size = (ffd->ctx->sample_rate > 24000) ? 1152 : 576;
 		}
 		/*may be 0 (cfg not known yet)*/
-		ffd->out_size = ffd->ctx->bits_per_sample * ffd->ctx->channels * ffd->ctx->frame_size / 8;
-		if (!ffd->ctx->bits_per_sample) ffd->ctx->bits_per_sample = 16;
+		ffd->out_size = ffd->ctx->channels * ffd->ctx->frame_size * 2 /*16 / 8*/;
 		if (!ffd->ctx->sample_rate) ffd->ctx->sample_rate = 44100;
 		if (!ffd->ctx->channels) ffd->ctx->channels = 2;
 	} else {
@@ -258,11 +253,14 @@ static GF_Err FFDEC_AttachStream(GF_BaseDecoder *plug, u16 ES_ID, char *decSpecI
 		case CODEC_ID_MJPEG:
 		case CODEC_ID_MJPEGB:
 		case CODEC_ID_LJPEG:
+#if (LIBAVCODEC_VERSION_INT > ((51<<16)+(20<<8)+0) )
+		case CODEC_ID_GIF:
+#endif
 			ffd->pix_fmt = GF_PIXEL_RGB_24; 
 			break;
 		case CODEC_ID_DVD_SUBTITLE:
 			ffd->frame = avcodec_alloc_frame();
-			avcodec_decode_video(ffd->ctx, ffd->frame, &gotpic, decSpecInfo, decSpecInfoSize);
+			avcodec_decode_video(ffd->ctx, ffd->frame, &gotpic, esd->decoderConfig->decoderSpecificInfo->data, esd->decoderConfig->decoderSpecificInfo->dataLength);
 			ffd->pix_fmt = GF_PIXEL_YV12; 
 			break;
 		default:
@@ -270,15 +268,15 @@ static GF_Err FFDEC_AttachStream(GF_BaseDecoder *plug, u16 ES_ID, char *decSpecI
 			break;
 		}
 		ffd->out_size = ffd->ctx->width * ffd->ctx->height * 3;
-		if (ffd->pix_fmt!=GF_PIXEL_RGB_24) ffd->out_size /= 2;
-	}
 
-#if 0
-	ffd->ctx->debug = FF_DEBUG_PICT_INFO | FF_DEBUG_BITSTREAM | FF_DEBUG_STARTCODE;
-	ffd->ctx->debug = 0xFFFFFFFF;
-	av_log_set_level(AV_LOG_DEBUG);
-	av_log_set_callback(gf_av_vlog);
+		ffd->out_pix_fmt = ffd->pix_fmt;
+#if defined(_WIN32_WCE) ||  defined(__SYMBIAN32__)
+#else
+//		if (ffd->pix_fmt == GF_PIXEL_YV12) ffd->out_pix_fmt = GF_PIXEL_RGB_24;
 #endif
+
+		if (ffd->out_pix_fmt != GF_PIXEL_RGB_24) ffd->out_size /= 2;
+	}
 	return GF_OK;
 }
 static GF_Err FFDEC_DetachStream(GF_BaseDecoder *plug, u16 ES_ID)
@@ -292,6 +290,9 @@ static GF_Err FFDEC_DetachStream(GF_BaseDecoder *plug, u16 ES_ID)
 		avcodec_close(ffd->ctx);
 		ffd->ctx = NULL;
 	}
+#ifdef FFMPEG_SWSCALE
+	if (ffd->sws_ctx) { sws_freeContext(ffd->sws_ctx); ffd->sws_ctx = NULL; }
+#endif
 	return GF_OK;
 }
 
@@ -330,7 +331,7 @@ static GF_Err FFDEC_GetCapabilities(GF_BaseDecoder *plug, GF_CodecCapability *ca
 		capability->cap.valueInt = ffd->ctx->channels;
 		break;
 	case GF_CODEC_BITS_PER_SAMPLE:
-		capability->cap.valueInt = ffd->ctx->bits_per_sample;
+		capability->cap.valueInt = 16;
 		break;
 	case GF_CODEC_BUFFER_MIN:
 		capability->cap.valueInt = (ffd->st==GF_STREAM_AUDIO) ? 4 : 1;
@@ -350,7 +351,8 @@ static GF_Err FFDEC_GetCapabilities(GF_BaseDecoder *plug, GF_CodecCapability *ca
 		capability->cap.valueInt = ffd->ctx->height;
 		break;
 	case GF_CODEC_STRIDE:
-		capability->cap.valueInt = (ffd->pix_fmt==GF_PIXEL_RGB_24) ? ffd->ctx->width*3 : ffd->ctx->width;
+		capability->cap.valueInt = ffd->ctx->width;
+		if (ffd->out_pix_fmt==GF_PIXEL_RGB_24) capability->cap.valueInt *= 3;
 		break;
 	case GF_CODEC_FPS:
 		capability->cap.valueFloat = 30.0f;
@@ -359,9 +361,7 @@ static GF_Err FFDEC_GetCapabilities(GF_BaseDecoder *plug, GF_CodecCapability *ca
 		capability->cap.valueInt = ffd->previous_par;
 		break;
 	case GF_CODEC_PIXEL_FORMAT:
-		if (ffd->ctx->width) {
-			capability->cap.valueInt = ffd->pix_fmt;
-		}
+		if (ffd->ctx->width) capability->cap.valueInt = ffd->out_pix_fmt;
 		break;
 	/*ffmpeg performs frame reordering internally*/
 	case GF_CODEC_REORDER:
@@ -457,11 +457,9 @@ redecode:
 
 		/*first config*/
 		if (!ffd->out_size) {
-			ffd->ctx->bits_per_sample = 16;
-			ffd->out_size = ffd->ctx->bits_per_sample * ffd->ctx->channels * ffd->ctx->frame_size / 8;
+			ffd->out_size = ffd->ctx->channels * ffd->ctx->frame_size * 2 /* 16 / 8 */;
 		}
 		if (ffd->out_size < (u32) gotpic) {
-			ffd->ctx->bits_per_sample = 16;
 			/*looks like relying on frame_size is not a good idea for all codecs, so we use gotpic*/
 			(*outBufferLength) = ffd->out_size = gotpic;
 			return GF_BUFFER_TOO_SMALL;
@@ -500,6 +498,7 @@ redecode:
 				u32 nalu_size;
 				u32 remain = inBufferLength;
 				char *start, *end;
+
 				start = inBuffer;
 				end = inBuffer + 4;
 				while (remain>4) {
@@ -516,7 +515,7 @@ redecode:
 					end++;
 					remain--;
 				}
-				nalu_size = end - start - 4;
+				nalu_size = (inBuffer+inBufferLength) - start - 4;
 				start[0] = (nalu_size>>24)&0xFF;
 				start[1] = (nalu_size>>16)&0xFF;
 				start[2] = (nalu_size>>8)&0xFF;
@@ -557,20 +556,23 @@ redecode:
 		/*recompute outsize in case on-the-fly change*/
 		if ((w != ffd->ctx->width) || (h != ffd->ctx->height)) {
 			outsize = ffd->ctx->width * ffd->ctx->height * 3;
-			if (ffd->pix_fmt!=GF_PIXEL_RGB_24) outsize /= 2;
+			if (ffd->out_pix_fmt != GF_PIXEL_RGB_24) outsize /= 2;
 			ffd->out_size = outsize;
 			*outBufferLength = ffd->out_size;
 			if (ffd->check_h264_isma) {
 				inBuffer[0] = inBuffer[1] = inBuffer[2] = 0;
 				inBuffer[3] = 1;
 			}
+#ifdef FFMPEG_SWSCALE
+			if (ffd->sws_ctx) { sws_freeContext(ffd->sws_ctx); ffd->sws_ctx = NULL; }
+#endif
 			return GF_BUFFER_TOO_SMALL;
 		}
 		/*check PAR in case on-the-fly change*/
 		if (!ffd->no_par_update && ffd->ctx->sample_aspect_ratio.num && ffd->ctx->sample_aspect_ratio.den) {
-			outsize = (ffd->ctx->sample_aspect_ratio.num<<16) | ffd->ctx->sample_aspect_ratio.den;
-			if (outsize!=ffd->previous_par) {
-				ffd->previous_par=outsize;
+			u32 new_par = (ffd->ctx->sample_aspect_ratio.num<<16) | ffd->ctx->sample_aspect_ratio.den;
+			if (new_par != ffd->previous_par) {
+				ffd->previous_par = new_par;
 				*outBufferLength = ffd->out_size;
 				return GF_BUFFER_TOO_SMALL;
 			}
@@ -580,8 +582,7 @@ redecode:
 		if (mmlevel	== GF_CODEC_LEVEL_SEEK) return GF_OK;
 
 		if (gotpic) {
-//#if defined(_WIN32_WCE) ||  defined(__SYMBIAN32__)
-#if 1
+#if defined(_WIN32_WCE) ||  defined(__SYMBIAN32__)
 			if (ffd->pix_fmt==GF_PIXEL_RGB_24) {
 				memcpy(outBuffer, ffd->frame->data[0], sizeof(char)*3*ffd->ctx->width);
 			} else {
@@ -615,7 +616,7 @@ redecode:
 			AVPicture pict;
 			u32 pix_out;
 			memset(&pict, 0, sizeof(pict));
-			if (ffd->pix_fmt==GF_PIXEL_RGB_24) {
+			if (ffd->out_pix_fmt==GF_PIXEL_RGB_24) {
 				pict.data[0] = outBuffer;
 				pict.linesize[0] = 3*ffd->ctx->width;
 				pix_out = PIX_FMT_RGB24;
@@ -629,10 +630,23 @@ redecode:
 				if (!mmlevel && ffd->frame->interlaced_frame) {
 					avpicture_deinterlace((AVPicture *) ffd->frame, (AVPicture *) ffd->frame, ffd->ctx->pix_fmt, ffd->ctx->width, ffd->ctx->height);
 				}
+
 			}
 			pict.data[3] = 0;
 			pict.linesize[3] = 0;
+#ifndef FFMPEG_SWSCALE
 			img_convert(&pict, pix_out, (AVPicture *) ffd->frame, ffd->ctx->pix_fmt, ffd->ctx->width, ffd->ctx->height);
+#else
+			if (!ffd->sws_ctx) 
+				ffd->sws_ctx = sws_getContext(ffd->ctx->width, ffd->ctx->height,
+		                        ffd->ctx->pix_fmt, ffd->ctx->width, ffd->ctx->height, pix_out, SWS_BICUBIC, 
+        	                NULL, NULL, NULL);
+			
+			if (ffd->sws_ctx)
+				sws_scale(ffd->sws_ctx, ffd->frame->data, ffd->frame->linesize, 0, ffd->ctx->height->codec->height, pict.data, pict.linesize);
+
+#endif
+
 			*outBufferLength = ffd->out_size;
 #endif
 		}
@@ -660,22 +674,25 @@ static Bool FFDEC_CanHandleStream(GF_BaseDecoder *plug, u32 StreamType, u32 Obje
 	check_4cc = 0;
 	
 	/*private from FFMPEG input*/
-	if (ObjectType == GPAC_FFMPEG_CODECS_OTI) {
+	if (ObjectType == GPAC_OTI_MEDIA_FFMPEG) {
 		bs = gf_bs_new(decSpecInfo, decSpecInfoSize, GF_BITSTREAM_READ);
 		codec_id = gf_bs_read_u32(bs);
 		gf_bs_del(bs);
 	}
 	/*private from IsoMedia input*/
-	else if (ObjectType == GPAC_EXTRA_CODECS_OTI) {
+	else if (ObjectType == GPAC_OTI_MEDIA_GENERIC) {
 		bs = gf_bs_new(decSpecInfo, decSpecInfoSize, GF_BITSTREAM_READ);
 		codec_id = gf_bs_read_u32(bs);
 		check_4cc = 1;
 		gf_bs_del(bs);
 	}
-	/*std MPEG-4 audio*/
 	else if (StreamType==GF_STREAM_AUDIO) {
+		/*std MPEG-2 audio*/
 		if ((ObjectType==0x69) || (ObjectType==0x6B)) codec_id = CODEC_ID_MP2;
+		/*std AC3 audio*/
+		//if (ObjectType==0xA5) codec_id = CODEC_ID_AC3;
 	} 
+	
 	/*std MPEG-4 visual*/
 	else if (StreamType==GF_STREAM_VISUAL) {
 		switch (ObjectType) {
