@@ -152,7 +152,7 @@ static u32 FFDemux_Run(void *par)
 	/*signal EOS*/
 	if (ffd->audio_ch) gf_term_on_sl_packet(ffd->service, ffd->audio_ch, NULL, 0, NULL, GF_EOS);
 	if (ffd->video_ch) gf_term_on_sl_packet(ffd->service, ffd->video_ch, NULL, 0, NULL, GF_EOS);
-	ffd->is_running = 0;
+	ffd->is_running = 2;
 
 	return 0;
 }
@@ -175,6 +175,7 @@ static Bool FFD_CanHandleURL(GF_InputService *plug, const char *url)
 	if (!strnicmp(szName, "rtsp://", 7)) return 0;
 	if (!strnicmp(szName, "rtspu://", 8)) return 0;
 	if (!strnicmp(szName, "rtp://", 6)) return 0;
+	if (!strnicmp(szName, "plato://", 8)) return 0;
 
 	ext = strrchr(szName, '.');
 	if (ext) {
@@ -188,6 +189,7 @@ static Bool FFD_CanHandleURL(GF_InputService *plug, const char *url)
 			|| !strcmp(szExt, "m4v") || !strcmp(szExt, "m4a") 
 			|| !strcmp(szExt, "3gp") || !strcmp(szExt, "3gpp") || !strcmp(szExt, "3gp2") || !strcmp(szExt, "3g2") 
 			|| !strcmp(szExt, "mp3") 
+			|| !strcmp(szExt, "ac3") 
 			|| !strcmp(szExt, "amr") 
 			|| !strcmp(szExt, "bt") || !strcmp(szExt, "wrl") || !strcmp(szExt, "x3dv") 
 			|| !strcmp(szExt, "xmt") || !strcmp(szExt, "xmta") || !strcmp(szExt, "x3d") 
@@ -199,9 +201,12 @@ static Bool FFD_CanHandleURL(GF_InputService *plug, const char *url)
 		if (gf_term_check_extension(plug, "video/x-mpeg-systems", "mpg mpeg mp2 mpa mpe mpv2", "MPEG 1/2 Movies", ext)) return 1;
 		if (gf_term_check_extension(plug, "audio/basic", "snd au", "Basic Audio", ext)) return 1;
 		if (gf_term_check_extension(plug, "audio/x-wav", "wav", "WAV Audio", ext)) return 1;
+		if (gf_term_check_extension(plug, "audio/vnd.wave", "wav", "WAV Audio", ext)) return 1;
 		if (gf_term_check_extension(plug, "video/x-ms-asf", "asf wma wmv asx asr", "WindowsMedia Movies", ext)) return 1;
 		if (gf_term_check_extension(plug, "video/x-ms-wmv", "asf wma wmv asx asr", "WindowsMedia Movies", ext)) return 1;
 		if (gf_term_check_extension(plug, "video/avi", "avi", "AVI Movies", ext)) return 1;
+		if (gf_term_check_extension(plug, "video/vnd.avi", "avi", "AVI Movies", ext)) return 1;
+
 		if (gf_term_check_extension(plug, "video/H263", "h263 263", "H263 Video", ext)) return 1;
 		if (gf_term_check_extension(plug, "video/H264", "h264 264", "H264 Video", ext)) return 1;
 		if (gf_term_check_extension(plug, "video/MPEG4", "cmp", "MPEG-4 Video", ext)) return 1;
@@ -255,7 +260,7 @@ static Bool FFD_CanHandleURL(GF_InputService *plug, const char *url)
 			} else if (!strstr(szExtList, szExt)) {
 				u32 len;
 				char *buf;
-				len = strlen(szExtList) + strlen(szExt) + 1;
+				len = strlen(szExtList) + strlen(szExt) + 10;
 				buf = malloc(sizeof(char)*len);
 				sprintf(buf, "\"%s ", szExt);
 				strcat(buf, &szExtList[1]);
@@ -302,21 +307,22 @@ static GF_ESD *FFD_GetESDescriptor(FFDemux *ffd, Bool for_audio)
 			break;
 		default:
 opaque_audio:
-			esd->decoderConfig->objectTypeIndication = GPAC_FFMPEG_CODECS_OTI;
+			esd->decoderConfig->objectTypeIndication = GPAC_OTI_MEDIA_FFMPEG;
 			bs = gf_bs_new(NULL, 0, GF_BITSTREAM_WRITE);
 			gf_bs_write_u32(bs, dec->codec_id);
 			gf_bs_write_u32(bs, dec->sample_rate);
 			gf_bs_write_u16(bs, dec->channels);
-			gf_bs_write_u16(bs, dec->bits_per_sample);
 			gf_bs_write_u16(bs, dec->frame_size);
+			gf_bs_write_u8(bs, 16);
+			gf_bs_write_u8(bs, 0);
+			/*ffmpeg specific*/
 			gf_bs_write_u16(bs, dec->block_align);
-
-			gf_bs_write_u32(bs, dec->codec_tag);
 			gf_bs_write_u32(bs, dec->bit_rate);
+			gf_bs_write_u32(bs, dec->codec_tag);
 			if (dec->extradata_size) {
 				gf_bs_write_data(bs, dec->extradata, dec->extradata_size);
 			}
-			gf_bs_get_content(bs, (unsigned char **) &esd->decoderConfig->decoderSpecificInfo->data, &esd->decoderConfig->decoderSpecificInfo->dataLength);
+			gf_bs_get_content(bs, (char **) &esd->decoderConfig->decoderSpecificInfo->data, &esd->decoderConfig->decoderSpecificInfo->dataLength);
 			gf_bs_del(bs);
 			break;
 		}
@@ -326,7 +332,10 @@ opaque_audio:
 		esd->slConfig->timestampResolution = ffd->video_tscale.den;
 		switch (dec->codec_id) {
 		case CODEC_ID_MPEG4:
-		case CODEC_ID_H264:
+		/*there is a bug in fragmentation of raw H264 in ffmpeg, the NALU startcode (0x00000001) is split across
+		two frames - we therefore force internal ffmpeg codec ID to avoid NALU size recompute
+		at the decoder level*/
+//		case CODEC_ID_H264:
 			/*if dsi not detected force use ffmpeg*/
 			if (!dec->extradata_size) goto opaque_video;
 			/*otherwise use any MPEG-4 Visual*/
@@ -345,18 +354,19 @@ opaque_audio:
 			break;
 		default:
 opaque_video:
-			esd->decoderConfig->objectTypeIndication = GPAC_FFMPEG_CODECS_OTI;
+			esd->decoderConfig->objectTypeIndication = GPAC_OTI_MEDIA_FFMPEG;
 			bs = gf_bs_new(NULL, 0, GF_BITSTREAM_WRITE);
 			gf_bs_write_u32(bs, dec->codec_id);
-			gf_bs_write_u32(bs, dec->width);
-			gf_bs_write_u32(bs, dec->height);
-			gf_bs_write_u32(bs, dec->codec_tag);
+			gf_bs_write_u16(bs, dec->width);
+			gf_bs_write_u16(bs, dec->height);
+			/*ffmpeg specific*/
 			gf_bs_write_u32(bs, dec->bit_rate);
+			gf_bs_write_u32(bs, dec->codec_tag);
 
 			if (dec->extradata_size) {
 				gf_bs_write_data(bs, dec->extradata, dec->extradata_size);
 			}
-			gf_bs_get_content(bs, (unsigned char **) &esd->decoderConfig->decoderSpecificInfo->data, &esd->decoderConfig->decoderSpecificInfo->dataLength);
+			gf_bs_get_content(bs, (char **) &esd->decoderConfig->decoderSpecificInfo->data, &esd->decoderConfig->decoderSpecificInfo->dataLength);
 			gf_bs_del(bs);
 			break;
 		}
@@ -400,16 +410,60 @@ static void FFD_SetupObjects(FFDemux *ffd)
 	}
 }
 
+static int ff_url_read(void *h, unsigned char *buf, int size)
+{
+	u32 read;
+	int full_size;
+	FFDemux *ffd = (FFDemux *)h;
+
+	gf_term_download_update_stats(ffd->dnload);
+	full_size = 0;
+	if (ffd->buffer_used) {
+		if (ffd->buffer_used>size) {
+			ffd->buffer_used-=size;
+			memcpy(ffd->buffer, ffd->buffer+size, sizeof(char)*ffd->buffer_used);
+			if (ffd->outdbg) fwrite(buf, size, 1, ffd->outdbg);
+			return size;
+		} 
+		full_size += ffd->buffer_used;
+		buf += ffd->buffer_used;
+		size -= ffd->buffer_used;
+		ffd->buffer_used = 0;
+	}
+
+	while (size) {
+		GF_Err e = gf_dm_sess_fetch_data(ffd->dnload, buf, size, &read);
+		if (e==GF_EOS) break;
+		/*we're sync!!*/
+		if (e==GF_IP_NETWORK_EMPTY) continue;
+		if (e) {
+			GF_LOG(GF_LOG_ERROR, GF_LOG_CONTAINER, ("[FFMPEG Demuxer] error fetching bytes from network: %s\n", gf_error_to_string(e) ) );
+			return -1;
+		}
+		full_size += read;
+		if (read==size) break;
+		size -= read;
+		buf += read;
+	}
+	if (ffd->outdbg) fwrite(ffd->buffer, full_size, 1, ffd->outdbg);
+	return full_size ? (int) full_size : -1;
+}
+
+static void FFD_NetIO(void *cbk, GF_NETIO_Parameter *param)
+{
+}
 
 static GF_Err FFD_ConnectService(GF_InputService *plug, GF_ClientService *serv, const char *url)
 {
 	GF_Err e;
 	s64 last_aud_pts;
-	s32 i;
+	u32 i;
+	Bool is_local;
 	const char *sOpt;
 	char *ext, szName[1000];
 	FFDemux *ffd = plug->priv;
 	AVInputFormat *av_in = NULL;
+	char szExt[20];
 
 	if (ffd->ctx) return GF_SERVICE_ERROR;
 
@@ -424,15 +478,54 @@ static GF_Err FFD_ConnectService(GF_InputService *plug, GF_ClientService *serv,
 		else if (!stricmp(&ext[1], "audio")) ffd->service_type = 2;
 		ext[0] = 0;
 	}
-	i = av_open_input_file(&ffd->ctx, szName, NULL, 0, NULL);
-	if (i<0) {
-		char szExt[20];
-		ext = strrchr(szName, '.');
-		strcpy(szExt, ext+1);
-		strlwr(szExt);
 
-		/*some extensions not supported by ffmpeg*/
-		if (ext && !strcmp(szExt, "cmp")) av_in = av_find_input_format("m4v");
+	/*some extensions not supported by ffmpeg, overload input format*/
+	ext = strrchr(szName, '.');
+	strcpy(szExt, ext ? ext+1 : "");
+	strlwr(szExt);
+	if (!strcmp(szExt, "cmp")) av_in = av_find_input_format("m4v");
+
+	is_local = (strnicmp(url, "file://", 7) && strstr(url, "://")) ? 0 : 1;
+
+	GF_LOG(GF_LOG_DEBUG, GF_LOG_CONTAINER, ("[FFMPEG] opening file %s - local %d - av_in %08x\n", url, is_local, av_in));
+
+	if (!is_local) {
+	    AVProbeData   pd;
+
+		/*setup wraper for FFMPEG I/O*/
+		ffd->buffer_size = 8192;
+		sOpt = gf_modules_get_option((GF_BaseInterface *)plug, "FFMPEG", "IOBufferSize"); 
+		if (sOpt) ffd->buffer_size = atoi(sOpt);
+		ffd->buffer = malloc(sizeof(char)*ffd->buffer_size);
+		ffd->outdbg = fopen("ffdeb.raw", "wb");
+
+		init_put_byte(&ffd->io, ffd->buffer, ffd->buffer_size, 0, ffd, ff_url_read, NULL, NULL);
+
+		ffd->dnload = gf_term_download_new(ffd->service, url, GF_NETIO_SESSION_NOT_THREADED  | GF_NETIO_SESSION_NOT_CACHED, FFD_NetIO, ffd);
+		if (!ffd->dnload) return GF_URL_ERROR;
+		while (1) {
+			e = gf_dm_sess_fetch_data(ffd->dnload, ffd->buffer, ffd->buffer_size, &ffd->buffer_used);
+			if (e==GF_EOS) break;
+			/*we're sync!!*/
+			if (e==GF_IP_NETWORK_EMPTY) continue;
+			if (e) goto err_exit;
+			if (ffd->buffer_used) break;
+		}
+
+		pd.filename = szName;
+		pd.buf_size = ffd->buffer_used;
+		pd.buf = ffd->buffer;
+		av_in = av_probe_input_format(&pd, 1);
+		if (!av_in) {
+			return GF_NOT_SUPPORTED;
+		}
+		if (ffd->outdbg) fwrite(ffd->buffer, ffd->buffer_used, 1, ffd->outdbg);
+		ffd->buffer_used = 0;
+
+		/*setup downloader*/
+		av_in->flags |= AVFMT_NOFILE;
+		i = av_open_input_stream(&ffd->ctx, &ffd->io, szName, av_in, NULL);
+	} else {
 		i = av_open_input_file(&ffd->ctx, szName, av_in, 0, NULL);
 	}
 
@@ -445,8 +538,15 @@ static GF_Err FFD_ConnectService(GF_InputService *plug, GF_ClientService *serv,
 	default: e = GF_SERVICE_ERROR; goto err_exit;
 	}
 
+	GF_LOG(GF_LOG_DEBUG, GF_LOG_CONTAINER, ("[FFMPEG] looking for streams in %s - %d streams - type %s\n", ffd->ctx->filename, ffd->ctx->nb_streams, ffd->ctx->iformat->name));
+
+    if (av_find_stream_info(ffd->ctx) <0) {
+		GF_LOG(GF_LOG_ERROR, GF_LOG_CONTAINER, ("[FFMPEG] cannot locate streams\n"));
+		e = GF_NOT_SUPPORTED;
+		goto err_exit;
+	}
+	GF_LOG(GF_LOG_DEBUG, GF_LOG_CONTAINER, ("[FFMPEG] file %s opened - %d streams\n", url, ffd->ctx->nb_streams));
 
-    if (av_find_stream_info(ffd->ctx) <0) goto err_exit;
 	/*figure out if we can use codecs or not*/
 	ffd->audio_st = ffd->video_st = -1;
     for (i = 0; i < ffd->ctx->nb_streams; i++) {
@@ -470,7 +570,10 @@ static GF_Err FFD_ConnectService(GF_InputService *plug, GF_ClientService *serv,
     }
 	if ((ffd->service_type==1) && (ffd->video_st<0)) goto err_exit;
 	if ((ffd->service_type==2) && (ffd->audio_st<0)) goto err_exit;
-	if ((ffd->video_st<0) && (ffd->audio_st<0)) goto err_exit;
+	if ((ffd->video_st<0) && (ffd->audio_st<0)) {
+		GF_LOG(GF_LOG_ERROR, GF_LOG_CONTAINER, ("[FFMPEG] No supported streams in file\n"));
+		goto err_exit;
+	}
 
 
 	sOpt = gf_modules_get_option((GF_BaseInterface *)plug, "FFMPEG", "DataBufferMS"); 
@@ -493,11 +596,13 @@ static GF_Err FFD_ConnectService(GF_InputService *plug, GF_ClientService *serv,
 	}
 
 	/*build seek*/
-	ffd->seekable = (av_seek_frame(ffd->ctx, -1, 0, AVSEEK_FLAG_BACKWARD)<0) ? 0 : 1;
-	if (!ffd->seekable) {
-	    av_close_input_file(ffd->ctx);
-		av_open_input_file(&ffd->ctx, szName, av_in, 0, NULL);
-	    av_find_stream_info(ffd->ctx);
+	if (is_local) {
+		ffd->seekable = (av_seek_frame(ffd->ctx, -1, 0, AVSEEK_FLAG_BACKWARD)<0) ? 0 : 1;
+		if (!ffd->seekable) {
+			av_close_input_file(ffd->ctx);
+			av_open_input_file(&ffd->ctx, szName, av_in, 0, NULL);
+			av_find_stream_info(ffd->ctx);
+		}
 	}
 
 	/*let's go*/
@@ -507,6 +612,7 @@ static GF_Err FFD_ConnectService(GF_InputService *plug, GF_ClientService *serv,
 	return GF_OK;
 
 err_exit:
+	GF_LOG(GF_LOG_ERROR, GF_LOG_CONTAINER, ("[FFMPEG] Error opening file %s: %s\n", url, gf_error_to_string(e)));
     if (ffd->ctx) av_close_input_file(ffd->ctx);
 	ffd->ctx = NULL;
 	gf_term_on_connect(serv, NULL, e);
@@ -564,7 +670,17 @@ static GF_Err FFD_CloseService(GF_InputService *plug)
 	ffd->audio_ch = ffd->video_ch = NULL;
 	ffd->audio_run = ffd->video_run = 0;
 
+	if (ffd->dnload) {
+		while (!ffd->is_running) gf_sleep(0);
+		ffd->is_running = 0;
+		gf_term_download_del(ffd->dnload);
+		ffd->dnload = NULL;
+	}
+	if (ffd->buffer) free(ffd->buffer);
+	ffd->buffer = NULL;
+
 	gf_term_on_disconnect(ffd->service, NULL, GF_OK);
+	if (ffd->outdbg) fclose(ffd->outdbg);
 	return GF_OK;
 }
 
@@ -631,6 +747,12 @@ static GF_Err FFD_ServiceCommand(GF_InputService *plug, GF_NetworkCommand *com)
 {
 	FFDemux *ffd = plug->priv;
 
+
+	if (com->command_type==GF_NET_SERVICE_HAS_AUDIO) {
+		if (ffd->audio_st>=0) return GF_OK;
+		return GF_NOT_SUPPORTED;
+	}
+
 	if (!com->base.on_channel) return GF_NOT_SUPPORTED;
 
 	switch (com->command_type) {
@@ -660,7 +782,7 @@ static GF_Err FFD_ServiceCommand(GF_InputService *plug, GF_NetworkCommand *com)
 
 		/*play on media stream, start thread*/
 		if ((ffd->audio_ch==com->base.on_channel) || (ffd->video_ch==com->base.on_channel)) {
-			if (!ffd->is_running) {
+			if (ffd->is_running!=1) {
 				ffd->is_running = 1;
 				gf_th_run(ffd->thread, FFDemux_Run, ffd);
 			}
@@ -722,8 +844,8 @@ void *New_FFMPEG_Demux()
 
 	ffd->CanHandleURLInService = FFD_CanHandleURLInService;
 
-	priv->thread = gf_th_new();
-	priv->mx = gf_mx_new();
+	priv->thread = gf_th_new("FFMPEG Demux");
+	priv->mx = gf_mx_new("FFMPEG Demux");
 
 	GF_REGISTER_MODULE_INTERFACE(ffd, GF_NET_CLIENT_INTERFACE, "FFMPEG Demuxer", "gpac distribution");
 	ffd->priv = priv;
