@@ -58,6 +58,14 @@
 /*include FFMPEG APIs*/
 #include <ffmpeg/avformat.h>
 
+void gf_av_vlog(void* avcl, int level, const char *fmt, va_list vl);
+
+
+#if LIBAVCODEC_VERSION_INT > ((52<<16)+(0<<8)+0)
+#define FFMPEG_SWSCALE
+#include <ffmpeg/swscale.h>
+#endif
+
 /*FFMPEG decoder module */
 typedef struct 
 {
@@ -72,6 +80,11 @@ typedef struct
     AVCodec *codec;
 	AVFrame *frame;
 	u32 pix_fmt;
+	u32 out_pix_fmt;
+
+#ifdef FFMPEG_SWSCALE
+	struct SwsContext *sws_ctx;
+#endif
 
 	/*for audio packed frames*/
 	u32 frame_start;
@@ -88,6 +101,9 @@ void FFDEC_Delete(void *ifce);
 
 */
 
+//#define FFMPEG_IO_BUF_SIZE	16384
+
+
 typedef struct
 {
 	/*the service we're responsible for*/
@@ -115,14 +131,23 @@ typedef struct
 
 	u32 service_type;
 	Bool unreliable_audio_timing;
+
+	/*IO wrapper*/
+	/*file downloader*/
+	GF_DownloadSession *dnload;
+
+	ByteIOContext   io;
+	char *buffer;
+	u32 buffer_size;
+
+	u32 buffer_used;
+
+	FILE *outdbg;
 } FFDemux;
 
 void *New_FFMPEG_Demux();
 void Delete_FFMPEG_Demux(void *ifce);
 
-/*this is the OTI (user-priv) used for all undefined codec used by ffmpeg (carrying specific info from
-AVContext*/
-#define GPAC_FFMPEG_CODECS_OTI			0x81
 
 /*The DSI sent is:
 
