@@ -31,7 +31,27 @@
 #include "gapi.h"
 
 #ifdef GPAC_USE_OGL_ES
-#pragma message("Compiling GAPI with OpenGL-ES support")
+
+#if (defined(WIN32) || defined(_WIN32_WCE)) && !defined(__GNUC__)
+
+#if 0
+# pragma message("Using OpenGL-ES Common Lite Profile")
+# pragma comment(lib, "libGLES_CL")
+
+#define GLES_NO_PBUFFER
+#define GLES_NO_PIXMAP
+
+#else
+# pragma message("Using OpenGL-ES Common Profile")
+# pragma comment(lib, "libGLES_CM")
+
+//#define GLES_NO_PIXMAP
+
+#endif
+
+#pragma comment(lib, "gx.lib")
+#endif
+
 #endif
 
 static Bool is_landscape = 0;
@@ -177,9 +197,12 @@ static void w32_translate_key(u32 wParam, u32 lParam, GF_EventKey *evt)
 	case VK_F3: evt->key_code = GF_KEY_F3; break;
 	case VK_F4: evt->key_code = GF_KEY_F4; break;
 	case VK_F5: evt->key_code = GF_KEY_F5; break;
-	case VK_F6: evt->key_code = GF_KEY_F6; break;
-	case VK_F7: evt->key_code = GF_KEY_F7; break;
+//	case VK_F6: evt->key_code = GF_KEY_F6; break;
+//	case VK_F7: evt->key_code = GF_KEY_F7; break;
+	case VK_F6: evt->key_code = GF_KEY_VOLUMEUP; break;
+	case VK_F7: evt->key_code = GF_KEY_VOLUMEDOWN; break;
 	case VK_F8: evt->key_code = GF_KEY_F8; break;
+		
 	case VK_F9: evt->key_code = GF_KEY_F9; break;
 	case VK_F10: evt->key_code = GF_KEY_F10; break;
 	case VK_F11: evt->key_code = GF_KEY_F11; break;
@@ -258,12 +281,21 @@ static void w32_translate_key(u32 wParam, u32 lParam, GF_EventKey *evt)
 			else if (res==ctx->keys.vkDown) evt->key_code = is_landscape ? GF_KEY_LEFT : GF_KEY_DOWN;
 			else if (res==ctx->keys.vkUp) evt->key_code = is_landscape ? GF_KEY_RIGHT : GF_KEY_UP;
 			else if (res==ctx->keys.vkStart) evt->key_code = GF_KEY_ENTER;
-			else if (res==ctx->keys.vkA) evt->key_code = GF_KEY_CONTROL;
-			else if (res==ctx->keys.vkB) evt->key_code = GF_KEY_SHIFT;
-			else if (res==ctx->keys.vkC) evt->key_code = GF_KEY_ALT;
-
-			else
+			else if (res==ctx->keys.vkA) 
+				evt->key_code = GF_KEY_MEDIAPREVIOUSTRACK;
+			else if (res==ctx->keys.vkB) 
+				evt->key_code = GF_KEY_MEDIANEXTTRACK;
+			else if (res==ctx->keys.vkC) 
+				evt->key_code = GF_KEY_SHIFT;
+			else if (res==0xc1) 
+				evt->key_code = GF_KEY_ALT;
+			else if (res==0xc2) 
+				evt->key_code = GF_KEY_CONTROL;
+			else if (res==0xc5) 
+				evt->key_code = GF_KEY_VOLUMEDOWN;
+			else {
 				evt->key_code = GF_KEY_UNIDENTIFIED;
+			}
 		}
 		break;
 	}
@@ -302,21 +334,14 @@ LRESULT APIENTRY GAPI_WindowProc(HWND hWnd, UINT msg, UINT wParam, LONG lParam)
 		break;
 
 	case WM_ERASEBKGND:
-//		evt.type = GF_EVENT_REFRESH;
-//		the_video_driver->on_event(the_video_driver->evt_cbk_hdl, &evt);
+		evt.type = GF_EVENT_REFRESH;
+		the_video_driver->on_event(the_video_driver->evt_cbk_hdl, &evt);
 		break;
 	case WM_PAINT:
 	{
-		PAINTSTRUCT ps;
-		HDC dc, hdcBitmap, old;
 		GAPIPriv *gctx = (GAPIPriv *)the_video_driver->opaque;
-		if (gctx->gx_mode) break;
-		dc = BeginPaint(gctx->hWnd, &ps);
-		hdcBitmap = CreateCompatibleDC((HDC)dc);
-		old = (HDC) SelectObject(hdcBitmap, gctx->bitmap);
-		BitBlt((HDC)dc, gctx->dst_blt.x, gctx->dst_blt.y, gctx->bb_width, gctx->bb_height, hdcBitmap, 0, 0, SRCCOPY);
-		SelectObject(hdcBitmap, old);
-		EndPaint(gctx->hWnd, &ps);
+		if (gctx->gx_mode || !gctx->bitmap) break;
+		BitBlt(gctx->hdc, gctx->dst_blt.x, gctx->dst_blt.y, gctx->bb_width, gctx->bb_height, gctx->hdcBitmap, 0, 0, SRCCOPY);
 	}
 		break;
 
@@ -364,7 +389,7 @@ void GAPI_WindowThread(void *par)
 	GAPIPriv *ctx = (GAPIPriv *)the_video_driver->opaque;
 
 	memset(&wc, 0, sizeof(WNDCLASS));
-	wc.hInstance = GetModuleHandle(_T("gapi.dll"));
+	wc.hInstance = GetModuleHandle(_T("gm_gapi.dll"));
 	wc.lpfnWndProc = GAPI_WindowProc;
 	wc.hCursor = LoadCursor (NULL, IDC_ARROW);
 	wc.hbrBackground = (HBRUSH)GetStockObject (BLACK_BRUSH);
@@ -389,6 +414,7 @@ void GAPI_WindowThread(void *par)
 
 void GAPI_SetupWindow(GF_VideoOutput *dr)
 {
+	GF_Err e;
 	GAPIPriv *ctx = (GAPIPriv *)dr->opaque;
 	if (the_video_driver) return;
 	the_video_driver = dr;
@@ -403,6 +429,46 @@ void GAPI_SetupWindow(GF_VideoOutput *dr)
 		/*override window proc*/
 		SetWindowLong(ctx->hWnd, GWL_WNDPROC, (DWORD) GAPI_WindowProc);
 	}
+
+#ifdef GPAC_USE_OGL_ES
+	ctx->use_pbuffer = 1;
+	dr->hw_caps |= GF_VIDEO_HW_OPENGL_OFFSCREEN_ALPHA;
+	e = GAPI_SetupOGL_ES_Offscreen(dr, 20, 20);
+	if (e!=GF_OK) {
+		dr->hw_caps &= ~GF_VIDEO_HW_OPENGL_OFFSCREEN_ALPHA;
+#ifndef GLES_NO_PIXMAP
+		e = GAPI_SetupOGL_ES_Offscreen(dr, 20, 20);
+#endif
+	}
+	if (!e) {
+		dr->hw_caps |= GF_VIDEO_HW_OPENGL_OFFSCREEN;
+		return;
+	}
+	ctx->use_pbuffer = 0;
+
+	WNDCLASS wc;
+	HINSTANCE hInst;
+	hInst = GetModuleHandle(_T("gm_gapi.dll") );
+	memset(&wc, 0, sizeof(WNDCLASS));
+	wc.hInstance = hInst;
+	wc.lpfnWndProc = GAPI_WindowProc;
+	wc.lpszClassName = _T("GPAC GAPI Offscreen");
+	RegisterClass (&wc);
+
+	ctx->gl_hwnd = CreateWindow(_T("GPAC GAPI Offscreen"), _T("GPAC GAPI Offscreen"), WS_POPUP, 0, 0, 120, 100, NULL, NULL, hInst, NULL);
+	if (!ctx->gl_hwnd) return;
+	ShowWindow(ctx->gl_hwnd, SW_HIDE);
+
+	dr->hw_caps |= GF_VIDEO_HW_OPENGL_OFFSCREEN_ALPHA;
+	e = GAPI_SetupOGL_ES_Offscreen(dr, 20, 20);
+
+	if (e!=GF_OK) {
+		dr->hw_caps &= ~GF_VIDEO_HW_OPENGL_OFFSCREEN_ALPHA;
+		e = GAPI_SetupOGL_ES_Offscreen(dr, 20, 20);
+	}
+	if (e==GF_OK) dr->hw_caps |= GF_VIDEO_HW_OPENGL_OFFSCREEN;
+#endif
+
 }
 
 void GAPI_ShutdownWindow(GF_VideoOutput *dr)
@@ -421,6 +487,12 @@ void GAPI_ShutdownWindow(GF_VideoOutput *dr)
 		ctx->orig_wnd_proc = 0L;
 	}
 	ctx->hWnd = NULL;
+#ifdef GPAC_USE_OGL_ES
+	PostMessage(ctx->gl_hwnd, WM_DESTROY, 0, 0);
+	ctx->gl_hwnd = NULL;
+	UnregisterClass(_T("GPAC GAPI Offscreen"), GetModuleHandle(_T("gm_gapi.dll") ));
+#endif
+
 	the_video_driver = NULL;
 }
 
@@ -432,7 +504,7 @@ GF_Err GAPI_Clear(GF_VideoOutput *dr, u32 color)
 	return GF_OK;
 }
 
-static void createPixmap(GAPIPriv *ctx, HDC dc, Bool for_gl)
+static void createPixmap(GAPIPriv *ctx, u32 pix_type)
 {
     const size_t    bmiSize = sizeof(BITMAPINFO) + 256U*sizeof(RGBQUAD);
     BITMAPINFO*     bmi;
@@ -460,10 +532,21 @@ static void createPixmap(GAPIPriv *ctx, HDC dc, Bool for_gl)
 		p[0] = 0x00ff0000; p[1] = 0x0000ff00; p[2] = 0x000000ff;
 		break;
 	}
-	if (for_gl) {
-		ctx->bitmap = CreateDIBSection(dc, bmi, DIB_RGB_COLORS, (void **) &ctx->bits, NULL, 0);
+
+	ctx->hdc = GetDC(ctx->hWnd);
+	
+	if (pix_type==2) {
+#ifdef GPAC_USE_OGL_ES
+		ctx->gl_bitmap = CreateDIBSection(ctx->hdc, bmi, DIB_RGB_COLORS, (void **) &ctx->gl_bits, NULL, 0);
+#endif
+	} else if (pix_type==1) {
+		ctx->hdcBitmap = CreateCompatibleDC(ctx->hdc);
+		ctx->bitmap = CreateDIBSection(ctx->hdc, bmi, DIB_RGB_COLORS, (void **) &ctx->bits, NULL, 0);
+		ctx->old_bitmap = (HBITMAP) SelectObject(ctx->hdcBitmap, ctx->bitmap);
 	} else {
-		ctx->bitmap = CreateDIBSection(dc, bmi, DIB_RGB_COLORS, (void **) &ctx->backbuffer, NULL, 0);
+		ctx->hdcBitmap = CreateCompatibleDC(ctx->hdc);
+		ctx->bitmap = CreateDIBSection(ctx->hdc, bmi, DIB_RGB_COLORS, (void **) &ctx->backbuffer, NULL, 0);
+		ctx->old_bitmap = (HBITMAP) SelectObject(ctx->hdcBitmap, ctx->bitmap);
 		/*watchout - win32 always create DWORD align memory, so align our pitch*/
 		while ((ctx->bb_pitch % 4) != 0) ctx->bb_pitch ++;
 	}
@@ -473,14 +556,23 @@ static void createPixmap(GAPIPriv *ctx, HDC dc, Bool for_gl)
 
 #ifdef GPAC_USE_OGL_ES
 
-void GAPI_ReleaseOGL_ES(GAPIPriv *ctx)
+void GAPI_ReleaseOGL_ES(GAPIPriv *ctx, Bool offscreen_only)
 {
 	if (ctx->egldpy) {
+		eglMakeCurrent(ctx->egldpy, EGL_NO_SURFACE, EGL_NO_SURFACE, EGL_NO_CONTEXT);
+
 		if (ctx->eglctx) eglDestroyContext(ctx->egldpy, ctx->eglctx);
-		if (ctx->surface ) eglDestroySurface(ctx->egldpy, ctx->surface);
+		ctx->eglctx = 0;
+		if (ctx->surface) eglDestroySurface(ctx->egldpy, ctx->surface);
+		ctx->surface = 0;
 		if (ctx->egldpy) eglTerminate(ctx->egldpy);
 		ctx->egldpy = 0;
 	}
+    if (ctx->gl_bitmap) DeleteObject(ctx->gl_bitmap);
+	ctx->gl_bitmap = NULL;
+
+	if (offscreen_only) return;
+
     if (ctx->bitmap) DeleteObject(ctx->bitmap);
 	ctx->bitmap = NULL;
 }
@@ -488,31 +580,110 @@ void GAPI_ReleaseOGL_ES(GAPIPriv *ctx)
 GF_Err GAPI_SetupOGL_ES(GF_VideoOutput *dr) 
 {
 	EGLint n, maj, min;
+	u32 i;
 	GF_Event evt;
-	HDC dc;
 	GAPICTX(dr)
-	static int atts[15];
-	atts[0] = EGL_RED_SIZE; atts[1] = (gctx->pixel_format==GF_PIXEL_RGB_24) ? 8 : 5;
-	atts[2] = EGL_GREEN_SIZE; atts[3] = (gctx->pixel_format==GF_PIXEL_RGB_24) ? 8 : (gctx->pixel_format==GF_PIXEL_RGB_565) ? 6 : 5;
-	atts[4] = EGL_BLUE_SIZE; atts[5] = (gctx->pixel_format==GF_PIXEL_RGB_24) ? 8 : 5;
-	/*no supported...*/
-	atts[6] = EGL_ALPHA_SIZE; atts[7] = EGL_DONT_CARE;
-	atts[8] = EGL_DEPTH_SIZE; atts[9] = 32;
-	atts[10] = EGL_STENCIL_SIZE; atts[11] = EGL_DONT_CARE;
-	atts[12] = EGL_SURFACE_TYPE; atts[13] = EGL_PIXMAP_BIT;
-	atts[14] = EGL_NONE;
+	static int atts[32];
+	const char *opt;
+
+	i=0;
+	atts[i++] = EGL_RED_SIZE; atts[i++] = (gctx->pixel_format==GF_PIXEL_RGB_24) ? 8 : 5;
+	atts[i++] = EGL_GREEN_SIZE; atts[i++] = (gctx->pixel_format==GF_PIXEL_RGB_24) ? 8 : (gctx->pixel_format==GF_PIXEL_RGB_565) ? 6 : 5;
+	atts[i++] = EGL_BLUE_SIZE; atts[i++] = (gctx->pixel_format==GF_PIXEL_RGB_24) ? 8 : 5;
+	opt = gf_modules_get_option((GF_BaseInterface *)dr, "Video", "GLNbBitsDepth");
+	atts[i++] = EGL_DEPTH_SIZE; atts[i++] = opt ? atoi(opt) : 16;
+	atts[i++] = EGL_SURFACE_TYPE; 
+
+#ifdef GLES_NO_PIXMAP
+	atts[i++] = EGL_WINDOW_BIT;
+#else
+	atts[i++] = EGL_PIXMAP_BIT;
+//	atts[i++] = gctx->fullscreen ? EGL_WINDOW_BIT : EGL_PIXMAP_BIT;
+#endif
+	atts[i++] = EGL_ALPHA_SIZE; atts[i++] = EGL_DONT_CARE;
+	atts[i++] = EGL_STENCIL_SIZE; atts[i++] = EGL_DONT_CARE;
+	atts[i++] = EGL_NONE;
 
 	/*whenever window is resized we must reinit OGL-ES*/
-	GAPI_ReleaseOGL_ES(gctx);
+	GAPI_ReleaseOGL_ES(gctx, 0);
 
 	if (!gctx->fullscreen) {
 		RECT rc;
 		::GetClientRect(gctx->hWnd, &rc);
 		gctx->bb_width = rc.right-rc.left;
 		gctx->bb_height = rc.bottom-rc.top;
-		dc = GetDC(gctx->hWnd);
-		createPixmap(gctx, dc, 1);
-		ReleaseDC(gctx->hWnd, dc);
+
+#ifndef GLES_NO_PIXMAP
+		createPixmap(gctx, 1);
+#endif
+	}
+
+	gctx->egldpy = eglGetDisplay(EGL_DEFAULT_DISPLAY);
+	if (!gctx->egldpy) {
+		GF_LOG(GF_LOG_ERROR, GF_LOG_MMIO, ("[GAPI] Cannot get OpenGL display\n"));
+		return GF_IO_ERR;
+	}
+	if (!eglInitialize(gctx->egldpy, &maj, &min)) {
+		gctx->egldpy = NULL;
+		GF_LOG(GF_LOG_ERROR, GF_LOG_MMIO, ("[GAPI] Cannot initialize OpenGL layer\n"));
+		return GF_IO_ERR;
+	}
+
+	if (!eglChooseConfig(gctx->egldpy, atts, &gctx->eglconfig, 1, &n) || (eglGetError() != EGL_SUCCESS)) {
+		GF_LOG(GF_LOG_ERROR, GF_LOG_MMIO, ("[GAPI] Cannot choose OpenGL config\n"));
+		return GF_IO_ERR;
+	}
+
+	if (gctx->fullscreen
+#ifdef GLES_NO_PIXMAP
+		|| 1
+#endif
+		) {
+		gctx->surface = eglCreateWindowSurface(gctx->egldpy, gctx->eglconfig, gctx->hWnd, 0);
+	} else {
+		gctx->surface = eglCreatePixmapSurface(gctx->egldpy, gctx->eglconfig, gctx->bitmap, 0);
+	}
+
+	if (!gctx->surface) {
+		GF_LOG(GF_LOG_ERROR, GF_LOG_MMIO, ("[GAPI] Cannot create OpenGL surface - error %d\n", eglGetError()));
+		return GF_IO_ERR; 
+	}
+	gctx->eglctx = eglCreateContext(gctx->egldpy, gctx->eglconfig, NULL, NULL);
+	if (!gctx->eglctx) {
+		GF_LOG(GF_LOG_ERROR, GF_LOG_MMIO, ("[GAPI] Cannot create OpenGL context\n"));
+		eglDestroySurface(gctx->egldpy, gctx->surface);
+		gctx->surface = 0L;
+		return GF_IO_ERR; 
+	}
+    if (!eglMakeCurrent(gctx->egldpy, gctx->surface, gctx->surface, gctx->eglctx)) {
+		GF_LOG(GF_LOG_ERROR, GF_LOG_MMIO, ("[GAPI] Cannot bind OpenGL context\n"));
+		eglDestroyContext(gctx->egldpy, gctx->eglctx);
+		gctx->eglctx = 0L;
+		eglDestroySurface(gctx->egldpy, gctx->surface);
+		gctx->surface = 0L;
+		return GF_IO_ERR;
+	}
+	GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("[GAPI] OpenGL initialize - %d x %d \n", gctx->bb_width, gctx->bb_height));
+	evt.type = GF_EVENT_VIDEO_SETUP;
+	dr->on_event(dr->evt_cbk_hdl, &evt);	
+	return GF_OK;
+}
+
+
+
+GF_Err GAPI_SetupOGL_ES_Offscreen(GF_VideoOutput *dr, u32 width, u32 height) 
+{
+	int atts[15];
+	const char *opt;
+	EGLint n, maj, min;
+
+	GAPICTX(dr)
+
+	GAPI_ReleaseOGL_ES(gctx, 1);
+
+	if (!gctx->use_pbuffer) {
+		SetWindowPos(gctx->gl_hwnd, NULL, 0, 0, width, height, SWP_NOZORDER | SWP_NOMOVE);
+		createPixmap(gctx, 2);
 	}
 
 	gctx->egldpy = eglGetDisplay(/*gctx->dpy*/EGL_DEFAULT_DISPLAY);
@@ -520,15 +691,30 @@ GF_Err GAPI_SetupOGL_ES(GF_VideoOutput *dr)
 		gctx->egldpy = NULL;
 		return GF_IO_ERR;
 	}
+	atts[0] = EGL_RED_SIZE; atts[1] = 8;
+	atts[2] = EGL_GREEN_SIZE; atts[3] = 8;
+	atts[4] = EGL_BLUE_SIZE; atts[5] = 8;
+	atts[6] = EGL_ALPHA_SIZE; atts[7] = (dr->hw_caps & GF_VIDEO_HW_OPENGL_OFFSCREEN_ALPHA) ? 8 : EGL_DONT_CARE;
+	opt = gf_modules_get_option((GF_BaseInterface *)dr, "Video", "GLNbBitsDepth");
+	atts[8] = EGL_DEPTH_SIZE; atts[9] = opt ? atoi(opt) : 16;
+
+	atts[10] = EGL_STENCIL_SIZE; atts[11] = EGL_DONT_CARE;
+	atts[12] = EGL_SURFACE_TYPE; atts[13] = gctx->use_pbuffer ? EGL_PBUFFER_BIT : EGL_PIXMAP_BIT;
+	atts[14] = EGL_NONE;
 
 	eglGetConfigs(gctx->egldpy, NULL, 0, &n);
 	if (!eglChooseConfig(gctx->egldpy, atts, &gctx->eglconfig, 1, &n)) {
 		return GF_IO_ERR;
 	}
-	if (gctx->fullscreen) {
-		gctx->surface = eglCreateWindowSurface(gctx->egldpy, gctx->eglconfig, gctx->hWnd, 0);
+
+	if (!gctx->use_pbuffer) {
+		gctx->surface = eglCreatePixmapSurface(gctx->egldpy, gctx->eglconfig, gctx->gl_bitmap, 0);
 	} else {
-		gctx->surface = eglCreatePixmapSurface(gctx->egldpy, gctx->eglconfig, gctx->bitmap, 0);
+		atts[0] = EGL_WIDTH; atts[1] = width;
+		atts[2] = EGL_HEIGHT; atts[3] = height;
+		atts[4] = EGL_NONE;
+
+		gctx->surface = eglCreatePbufferSurface(gctx->egldpy, gctx->eglconfig, atts);
 	}
 
 	if (!gctx->surface) {
@@ -547,26 +733,34 @@ GF_Err GAPI_SetupOGL_ES(GF_VideoOutput *dr)
 		gctx->surface = 0L;
 		return GF_IO_ERR;
 	}
-	evt.type = GF_EVENT_VIDEO_SETUP;
-	dr->on_event(dr->evt_cbk_hdl, &evt);	
 	return GF_OK;
 }
+
 #endif
 
 
 void GAPI_ReleaseObjects(GAPIPriv *ctx)
 {
 #ifdef GPAC_USE_OGL_ES
-	if (ctx->is_3D) GAPI_ReleaseOGL_ES(ctx);
+	if (ctx->output_3d_type) GAPI_ReleaseOGL_ES(ctx, 0);
 	else
 #endif
     if (ctx->bitmap) DeleteObject(ctx->bitmap);
 	else if (ctx->backbuffer) free(ctx->backbuffer);
 	ctx->backbuffer = NULL;
 	ctx->bitmap = NULL;
+
+	if (ctx->hdcBitmap) {
+		if (ctx->old_bitmap) SelectObject(ctx->hdcBitmap, ctx->old_bitmap);
+		ctx->old_bitmap = NULL;
+		DeleteDC(ctx->hdcBitmap);
+		ctx->hdcBitmap = NULL;
+	}
+	if (ctx->hdc) ReleaseDC(ctx->hWnd, ctx->hdc);
+	ctx->hdc = NULL;
 }
 
-GF_Err GAPI_Setup(GF_VideoOutput *dr, void *os_handle, void *os_display, Bool noover, GF_GLConfig *cfg)
+GF_Err GAPI_Setup(GF_VideoOutput *dr, void *os_handle, void *os_display, Bool noover)
 {
 	struct GXDisplayProperties gx = GXGetDisplayProperties();
 	RECT rc;
@@ -608,21 +802,9 @@ GF_Err GAPI_Setup(GF_VideoOutput *dr, void *os_handle, void *os_display, Bool no
 	gctx->x_pitch = gx.cbxPitch;
 	gctx->y_pitch = gx.cbyPitch;
 
-	if (cfg) {
-#ifdef GPAC_USE_OGL_ES
-		gctx->gl_cfg = *cfg;
-		gctx->is_3D = 1;
-#else
-		return GF_NOT_SUPPORTED;
-#endif
-	}
-
 	GAPI_SetupWindow(dr);
 	if (!gctx->hWnd) return GF_IO_ERR;
-#ifdef GPAC_USE_OGL_ES
-	if (gctx->is_3D) return GF_OK;
-#endif
-	
+
 	/*setup GX*/
 	if (!GXOpenDisplay(gctx->hWnd, 0L)) {
 		MessageBox(NULL, _T("Cannot open display"), _T("GAPI Error"), MB_OK);
@@ -648,14 +830,16 @@ static void GAPI_Shutdown(GF_VideoOutput *dr)
 
 static GF_Err GAPI_SetFullScreen(GF_VideoOutput *dr, Bool bOn, u32 *outWidth, u32 *outHeight)
 {
+	Bool is_wide_scene = (bOn==2) ? 1 : 0;
 	GF_Err e;
 	GAPICTX(dr);
 
 	if (!gctx) return GF_BAD_PARAM;
+	if (is_wide_scene) bOn = 1;
 	if (bOn == gctx->fullscreen) return GF_OK;
 
 #ifdef GPAC_USE_OGL_ES
-	if (gctx->is_3D) {
+	if (gctx->output_3d_type==1) {
 		gctx->fullscreen = bOn;
 		return GAPI_SetupOGL_ES(dr);
 	}
@@ -684,8 +868,8 @@ static GF_Err GAPI_SetFullScreen(GF_VideoOutput *dr, Bool bOn, u32 *outWidth, u3
 			gctx->backup_w = *outWidth;
 			gctx->backup_h = *outHeight;
 
-			if ((gctx->bb_width > gctx->bb_height) && (gctx->screen_w > gctx->screen_h)) is_landscape = 0;
-			else if ((gctx->bb_width < gctx->bb_height) && (gctx->screen_w < gctx->screen_h)) is_landscape = 0;
+			if (is_wide_scene && (gctx->screen_w > gctx->screen_h)) is_landscape = 0;
+			else if (!is_wide_scene && (gctx->screen_w < gctx->screen_h)) is_landscape = 0;
 			else is_landscape = 1;
 
 			if (is_landscape) {
@@ -824,17 +1008,21 @@ static GF_Err GAPI_Flush(GF_VideoOutput *dr, GF_Window *dest)
 	gf_mx_p(gctx->mx);
 	
 #ifdef GPAC_USE_OGL_ES
-	if (gctx->is_3D) {
+	if (gctx->output_3d_type==1) {
+#ifndef GLES_NO_PIXMAP
 		if (gctx->fullscreen && gctx->surface && gctx->egldpy) {
+#endif
 			if (gctx->erase_dest) {
 				InvalidateRect(gctx->hWnd, NULL, TRUE);
 				gctx->erase_dest = 0;
 			}
 			eglSwapBuffers(gctx->egldpy, gctx->surface);
+#ifndef GLES_NO_PIXMAP
 		} else {
 		    InvalidateRect(gctx->hWnd, NULL, gctx->erase_dest);
 			gctx->erase_dest = 0;
 		}
+#endif
 		gf_mx_v(gctx->mx);
 		return GF_OK;
 	}
@@ -877,10 +1065,24 @@ static GF_Err GAPI_ProcessEvent(GF_VideoOutput *dr, GF_Event *evt)
 		/*nothing to do since we don't own the window*/
 		break;
 	case GF_EVENT_VIDEO_SETUP:
+		switch (evt->setup.opengl_mode) {
+		case 0:
 #ifdef GPAC_USE_OGL_ES
-		if (gctx->is_3D) return GAPI_SetupOGL_ES(the_video_driver);
+			gctx->output_3d_type = 0;
 #endif
-		return GAPI_InitBackBuffer(dr, evt->size.width, evt->size.height);
+			return GAPI_InitBackBuffer(dr, evt->setup.width, evt->setup.height);
+#ifdef GPAC_USE_OGL_ES
+		case 1:
+			gctx->output_3d_type = 1;
+			return GAPI_SetupOGL_ES(the_video_driver);
+		case 2:
+			gctx->output_3d_type = 2;
+			return GAPI_SetupOGL_ES_Offscreen(the_video_driver, evt->setup.width, evt->setup.height);
+#else
+		default:
+			return GF_NOT_SUPPORTED;
+#endif
+		}
 	}
 	return GF_OK;
 }
@@ -905,9 +1107,7 @@ static GF_Err GAPI_InitBackBuffer(GF_VideoOutput *dr, u32 VideoWidth, u32 VideoH
 		gctx->backbuffer = (char *) malloc(sizeof(unsigned char) * gctx->bb_size);
 		gctx->gx_mode = 1;
 	} else {
-		HDC dc = GetDC(gctx->hWnd);
-		createPixmap(gctx, dc, 0);
-		::ReleaseDC(gctx->hWnd, dc);
+		createPixmap(gctx, 0);
 		gctx->gx_mode = 0;
 	}
 	RECT rc;
@@ -946,14 +1146,14 @@ static void *NewGAPIVideoOutput()
 
 	priv = (GAPIPriv *) malloc(sizeof(GAPIPriv));
 	memset(priv, 0, sizeof(GAPIPriv));
-	priv->mx = gf_mx_new();
+	priv->mx = gf_mx_new("GAPI");
 	driv->opaque = priv;
 	priv->force_gx = 0;
 
 	/*alpha and keying to do*/
 	driv->hw_caps = GF_VIDEO_HW_CAN_ROTATE;
 #ifdef GPAC_USE_OGL_ES
-	driv->hw_caps = GF_VIDEO_HW_HAS_OPENGL;
+	driv->hw_caps = GF_VIDEO_HW_OPENGL;
 #endif
 
 	driv->Setup = GAPI_Setup;
