@@ -72,19 +72,28 @@ typedef struct
 	Bool erase_dest;
 	u32 off_x, off_y;
 
-	HBITMAP bitmap;
+	HBITMAP bitmap, old_bitmap;
     DWORD * bits;
+	HDC hdcBitmap, hdc;
+
+
 #ifdef GPAC_USE_OGL_ES
-	Bool is_3D;
-	GF_GLConfig gl_cfg;
+	u32 output_3d_type;
     EGLDisplay egldpy;
     EGLSurface surface;
     EGLConfig eglconfig;
     EGLContext eglctx;
+
+	HBITMAP gl_bitmap;
+    DWORD *gl_bits;
+	HWND gl_hwnd;
+	Bool use_pbuffer;
 #endif
 
 } GAPIPriv;
 
+GF_Err GAPI_SetupOGL_ES_Offscreen(GF_VideoOutput *dr, u32 width, u32 height) ;
+
 
 #ifdef __cplusplus
 }
