@@ -350,7 +350,7 @@ static GF_Descriptor *ISOR_GetServiceDesc(GF_InputService *plug, u32 expect_type
 			}
 			else trackID = atoi(ext+1);
 
-			if (!trackID) return NULL;
+			/*if trackID is 0, assume this is a fragment identifier*/
 		}
 	}
 
@@ -394,9 +394,9 @@ static GF_Descriptor *ISOR_GetServiceDesc(GF_InputService *plug, u32 expect_type
 #ifndef GPAC_DISABLE_LOG
 			GF_Err e = gf_isom_last_error(read->mov);
 			if (e) {
-				GF_LOG(GF_LOG_ERROR, GF_LOG_SERVICE, ("[IsoMedia] Cannot fetch MPEG-4 IOD (error %s) - generating one\n", gf_error_to_string(e) ));
+				GF_LOG(GF_LOG_ERROR, GF_LOG_NETWORK, ("[IsoMedia] Cannot fetch MPEG-4 IOD (error %s) - generating one\n", gf_error_to_string(e) ));
 			} else {
-				GF_LOG(GF_LOG_DEBUG, GF_LOG_SERVICE, ("[IsoMedia] No MPEG-4 IOD found in file - generating one\n"));
+				GF_LOG(GF_LOG_DEBUG, GF_LOG_NETWORK, ("[IsoMedia] No MPEG-4 IOD found in file - generating one\n"));
 			}
 #endif
 		}
@@ -528,6 +528,7 @@ GF_Err ISOR_ConnectChannel(GF_InputService *plug, LPNETCHANNEL channel, const ch
 	}
 
 	ch->has_edit_list = gf_isom_get_edit_segment_count(ch->owner->mov, ch->track) ? 1 : 0;
+	ch->has_rap = (gf_isom_has_sync_points(ch->owner->mov, ch->track)==1) ? 1 : 0;
 	ch->time_scale = gf_isom_get_media_timescale(ch->owner->mov, ch->track);
 
 exit:
@@ -749,7 +750,7 @@ GF_Err ISOR_ServiceCommand(GF_InputService *plug, GF_NetworkCommand *com)
 			if (com->play.end_range >= 0) ch->end = (u64) (s64) (com->play.end_range*ch->time_scale);
 		}
 		ch->is_playing = 1;
-		GF_LOG(GF_LOG_DEBUG, GF_LOG_SERVICE, ("[IsoMedia] Starting channel playback "LLD" to "LLD" (%g to %g)\n", ch->start, ch->end, com->play.start_range, com->play.end_range));
+		GF_LOG(GF_LOG_DEBUG, GF_LOG_NETWORK, ("[IsoMedia] Starting channel playback "LLD" to "LLD" (%g to %g)\n", ch->start, ch->end, com->play.start_range, com->play.end_range));
 		return GF_OK;
 	case GF_NET_CHAN_STOP:
 		isor_reset_reader(ch);
@@ -758,6 +759,8 @@ GF_Err ISOR_ServiceCommand(GF_InputService *plug, GF_NetworkCommand *com)
 	/*nothing to do on MP4 for channel config*/
 	case GF_NET_CHAN_CONFIG:
 		return GF_OK;
+	case GF_NET_CHAN_GET_PIXEL_AR:
+		return gf_isom_get_pixel_aspect_ratio(read->mov, ch->track, 1, &com->par.hSpacing, &com->par.vSpacing);
 	case GF_NET_CHAN_GET_DSI:
 	{
 		/*it may happen that there are conflicting config when using ESD URLs...*/
