@@ -37,26 +37,20 @@
 #include <linux/dvb/dmx.h>
 #include <linux/dvb/frontend.h>
 
-// DVB devices definition
-// TODO: using adapter0/..0 by default; multi-frontend to be done
-#define DMX   "/dev/dvb/adapter0/demux0"
-#define FRONT "/dev/dvb/adapter0/frontend0"
-#define DVR   "/dev/dvb/adapter0/dvr0"
-
 typedef struct {
 	u32 freq;
 	u16 vpid;
 	u16 apid; 
-    fe_spectral_inversion_t specInv;
-    fe_modulation_t modulation;
-    fe_bandwidth_t bandwidth;
-    fe_transmit_mode_t TransmissionMode;
-    fe_guard_interval_t guardInterval;
-    fe_code_rate_t HP_CodeRate;
-    fe_code_rate_t LP_CodeRate;
-    fe_hierarchy_t hierarchy;
-    
-    int ts_fd;
+	fe_spectral_inversion_t specInv;
+	fe_modulation_t modulation;
+	fe_bandwidth_t bandwidth;
+	fe_transmit_mode_t TransmissionMode;
+	fe_guard_interval_t guardInterval;
+	fe_code_rate_t HP_CodeRate;
+	fe_code_rate_t LP_CodeRate;
+	fe_hierarchy_t hierarchy;
+
+	int ts_fd;
 } GF_Tuner;
 
 #define DVB_BUFFER_SIZE 3760							// DVB buffer size 188x20
@@ -65,18 +59,26 @@ typedef struct {
 
 #define UDP_BUFFER_SIZE	0x40000
 
+typedef struct {
+	char *fragment;
+	u32 id;
+	/*if only pid is requested*/
+	u32 pid;
+} M2TSIn_Prog;
+
 typedef struct
 {
 	GF_ClientService *service;
-	Bool do_regulate;
 	GF_M2TS_Demuxer *ts;
-	char *program;
-	u32 prog_id;
+
+	Bool request_all_pids;
+	GF_List *requested_progs;
+	GF_List *requested_pids;
 
 	/*demuxer thread*/
 	GF_Thread *th;
 	u32 run_state;
-
+	GF_Mutex *mx;
 	/*net playing*/
 	GF_Socket *sock;
 	
@@ -86,30 +88,55 @@ typedef struct
 #endif	
 	/*local file playing*/
 	FILE *file;
+	char filename[GF_MAX_PATH];
 	u32 start_range, end_range;
 	u32 file_size;
 	Double duration;
 	u32 nb_playing;
+	Bool file_regulate;
+	u64 pcr_last;
+	u32 stb_at_last_pcr;
+	u32 nb_pck;
+
+	Bool epg_requested;
+	Bool has_eit;
+	LPNETCHANNEL eit_channel;
 } M2TSIn;
 
+#define M2TS_BUFFER_MAX 400
 
 
 #ifdef GPAC_HAS_LINUX_DVB
 
 static GF_Err gf_dvb_tune(GF_Tuner *tuner, char *url, const char *chan_path) {
 	struct dmx_pes_filter_params pesFilterParams;
-    struct dvb_frontend_parameters frp;
-    int demux1, front1;
-    FILE *chanfile;
-    char line[255], chan_name_t[255];
-	char freq_str[255], inv[255], bw[255], lcr[255], hier[255], cr[255], mod[255], transm[255], gi[255], apid_str[255], vpid_str[255];
-    const char *chan_conf = ":%255[^:]:%255[^:]:%255[^:]:%255[^:]:%255[^:]:%255[^:]:%255[^:]:%255[^:]:%255[^:]:%255[^:]:%255[^:]:";
-    char *chan_name;
+	struct dvb_frontend_parameters frp;
+	int demux1, front1;
+	FILE *chanfile;
+	char line[255], chan_name_t[255];
+	char freq_str[255], inv[255], bw[255], lcr[255], hier[255], cr[255], 
+		 mod[255], transm[255], gi[255], apid_str[255], vpid_str[255];
+	const char *chan_conf = ":%255[^:]:%255[^:]:%255[^:]:%255[^:]:%255[^:]:%255[^:]:%255[^:]:%255[^:]:%255[^:]:%255[^:]:%255[^:]:";
+	char *chan_name;
+	char *tmp;
+	char frontend_name[100], demux_name[100], dvr_name[100];
+	u32 adapter_num;
+
+	chanfile = fopen(chan_path, "r");
+	if (!chanfile) return GF_BAD_PARAM;
+
+	chan_name = url+6; // 6 = strlen("dvb://")
 	
-    if((chanfile=fopen(chan_path, "r"))==NULL) {
-		return GF_BAD_PARAM;
+	// support for multiple frontends
+	tmp = strchr(chan_name, '@');
+	if (tmp) {
+		adapter_num = atoi(tmp+1);
+		tmp[0] = 0;
+	} else {
+		adapter_num = 0;
 	}
-	chan_name=url+6;
+	GF_LOG(GF_LOG_DEBUG, GF_LOG_CONTAINER, ("Channel name %s\n", chan_name));
+
 	while(!feof(chanfile)) {
 		if ( fgets(line, 255, chanfile) != NULL) {
 			if (line[0]=='#') continue;
@@ -177,18 +204,28 @@ static GF_Err gf_dvb_tune(GF_Tuner *tuner, char *url, const char *chan_path) {
 			}
 		}
 	}
-    fclose(chanfile);
+	fclose(chanfile);
         
-    // Open frontend
-    if((front1 = open(FRONT,O_RDWR)) < 0){
-    	return GF_IO_ERR;
-    }
-    // Open demuxes
-    if ((demux1=open(DMX, O_RDWR|O_NONBLOCK)) < 0){
-        return GF_IO_ERR;
-    }
+	sprintf(frontend_name, "/dev/dvb/adapter%d/frontend0", adapter_num);
+	sprintf(demux_name, "/dev/dvb/adapter%d/demux0", adapter_num);
+	sprintf(dvr_name, "/dev/dvb/adapter%d/dvr0", adapter_num);
+
+	// Open frontend
+	if((front1 = open(frontend_name,O_RDWR|O_NONBLOCK)) < 0){
+		GF_LOG(GF_LOG_ERROR, GF_LOG_CONTAINER, ("Cannot open frontend %s.\n", frontend_name));
+  		return GF_IO_ERR;
+	} else {
+		GF_LOG(GF_LOG_DEBUG, GF_LOG_CONTAINER, ("Frontend %s opened.\n", frontend_name));
+	}
+	// Open demuxes
+	if ((demux1=open(demux_name, O_RDWR|O_NONBLOCK)) < 0){
+		GF_LOG(GF_LOG_ERROR, GF_LOG_CONTAINER, ("Cannot open demux %s\n", demux_name));
+  		return GF_IO_ERR;
+	} else {
+		GF_LOG(GF_LOG_ERROR, GF_LOG_CONTAINER, ("Demux %s opened.\n", demux_name));
+	}
 	// Set FrontendParameters - DVB-T
-    frp.frequency = tuner->freq;
+	frp.frequency = tuner->freq;
 	frp.inversion = tuner->specInv;
 	frp.u.ofdm.bandwidth = tuner->bandwidth;
 	frp.u.ofdm.code_rate_HP = tuner->HP_CodeRate;
@@ -197,8 +234,8 @@ static GF_Err gf_dvb_tune(GF_Tuner *tuner, char *url, const char *chan_path) {
 	frp.u.ofdm.transmission_mode = tuner->TransmissionMode;
 	frp.u.ofdm.guard_interval = tuner->guardInterval;
 	frp.u.ofdm.hierarchy_information = tuner->hierarchy;
-    // Set frontend
-    if (ioctl(front1, FE_SET_FRONTEND, &frp) < 0){
+	// Set frontend
+	if (ioctl(front1, FE_SET_FRONTEND, &frp) < 0){
    		return GF_IO_ERR;
 	}
 	// Set dumex
@@ -210,9 +247,10 @@ static GF_Err gf_dvb_tune(GF_Tuner *tuner, char *url, const char *chan_path) {
 	if (ioctl(demux1, DMX_SET_PES_FILTER, &pesFilterParams) < 0){
   		return GF_IO_ERR;
 	}
-	if ((tuner->ts_fd = open(DVR, O_RDONLY/*|O_NONBLOCK*/)) < 0){
+  /* The following code differs from mplayer and alike because the device is opened in blocking mode */
+	if ((tuner->ts_fd = open(dvr_name, O_RDONLY/*|O_NONBLOCK*/)) < 0){
 	        return GF_IO_ERR;
-  	}	
+ 	}	
 	return GF_OK;
 }
 #endif
@@ -228,7 +266,7 @@ static Bool M2TS_CanHandleURL(GF_InputService *plug, const char *url)
 		|| !strnicmp(url, "dvb://", 6) 
 #endif
 	) {
-		return 1;
+  	return 1;
 	}
 	
 	sExt = strrchr(url, '.');
@@ -237,38 +275,92 @@ static Bool M2TS_CanHandleURL(GF_InputService *plug, const char *url)
 	return 0;
 }
 
-#define REGULATE_TIME_SLOT	200
-
-/*!! FIXME - THIS IS PLAIN WRONG AND UGLY - WE NEED TO ESTIMATE THE TS BITRATE AND REGULATE BASED ON THAT !!!*/
-static void M2TS_Regulate(M2TSIn *m2ts)
+#ifdef GPAC_HAS_LINUX_DVB
+static u32 gf_dvb_get_freq_from_url(const char *channels_config_path, const char *url) 
 {
-	u32 to_sleep;
-	GF_NetworkCommand com;
+	FILE *channels_config_file;
+	char line[255], name[255], *tmp, *channel_name;
 
-	com.command_type = GF_NET_BUFFER_QUERY;
-	/*sleep untill the buffer occupancy is too low - note that this work because all streams in this
-	demuxer are synchronized*/
-	while (m2ts->run_state) {
-		/*wait for a connected channel !!*/
-		if (!m2ts->nb_playing && m2ts->do_regulate) {
-			gf_sleep(50);
-		} else {
-			gf_term_on_command(m2ts->service, &com, GF_OK);
-			if (com.buffer.occupancy < REGULATE_TIME_SLOT) {
-				//gf_sleep(1);
-				return;
-			}
-			to_sleep = com.buffer.occupancy  - REGULATE_TIME_SLOT;
-			//fprintf(stdout, "MPEG-2 TS regulate: %d ms in buffers - sleeping %d ms\n", com.buffer.occupancy, to_sleep);
-			while (m2ts->run_state && (to_sleep>REGULATE_TIME_SLOT)) {
-				gf_sleep(REGULATE_TIME_SLOT);
-				to_sleep -= REGULATE_TIME_SLOT;
+	u32 freq;
+
+	/* get rid of trailing @ */
+	tmp = strchr(url, '@');
+	if (tmp) tmp[0] = 0;
+
+	channel_name = url+6; 
+
+	channels_config_file = fopen(channels_config_path, "r");
+	if (!channels_config_file) return GF_BAD_PARAM;
+
+	freq = 0;
+	while(!feof(channels_config_file)) {
+		if ( fgets(line, 255, channels_config_file) != NULL) {
+			if (line[0]=='#') continue;
+			if (line[0]=='\r') continue;
+			if (line[0]=='\n') continue;
+
+			tmp = strchr(line, ':');
+			tmp[0] = 0;
+			if (!strcmp(line, channel_name)) {				
+				char *tmp2;
+				tmp++;
+				tmp2 = strchr(tmp, ':');
+				if (tmp2) tmp2[0] = 0;
+				freq = (u32)atoi(tmp);
+				break;
 			}
 		}
 	}
+	return freq;
+}
+#endif
+
+static Bool M2TS_CanHandleURLInService(GF_InputService *plug, const char *url)
+{
+	Bool ret = 0;
+	M2TSIn *m2ts = (M2TSIn *)plug->priv;
+
+#ifdef GPAC_HAS_LINUX_DVB
+	if (!stricmp(url, "dvb://EPG")) return 1;
+	if (!strnicmp(url, "dvb://", 6)) {		
+		GF_LOG(GF_LOG_DEBUG, GF_LOG_CONTAINER, ("[DVBIn] Checking reuse of the same tuner for %s\n", url));
+		const char *chan_conf = gf_modules_get_option((GF_BaseInterface *)plug, "DVB", "ChannelsFile");
+		if (!chan_conf) {
+			GF_LOG(GF_LOG_ERROR, GF_LOG_CONTAINER, ("[DVBIn] Cannot locate channel configuration file\n"));
+			ret = 0;
+		}
+
+		/* if the tuner is already tuned to the same frequence, nothing needs to be done */
+		else if (m2ts->tuner->freq != 0) {
+			char *frag = strchr(url, '#');
+			if (frag) frag[0] = 0;
+			if (m2ts->tuner->freq == gf_dvb_get_freq_from_url(chan_conf, url)) {
+				GF_LOG(GF_LOG_DEBUG, GF_LOG_CONTAINER, ("[DVBIn] Reusing the same tuner for %s\n", url));
+				ret = 1;
+			}
+			if (frag) frag[0] = '#';
+		} 
+	} else 
+#endif
+	if (!strnicmp(url, "udp://", 6) 
+		|| !strnicmp(url, "mpegts-udp://", 13) 
+		|| !strnicmp(url, "mpegts-tcp://", 13))
+	{	
+		/* TODO: check IP address ...*/
+		ret = 0;
+	} else {
+		char *frag = strchr(url, '#');
+		if (frag) frag[0] = 0;
+		if (!strlen(url) || !strcmp(url, m2ts->filename)) {
+			GF_LOG(GF_LOG_DEBUG, GF_LOG_CONTAINER, ("[DVBIn] Reusing the same input file for %s\n", url));
+ 			ret = 1;
+		}
+		if (frag) frag[0] = '#';
+	}
+	return ret;
 }
 
-static void MP2TS_DeclareStream(M2TSIn *m2ts, GF_M2TS_PES *stream)
+static GF_ObjectDescriptor *MP2TS_GetOD(M2TSIn *m2ts, GF_M2TS_PES *stream, char *dsi, u32 dsi_size, u32 *streamType)
 {
 	GF_ObjectDescriptor *od;
 	GF_ESD *esd;
@@ -303,13 +395,23 @@ static void MP2TS_DeclareStream(M2TSIn *m2ts, GF_M2TS_PES *stream)
 		esd->decoderConfig->streamType = GF_STREAM_AUDIO;
 		esd->decoderConfig->objectTypeIndication = 0x69;
 		break;
+	case GF_M2TS_AUDIO_LATM_AAC:
 	case GF_M2TS_AUDIO_AAC:
+		if (!dsi) {
+			/*discard regulate until we fetch the AAC config*/
+			m2ts->file_regulate = 0;
+			/*turn on parsing*/
+			gf_m2ts_set_pes_framing(stream, GF_M2TS_PES_FRAMING_DEFAULT);
+			gf_odf_desc_del((GF_Descriptor *)esd);
+			return NULL;
+		}
 		esd->decoderConfig->streamType = GF_STREAM_AUDIO;
 		esd->decoderConfig->objectTypeIndication = 0x40;
 		break;
+	case GF_M2TS_SYSTEMS_MPEG4_SECTIONS:
 	default:
 		gf_odf_desc_del((GF_Descriptor *)esd);
-		return;
+		return NULL;
 	}
 	esd->decoderConfig->bufferSizeDB = 0;
 
@@ -321,22 +423,29 @@ static void MP2TS_DeclareStream(M2TSIn *m2ts, GF_M2TS_PES *stream)
 	esd->slConfig->timestampResolution = 90000;
 	
 	/*decoder config*/
-	if (0) {
-		esd->decoderConfig->decoderSpecificInfo->dataLength = 0;
+	if (dsi) {
+		esd->decoderConfig->decoderSpecificInfo->data = malloc(sizeof(char)*dsi_size);
+		memcpy(esd->decoderConfig->decoderSpecificInfo->data, dsi, sizeof(char)*dsi_size);
+		esd->decoderConfig->decoderSpecificInfo->dataLength = dsi_size;
 	}
 
 	/*declare object to terminal*/
 	od = (GF_ObjectDescriptor*)gf_odf_desc_new(GF_ODF_OD_TAG);
 	gf_list_add(od->ESDescriptors, esd);
-	od->objectDescriptorID = stream->pid;	
-	/*declare but don't regenerate scene*/
-	gf_term_add_media(m2ts->service, (GF_Descriptor*)od, 1);
-	m2ts->do_regulate = 1;
-	/*wait for connection*/
-	while (!stream->user) gf_sleep(0);
+	od->objectDescriptorID = stream->pid;
+	if (streamType) *streamType = esd->decoderConfig->streamType;
+	return od;
 }
 
-static void MP2TS_SetupProgram(M2TSIn *m2ts, GF_M2TS_Program *prog)
+static void MP2TS_DeclareStream(M2TSIn *m2ts, GF_M2TS_PES *stream, char *dsi, u32 dsi_size)
+{
+	GF_ObjectDescriptor *od = MP2TS_GetOD(m2ts, stream, dsi, dsi_size, NULL);
+	if (!od) return;
+	/*declare but don't regenerate scene*/
+	gf_term_add_media(m2ts->service, (GF_Descriptor*)od, 1);
+}
+
+static void MP2TS_SetupProgram(M2TSIn *m2ts, GF_M2TS_Program *prog, Bool regenerate_scene, Bool no_declare)
 {
 	u32 i, count;
 
@@ -352,14 +461,19 @@ static void MP2TS_SetupProgram(M2TSIn *m2ts, GF_M2TS_Program *prog)
 		if (!found) return;
 	}
 #endif	
+	m2ts->file_regulate = no_declare ? 0 : 1;
+
 	for (i=0; i<count; i++) {
-		GF_M2TS_PES *pes = gf_list_get(prog->streams, i);
-		if (pes->pid==prog->pmt_pid) continue;
-		gf_m2ts_set_pes_framing(pes, GF_M2TS_PES_FRAMING_SKIP);
-		MP2TS_DeclareStream(m2ts, pes);
+		GF_M2TS_ES *es = gf_list_get(prog->streams, i);
+		if (es->pid==prog->pmt_pid) continue;
+		/*move to skip mode for all PES until asked for playback*/
+		if (!(es->flags & GF_M2TS_ES_IS_SECTION) && !es->user)
+			gf_m2ts_set_pes_framing((GF_M2TS_PES *)es, GF_M2TS_PES_FRAMING_SKIP);
+		if (!prog->pmt_iod && !no_declare) MP2TS_DeclareStream(m2ts, (GF_M2TS_PES *)es, NULL, 0);
 	}
 	/*force scene regeneration*/
-	gf_term_add_media(m2ts->service, NULL, 0);
+	if (regenerate_scene) 
+		gf_term_add_media(m2ts->service, NULL, 0);
 }
 
 static void MP2TS_SendPacket(M2TSIn *m2ts, GF_M2TS_PES_PCK *pck)
@@ -368,7 +482,6 @@ static void MP2TS_SendPacket(M2TSIn *m2ts, GF_M2TS_PES_PCK *pck)
 
 	if (!pck->stream->user) return;
 
-
 	if (!pck->stream->program->first_dts && pck->PTS) {
 		pck->stream->program->first_dts = (pck->DTS ? pck->DTS : pck->PTS) - m2ts->start_range * 90;
 	}
@@ -393,10 +506,108 @@ static GFINLINE void MP2TS_SendSLPacket(M2TSIn *m2ts, GF_M2TS_SL_PCK *pck)
 	gf_term_on_sl_packet(m2ts->service, pck->stream->user, pck->data, pck->data_len, NULL, GF_OK);
 }
 
+static GF_ObjectDescriptor *M2TS_GenerateEPG_OD(M2TSIn *m2ts) 
+{
+	/* declaring a special stream for displaying eit */
+	GF_ObjectDescriptor *od;
+	GF_ESD *esd;
+
+	/*create a stream description for this channel*/
+	esd = gf_odf_desc_esd_new(0);
+	esd->ESID = GF_M2TS_PID_EIT_ST_CIT;
+	esd->OCRESID = GF_M2TS_PID_EIT_ST_CIT;
+	esd->decoderConfig->streamType = GF_STREAM_PRIVATE_SCENE;
+	esd->decoderConfig->objectTypeIndication = GPAC_OTI_PRIVATE_SCENE_EPG;
+	esd->decoderConfig->bufferSizeDB = 0;
+
+	/*we only use AUstart indicator
+	esd->slConfig->useAccessUnitStartFlag = 1;
+	esd->slConfig->useAccessUnitEndFlag = 0;
+	esd->slConfig->useRandomAccessPointFlag = 1;
+	esd->slConfig->AUSeqNumLength = 0;
+	esd->slConfig->timestampResolution = 90000;*/
+
+	/*declare object to terminal*/
+	od = (GF_ObjectDescriptor*)gf_odf_desc_new(GF_ODF_OD_TAG);
+	gf_list_add(od->ESDescriptors, esd);
+	od->objectDescriptorID = GF_M2TS_PID_EIT_ST_CIT;	
+	return od;
+}
+
+static void M2TS_FlushRequested(M2TSIn *m2ts)
+{
+	u32 i, j, req_prog_count, count, prog_id, found;
+
+	gf_mx_p(m2ts->mx);
+
+	found = 0;
+	count = gf_list_count(m2ts->requested_pids);
+	for (i=0; i<count; i++) {
+		M2TSIn_Prog *req_pid = gf_list_get(m2ts->requested_pids, i);
+		GF_M2TS_ES *es = m2ts->ts->ess[req_pid->pid];
+		if (es==NULL) continue;
+
+		/*move to skip mode for all PES until asked for playback*/
+		if (!(es->flags & GF_M2TS_ES_IS_SECTION) && !es->user) 
+			gf_m2ts_set_pes_framing((GF_M2TS_PES *)es, GF_M2TS_PES_FRAMING_SKIP);
+		MP2TS_DeclareStream(m2ts, (GF_M2TS_PES *)es, NULL, 0);
+		gf_list_rem(m2ts->requested_pids, i);
+		free(req_pid);
+		i--;
+		count--;
+		found++;
+	}
+	req_prog_count = gf_list_count(m2ts->requested_progs);
+	for (i = 0; i < req_prog_count; i++) {
+		M2TSIn_Prog *req_prog = gf_list_get(m2ts->requested_progs, i);
+		prog_id = atoi(req_prog->fragment);
+		count = gf_list_count(m2ts->ts->SDTs);
+		for (j=0; j<count; j++) {
+			GF_M2TS_SDT *sdt = gf_list_get(m2ts->ts->SDTs, j);
+			if (!stricmp(sdt->service, req_prog->fragment)) req_prog->id = sdt->service_id;
+			else if (sdt->service_id==prog_id)  req_prog->id = sdt->service_id;
+		}
+		if (req_prog->id) {
+			GF_M2TS_Program *ts_prog;
+			count = gf_list_count(m2ts->ts->programs);
+			for (j=0; j<count; j++) {
+				ts_prog = gf_list_get(m2ts->ts->programs, j);
+				if (ts_prog->number==req_prog->id) {
+					MP2TS_SetupProgram(m2ts, ts_prog, 0, 0);
+					found++;
+					free(req_prog->fragment);
+					free(req_prog);
+					gf_list_rem(m2ts->requested_progs, i);
+					req_prog_count--;
+					i--;
+					break;
+				}
+			}
+		}
+	}
+
+	if (m2ts->epg_requested) {
+		if (!m2ts->has_eit) {
+			GF_ObjectDescriptor *od = M2TS_GenerateEPG_OD(m2ts);
+			/*declare but don't regenerate scene*/
+			gf_term_add_media(m2ts->service, (GF_Descriptor*)od, 0);
+			m2ts->has_eit = 1;
+		}
+	} else {
+		/*force scene regeneration only when EPG is not requested*/
+		if (found)
+			gf_term_add_media(m2ts->service, NULL, 0);
+	}
+
+	gf_mx_v(m2ts->mx);
+}
+
 static void M2TS_OnEvent(GF_M2TS_Demuxer *ts, u32 evt_type, void *param)
 {
 	M2TSIn *m2ts = (M2TSIn *) ts->user;
 	switch (evt_type) {
+	case GF_M2TS_EVT_PAT_UPDATE:
+		break;
 	case GF_M2TS_EVT_PAT_FOUND:
 		/* In case the TS has one program, wait for the PMT to send connect, in case of IOD in PMT */
 		if (gf_list_count(m2ts->ts->programs) != 1)
@@ -406,36 +617,24 @@ static void M2TS_OnEvent(GF_M2TS_Demuxer *ts, u32 evt_type, void *param)
 		if (gf_list_count(m2ts->ts->programs) == 1)
 			gf_term_on_connect(m2ts->service, NULL, GF_OK);
 		
-		/*do not setup if we've been asked for a dedicated program*/
-		if (!m2ts->program) MP2TS_SetupProgram(m2ts, param);
+		/*do not declare if  single program was requested for playback*/
+		MP2TS_SetupProgram(m2ts, param, m2ts->request_all_pids, m2ts->request_all_pids ? 0 : 1);
+
+		M2TS_FlushRequested(m2ts);
 		break;
-	case GF_M2TS_EVT_PAT_UPDATE:
+	case GF_M2TS_EVT_PMT_REPEAT:
 	case GF_M2TS_EVT_PMT_UPDATE:
-	case GF_M2TS_EVT_SDT_UPDATE:
+		M2TS_FlushRequested(m2ts);
 		break;
+	case GF_M2TS_EVT_SDT_REPEAT:
+	case GF_M2TS_EVT_SDT_UPDATE:
 	case GF_M2TS_EVT_SDT_FOUND:
-		if (m2ts->program) {
-			u32 i, count, prog_id;
-			prog_id = atoi(m2ts->program);
-			count = gf_list_count(ts->SDTs);
-			for (i=0; i<count; i++) {
-				GF_M2TS_SDT *sdt = gf_list_get(ts->SDTs, i);
-				if (!stricmp(sdt->service, m2ts->program)) m2ts->prog_id = sdt->service_id;
-				else if (sdt->service_id==prog_id)  m2ts->prog_id = sdt->service_id;
-			}
-			if (m2ts->prog_id) {
-				GF_M2TS_Program *prog;
-				free(m2ts->program);
-				m2ts->program = NULL;
-				count = gf_list_count(ts->programs);
-				for (i=0; i<count; i++) {
-					prog = gf_list_get(ts->programs, i);
-					if (prog->number==m2ts->prog_id) {
-						MP2TS_SetupProgram(m2ts, prog);
-						break;
-					}
-				}
-			}
+		M2TS_FlushRequested(m2ts);
+		break;
+	case GF_M2TS_EVT_DVB_GENERAL:
+		if (m2ts->eit_channel) {
+			GF_M2TS_SL_PCK *pck = (GF_M2TS_SL_PCK *)param;
+			gf_term_on_sl_packet(m2ts->service, m2ts->eit_channel, pck->data, pck->data_len, NULL, GF_OK);
 		}
 		break;
 	case GF_M2TS_EVT_PES_PCK:
@@ -444,6 +643,44 @@ static void M2TS_OnEvent(GF_M2TS_Demuxer *ts, u32 evt_type, void *param)
 	case GF_M2TS_EVT_SL_PCK:
 		MP2TS_SendSLPacket(m2ts, param);
 		break;
+	case GF_M2TS_EVT_AAC_CFG:
+	{
+		GF_M2TS_PES_PCK *pck = (GF_M2TS_PES_PCK*)param;
+		if (!pck->stream->first_dts) {
+			gf_m2ts_set_pes_framing(pck->stream, GF_M2TS_PES_FRAMING_SKIP);
+			MP2TS_DeclareStream(m2ts, pck->stream, pck->data, pck->data_len);
+			m2ts->file_regulate = 1;
+			/*force scene regeneration*/
+			gf_term_add_media(m2ts->service, NULL, 0);
+		}
+	}
+		break;
+	case GF_M2TS_EVT_PES_PCR:
+		if (m2ts->file_regulate) {
+			u64 pcr = ((GF_M2TS_PES_PCK *) param)->PTS;
+			u32 stb = gf_sys_clock();
+			if (m2ts->pcr_last) {
+				s32 diff;
+				u64 pcr_diff = (pcr - m2ts->pcr_last);
+				pcr_diff /= 27000;
+				diff = (u32) pcr_diff - (stb - m2ts->stb_at_last_pcr);
+				if (diff>0 && (diff<1000) ) {
+					/*query buffer level, don't sleep if too low*/
+					GF_NetworkCommand com;
+					com.command_type = GF_NET_BUFFER_QUERY;
+					gf_term_on_command(m2ts->service, &com, GF_OK);
+					if (com.buffer.occupancy) gf_sleep(diff);
+				
+					m2ts->nb_pck = 0;
+					m2ts->pcr_last = pcr;
+					m2ts->stb_at_last_pcr = gf_sys_clock();
+				}
+			} else {
+				m2ts->pcr_last = pcr;
+				m2ts->stb_at_last_pcr = gf_sys_clock();
+			}
+		}
+		break;
 	}
 }
 
@@ -488,7 +725,6 @@ u32 M2TS_Run(void *_p)
 				/*FIXME: we assume only simple RTP packaging (no CSRC nor extensions)*/
 				if ((data[0] != 0x47) && ((data[1] & 0x7F) == 33) ) {
 					is_rtp = 1;
-					//fprintf(stdout, "MPEG-TS over RTP detected\n", size);
 				}
 			}
 			/*process chunk*/
@@ -517,11 +753,15 @@ u32 M2TS_Run(void *_p)
 			if (!size) break;
 			/*process chunk*/
 			gf_m2ts_process_data(m2ts->ts, data, size);
-		
-			/*regulate file reading*/
-			M2TS_Regulate(m2ts);
+
+			m2ts->nb_pck++;
+			/*if asked to regulate, wait until we get a play request*/
+			while (m2ts->run_state && !m2ts->nb_playing && m2ts->file_regulate) {
+				gf_sleep(50);
+				continue;
+			}
 		}
-		fprintf(stdout, "\nEOS reached\n");
+		GF_LOG(GF_LOG_ERROR, GF_LOG_CONTAINER, ("\nEOS reached\n"));
 		if (m2ts->nb_playing) {
 			for (i=0; i<GF_M2TS_MAX_STREAMS; i++) {
 				GF_M2TS_PES *pes = (GF_M2TS_PES *)m2ts->ts->ess[i];
@@ -562,6 +802,7 @@ void M2TS_SetupDVB(GF_InputService *plug, M2TSIn *m2ts, char *url)
 		e = GF_NOT_SUPPORTED;
 		goto exit;
 	}
+
 	chan_conf = gf_modules_get_option((GF_BaseInterface *)plug, "DVB", "ChannelsFile");
 	if (!chan_conf) {
 		GF_LOG(GF_LOG_ERROR, GF_LOG_CONTAINER, ("[DVBIn] Cannot locate channel configuration file\n"));
@@ -569,25 +810,26 @@ void M2TS_SetupDVB(GF_InputService *plug, M2TSIn *m2ts, char *url)
 		goto exit;
 	}
 	
-	if (m2ts->tuner == NULL) { 
-		m2ts->tuner = malloc(sizeof(GF_Tuner));
-	}
+	if (!m2ts->tuner) GF_SAFEALLOC(m2ts->tuner, GF_Tuner);
+
+	if (m2ts->tuner->freq != 0 && m2ts->tuner->freq == gf_dvb_get_freq_from_url(chan_conf, url)) {
+		GF_LOG(GF_LOG_DEBUG, GF_LOG_CONTAINER, ("[DVBIn] Tuner already tuned to that frequency\n"));
+		goto exit;
+	} 
 
 	e = gf_dvb_tune(m2ts->tuner, url, chan_conf);
 	if (e) goto exit;
 
-	m2ts->th = gf_th_new();
+	m2ts->th = gf_th_new("MPEG-2 TS Demux");
 	/*start playing for tune-in*/
 	gf_th_run(m2ts->th, M2TS_Run, m2ts);
 
 exit:
-	if (e) {
-		gf_term_on_connect(m2ts->service, NULL, e);
-	}
+	if (e) gf_term_on_connect(m2ts->service, NULL, e);
 }
 #endif
 
-void M2TS_SetupLive(M2TSIn *m2ts, char *url)
+void M2TS_SetupLive(GF_InputService *plug, M2TSIn *m2ts, char *url)
 {
 	GF_Err e = GF_OK;
 	char *str;
@@ -621,10 +863,18 @@ void M2TS_SetupLive(M2TSIn *m2ts, char *url)
 
 	/*do we have a source ?*/
 	if (strlen(url) && strcmp(url, "localhost") ) {
+		const char *mob_ip = NULL;
+		const char *mob_on = gf_modules_get_option((GF_BaseInterface*)plug, "Network", "MobileIPEnabled");
+		if (mob_on && !strcmp(mob_on, "yes"))
+			mob_ip = gf_modules_get_option((GF_BaseInterface*)plug, "Network", "MobileIP");
+
 		if (gf_sk_is_multicast_address(url)) {
-			gf_sk_setup_multicast(m2ts->sock, url, port, 0, 0, NULL);
+			const char *mcast_ifce = gf_modules_get_option((GF_BaseInterface *) plug, "Network", "DefaultMCastInterface");
+			if (mcast_ifce) mob_ip = mcast_ifce;
+
+			gf_sk_setup_multicast(m2ts->sock, url, port, 0, 0, (char*)mob_ip);
 		} else {
-			gf_sk_bind(m2ts->sock, port, url, 0, GF_SOCK_REUSE_PORT);
+			gf_sk_bind(m2ts->sock, (char*)mob_ip, port, url, 0, GF_SOCK_REUSE_PORT);
 		}
 	}
 	if (str) str[0] = ':';
@@ -632,7 +882,7 @@ void M2TS_SetupLive(M2TSIn *m2ts, char *url)
 	gf_sk_set_buffer_size(m2ts->sock, 0, UDP_BUFFER_SIZE);
 	gf_sk_set_block_mode(m2ts->sock, 0);
 
-	m2ts->th = gf_th_new();
+	m2ts->th = gf_th_new("MPEG-2 TS Demux");
 	gf_th_set_priority(m2ts->th, GF_THREAD_PRIORITY_HIGHEST);
 	/*start playing for tune-in*/
 	gf_th_run(m2ts->th, M2TS_Run, m2ts);
@@ -650,10 +900,17 @@ void M2TS_SetupFile(M2TSIn *m2ts, char *url)
 	u32 size, fsize;
 	s32 nb_rwd;
 #endif
+	if (m2ts->file && !strcmp(m2ts->filename, url)) {
+		GF_LOG(GF_LOG_DEBUG, GF_LOG_CONTAINER, ("[MPEGTSIn] TS already being processed\n"));
+		return;
+	} 
+
 	m2ts->file = fopen(url, "rb");
 	if (!m2ts->file) {
 		gf_term_on_connect(m2ts->service, NULL, GF_URL_ERROR);
+		return;
 	}
+	strcpy(m2ts->filename, url);
 
 	fseek(m2ts->file, 0, SEEK_END);
 	m2ts->file_size = ftell(m2ts->file);
@@ -708,7 +965,7 @@ void M2TS_SetupFile(M2TSIn *m2ts, char *url)
 	m2ts->end_range = m2ts->start_range = 0;
 	m2ts->nb_playing = 0;
 
-	m2ts->th = gf_th_new();
+	m2ts->th = gf_th_new("MPEG-2 TS Demux");
 	/*start playing for tune-in*/
 	gf_th_run(m2ts->th, M2TS_Run, m2ts);
 }
@@ -716,28 +973,22 @@ void M2TS_SetupFile(M2TSIn *m2ts, char *url)
 static GF_Err M2TS_ConnectService(GF_InputService *plug, GF_ClientService *serv, const char *url)
 {
 	char szURL[2048];
-	char *ext;
+	char *frag;
 	M2TSIn *m2ts = plug->priv;
 	m2ts->service = serv;
 
-	if (m2ts->program) free(m2ts->program);
-	m2ts->program = NULL;
-	m2ts->prog_id = 0;
 	strcpy(szURL, url);
-	ext = strrchr(szURL, '#');
-	if (ext) {
-		m2ts->program = strdup(ext+1);
-		ext[0] = 0;
-	}
+	frag = strrchr(szURL, '#');
+	if (frag) frag[0] = 0;
 
-	m2ts->do_regulate = 0;
+	m2ts->file_regulate = 0;
 	m2ts->duration = 0;
 
 	if (!strnicmp(url, "udp://", 6)
 		|| !strnicmp(url, "mpegts-udp://", 13) 
 		|| !strnicmp(url, "mpegts-tcp://", 13) 
 		) {
-		M2TS_SetupLive(m2ts, (char *) szURL);
+		M2TS_SetupLive(plug, m2ts, (char *) szURL);
 	} 
 #ifdef GPAC_HAS_LINUX_DVB
 	else if (!strnicmp(url, "dvb://", 6)) {
@@ -755,6 +1006,7 @@ static GF_Err M2TS_CloseService(GF_InputService *plug)
 {
 	M2TSIn *m2ts = plug->priv;
 
+	GF_LOG(GF_LOG_ERROR, GF_LOG_CONTAINER, ("destroying TSin\n"));
 	if (m2ts->th) {
 		if (m2ts->run_state == 1) {
 			m2ts->run_state = 0;
@@ -772,21 +1024,68 @@ static GF_Err M2TS_CloseService(GF_InputService *plug)
 
 static GF_Descriptor *M2TS_GetServiceDesc(GF_InputService *plug, u32 expect_type, const char *sub_url)
 {
-	u32 i=0;
 	M2TSIn *m2ts = plug->priv;
-	GF_Descriptor *desc;
-	if (gf_list_count(m2ts->ts->programs) == 1) {
-		GF_M2TS_Program *prog = gf_list_get(m2ts->ts->programs, 0);
-		if (prog->pmt_iod) {
-			m2ts->do_regulate = 1;
-			gf_odf_desc_copy((GF_Descriptor *)prog->pmt_iod, &desc);
+	GF_Descriptor *desc = NULL;
+	char *frag;
+
+	frag = sub_url ? strrchr(sub_url, '#') : NULL;
+	/*we have been requested the entire TS*/
+	if (!stricmp(sub_url, "dvb://EPG")) {
+		m2ts->epg_requested = 1;
+	} else if (!frag) {
+		m2ts->request_all_pids = 1;
+	} else {
+		M2TSIn_Prog *prog;
+		frag++;
+
+		/*we need exclusive access*/
+		gf_mx_p(m2ts->mx);
+		if (!strnicmp(frag, "pid=", 4)) {
+			GF_SAFEALLOC(prog, M2TSIn_Prog);
+			prog->pid = atoi(frag+4);
+			gf_list_add(m2ts->requested_pids, prog);
+		} else if (!strnicmp(frag, "EPG", 3)) {
+			m2ts->epg_requested = 1;
+		} else {
+			u32 i, count;
+			count = gf_list_count(m2ts->requested_progs);
+			prog = NULL;
+			for (i=0; i<count; i++) {
+				prog = gf_list_get(m2ts->requested_progs, i);
+				if (!strcmp(prog->fragment, frag))
+					break;
+				prog = NULL;
+			}
+			if (!prog) {
+				GF_SAFEALLOC(prog, M2TSIn_Prog);
+				gf_list_add(m2ts->requested_progs, prog);
+				prog->fragment = strdup(frag);
+			}
+		}
+		gf_mx_v(m2ts->mx);
+	}
+
+	if (expect_type==GF_MEDIA_OBJECT_SCENE) {	
+		if (gf_list_count(m2ts->ts->programs) == 1) {
+			GF_M2TS_Program *prog = gf_list_get(m2ts->ts->programs, 0);
+			if (prog->pmt_iod) {
+				gf_odf_desc_copy((GF_Descriptor *)prog->pmt_iod, &desc);
+				return desc;
+			}
+		} 
+		if (m2ts->epg_requested) {
+			GF_ObjectDescriptor *od = M2TS_GenerateEPG_OD(m2ts);
+			m2ts->epg_requested = 0;
+			return (GF_Descriptor *)od;
+		} else {
+			/*returning an empty IOD means "no scene description", let the terminal handle all media objects*/
+			desc = gf_odf_desc_new(GF_ODF_IOD_TAG);
+			((GF_ObjectDescriptor *) desc)->objectDescriptorID = 1;
 			return desc;
 		}
-	} 
-	/*returning an empty IOD means "no scene description", let the terminal handle all media objects*/
-	desc = gf_odf_desc_new(GF_ODF_IOD_TAG);
-	((GF_ObjectDescriptor *) desc)->objectDescriptorID = 1;
-	return desc;
+	}
+
+	return NULL;
 }
 
 static GF_Err M2TS_ConnectChannel(GF_InputService *plug, LPNETCHANNEL channel, const char *url, Bool upstream)
@@ -823,10 +1122,14 @@ static GF_Err M2TS_ConnectChannel(GF_InputService *plug, LPNETCHANNEL channel, c
 			}
 		} 
 
-		if ((ES_ID<GF_M2TS_MAX_STREAMS) && m2ts->ts->ess[ES_ID]) {
+		if (ES_ID == 18) {
+			e = GF_OK; /* 18 is the PID of EIT packets */
+			m2ts->eit_channel = channel;
+		} else if ((ES_ID<GF_M2TS_MAX_STREAMS) && m2ts->ts->ess[ES_ID]) {
 			GF_M2TS_PES *pes = (GF_M2TS_PES *)m2ts->ts->ess[ES_ID];
-			if (pes->user) e = GF_SERVICE_ERROR;
-			else {
+			if (pes->user) {
+				e = GF_SERVICE_ERROR;
+			} else {
 				pes->user = channel;
 				e = GF_OK;
 			}
@@ -864,6 +1167,11 @@ static GF_Err M2TS_ServiceCommand(GF_InputService *plug, GF_NetworkCommand *com)
 	GF_M2TS_PES *pes;
 	M2TSIn *m2ts = plug->priv;
 
+	if (com->command_type==GF_NET_SERVICE_HAS_AUDIO) {
+		char *frag = strchr(com->audio.base_url, '#');
+		if (frag && !strnicmp(frag, "#pid=", 5)) return GF_NOT_SUPPORTED;
+		return GF_OK;
+	}
 	if (!com->base.on_channel) return GF_NOT_SUPPORTED;
 	switch (com->command_type) {
 	/*we cannot pull complete AUs from the stream*/
@@ -873,7 +1181,7 @@ static GF_Err M2TS_ServiceCommand(GF_InputService *plug, GF_NetworkCommand *com)
 	case GF_NET_CHAN_INTERACTIVE:
 		return GF_NOT_SUPPORTED;
 	case GF_NET_CHAN_BUFFER:
-		com->buffer.max = REGULATE_TIME_SLOT;
+		com->buffer.max = M2TS_BUFFER_MAX;
 		com->buffer.min = 0;
 		return GF_OK;
 	case GF_NET_CHAN_DURATION:
@@ -881,8 +1189,14 @@ static GF_Err M2TS_ServiceCommand(GF_InputService *plug, GF_NetworkCommand *com)
 		return GF_OK;
 	case GF_NET_CHAN_PLAY:
 		pes = M2TS_GetChannel(m2ts, com->base.on_channel);
-		if (!pes) return GF_STREAM_NOT_FOUND;
+		if (!pes) {
+			if (com->base.on_channel == m2ts->eit_channel) {
+				return GF_OK;
+			}
+			return GF_STREAM_NOT_FOUND;
+		}
 		gf_m2ts_set_pes_framing(pes, GF_M2TS_PES_FRAMING_DEFAULT);
+		GF_LOG(GF_LOG_ERROR, GF_LOG_CONTAINER, ("Setting default reframing\n"));
 		/*this is a multplex, only trigger the play command for the first stream activated*/
 		if (!m2ts->nb_playing) {
 			m2ts->start_range = (u32) (com->play.start_range*1000);
@@ -896,7 +1210,12 @@ static GF_Err M2TS_ServiceCommand(GF_InputService *plug, GF_NetworkCommand *com)
 		return GF_OK;
 	case GF_NET_CHAN_STOP:
 		pes = M2TS_GetChannel(m2ts, com->base.on_channel);
-		if (!pes) return GF_STREAM_NOT_FOUND;
+		if (!pes) {
+			if (com->base.on_channel == m2ts->eit_channel) {
+				return GF_OK;
+			}
+			return GF_STREAM_NOT_FOUND;
+		}
 		gf_m2ts_set_pes_framing(pes, GF_M2TS_PES_FRAMING_SKIP);
 		/*FIXME HOORIBLE HACK*/
 		return GF_OK;
@@ -922,6 +1241,7 @@ GF_InputService *NewM2TSReader()
 	GF_REGISTER_MODULE_INTERFACE(plug, GF_NET_CLIENT_INTERFACE, "GPAC MPEG-2 TS Reader", "gpac distribution")
 
 	plug->CanHandleURL = M2TS_CanHandleURL;
+	plug->CanHandleURLInService = M2TS_CanHandleURLInService;
 	plug->ConnectService = M2TS_ConnectService;
 	plug->CloseService = M2TS_CloseService;
 	plug->GetServiceDescriptor = M2TS_GetServiceDesc;
@@ -932,17 +1252,37 @@ GF_InputService *NewM2TSReader()
 	reader = malloc(sizeof(M2TSIn));
 	memset(reader, 0, sizeof(M2TSIn));
 	plug->priv = reader;
+	reader->requested_progs = gf_list_new();
+	reader->requested_pids = gf_list_new();
 	reader->ts = gf_m2ts_demux_new();
 	reader->ts->on_event = M2TS_OnEvent;
 	reader->ts->user = reader;
+	reader->mx = gf_mx_new("MPEG2 Demux");
 	return plug;
 }
 
 void DeleteM2TSReader(void *ifce)
 {
+	u32 i, count;
+
 	GF_InputService *plug = (GF_InputService *) ifce;
 	M2TSIn *m2ts = plug->priv;
+
+	count = gf_list_count(m2ts->requested_progs);
+	for (i = 0; i < count; i++) {
+		M2TSIn_Prog *prog = gf_list_get(m2ts->requested_progs, i);
+		free(prog->fragment);
+		free(prog);
+	}
+	gf_list_del(m2ts->requested_progs);
+	count = gf_list_count(m2ts->requested_pids);
+	for (i = 0; i < count; i++) {
+		M2TSIn_Prog *prog = gf_list_get(m2ts->requested_pids, i);
+		free(prog);
+	}
+	gf_list_del(m2ts->requested_pids);
 	gf_m2ts_demux_del(m2ts->ts);
+	gf_mx_del(m2ts->mx);
 	free(m2ts);
 	free(plug);
 }
@@ -970,4 +1310,3 @@ void ShutdownInterface(GF_BaseInterface *ifce)
 	case GF_NET_CLIENT_INTERFACE:  DeleteM2TSReader(ifce); break;
 	}
 }
-
