@@ -172,7 +172,7 @@ static GF_ObjectDescriptor *OGG_GetOD(OGGStream *st)
 	od->objectDescriptorID = (st->info.streamType==GF_STREAM_AUDIO) ? 3 : 2;
 	esd = gf_odf_desc_esd_new(0);
 	esd->decoderConfig->streamType = st->info.streamType;
-	esd->decoderConfig->objectTypeIndication = GPAC_OGG_MEDIA_OTI;
+	esd->decoderConfig->objectTypeIndication = GPAC_OTI_MEDIA_OGG;
 	esd->decoderConfig->avgBitrate = st->info.bitrate;
 	esd->ESID = st->ESID;
 
@@ -467,7 +467,7 @@ void OGG_Process(OGGReader *read)
 			}
 			gf_bs_write_u16(bs, oggpacket.bytes);
 			gf_bs_write_data(bs, oggpacket.packet, oggpacket.bytes);
-			gf_bs_get_content(bs, (unsigned char **)&st->dsi, &st->dsi_len);
+			gf_bs_get_content(bs, (char **)&st->dsi, &st->dsi_len);
 			gf_bs_del(bs);
 			st->parse_headers--;
 			if (!st->parse_headers) {
@@ -546,14 +546,10 @@ static u32 OggDemux(void *par)
 
 		if (!read->bos_done) continue;
 		
-#if 0
-		/*idle*/
+		/*wait for stream connection*/
 		while (!read->kill_demux && !read->nb_playing) {
-			/*send OD updates*/
-			OGG_SendStreams(read);
 			gf_sleep(20);
 		}
-#endif
 
 		/*(re)starting, seek*/
 		if (read->do_seek) {
@@ -592,7 +588,7 @@ static u32 OggDemux(void *par)
 				}
 			}
 			if (!i || !read->nb_playing) break;
-			gf_sleep(10);
+			gf_sleep(0);
 		}
 	}
     ogg_sync_clear(&read->oy);
@@ -611,7 +607,8 @@ Bool OGG_CheckFile(OGGReader *read)
 	Bool has_stream = 0;
 	fseek(read->ogfile, 0, SEEK_SET);
 
-    ogg_sync_init(&read->oy);
+	ogg_sync_init(&read->oy);
+	memset(&the_info, 0, sizeof(OGGInfo));	
 	max_gran = 0;
 	while (1) {
 		if (!OGG_ReadPage(read, &oggpage)) break;
@@ -948,7 +945,7 @@ GF_InputService *OGG_LoadDemux()
 	reader = malloc(sizeof(OGGReader));
 	memset(reader, 0, sizeof(OGGReader));
 	reader->streams = gf_list_new();
-	reader->demuxer = gf_th_new();
+	reader->demuxer = gf_th_new("OGGDemux");
 	reader->data_buffer_ms = 1000;
 
 	plug->priv = reader;
