@@ -36,13 +36,8 @@ typedef struct
 } RawContext;
 
 
-#ifdef DANAE
-#define RAW_OUT_PIXEL_FORMAT		GF_PIXEL_ARGB
-#define NBPP						4
-#else
 #define RAW_OUT_PIXEL_FORMAT		GF_PIXEL_RGB_24
 #define NBPP						3
-#endif
 
 #define RAWCTX	RawContext *rc = (RawContext *)dr->opaque
 
@@ -58,10 +53,8 @@ static GF_Err raw_resize(GF_VideoOutput *dr, u32 w, u32 h)
 	return GF_OK;
 }
 
-GF_Err RAW_Setup(GF_VideoOutput *dr, void *os_handle, void *os_display, Bool no_proc_override, GF_GLConfig *cfg)
+GF_Err RAW_Setup(GF_VideoOutput *dr, void *os_handle, void *os_display, u32 init_flags)
 {
-	/*fixme, we could try with OpenGL p-buffers*/
-	if (cfg) return GF_NOT_SUPPORTED;
 	raw_resize(dr, 100, 100);
 	return GF_OK;
 }
@@ -98,13 +91,9 @@ static GF_Err RAW_ProcessEvent(GF_VideoOutput *dr, GF_Event *evt)
 {
 	if (evt) {
 		switch (evt->type) {
-#ifdef DANAE
-		case GF_EVENT_SIZE:
-			return raw_resize(dr, evt->size.width, evt->size.height);
-#else
 		case GF_EVENT_VIDEO_SETUP:
-			return raw_resize(dr, evt->size.width, evt->size.height);
-#endif
+			if (evt->setup.opengl_mode) return GF_NOT_SUPPORTED;
+			return raw_resize(dr, evt->setup.width, evt->setup.height);
 		}
 	}
 	return GF_OK;
