@@ -51,7 +51,11 @@ static void RT_LoadPrefs(GF_InputService *plug, RTPClient *rtp)
 	*/
 	/*if UDP not available don't try it*/
 	sOpt = gf_modules_get_option((GF_BaseInterface *)plug, "Network", "UDPNotAvailable");
-	if (!rtp->transport_mode && sOpt && !stricmp(sOpt, "yes")) rtp->transport_mode = 1;
+	if (sOpt && !stricmp(sOpt, "yes")) {
+		if (!rtp->transport_mode) rtp->transport_mode = 1;
+		/*turn it off*/
+		gf_modules_set_option((GF_BaseInterface *)plug, "Network", "UDPNotAvailable", "no");
+	}
 	
 	if (!rtp->transport_mode) {
 		sOpt = gf_modules_get_option((GF_BaseInterface *)plug, "Network", "UDPTimeout");
@@ -69,17 +73,6 @@ static void RT_LoadPrefs(GF_InputService *plug, RTPClient *rtp)
 		rtp->time_out = 30000;
 	}
 
-	sOpt = gf_modules_get_option((GF_BaseInterface *)plug, "Streaming", "StreamControl");
-	if (sOpt && !stricmp(sOpt, "independent")) {
-		rtp->stream_control_type = RTSP_CONTROL_INDEPENDENT;
-	} else if (sOpt && !stricmp(sOpt, "RTSP2")) {
-		rtp->stream_control_type = RTSP_CONTROL_RTSP_V2;
-	} else {
-		rtp->stream_control_type = RTSP_CONTROL_AGGREGATE;
-	}
-
-
-
 	/*packet drop emulation*/
 	sOpt = gf_modules_get_option((GF_BaseInterface *)plug, "Streaming", "FirstPacketDrop");
 	if (sOpt) {
@@ -155,6 +148,12 @@ u32 RP_Thread(void *param)
 		i=0;
 		while ((sess = (RTSPSession *)gf_list_enum(rtp->sessions, &i))) {
 			RP_ProcessCommands(sess);
+
+			if (sess->connect_error) {
+				gf_term_on_connect(sess->owner->service, NULL, sess->connect_error);
+				sess->connect_error = 0;
+			}
+
 		}
 
 		gf_mx_v(rtp->mx);
@@ -189,8 +188,9 @@ static Bool RP_CanHandleURL(GF_InputService *plug, const char *url)
 	return 0;
 }
 
-static GF_Err RP_ConnectService(GF_InputService *plug, GF_ClientService *serv, const char *url)
+GF_Err RP_ConnectServiceEx(GF_InputService *plug, GF_ClientService *serv, const char *url, Bool skip_migration)
 {
+	char *session_cache;
 	RTSPSession *sess;
 	RTPClient *priv = (RTPClient *)plug->priv;
 
@@ -200,7 +200,7 @@ static GF_Err RP_ConnectService(GF_InputService *plug, GF_ClientService *serv, c
 	if (priv->dnload) gf_term_download_del(priv->dnload);
 	priv->dnload = NULL;
 
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_SERVICE, ("[RTP] Opening service %s\n", url));
+	GF_LOG(GF_LOG_INFO, GF_LOG_NETWORK, ("[RTP] Opening service %s\n", url));
 
 	/*load preferences*/
 	RT_LoadPrefs(plug, priv);
@@ -208,12 +208,33 @@ static GF_Err RP_ConnectService(GF_InputService *plug, GF_ClientService *serv, c
 	/*start thread*/
 	gf_th_run(priv->th, RP_Thread, priv);
 
+	if (!skip_migration) {
+		session_cache = (char *) gf_modules_get_option((GF_BaseInterface *) plug, "Streaming", "SessionMigrationFile");
+		if (session_cache && session_cache[0]) {
+			FILE *f = fopen(session_cache, "rb");
+			if (f) {
+				fclose(f);
+				GF_LOG(GF_LOG_INFO, GF_LOG_NETWORK, ("[RTP] Restarting RTSP session from %s\n", session_cache));
+				RP_FetchSDP(priv, (char *) session_cache, NULL, (char *) url);
+				return GF_OK;
+			}
+			if (!strncmp(session_cache, "http://", 7)) {
+				GF_LOG(GF_LOG_INFO, GF_LOG_NETWORK, ("[RTP] Restarting RTSP session from %s\n", session_cache));
+				RP_FetchSDP(priv, (char *) session_cache, NULL, (char *) url);
+				return GF_OK;
+			}
+		}
+	}
+
+
 	/*local or remote SDP*/
 	if (strstr(url, "data:application/sdp") || (strnicmp(url, "rtsp", 4) && strstr(url, ".sdp")) ) {
-		RP_FetchSDP(plug, (char *) url, NULL);
+		RP_FetchSDP(priv, (char *) url, NULL, NULL);
+		return GF_OK;
 	}
+	
 	/*rtsp and rtsp over udp*/
-	else if (!strnicmp(url, "rtsp://", 7) || !strnicmp(url, "rtspu://", 8)) {
+	if (!strnicmp(url, "rtsp://", 7) || !strnicmp(url, "rtspu://", 8)) {
 		char *the_url = strdup(url);
 		char *the_ext = strrchr(the_url, '#');
 		if (the_ext) {
@@ -228,16 +249,21 @@ static GF_Err RP_ConnectService(GF_InputService *plug, GF_ClientService *serv, c
 		} else {
 			RP_Describe(sess, 0, NULL);
 		}
+		return GF_OK;
 	}
+
 	/*direct RTP (no control) or embedded data - this means the service is attached to a single channel (no IOD)
 	reply right away*/
-	else {
-		gf_term_on_connect(serv, NULL, GF_OK);
-		RP_SetupObjects(priv);
-	}
+	gf_term_on_connect(serv, NULL, GF_OK);
+	RP_SetupObjects(priv);
 	return GF_OK;
 }
 
+GF_Err RP_ConnectService(GF_InputService *plug, GF_ClientService *serv, const char *url)
+{
+	return RP_ConnectServiceEx(plug, serv, url, 0);
+}
+
 static void RP_FlushCommands(RTPClient *rtp)
 {
 	u32 i, nb_com;
@@ -247,7 +273,8 @@ static void RP_FlushCommands(RTPClient *rtp)
 		nb_com = 0;
 		i=0;
 		while ((sess = (RTSPSession *)gf_list_enum(rtp->sessions, &i))) {
-			nb_com += gf_list_count(sess->rtsp_commands);
+			if (!sess->connect_error)
+				nb_com += gf_list_count(sess->rtsp_commands);
 		}
 		if (!nb_com) break;
 		gf_sleep(10);
@@ -257,15 +284,38 @@ static void RP_FlushCommands(RTPClient *rtp)
 static GF_Err RP_CloseService(GF_InputService *plug)
 {
 	u32 i;
+	const char *opt;
 	RTSPSession *sess;
 	RTPClient *rtp = (RTPClient *)plug->priv;
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_SERVICE, ("[RTP] Closing service\n"));
+	GF_LOG(GF_LOG_DEBUG, GF_LOG_NETWORK, ("[RTP] Closing service\n"));
 
 	RP_FlushCommands(rtp);
-	/*send teardown on all sessions*/
-	i=0;
-	while ((sess = (RTSPSession *)gf_list_enum(rtp->sessions, &i))) {
-		RP_Teardown(sess, NULL);
+
+	opt = gf_modules_get_option((GF_BaseInterface *) plug, "Network", "SessionMigration");
+	if (opt && !strcmp(opt, "yes") ) {
+		opt = gf_modules_get_option((GF_BaseInterface *) plug, "Streaming", "SessionMigrationPause");
+		if (opt && !strcmp(opt, "yes")) {
+			GF_NetworkCommand com;
+			com.command_type = GF_NET_CHAN_PAUSE;
+			com.base.on_channel = NULL;
+			/*send pause on all sessions*/
+			i=0;
+			while ((sess = (RTSPSession *)gf_list_enum(rtp->sessions, &i))) {
+				RP_UserCommand(sess, NULL, &com);
+			}
+		}
+
+		RP_SaveSessionState(rtp);
+	} else {
+		/*remove session state file*/
+		if (rtp->session_state) {
+			gf_delete_file(rtp->session_state);
+		}
+		/*send teardown on all sessions*/
+		i=0;
+		while ((sess = (RTSPSession *)gf_list_enum(rtp->sessions, &i))) {
+			RP_Teardown(sess, NULL);
+		}
 	}
 	RP_FlushCommands(rtp);
 
@@ -282,8 +332,8 @@ static GF_Descriptor *RP_GetServiceDesc(GF_InputService *plug, u32 expect_type,
 	GF_Descriptor *desc;
 	RTPClient *priv = (RTPClient *)plug->priv;
 
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_SERVICE, ("[RTP] Fetching service descriptor\n"));
-	if ((expect_type!=GF_MEDIA_OBJECT_UNDEF) && (expect_type!=GF_MEDIA_OBJECT_SCENE)) {
+	GF_LOG(GF_LOG_DEBUG, GF_LOG_NETWORK, ("[RTP] Fetching service descriptor\n"));
+	if ((expect_type!=GF_MEDIA_OBJECT_UNDEF) && (expect_type!=GF_MEDIA_OBJECT_SCENE) && (expect_type!=GF_MEDIA_OBJECT_UPDATES)) {
 		/*ignore the SDP IOD and regenerate one*/
 		if (priv->session_desc) gf_odf_desc_del(priv->session_desc);
 		priv->session_desc = NULL;
@@ -306,7 +356,7 @@ static GF_Err RP_ConnectChannel(GF_InputService *plug, LPNETCHANNEL channel, con
 	if (upstream) return GF_NOT_SUPPORTED;
 
 
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_SERVICE, ("[RTP] Connecting channel @%08x - %s\n", channel, url));
+	GF_LOG(GF_LOG_DEBUG, GF_LOG_NETWORK, ("[RTP] Connecting channel @%08x - %s\n", channel, url));
 
 	ch = RP_FindChannel(priv, channel, 0, (char *) url, 0);
 	if (ch && (ch->status != RTP_Disconnected) ) return GF_SERVICE_ERROR;
@@ -347,7 +397,13 @@ static GF_Err RP_ConnectChannel(GF_InputService *plug, LPNETCHANNEL channel, con
 
 		return GF_OK;
 	}
-	
+	/*session migration resume - don't send data to the server*/
+	if (ch->status==RTP_SessionResume) {
+		ch->flags |= RTP_CONNECTED;
+		RP_InitStream(ch, 0);
+		RP_ConfirmChannelConnect(ch, GF_OK);
+		return GF_OK;
+	}
 	/*send a DESCRIBE (not a setup) on the channel. If the channel is already created then the
 	describe is skipped and a SETUP is sent directly, otherwise the channel is first described then setup*/
 	if (sess) RP_Describe(sess, es_url, channel);
@@ -363,13 +419,14 @@ static GF_Err RP_DisconnectChannel(GF_InputService *plug, LPNETCHANNEL channel)
 	RTPStream *ch;
 	RTPClient *priv = (RTPClient *)plug->priv;
 
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_SERVICE, ("[RTP] Disconnecting channel @%08x\n", channel));
+	GF_LOG(GF_LOG_DEBUG, GF_LOG_NETWORK, ("[RTP] Disconnecting channel @%08x\n", channel));
 
 	ch = RP_FindChannel(priv, channel, 0, NULL, 0);
 	if (!ch) return GF_STREAM_NOT_FOUND;
 	gf_mx_p(priv->mx);
 	/*disconnect stream BUT DO NOT DELETE IT since we don't store SDP*/
-	RP_DisconnectStream(ch);
+	ch->flags &= ~RTP_CONNECTED;
+	ch->channel = NULL;
 	gf_mx_v(priv->mx);
 	gf_term_on_disconnect(priv->service, channel, GF_OK);
 	return GF_OK;
@@ -380,6 +437,17 @@ static GF_Err RP_ServiceCommand(GF_InputService *plug, GF_NetworkCommand *com)
 	RTPStream *ch;
 	RTPClient *priv = (RTPClient *)plug->priv;
 
+
+	if (com->command_type==GF_NET_SERVICE_HAS_AUDIO) {
+		u32 i;
+		for (i=0; i<gf_list_count(priv->channels); i++) {
+			ch = gf_list_get(priv->channels, i);
+			if (ch->depacketizer->sl_map.StreamType==GF_STREAM_AUDIO)
+				return GF_OK;
+		}
+		return GF_NOT_SUPPORTED;
+	}
+
 	/*ignore commands other than channels one*/
 	if (!com->base.on_channel) {
 		if (com->command_type==GF_NET_IS_CACHABLE) return GF_OK;
@@ -427,13 +495,24 @@ static GF_Err RP_ServiceCommand(GF_InputService *plug, GF_NetworkCommand *com)
 	however we must store some info not carried in SDP*/
 	case GF_NET_CHAN_CONFIG:
 		if (com->cfg.frame_duration) ch->depacketizer->sl_hdr.au_duration = com->cfg.frame_duration;
+		ch->ts_res = com->cfg.sl_config.timestampResolution;
 		return GF_OK;
 
 	case GF_NET_CHAN_PLAY:
-		GF_LOG(GF_LOG_DEBUG, GF_LOG_SERVICE, ("[RTP] Processing play on channel @%08x - %s\n", ch, ch->rtsp ? "RTSP control" : "No control (RTP)" ));
+		GF_LOG(GF_LOG_DEBUG, GF_LOG_NETWORK, ("[RTP] Processing play on channel @%08x - %s\n", ch, ch->rtsp ? "RTSP control" : "No control (RTP)" ));
 		/*is this RTSP or direct RTP?*/
 		ch->flags &= ~RTP_EOS;
 		if (ch->rtsp) {
+			if (ch->status==RTP_SessionResume) {
+				const char *opt = gf_modules_get_option((GF_BaseInterface *) plug, "Streaming", "SessionMigrationPause");
+				if (opt && !strcmp(opt, "yes")) {
+					ch->status = RTP_Connected;
+					com->play.start_range = ch->current_start;
+				} else {
+					ch->status = RTP_Running;
+					return GF_OK;
+				}
+			}
 			RP_UserCommand(ch->rtsp, ch, com);
 		} else {
 			ch->status = RTP_Running;
@@ -446,6 +525,7 @@ static GF_Err RP_ServiceCommand(GF_InputService *plug, GF_NetworkCommand *com)
 			} else {
 				/*direct channel, store current start*/
 				ch->current_start = com->play.start_range;
+				ch->flags |= GF_RTP_NEW_AU;
 				gf_rtp_depacketizer_reset(ch->depacketizer, 0);
 			}
 		}
@@ -453,7 +533,11 @@ static GF_Err RP_ServiceCommand(GF_InputService *plug, GF_NetworkCommand *com)
 	case GF_NET_CHAN_STOP:
 		/*is this RTSP or direct RTP?*/
 		if (ch->rtsp) {
-			RP_UserCommand(ch->rtsp, ch, com);
+			const char *opt = gf_modules_get_option((GF_BaseInterface *) plug, "Network", "SessionMigration");
+			if (opt && !strcmp(opt, "yes")) {
+			} else {
+				RP_UserCommand(ch->rtsp, ch, com);
+			}
 		} else {
 			ch->status = RTP_Connected;
 		}
@@ -466,7 +550,7 @@ static GF_Err RP_ServiceCommand(GF_InputService *plug, GF_NetworkCommand *com)
 		return GF_OK;
 
 	case GF_NET_CHAN_GET_DSI:
-		if (ch->depacketizer->sl_map.configSize) {
+		if (ch->depacketizer && ch->depacketizer->sl_map.configSize) {
 			com->get_dsi.dsi_len = ch->depacketizer->sl_map.configSize;
 			com->get_dsi.dsi = (char*)malloc(sizeof(char)*com->get_dsi.dsi_len);
 			memcpy(com->get_dsi.dsi, ch->depacketizer->sl_map.config, sizeof(char)*com->get_dsi.dsi_len);
@@ -523,7 +607,7 @@ static GF_Err RP_ChannelGetSLP(GF_InputService *plug, LPNETCHANNEL channel, char
 		memset(out_sl_hdr, 0, sizeof(GF_SLHeader));
 		out_sl_hdr->accessUnitEndFlag = 1;
 		out_sl_hdr->accessUnitStartFlag = 1;
-		out_sl_hdr->compositionTimeStamp = (u64) (ch->current_start * 1000);
+		out_sl_hdr->compositionTimeStamp = (u64) (ch->current_start * ch->ts_res);
 		out_sl_hdr->compositionTimeStampFlag = 1;
 		out_sl_hdr->randomAccessPointFlag = 1;
 		*out_reception_status = GF_OK;
@@ -606,8 +690,8 @@ GF_InputService *RTP_Load()
 	plug->priv = priv;
 	
 	priv->time_out = 30000;
-	priv->mx = gf_mx_new();
-	priv->th = gf_th_new();
+	priv->mx = gf_mx_new("RTPDemux");
+	priv->th = gf_th_new("RTPDemux");
 	
 	return plug;
 }
@@ -629,6 +713,10 @@ void RTP_Delete(GF_BaseInterface *bi)
 	}
 	assert(retry);
 
+	if (rtp->session_state) free(rtp->session_state);
+	if (rtp->remote_session_state) free(rtp->remote_session_state);
+
+
 	RP_cleanup(rtp);
 	gf_th_del(rtp->th);
 	gf_mx_del(rtp->mx);
