@@ -41,13 +41,6 @@
 #define RTSP_LANGUAGE		"English"
 
 
-enum
-{
-	RTSP_CONTROL_AGGREGATE = 0,
-	RTSP_CONTROL_INDEPENDENT,
-	RTSP_CONTROL_RTSP_V2,
-};
-
 /*the rtsp/rtp client*/
 typedef struct
 {
@@ -89,8 +82,10 @@ typedef struct
 	/*for single-object control*/
 	u32 media_type;
 
-	u32 stream_control_type;
-
+	/*location of the session state (SDP file)*/
+	char *session_state;
+	char *remote_session_state;
+	char *tmp_buf;
 	/*if set ANNOUNCE (sent by server) will be handled*/
 //	Bool handle_announce;
 } RTPClient;
@@ -100,7 +95,9 @@ enum
 	RTSP_AGG_CONTROL = 1,
 	RTSP_TCP_FLUSH = 1<<1,
 	RTSP_FORCE_INTER = 1<<2,
-	RTSP_WAIT_REPLY = 1<<3
+	RTSP_WAIT_REPLY = 1<<3,
+	RTSP_DSS_SERVER = 1<<4,
+	RTSP_AGG_ONLY = 1<<5,
 };
 
 /*rtsp session*/
@@ -125,6 +122,7 @@ typedef struct _rtp_session
 	Double last_range;
 	u32 command_time;
 	GF_List *rtsp_commands;
+	GF_Err connect_error;
 } RTSPSession;
 
 /*creates new RTSP session handler*/
@@ -142,7 +140,7 @@ void RP_ProcessCommands(RTSPSession *sess);
 enum
 {
 	/*channel is setup and waits for connection request*/
-	RTP_Setup = 0,
+	RTP_Setup,
 	/*waiting for server reply*/
 	RTP_WaitingForAck,
 	/*connection OK*/
@@ -152,7 +150,10 @@ enum
 	/*deconnection OK - a download channel can automatically disconnect when download is done*/
 	RTP_Disconnected,
 	/*service/channel is not (no longer) available/found and should be removed*/
-	RTP_Unavailable
+	RTP_Unavailable,
+
+	RTP_SessionResume
+
 };
 
 
@@ -208,7 +209,7 @@ typedef struct
 	u32 ES_ID;
 	char *control;
 
-	/*rtp recieve buffer*/
+	/*rtp receive buffer*/
 	char buffer[RTP_BUFFER_SIZE];
 	/*set at seek stages to resync app NPT to RTP time*/
 	u32 check_rtp_time;
@@ -222,16 +223,18 @@ typedef struct
 	u32 last_udp_time;
 	/*RTP stats*/
 	u32 rtp_bytes, rtcp_bytes, stat_start_time, stat_stop_time;
+	u32 ts_res;
 } RTPStream;
 
+GF_Err RP_ConnectServiceEx(GF_InputService *plug, GF_ClientService *serv, const char *url, Bool skip_migration);
+
+
 /*creates new RTP stream from SDP info*/
 RTPStream *RP_NewStream(RTPClient *rtp, GF_SDPMedia *media, GF_SDPInfo *sdp, RTPStream *input_stream);
 /*destroys RTP stream */
 void RP_DeleteStream(RTPStream *ch);
 /*resets stream state and inits RTP sockets if ResetOnly is false*/
 GF_Err RP_InitStream(RTPStream *ch, Bool ResetOnly);
-/*disconnect stream but keeps its config alive*/
-void RP_DisconnectStream(RTPStream *ch);
 
 /*RTSP -> RTP de-interleaving callback*/
 GF_Err RP_DataOnTCP(GF_RTSPSession *sess, void *cbck, char *buffer, u32 bufferSize, Bool IsRTCP);
@@ -239,7 +242,7 @@ GF_Err RP_DataOnTCP(GF_RTSPSession *sess, void *cbck, char *buffer, u32 bufferSi
 void RP_ConfirmChannelConnect(RTPStream *ch, GF_Err e);
 
 /*fetch sdp file - stream is the RTP channel this sdp describes, or NULL if session sdp*/
-void RP_FetchSDP(GF_InputService *plug, char *url, RTPStream *stream);
+void RP_FetchSDP(RTPClient *rtp, char *url, RTPStream *stream, char *original_url);
 
 /*locate RTP stream by channel or ES_ID or control*/
 RTPStream *RP_FindChannel(RTPClient *rtp, LPNETCHANNEL ch, u32 ES_ID, char *es_control, Bool remove_stream);
@@ -324,8 +327,12 @@ typedef struct _sdp_fetch
 	RTPStream *chan;
 
 	char *remote_url;
+	char *original_url;
 } SDPFetch;
 
+
+void RP_SaveSessionState(RTPClient *rtp);
+
 #endif
 
 
