@@ -138,7 +138,7 @@ void RP_ProcessCommands(RTSPSession *sess)
 	com->User_Agent = RTSP_CLIENTNAME;
 	com->Accept_Language = RTSP_LANGUAGE;
 	/*if no session assigned and a session ID is valid, use it*/
-	if ((sess->owner->stream_control_type!=RTSP_CONTROL_INDEPENDENT) && sess->session_id && !com->Session)
+	if (sess->session_id && !com->Session)
 		com->Session = sess->session_id;
 
 	e = GF_OK;
@@ -227,9 +227,11 @@ RTSPSession *RP_NewSession(RTPClient *rtp, char *session_control)
 	RTSPSession *tmp;
 	GF_RTSPSession *rtsp;
 
+	if (!session_control) return NULL;
+
 	/*little fix: some servers don't understand DESCRIBE URL/trackID=, so remove the trackID...*/
 	szCtrl = strdup(session_control);
-	szExt = strrchr(szCtrl, '.');
+	szExt = szCtrl ? strrchr(szCtrl, '.') : NULL;
 	if (szExt) {
 		szExt = strchr(szExt, '/');
 		if (szExt) {
@@ -246,6 +248,13 @@ RTSPSession *RP_NewSession(RTPClient *rtp, char *session_control)
 	tmp->owner = rtp;
 	tmp->session = rtsp;
 
+
+	szCtrl = (char *)gf_modules_get_option((GF_BaseInterface *) gf_term_get_service_interface(rtp->service), "Network", "MobileIPEnabled");
+	if (szCtrl && !strcmp(szCtrl, "yes")) {
+		char *ip = (char *)gf_modules_get_option((GF_BaseInterface *) gf_term_get_service_interface(rtp->service), "Network", "MobileIP");
+		gf_rtsp_set_mobile_ip(rtsp, ip);
+	}
+
 	if (rtp->transport_mode) {
 		gf_rtsp_set_buffer_size(rtsp, RTSP_TCP_BUFFER_SIZE);
 	} else {
