@@ -40,7 +40,7 @@ void RP_StopChannel(RTPStream *ch)
 	if (!ch || !ch->rtsp) return;
 
 	ch->flags &= ~RTP_SKIP_NEXT_COM;
-	ch->status = RTP_Disconnected;
+	//ch->status = RTP_Disconnected;
 	//remove interleaved
 	if (gf_rtp_is_interleaved(ch->rtp_ch)) {
 		gf_rtsp_unregister_interleave(ch->rtsp->session, gf_rtp_get_low_interleave_id(ch->rtp_ch));
@@ -64,15 +64,7 @@ Bool RP_SessionActive(RTPStream *ch)
 static void RP_QueueCommand(RTSPSession *sess, RTPStream *ch, GF_RTSPCommand *com, Bool needs_sess_id)
 {
 	if (needs_sess_id) {
-		switch (sess->owner->stream_control_type) {
-		case RTSP_CONTROL_INDEPENDENT:
-			if (!ch) com->Session = sess->session_id;
-			else com->Session = ch->session_id;
-			break;
-		default:
-			com->Session = sess->session_id;
-			break;
-		}
+		com->Session = sess->session_id;
 	}
 	if (gf_mx_try_lock(sess->owner->mx)) {
 		gf_list_add(sess->rtsp_commands, com);
@@ -90,24 +82,54 @@ static void RP_QueueCommand(RTSPSession *sess, RTPStream *ch, GF_RTSPCommand *co
 
 void RP_Setup(RTPStream *ch)
 {
+	u16 def_first_port;
+	const char *opt;
 	GF_RTSPCommand *com;
 	GF_RTSPTransport *trans;
 
 	com = gf_rtsp_command_new();	
 	com->method = strdup(GF_RTSP_SETUP);
 
-	//setup ports if unicast non interleaved
+	def_first_port = 0;
+	opt = gf_modules_get_option((GF_BaseInterface *) gf_term_get_service_interface(ch->owner->service), "Streaming", "ForceFirstPort");
+	if (opt) def_first_port = atoi(opt);
+
+	opt = gf_modules_get_option((GF_BaseInterface *) gf_term_get_service_interface(ch->owner->service), "Streaming", "ForceMulticastIP");
+
+	//setup ports if unicast non interleaved or multicast
 	if (gf_rtp_is_unicast(ch->rtp_ch) && (ch->owner->transport_mode != 1) && !gf_rtp_is_interleaved(ch->rtp_ch) ) {
-		u16 def_first_port = 0;
-		const char *opt = gf_modules_get_option((GF_BaseInterface *) gf_term_get_service_interface(ch->owner->service), "Streaming", "ForceFirstPort");
-		if (opt) def_first_port = atoi(opt);
+		gf_rtp_set_ports(ch->rtp_ch, def_first_port);
+	} else if (opt) {
 		gf_rtp_set_ports(ch->rtp_ch, def_first_port);
 	}
 
 	trans = gf_rtsp_transport_clone(gf_rtp_get_transport(ch->rtp_ch));
 
-	/*override transport*/
-	if (ch->rtsp->flags & RTSP_FORCE_INTER) {
+	/*some servers get confused when trying to resetup on the same remote ports, so reset info*/
+	trans->port_first = trans->port_last = 0;
+	trans->SSRC = 0;
+
+	/*override transport: */
+		/*1: multicast forced*/
+	opt = gf_modules_get_option((GF_BaseInterface *) gf_term_get_service_interface(ch->owner->service), "Streaming", "ForceMulticastIP");
+	if (opt) {
+		trans->IsUnicast = 0;
+		trans->destination = strdup(opt);
+		opt = gf_modules_get_option((GF_BaseInterface *) gf_term_get_service_interface(ch->owner->service), "Streaming", "ForceMulticastTTL");
+		trans->TTL = opt ? atoi(opt) : 127;
+		if (trans->Profile) free(trans->Profile);
+		trans->Profile = strdup(GF_RTSP_PROFILE_RTP_AVP);
+		if (!(ch->rtsp->flags & RTSP_DSS_SERVER) ) {
+			trans->port_first = trans->client_port_first;
+			trans->port_last = trans->client_port_last;
+			/*this is correct but doesn't work with DSS: the server expects "client_port" to indicate 
+			the multicast port, not "port" - this will send both*/
+			//trans->client_port_first = trans->client_port_last = 0;
+		}
+		gf_rtp_setup_transport(ch->rtp_ch, trans, NULL);
+	}
+		/*2: RTP over RTSP forced*/
+	else if (ch->rtsp->flags & RTSP_FORCE_INTER) {
 		if (trans->Profile) free(trans->Profile);
 		trans->Profile = strdup(GF_RTSP_PROFILE_RTP_AVP_TCP);
 		gf_rtp_setup_transport(ch->rtp_ch, trans, NULL);
@@ -117,9 +139,6 @@ void RP_Setup(RTPStream *ch)
 		free(trans->source);
 		trans->source = NULL;
 	}
-	/*some servers get confused when trying to resetup on the same remote ports, so reset info*/
-	trans->port_first = trans->port_last = 0;
-	trans->SSRC = 0;
 
 	/*turn off interleaving in case of re-setup, some servers don't like it (we still signal it
 	through RTP/AVP/TCP profile so it's OK)*/
@@ -188,24 +207,12 @@ void RP_ProcessSetup(RTSPSession *sess, GF_RTSPCommand *com, GF_Err e)
 	if (!ch) goto exit;
 
 	/*assign session ID*/
-	switch (sess->owner->stream_control_type) {
-	case RTSP_CONTROL_INDEPENDENT:
-		if (!sess->rtsp_rsp->Session) {
-			e = GF_SERVICE_ERROR;
-			goto exit;
-		}
-		if (!ch->session_id) ch->session_id = strdup(sess->rtsp_rsp->Session);
-		assert(!sess->session_id);
-		break;
-	default:
-		if (!sess->rtsp_rsp->Session) {
-			e = GF_SERVICE_ERROR;
-			goto exit;
-		}
-		if (!sess->session_id) sess->session_id = strdup(sess->rtsp_rsp->Session);
-		assert(!ch->session_id);
-		break;
+	if (!sess->rtsp_rsp->Session) {
+		e = GF_SERVICE_ERROR;
+		goto exit;
 	}
+	if (!sess->session_id) sess->session_id = strdup(sess->rtsp_rsp->Session);
+	assert(!ch->session_id);
 
 	/*transport setup: break at the first correct transport */
 	i=0;
@@ -315,6 +322,7 @@ exit:
 	com->user_data = NULL;
 	if (e) {
 		if (!ch_desc) {
+			sess->connect_error = e;
 			return e;
 		} else if (ch) {
 			RP_ConfirmChannelConnect(ch, e);
@@ -340,6 +348,14 @@ void RP_Describe(RTSPSession *sess, char *esd_url, LPNETCHANNEL channel)
 		ch = RP_FindChannel(sess->owner, channel, 0, esd_url, 0);
 		if (ch) {
 			if (!ch->channel) ch->channel = channel;
+			switch (ch->status) {
+			case RTP_Connected:
+			case RTP_Running:
+				RP_ConfirmChannelConnect(ch, GF_OK);
+				return;
+			default:
+				break;
+			}
 			ch_desc = (ChannelDescribe *)malloc(sizeof(ChannelDescribe));
 			ch_desc->esd_url = esd_url ? strdup(esd_url) : NULL;
 			ch_desc->channel = channel;
@@ -391,10 +407,10 @@ Bool RP_PreprocessUserCom(RTSPSession *sess, GF_RTSPCommand *com)
 
 	ch_ctrl = NULL;
 	if (strcmp(com->method, GF_RTSP_TEARDOWN)) ch_ctrl = (ChannelControl *)com->user_data;
-	if (!ch_ctrl) return 1;
+	if (!ch_ctrl || !ch_ctrl->ch) return 1;
 	ch = ch_ctrl->ch;
 	
-	if (!channel_is_valid(sess->owner, ch)) {
+	if (!ch->channel || !channel_is_valid(sess->owner, ch)) {
 		free(ch_ctrl);
 		com->user_data = NULL;
 		return 0;
@@ -403,16 +419,6 @@ Bool RP_PreprocessUserCom(RTSPSession *sess, GF_RTSPCommand *com)
 	assert(ch->rtsp == sess);
 	assert(ch->channel==ch_ctrl->com.base.on_channel);
 
-	if (sess->owner->stream_control_type == RTSP_CONTROL_INDEPENDENT) {
-		/*re-SETUP failed*/
-		if (!ch->session_id) {
-			e = GF_SERVICE_ERROR;
-			goto err_exit;
-		}
-		com->Session = ch->session_id;
-		return 1;
-	}
-
 	skip_it = 0;
 	if (!com->Session) {
 		/*re-SETUP failed*/
@@ -466,22 +472,31 @@ void RP_ProcessUserCommand(RTSPSession *sess, GF_RTSPCommand *com, GF_Err e)
 
 	ch_ctrl = (ChannelControl *)com->user_data;
 	ch = ch_ctrl->ch;
+	if (ch) {
+		if (!ch->channel || !channel_is_valid(sess->owner, ch)) {
+			free(ch_ctrl);
+			com->user_data = NULL;
+			return;
+		}
 
-	if (!channel_is_valid(sess->owner, ch)) {
-		free(ch_ctrl);
-		com->user_data = NULL;
-		return;
+		assert(ch->channel==ch_ctrl->com.base.on_channel);
 	}
 
-	assert(ch->channel==ch_ctrl->com.base.on_channel);
-
 	/*some consistency checking: on interleaved sessions, some servers do NOT reply to the 
 	teardown. If our command is STOP just skip the error notif*/
 	if (e) {
 		if (!strcmp(com->method, GF_RTSP_TEARDOWN)) {
 			goto process_reply;
 		} else {
-			goto err_exit;
+			/*spec is not really clear about what happens if the server doesn't support 
+			non aggregated operations. Since this happen only on pause/play, we consider 
+			that no error occured and wait for next play*/
+			if (sess->rtsp_rsp->ResponseCode == NC_RTSP_Only_Aggregate_Operation_Allowed) {
+				sess->flags |= RTSP_AGG_ONLY;
+				sess->rtsp_rsp->ResponseCode = NC_RTSP_OK;
+			} else {
+				goto err_exit;
+			}
 		}
 	}
 
@@ -563,10 +578,12 @@ process_reply:
 		}
 		ch->flags &= ~RTP_SKIP_NEXT_COM;
 	} else if (ch_ctrl->com.command_type == GF_NET_CHAN_PAUSE) {
-		SkipCommandOnSession(ch);
-		ch->flags &= ~RTP_SKIP_NEXT_COM;
+		if (ch) {
+			SkipCommandOnSession(ch);
+			ch->flags &= ~RTP_SKIP_NEXT_COM;
+
+		}
 	} else if (ch_ctrl->com.command_type == GF_NET_CHAN_STOP) {
-		assert(0);
 	}
 	free(ch_ctrl);
 	com->user_data = NULL;
@@ -574,14 +591,17 @@ process_reply:
 
 
 err_exit:
-	ch->status = RTP_Disconnected;
 	gf_term_on_command(sess->owner->service, &ch_ctrl->com, e);
-	gf_rtsp_reset_aggregation(ch->rtsp->session);
-	ch->check_rtp_time = 0;
+	if (ch) {
+		ch->status = RTP_Disconnected;
+		gf_rtsp_reset_aggregation(ch->rtsp->session);
+		ch->check_rtp_time = 0;
+	}
 	free(ch_ctrl);
 	com->user_data = NULL;
 }
 
+#if 0
 static void RP_FlushAndTearDown(RTSPSession *sess)
 {
 	GF_RTSPCommand *com;
@@ -608,7 +628,7 @@ static void RP_FlushAndTearDown(RTSPSession *sess)
 	com->method = strdup(GF_RTSP_TEARDOWN);
 	RP_QueueCommand(sess, NULL, com, 1);
 }
-
+#endif
 
 
 void RP_UserCommand(RTSPSession *sess, RTPStream *ch, GF_NetworkCommand *command)
@@ -616,16 +636,28 @@ void RP_UserCommand(RTSPSession *sess, RTPStream *ch, GF_NetworkCommand *command
 	RTPStream *a_ch;
 	ChannelControl *ch_ctrl;
 	u32 i;
+	Bool needs_setup = 0;
 	GF_RTSPCommand *com;
 	GF_RTSPRange *range;
 
-	assert(ch->rtsp==sess);
+	switch (command->command_type) {
+	case GF_NET_CHAN_PLAY:
+	case GF_NET_CHAN_RESUME:
+		needs_setup = 1;
+		break;
+	case GF_NET_CHAN_PAUSE:
+	case GF_NET_CHAN_STOP:
+		break;
+	default:
+		gf_term_on_command(sess->owner->service, command, GF_NOT_SUPPORTED);
+		return;
+	}
+
 	
 	/*we may need to re-setup stream/session*/
-	if ( (command->command_type==GF_NET_CHAN_PLAY) || (command->command_type==GF_NET_CHAN_RESUME) || (command->command_type==GF_NET_CHAN_PAUSE)) {
+	if (needs_setup) {
 		if (ch->status == RTP_Disconnected) {
-			if ( (sess->owner->stream_control_type==RTSP_CONTROL_AGGREGATE)
-				&& (sess->flags & RTSP_AGG_CONTROL)) {
+			if (sess->flags & RTSP_AGG_CONTROL) {
 				i=0;
 				while ((a_ch = (RTPStream *)gf_list_enum(sess->owner->channels, &i))) {
 					if (a_ch->rtsp != sess) continue;
@@ -686,52 +718,58 @@ void RP_UserCommand(RTSPSession *sess, RTPStream *ch, GF_NetworkCommand *command
 			com->Range = range;
 		}
 
-		if (!(sess->flags & RTSP_AGG_CONTROL) && strlen(ch->control)) com->ControlString = strdup(ch->control);
+		if (sess->flags & RTSP_AGG_CONTROL) 
+			SkipCommandOnSession(ch);
+		else if (strlen(ch->control)) 
+			com->ControlString = strdup(ch->control);
+
 		if (RP_SessionActive(ch)) {
 			if (!com->ControlString && ch->control) com->ControlString = strdup(ch->control);
 		} else {
-			if ((sess->owner->stream_control_type!=RTSP_CONTROL_INDEPENDENT) && com->ControlString) {
+			if (com->ControlString) {
 				free(com->ControlString);
 				com->ControlString=NULL;
 			}
 		}
 
 	} else if (command->command_type==GF_NET_CHAN_PAUSE) {
-		range = gf_rtsp_range_new();
-		range->start = ch->range_start;
-		range->end = ch->range_end;
 		com->method = strdup(GF_RTSP_PAUSE);
-		/*update current time*/
-		ch->current_start += gf_rtp_get_current_time(ch->rtp_ch);
-		range->start = ch->current_start;
-		range->end = -1.0;
-		com->Range = range;
-
-		ch->stat_stop_time = gf_sys_clock();
-
+		if (ch) {
+			range = gf_rtsp_range_new();
+			/*update current time*/
+			ch->current_start += gf_rtp_get_current_time(ch->rtp_ch);
+			ch->stat_stop_time = gf_sys_clock();
+			range->start = ch->current_start;
+			range->end = -1.0;
+			com->Range = range;
+		} 
 	}
-	//nb: we could actually send a PAUSE in order to keep the session alive
-	//but let's be nice to the server
 	else if (command->command_type==GF_NET_CHAN_STOP) {
 		ch->current_start = 0;
 		ch->stat_stop_time = gf_sys_clock();
-		RP_StopChannel(ch);
-		if (com) gf_rtsp_command_del(com);
 
-		/*use stream-control*/
-		switch (ch->owner->stream_control_type) {
-		case RTSP_CONTROL_AGGREGATE:
-			/*last stream running*/
-			//if (!RP_SessionActive(ch)) RP_FlushAndTearDown(sess);
-			break;
-		/*FIXME - according to trhe current draft, the stream's session must be paused before ...*/
-		case RTSP_CONTROL_RTSP_V2:
-		case RTSP_CONTROL_INDEPENDENT:
-		default:
-			RP_Teardown(sess, ch);
-			break;
+		ch->status = RTP_Connected;
+		RP_InitStream(ch, 1);
+
+		/*if server only support aggregation on pause, skip the command or issue
+		a teardown if last active stream*/
+		if (ch->rtsp->flags & RTSP_AGG_ONLY) {
+			RP_StopChannel(ch);
+			if (com) gf_rtsp_command_del(com);
+			if (!RP_SessionActive(ch))
+				RP_Teardown(sess, ch);
+			return;
+		} 
+		/* otherwise send a PAUSE on the stream */
+		else {
+			range = gf_rtsp_range_new();
+			range->start = 0;
+			range->end = -1;
+			com->method = strdup(GF_RTSP_PAUSE);
+			com->Range = range;
+			/*only pause the specified stream*/
+			if (ch->control) com->ControlString = strdup(ch->control);
 		}
-		return;
 	} else {
 		gf_term_on_command(sess->owner->service, command, GF_NOT_SUPPORTED);
 		gf_rtsp_command_del(com);
@@ -767,26 +805,15 @@ void RP_Teardown(RTSPSession *sess, RTPStream *ch)
 {
 	GF_RTSPCommand *com;
 
-	switch (sess->owner->stream_control_type) {
-	case RTSP_CONTROL_AGGREGATE:
-		/*we need a session id*/
-		if (!sess->session_id) return;
-		/*ignore teardown on channels*/
-		if ((sess->flags & RTSP_AGG_CONTROL) && ch) return;
-		break;
-	case RTSP_CONTROL_RTSP_V2:
-		/*we need a session id*/
-		if (!sess->session_id) return;
-		/*do not ignore teardown on channels*/
-		break;
-	case RTSP_CONTROL_INDEPENDENT:
-		/*todo*/
-		break;
-	}
+	/*we need a session id*/
+	if (!sess->session_id) return;
+	/*ignore teardown on channels*/
+	if ((sess->flags & RTSP_AGG_CONTROL) && ch) return;
 
 	com = gf_rtsp_command_new();
 	com->method = strdup(GF_RTSP_TEARDOWN);
-	if (ch && ch->control && (sess->owner->stream_control_type == RTSP_CONTROL_RTSP_V2)) {
+	/*this only works in RTSP2*/
+	if (ch && ch->control) {
 		com->ControlString = strdup(ch->control);
 		com->user_data = ch;
 	}
