@@ -23,13 +23,13 @@
  */
 
 
+/*SDL*/
+#include <SDL.h>
+
 /*driver interfaces*/
 #include <gpac/modules/audio_out.h>
 #include <gpac/modules/video_out.h>
 #include <gpac/thread.h>
-#include <gpac/list.h>
-/*SDL*/
-#include <SDL.h>
 
 /*SDL init routines*/
 Bool SDLOUT_InitSDL();
@@ -54,7 +54,9 @@ typedef struct
 
 	u32 width, height;
 
-	Bool is_3D_out;
+	SDL_Surface *offscreen_gl;
+
+	u32 output_3d_type;
 	void *os_handle;
 } SDLVidCtx;
 
