@@ -238,7 +238,8 @@ static GFINLINE int gray_move_to( EVG_Vector*  to, EVG_Raster   raster)
 static void gray_render_scanline( TRaster *raster,  TCoord  ey, TPos x1, TCoord y1, TPos x2, TCoord y2)
 {
 	TCoord  ex1, ex2, fx1, fx2, delta;
-	long    p, first, dx;
+	long    p, first;
+	long dx;
 	int     incr, lift, mod, rem;
 	
 	dx = x2 - x1;
@@ -324,7 +325,8 @@ static GFINLINE void gray_render_line(TRaster *raster, TPos  to_x, TPos  to_y, i
 {
 	TCoord  min, max;
 	TCoord  ey1, ey2, fy1, fy2;
-	TPos    dx, dy, x, x2;
+	TPos    x, x2;
+	long dx, dy;
 	long    p, first;
 	int     delta, rem, mod, lift, incr;
 
@@ -748,6 +750,16 @@ int evg_raster_render(EVG_Raster raster, EVG_Raster_Params*  params)
 			sl->num = 0;
 		}
 	}
+
+#if 0
+	for (i=0; i<raster->max_lines; i++) {
+		free(raster->scanlines[i].cells);
+	}
+	free(raster->scanlines);
+	raster->scanlines = 0;
+	raster->max_lines = 0;
+#endif
+
 	return 0;
 }
 
