@@ -346,3 +346,192 @@ GF_Err evg_surface_clear_rgb32(GF_SURFACE surf, GF_IRect rc, GF_Color col)
 }
 
 
+
+/*
+		32 bit RGBA
+*/
+
+static void overmask_rgba(u32 src, char *dst, u32 alpha)
+{
+	u8 srca = GF_COL_A(src);
+	u8 srcr = GF_COL_R(src);
+	u8 srcg = GF_COL_G(src);
+	u8 srcb = GF_COL_B(src);
+	u8 dsta = dst[3];
+	srca = mul255(srca, alpha);
+	if (dsta) {
+		u8 dstr = dst[0];
+		u8 dstg = dst[1];
+		u8 dstb = dst[2];
+		dst[0] = mul255(srca, srcr - dstr) + dstr;
+		dst[1] = mul255(srca, srcg - dstg) + dstg;
+		dst[2] = mul255(srca, srcb - dstb) + dstb;
+		dst[3] = mul255(srca, srca) + mul255(255-srca, dsta);
+	} else {
+		dst[0] = srcr;
+		dst[1] = srcg;
+		dst[2] = srcb;
+		dst[3] = srca;
+	}
+}
+
+static void overmask_rgba_const_run(u32 src, char *dst, u32 count)
+{
+	u8 srca = GF_COL_A(src);
+	u8 srcr = GF_COL_R(src);
+	u8 srcg = GF_COL_G(src);
+	u8 srcb = GF_COL_B(src);
+
+	while (count) {
+		u8 dsta = dst[3];
+		/*special case for RGBA: if dst alpha is 0, consider the surface is empty and copy pixel*/
+		if (dsta) {
+			u8 dstr = dst[0];
+			u8 dstg = dst[1];
+			u8 dstb = dst[2];
+			dst[0] = (u8) mul255(srca, srcr - dstr) + dstr;
+			dst[1] = (u8) mul255(srca, srcg - dstg) + dstg;
+			dst[2] = (u8) mul255(srca, srcb - dstb) + dstb;
+			dst[3] = (u8) mul255(srca, srca) + mul255(255-srca, dsta);
+		} else {
+			dst[0] = srcr;
+			dst[1] = srcg;
+			dst[2] = srcb;
+			dst[3] = srca;
+		}
+		dst+=4;
+		count--;
+	}
+}
+
+void evg_rgba_fill_const(s32 y, s32 count, EVG_Span *spans, EVGSurface *surf)
+{
+	u32 col = surf->fill_col;
+	u32 new_a, fin, col_no_a;
+	u8 a, r, g, b;
+	char *dst = (surf->pixels + y * surf->stride);
+	char *p;
+	s32 i;
+	u32 len;
+	u8 aa_lev = surf->AALevel;
+
+	a = GF_COL_A(col);
+	r = GF_COL_R(col);
+	g = GF_COL_G(col);
+	b = GF_COL_B(col);
+	col_no_a = col & 0x00FFFFFF;
+
+	for (i=0; i<count; i++) {
+		if (spans[i].coverage<aa_lev) continue;
+		p = dst + spans[i].x*4;
+		len = spans[i].len;
+	
+		if (spans[i].coverage != 0xFF) {
+			new_a = spans[i].coverage;
+			fin = (new_a<<24) | col_no_a;
+			overmask_rgba_const_run(fin, p, len);
+		} else {
+			while (len--) {
+				*(p) = r;
+				*(p+1) = g;
+				*(p+2) = b;
+				*(p+3) = a;
+				p += 4;
+			}
+		}
+	}
+}
+
+void evg_rgba_fill_const_a(s32 y, s32 count, EVG_Span *spans, EVGSurface *surf)
+{
+	u32 *dst = (u32 *) (surf->pixels + y * surf->stride);
+	u32 a, fin, col_no_a;
+	u8 aa_lev = surf->AALevel;
+	s32 i;
+
+	a = GF_COL_A(surf->fill_col);
+	col_no_a = surf->fill_col & 0x00FFFFFF;
+
+	for (i=0; i<count; i++) {
+		if (spans[i].coverage<aa_lev) continue;
+		fin = mul255(a, spans[i].coverage);
+		fin = (fin<<24) | col_no_a;
+		overmask_rgba_const_run(fin, (char*) &dst[spans[i].x], spans[i].len);
+	}
+}
+
+
+void evg_rgba_fill_var(s32 y, s32 count, EVG_Span *spans, EVGSurface *surf)
+{
+	u8 *dst = surf->pixels + y * surf->stride;
+	u8 *p;
+	u8 spanalpha, col_a;
+	s32 i;
+	u32 len;
+	u32 *col;
+	u8 aa_lev = surf->AALevel;
+
+	for (i=0; i<count; i++) {
+		if (spans[i].coverage<aa_lev) continue;
+		p = dst + spans[i].x * 4;
+		len = spans[i].len;
+		spanalpha = spans[i].coverage;
+		surf->sten->fill_run(surf->sten, surf, spans[i].x, y, len);
+		col = surf->stencil_pix_run;
+		while (len--) {
+			col_a = GF_COL_A(*col);
+			if (col_a) {
+				if ((spanalpha!=0xFF) || (col_a != 0xFF)) {
+					overmask_rgba(*col, p, spanalpha);
+				} else {
+					u32 _col = *col;
+					p[0] = GF_COL_R(_col);
+					p[1] = GF_COL_G(_col);
+					p[2] = GF_COL_B(_col);
+					p[3] = GF_COL_A(_col);
+				}
+			}
+			col++;
+			p += 4;
+		}
+	}
+}
+
+GF_Err evg_surface_clear_rgba(GF_SURFACE surf, GF_IRect rc, GF_Color col)
+{
+	u8 *data;
+	u8 a, r, g, b;
+	u32 x, y, w, h, st, sx, sy;
+	EVGSurface *_this = (EVGSurface *)surf;
+	st = _this->stride;
+
+	h = rc.height;
+	w = rc.width;
+	sx = 4 * rc.x;
+	sy = rc.y;
+
+	a = GF_COL_A(col);
+	r = GF_COL_R(col);
+	g = GF_COL_G(col);
+	b = GF_COL_B(col);
+
+	if (a) {
+		for (y = 0; y < h; y++) {
+			data = (_this ->pixels + (sy+y)* st + sx);
+			for (x = 0; x < w; x++) {
+				*(data) = r;
+				*(data+1) = g;
+				*(data+2) = b;
+				*(data+3) = a;
+				data += 4;
+			}
+		}
+	} else {
+		u32 sw = 4*w;
+		for (y = 0; y < h; y++) {
+			memset(_this ->pixels + (sy+y)* st + sx, 0, sizeof(char)*sw);
+		}
+	}
+	return GF_OK;
+}
+
