@@ -25,7 +25,7 @@
 #include "rast_soft.h"
 
 EVGStencil *evg_solid_brush();
-EVGStencil *evg_gf_sr_texture_brush();
+EVGStencil *evg_texture_brush();
 EVGStencil *evg_linear_gradient_brush();
 EVGStencil *evg_radial_gradient_brush();
 
@@ -147,7 +147,7 @@ GF_STENCIL evg_stencil_new(GF_Raster2D *_dr, GF_StencilType type)
 		st = evg_radial_gradient_brush();
 		break;
 	case GF_STENCIL_TEXTURE:
-		st = evg_gf_sr_texture_brush();
+		st = evg_texture_brush();
 		break;
 	default:
 		return 0L;
@@ -225,20 +225,30 @@ GF_Err evg_stencil_set_brush_color(GF_STENCIL st, GF_Color c)
 
 static void lgb_fill_run(EVGStencil *p, EVGSurface *surf, s32 x, s32 y, u32 count) 
 {
+	Bool has_cmat, has_a;
 	Fixed _res;
-	s32 val, inc;
+	s32 val;
+	u32 col, ca;
 	u32 *data = surf->stencil_pix_run;
 	u32 shifter = (EVGGRADIENTSCALEBITS - EVGGRADIENTBITS);
 	EVG_LinearGradient *_this = (EVG_LinearGradient *) p;
 
+	has_cmat = _this->cmat.identity ? 0 : 1;
+	has_a = (_this->alpha==0xFF) ? 0 : 1;
+
 	/*no need to move x & y to fixed*/
 	_res = (Fixed) (x * _this->smat.m[0] + y * _this->smat.m[1] + _this->smat.m[2]);
-	/*but remove val to int*/
-	val = FIX2INT(_res);
-	inc = FIX2INT(_this->smat.m[0]);
 	while (count) {
-		*data++ = gradient_get_color((EVG_BaseGradient *)_this, (val >> shifter) ); 
-		val += inc;
+		val = FIX2INT(_res);
+		_res += _this->smat.m[0];
+
+		col = gradient_get_color((EVG_BaseGradient *)_this, (val >> shifter) );
+		if (has_a) {
+			ca = ((GF_COL_A(col) + 1) * _this->alpha) >> 8;
+			col = ( ((ca<<24) & 0xFF000000) ) | (col & 0x00FFFFFF);
+		} 
+		if (has_cmat) col = gf_cmx_apply(&p->cmat, col);
+		*data++ = col;
 		count--;
 	}
 }
@@ -286,6 +296,7 @@ EVGStencil *evg_linear_gradient_brush()
 	tmp->type = GF_STENCIL_LINEAR_GRADIENT;
 	for(i=0; i<EVGGRADIENTSLOTS; i++) tmp->pos[i]=-1;
 
+	tmp->alpha = 0xFF;
 	evg_stencil_set_linear_gradient((EVGStencil *)tmp, 0, 0, FIX_ONE, 0);
 	return (EVGStencil *) tmp;
 }
@@ -298,9 +309,9 @@ EVGStencil *evg_linear_gradient_brush()
 static void rg_fill_run(EVGStencil *p, EVGSurface *surf, s32 _x, s32 _y, u32 count) 
 {
 	Fixed x, y, dx, dy, b, val;
-	Bool has_cmat;
+	Bool has_cmat, has_a;
 	s32 pos;
-	u32 col;
+	u32 col, ca;
 	u32 *data = surf->stencil_pix_run;
 	EVG_RadialGradient *_this = (EVG_RadialGradient *) p;
 
@@ -309,6 +320,7 @@ static void rg_fill_run(EVGStencil *p, EVGSurface *surf, s32 _x, s32 _y, u32 cou
 	gf_mx2d_apply_coords(&_this->smat, &x, &y);
 
 	has_cmat = _this->cmat.identity ? 0 : 1;
+	has_a = (_this->alpha==0xFF) ? 0 : 1;
 
 	dx = x - _this->d_f.x;
 	dy = y - _this->d_f.y;
@@ -317,12 +329,15 @@ static void rg_fill_run(EVGStencil *p, EVGSurface *surf, s32 _x, s32 _y, u32 cou
 		val = gf_mulfix(b, b) + gf_mulfix(_this->rad, gf_mulfix(dx, dx)+gf_mulfix(dy, dy));
 		b += gf_sqrt(val);
 		pos = FIX2INT(EVGGRADIENTBUFFERSIZE*b);
-		if (has_cmat) {
-			col = gradient_get_color((EVG_BaseGradient *)_this, pos);
-			*data++ = gf_cmx_apply(&p->cmat, col);
-		} else {
-		*data++ = gradient_get_color((EVG_BaseGradient *)_this, pos);
-		}
+
+		col = gradient_get_color((EVG_BaseGradient *)_this, pos);
+		if (has_a) {
+			ca = ((GF_COL_A(col) + 1) * _this->alpha) >> 8;
+			col = ( ((ca<<24) & 0xFF000000) ) | (col & 0x00FFFFFF);
+		} 
+		if (has_cmat) col = gf_cmx_apply(&p->cmat, col);		
+		*data++ = col;
+
 		dx += _this->d_i.x;
 		dy += _this->d_i.y;
 		count--;
@@ -362,6 +377,7 @@ EVGStencil *evg_radial_gradient_brush()
 	tmp->center.x = tmp->center.y = FIX_ONE/2;
 	tmp->focus = tmp->center;
 	tmp->radius = tmp->center;
+	tmp->alpha = 0xFF;
 	return (EVGStencil *) tmp;
 }
 
@@ -381,16 +397,15 @@ GF_Err evg_stencil_set_radial_gradient(GF_STENCIL st, Fixed cx, Fixed cy, Fixed
 }
 
 /*
-	Texture stencil 
+	Texture stencil - this is just a crude texture mapper, it lacks precision and filtering ...
 */
+
+#if 0
 static s32
 mul255(s32 a, s32 b)
 {
 	return ((a+1) * b) >> 8;
 }
-
-#if 0
-/*thx to charcoal for the bilinear filter*/
 static u32 EVG_LERP(u32 c0, u32 c1, u8 t)
 {
 	s32 a0, r0, g0, b0;
@@ -427,14 +442,20 @@ static void bmp_fill_run(EVGStencil *p, EVGSurface *surf, s32 _x, s32 _y, u32 co
 	x = INT2FIX(_x);
 	y = INT2FIX(_y);
 	gf_mx2d_apply_coords(&_this->smat, &x, &y);
+
+	/*ugly hack we may have a numerical stability issue*/
+	if (ABS(x) < FIX_ONE/10) x = 0;
+	if (ABS(y) < FIX_ONE/10) y = 0;
+	
 	_fd = INT2FIX(_this->width); 
 	repeat_s = _this->mod & GF_TEXTURE_REPEAT_S;
 	if (!repeat_s && (x < - _fd)) x = 0;
 	while (x<0) x += _fd;
+
 	_fd = INT2FIX(_this->height); 
 	repeat_t = _this->mod & GF_TEXTURE_REPEAT_T;
 	if (!repeat_t && (y < - _fd)) y = 0;
-	while (y<0) y += _fd;
+	while (y<0) y += _fd;	
 
 	y0 = (s32) FIX2INT(y);
 	has_alpha = (_this->alpha != 255) ? 1 : 0;
@@ -444,6 +465,7 @@ static void bmp_fill_run(EVGStencil *p, EVGSurface *surf, s32 _x, s32 _y, u32 co
 
 	while (count) {
 		x0 = FIX2INT(x);
+		assert((s32)x0 >=0);
 
 		if (repeat_s) {
 			x0 = (x0) % _this->width;
@@ -451,12 +473,16 @@ static void bmp_fill_run(EVGStencil *p, EVGSurface *surf, s32 _x, s32 _y, u32 co
 			x0 = MIN(x0, _this->width-1);
 		}
 		x += _this->inc_x;
+		if (x<0) x+=INT2FIX(_this->width);
 		
 		y0 = FIX2INT(y);
+		assert((s32)y0 >=0);
 		if (repeat_t) {
 			y0 = (y0) % _this->height;
-		} else if (y0 >= _this->width) y0 = _this->height-1;
+		} else if (y0 >= _this->height) 
+			y0 = _this->height-1;
 		y += _this->inc_y;
+		if (y<0) y+=INT2FIX(_this->height);
 
 		pix = _this->tx_get_pixel(_this->pixels + _this->stride*y0 + _this->Bpp*x0);
 
@@ -513,20 +539,22 @@ static void bmp_fill_run_straight(EVGStencil *p, EVGSurface *surf, s32 _x, s32 _
 	u32 *data = surf->stencil_pix_run;
 	EVG_Texture *_this = (EVG_Texture *) p;
 
-	/*get texture coords in FIXED*/
+	/*get texture coords in FIXED - offset*/
 	x = _this->smat.m[0]*_x + _this->smat.m[2];
 	y = _this->smat.m[4]*_y + _this->smat.m[5];
 
+	/*ugly hack we may have a numerical stability issue*/
+	if (ABS(x)< FIX_ONE/10) x = 0;
+	if (ABS(y)< FIX_ONE/10) y = 0;
+
 	/* and move in absolute coords*/
 	_fdim = INT2FIX(_this->width);
-	if (!(_this->mod & GF_TEXTURE_REPEAT_S)) {
-		if (x<- _fdim) x=0;
-		repeat_s = 0;
-	} else repeat_s = 1;
-
+	repeat_s = (_this->mod & GF_TEXTURE_REPEAT_S);
+	if (!repeat_s && (x <- _fdim)) x=0;
 	while (x<0) x += _fdim;
+
 	_fdim = INT2FIX(_this->height);
-	if (!(_this->mod & GF_TEXTURE_REPEAT_T) && (y<- _fdim)) y=0;
+	if (!(_this->mod & GF_TEXTURE_REPEAT_T) && (y <- _fdim)) y = 0;
 	while (y<0) y += _fdim;
 
 	y0 = FIX2INT(y);
@@ -582,7 +610,7 @@ void evg_bmp_init(EVGStencil *p)
 }
 
 
-EVGStencil *evg_gf_sr_texture_brush()
+EVGStencil *evg_texture_brush()
 {
 	EVG_Texture *tmp;
 	GF_SAFEALLOC(tmp, EVG_Texture);
@@ -612,7 +640,7 @@ u32 get_pix_565(char *pix) { u16 val = *(u16*)pix; return GF_COL_ARGB(0xFF,  (u8
 u32 get_pix_grey(char *pix) { u8 val = *pix; return GF_COL_ARGB(0xFF, val, val, val); }
 u32 get_pix_alphagrey(char *pix) { return GF_COL_ARGB((u8) *(pix+1), (u8) *pix, (u8) *pix, (u8) *pix); }
 
-static void gf_sr_texture_set_callback(EVG_Texture *_this)
+static void texture_set_callback(EVG_Texture *_this)
 {
 	switch (_this->pixel_format) {
 	case GF_PIXEL_RGBA:
@@ -699,7 +727,7 @@ GF_Err evg_stencil_set_texture(GF_STENCIL st, char *pixels, u32 width, u32 heigh
 	_this->height = height;
 	_this->stride = stride;
 	_this->pixels = (char *) pixels;
-	gf_sr_texture_set_callback(_this);
+	texture_set_callback(_this);
 	return GF_OK;
 }
 
@@ -729,11 +757,15 @@ GF_Err evg_stencil_set_color_matrix(GF_STENCIL st, GF_ColorMatrix *cmat)
 	return GF_OK;
 }
 
-GF_Err evg_stencil_set_texture_alpha(GF_STENCIL st, u8 alpha)
+GF_Err evg_stencil_set_alpha(GF_STENCIL st, u8 alpha)
 {
 	EVG_Texture *_this = (EVG_Texture *)st;
-	if (!_this || (_this->type!=GF_STENCIL_TEXTURE)) return GF_BAD_PARAM;
-	_this->alpha = alpha;
+	if (!_this) return GF_BAD_PARAM;
+	if (_this->type==GF_STENCIL_SOLID) return GF_BAD_PARAM;
+	if (_this->type==GF_STENCIL_TEXTURE)
+		_this->alpha = alpha;
+	else
+		((EVG_BaseGradient*)st)->alpha = alpha;
 	return GF_OK;
 }
 
@@ -777,7 +809,7 @@ void evg_set_texture_active(EVGStencil *st)
 	_this->is_converted = 1;
 	_this->pixels = (char *) _this->conv_buf;
 	_this->stride = _this->Bpp * _this->width;
-	gf_sr_texture_set_callback(_this);
+	texture_set_callback(_this);
 }
 
 GF_Err evg_stencil_create_texture(GF_STENCIL st, u32 width, u32 height, GF_PixelFormat pixelFormat)
@@ -818,6 +850,6 @@ GF_Err evg_stencil_create_texture(GF_STENCIL st, u32 width, u32 height, GF_Pixel
 	_this->pixels = (char *) malloc(sizeof(char) * _this->stride * _this->height);
 	memset(_this->pixels, 0, sizeof(char) * _this->stride * _this->height);
 	_this->owns_texture = 1;
-	gf_sr_texture_set_callback(_this);
+	texture_set_callback(_this);
 	return GF_OK;
 }
