@@ -116,6 +116,7 @@ GF_Err evg_surface_attach_to_buffer(GF_SURFACE _this, char *pixels, u32 width, u
 		break;
 	case GF_PIXEL_RGB_32:
 	case GF_PIXEL_ARGB:
+	case GF_PIXEL_RGBA:
 		BPP = 4;
 		break;
 	case GF_PIXEL_BGR_32:
@@ -123,9 +124,11 @@ GF_Err evg_surface_attach_to_buffer(GF_SURFACE _this, char *pixels, u32 width, u
 		return GF_NOT_SUPPORTED;
 	}
 	surf->stride = stride;
+	if (!surf->stencil_pix_run || (surf->width != width)) {
+		if (surf->stencil_pix_run) free(surf->stencil_pix_run);
+		surf->stencil_pix_run = (u32 *) malloc(sizeof(u32) * (width+2));
+	}
 	surf->width = width;
-	if (surf->stencil_pix_run) free(surf->stencil_pix_run);
-	surf->stencil_pix_run = (u32 *) malloc(sizeof(u32) * (width+2));
 	surf->height = height;
 	surf->pixels = (char*)pixels;
 	surf->pixelFormat = pixelFormat;
@@ -166,6 +169,7 @@ GF_Err evg_surface_attach_to_texture(GF_SURFACE _this, GF_STENCIL sten)
 		break;
 	case GF_PIXEL_RGB_32:
 	case GF_PIXEL_ARGB:
+	case GF_PIXEL_RGBA:
 		BPP = 4;
 		break;
 	default:
@@ -241,10 +245,12 @@ GF_Err evg_surface_clear(GF_SURFACE _this, GF_IRect *rc, u32 color)
 	case GF_PIXEL_ARGB:
 	case GF_PIXEL_RGB_32:
 		return evg_surface_clear_argb(surf, clear, color);
+	case GF_PIXEL_RGBA:
+		return evg_surface_clear_rgba(surf, clear, color);
 	case GF_PIXEL_BGR_24:
-		return evg_surface_clear_rgb(surf, clear, color);
-	case GF_PIXEL_RGB_24:
 		return evg_surface_clear_bgr(surf, clear, color);
+	case GF_PIXEL_RGB_24:
+		return evg_surface_clear_rgb(surf, clear, color);
 	case GF_PIXEL_RGB_565:
 		return evg_surface_clear_565(surf, clear, color);
 #ifdef GF_RGB_444_SUPORT
@@ -362,6 +368,19 @@ static Bool setup_grey_callback(EVGSurface *surf)
 		}
 		break;
 
+	case GF_PIXEL_RGBA:
+		if (use_const) {
+			if (!a) return 0;
+			if (a!=0xFF) {
+				surf->ftparams.gray_spans = (EVG_Raster_Span_Func) evg_rgba_fill_const_a;
+			} else {
+				surf->ftparams.gray_spans = (EVG_Raster_Span_Func) evg_rgba_fill_const;
+			}
+		} else {
+			surf->ftparams.gray_spans = (EVG_Raster_Span_Func) evg_rgba_fill_var;
+		}
+		break;
+
 	case GF_PIXEL_RGB_32:
 		if (use_const) {
 			if (!a) return 0;
@@ -505,6 +524,7 @@ GF_Err evg_surface_set_path(GF_SURFACE _this, GF_Path *gp)
 	return GF_OK;
 }
 
+//static void gray_spans_stub(s32 y, s32 count, EVG_Span *spans, EVGSurface *surf){}
 
 GF_Err evg_surface_fill(GF_SURFACE _this, GF_STENCIL stencil)
 {
@@ -520,6 +540,8 @@ GF_Err evg_surface_fill(GF_SURFACE _this, GF_STENCIL stencil)
 	/*setup ft raster calllbacks*/
 	if (!setup_grey_callback(surf)) return GF_OK;
 
+//	surf->ftparams.gray_spans = gray_spans_stub;
+
 	get_surface_world_matrix(surf, &mat);
 
 	restore_filter = 0;
@@ -538,7 +560,12 @@ GF_Err evg_surface_fill(GF_SURFACE _this, GF_STENCIL stencil)
 		switch (sten->type) {
 		case GF_STENCIL_TEXTURE:
 			if (! ((EVG_Texture *)sten)->pixels) return GF_BAD_PARAM;
-			if (surf->center_coords) gf_mx2d_add_scale(&sten->smat, FIX_ONE, -FIX_ONE);
+
+			if (((EVG_Texture *)sten)->mod & GF_TEXTURE_FLIP) {
+				if (!surf->center_coords) gf_mx2d_add_scale(&sten->smat, FIX_ONE, -FIX_ONE);
+			} else {
+				if (surf->center_coords) gf_mx2d_add_scale(&sten->smat, FIX_ONE, -FIX_ONE);
+			}
 			evg_set_texture_active(sten);
 			gf_mx2d_add_matrix(&sten->smat, &st_mat);
 			gf_mx2d_add_matrix(&sten->smat, &mat);
@@ -569,7 +596,8 @@ GF_Err evg_surface_fill(GF_SURFACE _this, GF_STENCIL stencil)
 			gf_mx2d_add_matrix(&sten->smat, &mat);
 			gf_mx2d_inverse(&sten->smat);
 			gf_mx2d_add_translation(&sten->smat, -rad->center.x, -rad->center.y);
-			gf_mx2d_add_scale(&sten->smat, gf_divfix(FIX_ONE, rad->radius.x), gf_invfix(rad->radius.y));
+			gf_mx2d_add_scale(&sten->smat, gf_invfix(rad->radius.x), gf_invfix(rad->radius.y));
+
 			rad->d_f.x = gf_divfix(rad->focus.x - rad->center.x, rad->radius.x);
 			rad->d_f.y = gf_divfix(rad->focus.y - rad->center.y, rad->radius.y);
 			/*init*/
