@@ -78,7 +78,7 @@ typedef struct
 	M_Transform2D *tr_track, *tr_box, *tr_scroll;
 	M_Material2D *mat_track, *mat_box;
 	M_Layer2D *dlist;
-	M_Rectangle *rec_box;
+	M_Rectangle *rec_box, *rec_track;
 
 	M_TimeSensor *ts_blink, *ts_scroll;
 	M_ScalarInterpolator *process_blink, *process_scroll;
@@ -111,7 +111,7 @@ static void TTD_UpdateSizeInfo(TTDPriv *priv)
 		}
 		gf_sg_get_scene_size_info(priv->sg, &w, &h);
 		if (!w || !h) return;
-		gf_is_force_scene_size(priv->inlineScene, w, h);
+		gf_inline_force_scene_size(priv->inlineScene, w, h);
 	}
 
 	if (!w || !h) return;
@@ -133,7 +133,7 @@ static void TTD_UpdateSizeInfo(TTDPriv *priv)
 		}
 		if (set_size) {
 			gf_sg_set_scene_size_info(priv->sg, w, h, 1);
-			gf_is_force_scene_size(priv->inlineScene, w, h);
+			gf_inline_force_scene_size(priv->inlineScene, w, h);
 		}
 	} else {
 		/*otherwise override (mainly used for SRT & TTXT file direct loading*/
@@ -193,9 +193,9 @@ static GF_Err TTD_SetCapabilities(GF_BaseDecoder *plug, const GF_CodecCapability
 		if (capability.cap.valueInt) {
 			TTD_ResetDisplay(priv);
 			TTD_UpdateSizeInfo(priv);
-			gf_is_register_extra_graph(priv->inlineScene, priv->sg, 0);
+			gf_inline_register_extra_graph(priv->inlineScene, priv->sg, 0);
 		} else {
-			gf_is_register_extra_graph(priv->inlineScene, priv->sg, 1);
+			gf_inline_register_extra_graph(priv->inlineScene, priv->sg, 1);
 		}			
 	}
 	return GF_OK;
@@ -236,28 +236,18 @@ static GFINLINE GF_Node *ttd_create_node(TTDPriv *ttd, u32 tag, const char *def_
 	return n;
 }
 
-static GF_Err TTD_AttachStream(GF_BaseDecoder *plug, 
-									 u16 ES_ID, 
-									 char *decSpecInfo, 
-									 u32 decSpecInfoSize, 
-									 u16 DependsOnES_ID,
-									 u32 objectTypeIndication, 
-									 Bool Upstream)
+static GF_Err TTD_AttachStream(GF_BaseDecoder *plug, GF_ESD *esd)
 {
 	TTDPriv *priv = (TTDPriv *)plug->privateStack;
 	GF_Err e;
-	GF_DefaultDescriptor dsi;
 	GF_Node *root, *n1, *n2;
 	const char *opt;
 	/*no scalable, no upstream*/
-	if (priv->nb_streams || Upstream) return GF_NOT_SUPPORTED;
-	if (!decSpecInfo || !decSpecInfoSize) return GF_NON_COMPLIANT_BITSTREAM;
+	if (priv->nb_streams || esd->decoderConfig->upstream) return GF_NOT_SUPPORTED;
+	if (!esd->decoderConfig->decoderSpecificInfo || !esd->decoderConfig->decoderSpecificInfo->data) return GF_NON_COMPLIANT_BITSTREAM;
 
 	priv->cfg = (GF_TextConfig *) gf_odf_desc_new(GF_ODF_TEXT_CFG_TAG);
-	dsi.tag = GF_ODF_DSI_TAG;
-	dsi.data = decSpecInfo;
-	dsi.dataLength = decSpecInfoSize;
-	e = gf_odf_get_text_config(&dsi, (u8) objectTypeIndication, priv->cfg);
+	e = gf_odf_get_text_config(esd->decoderConfig->decoderSpecificInfo, (u8) esd->decoderConfig->objectTypeIndication, priv->cfg);
 	if (e) {
 		gf_odf_desc_del((GF_Descriptor *) priv->cfg);
 		priv->cfg = NULL;
@@ -288,10 +278,11 @@ static GF_Err TTD_AttachStream(GF_BaseDecoder *plug,
 	((M_Appearance *) ((M_Shape *)n1)->appearance)->material = (GF_Node *) priv->mat_track;
 	gf_node_register((GF_Node *) priv->mat_track, ((M_Shape *)n1)->appearance);
 	n2 = ttd_create_node(priv, TAG_MPEG4_Rectangle, NULL);
-	((M_Rectangle *)n2)->size.x = priv->cfg->text_width;
-	((M_Rectangle *)n2)->size.y = priv->cfg->text_height;
+	((M_Rectangle *)n2)->size.x = 0;
+	((M_Rectangle *)n2)->size.y = 0;
 	((M_Shape *)n1)->geometry = n2;
 	gf_node_register(n2, n1);
+	priv->rec_track = (M_Rectangle *)n2;
 
 	/*txt box background*/
 	priv->tr_box = (M_Transform2D *) ttd_create_node(priv, TAG_MPEG4_Transform2D, NULL);
@@ -306,8 +297,8 @@ static GF_Err TTD_AttachStream(GF_BaseDecoder *plug,
 	((M_Appearance *) ((M_Shape *)n1)->appearance)->material = (GF_Node *)priv->mat_box;
 	gf_node_register((GF_Node *)priv->mat_box, ((M_Shape *)n1)->appearance);
 	priv->rec_box = (M_Rectangle *) ttd_create_node(priv, TAG_MPEG4_Rectangle, NULL);
-	priv->rec_box->size.x = priv->cfg->text_width;
-	priv->rec_box->size.y = priv->cfg->text_height;
+	priv->rec_box->size.x = 0;
+	priv->rec_box->size.y = 0;
 	((M_Shape *)n1)->geometry = (GF_Node *) priv->rec_box;
 	gf_node_register((GF_Node *) priv->rec_box, n1);
 
@@ -355,7 +346,7 @@ static GF_Err TTD_DetachStream(GF_BaseDecoder *plug, u16 ES_ID)
 	TTDPriv *priv = (TTDPriv *)plug->privateStack;
 	if (!priv->nb_streams) return GF_BAD_PARAM;
 
-	gf_is_register_extra_graph(priv->inlineScene, priv->sg, 1);
+	gf_inline_register_extra_graph(priv->inlineScene, priv->sg, 1);
 
 	gf_node_unregister((GF_Node *) priv->ts_blink, NULL);
 	gf_node_unregister((GF_Node *) priv->process_blink, NULL);
@@ -612,7 +603,7 @@ static void TTD_NewTextChunk(TTDPriv *priv, GF_TextSampleDescriptor *tsd, M_Form
 
 				/*spliting lines, duplicate node*/
 
-				n2 = gf_node_clone(priv->sg, txt_model, NULL);
+				n2 = gf_node_clone(priv->sg, txt_model, NULL, "", 1);
 				if (tc->hlink && tc->hlink->URL) {
 					GF_Node *t = ((M_Anchor *)n2)->children->node;
 					text = (M_Text *) ((M_Shape *)t)->geometry;
@@ -768,6 +759,7 @@ static void TTD_ApplySample(TTDPriv *priv, GF_TextSample *txt, u32 sdi, Bool is_
 		priv->mat_track->transparency = FIX_ONE;
 		n = priv->mat_box;
 	}
+
 	n->transparency = FIX_ONE - INT2FIX((td->back_color>>24) & 0xFF) / 255;
 	n->emissiveColor.red = INT2FIX((td->back_color>>16) & 0xFF) / 255;
 	n->emissiveColor.green = INT2FIX((td->back_color>>8) & 0xFF) / 255;
@@ -792,9 +784,31 @@ static void TTD_ApplySample(TTDPriv *priv, GF_TextSample *txt, u32 sdi, Bool is_
 		thh = priv->cfg->text_height;
 	}
 
-	priv->dlist->size.x = priv->rec_box->size.x = INT2FIX(thw);
-	priv->dlist->size.y = priv->rec_box->size.y = INT2FIX(thh);
-	gf_node_changed((GF_Node *) priv->rec_box, NULL);
+	priv->dlist->size.x = INT2FIX(thw);
+	priv->dlist->size.y = INT2FIX(thh);
+
+	/*disable backgrounds if not used*/
+	if (priv->mat_track->transparency<FIX_ONE) {
+		if (priv->rec_track->size.x != priv->cfg->text_width) {
+			priv->rec_track->size.x = priv->cfg->text_width;
+			priv->rec_track->size.y = priv->cfg->text_height;
+			gf_node_changed((GF_Node *) priv->rec_track, NULL);
+		}
+	} else if (priv->rec_track->size.x) {
+		priv->rec_track->size.x = priv->rec_track->size.y = 0;
+		gf_node_changed((GF_Node *) priv->rec_box, NULL);
+	}
+	
+	if (priv->mat_box->transparency<FIX_ONE) {
+		if (priv->rec_box->size.x != priv->dlist->size.x) {
+			priv->rec_box->size.x = priv->dlist->size.x;
+			priv->rec_box->size.y = priv->dlist->size.y;
+			gf_node_changed((GF_Node *) priv->rec_box, NULL);
+		}
+	} else if (priv->rec_box->size.x) {
+		priv->rec_box->size.x = priv->rec_box->size.y = 0;
+		gf_node_changed((GF_Node *) priv->rec_box, NULL);
+	}
 
 	form = (M_Form *) ttd_create_node(priv, TAG_MPEG4_Form, NULL);
 	form->size.x = INT2FIX(thw);
