@@ -26,13 +26,218 @@
 #include "x11_out.h"
 #include <gpac/constants.h>
 
+
 void X11_SetupWindow (GF_VideoOutput * vout);
 
-//=====================================
-/*
- * Flush video: draw main image(surface/pixmap) -> os_handle
- */
-//=====================================
+
+#ifdef GPAC_HAS_X11_XV
+static void X11_DestroyOverlay(XWindow *xwin)
+{
+	if (xwin->overlay) XFree(xwin->overlay);
+	xwin->overlay = NULL;
+	xwin->xv_pf_format = 0;	
+	if (xwin->xvport>=0) {
+		XvUngrabPort(xwin->display, xwin->xvport, CurrentTime );
+		xwin->xvport = -1;
+	}
+}
+
+static Bool is_same_yuv(u32 pf, u32 a_pf)
+{
+	if (pf==a_pf) return 1;
+	return 0;
+	switch (pf) {
+	case GF_PIXEL_YV12:
+	case GF_PIXEL_I420:
+	case GF_PIXEL_IYUV:
+		switch (a_pf) {
+		case GF_PIXEL_YV12:
+		case GF_PIXEL_I420:
+		case GF_PIXEL_IYUV:
+			return 1;
+		default:
+			return 0;
+		}
+	case GF_PIXEL_UYVY:
+	case GF_PIXEL_UYNV:
+	case GF_PIXEL_Y422:
+		switch (a_pf) {
+		case GF_PIXEL_UYVY:
+		case GF_PIXEL_UYNV:
+		case GF_PIXEL_Y422:
+			return 1;
+		default:
+			return 0;
+		}
+	case GF_PIXEL_YUY2:
+	case GF_PIXEL_YUNV:
+		switch (a_pf) {
+		case GF_PIXEL_YUY2:
+		case GF_PIXEL_YUNV:
+			return 1;
+		default:
+			return 0;
+		}
+	}
+	return 0;
+}
+static u32 X11_GetPixelFormat(u32 pf)
+{
+	return GF_4CC((pf)&0xff, (pf>>8)&0xff, (pf>>16)&0xff, (pf>>24)&0xff);
+}
+static int X11_GetXVideoPort(GF_VideoOutput *vout, u32 pixel_format, Bool check_color)
+{
+	XWindow *xwin = (XWindow *)vout->opaque;
+	Bool has_color_key = 0;
+	XvAdaptorInfo *adaptors;
+	unsigned int i;
+	unsigned int nb_adaptors;
+	int selected_port;
+
+    if (XvQueryExtension(xwin->display, &i, &i, &i, &i, &i ) != Success) return -1;
+    if (XvQueryAdaptors(xwin->display, DefaultRootWindow(xwin->display), &nb_adaptors, &adaptors) != Success) return -1;
+
+    selected_port = -1;
+    for (i=0; i < nb_adaptors; i++) {
+        XvImageFormatValues *formats;
+        int j, num_formats, port;
+
+        if( !( adaptors[i].type & XvInputMask ) || !(adaptors[i].type & XvImageMask ) )
+            continue;
+
+        /* Check for our format... */
+        formats = XvListImageFormats(xwin->display, adaptors[i].base_id, &num_formats);
+
+        for (j=0; j<num_formats && (selected_port == -1 ); j++) {
+		XvAttribute *attr;
+		int k, nb_attributes;
+		u32 pformat = X11_GetPixelFormat(formats[j].id);
+
+		if( !is_same_yuv(pformat, pixel_format) ) continue;
+
+		/* Grab first port  supporting this format */
+		for (port=adaptors[i].base_id; (port < (int)(adaptors[i].base_id + adaptors[i].num_ports) ) && (selected_port == -1); port++) {
+			if (port==xwin->xvport) continue;
+
+			attr = XvQueryPortAttributes(xwin->display, port, &nb_attributes);
+			for (k=0; k<nb_attributes; k++ ) {
+			        if (!strcmp(attr[k].name, "XV_COLORKEY")) {
+					const Atom ckey = XInternAtom(xwin->display, "XV_COLORKEY", False);
+					XvGetPortAttribute(xwin->display, port, ckey, &vout->overlay_color_key);
+					has_color_key = 1;
+					vout->overlay_color_key |= 0xFF000000;
+				}
+/*			        else if (!strcmp(attr[k].name, "XV_AUTOPAINT_COLORKEY")) {
+			            const Atom paint = XInternAtom(xwin->display, "XV_AUTOPAINT_COLORKEY", False);
+			            XvSetPortAttribute(xwin->display, port, paint, 1);
+				}
+*/			}
+			if (check_color && !has_color_key) continue;
+
+			if (XvGrabPort(xwin->display, port, CurrentTime) == Success) {
+				selected_port = port;
+				xwin->xv_pf_format = formats[j].id;
+			}
+		}
+		if (selected_port == -1 )
+	                continue;
+ 
+       }
+        if (formats != NULL) XFree(formats);
+    }
+    if (nb_adaptors > 0) 
+	XvFreeAdaptorInfo(adaptors);
+
+   return selected_port;
+}
+static GF_Err X11_InitOverlay(GF_VideoOutput *vout, u32 VideoWidth, u32 VideoHeight)
+{
+	XWindow *xwin = (XWindow *)vout->opaque;
+	if (xwin->overlay && (VideoWidth<=xwin->overlay->width) && (VideoHeight<=xwin->overlay->height)) {
+		return GF_OK;	
+	}
+
+	X11_DestroyOverlay(xwin);
+
+	xwin->xvport = X11_GetXVideoPort(vout, GF_PIXEL_I420, 0);
+	if (xwin->xvport<0) 
+		xwin->xvport = X11_GetXVideoPort(vout, GF_PIXEL_YUY2, 0);
+
+	if (xwin->xvport<0) {
+		return GF_NOT_SUPPORTED;
+	}
+
+        /* Create overlay image */
+	xwin->overlay = XvCreateImage(xwin->display, xwin->xvport, xwin->xv_pf_format, NULL, VideoWidth, VideoHeight);
+	if (!xwin->overlay) {
+		GF_LOG(GF_LOG_ERROR, GF_LOG_MMIO, ("[X11] Xv Overlay Creation Failure\n"));
+		return GF_IO_ERR;
+	}
+
+	GF_LOG(GF_LOG_INFO, GF_LOG_MMIO, ("[X11] Overlay init %d x %d - pixel format %s - XV port %d\n", 
+			VideoWidth, VideoHeight, 
+			gf_4cc_to_str(vout->yuv_pixel_format), xwin->xvport ));
+
+	return GF_OK;
+}
+
+GF_Err X11_Blit(struct _video_out *vout, GF_VideoSurface *video_src, GF_Window *src, GF_Window *dest, u32 overlay_type)
+{
+	XvImage *overlay;
+	int xvport;
+	Drawable dst_dr;
+	GF_Err e;
+	Window cur_wnd;
+	XWindow *xwin = (XWindow *)vout->opaque;
+
+	if (video_src->pixel_format != GF_PIXEL_YV12) return GF_NOT_SUPPORTED;
+	cur_wnd = xwin->fullscreen ? xwin->full_wnd : xwin->wnd;
+	/*init if needed*/
+	if ((xwin->xvport<0) || !xwin->overlay) {
+		e = X11_InitOverlay(vout, video_src->width, video_src->height);
+		if (e) return e;
+	}
+
+	/*different size, recreate an image*/
+	if ((xwin->overlay->width != video_src->width) || (xwin->overlay->height != video_src->height)) {
+		if (xwin->overlay) XFree(xwin->overlay);
+		xwin->overlay = XvCreateImage(xwin->display, xwin->xvport, xwin->xv_pf_format, NULL, video_src->width, video_src->height);
+		if (!xwin->overlay) return GF_IO_ERR;
+	}
+	GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("[X11] Blit surface to dest %d x %d - overlay type %s\n", dest->w, dest->h,
+					(overlay_type==0)? "none" : ((overlay_type==1) ? "Top-Level" : "ColorKey") 
+	));
+
+	overlay = xwin->overlay;
+	xvport = xwin->xvport;
+
+	overlay->data = video_src->video_buffer;
+
+	overlay->num_planes = 3;
+	overlay->pitches[0] = video_src->width;
+	overlay->pitches[1] = xwin->overlay->pitches[2] = video_src->width/2;
+	overlay->offsets[0] = 0;
+	overlay->offsets[1] = video_src->width*video_src->height;
+	overlay->offsets[2] = 5*video_src->width*video_src->height/4;
+
+	dst_dr = cur_wnd;
+	if (!overlay_type) {
+		if (!xwin->pixmap) {
+			GF_LOG(GF_LOG_ERROR, GF_LOG_MMIO, ("[X11] Back buffer not configured for Blt\n"));
+			return GF_BAD_PARAM;
+		}
+		dst_dr = xwin->pixmap;
+	}
+     XvPutImage(xwin->display, xvport, dst_dr, xwin->the_gc, overlay, 
+		src->x, src->y, src->w, src->h,
+		dest->x, dest->y, dest->w, dest->h);
+
+	return GF_OK;
+}
+#endif
+
+
+/*Flush video */
 GF_Err X11_Flush(struct _video_out *vout, GF_Window * dest)
 {
 	Window cur_wnd;
@@ -41,33 +246,33 @@ GF_Err X11_Flush(struct _video_out *vout, GF_Window * dest)
 	 */
 	X11VID ();
 
-	XSync(xWindow->display, False);
-
 	if (!xWindow->is_init) return GF_OK;
 	cur_wnd = xWindow->fullscreen ? xWindow->full_wnd : xWindow->wnd;
 
-	if (xWindow->is_3D_out) {
+	if (xWindow->output_3d_mode==1) {
 #ifdef GPAC_HAS_OPENGL
+		XSync(xWindow->display, False);
+		glFlush();
 		glXSwapBuffers(xWindow->display, cur_wnd);
 #endif
 		return GF_OK;
 	}
 
-	switch (xWindow->videoaccesstype) {
 #ifdef GPAC_HAS_X11_SHM
-	case VIDEO_XI_SHMSTD:
-	  XShmPutImage (xWindow->display, cur_wnd, xWindow->the_gc, xWindow->surface,
-			0, 0, dest->x, dest->y, dest->w, dest->h, True);
-		break;
-	case VIDEO_XI_SHMPIXMAP:
+	if (xWindow->pixmap) {
 		XClearWindow(xWindow->display, cur_wnd);
-		break;
+		XSync(xWindow->display, False);
+	} else if (xWindow->use_shared_memory) {
+		XSync(xWindow->display, False);
+		XShmPutImage (xWindow->display, cur_wnd, xWindow->the_gc, xWindow->surface,
+			0, 0, dest->x, dest->y, dest->w, dest->h, True);
+	} else
 #endif
-	case VIDEO_XI_STANDARD:
+	{
+		XSync(xWindow->display, False);
 		XRaiseWindow(xWindow->display, xWindow->wnd);
 		XPutImage (xWindow->display, cur_wnd, xWindow->the_gc, xWindow->surface,
 			0, 0, dest->x, dest->y, dest->w, dest->h);
-		break;
 	}
 	return GF_OK;
 }
@@ -314,7 +519,9 @@ static void X11_HandleEvents(GF_VideoOutput *vout)
 	X11VID();
 	unsigned char keybuf[32];
 	XEvent xevent;
-
+#ifdef ENABLE_JOYSTICK
+	struct js_event jsevent;
+#endif
 	the_window = xWindow->fullscreen ? xWindow->full_wnd : xWindow->wnd;
 	XSync(xWindow->display, False);
 
@@ -324,7 +531,7 @@ static void X11_HandleEvents(GF_VideoOutput *vout)
 		  switch (xevent.type) {
 		    /*
 		     * X11 window resized event
-		     * must inform GPAC to resize os_handle wnd
+		     * must inform GPAC
 		     */
 		  case ConfigureNotify:
 		    if ((unsigned int) xevent.xconfigure.width != xWindow->w_width
@@ -334,6 +541,11 @@ static void X11_HandleEvents(GF_VideoOutput *vout)
 			xWindow->w_width = evt.size.width = xevent.xconfigure.width;
 			xWindow->w_height = evt.size.height = xevent.xconfigure.height;
 			vout->on_event(vout->evt_cbk_hdl, &evt);
+		      } else {
+			evt.type = GF_EVENT_MOVE;
+			evt.move.x = xevent.xconfigure.x;
+			evt.move.y = xevent.xconfigure.y;
+			vout->on_event(vout->evt_cbk_hdl, &evt);
 		      }
 		    break;
 		    /*
@@ -353,8 +565,10 @@ static void X11_HandleEvents(GF_VideoOutput *vout)
 		    }
 		    break;
 
-		  case KeyPress:
-		  case KeyRelease:
+#ifndef ENABLE_JOYSTICK
+		  	
+		      case KeyPress:
+		      case KeyRelease:
 		    x11_translate_key(XKeycodeToKeysym (xWindow->display, xevent.xkey.keycode, 0), &evt.key);
 			evt.type = (xevent.type ==KeyPress) ? GF_EVENT_KEYDOWN : GF_EVENT_KEYUP;
 			vout->on_event (vout->evt_cbk_hdl, &evt);
@@ -414,7 +628,6 @@ static void X11_HandleEvents(GF_VideoOutput *vout)
 		    evt.mouse.y = xevent.xmotion.y;
 		    vout->on_event (vout->evt_cbk_hdl, &evt);
 		    break;
-		    
 		  case PropertyNotify:
 		    break;
 		  case MapNotify:
@@ -439,8 +652,131 @@ static void X11_HandleEvents(GF_VideoOutput *vout)
 		    
 		  default:
 		    break;
+#endif
 		  }
 	 }
+
+#ifdef ENABLE_JOYSTICK
+	while (read (xWindow->fd, &jsevent, sizeof(struct js_event)) > 0){
+    	s32 mul;
+	
+		switch (jsevent.type) {
+		  case JS_EVENT_BUTTON:
+		    
+		    if (jsevent.value){
+		      /*pressed event*/
+		      if (!xWindow->fullscreen && !xWindow->has_focus ) xWindow->has_focus = 1;
+		      if (jsevent.number<4) evt.type = GF_EVENT_MOUSEDOWN;
+		      else evt.type = GF_EVENT_KEYDOWN;
+		      
+		    } else { 
+		    	
+			  if (jsevent.number<4) evt.type = GF_EVENT_MOUSEUP;
+			  else evt.type = GF_EVENT_KEYUP;
+			      
+		    }
+
+		     
+		      switch (jsevent.number){
+		        case 0:
+			     evt.mouse.button = GF_MOUSE_LEFT;
+			     evt.mouse.x = xWindow->prev_x;
+			     evt.mouse.y = xWindow->prev_y;
+		         vout->on_event (vout->evt_cbk_hdl, &evt);
+			      break;
+
+				case 1:
+				 evt.mouse.button = GF_MOUSE_MIDDLE;
+			     evt.mouse.x = xWindow->prev_x;
+			     evt.mouse.y = xWindow->prev_y;
+				 vout->on_event (vout->evt_cbk_hdl, &evt);
+				 break;
+	
+				case 2:
+				 evt.mouse.button = GF_MOUSE_RIGHT;
+			     evt.mouse.x = xWindow->prev_x;
+			     evt.mouse.y = xWindow->prev_y;
+			     vout->on_event (vout->evt_cbk_hdl, &evt);
+			     break;
+			     
+				case 4:
+				case 5:
+				case 6:
+				case 7:
+				 evt.key.key_code = GF_KEY_JOYSTICK; 
+				 evt.key.hw_code = jsevent.number;
+				 vout->on_event (vout->evt_cbk_hdl, &evt);
+			     break;
+			     
+				default:
+				 break;
+		      }
+
+		    
+		  break;
+		  case JS_EVENT_AXIS:
+			  if (ABS(jsevent.value)>16000) mul = 6;
+			  else if (ABS(jsevent.value)>1024) mul = 2;
+			  else mul = 1;
+			  if (jsevent.number==0) {
+				  if (jsevent.value>0) xWindow->prev_x += mul;
+				  else if (jsevent.value<0) {
+					  xWindow->prev_x -= mul;
+					//  if (xWindow->prev_x <0) xWindow->prev_x=0;
+				  }
+			  if (vout->centered_mode==1){	  
+				  xWindow->prev_x = ((0xFFFF/2) + jsevent.value) * xWindow->w_width;
+				  xWindow->prev_x /= 0xFFFF; 
+			  }
+			
+			  }
+			  else if (jsevent.number==1) {
+				  if (jsevent.value>0) xWindow->prev_y += mul;
+				  else if (jsevent.value<0) {
+					  xWindow->prev_y -= mul;
+				//	  if (xWindow->prev_y <0) xWindow->prev_y=0;
+				  }
+				  if (vout->centered_mode==1) {
+				  xWindow->prev_y = ((0xFFFF/2) + jsevent.value) * xWindow->w_height;
+				  xWindow->prev_y /= 0xFFFF; 
+				  }
+		/*		  if (jsevent.value>0) xWindow->prev_y += mul;
+				  else xWindow->prev_y -= mul; */
+
+			  } 
+			/*  fprintf(stdout, "X %d - y %d\n", xWindow->prev_x, xWindow->prev_y); */
+		    evt.type = GF_EVENT_MOUSEMOVE;
+		    evt.mouse.x = xWindow->prev_x;
+		    evt.mouse.y = xWindow->prev_y;
+		    vout->on_event (vout->evt_cbk_hdl, &evt); 
+		    break;
+
+		  case PropertyNotify:
+		    break;
+		  case MapNotify:
+		    break;
+		  case CirculateNotify:
+		    break;
+		  case UnmapNotify:
+		    break;
+		  case ReparentNotify:
+		    break;
+		  case FocusOut:
+		//	if (!xWindow->fullscreen) xWindow->has_focus = 0;
+		    break;
+		  case FocusIn:
+		//	if (!xWindow->fullscreen) xWindow->has_focus = 1;
+		    break;
+
+		  case DestroyNotify:
+		      evt.type = GF_EVENT_QUIT;
+		      vout->on_event(vout->evt_cbk_hdl, &evt);
+		    break;
+	
+		//  case JS_EVENT_INIT:
+		}	
+	}
+#endif
 }
 
 
@@ -450,10 +786,14 @@ static GF_Err X11_SetupGL(GF_VideoOutput *vout)
 {
   GF_Event evt;
   XWindow *xWin = (XWindow *)vout->opaque;
+
+  GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("[X11] Setting up GL for display %d\n", xWin->display));
   XSync(xWin->display, False);
   xWin->glx_context = glXCreateContext(xWin->display,xWin->glx_visualinfo, NULL, True);
   XSync(xWin->display, False);
   if (!xWin->glx_context) return GF_IO_ERR;
+  if (xWin->output_3d_mode==2)  return GF_IO_ERR;
+
   if ( ! glXMakeCurrent(xWin->display, xWin->fullscreen ? xWin->full_wnd : xWin->wnd, xWin->glx_context) ) return GF_IO_ERR;
   XSync(xWin->display, False);
   evt.type = GF_EVENT_VIDEO_SETUP;
@@ -462,6 +802,59 @@ static GF_Err X11_SetupGL(GF_VideoOutput *vout)
   return GF_OK;
 }
 
+static GF_Err X11_SetupGLPixmap(GF_VideoOutput *vout, u32 width, u32 height)
+{
+  XWindow *xWin = (XWindow *)vout->opaque;
+
+  if (xWin->glx_context) {
+    glXMakeCurrent(xWin->display, None, NULL);
+    glXDestroyContext(xWin->display, xWin->glx_context);
+    xWin->glx_context = NULL;
+  }
+  if (xWin->gl_offscreen) glXDestroyGLXPixmap(xWin->display, xWin->gl_offscreen);
+  xWin->gl_offscreen = 0;
+  if (xWin->gl_pixmap) XFreePixmap(xWin->display, xWin->gl_pixmap);
+  xWin->gl_pixmap = 0;
+
+  if (xWin->offscreen_type) {
+	  GF_LOG(GF_LOG_INFO, GF_LOG_MMIO, ("[X11] Using offscreen GL context through XWindow\n"));
+	  if (xWin->offscreen_type==2) {
+		XSync(xWin->display, False);
+		XMapWindow (xWin->display, (Window) xWin->gl_wnd);
+		XSync(xWin->display, False);
+//		XSetWMNormalHints (xWin->display, xWin->gl_wnd, Hints);
+ 		XStoreName (xWin->display, xWin->gl_wnd, "GPAC Offscreeen Window - debug mode");
+	}
+
+	  XSync(xWin->display, False);
+	  xWin->glx_context = glXCreateContext(xWin->display,xWin->glx_visualinfo, NULL, True);
+	  XSync(xWin->display, False);
+	  if (!xWin->glx_context) return GF_IO_ERR;
+	  if ( ! glXMakeCurrent(xWin->display, xWin->gl_wnd, xWin->glx_context) ) return GF_IO_ERR;
+	  XSync(xWin->display, False);
+
+  } else {
+  GF_LOG(GF_LOG_INFO, GF_LOG_MMIO, ("[X11] Using offscreen GL context through glXPixmap\n"));
+  xWin->gl_pixmap = XCreatePixmap(xWin->display, xWin->gl_wnd, width, height, xWin->depth);
+  if (!xWin->gl_pixmap) return GF_IO_ERR;
+ 
+  xWin->gl_offscreen = glXCreateGLXPixmap(xWin->display, xWin->glx_visualinfo, xWin->gl_pixmap);
+  if (!xWin->gl_offscreen) return GF_IO_ERR;
+ 
+  XSync(xWin->display, False);
+  xWin->glx_context = glXCreateContext(xWin->display, xWin->glx_visualinfo, 0, GL_FALSE);
+  XSync(xWin->display, False);
+  if (!xWin->glx_context) return GF_IO_ERR;
+
+ XSync(xWin->display, False);
+ fprintf(stdout, "!! Activating GLContext on GLPixmap - this may crash !!\n");
+ glXMakeCurrent(xWin->display, xWin->gl_offscreen, xWin->glx_context);
+  }
+
+  GF_LOG(GF_LOG_INFO, GF_LOG_MMIO, ("[X11] Offscreen GL context setup\n"));
+  return GF_OK;
+}
+
 static void X11_ReleaseGL(XWindow *xWin)
 {
   XSync(xWin->display, False);
@@ -473,43 +866,42 @@ static void X11_ReleaseGL(XWindow *xWin)
   xWin->is_init = 0;
   XSync(xWin->display, False);
 }
+
 #endif
 
 
 static void X11_ReleaseBackBuffer (GF_VideoOutput * vout)
 {
 	X11VID ();
-	if (xWindow->is_init == 1) {
-		switch (xWindow->videoaccesstype) {
 #ifdef GPAC_HAS_X11_SHM
-		case VIDEO_XI_SHMSTD:
-			XShmDetach (xWindow->display, xWindow->shmseginfo);
-			if (xWindow->surface) XDestroyImage(xWindow->surface);
-			xWindow->surface = NULL;
-			if (xWindow->shmseginfo->shmaddr) shmdt(xWindow->shmseginfo->shmaddr);
-			if (xWindow->shmseginfo->shmid >= 0)
-				shmctl (xWindow->shmseginfo->shmid, IPC_RMID, NULL);
-			free (xWindow->shmseginfo);
-			xWindow->shmseginfo = NULL;
-			break;
-		case VIDEO_XI_SHMPIXMAP:
-			XShmDetach(xWindow->display, xWindow->shmseginfo);
-			XFreePixmap(xWindow->display, xWindow->pixmap);
-			if (xWindow->shmseginfo->shmaddr) shmdt (xWindow->shmseginfo->shmaddr);
-			if (xWindow->shmseginfo->shmid >= 0) shmctl (xWindow->shmseginfo->shmid, IPC_RMID, NULL);
-			free (xWindow->shmseginfo);
-			xWindow->shmseginfo = NULL;
-			break;
-#endif
-		case VIDEO_XI_STANDARD:
-			if (xWindow->back_buffer->buffer) free (xWindow->back_buffer->buffer);
-			xWindow->back_buffer->buffer = NULL;
-			if (xWindow->surface) XFree(xWindow->surface);
-			xWindow->surface = NULL;
-			break;
-		}
-		xWindow->is_init = 0;
+	if (xWindow->shmseginfo) XShmDetach (xWindow->display, xWindow->shmseginfo);
+	if (xWindow->pixmap) {
+		XFreePixmap(xWindow->display, xWindow->pixmap);
+		xWindow->pixmap = 0L;
+		xWindow->pwidth = xWindow->pheight = 0;
+	} else {
+		if (xWindow->surface) XDestroyImage(xWindow->surface);
+		xWindow->surface = NULL;
 	}
+	if (xWindow->shmseginfo) {
+		if (xWindow->shmseginfo->shmaddr) shmdt(xWindow->shmseginfo->shmaddr);
+		if (xWindow->shmseginfo->shmid >= 0)
+			shmctl (xWindow->shmseginfo->shmid, IPC_RMID, NULL);
+		free (xWindow->shmseginfo);
+		xWindow->shmseginfo = NULL;
+	}
+#endif
+	if (xWindow->surface) {
+		if (xWindow->surface->data)
+			free (xWindow->surface->data);
+		XFree(xWindow->surface);
+		xWindow->surface = NULL;
+	}
+	xWindow->is_init = 0;
+#ifdef GPAC_HAS_X11_XV
+	X11_DestroyOverlay(xWindow);
+#endif
+	
 }
 
 /*
@@ -530,55 +922,56 @@ GF_Err X11_InitBackBuffer (GF_VideoOutput * vout, u32 VideoWidth, u32 VideoHeigh
 
 	X11_ReleaseBackBuffer(vout);
 	/*WATCHOUT seems we need even width when using shared memory extensions*/
-	if ((xWindow->videoaccesstype!=VIDEO_XI_STANDARD) && (VideoWidth%2)) 
+	if (xWindow->use_shared_memory && (VideoWidth%2)) 
 	  VideoWidth++;
 
-	xWindow->back_buffer->BPP = xWindow->bpp;
-	xWindow->back_buffer->width = VideoWidth;
-	xWindow->back_buffer->height = VideoHeight;
-	xWindow->back_buffer->pitch = VideoWidth * xWindow->bpp;
 	size = VideoWidth * VideoHeight * xWindow->bpp;
 	cur_wnd = xWindow->fullscreen ? xWindow->full_wnd : xWindow->wnd;
 
-	switch (xWindow->videoaccesstype) {
 #ifdef GPAC_HAS_X11_SHM
-	case VIDEO_XI_SHMPIXMAP:
+	/*if we're using YUV blit to offscreen, we must use a pixmap*/
+	if (vout->hw_caps & GF_VIDEO_HW_HAS_YUV) {
 		GF_SAFEALLOC(xWindow->shmseginfo, XShmSegmentInfo);
-		xWindow->shmseginfo->shmid = shmget(IPC_PRIVATE, size, IPC_CREAT | 0777);
+		xWindow->shmseginfo->shmid = shmget(IPC_PRIVATE, size, IPC_CREAT | 0776);
 		xWindow->shmseginfo->shmaddr = shmat(xWindow->shmseginfo->shmid, 0, 0);
-		xWindow->back_buffer->buffer = (unsigned char *) xWindow->shmseginfo->shmaddr;
 		xWindow->shmseginfo->readOnly = False;
-		XShmAttach (xWindow->display, xWindow->shmseginfo);
-		xWindow->pixmap = XShmCreatePixmap (xWindow->display, cur_wnd,
-								xWindow->back_buffer->buffer, xWindow->shmseginfo,
-								VideoWidth, VideoHeight, xWindow->depth);
+		if (!XShmAttach (xWindow->display, xWindow->shmseginfo)) {
+			GF_LOG(GF_LOG_ERROR, GF_LOG_MMIO, ("[X11] Failed to attach shared memory!\n"));
+		}
+		xWindow->pixmap = XShmCreatePixmap(xWindow->display, cur_wnd,
+							(unsigned char *) xWindow->shmseginfo->shmaddr, xWindow->shmseginfo,
+							VideoWidth, VideoHeight, xWindow->depth);
+		memset((unsigned char *) xWindow->shmseginfo->shmaddr, 0, sizeof(char)*size);
 		XSetWindowBackgroundPixmap (xWindow->display, cur_wnd, xWindow->pixmap);
-		break;
-
-	case VIDEO_XI_SHMSTD:
+		xWindow->pwidth = VideoWidth;
+		xWindow->pheight = VideoHeight;
+		GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("[X11] Using X11 Pixmap %08x\n", (u32)xWindow->pixmap));
+	} else if (xWindow->use_shared_memory) {
 		GF_SAFEALLOC(xWindow->shmseginfo, XShmSegmentInfo);
 		xWindow->surface = XShmCreateImage (xWindow->display, xWindow->visual,
-										 xWindow->depth, ZPixmap, NULL, xWindow->shmseginfo, 
-										 VideoWidth, VideoHeight);
-		xWindow->shmseginfo->shmid = shmget(IPC_PRIVATE,
-									 xWindow->surface->bytes_per_line * xWindow->surface->height,
+									 xWindow->depth, ZPixmap, NULL, xWindow->shmseginfo, 
+									 VideoWidth, VideoHeight);
+		xWindow->shmseginfo->shmid = shmget(IPC_PRIVATE, xWindow->surface->bytes_per_line * xWindow->surface->height,
 									 IPC_CREAT | 0777);
 
 		xWindow->surface->data = xWindow->shmseginfo->shmaddr = shmat(xWindow->shmseginfo->shmid, NULL, 0);
-		xWindow->back_buffer->buffer = (unsigned char *) xWindow->surface->data;
 		xWindow->shmseginfo->readOnly = False;
 		XShmAttach (xWindow->display, xWindow->shmseginfo);
-		break;
+	} else
 #endif
-
-	case VIDEO_XI_STANDARD:
-		xWindow->back_buffer->buffer = (char *) malloc(sizeof(char)*size);
+	{
+		char *data = (char *) malloc(sizeof(char)*size);
 		xWindow->surface = XCreateImage (xWindow->display, xWindow->visual,
 				      xWindow->depth, ZPixmap,
 				      0,
-				      xWindow->back_buffer->buffer,
+				      data,
 				      VideoWidth, VideoHeight,
-				      xWindow->bpp*8, xWindow->back_buffer->pitch);
+				      xWindow->bpp*8, xWindow->bpp*VideoWidth);
+		if (!xWindow->surface) {
+			free(data);
+			return GF_IO_ERR;
+		}
+
 	}
 	xWindow->is_init = 1;
 	return GF_OK;
@@ -590,7 +983,9 @@ GF_Err X11_InitBackBuffer (GF_VideoOutput * vout, u32 VideoWidth, u32 VideoHeigh
 GF_Err X11_ResizeBackBuffer (struct _video_out *vout, u32 newWidth, u32 newHeight)
 {
 	X11VID ();
-	if ((newWidth != xWindow->back_buffer->width) || (newHeight != xWindow->back_buffer->height)) {
+	u32 w = xWindow->surface ? xWindow->surface->width : xWindow->pwidth;
+	u32 h = xWindow->surface ? xWindow->surface->height : xWindow->pheight;
+	if (!xWindow->is_init || (newWidth != w) || (newHeight != h)) {
 		if ((newWidth >= 32) && (newHeight >= 32))
 			return X11_InitBackBuffer (vout, newWidth, newHeight);
 	}
@@ -617,6 +1012,11 @@ GF_Err X11_ProcessEvent (struct _video_out * vout, GF_Event * evt)
 		/*if owning the window and not in fullscreen, resize it (initial scene size)*/
 	xWindow->w_width = evt->size.width;
 	xWindow->w_height = evt->size.height;
+#ifdef ENABLE_JOYSTICK
+    xWindow->prev_x = xWindow->w_width/2;
+    xWindow->prev_y = xWindow->w_height/2;
+#endif     
+        
 	  if (!xWindow->fullscreen) {
 	    if (xWindow->par_wnd) {
 	      XWindowAttributes pwa;
@@ -629,11 +1029,21 @@ GF_Err X11_ProcessEvent (struct _video_out * vout, GF_Event * evt)
 	  }
 	  break;
 	case GF_EVENT_VIDEO_SETUP:
-		/*and resetup HW*/
+		switch (evt->setup.opengl_mode){
 #ifdef GPAC_HAS_OPENGL
-		if (xWindow->is_3D_out) return X11_SetupGL(vout);
+		case 1:
+			xWindow->output_3d_mode = 1;
+			return X11_SetupGL(vout);
+		case 2:
+			xWindow->output_3d_mode = 2;
+			return X11_SetupGLPixmap(vout, evt->setup.width, evt->setup.height);
 #endif
-		return X11_ResizeBackBuffer(vout, evt->size.width, evt->size.height);
+		case 0:
+			xWindow->output_3d_mode = 0;
+			return X11_ResizeBackBuffer(vout, evt->setup.width, evt->setup.height);
+		default:
+			return GF_NOT_SUPPORTED;
+		}
 	}
 	} else {
 	  X11_HandleEvents(vout);
@@ -648,7 +1058,7 @@ GF_Err X11_SetFullScreen (struct _video_out * vout, u32 bFullScreenOn, u32 * scr
 	X11VID ();
 	xWindow->fullscreen = bFullScreenOn;
 #ifdef GPAC_HAS_OPENGL
-	if (xWindow->is_3D_out) X11_ReleaseGL(xWindow);
+	if (xWindow->output_3d_mode==1) X11_ReleaseGL(xWindow);
 #endif
 
 	if (bFullScreenOn) {
@@ -684,7 +1094,7 @@ GF_Err X11_SetFullScreen (struct _video_out * vout, u32 bFullScreenOn, u32 * scr
 		/*backbuffer resize will be done right after this is called */
 	}
 #ifdef GPAC_HAS_OPENGL
-	if (xWindow->is_3D_out) X11_SetupGL(vout);
+	if (xWindow->output_3d_mode==1) X11_SetupGL(vout);
 #endif
 	return GF_OK;
 }
@@ -698,12 +1108,22 @@ GF_Err X11_LockBackBuffer(struct _video_out * vout, GF_VideoSurface * vi, u32 do
 
 	if (do_lock) {
 		if (!vi) return GF_BAD_PARAM;
-		vi->width = xWindow->back_buffer->width;
-		vi->height = xWindow->back_buffer->height;
-		vi->pitch = xWindow->back_buffer->pitch;
-		vi->pixel_format = xWindow->pixel_format;
-		vi->video_buffer = xWindow->back_buffer->buffer;
-		vi->is_hardware_memory = (xWindow->videoaccesstype==VIDEO_XI_STANDARD) ? 0 : 1;
+		if (xWindow->surface) {
+			vi->width = xWindow->surface->width;
+			vi->height = xWindow->surface->height;
+			vi->pitch = xWindow->surface->width*xWindow->bpp;
+			vi->pixel_format = xWindow->pixel_format;
+			vi->video_buffer = xWindow->surface->data;
+		} else {
+#ifdef GPAC_HAS_X11_SHM
+			vi->width = xWindow->pwidth;
+			vi->height = xWindow->pheight;
+			vi->pitch = xWindow->pwidth*xWindow->bpp;
+			vi->pixel_format = xWindow->pixel_format;
+			vi->video_buffer = (unsigned char *) xWindow->shmseginfo->shmaddr;
+#endif
+		}
+		vi->is_hardware_memory = (xWindow->use_shared_memory) ? 1 : 0;
 		return GF_OK;
 	} else {
 		return GF_OK;
@@ -732,6 +1152,21 @@ static int X11_BadAccess_ByPass(Display * display,
 	return 0;
 }
 
+
+static void X11_XScreenSaverState(XWindow *xwin, Bool turn_on)
+{
+	return;
+ 	if (turn_on) {
+		if (!xwin->ss_t) return;
+	        XSetScreenSaver(xwin->display, xwin->ss_t, xwin->ss_i, xwin->ss_b, xwin->ss_e);
+	} else {
+		/* Save state information */
+		XGetScreenSaver(xwin->display, &xwin->ss_t, &xwin->ss_i, &xwin->ss_b, &xwin->ss_e);
+		if (xwin->ss_t)
+		        XSetScreenSaver(xwin->display, 0, xwin->ss_i, xwin->ss_b, xwin->ss_e);
+	}
+}
+
 /*
  * Setup X11 wnd System
  */
@@ -764,9 +1199,9 @@ X11_SetupWindow (GF_VideoOutput * vout)
 	xWindow->bpp = xWindow->depth / 8;
 	xWindow->bpp = xWindow->bpp == 3 ? 4 : xWindow->bpp;
 
+xWindow->screennum=0;
 	vout->max_screen_width = DisplayWidth(xWindow->display, xWindow->screennum);
 	vout->max_screen_height = DisplayHeight(xWindow->display, xWindow->screennum);
-
 	/*
 	 * Full screen wnd
 	 */
@@ -783,7 +1218,7 @@ X11_SetupWindow (GF_VideoOutput * vout)
 
 	if (!xWindow->par_wnd) {
 		xWindow->w_width = 320;
-		xWindow->w_height = 20;
+		xWindow->w_height = 240;
 		xWindow->wnd = XCreateWindow (xWindow->display,
 					   RootWindowOfScreen(xWindow->screenptr), 0, 0,
 					   xWindow->w_width, xWindow->w_height, 0,
@@ -814,7 +1249,7 @@ X11_SetupWindow (GF_VideoOutput * vout)
 	XSync(xWindow->display, False);
 	XSetErrorHandler(old_handler);
 	if (selectinput_err) {
-       	XSelectInput(xWindow->display, xWindow->wnd,
+	       	XSelectInput(xWindow->display, xWindow->wnd,
 			StructureNotifyMask | PropertyChangeMask | ExposureMask |
 			KeyPressMask | KeyReleaseMask);
 
@@ -825,8 +1260,8 @@ X11_SetupWindow (GF_VideoOutput * vout)
 
 	XSizeHints *Hints = XAllocSizeHints ();
 	Hints->flags = PSize | PMinSize;
-	Hints->min_width = 32;
-	Hints->min_height = 32;
+	Hints->min_width = 64;
+	Hints->min_height = 64;
 	Hints->max_height = 4096;
 	Hints->max_width = 4096;
 	if (!xWindow->par_wnd) {
@@ -837,11 +1272,9 @@ X11_SetupWindow (GF_VideoOutput * vout)
 	Hints->y = 0;
 	Hints->flags |= USPosition;
 	XSetWMNormalHints (xWindow->display, xWindow->full_wnd, Hints);
-	XFree (Hints);
-
 
 	xWindow->the_gc = XCreateGC (xWindow->display, xWindow->wnd, 0, NULL);
-	xWindow->videoaccesstype = VIDEO_XI_STANDARD;
+	xWindow->use_shared_memory = 0;
 
 #ifdef GPAC_HAS_X11_SHM
 	sOpt = gf_modules_get_option((GF_BaseInterface *)vout, "Video", "UseHardwareMemory");
@@ -849,20 +1282,48 @@ X11_SetupWindow (GF_VideoOutput * vout)
 	  int XShmMajor, XShmMinor;
 	  Bool XShmPixmaps;
 	  if (XShmQueryVersion(xWindow->display, &XShmMajor, &XShmMinor, &XShmPixmaps)) {
-	    /*this is disabled due to flip pb (we cannot reposition backbuffer)*/
-	    if (0 && XShmPixmaps && (XShmPixmapFormat(xWindow->display) == ZPixmap)) {
-		xWindow->videoaccesstype = VIDEO_XI_SHMPIXMAP;
-	    } else {
-	      xWindow->videoaccesstype = VIDEO_XI_SHMSTD;
-	      GF_LOG(GF_LOG_INFO, GF_LOG_MMIO, ("[X11] Using X11 Hardware Blit\n"));
-	    }
+		xWindow->use_shared_memory = 1;
+		GF_LOG(GF_LOG_INFO, GF_LOG_MMIO, ("[X11] Using X11 Shared Memory\n"));
+		if ((XShmPixmaps==True) && (XShmPixmapFormat(xWindow->display)==ZPixmap)) {
+			GF_LOG(GF_LOG_INFO, GF_LOG_MMIO, ("[X11] X11 Shared Memory Pixmaps available\n"));
+		}
 	  }
 	}
+
 #endif
 
-	GF_SAFEALLOC(xWindow->back_buffer, X11WrapSurface);
-	xWindow->back_buffer->id = -1;
+#ifdef GPAC_HAS_X11_XV
+	sOpt = gf_modules_get_option((GF_BaseInterface *)vout, "Video", "DisableColorKeying");
+	if (sOpt && !strcmp(sOpt, "yes")) {
+		xWindow->xvport = X11_GetXVideoPort(vout, GF_PIXEL_I420, 0);
+	} else {
+		xWindow->xvport = X11_GetXVideoPort(vout, GF_PIXEL_I420, 1);
+		if (xWindow->xvport<0) {
+			GF_LOG(GF_LOG_INFO, GF_LOG_MMIO, ("[X11] Hardware has no color keying\n"));
+			vout->overlay_color_key = 0;
+			xWindow->xvport = X11_GetXVideoPort(vout, GF_PIXEL_I420, 0);
+		} else {
+			GF_LOG(GF_LOG_INFO, GF_LOG_MMIO, ("[X11] Hardware uses color key %08x\n", vout->overlay_color_key));
+		}
+	}
+	if (xWindow->xvport>=0) {
+		XvUngrabPort(xWindow->display, xWindow->xvport, CurrentTime );
+		xWindow->xvport = -1;
+		vout->yuv_pixel_format = X11_GetPixelFormat(xWindow->xv_pf_format);
+		vout->Blit = X11_Blit;
+		vout->hw_caps |= GF_VIDEO_HW_HAS_YUV_OVERLAY;
+		GF_LOG(GF_LOG_INFO, GF_LOG_MMIO, ("[X11] Using XV YUV Overlays\n"));
 
+#ifdef GPAC_HAS_X11_SHM
+		/*if user asked for YUV->RGB on offscreen, do it (it may crash the system)*/
+		sOpt = gf_modules_get_option((GF_BaseInterface *)vout, "Video", "EnableOffscreenYUV");
+		if (sOpt && !strcmp(sOpt, "yes")) {
+			vout->hw_caps |= GF_VIDEO_HW_HAS_YUV;
+			GF_LOG(GF_LOG_INFO, GF_LOG_MMIO, ("[X11] Using XV Offscreen YUV2RGB acceleration\n"));
+		}
+#endif
+	} 
+#endif
 
 	XSetWindowAttributes xsw;
 	xsw.border_pixel = WhitePixel (xWindow->display, xWindow->screennum);
@@ -894,49 +1355,83 @@ X11_SetupWindow (GF_VideoOutput * vout)
 		XSendEvent (xWindow->display,RootWindowOfScreen (xWindow->screenptr), False,
 			    mask, &ev);
 	}
-#ifdef GPAC_HAS_OPENGL
-	if (xWindow->is_3D_out) {
-	  int attribs[64];
-	  int i;
 
+	/*openGL setup*/
+#ifdef GPAC_HAS_OPENGL
+	{
+	  int attribs[64];
+	  int i, nb_bits;
+
+	  sOpt = gf_modules_get_option((GF_BaseInterface *)vout, "Video", "GLNbBitsPerComponent");
+	  nb_bits = sOpt ? atoi(sOpt) : 5;
 	  i=0;
 	  attribs[i++] = GLX_RGBA;
 	  attribs[i++] = GLX_RED_SIZE;
-	  attribs[i++] = 5;
+	  attribs[i++] = nb_bits;
 	  attribs[i++] = GLX_GREEN_SIZE;
-	  attribs[i++] = 5;
+	  attribs[i++] = nb_bits;
 	  attribs[i++] = GLX_BLUE_SIZE;
-	  attribs[i++] = 5;
-	  attribs[i++] = GLX_DEPTH_SIZE;
-	  attribs[i++] = 16;
-	  if (xWindow->gl_cfg.double_buffered) attribs[i++] = GLX_DOUBLEBUFFER;
+	  attribs[i++] = nb_bits;
+	  sOpt = gf_modules_get_option((GF_BaseInterface *)vout, "Video", "GLNbBitsDepth");
+	  nb_bits = sOpt ? atoi(sOpt) : 0;
+	  if (nb_bits) {
+		  attribs[i++] = GLX_DEPTH_SIZE;
+		  attribs[i++] = nb_bits;
+	  }
+	  sOpt = gf_modules_get_option((GF_BaseInterface *)vout, "Video", "UseGLDoubleBuffering");
+	  if (!sOpt || !strcmp(sOpt, "yes")) attribs[i++] = GLX_DOUBLEBUFFER;
 	  attribs[i++] = None;
 	  xWindow->glx_visualinfo = glXChooseVisual(xWindow->display, xWindow->screennum, attribs);
 	  if (!xWindow->glx_visualinfo) {
 		  GF_LOG(GF_LOG_ERROR, GF_LOG_MMIO, ("[X11] Error selecting GL display\n"));
 	  }
 	}
+
+	xWindow->gl_wnd = XCreateWindow (xWindow->display, RootWindowOfScreen (xWindow->screenptr),
+								   0,
+								   0,
+								   200,
+								   200, 0,
+								   xWindow->depth, InputOutput,
+								   xWindow->visual, 0, NULL);
+
+	XSync(xWindow->display, False);
+	XUnmapWindow(xWindow->display, (Window) xWindow->gl_wnd);
+	XSync(xWindow->display, False);
+
+	sOpt = gf_modules_get_option((GF_BaseInterface *)vout, "Video", "X113DOffscreenMode");
+        if (sOpt && !strcmp(sOpt, "Window")) {
+		xWindow->offscreen_type = 1;
+        } else if (sOpt && !strcmp(sOpt, "VisibleWindow")) {
+		xWindow->offscreen_type = 2;
+		XSetWMNormalHints (xWindow->display, xWindow->gl_wnd, Hints);
+       } else if (sOpt && !strcmp(sOpt, "Pixmap")) {
+		xWindow->offscreen_type = 0;
+	} else {
+		xWindow->offscreen_type = 0;
+	}
 #endif
+
+
+	/*turn off xscreensaver*/
+	X11_XScreenSaverState(xWindow, 0);
+
 	xWindow->setup_done = 1;
+	XFree (Hints);
 }
 
-GF_Err X11_Setup(struct _video_out *vout, void *os_handle, void *os_display, u32 no_proc_override, GF_GLConfig * cfg)
+GF_Err X11_Setup(struct _video_out *vout, void *os_handle, void *os_display, u32 no_proc_override)
 {
 	X11VID ();
 	/*assign window if any, NEVER display*/
 	xWindow->par_wnd = (Window) os_handle;
+
 	/*OSMOZILLA HACK*/
 	if (os_display) xWindow->no_select_input = 1;
-	if (cfg) {
-#ifdef GPAC_HAS_OPENGL
-		xWindow->is_3D_out = 1;
-	  xWindow->gl_cfg = *cfg;
-#else
-	  return GF_NOT_SUPPORTED;
+
+#ifdef ENABLE_JOYSTICK
+	xWindow->fd = open ("/dev/input/js0", O_NONBLOCK);	
 #endif
-	} else {
-	  xWindow->is_3D_out = 0;
-	}
 	/*the rest is done THROUGH THE MAIN RENDERER TRHEAD!!*/
 	return GF_OK;
 }
@@ -946,23 +1441,37 @@ void X11_Shutdown (struct _video_out *vout)
 {
 	X11VID ();
 
-	if (xWindow->is_3D_out) {
+	if (xWindow->output_3d_mode==1) {
 #ifdef GPAC_HAS_OPENGL
 		X11_ReleaseGL(xWindow);
 #endif
 	} else {
 		X11_ReleaseBackBuffer (vout);
 	}
-	free(xWindow->back_buffer);
+#ifdef ENABLE_JOYSTICK
+	close(xWindow->fd);	
+#endif
 	XFreeGC (xWindow->display, xWindow->the_gc);
 	XUnmapWindow (xWindow->display, (Window) xWindow->wnd);
 	XDestroyWindow (xWindow->display, (Window) xWindow->wnd);
 	XDestroyWindow (xWindow->display, (Window) xWindow->full_wnd);
+#ifdef GPAC_HAS_OPENGL
+	if (xWindow->gl_offscreen) glXDestroyGLXPixmap(xWindow->display, xWindow->gl_offscreen);
+	if (xWindow->gl_pixmap) XFreePixmap(xWindow->display, xWindow->gl_pixmap);
+	XUnmapWindow (xWindow->display, (Window) xWindow->gl_wnd);
+	XDestroyWindow (xWindow->display, (Window) xWindow->gl_wnd);
+#endif
+
+	/*restore xscreen saver*/
+	X11_XScreenSaverState(xWindow, 1);
+
 	XCloseDisplay (xWindow->display);
 	free (xWindow);
 }
 
 
+
+
 void *NewX11VideoOutput ()
 {
 	GF_VideoOutput *driv;
@@ -984,7 +1493,9 @@ void *NewX11VideoOutput ()
 	driv->Shutdown = X11_Shutdown;
 	driv->LockBackBuffer = X11_LockBackBuffer;
 	driv->ProcessEvent = X11_ProcessEvent;
-	driv->hw_caps = GF_VIDEO_HW_HAS_OPENGL;
+	driv->hw_caps = GF_VIDEO_HW_OPENGL;
+	/*fixme - needs a better detection scheme*/
+	driv->hw_caps |= GF_VIDEO_HW_OPENGL_OFFSCREEN | GF_VIDEO_HW_OPENGL_OFFSCREEN_ALPHA;
 	return (void *) driv;
 
 }
@@ -1033,3 +1544,4 @@ ShutdownInterface (GF_BaseInterface *ifce)
 		break;
 	}
 }
+
