@@ -39,6 +39,10 @@ extern "C"
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
 
+#if !defined(GPAC_DISABLE_3D) && !defined(GPAC_USE_OGL_ES) && !defined(GPAC_USE_TINYGL)
+#define GPAC_HAS_OPENGL
+#endif
+
 #ifdef GPAC_HAS_OPENGL
 #include <GL/glx.h>
 #endif
@@ -49,23 +53,22 @@ extern "C"
 #include <sys/shm.h>
 #endif
 
-#define X11VID()	XWindow *xWindow = (XWindow *)vout->opaque;
+#ifdef GPAC_HAS_X11_XV
+#include <X11/extensions/Xv.h>
+#include <X11/extensions/Xvlib.h>
+#endif
 
-#define VIDEO_XI_STANDARD     0x00	/* Use standard Xlib calls */
-#define VIDEO_XI_SHMSTD       0x01	/* Use Xlib shared memory extension */
-#define VIDEO_XI_SHMPIXMAP    0x02	/* Use shared memory pixmap */
+#ifdef ENABLE_JOYSTICK
+#include <linux/joystick.h>
+#include <unistd.h>
+#include <fcntl.h>
+#endif
+
+#define X11VID()	XWindow *xWindow = (XWindow *)vout->opaque;
 
 #define RGB555(r,g,b) (((r&248)<<7) + ((g&248)<<2)  + (b>>3))
 #define RGB565(r,g,b) (((r&248)<<8) + ((g&252)<<3)  + (b>>3))
 
-typedef struct
-{
-	unsigned char *buffer;	//surface buffer
-	u32 pitch, pixel_format, width, height, BPP;	//
-	u32 id;		//
-}
-X11WrapSurface;
-
 typedef struct
 {
 	Window par_wnd;	//main window handler passed to module, NULL otherwise
@@ -78,18 +81,24 @@ typedef struct
 	Visual *visual;		//...
 	GC the_gc;			//graphics context
 	XImage *surface;	//main drawing image: software mode
+	Pixmap pixmap;
+	u32 pwidth, pheight;
 	Atom WM_DELETE_WINDOW;	//window deletion
 
-	X11WrapSurface *back_buffer;	//back buffer
-	Colormap colormap;	//not for now
-	int videoaccesstype;	//
+	Bool use_shared_memory;	//
+	/*screensaver state*/
+        int ss_t, ss_b, ss_i, ss_e;
 
 #ifdef GPAC_HAS_X11_SHM
-	Pixmap pixmap;		//main drawing image : local sharememory mode
 	XShmSegmentInfo *shmseginfo;
 #endif
-	
-	GF_List *surfaces;	//surfaces list
+
+	/*YUV overlay*/	
+#ifdef GPAC_HAS_X11_XV
+	int xvport;
+	u32 xv_pf_format;
+	XvImage *overlay;
+#endif
 
 	Bool is_init, fullscreen, has_focus;
 
@@ -98,11 +107,20 @@ typedef struct
 
 	u32 w_width, w_height;
 	u32 depth, bpp, pixel_format;
-	Bool is_3D_out;
+	u32 output_3d_mode;
+
 #ifdef GPAC_HAS_OPENGL
 	XVisualInfo *glx_visualinfo;
 	GLXContext glx_context;
-	GF_GLConfig gl_cfg;
+	Pixmap gl_pixmap;
+	GLXPixmap gl_offscreen;
+	Window gl_wnd;
+	u32 offscreen_type;
+#endif
+#ifdef ENABLE_JOYSTICK
+	/*joystick device file descriptor*/
+	u32 fd;
+	s32 prev_x, prev_y;
 #endif
 } XWindow;
 
