@@ -90,13 +90,12 @@ GF_BifsDecoder *gf_bifs_decoder_new(GF_SceneGraph *scenegraph, Bool command_dec)
 
 	tmp->pCurrentProto = NULL;
 	tmp->scenegraph = scenegraph;
+	tmp->command_buffers = gf_list_new();
 	if (command_dec) {
 		tmp->dec_memory_mode = 1;
 		tmp->force_keep_qp = 1;
-		tmp->command_buffers = gf_list_new();
 	}
 	tmp->current_graph = NULL;
-//	tmp->mx = gf_mx_new();
 	return tmp;
 }
 
@@ -120,7 +119,14 @@ GF_Err gf_bifs_decoder_configure_stream(GF_BifsDecoder * codec, u16 ESID, char *
 	BIFSStreamInfo *pInfo;
 	GF_Err e;
 	
-	if (!DecoderSpecificInfo) return GF_BAD_PARAM;
+	if (!DecoderSpecificInfo) {
+		/* Hack for T-DMB non compliant streams */
+		GF_SAFEALLOC(pInfo, BIFSStreamInfo);
+		pInfo->ESID = ESID;
+		pInfo->config.PixelMetrics = 1;
+		pInfo->config.version = (objectTypeIndication==2) ? 1 : 2;
+		return gf_list_add(codec->streamInfo, pInfo);
+	}
 //	gf_mx_p(codec->mx);
 	if (gf_bifs_dec_get_stream(codec, ESID) != NULL) {
 //		gf_mx_v(codec->mx);
@@ -188,8 +194,6 @@ GF_Err gf_bifs_decoder_remove_stream(GF_BifsDecoder *codec, u16 ESID)
 GF_EXPORT
 void gf_bifs_decoder_del(GF_BifsDecoder *codec)
 {	
-	if (codec->GlobalQP) gf_node_unregister((GF_Node *) codec->GlobalQP, NULL);
-	
 	assert(gf_list_count(codec->QPs)==0);
 	gf_list_del(codec->QPs);
 
@@ -200,10 +204,14 @@ void gf_bifs_decoder_del(GF_BifsDecoder *codec)
 		gf_list_rem(codec->streamInfo, 0);
 	}
 	gf_list_del(codec->streamInfo);
-	if (codec->dec_memory_mode) {
-		assert(gf_list_count(codec->command_buffers) == 0);
-		gf_list_del(codec->command_buffers);
+
+	while (gf_list_count(codec->command_buffers)) {
+		CommandBufferItem *cbi = (CommandBufferItem *)gf_list_get(codec->command_buffers, 0);
+		free(cbi);
+		gf_list_rem(codec->command_buffers, 0);
 	}
+	gf_list_del(codec->command_buffers);
+
 //	gf_mx_del(codec->mx);
 	free(codec);
 }
@@ -246,7 +254,6 @@ GF_Err gf_bifs_decode_au(GF_BifsDecoder *codec, u16 ESID, char *data, u32 data_l
 	/*reset current config*/
 	codec->info = NULL;
 	codec->current_graph = NULL;
-
 //	gf_mx_v(codec->mx);
 	return e;
 }
@@ -274,7 +281,6 @@ GF_BifsEncoder *gf_bifs_encoder_new(GF_SceneGraph *graph)
 	tmp->QPs = gf_list_new();
 	tmp->streamInfo = gf_list_new();
 	tmp->info = NULL;	
-//	tmp->mx = gf_mx_new();
 	tmp->encoded_nodes = gf_list_new();
 	tmp->scene_graph = graph;
 	return tmp;
@@ -295,7 +301,6 @@ static BIFSStreamInfo *BE_GetStream(GF_BifsEncoder * codec, u16 ESID)
 GF_EXPORT
 void gf_bifs_encoder_del(GF_BifsEncoder *codec)
 {	
-	if (codec->GlobalQP) gf_node_unregister((GF_Node *) codec->GlobalQP, NULL);
 	assert(gf_list_count(codec->QPs)==0);
 	gf_list_del(codec->QPs);
 	/*destroy all config*/
@@ -324,7 +329,7 @@ GF_Err gf_bifs_encoder_new_stream(GF_BifsEncoder *codec, u16 ESID, GF_BIFSConfig
 	
 	GF_SAFEALLOC(pInfo, BIFSStreamInfo);
 	pInfo->ESID = ESID;
-	pInfo->UseName = encodeNames;
+	codec->UseName = encodeNames;
 	pInfo->config.Height = cfg->pixelHeight;
 	pInfo->config.Width = cfg->pixelWidth;
 	pInfo->config.NodeIDBits = cfg->nodeIDbits;
