@@ -23,7 +23,6 @@
  */
 
 
-#include <gpac/internal/scenegraph_dev.h>
 #include <gpac/internal/bifs_dev.h>
 #include "quant.h" 
 
@@ -141,9 +140,9 @@ static GF_Err BD_DecMultipleReplace(GF_BifsDecoder * codec, GF_BitStream *bs)
 	
 	flag = gf_bs_read_int(bs, 1);
 	if (flag) {
-		e = gf_bifs_dec_node_mask(codec, bs, node);
+		e = gf_bifs_dec_node_mask(codec, bs, node, 0);
 	} else {
-		e = gf_bifs_dec_node_list(codec, bs, node);
+		e = gf_bifs_dec_node_list(codec, bs, node, 0);
 	}
 	return e;
 }
@@ -155,8 +154,11 @@ static GF_Err BD_DecGlobalQuantizer(GF_BifsDecoder * codec, GF_BitStream *bs)
 	node = gf_bifs_dec_node(codec, bs, NDT_SFWorldNode);
 
 	/*reset global QP*/
-	if (codec->GlobalQP) gf_node_unregister((GF_Node *) codec->GlobalQP, NULL);
-	codec->GlobalQP = codec->ActiveQP = NULL;
+	if (codec->scenegraph->global_qp) {
+		gf_node_unregister(codec->scenegraph->global_qp, NULL);
+		codec->scenegraph->global_qp = NULL;
+	}
+	codec->ActiveQP = NULL;
 	
 	if (!node || (gf_node_get_tag(node) != TAG_MPEG4_QuantizationParameter)) {
 		if (node) gf_node_unregister(node, NULL);
@@ -164,9 +166,10 @@ static GF_Err BD_DecGlobalQuantizer(GF_BifsDecoder * codec, GF_BitStream *bs)
 	}
 
 	/*register global QP*/
-	codec->GlobalQP = codec->ActiveQP = (M_QuantizationParameter *) node;
-	codec->GlobalQP->isLocal = 0;
+	codec->scenegraph->global_qp = node;
 	gf_node_register(node, NULL);
+	codec->ActiveQP = (M_QuantizationParameter *) node;
+	codec->ActiveQP->isLocal = 0;
 	return GF_OK;
 }
 
@@ -180,6 +183,110 @@ static GF_Err BD_DecNodeDeleteEx(GF_BifsDecoder * codec, GF_BitStream *bs)
 	return gf_node_replace(node, NULL, 1);
 }
 
+static GF_Err BD_DecOperandReplace(GF_BifsDecoder * codec, GF_BitStream *bs)
+{
+	s32 pos;
+	GF_FieldInfo field;
+	GF_FieldInfo src_field;
+	GF_Err e;
+	u32 NodeID, NumBits, ind, field_ind, type, src_type, dst_type;
+	GF_Node *node, *src;
+	void *src_ptr, *dst_ptr;
+
+	NodeID = 1 + gf_bs_read_int(bs, codec->info->config.NodeIDBits);
+	node = gf_sg_find_node(codec->current_graph, NodeID);
+	if (!node) return GF_NON_COMPLIANT_BITSTREAM;
+	NumBits = gf_get_bit_size(gf_node_get_num_fields_in_mode(node, GF_SG_FIELD_CODING_IN)-1);
+	ind = gf_bs_read_int(bs, NumBits);
+	e = gf_bifs_get_field_index(node, ind, GF_SG_FIELD_CODING_IN, &field_ind);
+	if (e) return e;
+	e = gf_node_get_field(node, field_ind, &field);
+	if (e) return e;
+
+	dst_type = field.fieldType;
+	dst_ptr = field.far_ptr;
+	pos = -2;
+	if (! gf_sg_vrml_is_sf_field(field.fieldType)) {
+		type = gf_bs_read_int(bs, 2);
+		switch (type) {
+		/*no index*/
+		case 0:
+			break;
+		/*specified*/
+		case 1:
+			pos = gf_bs_read_int(bs, 8);
+			break;
+		/*first*/
+		case 2:
+			pos = 0;
+			break;
+		/*last*/
+		case 3:
+			/*-1 means append*/
+			pos = ((GenMFField *)field.far_ptr)->count;
+			if (!pos) 
+				return GF_NON_COMPLIANT_BITSTREAM;
+			pos -= 1;
+			break;
+		default:
+			return GF_NON_COMPLIANT_BITSTREAM;
+		}
+		if (pos>-2) {
+			dst_type = gf_sg_vrml_get_sf_type(field.fieldType);
+			e = gf_sg_vrml_mf_get_item(field.far_ptr, field.fieldType, &dst_ptr, pos);
+			if (e) return e;
+		}
+	}
+
+	NodeID = 1 + gf_bs_read_int(bs, codec->info->config.NodeIDBits);
+	src = gf_sg_find_node(codec->current_graph, NodeID);
+	if (!src) return GF_NON_COMPLIANT_BITSTREAM;
+	NumBits = gf_get_bit_size(gf_node_get_num_fields_in_mode(src, GF_SG_FIELD_CODING_IN)-1);
+	ind = gf_bs_read_int(bs, NumBits);
+	e = gf_bifs_get_field_index(src, ind, GF_SG_FIELD_CODING_IN, &field_ind);
+	if (e) return e;
+	e = gf_node_get_field(src, field_ind, &src_field);
+	if (e) return e;
+
+	src_type = src_field.fieldType;
+	src_ptr = src_field.far_ptr;
+	pos = -2;
+	if (! gf_sg_vrml_is_sf_field(src_field.fieldType)) {
+		type = gf_bs_read_int(bs, 2);
+		switch (type) {
+		/*no index*/
+		case 0:
+			break;
+		/*specified*/
+		case 1:
+			pos = gf_bs_read_int(bs, 8);
+			break;
+		/*first*/
+		case 2:
+			pos = 0;
+			break;
+		/*last*/
+		case 3:
+			/*-1 means append*/
+			pos = ((GenMFField *)src_field.far_ptr)->count;
+			if (!pos) 
+				return GF_NON_COMPLIANT_BITSTREAM;
+			pos -= 1;
+			break;
+		default:
+			return GF_NON_COMPLIANT_BITSTREAM;
+		}
+		if (pos>-2) {
+			src_type = gf_sg_vrml_get_sf_type(src_field.fieldType);
+			e = gf_sg_vrml_mf_get_item(src_field.far_ptr, src_field.fieldType, &src_ptr, pos);
+			if (e) return e;
+		}
+	}
+	if (src_type!=dst_type) return GF_NON_COMPLIANT_BITSTREAM;
+	gf_sg_vrml_field_copy(dst_ptr, src_ptr, src_type);
+	return GF_OK;
+}
+
 static GF_Err BD_DecExtendedUpdate(GF_BifsDecoder * codec, GF_BitStream *bs)
 {
 	u32 type;
@@ -200,6 +307,8 @@ static GF_Err BD_DecExtendedUpdate(GF_BifsDecoder * codec, GF_BitStream *bs)
 		return BD_DecGlobalQuantizer(codec, bs);
 	case 6:
 		return BD_DecNodeDeleteEx(codec, bs);
+	case 7:
+		return BD_DecOperandReplace(codec, bs);
 	default:
 		return GF_BIFS_UNKNOWN_VERSION;
 	}
@@ -245,7 +354,8 @@ static GF_Err BD_DecNodeInsert(GF_BifsDecoder * codec, GF_BitStream *bs)
 	if (!e) {
 		GF_FieldInfo field;
 		/*get it by name in case no add/removeChildren*/
-		gf_node_get_field_by_name(def, "children", &field);
+		e = gf_node_get_field_by_name(def, "children", &field);
+		if (e) return e;
 		gf_bifs_check_field_change(def, &field);
 	}
 	return e;
@@ -586,7 +696,7 @@ static GF_Err BD_DecRouteReplace(GF_BifsDecoder * codec, GF_BitStream *bs)
 	ptr = NULL; 
 	if (r) {
 		ptr = gf_sg_route_get_name(r);
-		gf_sg_route_del(r);
+//		gf_sg_route_del(r);
 	}
 #endif
 
@@ -610,11 +720,27 @@ static GF_Err BD_DecRouteReplace(GF_BifsDecoder * codec, GF_BitStream *bs)
 	e = gf_bifs_get_field_index(InNode, ind, GF_SG_FIELD_CODING_IN, &toID);
 	if (e) return e;
 
-	r = gf_sg_route_new(codec->current_graph, OutNode, fromID, InNode, toID);
-	if (!r) return GF_OUT_OF_MEM;
-	gf_sg_route_set_id(r, RouteID);
-	if (ptr) e = gf_sg_route_set_name(r, name);
+	if (r) {
+		if (r->FromNode->sgprivate->interact)
+			gf_list_del_item(r->FromNode->sgprivate->interact->routes, r);
 
+		r->is_setup = 0;
+		r->lastActivateTime = 0;
+
+		r->FromNode = OutNode;
+		r->FromField.fieldIndex = fromID;
+		r->ToNode = InNode;
+		r->ToField.fieldIndex = toID;
+
+		if (!r->FromNode->sgprivate->interact) GF_SAFEALLOC(r->FromNode->sgprivate->interact, struct _node_interactive_ext);
+		if (!r->FromNode->sgprivate->interact->routes) r->FromNode->sgprivate->interact->routes = gf_list_new();
+		gf_list_add(r->FromNode->sgprivate->interact->routes, r);
+	} else {
+		r = gf_sg_route_new(codec->current_graph, OutNode, fromID, InNode, toID);
+		if (!r) return GF_OUT_OF_MEM;
+		gf_sg_route_set_id(r, RouteID);
+		if (ptr) e = gf_sg_route_set_name(r, name);
+	}
 	return e;
 }
 
@@ -668,7 +794,7 @@ GF_Err gf_bifs_dec_proto_list(GF_BifsDecoder * codec, GF_BitStream *bs, GF_List
 		/*1- proto interface declaration*/
 		ID = gf_bs_read_int(bs, codec->info->config.ProtoIDBits);
 
-		if (codec->info->UseName) {
+		if (codec->UseName) {
 			gf_bifs_dec_name(bs, name);
 		} else {
 			sprintf(name, "Proto%d", numProtos);
@@ -688,7 +814,7 @@ GF_Err gf_bifs_dec_proto_list(GF_BifsDecoder * codec, GF_BitStream *bs, GF_List
 			event_type = gf_bs_read_int(bs, 2);
 			field_type = gf_bs_read_int(bs, 6);
 			
-			if (codec->info->UseName) {
+			if (codec->UseName) {
 				gf_bifs_dec_name(bs, name);
 			} else {
 				sprintf(name, "_field%d", numFields);
@@ -770,8 +896,13 @@ GF_Err gf_bifs_dec_proto_list(GF_BifsDecoder * codec, GF_BitStream *bs, GF_List
 				/*parse all nodes in SFWorldNode table*/
 				node = gf_bifs_dec_node(codec, bs, NDT_SFWorldNode);
 				if (!node) {
-					e = codec->LastError;
-					goto exit;
+					if (codec->LastError) {
+						e = codec->LastError;
+						goto exit;
+					} else {
+						flag = gf_bs_read_int(bs, 1);
+						continue;
+					}
 				}
 				e = gf_node_register(node, NULL);
 				if (e) goto exit;
@@ -901,7 +1032,7 @@ GF_Err gf_bifs_dec_route(GF_BifsDecoder * codec, GF_BitStream *bs, Bool is_inser
 	/*def'ed route*/
 	if (flag) {
 		RouteID = 1 + gf_bs_read_int(bs, codec->info->config.RouteIDBits);
-		if (codec->info->UseName) gf_bifs_dec_name(bs, name);
+		if (codec->UseName) gf_bifs_dec_name(bs, name);
 	}
 	/*origin*/
 	node_id = 1 + gf_bs_read_int(bs, codec->info->config.NodeIDBits);
@@ -928,7 +1059,7 @@ GF_Err gf_bifs_dec_route(GF_BifsDecoder * codec, GF_BitStream *bs, Bool is_inser
 	if (!r) return GF_OUT_OF_MEM;
 	if (RouteID) {
 		e = gf_sg_route_set_id(r, RouteID);
-		if (!e && codec->info->UseName) e = gf_sg_route_set_name(r, name);
+		if (!e && codec->UseName) e = gf_sg_route_set_name(r, name);
 	}
 	return e;
 }
@@ -947,8 +1078,7 @@ GF_Err BD_DecSceneReplace(GF_BifsDecoder * codec, GF_BitStream *bs, GF_List *pro
 	/*reserved*/
 	i = gf_bs_read_int(bs, 6);
 
-	codec->info->UseName = gf_bs_read_int(bs, 1);
-
+	codec->UseName = gf_bs_read_int(bs, 1);
 	/*parse PROTOS*/
 	e = gf_bifs_dec_proto_list(codec, bs, proto_list);
 	if (e) goto exit;
@@ -1000,6 +1130,8 @@ GF_Err gf_bifs_dec_command(GF_BifsDecoder * codec, GF_BitStream *bs)
 	GF_Err e;
 	e = codec->LastError = GF_OK;
 
+	codec->ActiveQP = (M_QuantizationParameter*)codec->scenegraph->global_qp;
+
 	while (1) {
 		u8 type = gf_bs_read_int(bs, 2);
 		switch (type) {
@@ -1023,6 +1155,8 @@ GF_Err gf_bifs_dec_command(GF_BifsDecoder * codec, GF_BitStream *bs)
 	while (gf_list_count(codec->QPs)) {
 		gf_bifs_dec_qp_remove(codec, 1);
 	}
+	gf_bifs_flush_command_list(codec);
+
 	return GF_OK;
 }
 
