@@ -24,7 +24,6 @@
 
 
 #include <gpac/internal/bifs_dev.h>
-#include <gpac/internal/scenegraph_dev.h>
 #include "quant.h" 
 
 GF_Err BE_EncProtoList(GF_BifsEncoder *codec, GF_List *protoList, GF_BitStream *bs);
@@ -140,16 +139,20 @@ static GF_Err BE_GlobalQuantizer(GF_BifsEncoder * codec, GF_Command *com, GF_Bit
 	if (e) return e;
 
 	/*reset global QP*/
-	if (codec->GlobalQP) gf_node_unregister((GF_Node *) codec->GlobalQP, NULL);
-	codec->GlobalQP = codec->ActiveQP = NULL;
+	if (codec->scene_graph->global_qp) {
+		gf_node_unregister(codec->scene_graph->global_qp, NULL);
+		codec->scene_graph->global_qp = NULL;
+	}
+	codec->ActiveQP = NULL;
 	
 	/*no QP*/
 	if (!inf->new_node) return GF_OK;
 
 	/*register global QP*/
-	codec->GlobalQP = codec->ActiveQP = (M_QuantizationParameter *) inf->new_node;
-	codec->GlobalQP->isLocal = 0;
+	codec->scene_graph->global_qp = inf->new_node;
 	gf_node_register(inf->new_node, NULL);
+	codec->ActiveQP = (M_QuantizationParameter *) inf->new_node;
+	codec->ActiveQP->isLocal = 0;
 	return GF_OK;
 }
 
@@ -397,7 +400,7 @@ GF_Err BE_RouteReplace(GF_BifsEncoder *codec, GF_Command *com, GF_BitStream *bs,
 		GF_BIFS_WRITE_INT(codec, bs, com->RouteID ? 1 : 0, 1, "isDEF", NULL);
 		if (com->RouteID) {
 			GF_BIFS_WRITE_INT(codec, bs, com->RouteID-1, codec->info->config.RouteIDBits, "RouteID", NULL);
-			if (codec->info->UseName) gf_bifs_enc_name(codec, bs, com->def_name);
+			if (codec->UseName) gf_bifs_enc_name(codec, bs, com->def_name);
 		}
 	} else {
 		GF_BIFS_WRITE_INT(codec, bs, com->RouteID - 1, codec->info->config.RouteIDBits, "RouteID", NULL);
@@ -460,7 +463,7 @@ GF_Err BE_EncProtoList(GF_BifsEncoder *codec, GF_List *protoList, GF_BitStream *
 		/*1- proto interface declaration*/
 		GF_BIFS_WRITE_INT(codec, bs, proto->ID, codec->info->config.ProtoIDBits, "protoID", NULL);
 
-		if (codec->info->UseName) gf_bifs_enc_name(codec, bs, proto->Name);
+		if (codec->UseName) gf_bifs_enc_name(codec, bs, proto->Name);
 
 		numFields = gf_list_count(proto->proto_fields);
 		for (j=0; j<numFields; j++) {
@@ -470,7 +473,7 @@ GF_Err BE_EncProtoList(GF_BifsEncoder *codec, GF_List *protoList, GF_BitStream *
 			GF_BIFS_WRITE_INT(codec, bs, proto_field->EventType, 2, "eventType", NULL);
 			GF_BIFS_WRITE_INT(codec, bs, proto_field->FieldType, 6, "fieldType", NULL);
 			
-			if (codec->info->UseName) gf_bifs_enc_name(codec, bs, proto_field->FieldName);
+			if (codec->UseName) gf_bifs_enc_name(codec, bs, proto_field->FieldName);
 			switch (proto_field->EventType) {
 			case GF_SG_EVENT_EXPOSED_FIELD:
 			case GF_SG_EVENT_FIELD:
@@ -506,14 +509,20 @@ GF_Err BE_EncProtoList(GF_BifsEncoder *codec, GF_List *protoList, GF_BitStream *
 			if (e) goto exit;
 
 			count = gf_list_count(proto->node_code);
-			for (j=0; j<count; j++) {
-				/*parse all nodes in SFWorldNode table*/
-				node = (GF_Node*)gf_list_get(proto->node_code, j);
-				e = gf_bifs_enc_node(codec, node, NDT_SFWorldNode, bs);
-				if (e) goto exit;
-				GF_BIFS_WRITE_INT(codec, bs, (j+1==count) ? 0 : 1, 1, "moreNodes", NULL);
+			/*BIFS cannot encode empty protos ! We therefore encode a NULL node instead*/
+			if (!count) {
+				gf_bifs_enc_node(codec, NULL, NDT_SFWorldNode, bs);
+				GF_BIFS_WRITE_INT(codec, bs, 0, 1, "moreNodes", NULL);
+			} else {
+				for (j=0; j<count; j++) {
+					/*parse all nodes in SFWorldNode table*/
+					node = (GF_Node*)gf_list_get(proto->node_code, j);
+					e = gf_bifs_enc_node(codec, node, NDT_SFWorldNode, bs);
+					if (e) goto exit;
+					GF_BIFS_WRITE_INT(codec, bs, (j+1==count) ? 0 : 1, 1, "moreNodes", NULL);
+				}
 			}
-			
+
 			/*encode routes routes*/
 			nbRoutes = count = gf_list_count(proto->sub_graph->Routes);
 			for (j=0; j<count; j++) {
@@ -614,7 +623,7 @@ GF_Err gf_bifs_enc_route(GF_BifsEncoder *codec, GF_Route *r, GF_BitStream *bs)
 	/*def'ed route*/
 	if (r->ID) {
 		GF_BIFS_WRITE_INT(codec, bs, r->ID-1, codec->info->config.RouteIDBits, "routeID", NULL);
-		if (codec->info->UseName) gf_bifs_enc_name(codec, bs, r->name);
+		if (codec->UseName) gf_bifs_enc_name(codec, bs, r->name);
 	}
 	/*origin*/
 	GF_BIFS_WRITE_INT(codec, bs, gf_node_get_id(r->FromNode) - 1, codec->info->config.NodeIDBits, "outNodeID", NULL);
@@ -640,10 +649,15 @@ GF_Err BE_SceneReplaceEx(GF_BifsEncoder *codec, GF_Command *com, GF_BitStream *b
 
 	/*reserved*/
 	GF_BIFS_WRITE_INT(codec, bs, 0, 6, "reserved", NULL);
-	GF_BIFS_WRITE_INT(codec, bs, codec->info->UseName ? 1 : 0, 1, "useName", NULL);
+	GF_BIFS_WRITE_INT(codec, bs, codec->UseName ? 1 : 0, 1, "useName", NULL);
 
-	e = BE_EncProtoList(codec, com->new_proto_list, bs);
-	if (e) goto exit;
+	if (gf_list_count(com->new_proto_list)) {
+		e = BE_EncProtoList(codec, com->new_proto_list, bs);
+		if (e) goto exit;
+	} else {
+		e = BE_EncProtoList(codec, com->in_scene->protos, bs);
+		if (e) goto exit;
+	}
 
 	/*NULL root is valid for ProtoLibraries*/
 	e = gf_bifs_enc_node(codec, com->node, NDT_SFTopNode, bs);
@@ -684,7 +698,7 @@ GF_Err BE_SceneReplace(GF_BifsEncoder *codec, GF_SceneGraph *graph, GF_BitStream
 
 	/*reserved*/
 	GF_BIFS_WRITE_INT(codec, bs, 0, 6, "reserved", NULL);
-	GF_BIFS_WRITE_INT(codec, bs, codec->info->UseName ? 1 : 0, 1, "useName", NULL);
+	GF_BIFS_WRITE_INT(codec, bs, codec->UseName ? 1 : 0, 1, "useName", NULL);
 
 	/*assign current graph*/
 	codec->scene_graph = graph;
@@ -745,22 +759,33 @@ GF_Err gf_bifs_enc_commands(GF_BifsEncoder *codec, GF_List *comList, GF_BitStrea
 			while (gf_list_count(codec->encoded_nodes)) gf_list_rem(codec->encoded_nodes, 0);
 			GF_BIFS_WRITE_INT(codec, bs, 3, 2, "SceneReplace", NULL);
 		
-			routes = gf_list_new();
-			/*now the trick: get all following InsertRoutes and convert as routes*/
-			for (; i<count-1; i++) {
-				GF_Route *r;
-				GF_Command *rcom = (GF_Command*)gf_list_get(comList, i+1);
-				if (rcom->tag!=GF_SG_ROUTE_INSERT) break;
-				GF_SAFEALLOC(r, GF_Route);
-				r->FromField.fieldIndex = rcom->fromFieldIndex;
-				r->FromNode = gf_sg_find_node(codec->scene_graph, rcom->fromNodeID);
-				r->ToField.fieldIndex = rcom->toFieldIndex;
-				r->ToNode = gf_sg_find_node(codec->scene_graph, rcom->toNodeID);
-				r->ID = rcom->RouteID;
-				r->name = rcom->def_name;
-				gf_list_add(routes, r);
+			if (!com->aggregated) {
+				routes = gf_list_new();
+				/*now the trick: get all following InsertRoutes and convert as routes*/
+				for (; i<count-1; i++) {
+					GF_Route *r;
+					GF_Command *rcom = (GF_Command*)gf_list_get(comList, i+1);
+					if (rcom->tag!=GF_SG_ROUTE_INSERT) break;
+					GF_SAFEALLOC(r, GF_Route);
+					r->FromField.fieldIndex = rcom->fromFieldIndex;
+					r->FromNode = gf_sg_find_node(codec->scene_graph, rcom->fromNodeID);
+					r->ToField.fieldIndex = rcom->toFieldIndex;
+					r->ToNode = gf_sg_find_node(codec->scene_graph, rcom->toNodeID);
+					r->ID = rcom->RouteID;
+					r->name = rcom->def_name;
+					gf_list_add(routes, r);
+				}
+				e = BE_SceneReplaceEx(codec, com, bs, routes);
+
+				while (gf_list_count(routes)) {
+					GF_Route *r = (GF_Route*)gf_list_get(routes, 0);
+					gf_list_rem(routes, 0);
+					free(r);
+				}
+				gf_list_del(routes);
+			} else {
+				e = BE_SceneReplaceEx(codec, com, bs, codec->scene_graph->Routes);
 			}
-			e = BE_SceneReplaceEx(codec, com, bs, routes);
 		}
 			break;
 		/*replace commands*/
@@ -824,14 +849,6 @@ GF_Err gf_bifs_enc_commands(GF_BifsEncoder *codec, GF_List *comList, GF_BitStrea
 		GF_BIFS_WRITE_INT(codec, bs, (i+1==count) ? 0 : 1, 1, "moreCommands", NULL);
 	}
 
-	if (routes) {
-		while (gf_list_count(routes)) {
-			GF_Route *r = (GF_Route*)gf_list_get(routes, 0);
-			gf_list_rem(routes, 0);
-			free(r);
-		}
-		gf_list_del(routes);
-	}
 	while (gf_list_count(codec->QPs)) gf_bifs_enc_qp_remove(codec, 1);
 	return e;
 }
