@@ -25,7 +25,6 @@
 
 
 #include <gpac/internal/bifs_dev.h>
-#include <gpac/internal/scenegraph_dev.h>
 #include "quant.h" 
 #include "script.h" 
 
@@ -207,21 +206,17 @@ GF_Err gf_bifs_dec_sf_field(GF_BifsDecoder * codec, GF_BitStream *bs, GF_Node *n
 		//notify the node - this is needed in case an enhencement layer replaces the buffer, in which case 
 		//the # ID Bits may change
 		SFCommandBufferChanged(codec, node);
-		/*memory mode, register command buffer for later parsing*/
-		if (codec->dec_memory_mode) {
+
+		/*
+		 1 - memory mode, register command buffer for later parsing
+		 2 - InputSensor only works on decompressed commands
+		*/
+		if (codec->dec_memory_mode || (node->sgprivate->tag==TAG_MPEG4_InputSensor)) {
 			CommandBufferItem *cbi = (CommandBufferItem *)malloc(sizeof(CommandBufferItem));
 			cbi->node = node;
 			cbi->cb = sfcb;
 			gf_list_add(codec->command_buffers, cbi);
 		}
-		/*InputSensor only work on decompressed commands*/
-		else if (node->sgprivate->tag==TAG_MPEG4_InputSensor) {
-			GF_Err BM_ParseCommand(GF_BifsDecoder *codec, GF_BitStream *bs, GF_List *com_list);
-			GF_BitStream *is_bs;
-			is_bs = gf_bs_new((char*)sfcb->buffer, sfcb->bufferSize, GF_BITSTREAM_READ);
-			e = BM_ParseCommand(codec, is_bs, sfcb->commandList);
-			gf_bs_del(is_bs);
-		}
 	} 
 		break;
 	case GF_SG_VRML_SFNODE:
@@ -507,7 +502,7 @@ GF_Err BD_SetProtoISed(GF_BifsDecoder * codec, u32 protofield, GF_Node *n, u32 n
 	}
 }
 
-GF_Err gf_bifs_dec_node_list(GF_BifsDecoder * codec, GF_BitStream *bs, GF_Node *node)
+GF_Err gf_bifs_dec_node_list(GF_BifsDecoder * codec, GF_BitStream *bs, GF_Node *node, Bool is_proto)
 {
 	u8 flag;
 	GF_Err e;
@@ -551,11 +546,13 @@ GF_Err gf_bifs_dec_node_list(GF_BifsDecoder * codec, GF_BitStream *bs, GF_Node *
 		e = gf_bifs_dec_field(codec, bs, node, &field);
 		if (e) return e;
 		flag = gf_bs_read_int(bs, 1);
+
+		if (is_proto) gf_sg_proto_mark_field_loaded(node, &field);
 	}
 	return codec->LastError;
 }
 
-GF_Err gf_bifs_dec_node_mask(GF_BifsDecoder * codec, GF_BitStream *bs, GF_Node *node)
+GF_Err gf_bifs_dec_node_mask(GF_BifsDecoder * codec, GF_BitStream *bs, GF_Node *node, Bool is_proto)
 {
 	u32 i, numFields, numProtoFields, index, flag, nbBits;
 	GF_Err e;
@@ -589,7 +586,7 @@ GF_Err gf_bifs_dec_node_mask(GF_BifsDecoder * codec, GF_BitStream *bs, GF_Node *
 			if (e) return e;
 		}
 	}
-	//Anim coding
+	//regular coding
 	else {
 		numFields = gf_node_get_num_fields_in_mode(node, GF_SG_FIELD_CODING_DEF);
 		for (i=0; i<numFields; i++) {
@@ -600,6 +597,8 @@ GF_Err gf_bifs_dec_node_mask(GF_BifsDecoder * codec, GF_BitStream *bs, GF_Node *
 			if (e) return e;
 			e = gf_bifs_dec_field(codec, bs, node, &field);
 			if (e) return e;
+
+			if (is_proto) gf_sg_proto_mark_field_loaded(node, &field);
 		}
 	}
 	return GF_OK;
@@ -652,7 +651,7 @@ GF_Node *gf_bifs_dec_node(GF_BifsDecoder * codec, GF_BitStream *bs, u32 NDT_Tag)
 {
 	u32 nodeID, NDTBits, node_type, node_tag, ProtoID, BVersion;
 	u8 node_flag;
-	Bool skip_init;
+	Bool skip_init, reset_qp14;
 	GF_Node *new_node;
 	GF_Err e;
 	GF_Proto *proto;
@@ -754,7 +753,7 @@ GF_Node *gf_bifs_dec_node(GF_BifsDecoder * codec, GF_BitStream *bs, u32 NDT_Tag)
 	if ((node_tag == TAG_MPEG4_IndexedFaceSet) && codec->info->config.Use3DMeshCoding) {
 		if (gf_bs_read_int(bs, 1)) {
 			nodeID = 1 + gf_bs_read_int(bs, codec->info->config.NodeIDBits);
-			if (codec->info->UseName) gf_bifs_dec_name(bs, name);
+			if (codec->UseName) gf_bifs_dec_name(bs, name);
 		}
 		/*parse the 3DMesh node*/
 		return NULL;
@@ -773,7 +772,7 @@ GF_Node *gf_bifs_dec_node(GF_BifsDecoder * codec, GF_BitStream *bs, u32 NDT_Tag)
 			return NULL;
 		}
 		nodeID = 1 + gf_bs_read_int(bs, codec->info->config.NodeIDBits);
-		if (codec->info->UseName) gf_bifs_dec_name(bs, name);
+		if (codec->UseName) gf_bifs_dec_name(bs, name);
 	}
 
 	new_node = NULL;
@@ -807,6 +806,7 @@ GF_Node *gf_bifs_dec_node(GF_BifsDecoder * codec, GF_BitStream *bs, u32 NDT_Tag)
 	
 	if (!new_node) {
 		if (proto) {
+			skip_init = 1;
 			/*create proto interface*/
 			new_node = gf_sg_proto_create_instance(codec->current_graph, proto);
 		} else {
@@ -833,6 +833,11 @@ GF_Node *gf_bifs_dec_node(GF_BifsDecoder * codec, GF_BitStream *bs, u32 NDT_Tag)
 
 	/*update default time fields except in proto parsing*/
 	if (!codec->pCurrentProto) UpdateTimeNode(codec, new_node);
+	/*nodes are only init outside protos */
+	else skip_init = 1;
+
+	/*if coords were not stored for QP14 before coding this node, reset QP14 it when leaving*/
+	reset_qp14 = !codec->coord_stored;
 
 	/*QP 14 is a special quant mode for IndexFace/Line(2D)Set to quantize the 
 	coordonate(2D) child, based on the first field parsed
@@ -844,10 +849,13 @@ GF_Node *gf_bifs_dec_node(GF_BifsDecoder * codec, GF_BitStream *bs, u32 NDT_Tag)
 	}
 
 	if (gf_bs_read_int(bs, 1)) {
-		e = gf_bifs_dec_node_mask(codec, bs, new_node);
+		e = gf_bifs_dec_node_mask(codec, bs, new_node, proto ? 1 : 0);
 	} else {
-		e = gf_bifs_dec_node_list(codec, bs, new_node);
+		e = gf_bifs_dec_node_list(codec, bs, new_node, proto ? 1 : 0);
 	}
+	if (codec->coord_stored && reset_qp14) 
+		gf_bifs_dec_qp14_reset(codec);
+
 	if (e) {
 		codec->LastError = e;
 		/*register*/
@@ -857,16 +865,10 @@ GF_Node *gf_bifs_dec_node(GF_BifsDecoder * codec, GF_BitStream *bs, u32 NDT_Tag)
 		return NULL;
 	}
 
-	/*nodes are only init outside protos */
-	if (!proto && !codec->pCurrentProto && new_node && !skip_init) gf_node_init(new_node);
+	if (!skip_init) 
+		gf_node_init(new_node);
 
 	switch (node_tag) {
-	case TAG_MPEG4_IndexedFaceSet:
-	case TAG_MPEG4_IndexedFaceSet2D:
-	case TAG_MPEG4_IndexedLineSet:
-	case TAG_MPEG4_IndexedLineSet2D:
-		gf_bifs_dec_qp14_reset(codec);
-		break;
 	case TAG_MPEG4_Coordinate:
 	case TAG_MPEG4_Coordinate2D:
 		gf_bifs_dec_qp14_enter(codec, 0);
@@ -884,7 +886,7 @@ GF_Node *gf_bifs_dec_node(GF_BifsDecoder * codec, GF_BitStream *bs, u32 NDT_Tag)
 	}
 
 	/*if new node is a proto and we're in the top scene, load proto code*/
-	if (proto && new_node && (codec->scenegraph == codec->current_graph)) {
+	if (proto && (codec->scenegraph == codec->current_graph)) {
 		codec->LastError = gf_sg_proto_load_code(new_node);
 	}
 	return new_node;
