@@ -25,7 +25,6 @@
 
 
 #include <gpac/internal/bifs_dev.h>
-#include <gpac/internal/scenegraph_dev.h>
 #include <gpac/internal/bifs_tables.h>
 #include "quant.h" 
 #include "script.h" 
@@ -302,9 +301,9 @@ GF_Route *gf_bifs_enc_is_field_ised(GF_BifsEncoder *codec, GF_Node *node, u32 fi
 	u32 i;
 	if (!codec->encoding_proto) return NULL;
 
-	if (node->sgprivate->interact && node->sgprivate->interact->events) {
+	if (node->sgprivate->interact && node->sgprivate->interact->routes) {
 		i=0;
-		while ((r = (GF_Route*)gf_list_enum(node->sgprivate->interact->events, &i))) {
+		while ((r = (GF_Route*)gf_list_enum(node->sgprivate->interact->routes, &i))) {
 			if (!r->IS_route) continue;
 			if ((r->ToNode == node) && (r->ToField.fieldIndex==fieldIndex)) return r;
 			else if ((r->FromNode == node) && (r->FromField.fieldIndex==fieldIndex)) return r;
@@ -502,7 +501,7 @@ GF_Err gf_bifs_enc_node(GF_BifsEncoder * codec, GF_Node *node, u32 NDT_Tag, GF_B
 {
 	u32 NDTBits, node_type, node_tag, BVersion, node_id;
 	const char *node_name;
-	Bool flag;
+	Bool flag, reset_qp14;
 	GF_Node *new_node;
 	GF_Err e;
 
@@ -521,7 +520,8 @@ GF_Err gf_bifs_enc_node(GF_BifsEncoder * codec, GF_Node *node, u32 NDT_Tag, GF_B
 	if (flag) {
 		gf_bs_write_int(bs, gf_node_get_id(node) - 1, codec->info->config.NodeIDBits);
 		new_node = gf_bifs_enc_find_node(codec, gf_node_get_id(node) );
-		if (!new_node) return codec->LastError = GF_SG_UNKNOWN_NODE;
+		if (!new_node) 
+			return codec->LastError = GF_SG_UNKNOWN_NODE;
 		
 		/*restore QP14 length*/
 		switch (gf_node_get_tag(new_node)) {
@@ -569,11 +569,14 @@ GF_Err gf_bifs_enc_node(GF_BifsEncoder * codec, GF_Node *node, u32 NDT_Tag, GF_B
 	GF_BIFS_WRITE_INT(codec, bs, node_id ? 1 : 0, 1, "DEF", NULL);
 	if (node_id) {
 		GF_BIFS_WRITE_INT(codec, bs, node_id - 1, codec->info->config.NodeIDBits, "NodeID", NULL);
-		if (codec->info->UseName) gf_bifs_enc_name(codec, bs, (char*) node_name );
+		if (codec->UseName) gf_bifs_enc_name(codec, bs, (char*) node_name );
 	}
 
 	/*no updates of time fields for now - NEEDED FOR A LIVE ENCODER*/
 
+	/*if coords were not stored for QP14 before coding this node, reset QP14 it when leaving*/
+	reset_qp14 = !codec->coord_stored;
+
 	/*QP14 case*/
 	switch (node_tag) {
 	case TAG_MPEG4_Coordinate:
@@ -584,13 +587,10 @@ GF_Err gf_bifs_enc_node(GF_BifsEncoder * codec, GF_Node *node, u32 NDT_Tag, GF_B
 	e = EncNodeFields(codec, bs, node);
 	if (e) return e;
 
-	switch (node_tag) {
-	case TAG_MPEG4_IndexedFaceSet:
-	case TAG_MPEG4_IndexedFaceSet2D:
-	case TAG_MPEG4_IndexedLineSet:
-	case TAG_MPEG4_IndexedLineSet2D:
+	if (codec->coord_stored && reset_qp14) 
 		gf_bifs_enc_qp14_reset(codec);
-		break;
+
+	switch (node_tag) {
 	case TAG_MPEG4_Coordinate:
 	case TAG_MPEG4_Coordinate2D:
 		gf_bifs_enc_qp14_enter(codec, 0);
