@@ -24,8 +24,6 @@
 
 
 #include <gpac/internal/bifs_dev.h>
-/*for scene replace tricks*/
-#include <gpac/internal/scenegraph_dev.h>
 #include "quant.h"
 
 GF_Err ParseMFFieldList(GF_BifsDecoder *codec, GF_BitStream *bs, GF_Node *node, GF_FieldInfo *field);
@@ -168,8 +166,11 @@ static GF_Err BM_ParseGlobalQuantizer(GF_BifsDecoder *codec, GF_BitStream *bs, G
 	node = gf_bifs_dec_node(codec, bs, NDT_SFWorldNode);
 
 	/*reset global QP*/
-	if (codec->GlobalQP) gf_node_unregister((GF_Node *) codec->GlobalQP, NULL);
-	codec->GlobalQP = codec->ActiveQP = NULL;
+	if (codec->scenegraph->global_qp) {
+		gf_node_unregister(codec->scenegraph->global_qp, NULL);
+	}
+	codec->ActiveQP = NULL;
+	codec->scenegraph->global_qp = NULL;
 	
 	if (node && (gf_node_get_tag(node) != TAG_MPEG4_QuantizationParameter)) {
 		gf_node_unregister(node, NULL);
@@ -177,10 +178,11 @@ static GF_Err BM_ParseGlobalQuantizer(GF_BifsDecoder *codec, GF_BitStream *bs, G
 	}
 
 	/*register global QP*/
-	codec->GlobalQP = codec->ActiveQP = (M_QuantizationParameter *) node;
-	codec->GlobalQP->isLocal = 0;
+	codec->ActiveQP = (M_QuantizationParameter *) node;
+	codec->ActiveQP->isLocal = 0;
+	codec->scenegraph->global_qp = node;
 	if (node) {
-		/*register TWICE: once for the command, and for the codec*/
+		/*register TWICE: once for the command, and for the scenegraph globalQP*/
 		node->sgprivate->num_instances = 2;
 	}
 	com = gf_sg_command_new(codec->current_graph, GF_SG_GLOBAL_QUANTIZER);
@@ -400,7 +402,7 @@ GF_Err BM_ParseRouteInsert(GF_BifsDecoder *codec, GF_BitStream *bs, GF_List *com
 	/*def'ed route*/
 	if (flag) {
 		RouteID = 1 + gf_bs_read_int(bs, codec->info->config.RouteIDBits);
-		if (codec->info->UseName) gf_bifs_dec_name(bs, name);
+		if (codec->UseName) gf_bifs_dec_name(bs, name);
 	}
 	/*origin*/
 	node_id = 1 + gf_bs_read_int(bs, codec->info->config.NodeIDBits);
@@ -425,7 +427,7 @@ GF_Err BM_ParseRouteInsert(GF_BifsDecoder *codec, GF_BitStream *bs, GF_List *com
 
 	com = gf_sg_command_new(codec->current_graph, GF_SG_ROUTE_INSERT);
 	com->RouteID = RouteID;
-	if (codec->info->UseName) com->def_name = strdup( name);
+	if (codec->UseName) com->def_name = strdup( name);
 	com->fromNodeID = gf_node_get_id(OutNode);
 	com->fromFieldIndex = outField;
 	com->toNodeID = gf_node_get_id(InNode);
@@ -739,6 +741,7 @@ GF_Err BM_SceneReplace(GF_BifsDecoder *codec, GF_BitStream *bs, GF_List *com_lis
 	codec->scenegraph->Routes = gf_list_new();
 	codec->current_graph = codec->scenegraph;
 	codec->LastError = BD_DecSceneReplace(codec, bs, com->new_proto_list);
+	com->use_names = codec->UseName;
 
 	/*restore*/
 	com->node = codec->scenegraph->RootNode;
@@ -810,6 +813,66 @@ void BM_EndOfStream(void *co)
 
 void gf_bs_set_eos_callback(GF_BitStream *bs, void (*EndOfStream)(void *par), void *par);
 
+
+GF_Err gf_bifs_flush_command_list(GF_BifsDecoder *codec)
+{
+	GF_BitStream *bs;
+	GF_Err e;
+	CommandBufferItem *cbi;
+	u32 NbPass = gf_list_count(codec->command_buffers);
+	GF_List *nextPass = gf_list_new();
+	while (NbPass) {
+		while (gf_list_count(codec->command_buffers)) {
+			cbi = (CommandBufferItem *)gf_list_get(codec->command_buffers, 0);
+			gf_list_rem(codec->command_buffers, 0);
+			codec->current_graph = gf_node_get_graph(cbi->node);
+			e = GF_OK;
+			if (cbi->cb->bufferSize) {
+				bs = gf_bs_new((char*)cbi->cb->buffer, cbi->cb->bufferSize, GF_BITSTREAM_READ);
+				gf_bs_set_eos_callback(bs, BM_EndOfStream, codec);
+				e = BM_ParseCommand(codec, bs, cbi->cb->commandList);
+				gf_bs_del(bs);
+			}
+			if (!e) {
+				free(cbi);
+				continue;
+			}
+			/*this may be an error or a dependency pb - reset coimmand list and move to next pass*/
+			while (gf_list_count(cbi->cb->commandList)) {
+				u32 i;
+				GF_CommandField *cf;
+				GF_Command *com = (GF_Command *)gf_list_get(cbi->cb->commandList, 0);
+				gf_list_rem(cbi->cb->commandList, 0);
+				cf = (GF_CommandField *) gf_list_get(com->command_fields, 0);
+				if (cf && cf->fieldType==GF_SG_VRML_SFCOMMANDBUFFER) {
+					for (i=0; i<gf_list_count(codec->command_buffers); i++) {
+						CommandBufferItem *cbi2 = (CommandBufferItem *)gf_list_get(codec->command_buffers, i);
+						if (cbi2->cb == cf->field_ptr) {
+							free(cbi2);
+							gf_list_rem(codec->command_buffers, i);
+							i--;
+						}
+					}
+				}
+				gf_sg_command_del(com);
+			}
+			gf_list_add(nextPass, cbi);
+		}
+		if (!gf_list_count(nextPass)) break;
+		/*prepare next pass*/
+		while (gf_list_count(nextPass)) {
+			cbi = (CommandBufferItem *)gf_list_get(nextPass, 0);
+			gf_list_rem(nextPass, 0);
+			gf_list_add(codec->command_buffers, cbi);
+		}
+		NbPass --;
+		if (NbPass > gf_list_count(codec->command_buffers)) NbPass = gf_list_count(codec->command_buffers);
+		codec->LastError = GF_OK;
+	}
+	gf_list_del(nextPass);
+	return GF_OK;
+}
+
 GF_EXPORT
 GF_Err gf_bifs_decode_command_list(GF_BifsDecoder *codec, u16 ESID, char *data, u32 data_length, GF_List *com_list)
 {
@@ -827,6 +890,8 @@ GF_Err gf_bifs_decode_command_list(GF_BifsDecoder *codec, u16 ESID, char *data,
 	/*setup current scene graph*/
 	codec->current_graph = codec->scenegraph;
 
+	codec->ActiveQP = (M_QuantizationParameter*) codec->scenegraph->global_qp;
+
 	bs = gf_bs_new(data, data_length, GF_BITSTREAM_READ);
 	gf_bs_set_eos_callback(bs, BM_EndOfStream, codec);
 
@@ -835,58 +900,7 @@ GF_Err gf_bifs_decode_command_list(GF_BifsDecoder *codec, u16 ESID, char *data,
 
 	/*decode conditionals / input sensors*/
 	if (!e) {
-		CommandBufferItem *cbi;
-		u32 NbPass = gf_list_count(codec->command_buffers);
-		GF_List *nextPass = gf_list_new();
-		while (NbPass) {
-			while (gf_list_count(codec->command_buffers)) {
-				cbi = (CommandBufferItem *)gf_list_get(codec->command_buffers, 0);
-				gf_list_rem(codec->command_buffers, 0);
-				codec->current_graph = gf_node_get_graph(cbi->node);
-				e = GF_OK;
-				if (cbi->cb->bufferSize) {
-					bs = gf_bs_new((char*)cbi->cb->buffer, cbi->cb->bufferSize, GF_BITSTREAM_READ);
-					gf_bs_set_eos_callback(bs, BM_EndOfStream, codec);
-					e = BM_ParseCommand(codec, bs, cbi->cb->commandList);
-					gf_bs_del(bs);
-				}
-				if (!e) {
-					free(cbi);
-					continue;
-				}
-				/*this may be an error or a dependency pb - reset coimmand list and move to next pass*/
-				while (gf_list_count(cbi->cb->commandList)) {
-					u32 i;
-					GF_CommandField *cf;
-					GF_Command *com = (GF_Command *)gf_list_get(cbi->cb->commandList, 0);
-					gf_list_rem(cbi->cb->commandList, 0);
-					cf = (GF_CommandField *) gf_list_get(com->command_fields, 0);
-					if (cf && cf->fieldType==GF_SG_VRML_SFCOMMANDBUFFER) {
-						for (i=0; i<gf_list_count(codec->command_buffers); i++) {
-							CommandBufferItem *cbi2 = (CommandBufferItem *)gf_list_get(codec->command_buffers, i);
-							if (cbi2->cb == cf->field_ptr) {
-								free(cbi2);
-								gf_list_rem(codec->command_buffers, i);
-								i--;
-							}
-						}
-					}
-					gf_sg_command_del(com);
-				}
-				gf_list_add(nextPass, cbi);
-			}
-			if (!gf_list_count(nextPass)) break;
-			/*prepare next pass*/
-			while (gf_list_count(nextPass)) {
-				cbi = (CommandBufferItem *)gf_list_get(nextPass, 0);
-				gf_list_rem(nextPass, 0);
-				gf_list_add(codec->command_buffers, cbi);
-			}
-			NbPass --;
-			if (NbPass > gf_list_count(codec->command_buffers)) NbPass = gf_list_count(codec->command_buffers);
-			codec->LastError = GF_OK;
-		}
-		gf_list_del(nextPass);
+		gf_bifs_flush_command_list(codec);
 	}
 	/*if err or not reset conditionals*/
 	while (gf_list_count(codec->command_buffers)) {
