@@ -58,7 +58,7 @@ GF_Err gf_bifs_dec_qp_set(GF_BifsDecoder *codec, GF_Node *qp)
 	assert(gf_node_get_tag(qp) == TAG_MPEG4_QuantizationParameter);
 
 	/*if we have an active QP, push it into the stack*/
-	if (codec->ActiveQP && (codec->ActiveQP != codec->GlobalQP) ) 
+	if (codec->ActiveQP && ((GF_Node*)codec->ActiveQP != codec->scenegraph->global_qp) ) 
 		gf_list_insert(codec->QPs, codec->ActiveQP, 0);
 	
 	codec->ActiveQP = (M_QuantizationParameter *)qp;
@@ -67,7 +67,7 @@ GF_Err gf_bifs_dec_qp_set(GF_BifsDecoder *codec, GF_Node *qp)
 
 GF_Err gf_bifs_dec_qp_remove(GF_BifsDecoder *codec, Bool ActivatePrev)
 {
-	if (!codec->force_keep_qp && codec->ActiveQP && (codec->ActiveQP != codec->GlobalQP) ) {
+	if (!codec->force_keep_qp && codec->ActiveQP && ((GF_Node*)codec->ActiveQP != codec->scenegraph->global_qp) ) {
 		gf_node_unregister((GF_Node *) codec->ActiveQP, NULL);
 	}
 	codec->ActiveQP = NULL;
@@ -76,8 +76,8 @@ GF_Err gf_bifs_dec_qp_remove(GF_BifsDecoder *codec, Bool ActivatePrev)
 	if (gf_list_count(codec->QPs)) {
 		codec->ActiveQP = (M_QuantizationParameter*)gf_list_get(codec->QPs, 0);
 		gf_list_rem(codec->QPs, 0);
-	} else if (codec->GlobalQP) {
-		codec->ActiveQP = codec->GlobalQP;
+	} else if (codec->scenegraph->global_qp) {
+		codec->ActiveQP = (M_QuantizationParameter *)codec->scenegraph->global_qp;
 	}
 	return GF_OK;
 }
@@ -351,7 +351,6 @@ GF_Err Q_DecNormal(GF_BifsDecoder *codec, GF_BitStream *bs, u32 NbBits, void *fi
 	return GF_OK;
 }
 
-
 GF_Err gf_bifs_dec_unquant_field(GF_BifsDecoder *codec, GF_BitStream *bs, GF_Node *node, GF_FieldInfo *field)
 {
 	Bool HasQ;
@@ -428,6 +427,7 @@ GF_Err gf_bifs_dec_unquant_field(GF_BifsDecoder *codec, GF_BitStream *bs, GF_Nod
 	default:
 		return GF_BAD_PARAM;
 	}
+
 	if (e) return e;
 	return GF_OK;
 }
