@@ -86,24 +86,38 @@ static void gf_rtp_parse_mpeg4(GF_RTPDepacketizer *rtp, GF_RTPHeader *hdr, char
 		if ((!num_au && rtp->sl_map.auh_first_min_len) || (num_au && rtp->sl_map.auh_min_len)) {
 			/*ISMACryp*/
 			if (rtp->flags & GF_RTP_HAS_ISMACRYP) {
+				u32 nbbits;
 				rtp->sl_hdr.isma_encrypted = 1;
 				if (rtp->flags & GF_RTP_ISMA_SEL_ENC) {
 					rtp->sl_hdr.isma_encrypted = gf_bs_read_int(hdr_bs, 1);
 					gf_bs_read_int(hdr_bs, 7);
+					au_hdr_size -= 8;
 				}
 				/*Note: ISMACryp ALWAYS indicates IV (BSO) and KEYIDX, even when sample is not encrypted. 
 				This is quite a waste when using selective encryption....*/
 				if (!num_au) {
-					rtp->sl_hdr.isma_BSO = gf_bs_read_int(hdr_bs, 8*rtp->sl_map.IV_length);
+					nbbits = 8*rtp->sl_map.IV_length;
+					if (nbbits) {
+						rtp->sl_hdr.isma_BSO = gf_bs_read_int(hdr_bs, nbbits);
+						au_hdr_size -= nbbits;
+					}
 				}
 				/*NOT SUPPORTED YET*/
 				else if (rtp->sl_map.IV_delta_length) {
-					rtp->sl_hdr.isma_BSO += gf_bs_read_int(hdr_bs, 8*rtp->sl_map.IV_delta_length);
+					nbbits = 8*rtp->sl_map.IV_delta_length;
+					if (nbbits) {
+						rtp->sl_hdr.isma_BSO += gf_bs_read_int(hdr_bs, nbbits);
+						au_hdr_size -= nbbits;
+					}
 				}
 				if (rtp->sl_map.KI_length) {
 					/*NOT SUPPORTED YET*/
 					if (!num_au || !(rtp->flags & GF_RTP_ISMA_HAS_KEY_IDX) ) {
-						gf_bs_skip_bytes(hdr_bs, rtp->sl_map.KI_length);
+						nbbits = 8*rtp->sl_map.KI_length;
+						if (nbbits) {
+							gf_bs_read_int(hdr_bs, nbbits);
+							au_hdr_size -= nbbits;
+						}
 					}
 				}
 			}
@@ -158,6 +172,8 @@ static void gf_rtp_parse_mpeg4(GF_RTPDepacketizer *rtp, GF_RTPHeader *hdr, char
 			if (rtp->sl_map.RandomAccessIndication) {
 				rtp->sl_hdr.randomAccessPointFlag = gf_bs_read_int(hdr_bs, 1);
 				au_hdr_size -= 1;
+				if (rtp->sl_hdr.randomAccessPointFlag)
+					rtp->sl_hdr.randomAccessPointFlag=1;
 			}
 			/*stream state - map directly to seqNum*/
 			if (rtp->sl_map.StreamStateIndication) {
@@ -208,6 +224,7 @@ static void gf_rtp_parse_mpeg4(GF_RTPDepacketizer *rtp, GF_RTPHeader *hdr, char
 		if (pay_start >= size) break;
 		num_au ++;
 	}
+	assert(!au_hdr_size);
 
 	if (hdr->Marker)
 		rtp->flags |= GF_RTP_NEW_AU;
@@ -569,7 +586,7 @@ static void gf_rtp_h264_flush(GF_RTPDepacketizer *rtp, GF_RTPHeader *hdr, Bool m
 	/*set F-bit since nal is corrupted*/
 	if (missed_end) data[4] |= 0x80;
 
-	rtp->sl_hdr.accessUnitEndFlag = hdr->Marker;
+	rtp->sl_hdr.accessUnitEndFlag = (rtp->flags & GF_RTP_UNRELIABLE_M) ? 0 : hdr->Marker;
 	rtp->sl_hdr.compositionTimeStampFlag = 1;
 	rtp->sl_hdr.compositionTimeStamp = hdr->TimeStamp;
 	rtp->sl_hdr.decodingTimeStamp = hdr->TimeStamp;
@@ -580,7 +597,7 @@ static void gf_rtp_h264_flush(GF_RTPDepacketizer *rtp, GF_RTPHeader *hdr, Bool m
 	free(data);
 }
 
-static void gf_rtp_parse_h264(GF_RTPDepacketizer *rtp, GF_RTPHeader *hdr, char *payload, u32 size)
+void gf_rtp_parse_h264(GF_RTPDepacketizer *rtp, GF_RTPHeader *hdr, char *payload, u32 size)
 {
 	char nalhdr[4];
 	u32 nal_type;
@@ -590,6 +607,10 @@ static void gf_rtp_parse_h264(GF_RTPDepacketizer *rtp, GF_RTPHeader *hdr, char *
 
 	/*set start*/
 	if (rtp->sl_hdr.compositionTimeStamp != hdr->TimeStamp) {
+		if (rtp->flags & GF_RTP_UNRELIABLE_M) {
+			rtp->sl_hdr.accessUnitEndFlag = 1;
+			rtp->on_sl_packet(rtp->udta, NULL, 0, &rtp->sl_hdr, GF_OK);
+		}
 		rtp->sl_hdr.accessUnitEndFlag = 0;
 		rtp->sl_hdr.accessUnitStartFlag = 1;
 		rtp->sl_hdr.compositionTimeStamp = hdr->TimeStamp;
@@ -597,6 +618,9 @@ static void gf_rtp_parse_h264(GF_RTPDepacketizer *rtp, GF_RTPHeader *hdr, char *
 		rtp->sl_hdr.decodingTimeStamp = hdr->TimeStamp;
 		rtp->sl_hdr.decodingTimeStampFlag = 1;
 		rtp->sl_hdr.randomAccessPointFlag = 0;
+	} else if (rtp->sl_hdr.accessUnitEndFlag) {
+		rtp->flags |= GF_RTP_UNRELIABLE_M;
+		GF_LOG(GF_LOG_ERROR, GF_LOG_RTP, ("[H264 RTP] error in Marker bit - switching to unreliable mode\n"));
 	}
 
 	/*single NALU*/
@@ -605,7 +629,8 @@ static void gf_rtp_parse_h264(GF_RTPDepacketizer *rtp, GF_RTPHeader *hdr, char *
 			rtp->sl_hdr.randomAccessPointFlag = 1;
 			rtp->flags &= ~GF_RTP_AVC_WAIT_RAP;
 		}
-		else if (rtp->flags & GF_RTP_AVC_WAIT_RAP) return;
+		else if (rtp->flags & GF_RTP_AVC_WAIT_RAP)
+			return;
 
 		rtp->sl_hdr.accessUnitEndFlag = 0;
 		/*signal NALU size on 4 bytes*/
@@ -613,7 +638,8 @@ static void gf_rtp_parse_h264(GF_RTPDepacketizer *rtp, GF_RTPHeader *hdr, char *
 		rtp->on_sl_packet(rtp->udta, nalhdr, 4, &rtp->sl_hdr, GF_OK);
 		rtp->sl_hdr.accessUnitStartFlag = 0;
 		rtp->sl_hdr.compositionTimeStampFlag = 0;
-		rtp->sl_hdr.accessUnitEndFlag = hdr->Marker;
+		rtp->sl_hdr.accessUnitEndFlag = (rtp->flags & GF_RTP_UNRELIABLE_M) ? 0 : hdr->Marker;
+
 		/*send NAL payload*/
 		rtp->on_sl_packet(rtp->udta, payload, size, &rtp->sl_hdr, GF_OK);
 	}
@@ -637,7 +663,7 @@ static void gf_rtp_parse_h264(GF_RTPDepacketizer *rtp, GF_RTPHeader *hdr, char *
 				rtp->sl_hdr.accessUnitStartFlag = 0;
 				rtp->sl_hdr.compositionTimeStampFlag = 0;
 			}
-			rtp->sl_hdr.accessUnitEndFlag = (hdr->Marker && (offset+nal_size==size)) ? 1 : 0;
+			rtp->sl_hdr.accessUnitEndFlag = (!(rtp->flags & GF_RTP_UNRELIABLE_M) && hdr->Marker && (offset+nal_size==size)) ? 1 : 0;
 			if (send) rtp->on_sl_packet(rtp->udta, payload+offset, nal_size, &rtp->sl_hdr, GF_OK);
 			offset += nal_size;
 		}
@@ -652,7 +678,8 @@ static void gf_rtp_parse_h264(GF_RTPDepacketizer *rtp, GF_RTPHeader *hdr, char *
 		if ((payload[1] & 0x1F) == 5) {
 			rtp->flags &= ~GF_RTP_AVC_WAIT_RAP;
 			rtp->sl_hdr.randomAccessPointFlag = 1;
-		} else if (rtp->flags & GF_RTP_AVC_WAIT_RAP) return;
+		} else if (rtp->flags & GF_RTP_AVC_WAIT_RAP)
+			return;
 
 		/*setup*/
 		if (!rtp->inter_bs) {
@@ -701,6 +728,126 @@ static void gf_rtp_parse_latm(GF_RTPDepacketizer *rtp, GF_RTPHeader *hdr, char *
 	}
 }
 
+static void gf_rtp_parse_3gpp_dims(GF_RTPDepacketizer *rtp, GF_RTPHeader *hdr, char *payload, u32 size)
+{
+	u32 du_size, offset, dsize, hdr_size;
+	char *data, dhdr[6];
+
+	u32 frag_state = ((payload[0]>>3) & 0x7);
+
+	rtp->sl_hdr.compositionTimeStamp = hdr->TimeStamp;
+	rtp->sl_hdr.compositionTimeStampFlag = 1;
+
+	if (rtp->flags & GF_RTP_NEW_AU) {
+		rtp->flags &= ~GF_RTP_NEW_AU;
+		rtp->sl_hdr.accessUnitStartFlag = 1;
+	}
+	rtp->sl_hdr.accessUnitEndFlag = 0;
+	if (hdr->Marker) rtp->flags |= GF_RTP_NEW_AU;
+
+	rtp->sl_hdr.randomAccessPointFlag = (payload[0] & 0x40);
+	rtp->sl_hdr.AU_sequenceNumber = (payload[0] & 0x7);
+
+	offset = 1;
+	while (offset < size) {
+		switch (frag_state) {
+		case 0:
+		{
+			GF_BitStream *bs;
+			bs = gf_bs_new(payload+offset, 2, GF_BITSTREAM_READ);
+			du_size = 2 + gf_bs_read_u16(bs);
+			gf_bs_del(bs);
+			if (hdr->Marker && offset+du_size>=size) {
+				rtp->sl_hdr.accessUnitEndFlag = 1;
+			}
+			rtp->on_sl_packet(rtp->udta, payload + offset, du_size, &rtp->sl_hdr, GF_OK);
+			rtp->sl_hdr.accessUnitStartFlag = 0;
+			offset += du_size;
+		}
+			break;
+		case 1:
+			if (rtp->inter_bs) gf_bs_del(rtp->inter_bs);
+
+			rtp->inter_bs = gf_bs_new(NULL, 0, GF_BITSTREAM_WRITE);
+			gf_bs_write_data(rtp->inter_bs, payload+offset, size-offset);
+			return;
+		case 2:
+			if (!rtp->inter_bs) return;
+			gf_bs_write_data(rtp->inter_bs, payload+offset, size-offset);
+			return;
+		case 3:
+			if (!rtp->inter_bs) return;
+			gf_bs_write_data(rtp->inter_bs, payload+offset, size-offset);
+			gf_bs_get_content(rtp->inter_bs, &data, &dsize);
+			gf_bs_del(rtp->inter_bs);
+
+			/*send unit header - if dims size is >0xFFFF, use our internal hack for large units*/
+			rtp->inter_bs = gf_bs_new(dhdr, 6, GF_BITSTREAM_WRITE);
+			if (dsize<=0xFFFF) {
+				gf_bs_write_u16(rtp->inter_bs, dsize);
+				hdr_size = 2;
+			} else {
+				gf_bs_write_u16(rtp->inter_bs, 0);
+				gf_bs_write_u32(rtp->inter_bs, dsize);
+				hdr_size = 6;
+			}
+			gf_bs_del(rtp->inter_bs);
+			rtp->inter_bs = NULL;
+
+			rtp->on_sl_packet(rtp->udta, dhdr, hdr_size, &rtp->sl_hdr, GF_OK);
+			rtp->sl_hdr.accessUnitStartFlag = 0;
+
+			rtp->sl_hdr.accessUnitEndFlag = hdr->Marker;
+			rtp->on_sl_packet(rtp->udta, data, dsize, &rtp->sl_hdr, GF_OK);
+			free(data);
+			return;
+		}
+	}
+
+}
+
+
+static void gf_rtp_parse_ac3(GF_RTPDepacketizer *rtp, GF_RTPHeader *hdr, char *payload, u32 size)
+{
+	u8 ft, nb_pck;
+
+	rtp->sl_hdr.compositionTimeStampFlag = 1;
+	rtp->sl_hdr.compositionTimeStamp = hdr->TimeStamp;
+	ft = payload[0];
+	nb_pck = payload[1];
+	payload += 2;
+	size -= 2;
+
+	if (!ft) {
+		GF_AC3Header hdr;
+		rtp->sl_hdr.accessUnitStartFlag = rtp->sl_hdr.accessUnitEndFlag = 1;
+		while (size) {
+			u32 offset;
+			if (!gf_ac3_parser(payload, size, &offset, &hdr, 0)) {
+				return;
+			}
+			if (offset) {
+				if (offset>size) return;
+				payload+=offset;
+				size-=offset;
+			}
+			rtp->on_sl_packet(rtp->udta, payload, hdr.framesize, &rtp->sl_hdr, GF_OK);
+			if (size < hdr.framesize) return;
+			size -= hdr.framesize;
+			payload += hdr.framesize;
+			rtp->sl_hdr.compositionTimeStamp += 1536;
+		}
+		rtp->flags |= GF_RTP_NEW_AU;
+	} else if (ft==3) {
+		rtp->sl_hdr.accessUnitStartFlag = 0;
+		rtp->sl_hdr.accessUnitEndFlag = hdr->Marker ? 1 : 0;
+		rtp->on_sl_packet(rtp->udta, payload, size, &rtp->sl_hdr, GF_OK);
+	} else {
+		rtp->sl_hdr.accessUnitStartFlag = 1;
+		rtp->sl_hdr.accessUnitEndFlag = 0;
+		rtp->on_sl_packet(rtp->udta, payload, size, &rtp->sl_hdr, GF_OK);
+	}
+}
 
 static u32 gf_rtp_get_payload_type(GF_RTPMap *map, GF_SDPMedia *media)
 {
@@ -738,6 +885,8 @@ static u32 gf_rtp_get_payload_type(GF_RTPMap *map, GF_SDPMedia *media)
 	else if (!stricmp(map->payload_name, "AMR-WB")) return GF_RTP_PAYT_AMR_WB;
 	else if (!stricmp(map->payload_name, "3gpp-tt")) return GF_RTP_PAYT_3GPP_TEXT;
 	else if (!stricmp(map->payload_name, "H264")) return GF_RTP_PAYT_H264_AVC;
+	else if (!stricmp(map->payload_name, "richmedia+xml")) return GF_RTP_PAYT_3GPP_DIMS;
+	else if (!stricmp(map->payload_name, "ac3")) return GF_RTP_PAYT_AC3;
 	else return 0;
 }
 
@@ -800,6 +949,11 @@ static GF_Err payt_set_param(GF_RTPDepacketizer *rtp, char *param_name, char *pa
 			rtp->sl_map.StreamType = GF_STREAM_AUDIO;
 			rtp->sl_map.ObjectTypeIndication = 0x40;
 		}
+		/*in case no IOD and no streamType/OTI in the file*/
+		else if (!stricmp(param_val, "avc-video") ) {
+			rtp->sl_map.StreamType = GF_STREAM_VISUAL;
+			rtp->sl_map.ObjectTypeIndication = 0x21;
+		}
 	}
 
 	else if (!stricmp(param_name, "DTSDeltaLength")) rtp->sl_map.DTSDeltaLength = atoi(param_val);
@@ -977,19 +1131,20 @@ static GF_Err gf_rtp_payt_setup(GF_RTPDepacketizer *rtp, GF_RTPMap *map, GF_SDPM
 		{
 			GF_BitStream *bs;
 			rtp->sl_map.StreamType = GF_STREAM_AUDIO;
-			rtp->sl_map.ObjectTypeIndication = GPAC_EXTRA_CODECS_OTI;
+			rtp->sl_map.ObjectTypeIndication = GPAC_OTI_MEDIA_GENERIC;
 			/*create DSI*/
 			bs = gf_bs_new(NULL, 0, GF_BITSTREAM_WRITE);
 			if (rtp->payt == GF_RTP_PAYT_AMR) {
 				gf_bs_write_u32(bs, GF_4CC('s', 'a', 'm', 'r'));
-				gf_bs_write_u16(bs, 8000);
+				gf_bs_write_u32(bs, 8000);
+				gf_bs_write_u16(bs, 1);
 				gf_bs_write_u16(bs, 160);
 			} else {
 				gf_bs_write_u32(bs, GF_4CC('s', 'a', 'w', 'b'));
-				gf_bs_write_u16(bs, 16000);
+				gf_bs_write_u32(bs, 16000);
+				gf_bs_write_u16(bs, 1);
 				gf_bs_write_u16(bs, 320);
 			}
-			gf_bs_write_u8(bs, 1);
 			gf_bs_write_u8(bs, 16);
 			gf_bs_write_u8(bs, 1);
 			gf_bs_get_content(bs, &rtp->sl_map.config, &rtp->sl_map.configSize);
@@ -1012,7 +1167,7 @@ static GF_Err gf_rtp_payt_setup(GF_RTPDepacketizer *rtp, GF_RTPMap *map, GF_SDPM
 			}
 
 			rtp->sl_map.StreamType = GF_STREAM_VISUAL;
-			rtp->sl_map.ObjectTypeIndication = GPAC_EXTRA_CODECS_OTI;
+			rtp->sl_map.ObjectTypeIndication = GPAC_OTI_MEDIA_GENERIC;
 			/*create DSI*/
 			bs = gf_bs_new(NULL, 0, GF_BITSTREAM_WRITE);
 			gf_bs_write_u32(bs, GF_4CC('h', '2', '6', '3'));
@@ -1175,6 +1330,25 @@ static GF_Err gf_rtp_payt_setup(GF_RTPDepacketizer *rtp, GF_RTPMap *map, GF_SDPM
 		/*assign depacketizer*/
 		rtp->depacketize = gf_rtp_parse_h264;
 		break;
+	/*todo - rewrite DIMS config*/
+	case GF_RTP_PAYT_3GPP_DIMS:
+		rtp->sl_map.StreamType = GF_STREAM_SCENE;
+		rtp->sl_map.ObjectTypeIndication = GPAC_OTI_SCENE_DIMS;
+		/*we will signal RAPs*/
+		rtp->sl_map.RandomAccessIndication = 1;
+		/*we map DIMS CTR to AU seq num, hence 3 bits*/
+		rtp->sl_map.StreamStateIndication = 3;
+		rtp->sl_map.IndexLength = 3;
+		/*assign depacketizer*/
+		rtp->depacketize = gf_rtp_parse_3gpp_dims;
+		break;
+	case GF_RTP_PAYT_AC3:
+		rtp->sl_map.StreamType = GF_STREAM_AUDIO;
+		rtp->sl_map.ObjectTypeIndication = 0xA5;
+		rtp->sl_map.RandomAccessIndication = 1;
+		/*assign depacketizer*/
+		rtp->depacketize = gf_rtp_parse_ac3;
+		break;
 	default:
 		return GF_NOT_SUPPORTED;
 	}
@@ -1220,7 +1394,11 @@ void gf_rtp_depacketizer_reset(GF_RTPDepacketizer *rtp, Bool full_reset)
 		if (rtp->inter_bs) gf_bs_del(rtp->inter_bs);
 		rtp->inter_bs = NULL;
 		rtp->flags |= GF_RTP_NEW_AU;
-		if (full_reset) memset(&rtp->sl_hdr, 0, sizeof(GF_SLHeader));
+		if (full_reset) {
+			u32 dur = rtp->sl_hdr.au_duration;
+			memset(&rtp->sl_hdr, 0, sizeof(GF_SLHeader));
+			rtp->sl_hdr.au_duration = dur;
+		}
 	}
 }
 
