@@ -115,7 +115,7 @@ GF_Err gp_rtp_builder_do_amr(GP_RTPPacketizer *builder, char *data, u32 data_siz
 	return GF_OK;
 }
 
-static GFINLINE u8 qes_get_rate_size(u32 idx, u32 *rates, u32 nb_rates)
+static GFINLINE u8 qes_get_rate_size(u32 idx, const u32 *rates, u32 nb_rates)
 {
 	u32 i;
 	for (i=0; i<nb_rates; i++) {
@@ -142,7 +142,7 @@ GF_Err gp_rtp_builder_do_qcelp(GP_RTPPacketizer *builder, char *data, u32 data_s
 	offset = 0;
 	while (data_size>offset) {
 		u8 frame_type = data[offset];
-		u8 size = qes_get_rate_size(frame_type, (u32 *)GF_QCELP_RATE_TO_SIZE, GF_QCELP_RATE_TO_SIZE_NB);
+		u8 size = qes_get_rate_size(frame_type, GF_QCELP_RATE_TO_SIZE, GF_QCELP_RATE_TO_SIZE_NB);
 		/*reserved, not sent)*/
 		if (frame_type>=5) {
 			offset += size;
@@ -278,7 +278,7 @@ GF_Err gp_rtp_builder_do_smv(GP_RTPPacketizer *builder, char *data, u32 data_siz
 	return GF_OK;
 }
 
-GF_Err gp_rtp_builder_do_h264(GP_RTPPacketizer *builder, char *data, u32 data_size, u8 IsAUEnd, u32 FullAUSize)
+GF_Err gp_rtp_builder_do_h263(GP_RTPPacketizer *builder, char *data, u32 data_size, u8 IsAUEnd, u32 FullAUSize)
 {
 	GF_BitStream *bs;
 	char hdr[2];
@@ -536,3 +536,235 @@ GF_Err gp_rtp_builder_do_tx3g(GP_RTPPacketizer *builder, char *data, u32 data_si
 	}
 	return GF_OK;
 }
+
+
+GF_Err gp_rtp_builder_do_dims(GP_RTPPacketizer *builder, char *data, u32 data_size, u8 IsAUEnd, u32 FullAUSize, u32 duration)
+{
+	u32 frag_state;
+	GF_BitStream *bs;
+	u32 offset;
+	Bool is_last_du;
+
+	/*the DIMS hinter doesn't perform inter-sample concatenation*/
+	if (!data) return GF_OK;
+
+	offset = 0;
+	builder->rtp_header.TimeStamp = (u32) builder->sl_header.compositionTimeStamp;
+	bs = gf_bs_new(data, data_size, GF_BITSTREAM_READ);
+	while (offset < data_size) {
+		u32 du_offset = 0;
+		u32 hdr_offset = 0;
+		u32 orig_size, du_size;
+		
+		orig_size = du_size = 2+gf_bs_read_u16(bs);
+		/*if dims size is >0xFFFF, use our internal hack for large units*/
+		if (du_size==2) {
+			orig_size = du_size = 2+gf_bs_read_u32(bs);
+			hdr_offset = 4;
+		}
+		gf_bs_skip_bytes(bs, du_size-2);
+
+		/*prepare M-bit*/
+		is_last_du = (offset+du_size==data_size) ? 1 : 0;
+
+		frag_state = 0;
+		while (du_size) {
+			u32 size_offset = 0;
+			u32 size = du_size;
+
+			/*does not fit, flush required*/
+			if (builder->bytesInPacket && (du_size + 1 + builder->bytesInPacket > builder->Path_MTU)) {
+				builder->OnPacketDone(builder->cbk_obj, &builder->rtp_header);
+				builder->bytesInPacket = 0;
+			}
+
+			/*fragmentation required*/
+			if (du_size + 1 > builder->Path_MTU) {
+				size = builder->Path_MTU - 1;
+				/*first fragment*/
+				if (!frag_state) {
+					/*size field is skipped !!*/
+					size_offset = 2 + hdr_offset;
+					frag_state = 1;
+
+					while (du_size - size_offset <= size) {
+						size--;
+					}
+				}
+				/*any middle fragment*/
+				else frag_state = 2;
+
+				builder->rtp_header.Marker = 0;
+			} 
+			/*last fragment*/
+			else if (frag_state) {
+				size = du_size;
+				frag_state = 3;
+				builder->rtp_header.Marker = is_last_du;
+			} else {
+				size = du_size;
+				builder->rtp_header.Marker = is_last_du;
+			}
+
+			if (frag_state && builder->bytesInPacket) {
+				builder->OnPacketDone(builder->cbk_obj, &builder->rtp_header);
+				builder->bytesInPacket = 0;
+			}
+
+			/*need a new packet*/
+			if (!builder->bytesInPacket) {
+				char dims_rtp_hdr[1];
+				
+				/*the unit is critical, increase counter (coded on 3 bits)*/
+				if (! (data[2+hdr_offset] & GF_DIMS_UNIT_P) && (frag_state <= 1) ) {
+					builder->last_au_sn++;
+					builder->last_au_sn %= 8;
+				}
+				/*set CTR value*/
+				dims_rtp_hdr[0] = builder->last_au_sn;
+				/*if M-bit is set in the dims unit header, replicate it*/
+				if (data[2+hdr_offset] & (1<<1) ) dims_rtp_hdr[0] |= (1<<6);
+				/*add unit fragmentation type*/
+				dims_rtp_hdr[0] |= (frag_state<<3);
+
+				builder->rtp_header.SequenceNumber += 1;
+				builder->OnNewPacket(builder->cbk_obj, &builder->rtp_header);
+				builder->OnData(builder->cbk_obj, (char *) dims_rtp_hdr, 1, 1);
+				builder->bytesInPacket = 1;
+			}
+
+			/*add payload*/
+			if (builder->OnDataReference) 
+				builder->OnDataReference(builder->cbk_obj, size, offset+du_offset+size_offset);
+			else
+				builder->OnData(builder->cbk_obj, data+offset+du_offset+size_offset, size, 0);
+
+			/*if fragmentation, force packet flush even on last packet since aggregation unit do not 
+			use the same packet format*/
+			if (frag_state) {
+				builder->OnPacketDone(builder->cbk_obj, &builder->rtp_header);
+				builder->bytesInPacket = 0;
+			} else {
+				builder->bytesInPacket += size;
+			}
+			du_offset += size+size_offset;
+			assert(du_size>= size+size_offset);
+			du_size -= size+size_offset;
+		}
+		offset += orig_size;
+	}
+	if (builder->bytesInPacket) {
+		builder->OnPacketDone(builder->cbk_obj, &builder->rtp_header);
+		builder->bytesInPacket = 0;
+	}
+	gf_bs_del(bs);
+	return GF_OK;
+}
+
+
+
+static void gf_rtp_ac3_flush(GP_RTPPacketizer *builder)
+{
+	char hdr[2];
+	if (!builder->bytesInPacket) return;
+
+	hdr[0] = builder->ac3_ft;
+	hdr[1] = builder->last_au_sn;
+	builder->OnData(builder->cbk_obj, hdr, 2, 1);
+
+	builder->OnPacketDone(builder->cbk_obj, &builder->rtp_header);
+	builder->bytesInPacket = 0;
+	builder->last_au_sn = 0;
+	builder->ac3_ft = 0;
+}
+
+GF_Err gp_rtp_builder_do_ac3(GP_RTPPacketizer *builder, char *data, u32 data_size, u8 IsAUEnd, u32 FullAUSize)
+{
+	char hdr[2];
+	u32 offset, nb_pck;
+
+	/*flush*/
+	if (!data) {
+		gf_rtp_ac3_flush(builder);
+		return GF_OK;
+	}
+	
+	if (
+		/*AU does not fit*/
+		(builder->bytesInPacket + data_size > builder->Path_MTU)
+	||
+		/*aggregation is not enabled*/
+		!(builder->flags & GP_RTP_PCK_USE_MULTI)
+	||
+		/*max ptime is exceeded*/
+		(builder->max_ptime && ( (u32) builder->sl_header.compositionTimeStamp >= builder->rtp_header.TimeStamp + builder->max_ptime) )
+
+	) {
+		gf_rtp_ac3_flush(builder);
+	}
+	
+	/*fits*/
+	if (builder->bytesInPacket + data_size < builder->Path_MTU) {
+		/*need a new packet*/
+		if (!builder->bytesInPacket) {
+			builder->rtp_header.TimeStamp = (u32) builder->sl_header.compositionTimeStamp;
+			builder->ac3_ft = 0;
+			builder->rtp_header.Marker = 1;
+			builder->rtp_header.SequenceNumber += 1;
+			builder->OnNewPacket(builder->cbk_obj, &builder->rtp_header);
+			/*2 bytes header*/
+			builder->bytesInPacket = 2;
+		}
+
+		/*add payload*/
+		if (builder->OnDataReference) 
+			builder->OnDataReference(builder->cbk_obj, data_size, 0);
+		else
+			builder->OnData(builder->cbk_obj, data, data_size, 0);
+
+		builder->bytesInPacket += data_size;
+		builder->last_au_sn++;
+		return GF_OK;
+	}
+
+	/*need fragmentation*/
+	assert(!builder->bytesInPacket);
+	offset = 0;
+	nb_pck = data_size / (builder->Path_MTU-2);
+	if (data_size % (builder->Path_MTU-2)) nb_pck++;
+	builder->last_au_sn = nb_pck;
+
+	while (offset < data_size) {
+		u32 pck_size = MIN(data_size-offset, builder->Path_MTU-2);
+
+		builder->rtp_header.Marker = 0;
+		builder->rtp_header.TimeStamp = (u32) builder->sl_header.compositionTimeStamp;
+		builder->rtp_header.SequenceNumber += 1;
+
+		if (!offset) {
+			builder->ac3_ft = (pck_size > 5*data_size/8) ? 1 : 2;
+		} else {
+			builder->ac3_ft = 3;
+			if (offset + pck_size == data_size)
+				builder->rtp_header.Marker = 1;
+		}
+		builder->OnNewPacket(builder->cbk_obj, &builder->rtp_header);
+
+		hdr[0] = builder->ac3_ft;
+		hdr[1] = builder->last_au_sn;
+		builder->OnData(builder->cbk_obj, hdr, 2, 1);
+
+		/*add payload*/
+		if (builder->OnDataReference) 
+			builder->OnDataReference(builder->cbk_obj, pck_size, offset);
+		else
+			builder->OnData(builder->cbk_obj, data+offset, pck_size, 0);
+
+		builder->OnPacketDone(builder->cbk_obj, &builder->rtp_header);
+		offset += pck_size;
+		builder->bytesInPacket = 0;
+	}
+
+	return GF_OK;
+}
+
