@@ -49,7 +49,7 @@ GF_Err gf_isom_parse_root_box(GF_Box **outBox, GF_BitStream *bs, u64 *bytesExpec
 }
 
 
-GF_Err gf_isom_parse_box(GF_Box **outBox, GF_BitStream *bs)
+GF_Err gf_isom_parse_box_ex(GF_Box **outBox, GF_BitStream *bs, u32 parent_type)
 {
 	u32 type, hdr_size;
 	u64 size, start, end;
@@ -85,7 +85,10 @@ proceed_box:
 			/*no size means till end of file - EXCEPT FOR some old QuickTime boxes...*/
 			if (type == GF_ISOM_BOX_TYPE_TOTL)
 				size = 12;
-			if (!size) size = gf_bs_available(bs) + 8;
+			if (!size) {
+				GF_LOG(GF_LOG_DEBUG, GF_LOG_CONTAINER, ("[iso file] Warning Read Box type %s size 0 reading till the end of file\n", gf_4cc_to_str(type)));
+				size = gf_bs_available(bs) + 8;
+			}
 		}
 	}
 	/*handle uuid*/
@@ -107,9 +110,15 @@ proceed_box:
 		return GF_ISOM_INVALID_FILE;
 	}
 
-	//OK, create the box based on the type
-	newBox = gf_isom_box_new(type);
-	if (!newBox) return GF_OUT_OF_MEM;
+	if (parent_type && (parent_type==GF_ISOM_BOX_TYPE_TREF)) {
+		newBox = gf_isom_box_new(GF_ISOM_BOX_TYPE_REFT);
+		if (!newBox) return GF_OUT_OF_MEM;
+		((GF_TrackReferenceTypeBox*)newBox)->reference_type = type;
+	} else {
+		//OK, create the box based on the type
+		newBox = gf_isom_box_new(type);
+		if (!newBox) return GF_OUT_OF_MEM;
+	}
 
 	//OK, init and read this box
 	if (type==GF_ISOM_BOX_TYPE_UUID) memcpy(((GF_UUIDBox *)newBox)->uuid, uuid, 16);
@@ -154,7 +163,10 @@ proceed_box:
 	return e;
 }
 
-
+GF_Err gf_isom_parse_box(GF_Box **outBox, GF_BitStream *bs)
+{
+	return gf_isom_parse_box_ex(outBox, bs, 0);
+}
 
 GF_Err gf_isom_full_box_read(GF_Box *ptr, GF_BitStream *bs)
 {
@@ -188,12 +200,12 @@ void gf_isom_box_array_del(GF_List *boxList)
 	gf_list_del(boxList);
 }
 
-GF_Err gf_isom_read_box_list(GF_Box *parent, GF_BitStream *bs, GF_Err (*add_box)(GF_Box *par, GF_Box *b))
+GF_Err gf_isom_read_box_list_ex(GF_Box *parent, GF_BitStream *bs, GF_Err (*add_box)(GF_Box *par, GF_Box *b), u32 parent_type)
 {
 	GF_Err e;
 	GF_Box *a;
 	while (parent->size) {
-		e = gf_isom_parse_box(&a, bs);
+		e = gf_isom_parse_box_ex(&a, bs, parent_type);
 		if (e) {
 			if (a) gf_isom_box_del(a);
 			return e;
@@ -213,6 +225,10 @@ GF_Err gf_isom_read_box_list(GF_Box *parent, GF_BitStream *bs, GF_Err (*add_box)
 	return GF_OK;
 }
 
+GF_Err gf_isom_read_box_list(GF_Box *parent, GF_BitStream *bs, GF_Err (*add_box)(GF_Box *par, GF_Box *b))
+{
+	return gf_isom_read_box_list_ex(parent, bs, add_box, 0);
+}
 
 //from here, for write/edit versions
 #ifndef GPAC_READ_ONLY
@@ -310,15 +326,8 @@ GF_Box *gf_isom_box_new(u32 boxType)
 {
 	GF_Box *a;
 	switch (boxType) {
-	case GF_ISOM_BOX_TYPE_HINT:
-	case GF_ISOM_BOX_TYPE_DPND:
-	case GF_ISOM_BOX_TYPE_MPOD:
-	case GF_ISOM_BOX_TYPE_SYNC:
-	case GF_ISOM_BOX_TYPE_IPIR:
-	case GF_ISOM_BOX_TYPE_CHAP:
-		a = reftype_New();
-		if (a) a->type = boxType;
-		return a;
+	case GF_ISOM_BOX_TYPE_REFT:
+		return reftype_New();
 	case GF_ISOM_BOX_TYPE_FREE: return free_New();
 	case GF_ISOM_BOX_TYPE_SKIP:
 		a = free_New();
@@ -388,7 +397,6 @@ GF_Box *gf_isom_box_new(u32 boxType)
 		a = ghnt_New();
 		if (a) a->type = boxType;
 		return a;
-
 	case GF_ISOM_BOX_TYPE_RTPO: return rtpo_New();
 	case GF_ISOM_BOX_TYPE_HNTI: return hnti_New();
 	case GF_ISOM_BOX_TYPE_SDP: return sdp_New();
@@ -414,7 +422,7 @@ GF_Box *gf_isom_box_new(u32 boxType)
 	case GF_ISOM_BOX_TYPE_PAYT: return payt_New();
 	case GF_ISOM_BOX_TYPE_NAME: return name_New();
 	
-#ifndef	GF_ISOM_NO_FRAGMENTS
+#ifndef	GPAC_ISOM_NO_FRAGMENTS
 	case GF_ISOM_BOX_TYPE_MVEX: return mvex_New();
 	case GF_ISOM_BOX_TYPE_MEHD: return mehd_New();
 	case GF_ISOM_BOX_TYPE_TREX: return trex_New();
@@ -473,8 +481,6 @@ GF_Box *gf_isom_box_new(u32 boxType)
 	case GF_ISOM_BOX_TYPE_IPRO: return ipro_New();
 	case GF_ISOM_BOX_TYPE_INFE: return infe_New();
 	case GF_ISOM_BOX_TYPE_IINF: return iinf_New();
-	case GF_ISOM_BOX_TYPE_IMIF: return imif_New();
-	case GF_ISOM_BOX_TYPE_IPMC: return ipmc_New();
 	case GF_ISOM_BOX_TYPE_SINF: return sinf_New();
 	case GF_ISOM_BOX_TYPE_FRMA: return frma_New();
 	case GF_ISOM_BOX_TYPE_SCHM: return schm_New();
@@ -523,6 +529,20 @@ GF_Box *gf_isom_box_new(u32 boxType)
 		a->type = GF_ISOM_BOX_TYPE_ODAF;
 		return a;
 
+	case GF_ISOM_BOX_TYPE_PASP: return pasp_New();
+	case GF_ISOM_BOX_TYPE_TSEL: return tsel_New();
+
+	case GF_ISOM_BOX_TYPE_DIMS: return dims_New();
+	case GF_ISOM_BOX_TYPE_DIMC: return dimC_New();
+	case GF_ISOM_BOX_TYPE_DIST: return diST_New();
+
+	case GF_ISOM_BOX_TYPE_METX:
+	case GF_ISOM_BOX_TYPE_METT:
+		return metx_New(boxType);
+
+	case GF_ISOM_BOX_TYPE_AC3: return ac3_New();
+	case GF_ISOM_BOX_TYPE_DAC3: return dac3_New();
+	
 	default:
 		a = defa_New();
 		if (a) a->type = boxType;
@@ -535,12 +555,7 @@ void gf_isom_box_del(GF_Box *a)
 {
 	if (!a) return;
 	switch (a->type) {
-	case GF_ISOM_BOX_TYPE_HINT:
-	case GF_ISOM_BOX_TYPE_DPND:
-	case GF_ISOM_BOX_TYPE_MPOD:
-	case GF_ISOM_BOX_TYPE_SYNC:
-	case GF_ISOM_BOX_TYPE_IPIR:
-	case GF_ISOM_BOX_TYPE_CHAP:
+	case GF_ISOM_BOX_TYPE_REFT:
 		reftype_del(a);
 		return;
 	case GF_ISOM_BOX_TYPE_FREE:
@@ -633,7 +648,7 @@ void gf_isom_box_del(GF_Box *a)
 	case GF_ISOM_BOX_TYPE_PAYT: payt_del(a); return;
 	case GF_ISOM_BOX_TYPE_NAME: name_del(a); return;
 
-#ifndef	GF_ISOM_NO_FRAGMENTS
+#ifndef	GPAC_ISOM_NO_FRAGMENTS
 	case GF_ISOM_BOX_TYPE_MVEX: mvex_del(a); return;
 	case GF_ISOM_BOX_TYPE_MEHD: mehd_del(a); return;
 	case GF_ISOM_BOX_TYPE_TREX: trex_del(a); return;
@@ -691,8 +706,6 @@ void gf_isom_box_del(GF_Box *a)
 	case GF_ISOM_BOX_TYPE_IPRO: ipro_del(a); return;
 	case GF_ISOM_BOX_TYPE_INFE: infe_del(a); return;
 	case GF_ISOM_BOX_TYPE_IINF: iinf_del(a); return;
-	case GF_ISOM_BOX_TYPE_IMIF: imif_del(a); return;
-	case GF_ISOM_BOX_TYPE_IPMC: ipmc_del(a); return;
 	case GF_ISOM_BOX_TYPE_SINF:	sinf_del(a); return;
 	case GF_ISOM_BOX_TYPE_FRMA:	frma_del(a); return;
 	case GF_ISOM_BOX_TYPE_SCHM:		schm_del(a); return;
@@ -744,6 +757,21 @@ void gf_isom_box_del(GF_Box *a)
 	case GF_ISOM_BOX_TYPE_ODKM: odkm_del(a); return;
 	case GF_ISOM_BOX_TYPE_ODAF: iSFM_del(a); return;
 
+	case GF_ISOM_BOX_TYPE_PASP: pasp_del(a); return;
+	case GF_ISOM_BOX_TYPE_TSEL: tsel_del(a); return;
+
+	case GF_ISOM_BOX_TYPE_METX:
+	case GF_ISOM_BOX_TYPE_METT:
+		metx_del(a);
+		return;
+
+	case GF_ISOM_BOX_TYPE_DIMS: dims_del(a); return;
+	case GF_ISOM_BOX_TYPE_DIMC: dimC_del(a); return;
+	case GF_ISOM_BOX_TYPE_DIST: diST_del(a); return;
+
+	case GF_ISOM_BOX_TYPE_AC3: ac3_del(a); return;
+	case GF_ISOM_BOX_TYPE_DAC3: dac3_del(a); return;
+
 	default:
 		defa_del(a);
 		return;
@@ -756,12 +784,7 @@ void gf_isom_box_del(GF_Box *a)
 GF_Err gf_isom_box_read(GF_Box *a, GF_BitStream *bs)
 {
 	switch (a->type) {
-	case GF_ISOM_BOX_TYPE_HINT:
-	case GF_ISOM_BOX_TYPE_DPND:
-	case GF_ISOM_BOX_TYPE_MPOD:
-	case GF_ISOM_BOX_TYPE_SYNC:
-	case GF_ISOM_BOX_TYPE_IPIR:
-	case GF_ISOM_BOX_TYPE_CHAP:
+	case GF_ISOM_BOX_TYPE_REFT:
 		return reftype_Read(a, bs);
 	case GF_ISOM_BOX_TYPE_FREE:
 	case GF_ISOM_BOX_TYPE_SKIP:
@@ -844,7 +867,7 @@ GF_Err gf_isom_box_read(GF_Box *a, GF_BitStream *bs)
 	case GF_ISOM_BOX_TYPE_PAYT: return payt_Read(a, bs);
 	case GF_ISOM_BOX_TYPE_NAME: return name_Read(a, bs);
 	
-#ifndef	GF_ISOM_NO_FRAGMENTS
+#ifndef	GPAC_ISOM_NO_FRAGMENTS
 	case GF_ISOM_BOX_TYPE_MVEX: return mvex_Read(a, bs);
 	case GF_ISOM_BOX_TYPE_MEHD: return mehd_Read(a, bs);
 	case GF_ISOM_BOX_TYPE_TREX: return trex_Read(a, bs);
@@ -902,8 +925,6 @@ GF_Err gf_isom_box_read(GF_Box *a, GF_BitStream *bs)
 	case GF_ISOM_BOX_TYPE_IPRO: return ipro_Read(a, bs);
 	case GF_ISOM_BOX_TYPE_INFE: return infe_Read(a, bs);
 	case GF_ISOM_BOX_TYPE_IINF: return iinf_Read(a, bs);
-	case GF_ISOM_BOX_TYPE_IMIF: return imif_Read(a, bs);
-	case GF_ISOM_BOX_TYPE_IPMC: return ipmc_Read(a, bs);
 	case GF_ISOM_BOX_TYPE_SINF: return sinf_Read(a, bs);
 	case GF_ISOM_BOX_TYPE_FRMA: return frma_Read(a, bs);
 	case GF_ISOM_BOX_TYPE_SCHM: return schm_Read(a, bs);
@@ -949,6 +970,20 @@ GF_Err gf_isom_box_read(GF_Box *a, GF_BitStream *bs)
 	case GF_ISOM_BOX_TYPE_ODKM: return odkm_Read(a, bs);
 	case GF_ISOM_BOX_TYPE_ODAF: return iSFM_Read(a, bs);
 
+	case GF_ISOM_BOX_TYPE_PASP: return pasp_Read(a, bs);
+	case GF_ISOM_BOX_TYPE_TSEL: return tsel_Read(a, bs);
+
+	case GF_ISOM_BOX_TYPE_METX:
+	case GF_ISOM_BOX_TYPE_METT:
+		return metx_Read(a, bs);
+
+	case GF_ISOM_BOX_TYPE_DIMS: return dims_Read(a, bs);
+	case GF_ISOM_BOX_TYPE_DIMC: return dimC_Read(a, bs);
+	case GF_ISOM_BOX_TYPE_DIST: return diST_Read(a, bs);
+
+	case GF_ISOM_BOX_TYPE_AC3: return ac3_Read(a, bs); 
+	case GF_ISOM_BOX_TYPE_DAC3: return dac3_Read(a, bs); 
+
 	default:
 		return defa_Read(a, bs);
 	}
@@ -960,12 +995,7 @@ GF_Err gf_isom_box_read(GF_Box *a, GF_BitStream *bs)
 GF_Err gf_isom_box_write(GF_Box *a, GF_BitStream *bs)
 {
 	switch (a->type) {
-	case GF_ISOM_BOX_TYPE_HINT:
-	case GF_ISOM_BOX_TYPE_DPND:
-	case GF_ISOM_BOX_TYPE_MPOD:
-	case GF_ISOM_BOX_TYPE_SYNC:
-	case GF_ISOM_BOX_TYPE_IPIR:
-	case GF_ISOM_BOX_TYPE_CHAP:
+	case GF_ISOM_BOX_TYPE_REFT:
 		return reftype_Write(a, bs);
 	case GF_ISOM_BOX_TYPE_FREE:
 	case GF_ISOM_BOX_TYPE_SKIP:
@@ -1051,7 +1081,7 @@ GF_Err gf_isom_box_write(GF_Box *a, GF_BitStream *bs)
 	case GF_ISOM_BOX_TYPE_PAYT: return payt_Write(a, bs);
 	case GF_ISOM_BOX_TYPE_NAME: return name_Write(a, bs);
 	
-#ifndef	GF_ISOM_NO_FRAGMENTS
+#ifndef	GPAC_ISOM_NO_FRAGMENTS
 	case GF_ISOM_BOX_TYPE_MVEX: return mvex_Write(a, bs);
 	case GF_ISOM_BOX_TYPE_MEHD: return mehd_Write(a, bs);
 	case GF_ISOM_BOX_TYPE_TREX: return trex_Write(a, bs);
@@ -1109,8 +1139,6 @@ GF_Err gf_isom_box_write(GF_Box *a, GF_BitStream *bs)
 	case GF_ISOM_BOX_TYPE_IPRO: return ipro_Write(a, bs);
 	case GF_ISOM_BOX_TYPE_INFE: return infe_Write(a, bs);
 	case GF_ISOM_BOX_TYPE_IINF: return iinf_Write(a, bs);
-	case GF_ISOM_BOX_TYPE_IMIF: return imif_Write(a, bs);
-	case GF_ISOM_BOX_TYPE_IPMC: return ipmc_Write(a, bs);
 	case GF_ISOM_BOX_TYPE_SINF: return sinf_Write(a, bs);
 	case GF_ISOM_BOX_TYPE_FRMA: return frma_Write(a, bs);
 	case GF_ISOM_BOX_TYPE_SCHM: return schm_Write(a, bs);
@@ -1156,6 +1184,20 @@ GF_Err gf_isom_box_write(GF_Box *a, GF_BitStream *bs)
 	case GF_ISOM_BOX_TYPE_ODKM: return odkm_Write(a, bs);
 	case GF_ISOM_BOX_TYPE_ODAF: return iSFM_Write(a, bs);
 
+	case GF_ISOM_BOX_TYPE_PASP: return pasp_Write(a, bs);
+	case GF_ISOM_BOX_TYPE_TSEL: return tsel_Write(a, bs);
+
+	case GF_ISOM_BOX_TYPE_METX:
+	case GF_ISOM_BOX_TYPE_METT:
+		return metx_Write(a, bs);
+
+	case GF_ISOM_BOX_TYPE_DIMS: return dims_Write(a, bs);
+	case GF_ISOM_BOX_TYPE_DIMC: return dimC_Write(a, bs);
+	case GF_ISOM_BOX_TYPE_DIST: return diST_Write(a, bs);
+
+	case GF_ISOM_BOX_TYPE_AC3: return ac3_Write(a, bs); 
+	case GF_ISOM_BOX_TYPE_DAC3: return dac3_Write(a, bs); 
+
 	default:
 		return defa_Write(a, bs);
 	}
@@ -1165,12 +1207,7 @@ GF_Err gf_isom_box_write(GF_Box *a, GF_BitStream *bs)
 GF_Err gf_isom_box_size(GF_Box *a)
 {
 	switch (a->type) {
-	case GF_ISOM_BOX_TYPE_HINT:
-	case GF_ISOM_BOX_TYPE_DPND:
-	case GF_ISOM_BOX_TYPE_MPOD:
-	case GF_ISOM_BOX_TYPE_SYNC:
-	case GF_ISOM_BOX_TYPE_IPIR:
-	case GF_ISOM_BOX_TYPE_CHAP:
+	case GF_ISOM_BOX_TYPE_REFT:
 		return reftype_Size(a);
 	case GF_ISOM_BOX_TYPE_FREE:
 	case GF_ISOM_BOX_TYPE_SKIP:
@@ -1255,7 +1292,7 @@ GF_Err gf_isom_box_size(GF_Box *a)
 	case GF_ISOM_BOX_TYPE_PAYT: return payt_Size(a);
 	case GF_ISOM_BOX_TYPE_NAME: return name_Size(a);
 		
-#ifndef	GF_ISOM_NO_FRAGMENTS
+#ifndef	GPAC_ISOM_NO_FRAGMENTS
 	case GF_ISOM_BOX_TYPE_MVEX: return mvex_Size(a);
 	case GF_ISOM_BOX_TYPE_MEHD: return mehd_Size(a);
 	case GF_ISOM_BOX_TYPE_TREX: return trex_Size(a);
@@ -1312,8 +1349,6 @@ GF_Err gf_isom_box_size(GF_Box *a)
 	case GF_ISOM_BOX_TYPE_IPRO: return ipro_Size(a);
 	case GF_ISOM_BOX_TYPE_INFE: return infe_Size(a);
 	case GF_ISOM_BOX_TYPE_IINF: return iinf_Size(a);
-	case GF_ISOM_BOX_TYPE_IMIF: return imif_Size(a);
-	case GF_ISOM_BOX_TYPE_IPMC: return ipmc_Size(a);
 	case GF_ISOM_BOX_TYPE_SINF: return sinf_Size(a);
 	case GF_ISOM_BOX_TYPE_FRMA: return frma_Size(a);
 	case GF_ISOM_BOX_TYPE_SCHM: return schm_Size(a);
@@ -1359,6 +1394,20 @@ GF_Err gf_isom_box_size(GF_Box *a)
 	case GF_ISOM_BOX_TYPE_ODKM: return odkm_Size(a);
 	case GF_ISOM_BOX_TYPE_ODAF: return iSFM_Size(a);
 
+	case GF_ISOM_BOX_TYPE_PASP: return pasp_Size(a);
+	case GF_ISOM_BOX_TYPE_TSEL: return tsel_Size(a);
+
+	case GF_ISOM_BOX_TYPE_METX:
+	case GF_ISOM_BOX_TYPE_METT:
+		return metx_Size(a);
+
+	case GF_ISOM_BOX_TYPE_DIMS: return dims_Size(a);
+	case GF_ISOM_BOX_TYPE_DIMC: return dimC_Size(a);
+	case GF_ISOM_BOX_TYPE_DIST: return diST_Size(a);
+
+	case GF_ISOM_BOX_TYPE_AC3: return ac3_Size(a); 
+	case GF_ISOM_BOX_TYPE_DAC3: return dac3_Size(a); 
+
 	default: return defa_Size(a);
 	}
 }
