@@ -130,7 +130,8 @@ GF_Err gf_isom_setup_hint_track(GF_ISOFile *movie, u32 trackNumber, u32 HintType
 	if (dpnd) return GF_BAD_PARAM;
 
 	//create our dep
-	dpnd = (GF_TrackReferenceTypeBox *) gf_isom_box_new(GF_ISOM_BOX_TYPE_HINT);
+	dpnd = (GF_TrackReferenceTypeBox *) gf_isom_box_new(GF_ISOM_BOX_TYPE_REFT);
+	dpnd->reference_type = GF_ISOM_BOX_TYPE_HINT;
 	e = tref_AddBox((GF_Box*)tref, (GF_Box *) dpnd);
 	if (e) return e;
 
@@ -495,7 +496,7 @@ GF_Err gf_isom_hint_sample_data(GF_ISOFile *the_file, u32 trackNumber, u32 Sourc
 			entry->hint_sample->AdditionalData = (char*)realloc(entry->hint_sample->AdditionalData, sizeof(char) * (entry->hint_sample->dataLength + DataLength));
 			if (AtBegin) {
 				if (entry->hint_sample->dataLength)
-					memmove(entry->hint_sample->AdditionalData + entry->hint_sample->dataLength, entry->hint_sample->AdditionalData, entry->hint_sample->dataLength);
+					memmove(entry->hint_sample->AdditionalData + DataLength, entry->hint_sample->AdditionalData, entry->hint_sample->dataLength);
 				memcpy(entry->hint_sample->AdditionalData, extra_data, DataLength);
 				/*offset existing DTE*/
 				gf_isom_hint_pck_offset(entry->hint_sample->HintType, pck, DataLength, SampleNumber);
