@@ -205,7 +205,7 @@ GF_Err gf_isom_close(GF_ISOFile *movie)
 	//write our movie to the file
 	if (movie->openMode != GF_ISOM_OPEN_READ) {
 		gf_isom_get_duration(movie);
-#ifndef	GF_ISOM_NO_FRAGMENTS
+#ifndef	GPAC_ISOM_NO_FRAGMENTS
 		//movie fragment mode, just store the fragment
 		if ( (movie->openMode == GF_ISOM_OPEN_WRITE) && (movie->FragmentsFlags & GF_ISOM_FRAG_WRITE_READY) ) {
 			e = StoreFragment(movie);
@@ -971,16 +971,14 @@ Bool gf_isom_has_time_offset(GF_ISOFile *the_file, u32 trackNumber)
 {
 	u32 i;
 	GF_CompositionOffsetBox *ctts;
-	GF_DttsEntry *pe;
 	GF_TrackBox *trak;
 	trak = gf_isom_get_track_from_file(the_file, trackNumber);
 	if (!trak || !trak->Media->information->sampleTable->CompositionOffset) return 0;
 
 	//return true at the first offset found
 	ctts = trak->Media->information->sampleTable->CompositionOffset;
-	i=0;
-	while ((pe = (GF_DttsEntry*)gf_list_enum(ctts->entryList, &i))) {
-		if (pe->decodingOffset && pe->sampleCount) return 1;
+	for (i=0; i<ctts->nb_entries; i++) {
+		if (ctts->entries[i].decodingOffset && ctts->entries[i].sampleCount) return 1;
 	}
 	return 0;
 }
@@ -995,6 +993,7 @@ Bool gf_isom_has_sync_shadows(GF_ISOFile *the_file, u32 trackNumber)
 	return 0;
 }
 
+GF_EXPORT
 Bool gf_isom_has_sample_dependency(GF_ISOFile *the_file, u32 trackNumber)
 {
 	GF_TrackBox *trak = gf_isom_get_track_from_file(the_file, trackNumber);
@@ -1093,6 +1092,24 @@ Bool gf_isom_is_self_contained(GF_ISOFile *the_file, u32 trackNumber, u32 sample
 	return Media_IsSelfContained(trak->Media, sampleDescriptionIndex);
 }
 
+/*retrieves given sample DTS*/
+u32 gf_isom_get_sample_from_dts(GF_ISOFile *the_file, u32 trackNumber, u64 dts)
+{
+	GF_Err e;
+	u32 sampleNumber, prevSampleNumber;
+	GF_TrackBox *trak;
+	GF_SampleTableBox *stbl;
+
+	trak = gf_isom_get_track_from_file(the_file, trackNumber);
+	if (!trak) return 0;
+
+	stbl = trak->Media->information->sampleTable;
+	
+	e = findEntryForTime(stbl, dts, 1, &sampleNumber, &prevSampleNumber);
+	if (e) return 0;
+	return sampleNumber;
+}
+
 
 //return a sample given a desired display time IN MEDIA TIME SCALE
 //and set the StreamDescIndex of this sample
@@ -1416,7 +1433,7 @@ u8 gf_isom_has_sync_points(GF_ISOFile *the_file, u32 trackNumber)
 	trak = gf_isom_get_track_from_file(the_file, trackNumber);
 	if (!trak) return 0;
 	if (trak->Media->information->sampleTable->SyncSample) {
-		if (!trak->Media->information->sampleTable->SyncSample->entryCount) return 2;	
+		if (!trak->Media->information->sampleTable->SyncSample->nb_entries) return 2;	
 		return 1;
 	}
 	return 0;
@@ -1430,7 +1447,7 @@ u32 gf_isom_get_sync_point_count(GF_ISOFile *the_file, u32 trackNumber)
 	trak = gf_isom_get_track_from_file(the_file, trackNumber);
 	if (!trak) return 0;
 	if (trak->Media->information->sampleTable->SyncSample) {
-		return trak->Media->information->sampleTable->SyncSample->entryCount;	
+		return trak->Media->information->sampleTable->SyncSample->nb_entries;	
 	}
 	return 0;
 }
@@ -1585,8 +1602,6 @@ u32 gf_isom_get_max_chunk_duration(GF_ISOFile *movie, u32 trackNumber)
 	u32 i, sample_per_chunk, sample_dur;
 	GF_SampleToChunkBox *stsc;
 	GF_TimeToSampleBox *stts;
-	GF_StscEntry *sc;
-	GF_SttsEntry *ts;
 	if (!movie || !trackNumber || !movie->moov) return 0;
 	trak = gf_isom_get_track_from_file(movie, trackNumber);
 	if (!trak) return 0;
@@ -1595,14 +1610,12 @@ u32 gf_isom_get_max_chunk_duration(GF_ISOFile *movie, u32 trackNumber)
 	stts = trak->Media->information->sampleTable->TimeToSample;
 
 	sample_per_chunk = 0;
-	i=0;
-	while ((sc = (GF_StscEntry *) gf_list_enum(stsc->entryList, &i))) {
-		if (sc->samplesPerChunk > sample_per_chunk) sample_per_chunk = sc->samplesPerChunk;
+	for (i=0; i<stsc->nb_entries; i++) {
+		if (stsc->entries[i].samplesPerChunk > sample_per_chunk) sample_per_chunk = stsc->entries[i].samplesPerChunk;
 	}
 	sample_dur = 0;
-	i=0;
-	while ((ts = (GF_SttsEntry *)gf_list_enum(stts->entryList, &i))) {
-		if (ts->sampleDelta > sample_dur) sample_dur = ts->sampleDelta;
+	for (i=0; i<stts->nb_entries; i++) {
+		if (stts->entries[i].sampleDelta > sample_dur) sample_dur = stts->entries[i].sampleDelta;
 	}
 	
 	//rescale to ms
@@ -1639,7 +1652,6 @@ GF_Err gf_isom_get_fragment_defaults(GF_ISOFile *the_file, u32 trackNumber,
 							 u32 *defaultRandomAccess, u8 *defaultPadding, u16 *defaultDegradationPriority)
 {
 	GF_TrackBox *trak;
-	GF_SttsEntry *ts_ent;
 	GF_StscEntry *sc_ent;
 	u32 i, j, maxValue, value;
 	GF_SampleTableBox *stbl;
@@ -1650,11 +1662,10 @@ GF_Err gf_isom_get_fragment_defaults(GF_ISOFile *the_file, u32 trackNumber,
 	//duration
 	if (defaultDuration) {
 		maxValue = value = 0;
-		i=0;
-		while ((ts_ent = (GF_SttsEntry *)gf_list_enum(stbl->TimeToSample->entryList, &i))) {
-			if (ts_ent->sampleCount>maxValue) {
-				value = ts_ent->sampleDelta;
-				maxValue = ts_ent->sampleCount;
+		for (i=0; i<stbl->TimeToSample->nb_entries; i++) {
+			if (stbl->TimeToSample->entries[i].sampleCount>maxValue) {
+				value = stbl->TimeToSample->entries[i].sampleDelta;
+				maxValue = stbl->TimeToSample->entries[i].sampleCount;
 			}
 		}
 		*defaultDuration = value;
@@ -1665,9 +1676,10 @@ GF_Err gf_isom_get_fragment_defaults(GF_ISOFile *the_file, u32 trackNumber,
 	}
 	//descIndex
 	if (defaultDescriptionIndex) {
+		GF_SampleToChunkBox *stsc= stbl->SampleToChunk;
 		maxValue = value = 0;
-		i=0;
-		while ((sc_ent = (GF_StscEntry *)gf_list_enum(stbl->SampleToChunk->entryList, &i))) {
+		for (i=0; i<stsc->nb_entries; i++) {
+			sc_ent = &stsc->entries[i];
 			if ((sc_ent->nextChunk - sc_ent->firstChunk) * sc_ent->samplesPerChunk > maxValue) {
 				value = sc_ent->sampleDescriptionIndex;
 				maxValue = (sc_ent->nextChunk - sc_ent->firstChunk) * sc_ent->samplesPerChunk;
@@ -1680,7 +1692,7 @@ GF_Err gf_isom_get_fragment_defaults(GF_ISOFile *the_file, u32 trackNumber,
 		//no sync table is ALL RAP
 		*defaultRandomAccess = stbl->SyncSample ? 0 : 1;
 		if (stbl->SyncSample 
-			&& (stbl->SyncSample->entryCount >= stbl->SampleSize->sampleCount/2)) {
+			&& (stbl->SyncSample->nb_entries >= stbl->SampleSize->sampleCount/2)) {
 			*defaultRandomAccess = 1;
 		}
 	}
@@ -1708,9 +1720,9 @@ GF_Err gf_isom_get_fragment_defaults(GF_ISOFile *the_file, u32 trackNumber,
 		*defaultDegradationPriority = 0;
 		if (stbl->DegradationPriority) {
 			maxValue = 0;
-			for (i=0; i<stbl->DegradationPriority->entryCount; i++) {
+			for (i=0; i<stbl->DegradationPriority->nb_entries; i++) {
 				value = 0;
-				for (j=0; j<stbl->DegradationPriority->entryCount; j++) {
+				for (j=0; j<stbl->DegradationPriority->nb_entries; j++) {
 					if (stbl->DegradationPriority->priorities[i]==stbl->DegradationPriority->priorities[j]) {
 						value ++;
 					}
@@ -1729,7 +1741,7 @@ GF_Err gf_isom_get_fragment_defaults(GF_ISOFile *the_file, u32 trackNumber,
 GF_EXPORT
 GF_Err gf_isom_refresh_fragmented(GF_ISOFile *movie, u64 *MissingBytes)
 {
-#ifdef	GF_ISOM_NO_FRAGMENTS
+#ifdef	GPAC_ISOM_NO_FRAGMENTS
 	return GF_NOT_SUPPORTED;
 #else
 	if (!movie || !movie->moov || !movie->moov->mvex) return GF_BAD_PARAM;
@@ -1901,6 +1913,7 @@ GF_Err gf_isom_get_audio_info(GF_ISOFile *movie, u32 trackNumber, u32 StreamDesc
 	case GF_ISOM_SUBTYPE_3GP_EVRC:
 	case GF_ISOM_SUBTYPE_3GP_QCELP:
 	case GF_ISOM_SUBTYPE_3GP_SMV:
+	case GF_ISOM_BOX_TYPE_AC3:
 		if (SampleRate) (*SampleRate) = ((GF_AudioSampleEntryBox*)entry)->samplerate_hi;
 		if (Channels) (*Channels) = ((GF_AudioSampleEntryBox*)entry)->channel_count;
 		if (bitsPerSample) (*bitsPerSample) = (u8) ((GF_AudioSampleEntryBox*)entry)->bitspersample;
@@ -1910,6 +1923,39 @@ GF_Err gf_isom_get_audio_info(GF_ISOFile *movie, u32 trackNumber, u32 StreamDesc
 	}
 }
 
+GF_EXPORT
+GF_Err gf_isom_get_pixel_aspect_ratio(GF_ISOFile *movie, u32 trackNumber, u32 StreamDescriptionIndex, u32 *hSpacing, u32 *vSpacing)
+{
+	GF_TrackBox *trak;
+	GF_SampleEntryBox *entry;
+	GF_SampleDescriptionBox *stsd;
+
+	trak = gf_isom_get_track_from_file(movie, trackNumber);
+	if (!trak) return GF_BAD_PARAM;
+
+	stsd = trak->Media->information->sampleTable->SampleDescription;
+	if (!stsd) return movie->LastError = GF_ISOM_INVALID_FILE;
+	if (!StreamDescriptionIndex || StreamDescriptionIndex > gf_list_count(stsd->boxList)) return movie->LastError = GF_BAD_PARAM;
+
+	entry = (GF_SampleEntryBox *)gf_list_get(stsd->boxList, StreamDescriptionIndex - 1);
+	//no support for generic sample entries (eg, no MPEG4 descriptor)
+	if (entry == NULL) return GF_BAD_PARAM;
+	
+	//valid for MPEG visual, JPG and 3GPP H263
+	switch (entry->type) {
+	case GF_ISOM_BOX_TYPE_ENCV:
+	case GF_ISOM_BOX_TYPE_MP4V:
+	case GF_ISOM_SUBTYPE_3GP_H263:
+	case GF_ISOM_BOX_TYPE_AVC1:
+	case GF_ISOM_BOX_TYPE_GNRV:
+		*hSpacing = ((GF_VisualSampleEntryBox*)entry)->pasp ? ((GF_VisualSampleEntryBox*)entry)->pasp->hSpacing : 0;
+		*vSpacing = ((GF_VisualSampleEntryBox*)entry)->pasp ? ((GF_VisualSampleEntryBox*)entry)->pasp->vSpacing : 0;
+		return GF_OK;
+	default:
+		return GF_BAD_PARAM;
+	}
+}
+
 GF_EXPORT
 const char *gf_isom_get_filename(GF_ISOFile *movie)
 {
@@ -2197,3 +2243,78 @@ GF_Err gf_isom_apple_get_tag(GF_ISOFile *mov, u32 tag, const char **data, u32 *d
 	return GF_OK;
 }
 
+
+
+
+GF_EXPORT
+GF_Err gf_isom_get_track_switch_group_count(GF_ISOFile *movie, u32 trackNumber, u32 *alternateGroupID, u32 *nb_groups)
+{
+	GF_UserDataMap *map;
+	GF_TrackBox *trak;
+
+	trak = gf_isom_get_track_from_file(movie, trackNumber);
+	if (!trak) return GF_BAD_PARAM;
+	*alternateGroupID = trak->Header->alternate_group;
+	*nb_groups = 0;
+	if (!trak->udta) return GF_OK;
+
+	map = udta_getEntry(trak->udta, GF_ISOM_BOX_TYPE_TSEL, NULL);
+	if (!map) return 0;
+	*nb_groups = gf_list_count(map->boxList);
+	return GF_OK;
+}
+
+GF_EXPORT
+const u32 *gf_isom_get_track_switch_parameter(GF_ISOFile *movie, u32 trackNumber, u32 group_index, u32 *switchGroupID, u32 *criteriaListSize)
+{
+	GF_TrackBox *trak;
+	GF_UserDataMap *map;
+	GF_TrackSelectionBox *tsel;
+
+	trak = gf_isom_get_track_from_file(movie, trackNumber);
+	if (!group_index || !trak || !trak->udta) return NULL;
+
+	map = udta_getEntry(trak->udta, GF_ISOM_BOX_TYPE_TSEL, NULL);
+	if (!map) return NULL;
+	tsel = gf_list_get(map->boxList, group_index-1);
+	*switchGroupID = tsel->switchGroup;
+	*criteriaListSize = tsel->attributeListCount;
+	return (const u32 *) tsel->attributeList;
+}
+
+
+GF_EXPORT
+GF_Err gf_isom_get_timed_meta_data_info(GF_ISOFile *file, u32 track, u32 sampleDescription, Bool *is_xml, const char **mime_or_namespace, const char **content_encoding, const char **schema_loc)
+{
+	GF_TrackBox *trak;
+	GF_MetaDataSampleEntryBox *ptr;
+	trak = gf_isom_get_track_from_file(file, track);
+	if (!trak || !sampleDescription) return GF_BAD_PARAM;
+	ptr = (GF_MetaDataSampleEntryBox*)gf_list_get(trak->Media->information->sampleTable->SampleDescription->boxList, sampleDescription-1);
+	if (!ptr) return GF_BAD_PARAM;
+
+	if (ptr->type==GF_ISOM_BOX_TYPE_METX) {
+		if (is_xml) *is_xml = 1;
+		if (schema_loc) *schema_loc = ptr->xml_schema_loc;
+	} else {
+		if (schema_loc) *schema_loc = NULL;
+	}
+	if (mime_or_namespace) *mime_or_namespace = ptr->mime_type_or_namespace;
+	if (content_encoding) *content_encoding = ptr->content_encoding;
+	return GF_OK;
+}
+
+u32 gf_isom_get_next_alternate_group_id(GF_ISOFile *movie)
+{
+	u32 id = 0;
+	u32 i=0;		
+
+	while (i< gf_isom_get_track_count(movie) ) {
+		GF_TrackBox *trak = gf_isom_get_track_from_file(movie, i+1);
+		if (trak->Header->alternate_group > id)
+			id = trak->Header->alternate_group;
+		i++;
+	}
+	return id+1;
+}
+
