@@ -27,7 +27,7 @@
 
 #ifndef GPAC_READ_ONLY
 
-#define GPAC_ISOM_CPRT_NOTICE "IsoMedia File Produced with GPAC "GPAC_VERSION
+#define GPAC_ISOM_CPRT_NOTICE "IsoMedia File Produced with GPAC "GPAC_FULL_VERSION
 
 static GF_Err gf_isom_insert_copyright(GF_ISOFile *movie)
 {
@@ -112,11 +112,13 @@ void ResetWriters(GF_List *writers)
 		if (writer->stco->type == GF_ISOM_BOX_TYPE_STCO) {
 			free(((GF_ChunkOffsetBox *)writer->stco)->offsets);
 			((GF_ChunkOffsetBox *)writer->stco)->offsets = NULL;
-			((GF_ChunkOffsetBox *)writer->stco)->entryCount = 0;
+			((GF_ChunkOffsetBox *)writer->stco)->nb_entries = 0;
+			((GF_ChunkOffsetBox *)writer->stco)->alloc_size = 0;
 		} else {
 			free(((GF_ChunkLargeOffsetBox *)writer->stco)->offsets);
 			((GF_ChunkLargeOffsetBox *)writer->stco)->offsets = NULL;
-			((GF_ChunkLargeOffsetBox *)writer->stco)->entryCount = 0;
+			((GF_ChunkLargeOffsetBox *)writer->stco)->nb_entries = 0;
+			((GF_ChunkLargeOffsetBox *)writer->stco)->alloc_size = 0;
 		}
 	}
 }
@@ -207,29 +209,29 @@ static GF_Err ShiftOffset(GF_ISOFile *file, GF_List *writers, u64 offset)
 		if (writer->mdia->mediaTrack->meta) ShiftMetaOffset(writer->mdia->mediaTrack->meta, offset);
 
 		//we have to proceed entry by entry in case a part of the media is not self-contained...
-		j=0;
-		while ((ent = (GF_StscEntry *)gf_list_enum(writer->stsc->entryList, &j))) {
+		for (j=0; j<writer->stsc->nb_entries; j++) {
+			ent = &writer->stsc->entries[j];
 			if (!Media_IsSelfContained(writer->mdia, ent->sampleDescriptionIndex)) continue;
 
 			//OK, get the chunk(s) number(s) and "shift" its (their) offset(s).
 			if (writer->stco->type == GF_ISOM_BOX_TYPE_STCO) {
 				stco = (GF_ChunkOffsetBox *) writer->stco;
 				//be carefull for the last entry, nextChunk is set to 0 in edit mode...
-				last = ent->nextChunk ? ent->nextChunk : stco->entryCount + 1;
+				last = ent->nextChunk ? ent->nextChunk : stco->nb_entries + 1;
 				for (k = ent->firstChunk; k < last; k++) {
 
 					if (stco->offsets[k-1] + offset > 0xFFFFFFFF) {
 						//too bad, rewrite the table....
 						co64 = (GF_ChunkLargeOffsetBox *) gf_isom_box_new(GF_ISOM_BOX_TYPE_CO64);
 						if (!co64) return GF_OUT_OF_MEM;
-						co64->entryCount = stco->entryCount;
-						co64->offsets = (u64*)malloc(co64->entryCount * sizeof(u64));
+						co64->nb_entries = stco->nb_entries;
+						co64->offsets = (u64*)malloc(co64->nb_entries * sizeof(u64));
 						if (!co64) {
 							gf_isom_box_del((GF_Box *)co64);
 							return GF_OUT_OF_MEM;
 						}
 						//duplicate the table 
-						for (l = 0; l < co64->entryCount; l++) {
+						for (l = 0; l < co64->nb_entries; l++) {
 							co64->offsets[l] = (u64) stco->offsets[l];
 							if (l + 1 == k) co64->offsets[l] += offset;
 						}
@@ -243,7 +245,7 @@ static GF_Err ShiftOffset(GF_ISOFile *file, GF_List *writers, u64 offset)
 			} else {
 				co64 = (GF_ChunkLargeOffsetBox *) writer->stco;
 				//be carefull for the last entry ...
-				last = ent->nextChunk ? ent->nextChunk : co64->entryCount + 1;
+				last = ent->nextChunk ? ent->nextChunk : co64->nb_entries + 1;
 				for (k = ent->firstChunk; k < last; k++) {
 					co64->offsets[k-1] += offset;
 				}
@@ -1000,14 +1002,15 @@ GF_Err DoInterleave(MovieWriter *mw, GF_List *writers, GF_BitStream *bs, u8 Emul
 	count = gf_list_count(writers);
 	//browse each groups
 	while (1) {
+		/*the max DTS the chunk of the current writer*/
+		u64 chunkLastDTS = 0;
+		/*the timescale related to the max DTS*/
+		u32 chunkLastScale = 0;
+
 		writeGroup = 1;
 
 		//proceed a group
 		while (writeGroup) {
-			/*the DTS for the end of this chunk*/
-			u64 chunkMaxDTS = 0;
-			/*the timescale DTS for the end of this chunk*/
-			u32 chunkMaxScale = 0;
 			curWriter = NULL;
 			for (i=0 ; i < count; i++) {
 				tmp = (TrackWriter*)gf_list_get(writers, i);
@@ -1032,8 +1035,8 @@ GF_Err DoInterleave(MovieWriter *mw, GF_List *writers, GF_BitStream *bs, u8 Emul
 
 					//can this sample fit in our chunk ?
 					if ( ( (DTS - tmp->DTSprev) + tmp->chunkDur) *  movie->moov->mvhd->timeScale > movie->interleavingTime * tmp->timeScale
-						/*try to keep chunk synchronized within the group*/
-						|| (drift_inter && chunkMaxDTS && ( ((u64)tmp->DTSprev*chunkMaxScale) > ((u64)chunkMaxDTS*tmp->timeScale)) ) 
+						/*drfit check: reject sample if outside our check window*/
+						|| (drift_inter && chunkLastDTS && ( ((u64)tmp->DTSprev*chunkLastScale) > ((u64)chunkLastDTS*tmp->timeScale)) ) 
 						) {
 						//in case the sample is longer than InterleaveTime
 						if (!tmp->chunkDur) {
@@ -1095,9 +1098,18 @@ GF_Err DoInterleave(MovieWriter *mw, GF_List *writers, GF_BitStream *bs, u8 Emul
 					}
 				}
 				/*record chunk end-time & track timescale for drift-controled interleaving*/
-				if (drift_inter && !chunkMaxDTS && curWriter) {
-					chunkMaxDTS = curWriter->DTSprev;
-					chunkMaxScale = curWriter->timeScale;
+				if (drift_inter && curWriter) {
+					chunkLastScale = curWriter->timeScale;
+					chunkLastDTS = curWriter->DTSprev;
+					/*add one interleave window drift - since the "maxDTS" is the previously written one, we will
+					have the following cases:
+					- sample doesn't fit: post-pone and force new chunk 
+					- sample time larger than previous chunk time + interleave: post-pone and force new chunk
+					- otherwise store and track becomes current reference
+
+					this ensures a proper drift regulation (max DTS diff is less than the interleaving window)
+					*/
+					chunkLastDTS += curWriter->timeScale * movie->interleavingTime / movie->moov->mvhd->timeScale;
 				}
 			}
 			//no sample found, we're done with this group
