@@ -59,7 +59,7 @@ GF_Err findEntryForTime(GF_SampleTableBox *stbl, u64 DTS, u8 useCTS, u32 *sample
 			stbl_GetSampleCTS(stbl->CompositionOffset, curSampNum, &CTSOffset);
 			//we're too far, rewind
 			if ( i && (curDTS + CTSOffset > DTS) ) {
-				ent = (GF_SttsEntry*)gf_list_get(stbl->TimeToSample->entryList, i);
+				ent = &stbl->TimeToSample->entries[i];
 				curSampNum -= ent->sampleCount;
 				curDTS -= ent->sampleDelta * ent->sampleCount;
 				i --;
@@ -77,9 +77,9 @@ GF_Err findEntryForTime(GF_SampleTableBox *stbl, u64 DTS, u8 useCTS, u32 *sample
 	}
 
 	//look for the DTS from this entry
-	count = gf_list_count(stbl->TimeToSample->entryList);
+	count = stbl->TimeToSample->nb_entries;
 	for (; i<count; i++) {
-		ent = (GF_SttsEntry*)gf_list_get(stbl->TimeToSample->entryList, i);
+		ent = &stbl->TimeToSample->entries[i];
 		if (useCTS) {
 			stbl_GetSampleCTS(stbl->CompositionOffset, curSampNum, &CTSOffset);
 		} else {
@@ -137,11 +137,9 @@ GF_Err stbl_GetSampleSize(GF_SampleSizeBox *stsz, u32 SampleNumber, u32 *Size)
 //Get the CTS offset of a given sample
 GF_Err stbl_GetSampleCTS(GF_CompositionOffsetBox *ctts, u32 SampleNumber, u32 *CTSoffset)
 {
-	u32 i, count;
-	GF_DttsEntry *ent;
+	u32 i;
 
 	(*CTSoffset) = 0;
-	ent = NULL;
 	//test on SampleNumber is done before
 	if (!ctts || !SampleNumber) return GF_BAD_PARAM;
 
@@ -152,19 +150,17 @@ GF_Err stbl_GetSampleCTS(GF_CompositionOffsetBox *ctts, u32 SampleNumber, u32 *C
 		ctts->r_currentEntryIndex = 0;
 		i = 0;
 	}
-	count = gf_list_count(ctts->entryList);
-	for (; i< count; i++) {
-		ent = (GF_DttsEntry*)gf_list_get(ctts->entryList, i);
-		if (SampleNumber < ctts->r_FirstSampleInEntry + ent->sampleCount) break;
+	for (; i< ctts->nb_entries; i++) {
+		if (SampleNumber < ctts->r_FirstSampleInEntry + ctts->entries[i].sampleCount) break;
 		//update our cache
 		ctts->r_currentEntryIndex += 1;
-		ctts->r_FirstSampleInEntry += ent->sampleCount;
+		ctts->r_FirstSampleInEntry += ctts->entries[i].sampleCount;
 	}
 	//no ent, set everything to 0...
-	if (!ent) return GF_OK;
+	if (i==ctts->nb_entries) return GF_OK;
 	/*asked for a sample not in table - this means CTTS is 0 (that's due to out internal packing construction of CTTS)*/
-	if (SampleNumber >= ctts->r_FirstSampleInEntry + ent->sampleCount) return GF_OK;
-	(*CTSoffset) = ent->decodingOffset;
+	if (SampleNumber >= ctts->r_FirstSampleInEntry + ctts->entries[i].sampleCount) return GF_OK;
+	(*CTSoffset) = ctts->entries[i].decodingOffset;
 	return GF_OK;
 }
 
@@ -179,7 +175,7 @@ GF_Err stbl_GetSampleDTS(GF_TimeToSampleBox *stts, u32 SampleNumber, u64 *DTS)
 
 	ent = NULL;
 	//use our cache
-	count = gf_list_count(stts->entryList);
+	count = stts->nb_entries;
 	if (stts->r_FirstSampleInEntry 
 		&& (stts->r_FirstSampleInEntry <= SampleNumber)
 		//this is for read/write access
@@ -193,7 +189,7 @@ GF_Err stbl_GetSampleDTS(GF_TimeToSampleBox *stts, u32 SampleNumber, u64 *DTS)
 	}
 
 	for (; i < count; i++) {
-		ent = (GF_SttsEntry*)gf_list_get(stts->entryList, i);
+		ent = &stts->entries[i];
 
 		//in our entry
 		if (ent->sampleCount + stts->r_FirstSampleInEntry >= 1 + SampleNumber) {
@@ -239,7 +235,7 @@ GF_Err stbl_GetSampleRAP(GF_SyncSampleBox *stss, u32 SampleNumber, u8 *IsRAP, u3
 	} else {
 		i = 0;
 	}
-	for (; i < stss->entryCount; i++) {
+	for (; i < stss->nb_entries; i++) {
 		//get the entry
 		if (stss->sampleNumbers[i] == SampleNumber) {
 			//update the cache
@@ -269,14 +265,14 @@ void GetGhostNum(GF_StscEntry *ent, u32 EntryIndex, u32 count, GF_SampleTableBox
 			//not specified in the spec, what if the last sample to chunk is no written?
 			if (stbl->ChunkOffset->type == GF_ISOM_BOX_TYPE_STCO) {
 				stco = (GF_ChunkOffsetBox *)stbl->ChunkOffset;
-				ghostNum = (stco->entryCount > ent->firstChunk) ? (1 + stco->entryCount - ent->firstChunk) : 1;
+				ghostNum = (stco->nb_entries > ent->firstChunk) ? (1 + stco->nb_entries - ent->firstChunk) : 1;
 			} else {
 				co64 = (GF_ChunkLargeOffsetBox *)stbl->ChunkOffset;
-				ghostNum = (co64->entryCount > ent->firstChunk) ? (1 + co64->entryCount - ent->firstChunk) : 1;
+				ghostNum = (co64->nb_entries > ent->firstChunk) ? (1 + co64->nb_entries - ent->firstChunk) : 1;
 			}
 		} else {
 			//this is an unknown case due to edit mode...
-			nextEnt = (GF_StscEntry*)gf_list_get(stbl->SampleToChunk->entryList, EntryIndex+1);
+			nextEnt = &stbl->SampleToChunk->entries[EntryIndex+1];
 			ghostNum = nextEnt->firstChunk - ent->firstChunk;
 		}
 	} else {
@@ -289,7 +285,7 @@ void GetGhostNum(GF_StscEntry *ent, u32 EntryIndex, u32 count, GF_SampleTableBox
 GF_Err stbl_GetSampleInfos(GF_SampleTableBox *stbl, u32 sampleNumber, u64 *offset, u32 *chunkNumber, u32 *descIndex, u8 *isEdited)
 {
 	GF_Err e;
-	u32 i, j, k, offsetInChunk, size, count;
+	u32 i, j, k, offsetInChunk, size;
 	GF_ChunkOffsetBox *stco;
 	GF_ChunkLargeOffsetBox *co64;
 	GF_StscEntry *ent;
@@ -299,9 +295,8 @@ GF_Err stbl_GetSampleInfos(GF_SampleTableBox *stbl, u32 sampleNumber, u64 *offse
 	(*isEdited) = 0;
 	if (!stbl || !sampleNumber) return GF_BAD_PARAM;
 
-	count = gf_list_count(stbl->SampleToChunk->entryList);
-	if (count==stbl->SampleSize->sampleCount) {
-		ent = (GF_StscEntry *)gf_list_get(stbl->SampleToChunk->entryList, sampleNumber-1);
+	if (stbl->SampleToChunk->nb_entries == stbl->SampleSize->sampleCount) {
+		ent = &stbl->SampleToChunk->entries[sampleNumber-1];
 		if (!ent) return GF_BAD_PARAM;
 		(*descIndex) = ent->sampleDescriptionIndex;
 		(*chunkNumber) = sampleNumber;
@@ -322,22 +317,21 @@ GF_Err stbl_GetSampleInfos(GF_SampleTableBox *stbl, u32 sampleNumber, u64 *offse
 		
 		i = stbl->SampleToChunk->currentIndex;
 //		ent = gf_list_get(stbl->SampleToChunk->entryList, i);
-		ent = stbl->SampleToChunk->currentEntry;
-		GetGhostNum(ent, i, count, stbl);
+		ent = &stbl->SampleToChunk->entries[stbl->SampleToChunk->currentIndex];
+		GetGhostNum(ent, i, stbl->SampleToChunk->nb_entries, stbl);
 		k = stbl->SampleToChunk->currentChunk;
 	} else {
 		i = 0;
 		stbl->SampleToChunk->currentIndex = 0;
 		stbl->SampleToChunk->currentChunk = 1;
 		stbl->SampleToChunk->firstSampleInCurrentChunk = 1;
-		ent = (GF_StscEntry*)gf_list_get(stbl->SampleToChunk->entryList, 0);
-		stbl->SampleToChunk->currentEntry = ent;
-		GetGhostNum(ent, 0, count, stbl);
+		ent = &stbl->SampleToChunk->entries[0];
+		GetGhostNum(ent, 0, stbl->SampleToChunk->nb_entries, stbl);
 		k = stbl->SampleToChunk->currentChunk;
 	}
 
 	//first get the chunk
-	for (; i < count; i++) {
+	for (; i < stbl->SampleToChunk->nb_entries; i++) {
 		//browse from the current chunk we're browsing from index 1
 		for (; k <= stbl->SampleToChunk->ghostNumber; k++) {
 			//browse all the samples in this chunk
@@ -351,12 +345,11 @@ GF_Err stbl_GetSampleInfos(GF_SampleTableBox *stbl, u32 sampleNumber, u64 *offse
 			stbl->SampleToChunk->currentChunk ++;
 		}
 		//not in this entry, get the next entry if not the last one
-		if (i+1 != count) {
-			ent = (GF_StscEntry*)gf_list_get(stbl->SampleToChunk->entryList, i+1);
+		if (i+1 != stbl->SampleToChunk->nb_entries) {
+			ent = &stbl->SampleToChunk->entries[i+1];
 			//update the GhostNumber
-			GetGhostNum(ent, i+1, count, stbl);
+			GetGhostNum(ent, i+1, stbl->SampleToChunk->nb_entries, stbl);
 			//update the entry in our cache
-			stbl->SampleToChunk->currentEntry = ent;
 			stbl->SampleToChunk->currentIndex = i+1;
 			stbl->SampleToChunk->currentChunk = 1;
 			k = 1;
@@ -383,11 +376,11 @@ sample_found:
 	//now get the chunk
 	if ( stbl->ChunkOffset->type == GF_ISOM_BOX_TYPE_STCO) {
 		stco = (GF_ChunkOffsetBox *)stbl->ChunkOffset;
-		if (stco->entryCount < (*chunkNumber) ) return GF_ISOM_INVALID_FILE;
+		if (stco->nb_entries < (*chunkNumber) ) return GF_ISOM_INVALID_FILE;
 		(*offset) = (u64) stco->offsets[(*chunkNumber) - 1] + (u64) offsetInChunk;
 	} else {
 		co64 = (GF_ChunkLargeOffsetBox *)stbl->ChunkOffset;
-		if (co64->entryCount < (*chunkNumber) ) return GF_ISOM_INVALID_FILE;
+		if (co64->nb_entries < (*chunkNumber) ) return GF_ISOM_INVALID_FILE;
 		(*offset) = co64->offsets[(*chunkNumber) - 1] + (u64) offsetInChunk;
 	}
 	return GF_OK;
