@@ -215,7 +215,7 @@ default_sync:
 	} else {
 		esd->slConfig->hasRandomAccessUnitsOnlyFlag = 0;
 		//signal we are NOT using sync points if no info is present in the table
-		esd->slConfig->useRandomAccessPointFlag = stbl->SyncSample->entryCount ? 1 : 0;
+		esd->slConfig->useRandomAccessPointFlag = stbl->SyncSample->nb_entries ? 1 : 0;
 	}
 	//do we have DegradationPriority ?
 	if (stbl->DegradationPriority) {
@@ -265,7 +265,7 @@ GF_Err GetESDForTime(GF_MovieBox *moov, u32 trackID, u64 CTS, GF_ESD **outESD)
 GF_Err Track_FindRef(GF_TrackBox *trak, u32 ReferenceType, GF_TrackReferenceTypeBox **dpnd)
 {
 	GF_TrackReferenceBox *ref;
-	GF_Box *a;
+	GF_TrackReferenceTypeBox *a;
 	u32 i;
 	if (! trak) return GF_BAD_PARAM;
 	if (! trak->References) {
@@ -274,9 +274,9 @@ GF_Err Track_FindRef(GF_TrackBox *trak, u32 ReferenceType, GF_TrackReferenceType
 	}
 	ref = trak->References;
 	i=0;
-	while ((a = (GF_Box *)gf_list_enum(ref->boxList, &i))) {
-		if (a->type == ReferenceType) {
-			*dpnd = (GF_TrackReferenceTypeBox *)a;
+	while ((a = (GF_TrackReferenceTypeBox *)gf_list_enum(ref->boxList, &i))) {
+		if (a->reference_type == ReferenceType) {
+			*dpnd = a;
 			return GF_OK;
 		}
 	}
@@ -338,7 +338,7 @@ GF_Err SetTrackDuration(GF_TrackBox *trak)
 }
 
 
-#ifndef	GF_ISOM_NO_FRAGMENTS
+#ifndef	GPAC_ISOM_NO_FRAGMENTS
 
 GF_Err MergeTrack(GF_TrackBox *trak, GF_TrackFragmentBox *traf, u64 *moof_offset)
 {
@@ -512,6 +512,10 @@ GF_Err NewMedia(GF_MediaBox **mdia, u32 MediaType, u32 TimeScale)
 		mediaInfo = gf_isom_box_new(GF_ISOM_BOX_TYPE_HMHD);
 		str = "GPAC ISO Hint Handler";
 		break;
+	case GF_ISOM_MEDIA_META:
+		mediaInfo = gf_isom_box_new(GF_ISOM_BOX_TYPE_NMHD);
+		str = "GPAC Timed MetaData Handler";
+		break;
 	case GF_ISOM_MEDIA_OD:
 		mediaInfo = gf_isom_box_new(GF_ISOM_BOX_TYPE_NMHD);
 		str = "GPAC MPEG-4 OD Handler";
@@ -522,7 +526,7 @@ GF_Err NewMedia(GF_MediaBox **mdia, u32 MediaType, u32 TimeScale)
 		break;
 	case GF_ISOM_MEDIA_SCENE:
 		mediaInfo = gf_isom_box_new(GF_ISOM_BOX_TYPE_NMHD);
-		str = "GPAC MPEG-4 BIFS Handler";
+		str = "GPAC MPEG-4 Scene Description Handler";
 		break;
 	case GF_ISOM_MEDIA_MPEG7:
 		mediaInfo = gf_isom_box_new(GF_ISOM_BOX_TYPE_NMHD);
@@ -544,6 +548,11 @@ GF_Err NewMedia(GF_MediaBox **mdia, u32 MediaType, u32 TimeScale)
 		mediaInfo = gf_isom_box_new(GF_ISOM_BOX_TYPE_NMHD);
 		str = "GPAC Streaming Text Handler";
 		break;
+	case GF_ISOM_MEDIA_DIMS:
+		mediaInfo = gf_isom_box_new(GF_ISOM_BOX_TYPE_VMHD);
+		MediaType = GF_ISOM_MEDIA_SCENE;
+		str = "GPAC DIMS Handler";
+		break;
 	default:
 		mediaInfo = gf_isom_box_new(GF_ISOM_BOX_TYPE_NMHD);
 		str = "GPAC IsoMedia Handler";
@@ -631,7 +640,8 @@ GF_Err Track_SetStreamDescriptor(GF_TrackBox *trak, u32 StreamDescriptionIndex,
 	e = Track_FindRef(trak, GF_ISOM_REF_DECODE, &dpnd);
 	if (e) return e;
 	if (!dpnd && esd->dependsOnESID) {
-		dpnd = (GF_TrackReferenceTypeBox *) gf_isom_box_new(GF_ISOM_BOX_TYPE_DPND);
+		dpnd = (GF_TrackReferenceTypeBox *) gf_isom_box_new(GF_ISOM_BOX_TYPE_REFT);
+		dpnd->reference_type = GF_ISOM_BOX_TYPE_DPND;
 		e = tref_AddBox((GF_Box*)tref, (GF_Box *) dpnd);
 		if (e) return e;
 		e = reftype_AddRefTrack(dpnd, esd->dependsOnESID, NULL);
@@ -645,7 +655,8 @@ GF_Err Track_SetStreamDescriptor(GF_TrackBox *trak, u32 StreamDescriptionIndex,
 	e = Track_FindRef(trak, GF_ISOM_REF_OCR, &dpnd);
 	if (e) return e;
 	if (!dpnd && esd->OCRESID) {
-		dpnd = (GF_TrackReferenceTypeBox *) gf_isom_box_new(GF_ISOM_BOX_TYPE_SYNC);
+		dpnd = (GF_TrackReferenceTypeBox *) gf_isom_box_new(GF_ISOM_BOX_TYPE_REFT);
+		dpnd->reference_type = GF_ISOM_BOX_TYPE_SYNC;
 		e = tref_AddBox((GF_Box*)tref, (GF_Box *) dpnd);
 		if (e) return e;
 		e = reftype_AddRefTrack(dpnd, esd->OCRESID, NULL);
@@ -664,7 +675,8 @@ GF_Err Track_SetStreamDescriptor(GF_TrackBox *trak, u32 StreamDescriptionIndex,
 		if (e) return e;
 		if (!dpnd) {
 			tmpRef = 0;
-			dpnd = (GF_TrackReferenceTypeBox *) gf_isom_box_new(GF_ISOM_BOX_TYPE_IPIR);
+			dpnd = (GF_TrackReferenceTypeBox *) gf_isom_box_new(GF_ISOM_BOX_TYPE_REFT);
+			dpnd->reference_type = GF_ISOM_BOX_TYPE_IPIR;
 			e = tref_AddBox((GF_Box*)tref, (GF_Box *) dpnd);
 			if (e) return e;
 			e = reftype_AddRefTrack(dpnd, esd->ipiPtr->IPI_ES_Id, &tmpRef);
