@@ -228,6 +228,7 @@ static GF_Err M4V_Reset(GF_M4VParser *m4v, u32 start)
 static GF_Err gf_m4v_parse_config_mpeg12(GF_M4VParser *m4v, GF_M4VDecSpecInfo *dsi)
 {
 	char p[4];
+	u32 ext_type;
 	s32 o_type;
 	u8 go, par;
 
@@ -249,9 +250,9 @@ static GF_Err gf_m4v_parse_config_mpeg12(GF_M4VParser *m4v, GF_M4VDecSpecInfo *d
 			dsi->VideoPL = 0x6A;
 			par = (p[3] >> 4) & 0xf;
 			switch (par) {
-			case 2: dsi->par_num = 4; dsi->par_den = 3; break;
-			case 3: dsi->par_num = 16; dsi->par_den = 9; break;
-			case 4: dsi->par_num = 2; dsi->par_den = 21; break;
+			case 2: dsi->par_num = dsi->height/3; dsi->par_den = dsi->width/4; break;
+			case 3: dsi->par_num = dsi->height/9; dsi->par_den = dsi->width/16; break;
+			case 4: dsi->par_num = dsi->height/2; dsi->par_den = dsi->width/21; break;
 			default: dsi->par_den = dsi->par_num = 0; break;
 			}
 			switch (p[3] & 0xf) {
@@ -273,14 +274,15 @@ static GF_Err gf_m4v_parse_config_mpeg12(GF_M4VParser *m4v, GF_M4VDecSpecInfo *d
 			break;
 		case M2V_EXT_START_CODE:
 			gf_bs_read_data(m4v->bs, p, 4);
-			if ( ((p[0] >> 4) & 0xf) == 1) {
+			ext_type = ((p[0] >> 4) & 0xf);
+			if (ext_type == 1) {
 			  dsi->VideoPL = 0x65;
 			  dsi->height = ((p[1] & 0x1) << 13) | ((p[2] & 0x80) << 5) | (dsi->height & 0x0fff);
 			  dsi->width = (((p[2] >> 5) & 0x3) << 12) | (dsi->width & 0x0fff);
 			}
 			break;
 		case M2V_PIC_START_CODE:
-			go = 0;
+			if (dsi->width) go = 0;
 			break;
 		default:
 			break;
@@ -797,14 +799,12 @@ u32 gf_m4a_get_profile(GF_M4ADecSpecInfo *cfg)
 	}
 }
 
-GF_EXPORT
-GF_Err gf_m4a_get_config(char *dsi, u32 dsi_size, GF_M4ADecSpecInfo *cfg)
-{
-	GF_BitStream *bs;
-	memset(cfg, 0, sizeof(GF_M4ADecSpecInfo));
-	if (!dsi || !dsi_size || (dsi_size<2) ) return GF_NON_COMPLIANT_BITSTREAM;
-	bs = gf_bs_new(dsi, dsi_size, GF_BITSTREAM_READ);
 
+
+GF_EXPORT
+GF_Err gf_m4a_parse_config(GF_BitStream *bs, GF_M4ADecSpecInfo *cfg, Bool size_known)
+{
+	memset(cfg, 0, sizeof(GF_M4ADecSpecInfo));
 	cfg->base_object_type = gf_bs_read_int(bs, 5);
 	/*extended object type*/
 	if (cfg->base_object_type==31) {
@@ -831,10 +831,81 @@ GF_Err gf_m4a_get_config(char *dsi, u32 dsi_size, GF_M4ADecSpecInfo *cfg)
 		cfg->sbr_object_type = gf_bs_read_int(bs, 5);
 	}
 
-	gf_bs_align(bs);
-	if (gf_bs_available(bs)>=2) {
-		u32 sync = gf_bs_read_int(bs, 11);
+	/*object cfg*/
+	switch (cfg->base_object_type) {
+	case 1:
+	case 2:
+	case 3:
+	case 4:
+	case 6:
+	case 7:
+	case 17:
+	case 19:
+	case 20:
+	case 21:
+	case 22:
+	case 23:
+	{
+		Bool fl_flag, ext_flag;
+		u32 delay;
+		/*frame length flag*/
+		fl_flag = gf_bs_read_int(bs, 1);
+		/*depends on core coder*/
+		delay = 0;
+		if (gf_bs_read_int(bs, 1))
+			delay = gf_bs_read_int(bs, 14);
+		ext_flag = gf_bs_read_int(bs, 1);
+		if (!cfg->nb_chan) {
+		}
+		if ((cfg->base_object_type == 6) || (cfg->base_object_type == 20)) {
+			gf_bs_read_int(bs, 3);
+		}
+		if (ext_flag) {
+			if (cfg->base_object_type == 22) {
+				gf_bs_read_int(bs, 5);
+				gf_bs_read_int(bs, 11);
+			}
+			if ((cfg->base_object_type == 17) 
+				|| (cfg->base_object_type == 19) 
+				|| (cfg->base_object_type == 20) 
+				|| (cfg->base_object_type == 23) 
+			) {
+				gf_bs_read_int(bs, 1);
+				gf_bs_read_int(bs, 1);
+				gf_bs_read_int(bs, 1);
+			}
+			ext_flag = gf_bs_read_int(bs, 1);
+		}
+	}
+		break;
+	}
+	/*ER cfg*/
+	switch (cfg->base_object_type) {
+	case 17:
+	case 19:
+	case 20:
+	case 21:
+	case 22:
+	case 23:
+	case 24:
+	case 25:
+	case 26:
+	case 27:
+	{
+		u32 epConfig = gf_bs_read_int(bs, 2);
+		if ((epConfig == 2) || (epConfig == 3) ) {
+		}
+		if (epConfig == 3) {
+			gf_bs_read_int(bs, 1);
+		}
+	}
+		break;
+	}
+
+	if (size_known && (cfg->base_object_type != 5) && gf_bs_available(bs)>=2) {
+		u32 sync = gf_bs_peek_bits(bs, 11, 0);
 		if (sync==0x2b7) {
+			gf_bs_read_int(bs, 11);
 			cfg->sbr_object_type = gf_bs_read_int(bs, 5);
 			cfg->has_sbr = gf_bs_read_int(bs, 1);
 			if (cfg->has_sbr) {
@@ -848,6 +919,96 @@ GF_Err gf_m4a_get_config(char *dsi, u32 dsi_size, GF_M4ADecSpecInfo *cfg)
 		}
 	}
 	cfg->audioPL = gf_m4a_get_profile(cfg);
+	return GF_OK;
+}
+
+GF_EXPORT
+GF_Err gf_m4a_get_config(char *dsi, u32 dsi_size, GF_M4ADecSpecInfo *cfg)
+{
+	GF_BitStream *bs;
+	if (!dsi || !dsi_size || (dsi_size<2) ) return GF_NON_COMPLIANT_BITSTREAM;
+	bs = gf_bs_new(dsi, dsi_size, GF_BITSTREAM_READ);
+	gf_m4a_parse_config(bs, cfg, 1);
+	gf_bs_del(bs);
+	return GF_OK;
+}
+
+
+GF_EXPORT
+GF_Err gf_m4a_write_config(GF_M4ADecSpecInfo *cfg, char **dsi, u32 *dsi_size)
+{
+	GF_BitStream *bs = gf_bs_new(NULL, 0, GF_BITSTREAM_WRITE);
+
+	/*extended object type*/
+	if (cfg->base_object_type>=32) {
+		gf_bs_write_int(bs, 31, 5);
+		gf_bs_write_int(bs, cfg->base_object_type-32, 6);
+	} else {
+		gf_bs_write_int(bs, cfg->base_object_type, 5);
+	}
+	gf_bs_write_int(bs, cfg->base_sr_index, 4);
+	if (cfg->base_sr_index == 0x0F) {
+		gf_bs_write_int(bs, cfg->base_sr, 24);
+	}
+	if (cfg->nb_chan == 8) {
+		gf_bs_write_int(bs, 7, 4);
+	} else {
+		gf_bs_write_int(bs, cfg->nb_chan, 4);
+	}
+
+	if (cfg->base_object_type==5) {
+		cfg->has_sbr = 1;
+		gf_bs_write_int(bs, cfg->sbr_sr_index, 4);
+		if (cfg->sbr_sr_index == 0x0F) {
+			gf_bs_write_int(bs, cfg->sbr_sr, 24);
+		}
+		gf_bs_write_int(bs, cfg->sbr_object_type, 5);
+	}
+
+	/*object cfg*/
+	switch (cfg->base_object_type) {
+	case 1:
+	case 2:
+	case 3:
+	case 4:
+	case 6:
+	case 7:
+	case 17:
+	case 19:
+	case 20:
+	case 21:
+	case 22:
+	case 23:
+	{
+		/*frame length flag*/
+		gf_bs_write_int(bs, 0, 1);
+		/*depends on core coder*/
+		gf_bs_write_int(bs, 0, 1);
+		/*ext flag*/
+		gf_bs_write_int(bs, 0, 1);
+		if ((cfg->base_object_type == 6) || (cfg->base_object_type == 20)) {
+			gf_bs_write_int(bs, 0, 3);
+		}
+	}
+		break;
+	}
+	/*ER cfg - not supported*/
+
+	/*implicit sbr - not used yet*/
+	if (0 && (cfg->base_object_type != 5) ) {
+		gf_bs_write_int(bs, 0x2b7, 11);
+		cfg->sbr_object_type = gf_bs_read_int(bs, 5);
+		cfg->has_sbr = gf_bs_read_int(bs, 1);
+		if (cfg->has_sbr) {
+			cfg->sbr_sr_index = gf_bs_read_int(bs, 4);
+			if (cfg->sbr_sr_index == 0x0F) {
+				cfg->sbr_sr = gf_bs_read_int(bs, 24);
+			} else {
+				cfg->sbr_sr = GF_M4ASampleRates[cfg->sbr_sr_index];
+			}
+		}
+	}
+	gf_bs_get_content(bs, dsi, dsi_size);
 	gf_bs_del(bs);
 	return GF_OK;
 }
@@ -1567,11 +1728,13 @@ static s32 avc_parse_slice(GF_BitStream *bs, AVCState *avc, AVCSliceInfo *si)
     si->frame_num = gf_bs_read_int(bs, si->sps->log2_max_frame_num);
 
 	si->field_pic_flag = 0;
+	si->bottom_field_flag = 0;
     if (si->sps->frame_mbs_only_flag) {
         /*s->picture_structure= PICT_FRAME;*/
     } else {
 		si->field_pic_flag = gf_bs_read_int(bs, 1);
-        if (si->field_pic_flag) si->bottom_field_flag = gf_bs_read_int(bs, 1);
+        if (si->field_pic_flag)
+			si->bottom_field_flag = gf_bs_read_int(bs, 1);
     }
 	if (si->nal_unit_type==GF_AVC_NALU_IDR_SLICE)
 		si->idr_pic_id = avc_get_ue(bs);
@@ -1731,6 +1894,7 @@ s32 AVC_ParseNALU(GF_BitStream *bs, u32 nal_hdr, AVCState *avc)
 			break;
 		}
 		if (avc->s_info.frame_num != n_state.frame_num) { ret = 1; break; }
+
 		if (avc->s_info.field_pic_flag != n_state.field_pic_flag) { ret = 1; break; }
 		if ((avc->s_info.nal_ref_idc != n_state.nal_ref_idc) &&
 			(!avc->s_info.nal_ref_idc || !n_state.nal_ref_idc)) {
@@ -1741,7 +1905,10 @@ s32 AVC_ParseNALU(GF_BitStream *bs, u32 nal_hdr, AVCState *avc)
 
 		if (avc->s_info.sps->poc_type == n_state.sps->poc_type) {
 			if (!avc->s_info.sps->poc_type) {
-				if (avc->s_info.poc_lsb != n_state.poc_lsb) { ret = 1; break; }
+				if (!n_state.bottom_field_flag && (avc->s_info.poc_lsb != n_state.poc_lsb)) { 
+					ret = 1; 
+					break;
+				}
 				if (avc->s_info.delta_poc_bottom != n_state.delta_poc_bottom) {
 					ret = 1;
 					break;
@@ -1765,6 +1932,9 @@ s32 AVC_ParseNALU(GF_BitStream *bs, u32 nal_hdr, AVCState *avc)
 			}
 		}
 		break;
+	case GF_AVC_NALU_SEQ_PARAM:
+	case GF_AVC_NALU_PIC_PARAM:
+		return 0;
 	default:
 		if (avc->s_info.nal_unit_type <= GF_AVC_NALU_IDR_SLICE) ret = 1;
 		else ret = 0;
@@ -1865,6 +2035,8 @@ u32 AVC_ReformatSEI_NALU(char *buffer, u32 nal_size, AVCState *avc)
 		/*fix payload size due to emulation prevention bytes*/
 		size_fix = 0;
 		num_zero = 0;
+		if (psize%255 == 0) num_zero = 1;
+
 		for (i = 0; i < psize + size_fix; i++)
 		{
 			if (!buffer[start + i])
@@ -1885,7 +2057,7 @@ u32 AVC_ReformatSEI_NALU(char *buffer, u32 nal_size, AVCState *avc)
 			while (var>=255) { new_buffer[written] = (char) 0xff; written++; var-=255;}
 			new_buffer[written] = (char) var; written++;
 			memcpy(new_buffer+written, buffer+start, sizeof(char)* (psize + size_fix));
-			written += psize;
+			written += psize + size_fix;
 		}
 
 		gf_bs_skip_bytes(bs, (u64) (psize + size_fix));
@@ -2036,12 +2208,6 @@ const char *gf_avc_get_profile_name(u8 video_prof)
 	}
 }
 
-
-#ifndef GPAC_READ_ONLY
-/*
-	AC3 parser
-	taken from MPEG4IP - MPL
-*/
 static u32 AC3_FindSyncCode(u8 *buf, u32 buflen)
 {
 	u32 end = buflen - 6;
@@ -2055,6 +2221,26 @@ static u32 AC3_FindSyncCode(u8 *buf, u32 buflen)
 	return buflen;
 }
 
+
+static Bool AC3_FindSyncCodeBS(GF_BitStream *bs)
+{
+	u8 b1;
+	u32 pos = (u32) gf_bs_get_position(bs);
+	u32 end = (u32) gf_bs_get_size(bs) - 6;
+
+	pos += 1;
+	b1 = gf_bs_read_u8(bs);
+	while (pos <= end) {
+		u8 b2 = gf_bs_read_u8(bs);
+		if ((b1 == 0x0b) && (b2==0x77)) {
+			gf_bs_seek(bs, pos-1);
+			return 1;
+		}
+		pos++;
+	}
+	return 0;
+}
+
 static const u32 ac3_sizecod_to_bitrate[] = {
   32000, 40000, 48000, 56000, 64000, 80000, 96000,
   112000, 128000, 160000, 192000, 224000, 256000,
@@ -2079,7 +2265,7 @@ static const u32 ac3_mod_to_chans[] = {
   2, 1, 2, 3, 3, 4, 4, 5
 };
 
-Bool gf_ac3_parser(u8 *buf, u32 buflen, u32 *pos, GF_AC3Header *hdr)
+Bool gf_ac3_parser(u8 *buf, u32 buflen, u32 *pos, GF_AC3Header *hdr, Bool full_parse)
 {
 	u32 fscod, frmsizecod, bsid, ac3_mod, freq, framesize;
 	if (buflen < 6) return 0;
@@ -2093,7 +2279,7 @@ Bool gf_ac3_parser(u8 *buf, u32 buflen, u32 *pos, GF_AC3Header *hdr)
 	ac3_mod = (buf[6] >> 5) & 0x7;
 	if (bsid >= 12) return 0;
 
-	if (hdr) memset(hdr, 0, sizeof(GF_AC3Header));
+	if (full_parse && hdr) memset(hdr, 0, sizeof(GF_AC3Header));
 	
 	if (hdr) {
 		hdr->bitrate = ac3_sizecod_to_bitrate[frmsizecod / 2];
@@ -2131,7 +2317,70 @@ Bool gf_ac3_parser(u8 *buf, u32 buflen, u32 *pos, GF_AC3Header *hdr)
 	return 1;
 }
 
-#endif
+
+Bool gf_ac3_parser_bs(GF_BitStream *bs, GF_AC3Header *hdr, Bool full_parse)
+{
+	u32 fscod, frmsizecod, bsid, ac3_mod, freq, framesize, pos, bsmod;
+	if (!hdr || (gf_bs_available(bs) < 6)) return 0;
+	if (!AC3_FindSyncCodeBS(bs)) return 0;
+
+	pos = (u32) gf_bs_get_position(bs);
+	gf_bs_read_u32(bs);
+	fscod = gf_bs_read_int(bs, 2);
+	frmsizecod = gf_bs_read_int(bs, 6);
+	bsid = gf_bs_read_int(bs, 5);
+	bsmod = gf_bs_read_int(bs, 3);
+	ac3_mod = gf_bs_read_int(bs, 3);
+
+	if (bsid >= 12) return 0;
+
+	//memset(hdr, 0, sizeof(GF_AC3Header));
+	
+	hdr->bitrate = ac3_sizecod_to_bitrate[frmsizecod / 2];
+	if (bsid > 8) hdr->bitrate = hdr->bitrate >> (bsid - 8);
+
+	switch (fscod) {
+	case 0:
+		freq = 48000;
+		framesize = ac3_sizecod0_to_framesize[frmsizecod / 2] * 2;
+		break;
+	case 1:
+		freq = 44100;
+		framesize = (ac3_sizecod1_to_framesize[frmsizecod / 2] + (frmsizecod & 0x1)) * 2;
+		break;
+	case 2:
+		freq = 32000;
+		framesize = ac3_sizecod2_to_framesize[frmsizecod / 2] * 2;
+		break;
+	default:
+	    return 0;
+	}
+	hdr->sample_rate = freq;
+	hdr->framesize = framesize;
+
+	if (full_parse) {
+		hdr->bsid = bsid;
+		hdr->bsmod = bsmod;
+		hdr->acmod = ac3_mod;
+		hdr->lfon = 0;
+		hdr->fscod = fscod;
+		hdr->brcode = hdr->bitrate/1000;
+	}
+
+	hdr->channels = ac3_mod_to_chans[ac3_mod];
+	if ((ac3_mod & 0x1) && (ac3_mod != 1)) gf_bs_read_int(bs, 2);
+	if (ac3_mod & 0x4) gf_bs_read_int(bs, 2);
+	if (ac3_mod == 0x2) gf_bs_read_int(bs, 2);
+	/*LFEon*/
+	if (gf_bs_read_int(bs, 1)) {
+		hdr->channels += 1;
+		hdr->lfon = 1;
+	}
+
+
+	gf_bs_seek(bs, pos);
+	return 1;
+}
 
 
 
@@ -2385,135 +2634,3 @@ u32 gf_vorbis_check_frame(GF_VorbisParser *vp, char *data, u32 data_length)
 	if (block_size == -1) return 0;
 	return ((vp->mode_flag[block_size]) ? vp->max_block : vp->min_block) / (2);
 }
-
-
-
-GF_EXPORT
-void gf_img_parse(GF_BitStream *bs, u8 *OTI, u32 *mtype, u32 *width, u32 *height, char **dsi, u32 *dsi_len)
-{
-	u8 b1, b2, b3;
-	u32 size, type;
-	u64 pos;
-	pos = gf_bs_get_position(bs);
-	gf_bs_seek(bs, 0);
-	
-	*mtype = *width = *height = 0;
-	*OTI = 0;
-	if (dsi) {
-		*dsi = NULL;
-		*dsi_len = 0;
-	}
-
-	b1 = gf_bs_read_u8(bs);
-	b2 = gf_bs_read_u8(bs);
-	b3 = gf_bs_read_u8(bs);
-	/*JPEG*/
-	if ((b1==0xFF) && (b2==0xD8) && (b3==0xFF)) {
-		gf_bs_read_u8(bs);
-		/*get frame header FFC0*/
-		while (gf_bs_available(bs)) {
-			u32 type, w, h;
-			if (gf_bs_read_u8(bs) != 0xFF) continue;
-			type = gf_bs_read_u8(bs);
-			switch (type) {
-			case 0xC0:
-			case 0xC1:
-			case 0xC2:
-				gf_bs_skip_bytes(bs, 3);
-				h = gf_bs_read_int(bs, 16);
-				w = gf_bs_read_int(bs, 16);
-				if ((w > *width) || (h > *height)) {
-					*width = w;
-					*height = h;
-				}
-				break;
-			case 0xD0:
-			case 0xD1:
-			case 0xD2:
-			case 0xD3:
-			case 0xD4:
-			case 0xD5:
-			case 0xD6:
-			case 0xD7:
-				break;
-			}
-		}
-		*OTI = 0x6C;
-		*mtype = GF_4CC('j','p','e','g');
-	}
-	/*PNG*/
-	else if ((b1==0x89) && (b2==0x50) && (b3==0x4E)) {
-		/*check for PNG sig*/
-		if ( (gf_bs_read_u8(bs) != 0x47) || (gf_bs_read_u8(bs) != 0x0D) || (gf_bs_read_u8(bs) != 0x0A) 
-			|| (gf_bs_read_u8(bs) != 0x1A) || (gf_bs_read_u8(bs) != 0x0A) ) goto exit;
-		gf_bs_read_u32(bs);
-		/*check for PNG IHDR*/
-		if ( (gf_bs_read_u8(bs) != 'I') || (gf_bs_read_u8(bs) != 'H') 
-			|| (gf_bs_read_u8(bs) != 'D') || (gf_bs_read_u8(bs) != 'R')) goto exit;
-
-		*width = gf_bs_read_u32(bs);
-		*height = gf_bs_read_u32(bs);
-		*OTI = 0x6D;
-		*mtype = GF_4CC('p','n','g',' ');
-	}
-	size = gf_bs_read_u8(bs);
-	type = gf_bs_read_u32(bs);
-	if ( ((size==12) && (type==GF_4CC('j','P',' ',' ') ))
-		|| (type==GF_4CC('j','p','2','h') ) ) {
-
-		if (type==GF_4CC('j','p','2','h')) {
-			*OTI = 0x6E;
-			*mtype = GF_4CC('j','p','2',' ');
-			goto j2k_restart;
-		}
-
-		type = gf_bs_read_u32(bs);
-		if (type!=0x0D0A870A) goto exit;
-
-		*OTI = 0x6E;
-		*mtype = GF_4CC('j','p','2',' ');
-
-		while (gf_bs_available(bs)) {
-j2k_restart:
-			size = gf_bs_read_u32(bs);
-			type = gf_bs_read_u32(bs);
-			switch (type) {
-			case GF_4CC('j','p','2','h'):
-				goto j2k_restart;
-			case GF_4CC('i','h','d','r'):
-			{
-				u16 nb_comp;
-				u8 BPC, C, UnkC, IPR;
-				*height = gf_bs_read_u32(bs);
-				*width = gf_bs_read_u32(bs);
-				nb_comp = gf_bs_read_u16(bs);
-				BPC = gf_bs_read_u8(bs);
-				C = gf_bs_read_u8(bs);
-				UnkC = gf_bs_read_u8(bs);
-				IPR = gf_bs_read_u8(bs);
-
-				if (dsi) {
-					GF_BitStream *bs_dsi = gf_bs_new(NULL, 0, GF_BITSTREAM_WRITE);
-					gf_bs_write_u32(bs_dsi, *height);
-					gf_bs_write_u32(bs_dsi, *width);
-					gf_bs_write_u16(bs_dsi, nb_comp);
-					gf_bs_write_u8(bs_dsi, BPC);
-					gf_bs_write_u8(bs_dsi, C);
-					gf_bs_write_u8(bs_dsi, UnkC);
-					gf_bs_write_u8(bs_dsi, IPR);
-					gf_bs_get_content(bs_dsi, dsi, dsi_len);
-					gf_bs_del(bs_dsi);
-				}
-				goto exit;
-			}
-				break;
-			default:
-				gf_bs_skip_bytes(bs, size-8);
-				break;
-			}
-		}
-	}
-
-exit:
-	gf_bs_seek(bs, pos);
-}
