@@ -25,12 +25,14 @@
  *
  */
 
+#include <gpac/setup.h>
 #include <gpac/internal/avilib.h>
 
 #ifndef GPAC_READ_ONLY
 
+
 #define PACKAGE "GPAC/avilib"
-#define VERSION GPAC_VERSION
+#define VERSION GPAC_FULL_VERSION
 
 #define INFO_LIST
 
@@ -1795,10 +1797,8 @@ avi_t *AVI_open_input_file(char *filename, int getIndex)
       AVI->aptr=0; //reset  
   }
 
-  if (AVI_errno) {
-      free(AVI);
-      return NULL;
-  }
+  if (AVI_errno) return NULL;
+
   return AVI;
 }
 
