@@ -423,6 +423,12 @@ GP_RTPPacketizer *gf_rtp_packetizer_create_and_init_from_file(GF_ISOFile *file,
 			has_mpeg4_mapping = 0;
 			nb_ch = 1;
 			break;
+		case GF_ISOM_SUBTYPE_AC3:
+			hintType = GF_RTP_PAYT_AC3;
+			streamType = GF_STREAM_AUDIO;
+			has_mpeg4_mapping = 1;
+			nb_ch = 1;
+			break;
 		case GF_ISOM_SUBTYPE_AVC_H264:
 		{
 			GF_AVCConfig *avcc = gf_isom_avc_config_get(file, TrackNum, 1);
@@ -775,6 +781,15 @@ GF_RTPHinter *gf_hinter_track_new(GF_ISOFile *file, u32 TrackNum,
 			oti = (TrackMediaSubType==GF_ISOM_SUBTYPE_3GP_EVRC) ? 0xA0 : 0xA1;
 			nb_ch = 1;
 			break;
+		case GF_ISOM_SUBTYPE_3GP_DIMS:
+			hintType = GF_RTP_PAYT_3GPP_DIMS;
+			streamType = GF_STREAM_SCENE;
+			break;
+		case GF_ISOM_SUBTYPE_AC3:
+			hintType = GF_RTP_PAYT_AC3;
+			streamType = GF_STREAM_AUDIO;
+			gf_isom_get_audio_info(file, TrackNum, 1, NULL, &nb_ch, NULL);
+			break;
 		default:
 			/*ERROR*/
 			hintType = 0;
@@ -942,6 +957,7 @@ void gf_hinter_track_del(GF_RTPHinter *tkHinter)
 GF_EXPORT
 GF_Err gf_hinter_track_process(GF_RTPHinter *tkHint)
 {
+	GF_Err e;
 	u32 i, descIndex, duration;
 	u64 ts;
 	u8 PadBits;
@@ -953,7 +969,8 @@ GF_Err gf_hinter_track_process(GF_RTPHinter *tkHint)
 	tkHint->TotalSample = gf_isom_get_sample_count(tkHint->file, tkHint->TrackNum);
 	ft = tkHint->rtp_p->sl_config.timestampResolution;
 	ft /= tkHint->OrigTimeScale;
-
+	
+	e = GF_OK;
 	for (i=0; i<tkHint->TotalSample; i++) {
 		samp = gf_isom_get_sample(tkHint->file, tkHint->TrackNum, i+1, &descIndex);
 		if (!samp) return GF_IO_ERR;
@@ -1022,12 +1039,12 @@ GF_Err gf_hinter_track_process(GF_RTPHinter *tkHint)
 				tkHint->base_offset_in_sample = samp->dataLength-remain;
 				remain -= size;
 				tkHint->rtp_p->sl_header.accessUnitEndFlag = remain ? 0 : 1;
-				gf_rtp_builder_process(tkHint->rtp_p, ptr, size, (u8) !remain, samp->dataLength, duration, (u8) (descIndex + SIDX_OFFSET_3GPP) );
+				e = gf_rtp_builder_process(tkHint->rtp_p, ptr, size, (u8) !remain, samp->dataLength, duration, (u8) (descIndex + SIDX_OFFSET_3GPP) );
 				ptr += size;
 				tkHint->rtp_p->sl_header.accessUnitStartFlag = 0;
 			}
 		} else {
-			gf_rtp_builder_process(tkHint->rtp_p, samp->data, samp->dataLength, 1, samp->dataLength, duration, (u8) (descIndex + SIDX_OFFSET_3GPP) );
+			e = gf_rtp_builder_process(tkHint->rtp_p, samp->data, samp->dataLength, 1, samp->dataLength, duration, (u8) (descIndex + SIDX_OFFSET_3GPP) );
 		}
 		tkHint->rtp_p->sl_header.packetSequenceNumber += 1;
 
@@ -1036,6 +1053,8 @@ GF_Err gf_hinter_track_process(GF_RTPHinter *tkHint)
 
 		tkHint->rtp_p->sl_header.AU_sequenceNumber += 1;
 		gf_isom_sample_del(&samp);
+
+		if (e) return e;
 	}
 
 	//flush
@@ -1228,6 +1247,37 @@ GF_Err gf_hinter_track_finalize(GF_RTPHinter *tkHint, Bool AddSystemInfo)
 		gf_isom_sdp_add_track_line(tkHint->file, tkHint->HintTrack, sdpLine); 
 		free(config_bytes); 
 	}
+	/*3GPP DIMS*/
+	else if (tkHint->rtp_p->rtp_payt==GF_RTP_PAYT_3GPP_DIMS) { 
+		GF_DIMSDescription dims;
+		char fmt[200];
+		gf_isom_get_visual_info(tkHint->file, tkHint->TrackNum, 1, &Width, &Height);
+
+		gf_isom_get_dims_description(tkHint->file, tkHint->TrackNum, 1, &dims);
+		sprintf(sdpLine, "a=fmtp:%d Version-profile=%d", tkHint->rtp_p->PayloadType, dims.profile);
+		if (! dims.fullRequestHost) {
+			strcat(sdpLine, ";useFullRequestHost=0");
+			sprintf(fmt, ";pathComponents=%d", dims.pathComponents);
+			strcat(sdpLine, fmt);
+		}
+		if (!dims.streamType) strcat(sdpLine, ";stream-type=secondary");
+		if (dims.containsRedundant == 1) strcat(sdpLine, ";contains-redundant=main");
+		else if (dims.containsRedundant == 2) strcat(sdpLine, ";contains-redundant=redundant");
+
+		if (dims.textEncoding && strlen(dims.textEncoding)) {
+			strcat(sdpLine, ";text-encoding=");
+			strcat(sdpLine, dims.textEncoding);
+		}
+		if (dims.contentEncoding && strlen(dims.contentEncoding)) {
+			strcat(sdpLine, ";content-coding=");
+			strcat(sdpLine, dims.contentEncoding);
+		}
+		if (dims.content_script_types && strlen(dims.content_script_types) ) {
+			strcat(sdpLine, ";content-script-types=");
+			strcat(sdpLine, dims.contentEncoding);
+		}
+		gf_isom_sdp_add_track_line(tkHint->file, tkHint->HintTrack, sdpLine);
+	}
 	/*extensions for some mobile phones*/
 	if (Width && Height) {
 		sprintf(sdpLine, "a=framesize:%d %d-%d", tkHint->rtp_p->PayloadType, Width, Height);
@@ -1283,7 +1333,7 @@ GF_Err gf_hinter_finalize(GF_ISOFile *file, u32 IOD_Profile, u32 bandwidth)
 		gf_isom_sdp_add_line(file, buf64);
 	}
 	//xtended attribute for copyright
-	sprintf(buf64, "a=x-copyright: %s", "MP4/3GP File hinted with GPAC " GPAC_VERSION " (C)2000-2005 - http://gpac.sourceforge.net");
+	sprintf(buf64, "a=x-copyright: %s", "MP4/3GP File hinted with GPAC " GPAC_FULL_VERSION " (C)2000-2005 - http://gpac.sourceforge.net");
 	gf_isom_sdp_add_line(file, buf64);
 
 	if (IOD_Profile == GF_SDP_IOD_NONE) return GF_OK;
@@ -1411,7 +1461,7 @@ GF_Err gf_hinter_finalize(GF_ISOFile *file, u32 IOD_Profile, u32 bandwidth)
 			}
 			/*only 1 MPEG-4 visual max and 1 MPEG-4 audio max for ISMA compliancy*/
 			if (!has_v && !has_a && (has_i_v<=1) && (has_i_a<=1)) {
-				sprintf(sdpLine, "a=isma-compliance=1,1.0,1");
+				sprintf(sdpLine, "a=isma-compliance:1,1.0,1");
 				gf_isom_sdp_add_line(file, sdpLine);
 			}
 		}
