@@ -32,6 +32,9 @@
 #include <gpac/internal/avilib.h>
 #include <gpac/internal/ogg.h>
 #include <gpac/internal/vobsub.h>
+#include <zlib.h>
+
+GF_Err gf_media_export_nhml(GF_MediaExporter *dumper, Bool dims_doc);
 
 static GF_Err gf_export_message(GF_MediaExporter *dumper, GF_Err e, char *format, ...)
 {
@@ -273,7 +276,7 @@ GF_Err gf_media_export_samples(GF_MediaExporter *dumper)
 				strcpy(szEXT, ".jp2");
 				gf_export_message(dumper, GF_OK, "Dumping JPEG 2000 image%s", szNum);
 				break;
-			case GPAC_OGG_MEDIA_OTI:
+			case GPAC_OTI_MEDIA_OGG:
 				strcpy(szEXT, ".theo");
 				gf_export_message(dumper, GF_OK, "Dumping Theora video sample%s", szNum);
 				break;
@@ -297,7 +300,7 @@ GF_Err gf_media_export_samples(GF_MediaExporter *dumper)
 				strcpy(szEXT, ".mp3");
 				gf_export_message(dumper, GF_OK, "Dumping MPEG-1/2 Audio (MP3) sample%s", szNum);
 				break;
-			case GPAC_OGG_MEDIA_OTI:
+			case GPAC_OTI_MEDIA_OGG:
 				strcpy(szEXT, ".vorb");
 				gf_export_message(dumper, GF_OK, "Dumping Vorbis audio sample%s", szNum);
 				break;
@@ -339,6 +342,12 @@ GF_Err gf_media_export_samples(GF_MediaExporter *dumper)
 	} else if (m_stype==GF_ISOM_SUBTYPE_3GP_H263) {
 		gf_export_message(dumper, GF_OK, "Extracting H263 Video sample%s", szNum);
 		strcpy(szEXT, ".263");
+	} else if (m_stype==GF_ISOM_SUBTYPE_3GP_DIMS) {
+		gf_export_message(dumper, GF_OK, "Extracting DIMS sample%s", szNum);
+		strcpy(szEXT, ".dims");
+	} else if (m_stype==GF_ISOM_SUBTYPE_AC3) {
+		gf_export_message(dumper, GF_OK, "Extracting AC3 sample%s", szNum);
+		strcpy(szEXT, ".ac3");
 	} else if (m_stype==GF_ISOM_SUBTYPE_AVC_H264) {
 		strcpy(szEXT, ".h264");
 		gf_export_message(dumper, GF_OK, "Dumping MPEG-4 AVC-H264 Visual sample%s", szNum);
@@ -456,9 +465,9 @@ static GF_Err gf_dump_to_vobsub(GF_MediaExporter *dumper, char *szName, u32 trac
 	for (i = 0; i < 16; i++) {
 		s32 y, u, v, r, g, b;
 
-		y = (s32)dsi[(i<<2)+1] - 0x10;
-		u = (s32)dsi[(i<<2)+3] - 0x80;
-		v = (s32)dsi[(i<<2)+2] - 0x80;
+		y = (s32)(u8)dsi[(i<<2)+1] - 0x10;
+		u = (s32)(u8)dsi[(i<<2)+3] - 0x80;
+		v = (s32)(u8)dsi[(i<<2)+2] - 0x80;
 		r = (298 * y           + 409 * v + 128) >> 8;
 		g = (298 * y - 100 * u - 208 * v + 128) >> 8;
 		b = (298 * y + 516 * u           + 128) >> 8;
@@ -491,7 +500,7 @@ static GF_Err gf_dump_to_vobsub(GF_MediaExporter *dumper, char *szName, u32 trac
 		u64 dts;
 		u32 hh, mm, ss, ms;
 
-		samp = gf_isom_get_sample(dumper->file, track, i+1, &di);
+		samp = gf_isom_get_sample(dumper->file, track, i, &di);
 		if (!samp) {
 			break;
 		}
@@ -607,7 +616,7 @@ GF_Err gf_media_export_native(GF_MediaExporter *dumper)
 				strcat(szName, ".png");
 				gf_export_message(dumper, GF_OK, "Extracting PNG image");
 				break;
-			case GPAC_OGG_MEDIA_OTI:
+			case GPAC_OTI_MEDIA_OGG:
 				strcat(szName, ".ogg");
 				gf_export_message(dumper, GF_OK, "Extracting Ogg video");
 				is_ogg = 1;
@@ -643,7 +652,7 @@ GF_Err gf_media_export_native(GF_MediaExporter *dumper)
 				strcat(szName, ".mp3");
 				gf_export_message(dumper, GF_OK, "Extracting MPEG-1/2 Audio (MP3)");
 				break;
-			case GPAC_OGG_MEDIA_OTI:
+			case GPAC_OTI_MEDIA_OGG:
 				strcat(szName, ".ogg");
 				is_ogg = 1;
 				gf_export_message(dumper, GF_OK, "Extracting Ogg audio");
@@ -720,6 +729,8 @@ GF_Err gf_media_export_native(GF_MediaExporter *dumper)
 		} else if (m_stype==GF_ISOM_SUBTYPE_3GP_H263) {
 			gf_export_message(dumper, GF_OK, "Extracting H263 Video");
 			strcat(szName, ".263");
+		} else if (m_stype==GF_ISOM_SUBTYPE_3GP_DIMS) {
+			return gf_media_export_nhml(dumper, 1);
 		} else if (m_stype==GF_ISOM_SUBTYPE_AVC_H264) {
 			avccfg = gf_isom_avc_config_get(dumper->file, track, 1);
 			strcat(szName, ".h264");
@@ -727,6 +738,9 @@ GF_Err gf_media_export_native(GF_MediaExporter *dumper)
 		} else if (m_type==GF_ISOM_MEDIA_FLASH) {
 			gf_export_message(dumper, GF_OK, "Extracting Macromedia Flash Movie");
 			strcat(szName, ".swf");
+		} else if (m_stype==GF_ISOM_SUBTYPE_AC3) {
+			gf_export_message(dumper, GF_OK, "Extracting AC3 Audio");
+			strcat(szName, ".ac3");
 		} else {
 			strcat(szName, ".");
 			strcat(szName, gf_4cc_to_str(m_stype));
@@ -1498,13 +1512,16 @@ GF_Err gf_media_export_avi(GF_MediaExporter *dumper)
 	return GF_OK;
 }
 
-GF_Err gf_media_export_nhml(GF_MediaExporter *dumper)
+GF_Err gf_media_export_nhml(GF_MediaExporter *dumper, Bool dims_doc)
 {
 	GF_ESD *esd;
 	char szName[1000], szMedia[1000];
 	FILE *med, *inf, *nhml;
 	Bool full_dump;
-	u32 track, i, di, count, pos;
+	u32 w, h;
+	Bool uncompress;
+	u32 track, i, di, count, pos, mstype;
+	const char *szRootName, *szSampleName;
 
 	track = gf_isom_get_track_by_id(dumper->file, dumper->trackID);
 	if (!track) return gf_export_message(dumper, GF_BAD_PARAM, "Invalid track ID %d", dumper->trackID);
@@ -1515,15 +1532,23 @@ GF_Err gf_media_export_nhml(GF_MediaExporter *dumper)
 	}
 	esd = gf_isom_get_esd(dumper->file, track, 1);
 	full_dump = (dumper->flags & GF_EXPORT_NHML_FULL) ? 1 : 0;
+	med = NULL;
+	if (dims_doc) {
+		sprintf(szName, "%s.dml", dumper->out_name);
+		szRootName = "DIMSStream";
+		szSampleName = "DIMSUnit";
+	} else {
+		sprintf(szMedia, "%s.media", dumper->out_name);
+		med = gf_f64_open(szMedia, "wb");
+		if (!med) {
+			if (esd) gf_odf_desc_del((GF_Descriptor *) esd);
+			return gf_export_message(dumper, GF_IO_ERR, "Error opening %s for writing - check disk access & permissions", szMedia);
+		}
 
-	sprintf(szMedia, "%s.media", dumper->out_name);
-	med = gf_f64_open(szMedia, "wb");
-	if (!med) {
-		if (esd) gf_odf_desc_del((GF_Descriptor *) esd);
-		return gf_export_message(dumper, GF_IO_ERR, "Error opening %s for writing - check disk access & permissions", szMedia);
+		sprintf(szName, "%s.nhml", dumper->out_name);
+		szRootName = "NHNTStream";
+		szSampleName = "NHNTSample";
 	}
-
-	sprintf(szName, "%s.nhml", dumper->out_name);
 	nhml = fopen(szName, "wt");
 	if (!nhml) {
 		fclose(med);
@@ -1531,9 +1556,11 @@ GF_Err gf_media_export_nhml(GF_MediaExporter *dumper)
 		return gf_export_message(dumper, GF_IO_ERR, "Error opening %s for writing - check disk access & permissions", szName);
 	}
 
+	mstype = gf_isom_get_media_subtype(dumper->file, track, 1);
+
 	/*write header*/
 	fprintf(nhml, "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n");
-	fprintf(nhml, "<NHNTStream version=\"1.0\" timeScale=\"%d\" ", gf_isom_get_media_timescale(dumper->file, track) );
+	fprintf(nhml, "<%s version=\"1.0\" timeScale=\"%d\" ", szRootName, gf_isom_get_media_timescale(dumper->file, track) );
 	if (esd) {
 		fprintf(nhml, "streamType=\"%d\" objectTypeIndication=\"%d\" ", esd->decoderConfig->streamType, esd->decoderConfig->objectTypeIndication);
 		if (esd->decoderConfig->decoderSpecificInfo  && esd->decoderConfig->decoderSpecificInfo->data) {
@@ -1546,7 +1573,6 @@ GF_Err gf_media_export_nhml(GF_MediaExporter *dumper)
 		gf_odf_desc_del((GF_Descriptor *) esd);
 
 		if (gf_isom_get_media_type(dumper->file, track)==GF_ISOM_MEDIA_VISUAL) {
-			u32 w, h;
 			gf_isom_get_visual_info(dumper->file, track, 1, &w, &h);
 			fprintf(nhml, "width=\"%d\" height=\"%d\" ", w, h);
 		}
@@ -1556,33 +1582,59 @@ GF_Err gf_media_export_nhml(GF_MediaExporter *dumper)
 			gf_isom_get_audio_info(dumper->file, track, 1, &sr, &nb_ch, &bps);
 			fprintf(nhml, "sampleRate=\"%d\" numChannels=\"%d\" ", sr, nb_ch);
 		}
-
-	} else {
+	} else if (!dims_doc) {
 		GF_GenericSampleDescription *sdesc = gf_isom_get_generic_sample_description(dumper->file, track, 1);
 		u32 mtype = gf_isom_get_media_type(dumper->file, track);
-		fprintf(nhml, "mediaType=\"%s\" mediaSubType=\"%s\" ", gf_4cc_to_str(mtype), gf_4cc_to_str(sdesc->codec_tag));
-		if (mtype==GF_ISOM_MEDIA_VISUAL) {
-			fprintf(nhml, "codecVendor=\"%s\" codecVersion=\"%d\" codecRevision=\"%d\" ", gf_4cc_to_str(sdesc->vendor_code), sdesc->version, sdesc->revision);
-			fprintf(nhml, "width=\"%d\" height=\"%d\" compressorName=\"%s\" temporalQuality=\"%d\" spatialQuality=\"%d\" horizontalResolution=\"%d\" verticalResolution=\"%d\" bitDepth=\"%d\" ",
-				sdesc->width, sdesc->height, sdesc->compressor_name, sdesc->temporal_quality, sdesc->spacial_quality, sdesc->h_res, sdesc->v_res, sdesc->depth);
-		} else if (mtype==GF_ISOM_MEDIA_AUDIO) {
-			fprintf(nhml, "codecVendor=\"%s\" codecVersion=\"%d\" codecRevision=\"%d\" ", gf_4cc_to_str(sdesc->vendor_code), sdesc->version, sdesc->revision);
-			fprintf(nhml, "sampleRate=\"%d\" numChannels=\"%d\" bitsPerSample=\"%d\" ", sdesc->samplerate, sdesc->nb_channels, sdesc->bits_per_sample);
+		fprintf(nhml, "mediaType=\"%s\" ", gf_4cc_to_str(mtype));
+		fprintf(nhml, "mediaSubType=\"%s\" ", gf_4cc_to_str(mstype ));
+		if (sdesc) {
+			if (mtype==GF_ISOM_MEDIA_VISUAL) {
+				fprintf(nhml, "codecVendor=\"%s\" codecVersion=\"%d\" codecRevision=\"%d\" ", gf_4cc_to_str(sdesc->vendor_code), sdesc->version, sdesc->revision);
+				fprintf(nhml, "width=\"%d\" height=\"%d\" compressorName=\"%s\" temporalQuality=\"%d\" spatialQuality=\"%d\" horizontalResolution=\"%d\" verticalResolution=\"%d\" bitDepth=\"%d\" ",
+					sdesc->width, sdesc->height, sdesc->compressor_name, sdesc->temporal_quality, sdesc->spacial_quality, sdesc->h_res, sdesc->v_res, sdesc->depth);
+			} else if (mtype==GF_ISOM_MEDIA_AUDIO) {
+				fprintf(nhml, "codecVendor=\"%s\" codecVersion=\"%d\" codecRevision=\"%d\" ", gf_4cc_to_str(sdesc->vendor_code), sdesc->version, sdesc->revision);
+				fprintf(nhml, "sampleRate=\"%d\" numChannels=\"%d\" bitsPerSample=\"%d\" ", sdesc->samplerate, sdesc->nb_channels, sdesc->bits_per_sample);
+			}
+			if (sdesc->extension_buf) {
+				sprintf(szName, "%s.info", dumper->out_name);
+				inf = fopen(szName, "wb");
+				if (inf) fwrite(sdesc->extension_buf, sdesc->extension_buf_size, 1, inf);
+				fclose(inf);
+				fprintf(nhml, "specificInfoFile=\"%s\" ", szName);
+				free(sdesc->extension_buf);
+			}
+			free(sdesc);
 		}
-		if (sdesc->extension_buf) {
-			sprintf(szName, "%s.info", dumper->out_name);
-			inf = fopen(szName, "wb");
-			if (inf) fwrite(sdesc->extension_buf, sdesc->extension_buf_size, 1, inf);
-			fclose(inf);
-			fprintf(nhml, "specificInfoFile=\"%s\" ", szName);
-			free(sdesc->extension_buf);
-		}
-		free(sdesc);
 	}
 
-	fprintf(nhml, "baseMediaFile=\"%s\" ", szMedia);
 	if (gf_isom_is_track_in_root_od(dumper->file, track)) fprintf(nhml, "inRootOD=\"yes\" ");
 	fprintf(nhml, "trackID=\"%d\" ", dumper->trackID);
+	
+	uncompress = 0;
+
+	if (mstype == GF_ISOM_MEDIA_DIMS) {
+		GF_DIMSDescription dims;
+
+		fprintf(nhml, "xmlns=\"http://www.3gpp.org/richmedia\" ");
+		gf_isom_get_visual_info(dumper->file, track, 1, &w, &h);
+		fprintf(nhml, "width=\"%d\" height=\"%d\" ", w, h);
+
+		gf_isom_get_dims_description(dumper->file, track, 1, &dims);
+		fprintf(nhml, "profile=\"%d\" level=\"%d\" pathComponents=\"%d\" ", dims.profile, dims.level, dims.pathComponents);
+		fprintf(nhml, "useFullRequestHost=\"%s\" stream_type=\"%s\" ", dims.fullRequestHost ? "yes" : "no", dims.streamType ? "primary" : "secondary");
+		fprintf(nhml, "contains_redundant=\"%s\" ", (dims.containsRedundant==1) ? "main" : (dims.containsRedundant==2) ? "redundant" : "main+redundant");
+		if (strlen(dims.textEncoding) ) fprintf(nhml, "text_encoding=\"%s\" ", dims.textEncoding);
+		if (strlen(dims.contentEncoding) ) {
+			fprintf(nhml, "content_encoding=\"%s\" ", dims.contentEncoding);
+			if (!strcmp(dims.contentEncoding, "deflate")) uncompress = 1;
+		}
+		if (dims.content_script_types) fprintf(nhml, "content_script_types=\"%s\" ", dims.content_script_types);
+	} else {
+		fprintf(nhml, "baseMediaFile=\"%s\" ", szMedia);
+	}
+	
+	
 	fprintf(nhml, ">\n");
 
 
@@ -1591,24 +1643,89 @@ GF_Err gf_media_export_nhml(GF_MediaExporter *dumper)
 	for (i=0; i<count; i++) {
 		GF_ISOSample *samp = gf_isom_get_sample(dumper->file, track, i+1, &di);
 		if (!samp) break;
-		fwrite(samp->data, samp->dataLength, 1, med);
 
-		fprintf(nhml, "<NHNTSample DTS=\""LLD"\" dataLength=\"%d\" ", LLD_CAST samp->DTS, samp->dataLength);
-		if (full_dump || samp->CTS_Offset) fprintf(nhml, "CTSOffset=\"%d\" ", samp->CTS_Offset);
-		if (samp->IsRAP==1) fprintf(nhml, "isRAP=\"yes\" ");
-		else if (samp->IsRAP==2) fprintf(nhml, "isSyncShadow=\"yes\" ");
-		else if (full_dump) fprintf(nhml, "isRAP=\"no\" ");
+		if (med)
+			fwrite(samp->data, samp->dataLength, 1, med);
 
-		if (full_dump) fprintf(nhml, "mediaOffset=\"%d\" ", pos);
-		fprintf(nhml, "/>\n");
+		if (dims_doc) {
+			GF_BitStream *bs = gf_bs_new(samp->data, samp->dataLength, GF_BITSTREAM_READ);
+
+			while (gf_bs_available(bs)) {
+				u32 pos = (u32) gf_bs_get_position(bs);
+				u16 size = gf_bs_read_u16(bs);
+				u8 flags = gf_bs_read_u8(bs);
+				u8 prev;
+				
+				if (pos+size+2>samp->dataLength)
+					break;
+
+				prev = samp->data[pos+2+size];
+				samp->data[pos+2+size] = 0;
+
+
+				fprintf(nhml, "<DIMSUnit time=\""LLD"\"", LLD_CAST samp->DTS);
+				/*DIMS flags*/
+				if (flags & GF_DIMS_UNIT_S) fprintf(nhml, " is-Scene=\"yes\"");
+				if (flags & GF_DIMS_UNIT_M) fprintf(nhml, " is-RAP=\"yes\"");
+				if (flags & GF_DIMS_UNIT_I) fprintf(nhml, " is-redundant=\"yes\"");
+				if (flags & GF_DIMS_UNIT_D) fprintf(nhml, " redundant-exit=\"yes\"");
+				if (flags & GF_DIMS_UNIT_P) fprintf(nhml, " priority=\"high\"");
+				if (flags & GF_DIMS_UNIT_C) fprintf(nhml, " compressed=\"yes\"");
+				fprintf(nhml, ">");
+				if (uncompress && (flags & GF_DIMS_UNIT_C)) {
+					char svg_data[2049];
+					int err;
+					u32 done = 0;
+					z_stream d_stream;
+					d_stream.zalloc = (alloc_func)0;
+					d_stream.zfree = (free_func)0;
+					d_stream.opaque = (voidpf)0;
+					d_stream.next_in  = (Bytef*)samp->data+pos+3;
+					d_stream.avail_in = size-1;
+					d_stream.next_out = (Bytef*)svg_data;
+					d_stream.avail_out = 2048;
+
+					err = inflateInit(&d_stream);
+					if (err == Z_OK) {
+						while ((s32) d_stream.total_in < size-1) {
+							err = inflate(&d_stream, Z_NO_FLUSH);
+							if (err < Z_OK) break;
+							svg_data[d_stream.total_out - done] = 0;
+							fprintf(nhml, svg_data);
+							if (err== Z_STREAM_END) break;
+							done = d_stream.total_out;
+							d_stream.avail_out = 2048;
+							d_stream.next_out = (Bytef*)svg_data;
+						}
+						inflateEnd(&d_stream);
+					}
+				} else {
+					fwrite(samp->data+pos+3, size-1, 1, nhml);
+				}
+				fprintf(nhml, "</DIMSUnit>\n");
+				
+				samp->data[pos+2+size] = prev;
+				gf_bs_skip_bytes(bs, size-1);
+			}
+			gf_bs_del(bs);
+
+		} else {
+			fprintf(nhml, "<NHNTSample DTS=\""LLD"\" dataLength=\"%d\" ", LLD_CAST samp->DTS, samp->dataLength);
+			if (full_dump || samp->CTS_Offset) fprintf(nhml, "CTSOffset=\"%d\" ", samp->CTS_Offset);
+			if (samp->IsRAP==1) fprintf(nhml, "isRAP=\"yes\" ");
+			else if (samp->IsRAP==2) fprintf(nhml, "isSyncShadow=\"yes\" ");
+			else if (full_dump) fprintf(nhml, "isRAP=\"no\" ");
+			if (full_dump) fprintf(nhml, "mediaOffset=\"%d\" ", pos);
+			fprintf(nhml, "/>\n");
+		}
 
 		pos += samp->dataLength;
 		gf_isom_sample_del(&samp);
 		gf_set_progress("NHML Export", i+1, count);
 		if (dumper->flags & GF_EXPORT_DO_ABORT) break;
 	}
-	fprintf(nhml, "</NHNTStream>\n");
-	fclose(med);
+	fprintf(nhml, "</%s>\n", szRootName);
+	if (med) fclose(med);
 	fclose(nhml);
 	return GF_OK;
 }
@@ -1734,6 +1851,11 @@ void m2ts_export_dump(GF_M2TS_Demuxer *ts, u32 evt_type, void *par)
 		GF_M2TS_PES_PCK *pck = (GF_M2TS_PES_PCK *)par;
 		fwrite(pck->data, pck->data_len, 1, dst);
 	}
+	else if (evt_type == GF_M2TS_EVT_SL_PCK) {
+		FILE *dst = (FILE*)ts->user;
+		GF_M2TS_SL_PCK *pck = (GF_M2TS_SL_PCK *)par;
+		fwrite(pck->data + 5, pck->data_len - 5, 1, dst);
+	}
 }
 
 GF_Err gf_media_export_ts_native(GF_MediaExporter *dumper)
@@ -1776,10 +1898,10 @@ GF_Err gf_media_export_ts_native(GF_MediaExporter *dumper)
 		if (pes->pid == dumper->trackID) {
 			stream = pes;
 			gf_m2ts_set_pes_framing(pes, GF_M2TS_PES_FRAMING_RAW);
+			break;
 		} else {
 			gf_m2ts_set_pes_framing(pes, GF_M2TS_PES_FRAMING_SKIP);
 		}
-		break;
 	}
 	if (!stream) {
 		fclose(src);
@@ -1869,7 +1991,7 @@ GF_Err gf_media_export(GF_MediaExporter *dumper)
 	else if (dumper->flags & GF_EXPORT_AVI) return gf_media_export_avi(dumper);
 	else if (dumper->flags & GF_EXPORT_MP4) return gf_media_export_isom(dumper);
 	else if (dumper->flags & GF_EXPORT_AVI_NATIVE) return gf_media_export_avi_track(dumper);
-	else if (dumper->flags & GF_EXPORT_NHML) return gf_media_export_nhml(dumper);
+	else if (dumper->flags & GF_EXPORT_NHML) return gf_media_export_nhml(dumper, 0);
 	else if (dumper->flags & GF_EXPORT_SAF) return gf_media_export_saf(dumper);
 	else return GF_BAD_PARAM;
 }
