@@ -67,7 +67,7 @@ GF_Err gf_odf_parse_descriptor(GF_BitStream *bs, GF_Descriptor **desc, u32 *desc
 	} while ( val & 0x80);
 	*desc_size = size;
 
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_CONTAINER, ("[ODF] Reading descriptor (tag %d size %d)\n", tag, size ));
+	GF_LOG(GF_LOG_DEBUG, GF_LOG_CODEC, ("[ODF] Reading descriptor (tag %d size %d)\n", tag, size ));
 
 	newDesc = gf_odf_create_descriptor(tag);
 	if (! newDesc) {
@@ -100,7 +100,7 @@ GF_Err gf_odf_parse_descriptor(GF_BitStream *bs, GF_Descriptor **desc, u32 *desc
 	*desc_size += sizeHeader - gf_odf_size_field_size(*desc_size);
 	*desc = newDesc;
 	if (err) {
-		GF_LOG(GF_LOG_ERROR, GF_LOG_CONTAINER, ("[ODF] Error reading descriptor (tag %d size %d): %s\n", tag, size, gf_error_to_string(err) ));
+		GF_LOG(GF_LOG_ERROR, GF_LOG_CODEC, ("[ODF] Error reading descriptor (tag %d size %d): %s\n", tag, size, gf_error_to_string(err) ));
 		gf_odf_delete_descriptor(newDesc);
 		*desc = NULL;
 	}
@@ -271,8 +271,6 @@ GF_Err gf_odf_write_descriptor_list_filter(GF_BitStream *bs, GF_List *descList,
 	return GF_OK;
 }
 
-static char szStreamText[20];
-
 GF_EXPORT
 const char *gf_odf_stream_type_name(u32 streamType)
 {
@@ -289,9 +287,7 @@ const char *gf_odf_stream_type_name(u32 streamType)
 	case GF_STREAM_INTERACT: return "Interaction";
 	case GF_STREAM_TEXT: return "Text";
 	case GF_STREAM_ND_SUBPIC: return "NeroDigital Subpicture";
-	default: 
-		sprintf(szStreamText, "Unknown (0x%02x)", streamType);
-		return szStreamText;
+	default: return "Unknown";
 	}
 }
 
@@ -322,7 +318,13 @@ GF_BIFSConfig *gf_odf_get_bifs_config(GF_DefaultDescriptor *dsi, u8 oti)
 	GF_Err e;
 	GF_BitStream *bs;
 	GF_BIFSConfig *cfg;
-	if (!dsi || !dsi->data || !dsi->dataLength ) return NULL;
+	if (!dsi || !dsi->data || !dsi->dataLength ) {
+		/* Hack for T-DMB non compliant streams (OnTimeTek ?) */
+		cfg = (GF_BIFSConfig *) gf_odf_desc_new(GF_ODF_BIFS_CFG_TAG);	
+		cfg->pixelMetrics = 1;
+		cfg->version = 1;
+		return cfg;
+	}
 	bs = gf_bs_new(dsi->data, dsi->dataLength, GF_BITSTREAM_READ);
 	
 	cfg = (GF_BIFSConfig *) gf_odf_desc_new(GF_ODF_BIFS_CFG_TAG);	
