@@ -38,7 +38,7 @@ struct __tag_bifs_engine
 	GF_SceneLoader load;
 	void *calling_object;
 	GF_StreamContext *sc;
-	
+	Bool owns_context;	
 	GF_BifsEncoder *bifsenc;
 	u32 stream_ts_res;
 	/* TODO: maybe the currentAUCount should be a GF_List of u32 
@@ -49,41 +49,17 @@ struct __tag_bifs_engine
 	u32 encoded_bifs_config_size;
 };
 
-
-static GF_Err gf_sm_live_setup(GF_BifsEngine *codec)
+static GF_Err gf_sm_setup_bifsenc(GF_BifsEngine *codec, GF_InitialObjectDescriptor *iod)
 {
 	GF_Err e;
 	char *data;
 	u32 data_len;
-	GF_InitialObjectDescriptor *iod;
-	u32	i, j, count, nbb;
+	u32	j, nbb;
 	GF_ESD *esd;
 	Bool is_in_iod, delete_desc, encode_names, delete_bcfg;
 	GF_BIFSConfig *bcfg;
 
 	e = GF_OK;
-
-	iod = (GF_InitialObjectDescriptor *) codec->ctx->root_od;
-	/*if no iod check we only have one bifs*/
-	if (!iod) {
-		count = 0;
-		i=0;
-		while ((codec->sc = (GF_StreamContext*)gf_list_enum(codec->ctx->streams, &i))) {
-			if (codec->sc->streamType == GF_STREAM_OD) count++;
-			codec->sc = NULL;
-		}
-		if (!iod && count>1) return GF_NOT_SUPPORTED;
-	}
-
-	codec->sc = NULL;
-	count = gf_list_count(codec->ctx->streams);
-	i=0;
-	while ((codec->sc = (GF_StreamContext*)gf_list_enum(codec->ctx->streams, &i))) {
-		if (codec->sc->streamType == GF_STREAM_SCENE) break;
-	}
-	if (!codec->sc) return GF_NOT_SUPPORTED;
-	if (!codec->sc->ESID) codec->sc->ESID = 1;
-
 	codec->bifsenc = gf_bifs_encoder_new(codec->ctx->scene_graph);
 
 	delete_desc = 0;
@@ -180,10 +156,48 @@ static GF_Err gf_sm_live_setup(GF_BifsEngine *codec)
 	codec->encoded_bifs_config_size = data_len;
 
 	esd->decoderConfig->objectTypeIndication = gf_bifs_encoder_get_version(codec->bifsenc, codec->sc->ESID);		
+	return GF_OK;
+}
+
+static GF_Err gf_sm_live_setup(GF_BifsEngine *codec)
+{
+	GF_Err e;
+	GF_InitialObjectDescriptor *iod;
+	u32	i, count;
+
+	e = GF_OK;
+
+	iod = (GF_InitialObjectDescriptor *) codec->ctx->root_od;
+	/*if no iod check we only have one bifs*/
+	if (!iod) {
+		count = 0;
+		i=0;
+		while ((codec->sc = (GF_StreamContext*)gf_list_enum(codec->ctx->streams, &i))) {
+			if (codec->sc->streamType == GF_STREAM_OD) count++;
+			codec->sc = NULL;
+		}
+		if (!iod && count>1) return GF_NOT_SUPPORTED;
+	}
+
+	codec->sc = NULL;
+	count = gf_list_count(codec->ctx->streams);
+	i=0;
+	while ((codec->sc = (GF_StreamContext*)gf_list_enum(codec->ctx->streams, &i))) {
+		if (codec->sc->streamType == GF_STREAM_SCENE) break;
+	}
+	if (!codec->sc) return GF_NOT_SUPPORTED;
+	if (!codec->sc->ESID) codec->sc->ESID = 1;
+
+	if (codec->sc->objectType <= GPAC_OTI_SCENE_BIFS_V2) {
+		return gf_sm_setup_bifsenc(codec, iod);
+	} else if (codec->sc->objectType == GPAC_OTI_SCENE_DIMS) {
+		/* TODO */
+		return GF_NOT_SUPPORTED;
+	}
 	return e;
 }
 
-static GF_Err gf_sm_live_encode_bifs_au(GF_BifsEngine *codec, u32 currentAUCount, 
+static GF_Err gf_sm_live_encode_scene_au(GF_BifsEngine *codec, u32 currentAUCount, 
 						  GF_Err (*AUCallback)(void *, char *, u32 , u64)
 						  )
 {
@@ -200,7 +214,17 @@ static GF_Err gf_sm_live_encode_bifs_au(GF_BifsEngine *codec, u32 currentAUCount
 		au = (GF_AUContext *)gf_list_get(codec->sc->AUs, j);
 		/*in case using XMT*/
 		if (au->timing_sec) au->timing = (u64) (au->timing_sec * codec->stream_ts_res);
-		e = gf_bifs_encode_au(codec->bifsenc, codec->sc->ESID, au->commands, &data, &size);
+		switch(codec->sc->objectType) {
+		case GPAC_OTI_SCENE_BIFS:
+		case GPAC_OTI_SCENE_BIFS_V2:
+			e = gf_bifs_encode_au(codec->bifsenc, codec->sc->ESID, au->commands, &data, &size);
+			break;
+		case GPAC_OTI_SCENE_DIMS:
+			break;
+		default:
+			GF_LOG(GF_LOG_ERROR, GF_LOG_SCENE, ("Cannot encode AU for Scene OTI %x\n", codec->sc->objectType));
+			break;
+		}
 		AUCallback(codec->calling_object, data, size, au->timing);
 		free(data);
 		data = NULL;
@@ -279,7 +303,7 @@ GF_Err gf_beng_encode_from_string(GF_BifsEngine *codec, char *auString, GF_Err (
 	e = gf_sm_load_string(&codec->load, auString, 0);
 	if (e) goto exit;
 
-	e = gf_sm_live_encode_bifs_au(codec, codec->currentAUCount, AUCallback); 
+	e = gf_sm_live_encode_scene_au(codec, codec->currentAUCount, AUCallback); 
 exit:
 	return e;
 }
@@ -316,7 +340,7 @@ GF_Err gf_beng_encode_from_file(GF_BifsEngine *codec, char *auFile, GF_Err (*AUC
 		goto exit;
 	}
 
-	e = gf_sm_live_encode_bifs_au(codec, codec->currentAUCount, AUCallback); 
+	e = gf_sm_live_encode_scene_au(codec, codec->currentAUCount, AUCallback); 
 	if (e) goto exit;
 exit:
 	return e;
@@ -325,15 +349,18 @@ exit:
 GF_EXPORT
 GF_Err gf_beng_encode_context(GF_BifsEngine *codec, GF_Err (*AUCallback)(void *, char *, u32 , u64 ))
 {
-	return gf_sm_live_encode_bifs_au(codec, 0, AUCallback);
+	return gf_sm_live_encode_scene_au(codec, 0, AUCallback);
 } 
 
 GF_EXPORT
 void gf_beng_terminate(GF_BifsEngine *codec)
 {
 	if (codec->bifsenc) gf_bifs_encoder_del(codec->bifsenc);
-	if (codec->ctx) gf_sm_del(codec->ctx);
-	if (codec->sg) gf_sg_del(codec->sg);
+
+	if (codec->owns_context) {
+		if (codec->ctx) gf_sm_del(codec->ctx);
+		if (codec->sg) gf_sg_del(codec->sg);
+	}
 	free(codec);
 }
 
@@ -361,6 +388,7 @@ GF_BifsEngine *gf_beng_init(void *calling_object, char * inputContext)
 	/*Step 1: create context and load input*/
 	codec->sg = gf_sg_new();
 	codec->ctx = gf_sm_new(codec->sg);
+	codec->owns_context = 1;
 	memset(&(codec->load), 0, sizeof(GF_SceneLoader));
 	codec->load.ctx = codec->ctx;
 	/*since we're encoding in BIFS we must get MPEG-4 nodes only*/
@@ -387,6 +415,36 @@ exit:
 	return NULL;
 }
 
+GF_EXPORT
+GF_BifsEngine *gf_beng_init_from_context(void *calling_object, GF_SceneManager *ctx)
+{
+	GF_BifsEngine *codec;
+	GF_Err e = GF_OK;
+
+	if (!ctx) return NULL;
+
+	GF_SAFEALLOC(codec, GF_BifsEngine)
+	if (!codec) return NULL;
+
+	codec->calling_object = calling_object;
+
+	/*Step 1: create context and load input*/
+	codec->sg = ctx->scene_graph;
+	codec->ctx = ctx;
+	codec->owns_context = 0;
+
+	e = gf_sm_live_setup(codec);
+	if (e!=GF_OK) {
+		GF_LOG(GF_LOG_ERROR, GF_LOG_SCENE, ("[BENG] cannot init scene encoder for context (error %s)\n", gf_error_to_string(e)));
+		goto exit;
+	}
+	return codec;
+
+exit:
+	gf_beng_terminate(codec);
+	return NULL;
+}
+
 GF_EXPORT
 GF_BifsEngine *gf_beng_init_from_string(void *calling_object, char * inputContext, u32 width, u32 height, Bool usePixelMetrics)
 {
@@ -403,6 +461,7 @@ GF_BifsEngine *gf_beng_init_from_string(void *calling_object, char * inputContex
 	/*Step 1: create context and load input*/
 	codec->sg = gf_sg_new();
 	codec->ctx = gf_sm_new(codec->sg);
+	codec->owns_context = 1;
 	memset(&(codec->load), 0, sizeof(GF_SceneLoader));
 	codec->load.ctx = codec->ctx;
 	/*since we're encoding in BIFS we must get MPEG-4 nodes only*/
