@@ -132,7 +132,7 @@ static void mp4_report(GF_SceneLoader *load, GF_Err e, char *format, ...)
 #endif
 }
 
-GF_Err gf_sm_load_run_MP4(GF_SceneLoader *load)
+GF_Err gf_sm_load_run_isom(GF_SceneLoader *load)
 {
 	GF_Err e;
 	FILE *logs;
@@ -197,7 +197,12 @@ GF_Err gf_sm_load_run_MP4(GF_SceneLoader *load)
 			if (esd->decoderConfig->objectTypeIndication<=2) {
 				if (!esd->dependsOnESID && nbBifs && !i) 
 					mp4_report(load, GF_OK, "several scene namespaces used or improper scene dependencies in file - import may be incorrect");
-				e = gf_bifs_decoder_configure_stream(bifs_dec, esd->ESID, esd->decoderConfig->decoderSpecificInfo->data, esd->decoderConfig->decoderSpecificInfo->dataLength, esd->decoderConfig->objectTypeIndication);
+				if (!esd->decoderConfig->decoderSpecificInfo) {
+					/* Hack for T-DMB non compliant streams */
+					e = gf_bifs_decoder_configure_stream(bifs_dec, esd->ESID, NULL, 0, esd->decoderConfig->objectTypeIndication);
+				} else {
+					e = gf_bifs_decoder_configure_stream(bifs_dec, esd->ESID, esd->decoderConfig->decoderSpecificInfo->data, esd->decoderConfig->decoderSpecificInfo->dataLength, esd->decoderConfig->objectTypeIndication);
+				}
 				if (e) goto exit;
 				nbBifs++;
 			}
@@ -280,7 +285,7 @@ exit:
 	return e;
 }
 
-GF_Err gf_sm_load_init_MP4(GF_SceneLoader *load)
+GF_Err gf_sm_load_init_isom(GF_SceneLoader *load)
 {
 	u32 i, track;
 	GF_BIFSConfig *bc;
@@ -354,9 +359,10 @@ GF_Err gf_sm_load_init_MP4(GF_SceneLoader *load)
 	return GF_OK;
 }
 
-void gf_sm_load_done_MP4(GF_SceneLoader *load)
+void gf_sm_load_done_isom(GF_SceneLoader *load)
 {
 	/*nothing to do the file is not ours*/
+
 }
 
 #endif
