@@ -40,12 +40,33 @@ typedef struct
 	GF_ChildNodeItem *last;
 } XMTNodeStack;
 
+
+/**/
+enum
+{
+	/*document is not yet initialized*/
+	XMT_STATE_INIT = 0,
+	/*document head is being parsed*/
+	XMT_STATE_HEAD = 1,
+	/*document body being parsed*/
+	XMT_STATE_BODY = 2,
+	/*commands are being parsed*/
+	XMT_STATE_COMMANDS = 3,
+	/*elements are being parsed*/
+	XMT_STATE_ELEMENTS = 4,
+	/*end of body parsing*/
+	XMT_STATE_BODY_END = 5,
+	/*end of parsing*/
+	XMT_STATE_END = 6,
+};
+
+
 typedef struct
 {	
 	/*1: XMT-A, 2: X3D, 3: XMT-O (not supported yet) */
 	u32 doc_type;
 	/*0: not init, 1: header, 2: body*/
-	u32 doc_state;
+	u32 state;
 	u32 current_node_tag;
 
 	GF_SceneLoader *load;
@@ -571,7 +592,7 @@ static u32 xmt_get_node_id(GF_XMTParser *parser, char *name)
 	return ID;
 }
 
-static u32 xmt_get_node_tag(GF_XMTParser *parser, char *node_name) 
+static u32 xmt_get_node_tag(GF_XMTParser *parser, const char *node_name) 
 {
 	u32 tag;
 	/*if VRML and allowing non MPEG4 nodes, use X3D*/
@@ -855,7 +876,11 @@ static u32 xmt_parse_sf_field(GF_XMTParser *parser, GF_FieldInfo *info, GF_Node
 	case GF_SG_VRML_SFCOMMANDBUFFER:
 	{
 		SFCommandBuffer *cb = (SFCommandBuffer *)info->far_ptr;
-		cb->buffer = (unsigned char*)parser->command;
+		if (parser->command_buffer) {
+			cb->buffer = (unsigned char*)parser->command_buffer;
+		} else {
+			cb->buffer = (unsigned char*)parser->command;
+		}
 		parser->command_buffer = cb;
 	}
 		break;
@@ -1154,7 +1179,7 @@ static void xmt_update_timenode(GF_XMTParser *parser, GF_Node *node)
 	}
 }
 
-static void xmt_strip_name(char *in, char *out)
+static void xmt_strip_name(const char *in, char *out)
 {
 	while (in[0]==' ') in++;
 	strcpy(out, in);
@@ -1164,7 +1189,7 @@ static void xmt_strip_name(char *in, char *out)
 static u32 xmt_get_ft_by_name(const char *_name)
 {
 	char name[1024];
-	xmt_strip_name((char *)_name, name);
+	xmt_strip_name(_name, name);
 
 	if (!strcmp(name, "Boolean") || !strcmp(name, "SFBool")) return GF_SG_VRML_SFBOOL;
 	else if (!strcmp(name, "Integer") || !strcmp(name, "SFInt32")) return GF_SG_VRML_SFINT32;
@@ -1403,7 +1428,7 @@ static GF_Node *xmt_parse_element(GF_XMTParser *parser, char *name, const char *
 					xmt_parse_mf_field(parser, &info, NULL, value);
 				}
 			} else if (gf_sg_vrml_get_sf_type(fType) != GF_SG_VRML_SFNODE) {
-				gf_sg_proto_field_set_value_undefined(parser->proto_field);
+				/*value not specified for exter proto*/
 			} 
 			/*SF/MFNode proto field: push node stack with container info but no parent*/
 			else {
@@ -1422,7 +1447,7 @@ static GF_Node *xmt_parse_element(GF_XMTParser *parser, char *name, const char *
 		if (parser->proto_field && (!strcmp(name, "node") || !strcmp(name, "nodes")) ) 
 			return NULL;
 		/*anim & QP info */
-		if (parser->proto_field  && !strcmp(name, "InterfaceCodingParameter")) {
+		if (parser->proto_field  && !strcmp(name, "InterfaceCodingParameters")) {
 			u32 qp_type, nbBits, hasMinMax, qp_sftype;
 			Fixed ftMin, ftMax;
 			ftMin = ftMax = 0;
@@ -1560,6 +1585,7 @@ static GF_Node *xmt_parse_element(GF_XMTParser *parser, char *name, const char *
 					return NULL;
 				}
 				node = gf_sg_proto_create_instance(parser->load->scene_graph, proto);
+				att->value = NULL;
 			}
 			else if (!strcmp(att->name, "USE")) {
 				node = xmt_find_node(parser, att->value);
@@ -1586,7 +1612,7 @@ static GF_Node *xmt_parse_element(GF_XMTParser *parser, char *name, const char *
 						parser->command_buffer = (SFCommandBuffer*)parent->container_field.far_ptr;
 						/*store command*/
 						parser->command_buffer->buffer = (unsigned char *)parser->command;
-						parser->doc_state = 3;
+						parser->state = XMT_STATE_COMMANDS;
 					}
 					return NULL;
 				}
@@ -1686,7 +1712,7 @@ static GF_Node *xmt_parse_element(GF_XMTParser *parser, char *name, const char *
 		/*all other fields*/
 		else {
 			e = gf_node_get_field_by_name(node, att->name, &info);
-			if (e) xmt_report(parser, GF_OK, "Warning: Unknown field %s for node %s - skipping", att->name, name);
+			if (e) xmt_report(parser, GF_OK, "Warning: Unknown field \"%s\" for node %s - skipping", att->name, name);
 			else if (gf_sg_vrml_is_sf_field(info.fieldType)) {
 				xmt_parse_sf_field(parser, &info, node, att->value);
 			} else {
@@ -1727,6 +1753,7 @@ static GF_Node *xmt_parse_element(GF_XMTParser *parser, char *name, const char *
 			gf_node_list_add_child_last( (GF_ChildNodeItem **)container.far_ptr, node, &parent->last);
 			gf_node_register(node, parent->node);
 		}
+		assert(parent->node);
 		gf_node_changed(parent->node, NULL);
 	}
 		
@@ -1796,7 +1823,7 @@ GF_Descriptor *xmt_parse_descriptor(GF_XMTParser *parser, char *name, const GF_X
 		GF_XMLAttribute *att = (GF_XMLAttribute *) &attributes[i];
 		if (!att->value || !strlen(att->value)) continue;
 		if (!strcmp(att->name, "binaryID")) binaryID = atoi(att->value);
-		else if (!strcmp(att->name, "objectDescriptorID")) xmt_desc_name = att->value;
+		else if (!stricmp(att->name, "objectDescriptorID")) xmt_desc_name = att->value;
 		else if (!strcmp(att->name, "ES_ID")) xmt_desc_name = att->value;
 		else if (!strcmp(att->name, "OCR_ES_ID")) ocr_ref = att->value;
 		else if (!strcmp(att->name, "dependsOn_ES_ID")) dep_ref = att->value;
@@ -1859,14 +1886,14 @@ GF_Descriptor *xmt_parse_descriptor(GF_XMTParser *parser, char *name, const GF_X
 	return desc;
 }
 
-static void xmt_parse_command(GF_XMTParser *parser, char *name, const GF_XMLAttribute *attributes, u32 nb_attributes)
+static void xmt_parse_command(GF_XMTParser *parser, const char *name, const GF_XMLAttribute *attributes, u32 nb_attributes)
 {
 	GF_Err e;
 	GF_FieldInfo info;
 	GF_CommandField *field;
 	u32 i;
 	if (!strcmp(name, "Scene")) {
-		parser->doc_state = 4;
+		parser->state = XMT_STATE_ELEMENTS;
 		return;
 	}
 
@@ -1915,7 +1942,7 @@ static void xmt_parse_command(GF_XMTParser *parser, char *name, const GF_XMLAttr
 			else if (!strcmp(att->name, "value")) fieldValue = att->value;
 		}
 		if (!fieldName) {
-			parser->doc_state = 4;
+			parser->state = XMT_STATE_ELEMENTS;
 			return;
 		}
 		e = gf_node_get_field_by_name(parser->command->node, fieldName, &info);
@@ -1924,7 +1951,7 @@ static void xmt_parse_command(GF_XMTParser *parser, char *name, const GF_XMLAttr
 			return;
 		}
 		if (gf_sg_vrml_get_sf_type(info.fieldType) == GF_SG_VRML_SFNODE) {
-			parser->doc_state = 4;
+			parser->state = XMT_STATE_ELEMENTS;
 			return;
 		}
 		if (!fieldValue) return;
@@ -1941,7 +1968,7 @@ static void xmt_parse_command(GF_XMTParser *parser, char *name, const GF_XMLAttr
 				xmt_parse_mf_field(parser, &info, parser->command->node, fieldValue);
 			}
 		} else {
-			parser->doc_state = 4;
+			parser->state = XMT_STATE_ELEMENTS;
 		}
 		return;
 	}
@@ -1967,7 +1994,7 @@ static void xmt_parse_command(GF_XMTParser *parser, char *name, const GF_XMLAttr
 			field->fieldIndex = info.fieldIndex;
 			field->fieldType = GF_SG_VRML_SFNODE;
 			field->pos = position;
-			parser->doc_state = 4;
+			parser->state = XMT_STATE_ELEMENTS;
 		} else if (fieldValue) {
 			field = gf_sg_command_field_new(parser->command);
 			field->fieldIndex = info.fieldIndex;
@@ -2035,11 +2062,11 @@ static void xmt_parse_command(GF_XMTParser *parser, char *name, const GF_XMLAttr
 			} else {
 				if (!strcmp(name, "Replace")) {
 					tag = GF_SG_NODE_REPLACE;
-					parser->doc_state = 4;
+					parser->state = XMT_STATE_ELEMENTS;
 				}
 				else if (!strcmp(name, "Insert")) {
 					tag = GF_SG_NODE_INSERT;
-					parser->doc_state = 4;
+					parser->state = XMT_STATE_ELEMENTS;
 				}
 				else if (!strcmp(name, "Delete")) tag = GF_SG_NODE_DELETE;
 			}
@@ -2072,21 +2099,21 @@ static void xmt_parse_command(GF_XMTParser *parser, char *name, const GF_XMLAttr
 		if (extended) {
 			if (!strcmp(extended, "globalQuant")) {
 				tag = GF_SG_GLOBAL_QUANTIZER;
-				parser->doc_state = 4;
+				parser->state = XMT_STATE_ELEMENTS;
 			}
 			else if (!strcmp(extended, "fields")) {
 				tag = GF_SG_MULTIPLE_REPLACE;
-				parser->doc_state = 3;;
+				parser->state = XMT_STATE_COMMANDS;
 			}
 			else if (!strcmp(extended, "indices")) {
 				tag = GF_SG_MULTIPLE_INDEXED_REPLACE;
-				parser->doc_state = 3;
+				parser->state = XMT_STATE_COMMANDS;
 			}
 			else if (!strcmp(extended, "deleteOrder")) tag = GF_SG_NODE_DELETE_EX;
 			else if (!strcmp(extended, "allProtos")) tag = GF_SG_PROTO_DELETE_ALL;
 			else if (!strcmp(extended, "proto") || !strcmp(extended, "protos")) {
 				if (!strcmp(name, "Insert")) {
-					parser->doc_state = 4;
+					parser->state = XMT_STATE_ELEMENTS;
 					tag = GF_SG_PROTO_INSERT;
 				}
 				else if (!strcmp(name, "Delete")) tag = GF_SG_PROTO_DELETE;
@@ -2106,6 +2133,7 @@ static void xmt_parse_command(GF_XMTParser *parser, char *name, const GF_XMLAttr
 		parser->command = gf_sg_command_new(parser->load->scene_graph, tag);
 		if (parser->command_buffer) {
 			gf_list_add(parser->command_buffer->commandList, parser->command);
+			parser->command_buffer->bufferSize++;
 		} else {
 			GF_StreamContext *stream = gf_sm_stream_find(parser->load->ctx, (u16) stream_id);
 			if (!stream || (stream->streamType!=GF_STREAM_SCENE)) stream_id = parser->base_scene_id;
@@ -2151,13 +2179,13 @@ static void xmt_parse_command(GF_XMTParser *parser, char *name, const GF_XMLAttr
 					} else {
 						field->fieldType = GF_SG_VRML_SFNODE;
 					} 
-					parser->doc_state = 4;
+					parser->state = XMT_STATE_ELEMENTS;
 				}
 			} else if (tag==GF_SG_NODE_INSERT) {
 				field = gf_sg_command_field_new(parser->command);
 				field->fieldType = GF_SG_VRML_SFNODE;
 				field->pos = position;
-				parser->doc_state = 4;
+				parser->state = XMT_STATE_ELEMENTS;
 			}
 		}
 		else if (routeName) {
@@ -2216,7 +2244,7 @@ static void xmt_parse_command(GF_XMTParser *parser, char *name, const GF_XMLAttr
 			if (!att->value || !strlen(att->value)) continue;
 			if (!strcmp(att->name, "begin")) au_time = atoi(att->value);
 			else if (!strcmp(att->name, "isRAP")) au_is_rap = !strcmp(att->value, "yes") ? 1 : 0;
-			else if (!strcmp(att->name, "objectDescriptorId")) od_ids = att->value;
+			else if (!stricmp(att->name, "objectDescriptorId")) od_ids = att->value;
 			else if (!strcmp(att->name, "ES_ID")) es_ids = att->value;
 		}
 
@@ -2305,18 +2333,18 @@ static void xmt_node_start(void *sax_cbck, const char *name, const char *name_sp
 	}
 
 	/*init doc state*/
-	if (!parser->doc_state) {
+	if (parser->state == XMT_STATE_INIT) {
 		/*XMT-A header*/
-		if ((parser->doc_type == 1) && !strcmp(name, "Header")) parser->doc_state = 1;
+		if ((parser->doc_type == 1) && !strcmp(name, "Header")) parser->state = XMT_STATE_HEAD;
 		/*X3D header*/
-		else if ((parser->doc_type == 2) && !strcmp(name, "head")) parser->doc_state = 1;
+		else if ((parser->doc_type == 2) && !strcmp(name, "head")) parser->state = XMT_STATE_HEAD;
 		/*XMT-O header*/
-		else if ((parser->doc_type == 3) && !strcmp(name, "head")) parser->doc_state = 1;
+		else if ((parser->doc_type == 3) && !strcmp(name, "head")) parser->state = XMT_STATE_HEAD;
 		return;
 	}
 
 	/*XMT-A header: parse OD/IOD*/
-	if ((parser->doc_type == 1) && (parser->doc_state == 1)) {
+	if ((parser->doc_type == 1) && (parser->state == XMT_STATE_HEAD)) {
 		GF_Descriptor *desc, *par;
 		par = (GF_Descriptor *)gf_list_last(parser->descriptors);
 		desc = xmt_parse_descriptor(parser, (char *) name, attributes, nb_attributes, par);
@@ -2325,12 +2353,12 @@ static void xmt_node_start(void *sax_cbck, const char *name, const char *name_sp
 	}
 
 	/*scene content*/	
-	if (parser->doc_state==2) {
+	if (parser->state==XMT_STATE_BODY) {
 		/*XMT-A body*/
-		if ((parser->doc_type == 1) && !strcmp(name, "Body")) parser->doc_state = 3;
+		if ((parser->doc_type == 1) && !strcmp(name, "Body")) parser->state = XMT_STATE_COMMANDS;
 		/*X3D scene*/
 		else if ((parser->doc_type == 2) && !strcmp(name, "Scene")) {
-			parser->doc_state = 4;
+			parser->state = XMT_STATE_ELEMENTS;
 			if (parser->load->ctx) {
 				parser->load->ctx->is_pixel_metrics = 0;
 				parser->load->ctx->scene_width = parser->load->ctx->scene_height = 0;
@@ -2338,12 +2366,12 @@ static void xmt_node_start(void *sax_cbck, const char *name, const char *name_sp
 			gf_sg_set_scene_size_info(parser->load->scene_graph, 0, 0, 0);
 		}
 		/*XMT-O body*/
-		else if ((parser->doc_type == 3) && !strcmp(name, "body")) parser->doc_state = 3;
+		else if ((parser->doc_type == 3) && !strcmp(name, "body")) parser->state = XMT_STATE_COMMANDS;
 		return;
 	}
 
 	/*XMT-A command*/
-	if ((parser->doc_type == 1) && (parser->doc_state == 3)) {
+	if ((parser->doc_type == 1) && (parser->state == XMT_STATE_COMMANDS)) {
 		/*OD command*/
 		if (parser->od_command) {
 			GF_Descriptor *desc, *par;
@@ -2351,14 +2379,14 @@ static void xmt_node_start(void *sax_cbck, const char *name, const char *name_sp
 			desc = xmt_parse_descriptor(parser, (char *) name, attributes, nb_attributes, par);
 			if (desc) gf_list_add(parser->descriptors, desc);
 		} else {
-			xmt_parse_command(parser, (char *) name, attributes, nb_attributes);
+			xmt_parse_command(parser, name, attributes, nb_attributes);
 		}
 		return;
 	}
 
 
 	/*node*/
-	if (parser->doc_state != 4) return;
+	if (parser->state != XMT_STATE_ELEMENTS) return;
 
 	top = (XMTNodeStack*)gf_list_last(parser->nodes);
 	if (!top) top = parser->x3d_root;
@@ -2383,7 +2411,7 @@ static void xmt_node_end(void *sax_cbck, const char *name, const char *name_spac
 	XMTNodeStack *top;
 	GF_Descriptor *desc;
 	GF_Node *node = NULL;
-	if (!parser->doc_type || !parser->doc_state) return;
+	if (!parser->doc_type || !parser->state) return;
 
 	top = (XMTNodeStack *)gf_list_last(parser->nodes);
 
@@ -2391,10 +2419,19 @@ static void xmt_node_end(void *sax_cbck, const char *name, const char *name_spac
 		/*check descr*/
 		desc = (GF_Descriptor*)gf_list_last(parser->descriptors);
 		if (desc && (desc->tag == gf_odf_get_tag_by_name((char *)name)) ) {
+
+			/*assign timescales once the ESD has been parsed*/
+			if (desc->tag == GF_ODF_ESD_TAG) {
+				GF_ESD *esd = (GF_ESD*)desc;
+				GF_StreamContext *sc = gf_sm_stream_new(parser->load->ctx, esd->ESID, esd->decoderConfig ? esd->decoderConfig->streamType : 0, esd->decoderConfig ? esd->decoderConfig->objectTypeIndication : 0);
+				if (sc && esd->slConfig && esd->slConfig->timestampResolution)
+					sc->timeScale = esd->slConfig->timestampResolution;
+			}
+
 			gf_list_rem_last(parser->descriptors);
 			if (gf_list_count(parser->descriptors)) return;
 
-			if ((parser->doc_type==1) && (parser->doc_state==1) && parser->load->ctx && !parser->load->ctx->root_od) {
+			if ((parser->doc_type==1) && (parser->state==XMT_STATE_HEAD) && parser->load->ctx && !parser->load->ctx->root_od) {
 				parser->load->ctx->root_od = (GF_ObjectDescriptor *)desc;
 			} 
 			else if (!parser->od_command) {
@@ -2415,11 +2452,11 @@ static void xmt_node_end(void *sax_cbck, const char *name, const char *name_spac
 
 			return;
 		}
-		if (parser->doc_state == 1) {
-			if ((parser->doc_type == 1) && !strcmp(name, "Header")) parser->doc_state = 2;
+		if (parser->state == XMT_STATE_HEAD) {
+			if ((parser->doc_type == 1) && !strcmp(name, "Header")) parser->state = XMT_STATE_BODY;
 			/*X3D header*/
 			else if ((parser->doc_type == 2) && !strcmp(name, "head")) {
-				parser->doc_state = 2;
+				parser->state = XMT_STATE_BODY;
 				/*create a group at root level*/
 				tag = xmt_get_node_tag(parser, "Group");
 				node = gf_node_new(parser->load->scene_graph, tag);
@@ -2432,13 +2469,13 @@ static void xmt_node_end(void *sax_cbck, const char *name, const char *name_spac
 				parser->x3d_root->node = node;
 			}
 			/*XMT-O header*/
-			else if ((parser->doc_type == 3) && !strcmp(name, "head")) parser->doc_state = 2;
+			else if ((parser->doc_type == 3) && !strcmp(name, "head")) parser->state = XMT_STATE_BODY;
 		}
-		else if (parser->doc_state == 4) {
+		else if (parser->state == XMT_STATE_ELEMENTS) {
 			assert((parser->doc_type != 1) || parser->command);
 			if (!strcmp(name, "Replace") || !strcmp(name, "Insert") || !strcmp(name, "Delete")) {
 				parser->command = NULL;
-				parser->doc_state = 3;
+				parser->state = XMT_STATE_COMMANDS;
 			}
 			/*end proto*/
 			else if (!strcmp(name, "ProtoDeclare") || !strcmp(name, "ExternProtoDeclare"))  {
@@ -2450,15 +2487,15 @@ static void xmt_node_end(void *sax_cbck, const char *name, const char *name_spac
 			}
 			else if (parser->proto_field && !strcmp(name, "field")) parser->proto_field = NULL;
 			/*end X3D body*/
-			else if ((parser->doc_type == 2) && !strcmp(name, "Scene")) parser->doc_state = 5;
+			else if ((parser->doc_type == 2) && !strcmp(name, "Scene")) parser->state = XMT_STATE_BODY_END;
 		}
-		else if (parser->doc_state == 3) {
+		else if (parser->state == XMT_STATE_COMMANDS) {
 			/*end XMT-A body*/
-			if ((parser->doc_type == 1) && !strcmp(name, "Body")) parser->doc_state = 5;
+			if ((parser->doc_type == 1) && !strcmp(name, "Body")) parser->state = XMT_STATE_BODY_END;
 			/*end X3D body*/
-			else if ((parser->doc_type == 2) && !strcmp(name, "Scene")) parser->doc_state = 5;
+			else if ((parser->doc_type == 2) && !strcmp(name, "Scene")) parser->state = XMT_STATE_BODY_END;
 			/*end XMT-O body*/
-			else if ((parser->doc_type == 3) && !strcmp(name, "body")) parser->doc_state = 5;
+			else if ((parser->doc_type == 3) && !strcmp(name, "body")) parser->state = XMT_STATE_BODY_END;
 
 			/*end scene command*/
 			else if (!strcmp(name, "Replace") || !strcmp(name, "Insert") || !strcmp(name, "Delete") )  {
@@ -2479,9 +2516,9 @@ static void xmt_node_end(void *sax_cbck, const char *name, const char *name_spac
 			}
 
 		}
-		else if (parser->doc_state == 5) {
+		else if (parser->state == XMT_STATE_BODY_END) {
 			/*end XMT-A*/
-			if ((parser->doc_type == 1) && !strcmp(name, "XMT-A")) parser->doc_state = 6;
+			if ((parser->doc_type == 1) && !strcmp(name, "XMT-A")) parser->state = XMT_STATE_END;
 			/*end X3D*/
 			else if ((parser->doc_type == 2) && !strcmp(name, "X3D")) {
 				while (1) {
@@ -2492,27 +2529,48 @@ static void xmt_node_end(void *sax_cbck, const char *name, const char *name_spac
 				}
 				gf_list_del(parser->script_to_load);
 				parser->script_to_load = NULL;
-				parser->doc_state = 6;
+				parser->state = XMT_STATE_END;
 			}
 			/*end XMT-O*/
-			else if ((parser->doc_type == 3) && !strcmp(name, "XMT-O")) parser->doc_state = 6;
+			else if ((parser->doc_type == 3) && !strcmp(name, "XMT-O")) parser->state = XMT_STATE_END;
 		}
 		return;
 	}
 	/*only remove created nodes ... */
-	tag = xmt_get_node_tag(parser, (char *) name);
+	tag = xmt_get_node_tag(parser, name);
 	if (!tag) {
-		if (top->container_field.name && !strcmp(name, top->container_field.name)) {
-
-			if (top->container_field.fieldType==GF_SG_VRML_SFCOMMANDBUFFER) {
-				parser->doc_state = 4;
-				parser->command = (GF_Command *) (void *) parser->command_buffer->buffer;
-				parser->command_buffer->buffer = NULL;
-				parser->command_buffer = NULL;
+		if (top->container_field.name) {
+			if (!strcmp(name, top->container_field.name)) {
+				if (top->container_field.fieldType==GF_SG_VRML_SFCOMMANDBUFFER) {
+					parser->state = XMT_STATE_ELEMENTS;
+					parser->command = (GF_Command *) (void *) parser->command_buffer->buffer;
+					parser->command_buffer->buffer = NULL;
+					parser->command_buffer = NULL;
+				}
+				top->container_field.far_ptr = NULL;
+				top->container_field.name = NULL;
+				top->last = NULL;
+			}
+			/*end of command inside an command (conditional.buffer replace)*/
+			else if (!strcmp(name, "Replace") || !strcmp(name, "Insert") || !strcmp(name, "Delete") )  {
+				if (parser->command_buffer) {
+					if (parser->command_buffer->bufferSize) {
+						parser->command_buffer->bufferSize--;
+					} else {
+						SFCommandBuffer *prev = (SFCommandBuffer *) parser->command_buffer->buffer;
+						parser->command_buffer->buffer = NULL;
+						parser->command_buffer = prev;
+					}
+					/*stay in command parsing mode (state 3) until we find </buffer>*/
+					parser->state = XMT_STATE_COMMANDS;
+				}
+			}
+			/*end of protofield node(s) content*/
+			else if (!strcmp(name, "node") || !strcmp(name, "nodes")) {
+				top->container_field.far_ptr = NULL;
+				top->container_field.name = NULL;
+				top->last = NULL;
 			}
-			top->container_field.far_ptr = NULL;
-			top->container_field.name = NULL;
-			top->last = NULL;
 		}
 		/*SF/MFNode proto field, just pop node stack*/
 		else if (!top->node && !strcmp(name, "field")) {
@@ -2756,7 +2814,7 @@ GF_Err gf_sm_load_init_xmt_string(GF_SceneLoader *load, char *str_data)
 
 		if (load->flags & GF_SM_LOAD_CONTEXT_READY) {
 			parser->doc_type = (load->type==GF_SM_LOAD_X3D) ? 2 : 1;
-			parser->doc_state = 3;
+			parser->state = XMT_STATE_COMMANDS;
 		}
 	}
 	e = gf_xml_sax_parse(parser->sax_parser, str_data);
@@ -2782,6 +2840,7 @@ GF_Err gf_sm_load_done_xmt(GF_SceneLoader *load)
 		gf_node_unregister(st->node, NULL);
 		free(st);
 	}
+	if (parser->x3d_root) free(parser->x3d_root);
 	gf_list_del(parser->nodes);
 	gf_list_del(parser->descriptors);
 	gf_list_del(parser->def_nodes);
