@@ -27,9 +27,10 @@
 #include <gpac/utf.h>
 #include <gpac/internal/scenegraph_dev.h>
 #include <gpac/nodes_x3d.h>
-#include <gpac/nodes_svg_da.h>
+#include <gpac/nodes_svg.h>
+#include <gpac/events.h>
 
-#ifndef GPAC_READ_ONLY
+//#ifndef GPAC_READ_ONLY
 
 #ifndef __SYMBIAN32__
 #include <wchar.h>
@@ -77,12 +78,14 @@ void SD_DumpSVG_Element(GF_SceneDumper *sdump, GF_Node *n, GF_Node *parent, Bool
 GF_Err gf_sm_dump_command_list(GF_SceneDumper *sdump, GF_List *comList, u32 indent, Bool skip_first_replace);
 
 GF_EXPORT
-GF_SceneDumper *gf_sm_dumper_new(GF_SceneGraph *graph, char *rad_name, char indent_char, u32 dump_mode)
+GF_SceneDumper *gf_sm_dumper_new(GF_SceneGraph *graph, char *_rad_name, char indent_char, u32 dump_mode)
 {
+	char rad_name[GF_MAX_PATH];
 	GF_SceneDumper *tmp;
 	if (!graph) return NULL;
 	GF_SAFEALLOC(tmp, GF_SceneDumper);
 
+	strcpy(rad_name, _rad_name ? _rad_name : "");
 	/*store original*/
 	tmp->dump_mode = dump_mode;
 
@@ -91,8 +94,9 @@ GF_SceneDumper *gf_sm_dumper_new(GF_SceneGraph *graph, char *rad_name, char inde
 	|| (dump_mode==GF_SM_DUMP_LASER) || (dump_mode==GF_SM_DUMP_SVG)) {
 		tmp->XMLDump = 1;
 		if (dump_mode==GF_SM_DUMP_LASER) tmp->LSRDump = 1;
-		if (rad_name) {
+		if (_rad_name) {
 			strcat(rad_name, tmp->LSRDump ? ".xsr" : ".svg");
+
 			tmp->trace = fopen(rad_name, "wt");
 			if (!tmp->trace) {
 				free(tmp);
@@ -120,7 +124,7 @@ GF_SceneDumper *gf_sm_dumper_new(GF_SceneGraph *graph, char *rad_name, char inde
 			}
 		}
 
-		if (rad_name) {
+		if (_rad_name) {
 			switch (dump_mode) {
 			case GF_SM_DUMP_X3D_XML: strcat(rad_name, ".x3d"); tmp->XMLDump = 1; tmp->X3DDump = 1; break;
 			case GF_SM_DUMP_XMTA: strcat(rad_name, ".xmt"); tmp->XMLDump = 1; break;
@@ -171,7 +175,8 @@ void SD_SetupDump(GF_SceneDumper *sdump, GF_Descriptor *root_od)
 {
 	if (sdump->XMLDump) {
 		fprintf(sdump->trace, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
-		fprintf(sdump->trace, "<!-- %s Scene Dump - GPAC version " GPAC_VERSION " -->\n", 
+		if (sdump->dump_mode==GF_SM_DUMP_XML) return;
+		fprintf(sdump->trace, "<!-- %s Scene Dump - GPAC version " GPAC_FULL_VERSION " -->\n", 
 			(sdump->dump_mode==GF_SM_DUMP_SVG) ? "SVG" : 
 			(sdump->dump_mode==GF_SM_DUMP_LASER) ? "LASeR" : 
 			sdump->X3DDump ? "X3D" : "XMT-A"
@@ -179,7 +184,8 @@ void SD_SetupDump(GF_SceneDumper *sdump, GF_Descriptor *root_od)
 	}
 	if (sdump->dump_mode==GF_SM_DUMP_SVG) return;
 	if (sdump->LSRDump) {
-		fprintf(sdump->trace, "<saf:SAFSession xmlns:saf=\"urn:mpeg:mpeg4:SAF:2005\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" xmlns:lsr=\"urn:mpeg:mpeg4:LASeR:2005\" xmlns=\"http://www.w3.org/2000/svg\">\n");
+		fprintf(sdump->trace, "<saf:SAFSession xmlns:saf=\"urn:mpeg:mpeg4:SAF:2005\" >\n");
+#ifndef GPAC_READ_ONLY
 		if (root_od) {
 			GF_ObjectDescriptor *iod = (GF_ObjectDescriptor *)root_od;
 			u32 i, count;
@@ -196,6 +202,7 @@ void SD_SetupDump(GF_SceneDumper *sdump, GF_Descriptor *root_od)
 			}
 			fprintf(sdump->trace, "</saf:sceneHeader>\n");
 		}
+#endif
 		return;
 	}
 
@@ -204,19 +211,22 @@ void SD_SetupDump(GF_SceneDumper *sdump, GF_Descriptor *root_od)
 		if (sdump->XMLDump) {
 			fprintf(sdump->trace, "<XMT-A xmlns=\"urn:mpeg:mpeg4:xmta:schema:2002\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:schemaLocation=\"urn:mpeg:mpeg4:xmta:schema:2002 xmt-a.xsd\">\n");
 			fprintf(sdump->trace, " <Header>\n");
+#ifndef GPAC_READ_ONLY
 			if (root_od) gf_odf_dump_desc(root_od, sdump->trace, 1, 1);
+#endif
 			fprintf(sdump->trace, " </Header>\n");
 			fprintf(sdump->trace, " <Body>\n");
 			if (!root_od) {
 				fprintf(sdump->trace, "  <Replace>\n");
-				fprintf(sdump->trace, "   <Scene>\n");
 			}
 		} else {
 			if (sdump->dump_mode==GF_SM_DUMP_VRML) {
 				fprintf(sdump->trace, "#VRML V2.0\n");
 			} else {
 				/*dump root OD*/
+#ifndef GPAC_READ_ONLY
 				if (root_od) gf_odf_dump_desc(root_od, sdump->trace, 0, 0);
+#endif
 			}
 			fprintf(sdump->trace, "\n");
 		}
@@ -225,7 +235,7 @@ void SD_SetupDump(GF_SceneDumper *sdump, GF_Descriptor *root_od)
 			fprintf(sdump->trace, "<!DOCTYPE X3D PUBLIC \"ISO//Web3D//DTD X3D 3.0//EN\" \"http://www.web3d.org/specifications/x3d-3.0.dtd\">\n");
 			fprintf(sdump->trace, "<X3D xmlns:xsd=\"http://www.w3.org/2001/XMLSchema-instance\" xsd:noNamespaceSchemaLocation=\"http://www.web3d.org/specifications/x3d-3.0.xsd\" version=\"3.0\">\n");
 			fprintf(sdump->trace, "<head>\n");
-			fprintf(sdump->trace, "<meta content=\"X3D File Converted/Dumped by GPAC Version %s\" name=\"generator\"/>\n", GPAC_VERSION);
+			fprintf(sdump->trace, "<meta content=\"X3D File Converted/Dumped by GPAC Version %s\" name=\"generator\"/>\n", GPAC_FULL_VERSION);
 			fprintf(sdump->trace, "</head>\n");
 			fprintf(sdump->trace, "<Scene>\n");
 		} else {
@@ -246,7 +256,6 @@ void SD_FinalizeDump(GF_SceneDumper *sdump, GF_Descriptor *root_od)
 
 	if (!sdump->X3DDump) {
 		if (!root_od) {
-			fprintf(sdump->trace, "   </Scene>\n");
 			fprintf(sdump->trace, "  </Replace>\n");
 		}
 		fprintf(sdump->trace, " </Body>\n");
@@ -314,7 +323,7 @@ void EndElement(GF_SceneDumper *sdump, const char *name, Bool had_sub_el)
 	}
 }
 
-void StartAttribute(GF_SceneDumper *sdump, char *name)
+void StartAttribute(GF_SceneDumper *sdump, const char *name)
 {
 	if (!sdump->trace) return;
 	if (!sdump->XMLDump) {
@@ -336,7 +345,7 @@ void EndAttribute(GF_SceneDumper *sdump)
 }
 
 
-void StartList(GF_SceneDumper *sdump, char *name)
+void StartList(GF_SceneDumper *sdump, const char *name)
 {
 	if (!sdump->trace) return;
 	DUMP_IND(sdump);
@@ -350,7 +359,7 @@ void StartList(GF_SceneDumper *sdump, char *name)
 	}
 }
 
-void EndList(GF_SceneDumper *sdump, char *name)
+void EndList(GF_SceneDumper *sdump, const char *name)
 {
 	if (!sdump->trace) return;
 	DUMP_IND(sdump);
@@ -423,23 +432,38 @@ void DumpBool(GF_SceneDumper *sdump, char *name, u32 value)
 	EndAttribute(sdump);
 }
 
-void DumpUTFString(GF_SceneDumper *sdump, char *str)
+void DumpUTFString(GF_SceneDumper *sdump, Bool escape_xml, char *str)
 {
 	u32 len, i;
 	u16 *uniLine;
 	if (!str) return;
 	len = strlen(str);
+	if (!len) return;
 	uniLine = (u16*)malloc(sizeof(u16) * len);
 	len = gf_utf8_mbstowcs(uniLine, len, (const char **) &str);
 	if (len != (size_t) (-1)) {
 		for (i=0; i<len; i++) {
-			if (uniLine[i] == (u16) '\"') fprintf(sdump->trace, "\\");
+			//if (uniLine[i] == (u16) '\"') fprintf(sdump->trace, "\\");
 			switch (uniLine[i]) {
-			case '\'': fprintf(sdump->trace, "&apos;"); break;
-			case '\"': fprintf(sdump->trace, "&quot;"); break;
-			case '&': fprintf(sdump->trace, "&amp;"); break;
-			case '>': fprintf(sdump->trace, "&gt;"); break;
-			case '<': fprintf(sdump->trace, "&lt;"); break;
+			case '\'': 
+				if (escape_xml) fprintf(sdump->trace, "&apos;"); 
+				else fprintf(sdump->trace, "'"); 
+				break;
+			case '\"': 
+				if (escape_xml) fprintf(sdump->trace, "&quot;"); 
+				else fprintf(sdump->trace, "\""); 
+				break;
+			case '&': 
+				fprintf(sdump->trace, "&amp;"); 
+				break;
+			case '>': 
+				fprintf(sdump->trace, "&gt;"); 
+				break;
+			case '<': 
+				fprintf(sdump->trace, "&lt;"); 
+				break;
+			case '\r': fprintf(sdump->trace, ""); break;
+			case '\n': fprintf(sdump->trace, ""); break;
 			default:
 			  if (uniLine[i]<128) {
 				fprintf(sdump->trace, "%c", (u8) uniLine[i]);
@@ -555,7 +579,7 @@ void DumpSFField(GF_SceneDumper *sdump, u32 type, void *ptr, Bool is_mf)
 		str = ((SFString *)ptr)->buffer;
 		if (str && str[0]) {
 			if (sdump->XMLDump) {
-				DumpUTFString(sdump, str);
+				DumpUTFString(sdump, 1, str);
 			} else if (!strchr(str, '\"')) {
 				fprintf(sdump->trace, "%s", str);
 			} else {
@@ -724,7 +748,7 @@ void DumpField(GF_SceneDumper *sdump, GF_Node *node, GF_FieldInfo field)
 				sdump->indent++;
 			}
 		} else {
-			StartAttribute(sdump, (char *)field.name);
+			StartAttribute(sdump, field.name);
 		}
 		DumpNode(sdump, *(GF_Node **)field.far_ptr, 0, NULL);
 		
@@ -745,14 +769,14 @@ void DumpField(GF_SceneDumper *sdump, GF_Node *node, GF_FieldInfo field)
 		}
 		list = * ((GF_ChildNodeItem **) field.far_ptr);
 		assert(list);
-		if (!sdump->XMLDump || !sdump->X3DDump) StartList(sdump, (char *) field.name);
+		if (!sdump->XMLDump || !sdump->X3DDump) StartList(sdump, field.name);
 		sdump->indent++;
 		while (list) {
 			DumpNode(sdump, list->node, 1, needs_field_container ? (char *) field.name : NULL);
 			list = list->next;
 		}
 		sdump->indent--;
-		if (!sdump->XMLDump || !sdump->X3DDump) EndList(sdump, (char *) field.name);
+		if (!sdump->XMLDump || !sdump->X3DDump) EndList(sdump, field.name);
 		return;
 	case GF_SG_VRML_SFCOMMANDBUFFER:
 	{
@@ -776,7 +800,7 @@ void DumpField(GF_SceneDumper *sdump, GF_Node *node, GF_FieldInfo field)
 		return;
 	}
 	if (gf_sg_vrml_is_sf_field(field.fieldType)) {
-		StartAttribute(sdump, (char *) field.name);
+		StartAttribute(sdump, field.name);
 		DumpSFField(sdump, field.fieldType, field.far_ptr, 0);
 		EndAttribute(sdump);
 	} else {
@@ -791,11 +815,11 @@ void DumpField(GF_SceneDumper *sdump, GF_Node *node, GF_FieldInfo field)
 				fprintf(sdump->trace, " %s=\'", (char *) field.name);
 				break;
 			default:
-				StartAttribute(sdump, (char *) field.name);
+				StartAttribute(sdump, field.name);
 				break;
 			}
 		} else {
-			StartAttribute(sdump, (char *) field.name);
+			StartAttribute(sdump, field.name);
 		}
 
 		if (!sdump->XMLDump) fprintf(sdump->trace, "[");
@@ -1121,9 +1145,9 @@ GF_Route *SD_GetISedField(GF_SceneDumper *sdump, GF_Node *node, GF_FieldInfo *fi
 		if (!r->IS_route) continue;
 		if ((r->ToNode==node) && (r->ToField.fieldIndex==field->fieldIndex)) return r;
 	}
-	if (!node || !node->sgprivate->interact || !node->sgprivate->interact->events) return NULL;
+	if (!node || !node->sgprivate->interact || !node->sgprivate->interact->routes) return NULL;
 	i=0;
-	while ((r = (GF_Route*)gf_list_enum(node->sgprivate->interact->events, &i))) {
+	while ((r = (GF_Route*)gf_list_enum(node->sgprivate->interact->routes, &i))) {
 		if (!r->IS_route) continue;
 		if (r->FromField.fieldIndex == field->fieldIndex) return r;
 	}
@@ -2246,11 +2270,24 @@ static char *lsr_format_node_id(GF_Node *n, u32 NodeID, char *str)
 	return str;
 }
 
+static char szLSRName[1024];
+
+static char *sd_get_lsr_namespace(GF_SceneGraph *sg)
+{
+	char *lsrns = (char *) gf_sg_get_namespace_qname(sg, GF_XMLNS_LASER);
+	if (lsrns) {
+		sprintf(szLSRName, "%s:", lsrns);
+		return szLSRName;
+	}
+	return "";
+}
+
 GF_Err DumpLSRNewScene(GF_SceneDumper *sdump, GF_Command *com)
 {
-	fprintf(sdump->trace, "<lsr:NewScene>\n");
+	char *lsrns = sd_get_lsr_namespace(com->in_scene);
+	fprintf(sdump->trace, "<%sNewScene>\n", lsrns);
 	SD_DumpSVG_Element(sdump, com->node, NULL, 0);
-	fprintf(sdump->trace, "</lsr:NewScene>\n");
+	fprintf(sdump->trace, "</%sNewScene>\n", lsrns);
 	return GF_OK;
 }
 
@@ -2259,12 +2296,13 @@ GF_Err DumpLSRAddReplaceInsert(GF_SceneDumper *sdump, GF_Command *com)
 	char szAtt[80000];
 	Bool is_text = 0;
 	GF_CommandField *f;
+	char *lsrns = sd_get_lsr_namespace(com->in_scene);
 
 	const char *com_name = (com->tag==GF_SG_LSR_REPLACE) ? "Replace" : ( (com->tag==GF_SG_LSR_ADD) ? "Add" : "Insert" );
 
 	DUMP_IND(sdump);
 
-	fprintf(sdump->trace, "<lsr:%s ref=\"%s\" ", com_name, lsr_format_node_id(com->node, com->RouteID, szAtt));
+	fprintf(sdump->trace, "<%s%s ref=\"%s\" ", lsrns, com_name, lsr_format_node_id(com->node, com->RouteID, szAtt));
 	f = (GF_CommandField *) gf_list_get(com->command_fields, 0);
 	if (f && (f->pos>=0) ) fprintf(sdump->trace, "index=\"%d\" ", f->pos);
 	if (f) {
@@ -2275,7 +2313,7 @@ GF_Err DumpLSRAddReplaceInsert(GF_SceneDumper *sdump, GF_Command *com)
 			else if (f->fieldType==SVG_Transform_Rotate_datatype) att_name = "rotation";
 			else if (f->fieldType==SVG_Transform_Translate_datatype) att_name = "translation";
 			else if (f->fieldIndex==(u32) -1) att_name = "textContent";
-			else att_name = (char*) gf_svg_get_attribute_name(f->fieldIndex);
+			else att_name = (char*) gf_svg_get_attribute_name(com->node, f->fieldIndex);
 
 			fprintf(sdump->trace, "attributeName=\"%s\" ", att_name);
 			if (f->field_ptr) {
@@ -2326,7 +2364,7 @@ GF_Err DumpLSRAddReplaceInsert(GF_SceneDumper *sdump, GF_Command *com)
 	}
 	sdump->indent--;
 	if (!is_text) DUMP_IND(sdump);
-	fprintf(sdump->trace, "</lsr:%s>\n", com_name);
+	fprintf(sdump->trace, "</%s%s>\n", lsrns, com_name);
 	return GF_OK;
 }
 GF_Err DumpLSRClean(GF_SceneDumper *sdump, GF_Command *com)
@@ -2337,8 +2375,9 @@ GF_Err DumpLSRDelete(GF_SceneDumper *sdump, GF_Command *com)
 {
 	char szID[1024];
 	GF_CommandField *f;
+	char *lsrns = sd_get_lsr_namespace(com->in_scene);
 	DUMP_IND(sdump);
-	fprintf(sdump->trace, "<lsr:Delete ref=\"%s\" ", lsr_format_node_id(com->node, com->RouteID, szID));
+	fprintf(sdump->trace, "<%sDelete ref=\"%s\" ", lsrns, lsr_format_node_id(com->node, com->RouteID, szID));
 	f = (GF_CommandField *) gf_list_get(com->command_fields, 0);
 	if (f && (f->pos>=0) ) fprintf(sdump->trace, "index=\"%d\" ", f->pos);
 	fprintf(sdump->trace, "/>\n");
@@ -2358,16 +2397,45 @@ GF_Err DumpLSRSave(GF_SceneDumper *sdump, GF_Command *com)
 }
 GF_Err DumpLSRSendEvent(GF_SceneDumper *sdump, GF_Command *com)
 {
+	char szID[1024];
+	char *lsrns = sd_get_lsr_namespace(com->in_scene);
+	DUMP_IND(sdump);
+	fprintf(sdump->trace, "<%sSendEvent ref=\"%s\" event=\"%s\"", lsrns, 
+		lsr_format_node_id(com->node, com->RouteID, szID),
+		gf_dom_event_get_name(com->send_event_name)
+		);
+	if (com->send_event_name <= GF_EVENT_MOUSEWHEEL)
+		fprintf(sdump->trace, " pointvalue=\"%g %g\"", FIX2FLT(com->send_event_x), FIX2FLT(com->send_event_y) );
+
+	switch (com->send_event_name) {
+	case GF_EVENT_KEYDOWN:
+	case GF_EVENT_LONGKEYPRESS:
+	case GF_EVENT_REPEAT_KEY:
+	case GF_EVENT_SHORT_ACCESSKEY:
+		if (com->send_event_integer) {
+			fprintf(sdump->trace, " stringvalue=\"%s\"", gf_dom_get_key_name(com->send_event_integer) );
+			break;
+		}
+	default:
+		if (com->send_event_integer)
+			fprintf(sdump->trace, " intvalue=\"%d\"", com->send_event_integer);
+		if (com->send_event_string)
+			fprintf(sdump->trace, " stringvalue=\"%s\"", com->send_event_string);
+		break;
+	}
+
+	fprintf(sdump->trace, "/>\n");
 	return GF_OK;
 }
 GF_Err DumpLSRActivate(GF_SceneDumper *sdump, GF_Command *com)
 {
 	char szID[1024];
+	char *lsrns = sd_get_lsr_namespace(com->in_scene);
 	DUMP_IND(sdump);
 	if (com->tag==GF_SG_LSR_ACTIVATE) {
-		fprintf(sdump->trace, "<lsr:Activate ref=\"%s\" />\n", lsr_format_node_id(com->node, com->RouteID, szID));
+		fprintf(sdump->trace, "<%sActivate ref=\"%s\" />\n", lsrns, lsr_format_node_id(com->node, com->RouteID, szID));
 	} else {
-		fprintf(sdump->trace, "<lsr:Deactivate ref=\"%s\" />\n", lsr_format_node_id(com->node, com->RouteID, szID));
+		fprintf(sdump->trace, "<%sDeactivate ref=\"%s\" />\n", lsrns, lsr_format_node_id(com->node, com->RouteID, szID));
 	}
 	return GF_OK;
 }
@@ -2524,13 +2592,13 @@ GF_Err gf_sm_dump_command_list(GF_SceneDumper *sdump, GF_List *comList, u32 inde
 void SD_DumpSVG_Element(GF_SceneDumper *sdump, GF_Node *n, GF_Node *parent, Bool is_root)
 {
 	GF_ChildNodeItem *list;
-	char attValue[81920];
+	char attName[100], attValue[81920];
 	u32 i, count, nID;
 	Bool needs_cr;
 	SVG_Element *svg = (SVG_Element *)n;
 	GF_FieldInfo info;
 	SVGAttribute *att;
-	u32 tag;
+	u32 tag, ns;
 	if (!n) return;
 
 	nID = gf_node_get_id(n);
@@ -2539,23 +2607,28 @@ void SD_DumpSVG_Element(GF_SceneDumper *sdump, GF_Node *n, GF_Node *parent, Bool
 	if (!nID) {
 		switch (tag) {
 		case TAG_SVG_listener:
-			if (gf_svg_get_attribute_by_tag(n, TAG_SVG_ATT_handler, 0, 0, &info)==GF_OK) {
+			if (0 && gf_node_get_attribute_by_tag(n, TAG_XMLEV_ATT_handler, 0, 0, &info)==GF_OK) {
 				if (((XMLRI*)info.far_ptr)->target && !gf_node_get_id(((XMLRI*)info.far_ptr)->target) ) 
 					return;
 			}
 			break;
 		case TAG_SVG_handler:
-			return;
+			/*this handler was not declared in the graph*/
+			if (!n->sgprivate->parents || (n->sgprivate->parents->node != parent))
+				return;
+			break;
 		case TAG_DOMText:
 		{
 			GF_DOMText *txt = (GF_DOMText *)n;
-			if (txt->textContent) {
-				if (txt->is_cdata) {
-					fprintf(sdump->trace, "<![CDATA[\n");
+			if (txt->textContent) {				
+				if ((txt->type==GF_DOM_TEXT_CDATA) ||
+					(parent->sgprivate->tag == TAG_SVG_script) ||
+					(parent->sgprivate->tag == TAG_SVG_handler)) {
+					fprintf(sdump->trace, "<![CDATA[");
 					fprintf(sdump->trace, "%s", txt->textContent);
-					fprintf(sdump->trace, "]]>\n");
-				} else {
-					DumpUTFString(sdump, txt->textContent);
+					fprintf(sdump->trace, "]]>");
+				} else if (txt->type==GF_DOM_TEXT_REGULAR) {
+					DumpUTFString(sdump, 0, txt->textContent);
 				}
 			}
 		}
@@ -2567,11 +2640,12 @@ void SD_DumpSVG_Element(GF_SceneDumper *sdump, GF_Node *n, GF_Node *parent, Bool
 		DUMP_IND(sdump);
 	}
 
+	/*register all namespaces specified on this element */
+	gf_xml_push_namespaces((GF_DOMNode *)n);
+
 	fprintf(sdump->trace, "<%s", gf_node_get_class_name(n));
+	ns = gf_xml_get_element_namespace(n);
 
-	if (is_root) 
-		fprintf(sdump->trace, " xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\"");
-	
 	if (nID) fprintf(sdump->trace, " id=\"%s\"", lsr_format_node_id(n, 0, attValue));
 
 	att = svg->attributes;
@@ -2580,7 +2654,21 @@ void SD_DumpSVG_Element(GF_SceneDumper *sdump, GF_Node *n, GF_Node *parent, Bool
 			att = att->next;
 			continue;
 		} 
-		else if (att->data_type==XMLRI_datatype) {
+		
+		info.fieldIndex = att->tag;
+		info.fieldType = att->data_type;
+		if (att->tag==TAG_DOM_ATT_any) {
+			u32 att_ns = ((GF_DOMFullAttribute*)att)->xmlns;
+			info.name = ((GF_DOMFullAttribute*)att)->name;
+			if ((att_ns != ns) && strncmp(info.name, "xmlns", 5)) {
+				sprintf(attName, "%s:%s", gf_sg_get_namespace_qname(gf_node_get_graph(n), att_ns), ((GF_DOMFullAttribute*)att)->name);
+				info.name = attName;
+			}
+		} else {
+			info.name = gf_svg_get_attribute_name(n, att->tag);
+		}
+
+		if (att->data_type==XMLRI_datatype) {
 			XMLRI *xlink = (XMLRI *)att->data;
 			if (xlink->type==XMLRI_ELEMENTID) {
 				if (!xlink->target || !gf_node_get_id((GF_Node*)xlink->target) ) {
@@ -2593,35 +2681,44 @@ void SD_DumpSVG_Element(GF_SceneDumper *sdump, GF_Node *n, GF_Node *parent, Bool
 				}
 			}
 			else if (xlink->type==XMLRI_STREAMID) {
-				fprintf(sdump->trace, "xlink:href=\"#stream%d\" ", xlink->lsr_stream_id);
+				fprintf(sdump->trace, " %s=\"#stream%d\"", info.name, xlink->lsr_stream_id);
+				att = att->next;
+				continue;
+			} else {
+				fprintf(sdump->trace, " %s=\"%s\"", info.name, xlink->string);
 				att = att->next;
 				continue;
 			}
 		}
-		info.fieldIndex = att->tag;
-		info.fieldType = att->data_type;
-		info.name = gf_svg_get_attribute_name(att->tag);
 		info.far_ptr = att->data;
 		gf_svg_dump_attribute((GF_Node*)svg, &info, attValue);
-		if (strlen(attValue)) fprintf(sdump->trace, " %s=\"%s\"", info.name, attValue);
+		if (/*strcmp(info.name, "xmlns") &&*/ (info.fieldType = strlen(attValue)))
+			fprintf(sdump->trace, " %s=\"%s\"", info.name, attValue);
 		fflush(sdump->trace);
 		att = att->next;
 	}
 
 	/*re-translate dynamically created listeners/handlers */
-	if (n->sgprivate->interact && n->sgprivate->interact->events) {
-		count = gf_list_count(n->sgprivate->interact->events);
+	if (n->sgprivate->interact && n->sgprivate->interact->dom_evt) {
+		count = gf_list_count(n->sgprivate->interact->dom_evt->evt_list);
 		for (i=0; i<count; i++) {
 			SVG_handlerElement *hdl;
-			GF_Node *listener = (GF_Node *)gf_list_get(n->sgprivate->interact->events, i);
-			if (!(listener->sgprivate->flags & 0x7FFFFFFF) ) continue;
-			if (gf_svg_get_attribute_by_tag(listener, TAG_SVG_ATT_handler, 0, 0, &info)==GF_OK) {
+			GF_Node *listener = (GF_Node *)gf_list_get(n->sgprivate->interact->dom_evt->evt_list, i);
+			/*this listener has been created for internal use*/
+			if (listener->sgprivate->parents) continue;
+			if (gf_node_get_attribute_by_tag(listener, TAG_XMLEV_ATT_handler, 0, 0, &info)==GF_OK) {
 				GF_DOMText *txt;
 				hdl = (SVG_handlerElement *) ((XMLRI*)info.far_ptr)->target;
 				if (!hdl) continue;
+				/*this handler was declared in the graph*/
+				if (hdl->sgprivate->parents 
+					&& (hdl->sgprivate->parents->next || (hdl->sgprivate->parents->node != listener))
+				)
+					continue;
+
 				txt = hdl->children ? (GF_DOMText*)hdl->children->node : NULL;
 				if (!txt || (txt->sgprivate->tag!=TAG_DOMText) || !txt->textContent) continue;
-				if (gf_svg_get_attribute_by_tag((GF_Node*)hdl, TAG_SVG_ATT_ev_event, 0, 0, &info)==GF_OK) {
+				if (gf_node_get_attribute_by_tag((GF_Node*)hdl, TAG_XMLEV_ATT_event, 0, 0, &info)==GF_OK) {
 					fprintf(sdump->trace, " on%s=\"%s\"", gf_dom_event_get_name( ((XMLEV_Event*)info.far_ptr)->type), txt->textContent);
 				}
 			}
@@ -2630,11 +2727,10 @@ void SD_DumpSVG_Element(GF_SceneDumper *sdump, GF_Node *n, GF_Node *parent, Bool
 	if (svg->children) fprintf(sdump->trace, ">");
 	else {
 		fprintf(sdump->trace, "/>");
-		if (!sdump->in_text) fprintf(sdump->trace, "\n");
 		return;
 	}
 
-	if (n->sgprivate->tag==TAG_SVG_conditional) {
+	if (n->sgprivate->tag==TAG_LSR_conditional) {
 		GF_DOMUpdates *up = svg->children ? (GF_DOMUpdates *)svg->children->node : NULL;
 		sdump->indent++;
 		if (up && (up->sgprivate->tag==TAG_DOMUpdates)) {
@@ -2651,7 +2747,7 @@ void SD_DumpSVG_Element(GF_SceneDumper *sdump, GF_Node *n, GF_Node *parent, Bool
 		return;
 	}
 
-	if (tag==TAG_SVG_text) sdump->in_text = 1;
+	if (tag==TAG_SVG_text || tag==TAG_SVG_textArea) sdump->in_text = 1;
 	needs_cr = 1;
 	sdump->indent++;
 	list = svg->children;
@@ -2660,11 +2756,13 @@ void SD_DumpSVG_Element(GF_SceneDumper *sdump, GF_Node *n, GF_Node *parent, Bool
 		SD_DumpSVG_Element(sdump, list->node, n, 0);
 		list = list->next;
 	}
+	if (!sdump->in_text) fprintf(sdump->trace, "\n");
 	sdump->indent--;
 	if (!sdump->in_text) DUMP_IND(sdump);
 	fprintf(sdump->trace, "</%s>", gf_node_get_class_name(n));
-	if (tag==TAG_SVG_text) sdump->in_text = 0;
-	if (!sdump->in_text) fprintf(sdump->trace, "\n");
+	if (tag==TAG_SVG_text || tag==TAG_SVG_textArea) sdump->in_text = 0;
+	/*removes all namespaces specified on this element */
+	gf_xml_pop_namespaces((GF_DOMNode *)n);
 }
 #endif
 
@@ -2722,6 +2820,64 @@ GF_Err SD_SetSceneGraph(GF_SceneDumper *sdump, GF_SceneGraph *sg)
 	return GF_OK;
 }
 
+GF_Err SD_DumpDOMElement(GF_SceneDumper *sdump, GF_DOMFullNode *node)
+{
+	const char *ns;
+	u32 child_type = 0;
+	GF_DOMFullAttribute *att;
+	GF_ChildNodeItem *child;
+	GF_DOMText *txt;
+	ns = gf_sg_get_namespace_qname(node->sgprivate->scenegraph, node->ns);
+
+	DUMP_IND(sdump);
+	if (ns) fprintf(sdump->trace, "<%s:%s", ns, node->name);
+	else fprintf(sdump->trace, "<%s", node->name);
+	att = (GF_DOMFullAttribute *)node->attributes;
+	while (att) {
+		fprintf(sdump->trace, " %s=\"%s\"", att->name, (char *) att->data);
+		att = (GF_DOMFullAttribute *)att->next;
+	}
+	if (!node->children) {
+		fprintf(sdump->trace, "/>\n");
+		return GF_OK;
+	}
+	fprintf(sdump->trace, ">");
+	sdump->indent++;
+	child = node->children;
+	while (child) {
+		switch(child->node->sgprivate->tag) {
+		case TAG_DOMFullNode:
+			if (!child_type) fprintf(sdump->trace, "\n");
+			child_type = 1;
+			SD_DumpDOMElement(sdump, (GF_DOMFullNode*)child->node);
+			break;
+		case TAG_DOMText:
+			child_type = 2;
+			txt = (GF_DOMText *)child->node;
+			if (txt->type==GF_DOM_TEXT_REGULAR) {
+				DumpUTFString(sdump, 0, txt->textContent);
+			} else if (txt->type==GF_DOM_TEXT_CDATA) {
+				fprintf(sdump->trace, "<![CDATA[");
+				fprintf(sdump->trace, "%s", txt->textContent);
+				fprintf(sdump->trace, "]]>");
+			}
+			break;
+		}
+		child = child->next;
+	}
+
+	sdump->indent--;
+	if (child_type!=2) {
+		DUMP_IND(sdump);
+	}
+
+	if (ns) fprintf(sdump->trace, "</%s:%s>\n", ns, node->name);
+	else fprintf(sdump->trace, "</%s>\n", node->name);
+
+	return GF_OK;
+}
+
+
 GF_EXPORT
 GF_Err gf_sm_dump_graph(GF_SceneDumper *sdump, Bool skip_proto, Bool skip_routes)
 {
@@ -2779,6 +2935,11 @@ GF_Err gf_sm_dump_graph(GF_SceneDumper *sdump, Bool skip_proto, Bool skip_routes
 		SD_DumpSVG_Element(sdump, sdump->sg->RootNode, NULL, 1);
 		return GF_OK;
 	}
+	else if (tag==TAG_DOMFullNode) {
+		sdump->dump_mode = GF_SM_DUMP_XML;
+		SD_SetupDump(sdump, NULL);
+		SD_DumpDOMElement(sdump, (GF_DOMFullNode*)sdump->sg->RootNode);
+	}
 #endif
 
 	return GF_OK;
@@ -2789,6 +2950,7 @@ GF_Err gf_sm_dump_graph(GF_SceneDumper *sdump, Bool skip_proto, Bool skip_routes
 
 static void ReorderAUContext(GF_List *sample_list, GF_AUContext *au, Bool lsr_dump)
 {
+	u64 autime, time;
 	u32 i;
 	Bool has_base;
 	GF_AUContext *ptr;
@@ -2809,14 +2971,16 @@ static void ReorderAUContext(GF_List *sample_list, GF_AUContext *au, Bool lsr_du
 		au->timing = (u64) (au->timing_sec * au->owner->timeScale);
 	}
 
+	autime = au->timing + au->owner->dump_time_offset;
 	has_base = 0;
 	i=0; 
 	while ((ptr = (GF_AUContext*)gf_list_enum(sample_list, &i))) {
+		time = ptr->timing + ptr->owner->dump_time_offset;
 		if (
 			/*time ordered*/
-			(ptr->timing_sec > au->timing_sec) 
+			(time > autime) 
 			/*set bifs first for first AU*/
-			|| (!has_base && (ptr->timing_sec == au->timing_sec) && (ptr->owner->streamType < au->owner->streamType) )
+			|| (!has_base && (time == autime) && (ptr->owner->streamType < au->owner->streamType) )
 			/*set OD first for laser*/
 			|| (lsr_dump && (au->owner->streamType==GF_STREAM_OD))
 		) {
@@ -2825,7 +2989,7 @@ static void ReorderAUContext(GF_List *sample_list, GF_AUContext *au, Bool lsr_du
 		}
 
 		has_base = 0;
-		if ( (ptr->owner->streamType == au->owner->streamType) && (ptr->timing_sec == au->timing_sec) ) has_base = 1;
+		if ( (ptr->owner->streamType == au->owner->streamType) && (time == autime) ) has_base = 1;
 	}
 	gf_list_add(sample_list, au);
 }
@@ -2892,7 +3056,10 @@ GF_Err gf_sm_dump(GF_SceneManager *ctx, char *rad_name, u32 dump_mode)
 		} else {
 			SD_DumpSVG_Element(dumper, com->node, NULL, 1);
 		}
-		goto exit;
+		SD_FinalizeDump(dumper, (GF_Descriptor *) ctx->root_od);
+		gf_sm_dumper_del(dumper);
+		gf_list_del(sample_list);
+		return e;
 	}
 #endif
 
@@ -2921,7 +3088,9 @@ GF_Err gf_sm_dump(GF_SceneManager *ctx, char *rad_name, u32 dump_mode)
 				if (dumper->LSRDump) {
 					dump_od_to_saf(dumper, au, indent);
 				} else {
+#ifndef GPAC_READ_ONLY
 					e = gf_odf_dump_com_list(au->commands, dumper->trace, indent+1, 0);
+#endif				
 				}
 				break;
 			case GF_STREAM_SCENE:
@@ -2965,7 +3134,9 @@ GF_Err gf_sm_dump(GF_SceneManager *ctx, char *rad_name, u32 dump_mode)
 				if (dumper->LSRDump) {
 					dump_od_to_saf(dumper, au, indent+1);
 				} else {
+#ifndef GPAC_READ_ONLY
 					e = gf_odf_dump_com_list(au->commands, dumper->trace, indent+1, 1);
+#endif
 				}
 				break;
 			case GF_STREAM_SCENE:
@@ -3006,12 +3177,11 @@ GF_Err gf_sm_dump(GF_SceneManager *ctx, char *rad_name, u32 dump_mode)
 		GF_LOG(GF_LOG_WARNING, GF_LOG_PARSER, ("[Scene Dump] MPEG-4 Commands found, not supported in %s - skipping\n", dumper->X3DDump ? "X3D" : "VRML"));
 	}
 
-exit:
 	SD_FinalizeDump(dumper, (GF_Descriptor *) ctx->root_od);
 	gf_sm_dumper_del(dumper);
 	gf_list_del(sample_list);
 	return e;
 }
 
-#endif
+//#endif
 
