@@ -28,24 +28,22 @@
 #include <gpac/bifs.h>
 #include <gpac/xml.h>
 #include <gpac/internal/scenegraph_dev.h>
+#include <gpac/internal/bifs_dev.h>
 
 
-
-static Bool gf_node_in_table_by_tag(u32 tag, u32 NDTType)
+Bool gf_node_in_table_by_tag(u32 tag, u32 NDTType)
 {
 	if (!tag) return 0;
 	if (tag==TAG_ProtoNode) return 1;
 	else if (tag<=GF_NODE_RANGE_LAST_MPEG4) {
 		u32 i;
-		u32 gf_bifs_get_node_type(u32 NDT_Tag, u32 NodeTag, u32 Version);
 
 		for (i=0;i<GF_BIFS_LAST_VERSION; i++) {
 			if (gf_bifs_get_node_type(NDTType, tag, i+1)) return 1;
 		}
 		return 0;
 	} else if (tag<=GF_NODE_RANGE_LAST_X3D) {
-		Bool X3D_IsNodeInTable(u32 NDT_Tag, u32 NodeTag);
-		return X3D_IsNodeInTable(NDTType, tag);
+		return gf_x3d_get_node_type(NDTType, tag);
 	}
 	return 0;
 }
@@ -54,7 +52,7 @@ Bool gf_node_in_table(GF_Node *node, u32 NDTType)
 {
 	u32 tag = node ? node->sgprivate->tag : 0;
 	if (tag==TAG_ProtoNode) {
-		tag = gf_sg_proto_get_render_tag(((GF_ProtoInstance *)node)->proto_interface);
+		tag = gf_sg_proto_get_root_tag(((GF_ProtoInstance *)node)->proto_interface);
 		if (tag==TAG_UndefinedNode) return 1;
 	}
 	return gf_node_in_table_by_tag(tag, NDTType);
@@ -113,7 +111,8 @@ GF_StreamContext *gf_sm_stream_find(GF_SceneManager *ctx, u16 ES_ID)
 	}
 	return NULL;
 }
-static void gf_sm_delete_stream(GF_StreamContext *sc)
+
+static void gf_sm_reset_stream(GF_StreamContext *sc)
 {
 	while (gf_list_count(sc->AUs)) {
 		GF_AUContext *au = (GF_AUContext *)gf_list_last(sc->AUs);
@@ -134,6 +133,11 @@ static void gf_sm_delete_stream(GF_StreamContext *sc)
 		gf_list_del(au->commands);
 		free(au);
 	}
+}
+
+static void gf_sm_delete_stream(GF_StreamContext *sc)
+{
+	gf_sm_reset_stream(sc);
 	gf_list_del(sc->AUs);
 	free(sc);
 }
@@ -160,6 +164,17 @@ void gf_sm_del(GF_SceneManager *ctx)
 	free(ctx);
 }
 
+GF_EXPORT
+void gf_sm_reset(GF_SceneManager *ctx)
+{
+	GF_StreamContext *sc;
+	u32 i=0;
+	while ( (sc = gf_list_enum(ctx->streams, &i)) ) {
+		gf_sm_reset_stream(sc);
+	}
+	if (ctx->root_od) gf_odf_desc_del((GF_Descriptor *) ctx->root_od);
+	ctx->root_od = NULL;
+}
 
 GF_EXPORT
 GF_AUContext *gf_sm_stream_au_new(GF_StreamContext *stream, u64 timing, Double time_sec, Bool isRap)
@@ -236,7 +251,9 @@ GF_Err gf_sm_make_random_access(GF_SceneManager *ctx)
 			gf_list_del(com->new_proto_list);
 			com->new_proto_list = ctx->scene_graph->protos;
 			ctx->scene_graph->protos = NULL;
-			/*FIXME - check routes & protos*/
+			/*indicate the command is the aggregated scene graph, so that PROTOs and ROUTEs 
+			are taken from the scenegraph when encoding*/
+			com->aggregated = 1;
 			gf_list_add(au->commands, com);
 		}
 	}
@@ -244,54 +261,42 @@ GF_Err gf_sm_make_random_access(GF_SceneManager *ctx)
 }
 
 
-GF_Err gf_sm_load_init_BT(GF_SceneLoader *load);
-void gf_sm_load_done_BT(GF_SceneLoader *load);
-GF_Err gf_sm_load_run_BT(GF_SceneLoader *load);
-GF_Err gf_sm_load_init_BTString(GF_SceneLoader *load, char *str);
-GF_Err gf_sm_load_done_BTString(GF_SceneLoader *load);
+GF_Err gf_sm_load_init_bt(GF_SceneLoader *load);
+void gf_sm_load_done_bt(GF_SceneLoader *load);
+GF_Err gf_sm_load_run_bt(GF_SceneLoader *load);
+GF_Err gf_sm_load_init_bt_string(GF_SceneLoader *load, char *str);
+GF_Err gf_sm_load_done_bt_string(GF_SceneLoader *load);
 
 GF_Err gf_sm_load_init_xmt(GF_SceneLoader *load);
 void gf_sm_load_done_xmt(GF_SceneLoader *load);
 GF_Err gf_sm_load_run_xmt(GF_SceneLoader *load);
 GF_Err gf_sm_load_init_xmt_string(GF_SceneLoader *load, char *str);
 
-GF_Err gf_sm_load_init_MP4(GF_SceneLoader *load);
-void gf_sm_load_done_MP4(GF_SceneLoader *load);
-GF_Err gf_sm_load_run_MP4(GF_SceneLoader *load);
+GF_Err gf_sm_load_init_isom(GF_SceneLoader *load);
+void gf_sm_load_done_isom(GF_SceneLoader *load);
+GF_Err gf_sm_load_run_isom(GF_SceneLoader *load);
 
 #ifndef GPAC_DISABLE_SVG
 
-#ifdef GPAC_ENABLE_SVG_SA
-GF_Err gf_sm_load_init_svg_sa(GF_SceneLoader *load);
-GF_Err gf_sm_load_done_svg_sa(GF_SceneLoader *load);
-GF_Err gf_sm_load_run_svg_sa(GF_SceneLoader *load);
-GF_Err gf_sm_load_init_svg_sa_string(GF_SceneLoader *load, char *str);
-#endif
-
-#ifdef GPAC_ENABLE_SVG_SANI
-GF_Err gf_sm_load_init_svg_sani(GF_SceneLoader *load);
-GF_Err gf_sm_load_done_svg_sani(GF_SceneLoader *load);
-GF_Err gf_sm_load_run_svg_sani(GF_SceneLoader *load);
-GF_Err gf_sm_load_init_svg_sani_string(GF_SceneLoader *load, char *str);
-#endif
-
-
 GF_Err gf_sm_load_init_svg(GF_SceneLoader *load);
 GF_Err gf_sm_load_done_svg(GF_SceneLoader *load);
 GF_Err gf_sm_load_run_svg(GF_SceneLoader *load);
 GF_Err gf_sm_load_init_svg_string(GF_SceneLoader *load, char *str);
-#endif
 
+GF_Err gf_sm_load_init_xbl(GF_SceneLoader *load);
+GF_Err gf_sm_load_done_xbl(GF_SceneLoader *load);
+GF_Err gf_sm_load_run_xbl(GF_SceneLoader *load);
+#endif
 
 #ifndef GPAC_READ_ONLY
 
-GF_Err gf_sm_load_init_SWF(GF_SceneLoader *load);
-void gf_sm_load_done_SWF(GF_SceneLoader *load);
-GF_Err gf_sm_load_run_SWF(GF_SceneLoader *load);
+GF_Err gf_sm_load_init_swf(GF_SceneLoader *load);
+void gf_sm_load_done_swf(GF_SceneLoader *load);
+GF_Err gf_sm_load_run_swf(GF_SceneLoader *load);
 
-GF_Err gf_sm_load_init_QT(GF_SceneLoader *load);
-void gf_sm_load_done_QT(GF_SceneLoader *load);
-GF_Err gf_sm_load_run_QT(GF_SceneLoader *load);
+GF_Err gf_sm_load_init_qt(GF_SceneLoader *load);
+void gf_sm_load_done_qt(GF_SceneLoader *load);
+GF_Err gf_sm_load_run_qt(GF_SceneLoader *load);
 #endif
 
 
@@ -309,21 +314,14 @@ static GF_Err gf_sm_load_init_from_string(GF_SceneLoader *load, char *str)
 	case GF_SM_LOAD_BT: 
 	case GF_SM_LOAD_VRML:
 	case GF_SM_LOAD_X3DV:
-		return gf_sm_load_init_BTString(load, str);
+		return gf_sm_load_init_bt_string(load, str);
 	case GF_SM_LOAD_XMTA:
 	case GF_SM_LOAD_X3D:
 		return gf_sm_load_init_xmt_string(load, str);
 #ifndef GPAC_DISABLE_SVG
-#ifdef GPAC_ENABLE_SVG_SA
-	case GF_SM_LOAD_SVG_SA: 
-		return gf_sm_load_init_svg_sa_string(load, str);
-#endif
-#ifdef GPAC_ENABLE_SVG_SANI
-	case GF_SM_LOAD_SVG_SANI: 
-		return gf_sm_load_init_svg_sani_string(load, str);
-#endif
 	case GF_SM_LOAD_SVG_DA: 
 	case GF_SM_LOAD_XSR: 
+	case GF_SM_LOAD_DIMS: 
 		return gf_sm_load_init_svg_string(load, str);
 #endif
 	case GF_SM_LOAD_SWF: 
@@ -342,7 +340,7 @@ static void gf_sm_load_done_string(GF_SceneLoader *load, Bool do_clean)
 	case GF_SM_LOAD_BT:
 	case GF_SM_LOAD_VRML:
 	case GF_SM_LOAD_X3DV:
-		gf_sm_load_done_BTString(load); 
+		gf_sm_load_done_bt_string(load); 
 		break;
 	case GF_SM_LOAD_XMTA:
 	case GF_SM_LOAD_X3D:
@@ -353,15 +351,8 @@ static void gf_sm_load_done_string(GF_SceneLoader *load, Bool do_clean)
 	/*we do not reset it here to enable SAX parsing*/
 	case GF_SM_LOAD_SVG_DA:
 	case GF_SM_LOAD_XSR:
+	case GF_SM_LOAD_DIMS:
 		break;
-#ifdef GPAC_ENABLE_SVG_SA
-	case GF_SM_LOAD_SVG_SA:
-		break;
-#endif
-#ifdef GPAC_ENABLE_SVG_SANI
-	case GF_SM_LOAD_SVG_SANI:
-		break;
-#endif
 #endif
 	default: 
 		break;
@@ -375,7 +366,7 @@ GF_Err gf_sm_load_string(GF_SceneLoader *load, char *str, Bool do_clean)
 	if (e) return e;
 	e = gf_sm_load_run(load);
 	gf_sm_load_done_string(load, do_clean);
-	return e;
+	return (e<0) ? e : GF_OK;
 }
 
 
@@ -412,12 +403,14 @@ GF_Err gf_sm_load_init(GF_SceneLoader *load)
 			else if (strstr(szExt, "mov")) load->type = GF_SM_LOAD_QT;
 			else if (strstr(szExt, "svg")) load->type = GF_SM_LOAD_SVG_DA;
 			else if (strstr(szExt, "xsr")) load->type = GF_SM_LOAD_XSR;
+			else if (strstr(szExt, "xbl")) load->type = GF_SM_LOAD_XBL;
 			else if (strstr(szExt, "xml")) {
 				char *rtype = gf_xml_get_root_type(load->fileName, &e);
 				if (rtype) {
 					if (!strcmp(rtype, "SAFSession")) load->type = GF_SM_LOAD_XSR;
 					else if (!strcmp(rtype, "XMT-A")) load->type = GF_SM_LOAD_XMTA;
 					else if (!strcmp(rtype, "X3D")) load->type = GF_SM_LOAD_X3D;
+					else if (!strcmp(rtype, "bindings")) load->type = GF_SM_LOAD_XBL;
 
 					free(rtype);
 				}
@@ -432,30 +425,26 @@ GF_Err gf_sm_load_init(GF_SceneLoader *load)
 	case GF_SM_LOAD_BT: 
 	case GF_SM_LOAD_VRML:
 	case GF_SM_LOAD_X3DV:
-		return gf_sm_load_init_BT(load);
+		return gf_sm_load_init_bt(load);
 	case GF_SM_LOAD_XMTA:
 	case GF_SM_LOAD_X3D:
 		return gf_sm_load_init_xmt(load);
 #ifndef GPAC_DISABLE_SVG
-#ifdef GPAC_ENABLE_SVG_SA
-	case GF_SM_LOAD_SVG_SA:
-		return gf_sm_load_init_svg_sa(load);
-#endif
-#ifdef GPAC_ENABLE_SVG_SANI
-	case GF_SM_LOAD_SVG_SANI:
-		return gf_sm_load_init_svg_sani(load);
-#endif
 	case GF_SM_LOAD_SVG_DA:
 	case GF_SM_LOAD_XSR:
+	case GF_SM_LOAD_DIMS:
 		return gf_sm_load_init_svg(load);
+
+	case GF_SM_LOAD_XBL: 
+		return gf_sm_load_init_xbl(load);
 #endif
 #ifndef GPAC_READ_ONLY
 	case GF_SM_LOAD_SWF: 
-		return gf_sm_load_init_SWF(load);
+		return gf_sm_load_init_swf(load);
 	case GF_SM_LOAD_QT: 
-		return gf_sm_load_init_QT(load);
+		return gf_sm_load_init_qt(load);
 	case GF_SM_LOAD_MP4:
-		return gf_sm_load_init_MP4(load);
+		return gf_sm_load_init_isom(load);
 #endif
 	}
 	return GF_NOT_SUPPORTED;
@@ -468,37 +457,33 @@ void gf_sm_load_done(GF_SceneLoader *load)
 	case GF_SM_LOAD_BT:
 	case GF_SM_LOAD_VRML:
 	case GF_SM_LOAD_X3DV:
-		gf_sm_load_done_BT(load); 
+		gf_sm_load_done_bt(load); 
 		break;
 	case GF_SM_LOAD_XMTA:
 	case GF_SM_LOAD_X3D:
 		gf_sm_load_done_xmt(load); 
 		break;
 #ifndef GPAC_DISABLE_SVG
-#ifdef GPAC_ENABLE_SVG_SA
-	case GF_SM_LOAD_SVG_SA:
-		gf_sm_load_done_svg_sa(load);
-		break;
-#endif
-#ifdef GPAC_ENABLE_SVG_SANI
-	case GF_SM_LOAD_SVG_SANI:
-		gf_sm_load_done_svg_sani(load);
-		break;
-#endif
-	case GF_SM_LOAD_XSR:
 	case GF_SM_LOAD_SVG_DA:
+	case GF_SM_LOAD_XSR:
+	case GF_SM_LOAD_DIMS:
 		gf_sm_load_done_svg(load);
 		break;
+
+	case GF_SM_LOAD_XBL:
+		gf_sm_load_done_xbl(load);
+		break;
 #endif
+
 #ifndef GPAC_READ_ONLY
 	case GF_SM_LOAD_SWF: 
-		gf_sm_load_done_SWF(load); 
+		gf_sm_load_done_swf(load); 
 		break;
 	case GF_SM_LOAD_QT: 
-		gf_sm_load_done_QT(load); 
+		gf_sm_load_done_qt(load); 
 		break;
 	case GF_SM_LOAD_MP4: 
-		gf_sm_load_done_MP4(load); 
+		gf_sm_load_done_isom(load); 
 		break;
 #endif
 	}
@@ -511,64 +496,30 @@ GF_Err gf_sm_load_run(GF_SceneLoader *load)
 	case GF_SM_LOAD_BT:
 	case GF_SM_LOAD_VRML:
 	case GF_SM_LOAD_X3DV:
-		return gf_sm_load_run_BT(load);
+		return gf_sm_load_run_bt(load);
 	case GF_SM_LOAD_XMTA:
 	case GF_SM_LOAD_X3D:
 		return gf_sm_load_run_xmt(load);
 #ifndef GPAC_DISABLE_SVG
-#ifdef GPAC_ENABLE_SVG_SA
-	case GF_SM_LOAD_SVG_SA:
-		return gf_sm_load_run_svg_sa(load);
-#endif
-#ifdef GPAC_ENABLE_SVG_SANI
-	case GF_SM_LOAD_SVG_SANI:
-		return gf_sm_load_run_svg_sani(load);
-#endif
-	case GF_SM_LOAD_XSR:
 	case GF_SM_LOAD_SVG_DA:
+	case GF_SM_LOAD_XSR:
+	case GF_SM_LOAD_DIMS:
 		return gf_sm_load_run_svg(load);
 #endif
-
+#ifndef GPAC_DISABLE_SVG
+	case GF_SM_LOAD_XBL:
+		return gf_sm_load_run_xbl(load);
+#endif
 #ifndef GPAC_READ_ONLY
 	case GF_SM_LOAD_SWF:
-		return gf_sm_load_run_SWF(load);
+		return gf_sm_load_run_swf(load);
 	case GF_SM_LOAD_QT: 
-		return gf_sm_load_run_QT(load);
+		return gf_sm_load_run_qt(load);
 	case GF_SM_LOAD_MP4: 
-		return gf_sm_load_run_MP4(load);
+		return gf_sm_load_run_isom(load);
 #endif
 	default:
 		return GF_BAD_PARAM;
 	}
 }
 
-#ifdef DANAE
-void *DANAE_NewSceneLoader(char *filename, void *scene_graph)
-{
-	GF_SceneLoader *scene_loader;
-	
-	GF_SAFEALLOC(scene_loader, GF_SceneLoader);
-	scene_loader->fileName = filename;
-	scene_loader->scene_graph = scene_graph;
-   	scene_loader->flags = GF_SM_LOAD_FOR_PLAYBACK;
-	if (gf_sm_load_init(scene_loader) != GF_OK) {
-		free(scene_loader);
-		return NULL;
-	}
-	return scene_loader;
-
-}
-
-void DANAE_SceneLoader_Parse(void *p)
-{
-	gf_sm_load_run(p);
-}
-
-void DANAE_SceneLoader_Terminate(void *p)
-{
-	gf_sm_load_done(p);
-	free(p);
-}
-
-#endif
-
