@@ -156,7 +156,7 @@ static void StatFixed(GF_SceneStatistics *stat, Fixed v, Bool scale)
 	if (stat->min_fixed > v) stat->min_fixed = v;
 }
 
-static void StatSFVec2f(GF_SceneStatistics *stat, SFVec2f *val)
+static void StatSVGPoint(GF_SceneStatistics *stat, SFVec2f *val)
 {
 	if (!stat) return;
 	if (stat->max_2d.x < val->x) stat->max_2d.x = val->x;
@@ -167,6 +167,15 @@ static void StatSFVec2f(GF_SceneStatistics *stat, SFVec2f *val)
 	StatFixed(stat, val->y, 0);
 }	
 
+static void StatSFVec2f(GF_SceneStatistics *stat, SFVec2f *val)
+{
+	if (!stat) return;
+	if (stat->max_2d.x < val->x) stat->max_2d.x = val->x;
+	if (stat->max_2d.y < val->y) stat->max_2d.y = val->y;
+	if (stat->min_2d.x > val->x) stat->min_2d.x = val->x;
+	if (stat->min_2d.y > val->y) stat->min_2d.y = val->y;
+}	
+
 static void StatSFVec3f(GF_SceneStatistics *stat, SFVec3f *val)
 {
 	if (!stat) return;
@@ -185,15 +194,21 @@ static void StatField(GF_SceneStatistics *stat, GF_FieldInfo *field)
 	switch (field->fieldType) {
 	case GF_SG_VRML_SFFLOAT:
 		stat->count_float++;
+		if (stat->max_fixed < *(SFFloat*)field->far_ptr)
+			stat->max_fixed = *(SFFloat*)field->far_ptr;
+		if (stat->min_fixed > *(SFFloat*)field->far_ptr)
+			stat->min_fixed = *(SFFloat*)field->far_ptr;
 		break;
 	case GF_SG_VRML_SFCOLOR:
 		stat->count_color++;
 		break;
 	case GF_SG_VRML_SFVEC2F:
 		stat->count_2f++;
+		StatSFVec2f(stat, field->far_ptr);
 		break;
 	case GF_SG_VRML_SFVEC3F:
 		stat->count_3f++;
+		StatSFVec3f(stat, field->far_ptr);
 		break;
 
 	case GF_SG_VRML_MFFLOAT:
@@ -240,31 +255,60 @@ static void StatSVGAttribute(GF_SceneStatistics *stat, GF_FieldInfo *field)
 {
 	u32 i = 0;
 
+	stat->nb_svg_attributes++;
+
 	switch (field->fieldType) {
 	case SVG_PathData_datatype:
 		{
 #if USE_GF_PATH
+			SVG_PathData *d = (SVG_PathData *)field->far_ptr;
+			for (i=0; i<d->n_points; i++) {
+				StatSVGPoint(stat, &(d->points[i]));
+				stat->count_2d ++;
+			}		
 #else
 			SVG_PathData *d = (SVG_PathData *)field->far_ptr;
 			for (i=0; i<gf_list_count(d->points); i++) {
 				SVG_Point *p = (SVG_Point *)gf_list_get(d->points, i);
-				StatSFVec2f(stat, (SFVec2f *)p);
+				StatSVGPoint(stat, (SFVec2f *)p);
 				stat->count_2d ++;
 			}
 #endif
 		}
 		break;
+	case SVG_ViewBox_datatype:
+		{
+			SVG_ViewBox *vB = (SVG_ViewBox *)field->far_ptr;
+			StatFixed(stat, vB->x, 0);
+			StatFixed(stat, vB->y, 0);
+			StatFixed(stat, vB->width, 0);
+			StatFixed(stat, vB->height, 0);
+		}
+		break;
 	case SVG_Points_datatype:
 	case SVG_Coordinates_datatype:
 		{
 			GF_List *points = *((GF_List **)field->far_ptr);
 			for (i=0; i<gf_list_count(points); i++) {
 				SVG_Point *p = (SVG_Point *)gf_list_get(points, i);
-				StatSFVec2f(stat, (SFVec2f *)p);
+				StatSVGPoint(stat, (SFVec2f *)p);
 				stat->count_2d ++;
 			}
 		}
 		break;
+	case SVG_Transform_datatype:
+		{
+			GF_Matrix2D *mx = &((SVG_Transform *)field->far_ptr)->mat;
+			if (!gf_mx2d_is_identity(*mx) && !(!mx->m[0] && !mx->m[1] && !mx->m[3] && !mx->m[4])) {
+				StatFixed(stat, mx->m[0], 1);
+				StatFixed(stat, mx->m[1], 1);
+				StatFixed(stat, mx->m[3], 1);
+				StatFixed(stat, mx->m[4], 1);				
+				StatFixed(stat, mx->m[2], 0);
+				StatFixed(stat, mx->m[5], 0);
+			} 
+		}
+		break;
 	case SVG_Motion_datatype:
 		{
 			GF_Matrix2D *mx = (GF_Matrix2D *)field->far_ptr;
@@ -338,11 +382,16 @@ static GF_Err StatNodeGraph(GF_StatManager *st, GF_Node *n)
 
 #ifndef GPAC_DISABLE_SVG
 	if ((n->sgprivate->tag>= GF_NODE_RANGE_FIRST_SVG) && (n->sgprivate->tag<= GF_NODE_RANGE_LAST_SVG)) {
-		GF_ChildNodeItem *list = ((SVG_Element *)n)->children;;
-		count = gf_node_get_field_count(n);
-		for (i=0; i<count; i++) {
-			gf_node_get_field(n, i, &field);
+		GF_ChildNodeItem *list = ((SVG_Element *)n)->children;
+		GF_DOMAttribute *atts = ((GF_DOMNode*)n)->attributes;
+		while (atts) {
+			field.far_ptr = atts->data;
+			field.fieldType = atts->data_type;
+			field.fieldIndex = atts->tag;
+			field.name = NULL; 
 			StatSVGAttribute(st->stats, &field);
+
+			atts = atts->next;
 		}
 		while (list) {
 			StatNodeGraph(st, list->node);
@@ -350,7 +399,9 @@ static GF_Err StatNodeGraph(GF_StatManager *st, GF_Node *n)
 		}
 	} else 
 #endif
-	{
+	if (n->sgprivate->tag == TAG_DOMText) {
+	} else if (n->sgprivate->tag == TAG_DOMFullNode) {
+	} else {
 		count = gf_node_get_field_count(n);
 	
 		for (i=0; i<count; i++) {
@@ -502,6 +553,10 @@ GF_Err gf_sm_stats_for_scene(GF_StatManager *stat, GF_SceneManager *sm)
 		while ((sc = (GF_StreamContext*)gf_list_enum(sm->streams, &i))) {
 			GF_AUContext *au;
 			if (sc->streamType != GF_STREAM_SCENE) continue;
+			
+			if (!stat->stats->base_layer)
+				stat->stats->base_layer = sc;
+
 			j=0;
 			while ((au = (GF_AUContext*)gf_list_enum(sc->AUs, &j))) {
 				e = gf_sm_stat_au(au->commands, stat);
