@@ -26,7 +26,6 @@
 #include <gpac/internal/scenegraph_dev.h>
 /*MPEG4 tags (for internal nodes)*/
 #include <gpac/nodes_mpeg4.h>
-#include <gpac/nodes_svg_sa.h>
 
 #include <gpac/internal/laser_dev.h>
 
@@ -182,18 +181,20 @@ GF_Err gf_sg_command_apply(GF_SceneGraph *graph, GF_Command *com, Double time_of
 		/*remove all protos and routes*/
 		while (gf_list_count(graph->routes_to_activate)) 
 			gf_list_rem(graph->routes_to_activate, 0);
-		
-		/*destroy all routes*/
-		while (gf_list_count(graph->Routes)) {
-			GF_Route *r = (GF_Route *)gf_list_get(graph->Routes, 0);
-			/*this will unregister the route from the graph, so don't delete the chain entry*/
-			gf_sg_route_del(r);
-		}
-		/*destroy all proto*/
-		while (gf_list_count(graph->protos)) {
-			GF_Proto *p = (GF_Proto*)gf_list_get(graph->protos, 0);
-			/*this will unregister the proto from the graph, so don't delete the chain entry*/
-			gf_sg_proto_del(p);
+
+		if (!com->aggregated) {
+			/*destroy all routes*/
+			while (gf_list_count(graph->Routes)) {
+				GF_Route *r = (GF_Route *)gf_list_get(graph->Routes, 0);
+				/*this will unregister the route from the graph, so don't delete the chain entry*/
+				gf_sg_route_del(r);
+			}
+			/*destroy all proto*/
+			while (gf_list_count(graph->protos)) {
+				GF_Proto *p = (GF_Proto*)gf_list_get(graph->protos, 0);
+				/*this will unregister the proto from the graph, so don't delete the chain entry*/
+				gf_sg_proto_del(p);
+			}
 		}
 		/*DO NOT TOUCH node registry*/
 		/*DO NOT TOUCH UNREGISTERED PROTOS*/
@@ -217,6 +218,42 @@ GF_Err gf_sg_command_apply(GF_SceneGraph *graph, GF_Command *com, Double time_of
 		if (inf->new_node) gf_node_register(inf->new_node, NULL);
 		break;
 
+	case GF_SG_FIELD_REPLACE_OP:
+	{
+		void *src_ptr;
+		u32 src_type;
+		u32 j=0;
+		GF_FieldInfo src;
+		def = gf_sg_find_node(com->in_scene, com->fromNodeID);
+		if (!def) return GF_SG_UNKNOWN_NODE;
+		e = gf_node_get_field(def, com->fromFieldIndex, &src);
+		if (e) return e;
+
+		src_type = src.fieldType;
+		src_ptr = src.far_ptr;
+		if (com->send_event_x>=0) {
+			src_type = gf_sg_vrml_get_sf_type(src.fieldType);
+			e = gf_sg_vrml_mf_get_item(src.far_ptr, src.fieldType, &src_ptr, com->send_event_x);
+			if (e) return e;
+		}
+
+		while ((inf = (GF_CommandField*)gf_list_enum(com->command_fields, &j))) {
+			e = gf_node_get_field(com->node, inf->fieldIndex, &field);
+			if (e) return e;
+			if (inf->pos>=0) {
+				void *slot_ptr;
+				e = gf_sg_vrml_mf_get_item(field.far_ptr, field.fieldType, & slot_ptr, inf->pos);
+				if (e) return e;
+				gf_sg_vrml_field_copy(slot_ptr, src_ptr, src_type);
+			} else {
+				gf_sg_vrml_field_copy(field.far_ptr, src_ptr, src_type);
+			}
+		}
+
+		SG_CheckFieldChange(com->node, &field);
+	}
+		break;
+
 	case GF_SG_MULTIPLE_REPLACE:
 	case GF_SG_FIELD_REPLACE:
 	{
@@ -256,6 +293,35 @@ GF_Err gf_sg_command_apply(GF_SceneGraph *graph, GF_Command *com, Double time_of
 					list = list->next;
 				}
 				break;
+			case GF_SG_VRML_SFCOMMANDBUFFER:
+			{
+				u32 i, count;
+				GF_SceneGraph *sg;
+				SFCommandBuffer *cb_dst = (SFCommandBuffer *)field.far_ptr;
+				SFCommandBuffer *cb_src = (SFCommandBuffer *)inf->field_ptr;
+
+				/*reset dest*/
+				while (gf_list_count(cb_dst->commandList)) {
+					GF_Command *sub_com = (GF_Command *)gf_list_get(cb_dst->commandList, 0);
+					gf_sg_command_del(sub_com);
+					gf_list_rem(cb_dst->commandList, 0);
+				}
+				if (cb_dst->buffer) {
+					free(cb_dst->buffer);
+					cb_dst->buffer = NULL;
+				}
+
+				/*clone command list*/
+				sg = gf_node_get_graph(com->node);
+				count = gf_list_count(cb_src->commandList);
+				for (i=0; i<count;i++) {
+					GF_Command *sub_com = (GF_Command *)gf_list_get(cb_src->commandList, i);
+					GF_Command *new_com = gf_sg_command_clone(sub_com, sg, 0);
+					gf_list_add(cb_dst->commandList, new_com);
+				}
+			}
+				break;
+
 			default:
 				/*this is a regular field, reset it and clone - we cannot switch pointers since the
 				original fields are NOT pointers*/
@@ -264,7 +330,9 @@ GF_Err gf_sg_command_apply(GF_SceneGraph *graph, GF_Command *com, Double time_of
 				}
 				if (e) return e;
 				gf_sg_vrml_field_copy(field.far_ptr, inf->field_ptr, field.fieldType);
-				if (field.fieldType==GF_SG_VRML_SFTIME) *(SFTime *)field.far_ptr = *(SFTime *)field.far_ptr + time_offset;
+				
+				if ((field.fieldType==GF_SG_VRML_SFTIME) && !strstr(field.name, "media"))
+					*(SFTime *)field.far_ptr = *(SFTime *)field.far_ptr + time_offset;
 				break;
 			}
 			SG_CheckFieldChange(com->node, &field);
@@ -570,7 +638,7 @@ GF_Err gf_sg_command_apply(GF_SceneGraph *graph, GF_Command *com, Double time_of
 					else if (inf->fieldType == SVG_TRANSFORM_ROTATE) gf_mx2d_add_rotation(dest, 0, 0, ((SVG_Point_Angle*)inf->field_ptr)->angle);
 				}
 			} else {
-				if ((inf->fieldIndex==(u32) -1) && (inf->fieldType==SVG_String_datatype)) {
+				if ((inf->fieldIndex==(u32) -1) && (inf->fieldType==DOM_String_datatype)) {
 					char *str = *(SVG_String*)inf->field_ptr;
 
 					if (com->tag == GF_SG_LSR_REPLACE) {
@@ -589,7 +657,7 @@ GF_Err gf_sg_command_apply(GF_SceneGraph *graph, GF_Command *com, Double time_of
 					|| (inf->fieldIndex==TAG_LSR_ATT_rotation)
 				) {
 					SVG_Transform *mx;
-					gf_svg_get_attribute_by_tag(com->node, TAG_SVG_ATT_transform, 1, 0, &a);
+					gf_node_get_attribute_by_tag(com->node, TAG_SVG_ATT_transform, 1, 0, &a);
 					mx = a.far_ptr;
 					if (com->tag == GF_SG_LSR_REPLACE) {
 						GF_Point2D scale, translate;
@@ -610,7 +678,7 @@ GF_Err gf_sg_command_apply(GF_SceneGraph *graph, GF_Command *com, Double time_of
 						if (inf->fieldIndex==TAG_LSR_ATT_rotation) gf_mx2d_add_rotation(&mx->mat, 0, 0, ((SVG_Point_Angle*)inf->field_ptr)->angle);
 					}
 				}
-				else if (gf_svg_get_attribute_by_tag(com->node, inf->fieldIndex, 1, 0, &a) == GF_OK) {
+				else if (gf_node_get_attribute_by_tag(com->node, inf->fieldIndex, 1, 0, &a) == GF_OK) {
 					b = a;
 					b.far_ptr = inf->field_ptr;
 					if (com->tag == GF_SG_LSR_REPLACE) {
@@ -630,7 +698,7 @@ GF_Err gf_sg_command_apply(GF_SceneGraph *graph, GF_Command *com, Double time_of
 			if (!fromNode) return GF_NON_COMPLIANT_BITSTREAM;
 			if (gf_node_get_field(fromNode, com->fromFieldIndex, &b) != GF_OK) return GF_NON_COMPLIANT_BITSTREAM;
 
-			if ((inf->fieldIndex==(u32) -1) && (inf->fieldType==SVG_String_datatype)) {
+			if ((inf->fieldIndex==(u32) -1) && (inf->fieldType==DOM_String_datatype)) {
 				char *str = *(SVG_String*)inf->field_ptr;
 
 				if (com->tag == GF_SG_LSR_REPLACE) {
@@ -664,6 +732,17 @@ GF_Err gf_sg_command_apply(GF_SceneGraph *graph, GF_Command *com, Double time_of
 		gf_node_deactivate(com->node);
 		gf_node_changed(com->node, NULL);
 		break;
+	case GF_SG_LSR_SEND_EVENT:
+	{
+		GF_DOM_Event evt;
+		memset(&evt, 0, sizeof(GF_DOM_Event));
+		evt.type = com->send_event_name;
+		evt.detail = com->send_event_integer;
+		evt.clientX = com->send_event_x;
+		evt.clientY = com->send_event_y;
+		gf_dom_event_fire(com->node, &evt);
+	}
+		break;
 #endif
 
 	default:
@@ -705,7 +784,7 @@ GF_Err gf_sg_command_apply_list(GF_SceneGraph *graph, GF_List *comList, Double t
 	return GF_OK;
 }
 
-GF_Command *gf_sg_command_clone(GF_Command *com, GF_SceneGraph *inGraph)
+GF_Command *gf_sg_command_clone(GF_Command *com, GF_SceneGraph *inGraph, Bool force_clone)
 {
 	u32 i, count;
 	GF_Command *dest;
@@ -715,8 +794,15 @@ GF_Command *gf_sg_command_clone(GF_Command *com, GF_SceneGraph *inGraph)
 	if (gf_list_count(com->new_proto_list)) return NULL;
 	dest = gf_sg_command_new(inGraph, com->tag);
 
+	if (com->in_scene!=inGraph) force_clone = 1;
+
 	/*node the command applies to - may be NULL*/
-	dest->node = gf_node_clone(inGraph, com->node, NULL);
+	if (force_clone) {
+		dest->node = gf_node_clone(inGraph, com->node, NULL, "", 0);
+	} else {
+		dest->node = com->node;
+		gf_node_register(dest->node, NULL);
+	}
 	/*route insert, replace and delete*/
 	dest->RouteID = com->RouteID;
 	if (com->def_name) dest->def_name = strdup(com->def_name);
@@ -724,6 +810,12 @@ GF_Command *gf_sg_command_clone(GF_Command *com, GF_SceneGraph *inGraph)
 	dest->fromFieldIndex = com->fromFieldIndex;
 	dest->toNodeID = com->toNodeID;
 	dest->toFieldIndex = com->toFieldIndex;
+	dest->send_event_integer = com->send_event_integer;
+	dest->send_event_x = com->send_event_x;
+	dest->send_event_y = com->send_event_y;
+	if (com->send_event_string)
+		dest->send_event_string = strdup(com->send_event_string);
+
 	dest->del_proto_list_size = com->del_proto_list_size;
 	if (com->del_proto_list_size) {
 		dest->del_proto_list = (u32*)malloc(sizeof(u32) * com->del_proto_list_size);
@@ -743,7 +835,12 @@ GF_Command *gf_sg_command_clone(GF_Command *com, GF_SceneGraph *inGraph)
 		}
 
 		if (fo->new_node) {
-			fd->new_node = gf_node_clone(inGraph, fo->new_node, dest->node);
+			if (force_clone) {
+				fd->new_node = gf_node_clone(inGraph, fo->new_node, dest->node, "", 0);
+			} else {
+				fd->new_node = fo->new_node;
+				gf_node_register(fd->new_node, NULL);
+			}
 			fd->field_ptr = &fd->new_node;
 		}
 		if (fo->node_list) {
@@ -752,7 +849,12 @@ GF_Command *gf_sg_command_clone(GF_Command *com, GF_SceneGraph *inGraph)
 			child = fo->node_list;
 			while (child) {
 				cur = (GF_ChildNodeItem*) malloc(sizeof(GF_ChildNodeItem));
-				cur->node = gf_node_clone(inGraph, child->node, dest->node);
+				if (force_clone) {
+					cur->node = gf_node_clone(inGraph, child->node, dest->node, "", 0);
+				} else {
+					cur->node = child->node;
+					gf_node_register(cur->node, NULL);
+				}
 				cur->next = NULL;
 				if (prev) prev->next = cur;
 				else fd->node_list = cur;
