@@ -28,12 +28,12 @@
 #include <gpac/nodes_x3d.h>
 
 
-Fixed Interpolate(Fixed keyValue1, Fixed keyValue2, Fixed fraction)
+static Fixed Interpolate(Fixed keyValue1, Fixed keyValue2, Fixed fraction)
 {
 	return gf_mulfix(keyValue2 - keyValue1, fraction) + keyValue1;
 }
 
-Fixed GetInterpolateFraction(Fixed key1, Fixed key2, Fixed fraction)
+static Fixed GetInterpolateFraction(Fixed key1, Fixed key2, Fixed fraction)
 {
 	Fixed keyDiff = key2 - key1;
 	assert((fraction >= key1) && (fraction <= key2));
@@ -81,7 +81,7 @@ static void CI2D_SetFraction(GF_Node *n)
 		}
 	}
 	//invalidate
-	gf_node_event_out_str(n, "value_changed");
+	gf_node_event_out(n, 3);//"value_changed"
 }
 
 Bool InitCoordinateInterpolator2D(M_CoordinateInterpolator2D *node)
@@ -146,7 +146,7 @@ static void CoordInt_SetFraction(GF_Node *n)
 	M_CoordinateInterpolator *_this = (M_CoordinateInterpolator *) n;
 
 	if (CI_SetFraction(_this->set_fraction, &_this->value_changed, &_this->key, &_this->keyValue)) 
-		gf_node_event_out_str(n, "value_changed");
+		gf_node_event_out(n, 3);//"value_changed"
 }
 
 Bool InitCoordinateInterpolator(M_CoordinateInterpolator *n)
@@ -166,7 +166,7 @@ static void NormInt_SetFraction(GF_Node *n)
 	for (i=0; i<_this->value_changed.count; i++) {
 		gf_vec_norm(&_this->value_changed.vals[i]);
 	}
-	gf_node_event_out_str(n, "value_changed");
+	gf_node_event_out(n, 3);//"value_changed"
 }
 
 Bool InitNormalInterpolator(M_NormalInterpolator *n)
@@ -210,7 +210,7 @@ static void ColorInt_SetFraction(GF_Node *node)
 			break;
 		}
 	}
-	gf_node_event_out_str(node, "value_changed");
+	gf_node_event_out(node, 3);//"value_changed"
 }
 
 Bool InitColorInterpolator(M_ColorInterpolator *node)
@@ -247,7 +247,7 @@ static void PosInt2D_SetFraction(GF_Node *node)
 			break;
 		}
 	}
-	gf_node_event_out_str(node, "value_changed");
+	gf_node_event_out(node, 3);//"value_changed"
 }
 
 Bool InitPositionInterpolator2D(M_PositionInterpolator2D *node)
@@ -284,7 +284,7 @@ static void PosInt_SetFraction(GF_Node *node)
 			break;
 		}
 	}
-	gf_node_event_out_str(node, "value_changed");
+	gf_node_event_out(node, 3);//"value_changed"
 }
 
 Bool InitPositionInterpolator(M_PositionInterpolator *node)
@@ -319,7 +319,7 @@ static void ScalarInt_SetFraction(GF_Node *node)
 			break;
 		}
 	}
-	gf_node_event_out_str(node, "value_changed");
+	gf_node_event_out(node, 3);//"value_changed"
 }
 Bool InitScalarInterpolator(M_ScalarInterpolator *node)
 {
@@ -328,8 +328,6 @@ Bool InitScalarInterpolator(M_ScalarInterpolator *node)
 	return 1;
 }
 
-
-/*taken from freeWRL*/
 GF_EXPORT
 SFRotation gf_sg_sfrotation_interpolate(SFRotation kv1, SFRotation kv2, Fixed fraction)
 {
@@ -351,29 +349,11 @@ SFRotation gf_sg_sfrotation_interpolate(SFRotation kv1, SFRotation kv2, Fixed fr
 		newa = -kv2.q;
 	}
 	olda = kv1.q;
-	testa = newa - olda;
-	/* make it so we smoothly transition */
-	if (ABS(testa) > GF_PI) {
-		if (ABS(testa) > GF_2PI) {
-			if (testa>0) {
-				olda += 2*GF_2PI;
-			} else {
-				newa += 2*GF_2PI; 
-			}
-		} else {
-			if (testa>0) {
-				olda += GF_2PI;
-			} else { 
-				newa += GF_2PI; 
-			}
-		}
-	}
 	if (stzero || endzero) {
 		res.x = stzero ? kv2.x : kv1.x;
 		res.y = stzero ? kv2.y : kv1.y;
 		res.z = stzero ? kv2.z : kv1.z;
 	}
-	/* now that we have angles straight (hah!) bounds check result */
 	res.q = olda + gf_mulfix(fraction, newa - olda);
 	if (res.q > GF_2PI) { 
 		res.q -= GF_2PI;
@@ -408,7 +388,7 @@ static void OrientInt_SetFraction(GF_Node *node)
 			break;
 		}
 	}
-	gf_node_event_out_str(node, "value_changed");
+	gf_node_event_out(node, 3);//"value_changed"
 }
 
 Bool InitOrientationInterpolator(M_OrientationInterpolator *node)
@@ -464,7 +444,7 @@ static void CI4D_SetFraction(GF_Node *n)
 		}
 	}
 	//invalidate
-	gf_node_event_out_str(n, "value_changed");
+	gf_node_event_out(n, 3);//"value_changed"
 }
 
 Bool InitCoordinateInterpolator4D(M_CoordinateInterpolator4D *node)
@@ -508,7 +488,7 @@ static void PI4D_SetFraction(GF_Node *node)
 			break;
 		}
 	}
-	gf_node_event_out_str(node, "value_changed");
+	gf_node_event_out(node, 3);//"value_changed"
 }
 
 Bool InitPositionInterpolator4D(M_PositionInterpolator4D *node)
@@ -560,7 +540,7 @@ static void BooleanSequencer_setFraction(GF_Node *n)
 			break;
 		}
 	}
-	gf_node_event_out_str(n, "value_changed");
+	gf_node_event_out(n, 3);//"value_changed"
 }
 
 static void BooleanSequencer_setNext(GF_Node *n)
@@ -573,7 +553,7 @@ static void BooleanSequencer_setNext(GF_Node *n)
 	val = (*prev_val + 1) % bs->keyValue.count;
 	*prev_val = val;
 	bs->value_changed = bs->keyValue.vals[val];
-	gf_node_event_out_str(n, "value_changed");
+	gf_node_event_out(n, 3);//"value_changed"
 }
 
 static void BooleanSequencer_setPrevious(GF_Node *n)
@@ -588,7 +568,7 @@ static void BooleanSequencer_setPrevious(GF_Node *n)
 	val %= bs->keyValue.count;
 	*prev_val = val;
 	bs->value_changed = bs->keyValue.vals[val];
-	gf_node_event_out_str(n, "value_changed");
+	gf_node_event_out(n, 3);//"value_changed"
 }
 static void DestroyBooleanSequencer(GF_Node *n, void *eff, Bool is_destroy)
 {
@@ -653,7 +633,7 @@ static void IntegerSequencer_setFraction(GF_Node *n)
 			break;
 		}
 	}
-	gf_node_event_out_str(n, "value_changed");
+	gf_node_event_out(n, 3);//"value_changed"
 }
 
 static void IntegerSequencer_setNext(GF_Node *n)
@@ -666,7 +646,7 @@ static void IntegerSequencer_setNext(GF_Node *n)
 	val = (*prev_val + 1) % is->keyValue.count;
 	*prev_val = val;
 	is->value_changed = is->keyValue.vals[val];
-	gf_node_event_out_str(n, "value_changed");
+	gf_node_event_out(n, 3);//"value_changed"
 }
 
 static void IntegerSequencer_setPrevious(GF_Node *n)
@@ -681,7 +661,7 @@ static void IntegerSequencer_setPrevious(GF_Node *n)
 	val %= is->keyValue.count;
 	*prev_val = val;
 	is->value_changed = is->keyValue.vals[val];
-	gf_node_event_out_str(n, "value_changed");
+	gf_node_event_out(n, 3);//"value_changed"
 }
 static void DestroyIntegerSequencer(GF_Node *n, void *eff, Bool is_destroy)
 {
