@@ -341,11 +341,11 @@ s32 gf_sg_proto_get_field_index_by_name(GF_Proto *proto, GF_Node *node, char *na
 	return -1;
 }
 
-GF_EXPORT
-GF_Node *gf_node_clone(GF_SceneGraph *inScene, GF_Node *orig, GF_Node *cloned_parent)
+
+GF_Node *gf_vrml_node_clone(GF_SceneGraph *inScene, GF_Node *orig, GF_Node *cloned_parent, char *inst_id_suffix)
 {
 	u32 i, count, id;
-	const char *orig_name;
+	char *szNodeName;
 	Bool is_script;
 	GF_Node *node, *child;
 	GF_ChildNodeItem *list, *last;
@@ -359,12 +359,27 @@ GF_Node *gf_node_clone(GF_SceneGraph *inScene, GF_Node *orig, GF_Node *cloned_pa
 	if (!orig) return NULL;
 
 	/*check for DEF/USE*/
-	orig_name  = gf_node_get_name_and_id(orig, &id);
+	szNodeName = NULL;
+	if (!inst_id_suffix) id = 0;
+	else {
+		const char *orig_name = gf_node_get_name_and_id(orig, &id);
+		/*generate clone IDs based on original one*/
+		if (inst_id_suffix[0] && id) {
+			id = gf_sg_get_next_available_node_id(inScene);
+			if (orig_name) {
+				szNodeName = malloc(sizeof(char)*(strlen(orig_name)+strlen(inst_id_suffix)+1));
+				strcpy(szNodeName, orig_name);
+				strcat(szNodeName, inst_id_suffix);
+			}
+		}
+	}
+
 	if (id) {
-		node = gf_sg_find_node(inScene, id);
+		node = szNodeName ? gf_sg_find_node_by_name(inScene, szNodeName) : gf_sg_find_node(inScene, id);
 		/*node already created, USE*/
 		if (node) {
 			gf_node_register(node, cloned_parent);
+			if (inst_id_suffix[0] && szNodeName) free(szNodeName);
 			return node;
 		}
 	}
@@ -396,14 +411,14 @@ GF_Node *gf_node_clone(GF_SceneGraph *inScene, GF_Node *orig, GF_Node *cloned_pa
 		/*duplicate it*/
 		switch (field.fieldType) {
 		case GF_SG_VRML_SFNODE:
-			child = gf_node_clone(inScene, (* ((GF_Node **) field_orig.far_ptr)), node);
+			child = gf_node_clone(inScene, (* ((GF_Node **) field_orig.far_ptr)), node, inst_id_suffix, 1);
 			*((GF_Node **) field.far_ptr) = child;
 			break;
 		case GF_SG_VRML_MFNODE:
 			last = NULL;
 			list = *( (GF_ChildNodeItem **) field_orig.far_ptr);
 			while (list) {
-				child = gf_node_clone(inScene, list->node, node);
+				child = gf_node_clone(inScene, list->node, node, inst_id_suffix, 1);
 				gf_node_list_add_child_last((GF_ChildNodeItem **) field.far_ptr, child, &last);
 				list = list->next;
 			}
@@ -431,14 +446,15 @@ GF_Node *gf_node_clone(GF_SceneGraph *inScene, GF_Node *orig, GF_Node *cloned_pa
 		M_InputSensor *clone_is = (M_InputSensor *)node;
 		M_InputSensor *orig_is = (M_InputSensor *)orig;
 		while ( (com_o = (GF_Command *)gf_list_enum(orig_is->buffer.commandList, &k) ) ) {
-			com_f = gf_sg_command_clone(com_o, node->sgprivate->scenegraph);
+			com_f = gf_sg_command_clone(com_o, node->sgprivate->scenegraph, 1);
 			gf_list_add(clone_is->buffer.commandList, com_f);
 		}
 	}
 
 	/*register node*/
-	if (id ) {
-		gf_node_set_id(node, id, orig_name);
+	if (id) {
+		gf_node_set_id(node, id, szNodeName);
+		if (inst_id_suffix[0] && szNodeName) free(szNodeName);
 	}
 	gf_node_register(node, cloned_parent);
 
@@ -482,26 +498,13 @@ GF_Node *gf_node_clone(GF_SceneGraph *inScene, GF_Node *orig, GF_Node *cloned_pa
 	if (node->sgprivate->tag == TAG_ProtoNode) {
 		node->sgprivate->UserCallback = NULL;
 		node->sgprivate->UserPrivate = NULL;
-		/*NO RENDER, this is filtered at the generic gf_node_render to cope with instanciations and externProto*/
+		/*NO RENDER, this is filtered at the generic gf_node_traverse to cope with instanciations and externProto*/
 		/*load code*/
 		gf_sg_proto_instanciate((GF_ProtoInstance *)node);
 	}
 	return node;
 }
 
-
-#ifdef GF_NODE_USE_POINTERS
-static GF_Err protoinst_get_field(GF_Node *node, GF_FieldInfo *info)
-{
-	info->NDTtype = NDT_SFWorldNode;
-	return gf_sg_proto_get_field(NULL, node, info);
-}
-static void protoinst_del(GF_Node *n)
-{
-	gf_sg_proto_del_instance((GF_ProtoInstance *)n);
-}
-#endif
-
 GF_Err gf_sg_proto_get_field_ind_static(GF_Node *Node, u32 inField, u8 IndexMode, u32 *allField)
 {
 	return gf_sg_proto_get_field_index((GF_ProtoInstance *)Node, inField, IndexMode, allField);
@@ -548,7 +551,7 @@ void gf_sg_proto_instanciate(GF_ProtoInstance *proto_node)
 {
 	GF_Node *node, *orig;
 	GF_Route *route, *r2;
-	u32 i;
+	u32 i, count;
 	GF_Proto *proto = proto_node->proto_interface;
 	GF_Proto *owner = proto;
 
@@ -563,6 +566,7 @@ void gf_sg_proto_instanciate(GF_ProtoInstance *proto_node)
 
 		/*this is an hardcoded proto - all routes, node modifications and co are handled internally*/
 		if (extern_lib == GF_SG_INTERNAL_PROTO) {
+			proto_node->sgprivate->flags |= GF_SG_NODE_DIRTY;
 			owner->parent_graph->NodeCallback(owner->parent_graph->userpriv, GF_SG_CALLBACK_INIT, (GF_Node *) proto_node, NULL);
 			return;
 		}
@@ -579,19 +583,22 @@ void gf_sg_proto_instanciate(GF_ProtoInstance *proto_node)
 				szName++;
 				if (sscanf(szName, "%d", &ID)) ID = (u32) -1;
 			}
+			/*if we have the proto name, use it*/
+			if (owner->Name) szName = owner->Name;
 			proto = gf_sg_find_proto(extern_lib, ID, szName);
 		}
-		if (!proto) proto = gf_sg_find_proto(extern_lib, owner->ID, NULL);
+		if (!proto) proto = gf_sg_find_proto(extern_lib, owner->ID, owner->Name);
 		if (!proto) proto = SG_FindProtoByInterface(extern_lib, owner);
 		/*couldn't find proto in the given lib, consider the proto as loaded (give up)*/
 		if (!proto) {
 			proto_node->is_loaded = 1;
 			return;
 		}
-		i=0;
-		while ((pfi = (GF_ProtoFieldInterface*)gf_list_enum(owner->proto_fields, &i))) {
+		/*cf VRML: once an external proto is loaded, copy back the default field values of the external proto*/
+		count = gf_list_count(owner->proto_fields);
+		for (i=0; i<count; i++) {
 			GF_ProtoField *pf = (GF_ProtoField *)gf_list_get(proto_node->fields, i);
-			if (pfi->val_not_loaded && !pf->has_been_accessed) {
+			if (!pf->has_been_accessed) {
 				pfi = (GF_ProtoFieldInterface*)gf_list_get(proto->proto_fields, i);
 				gf_sg_vrml_field_copy(pf->field_pointer, pfi->def_value, pfi->FieldType);
 			}
@@ -609,7 +616,7 @@ void gf_sg_proto_instanciate(GF_ProtoInstance *proto_node)
 	i=0;
 	while ((orig = (GF_Node*)gf_list_enum(proto->node_code, &i))) {
 		/*node is cloned in the new scenegraph and its parent is NULL */
-		node = gf_node_clone(proto_node->sgprivate->scenegraph, orig, NULL);
+		node = gf_node_clone(proto_node->sgprivate->scenegraph, orig, NULL, "", 1);
 		assert(node);
 
 		/*assign first rendering node*/
@@ -659,6 +666,13 @@ void gf_sg_proto_instanciate(GF_ProtoInstance *proto_node)
 	proto_node->is_loaded = 1;
 }
 
+void gf_sg_proto_mark_field_loaded(GF_Node *proto_inst, GF_FieldInfo *info)
+{	
+	GF_ProtoInstance *inst= (proto_inst->sgprivate->tag==TAG_ProtoNode) ? (GF_ProtoInstance *)proto_inst : NULL;
+	GF_ProtoField *pf = inst ? (GF_ProtoField *)gf_list_get(inst->fields, info->fieldIndex) : NULL;
+	if (pf) pf->has_been_accessed = 1;
+}
+
 GF_Node *gf_sg_proto_create_node(GF_SceneGraph *scene, GF_Proto *proto, GF_ProtoInstance *from_inst)
 {
 	u32 i;
@@ -677,15 +691,7 @@ GF_Node *gf_sg_proto_create_node(GF_SceneGraph *scene, GF_Proto *proto, GF_Proto
 	proto_node->proto_interface = proto;
 	gf_list_add(proto->instances, proto_node);
 
-#ifdef GF_NODE_USE_POINTERS
-	proto_node->sgprivate->node_del = protoinst_del;
-	proto_node->sgprivate->get_field = protoinst_get_field;
-	proto_node->sgprivate->get_field_count = gf_sg_proto_get_num_fields;
-	proto_node->sgprivate->name = strdup(proto->Name);
-#else
 	proto_node->proto_name = strdup(proto->Name);
-#endif
-
 
 	/*create the namespace*/
 	proto_node->sgprivate->scenegraph = gf_sg_new_subscene(scene);
@@ -695,10 +701,9 @@ GF_Node *gf_sg_proto_create_node(GF_SceneGraph *scene, GF_Proto *proto, GF_Proto
 	/*instanciate fields*/
 	i=0;
 	while ((field = (GF_ProtoFieldInterface*)gf_list_enum(proto->proto_fields, &i))) {
-		inst = (GF_ProtoField *)malloc(sizeof(GF_ProtoField));
+		GF_SAFEALLOC(inst, GF_ProtoField);
 		inst->EventType = field->EventType;
 		inst->FieldType = field->FieldType;
-		inst->has_been_accessed = 0;
 
 		/*this is OK to call on GF_Node (returns NULL) and MFNode (returns gf_list_new() )*/
 		inst->field_pointer = gf_sg_vrml_field_pointer_new(inst->FieldType);
@@ -709,6 +714,7 @@ GF_Node *gf_sg_proto_create_node(GF_SceneGraph *scene, GF_Proto *proto, GF_Proto
 			if (from_inst) {
 				from_field = (GF_ProtoField *)gf_list_get(from_inst->fields, i-1);
 				gf_sg_vrml_field_copy(inst->field_pointer, from_field->field_pointer, inst->FieldType);
+				inst->has_been_accessed = from_field->has_been_accessed;
 			} else {
 				gf_sg_vrml_field_copy(inst->field_pointer, field->def_value, inst->FieldType);
 			}
@@ -760,6 +766,7 @@ u32 gf_sg_proto_get_num_fields(GF_Node *node, u8 code_mode)
 
 void gf_sg_proto_del_instance(GF_ProtoInstance *inst)
 {
+	GF_SceneGraph *sg;
 	GF_ProtoField *field;
 	GF_Node *node;
 	u32 index;
@@ -806,20 +813,24 @@ void gf_sg_proto_del_instance(GF_ProtoInstance *inst)
 
 	if (inst->proto_interface) gf_list_del_item(inst->proto_interface->instances, inst);
 
-	gf_sg_del(inst->sgprivate->scenegraph);
+	sg = inst->sgprivate->scenegraph;
 
-#ifdef GF_NODE_USE_POINTERS
-	/*this is duplicated for proto since a proto declaration may be destroyed while instances are active*/
-	free((char *) inst->sgprivate->name);
-#else
 	free((char *) inst->proto_name);
-#endif
 
-	gf_node_free((GF_Node *)inst);
+	sg->pOwningProto = NULL;
+	/*and finally destroy the node. If the proto is a hardcoded one (UserCallback set), destroy the node first
+	since the hardcoded proto may need the scene graph when being destroyed*/
+	if (inst->sgprivate->UserCallback) {
+		gf_node_free((GF_Node *)inst);
+		gf_sg_del(sg);
+	} else {
+		gf_sg_del(sg);
+		gf_node_free((GF_Node *)inst);
+	}
 }
 
-/*Note on ISed fields: we cannot support fan-in on proto, eg we assume only one eventIn field can recieve events
-thus situations where a proto recieves eventIn from outside and the node with ISed eventIn recieves event 
+/*Note on ISed fields: we cannot support fan-in on proto, eg we assume only one eventIn field can receive events
+thus situations where a proto receives eventIn from outside and the node with ISed eventIn receives event 
 from inside the proto are undefined*/
 GF_Err gf_sg_proto_field_set_ised(GF_Proto *proto, u32 protoFieldIndex, GF_Node *node, u32 nodeFieldIndex)
 {
@@ -852,8 +863,8 @@ GF_Err gf_sg_proto_field_set_ised(GF_Proto *proto, u32 protoFieldIndex, GF_Node
 		r->ToField.fieldIndex = protoFieldIndex;
 		r->ToNode = NULL;
 		if (!node->sgprivate->interact) GF_SAFEALLOC(node->sgprivate->interact, struct _node_interactive_ext);
-		if (!node->sgprivate->interact->events) node->sgprivate->interact->events = gf_list_new();
-		gf_list_add(node->sgprivate->interact->events, r);
+		if (!node->sgprivate->interact->routes) node->sgprivate->interact->routes = gf_list_new();
+		gf_list_add(node->sgprivate->interact->routes, r);
 	} else {
 		switch (field.eventType) {
 		case GF_SG_EVENT_FIELD:
@@ -870,7 +881,7 @@ GF_Err gf_sg_proto_field_set_ised(GF_Proto *proto, u32 protoFieldIndex, GF_Node
 			r->ToField.fieldIndex = protoFieldIndex;
 			r->ToNode = NULL;
 			if (!node->sgprivate->interact) GF_SAFEALLOC(node->sgprivate->interact, struct _node_interactive_ext);
-			if (!node->sgprivate->interact->events) node->sgprivate->interact->events = gf_list_new();
+			if (!node->sgprivate->interact->routes) node->sgprivate->interact->routes = gf_list_new();
 			break;
 		default:
 			free(r);
@@ -913,8 +924,8 @@ GF_Err gf_sg_proto_instance_set_ised(GF_Node *protoinst, u32 protoFieldIndex, GF
 		r->ToField.fieldIndex = protoFieldIndex;
 		r->ToNode = protoinst;
 		if (!node->sgprivate->interact) GF_SAFEALLOC(node->sgprivate->interact, struct _node_interactive_ext);
-		if (!node->sgprivate->interact->events) node->sgprivate->interact->events = gf_list_new();
-		gf_list_add(node->sgprivate->interact->events, r);
+		if (!node->sgprivate->interact->routes) node->sgprivate->interact->routes = gf_list_new();
+		gf_list_add(node->sgprivate->interact->routes, r);
 	} else {
 		switch (field.eventType) {
 		case GF_SG_EVENT_FIELD:
@@ -931,8 +942,8 @@ GF_Err gf_sg_proto_instance_set_ised(GF_Node *protoinst, u32 protoFieldIndex, GF
 			r->ToField.fieldIndex = protoFieldIndex;
 			r->ToNode = protoinst;
 			if (!node->sgprivate->interact) GF_SAFEALLOC(node->sgprivate->interact, struct _node_interactive_ext);
-			if (!node->sgprivate->interact->events) node->sgprivate->interact->events = gf_list_new();
-			gf_list_add(node->sgprivate->interact->events, r);
+			if (!node->sgprivate->interact->routes) node->sgprivate->interact->routes = gf_list_new();
+			gf_list_add(node->sgprivate->interact->routes, r);
 			break;
 		default:
 			free(r);
@@ -1036,9 +1047,9 @@ void gf_sg_proto_check_field_change(GF_Node *node, u32 fieldIndex)
 	GF_Route *r;
 	if (!node) return;
 
-	if ((node->sgprivate->tag == TAG_ProtoNode) && node->sgprivate->interact && node->sgprivate->interact->events){
+	if ((node->sgprivate->tag == TAG_ProtoNode) && node->sgprivate->interact && node->sgprivate->interact->routes){
 		i=0;
-		while ((r = (GF_Route*)gf_list_enum(node->sgprivate->interact->events, &i))) {
+		while ((r = (GF_Route*)gf_list_enum(node->sgprivate->interact->routes, &i))) {
 			if (!r->IS_route) continue;
 			/*eventIn or exposedField*/
 			if ((r->FromNode == node) && (r->FromField.fieldIndex == fieldIndex) ) {
@@ -1060,7 +1071,7 @@ void gf_sg_proto_check_field_change(GF_Node *node, u32 fieldIndex)
 
 	/*search for IS routes_events in the node and activate them. Field can also be an eventOut !!*/
 	i=0;
-	while ((r = (GF_Route*)gf_list_enum(node->sgprivate->interact->events, &i))) {
+	while ((r = (GF_Route*)gf_list_enum(node->sgprivate->interact->routes, &i))) {
 		if (!r->IS_route) continue;
 		/*activate eventOuts*/
 		if ((r->FromNode == node) && (r->FromField.fieldIndex == fieldIndex)) {
@@ -1143,13 +1154,13 @@ const char *gf_sg_proto_get_class_name(GF_Proto *proto)
 	return (const char *) proto->Name;
 }
 
-u32 gf_sg_proto_get_render_tag(GF_Proto *proto)
+u32 gf_sg_proto_get_root_tag(GF_Proto *proto)
 {
 	GF_Node *n;
 	if (!proto) return TAG_UndefinedNode;
 	n = (GF_Node*)gf_list_get(proto->node_code, 0);
 	if (!n) return TAG_UndefinedNode;
-	if (n->sgprivate->tag == TAG_ProtoNode) return gf_sg_proto_get_render_tag(((GF_ProtoInstance *)n)->proto_interface);
+	if (n->sgprivate->tag == TAG_ProtoNode) return gf_sg_proto_get_root_tag(((GF_ProtoInstance *)n)->proto_interface);
 	return n->sgprivate->tag;
 }
 
@@ -1190,15 +1201,31 @@ GF_SceneGraph *Node_GetExternProtoScene(GF_Node *node)
 	return sg;
 }
 
-void gf_sg_proto_field_set_value_undefined(GF_ProtoFieldInterface *protofield)
+
+GF_EXPORT
+GF_Node *gf_node_get_proto_root(GF_Node *node)
 {
-	if (protofield) protofield->val_not_loaded = 1;
+	if (!node || (node->sgprivate->tag!=TAG_ProtoNode)) return NULL;
+	return ((GF_ProtoInstance *)node)->RenderingNode;
 }
 
-void gf_sg_proto_mark_field_loaded(GF_Node *proto_inst, GF_FieldInfo *info)
-{	
-	GF_ProtoInstance *inst= (proto_inst->sgprivate->tag==TAG_ProtoNode) ? (GF_ProtoInstance *)proto_inst : NULL;
-	GF_ProtoField *pf = inst ? (GF_ProtoField *)gf_list_get(inst->fields, info->fieldIndex) : NULL;
-	if (pf) pf->has_been_accessed = 1;
+GF_EXPORT
+GF_Node *gf_node_get_proto_parent(GF_Node *node)
+{
+	if (!node) return NULL;
+	if (node->sgprivate->scenegraph->pOwningProto) {
+		GF_Node *the_node = (GF_Node *) node->sgprivate->scenegraph->pOwningProto;
+		if (the_node != node) return the_node;
+	} 
+	return NULL;
 }
 
+GF_EXPORT
+Bool gf_node_is_proto_root(GF_Node *node)
+{
+	if (!node) return 0;
+	if (!node->sgprivate->scenegraph->pOwningProto) return 0;
+
+	if (gf_list_find(node->sgprivate->scenegraph->pOwningProto->node_code, node)>=0) return 1;
+	return 0;
+}
