@@ -43,12 +43,13 @@ GF_Route *gf_sg_route_new(GF_SceneGraph *sg, GF_Node *fromNode, u32 fromField, G
 	r->graph = sg;
 
 	if (!fromNode->sgprivate->interact) GF_SAFEALLOC(fromNode->sgprivate->interact, struct _node_interactive_ext);
-	if (!fromNode->sgprivate->interact->events) fromNode->sgprivate->interact->events = gf_list_new();
-	gf_list_add(fromNode->sgprivate->interact->events, r);
+	if (!fromNode->sgprivate->interact->routes) fromNode->sgprivate->interact->routes = gf_list_new();
+	gf_list_add(fromNode->sgprivate->interact->routes, r);
 	gf_list_add(sg->Routes, r);
 	return r;
 }
 
+
 void gf_sg_route_del(GF_Route *r)
 {
 	GF_SceneGraph *sg;
@@ -58,12 +59,12 @@ void gf_sg_route_del(GF_Route *r)
 
 	/*remove declared routes*/
 	ind = gf_list_del_item(r->graph->Routes, r);
-	/*remove route from node*/
-	if (r->FromNode && r->FromNode->sgprivate->interact && r->FromNode->sgprivate->interact->events) {
-		gf_list_del_item(r->FromNode->sgprivate->interact->events, r);
-		if (!gf_list_count(r->FromNode->sgprivate->interact->events)) {
-			gf_list_del(r->FromNode->sgprivate->interact->events);
-			r->FromNode->sgprivate->interact->events = NULL;
+	/*remove route from node - do this regardless of setup state since the route is registered upon creation*/
+	if (r->FromNode && r->FromNode->sgprivate->interact && r->FromNode->sgprivate->interact->routes) {
+		gf_list_del_item(r->FromNode->sgprivate->interact->routes, r);
+		if (!gf_list_count(r->FromNode->sgprivate->interact->routes)) {
+			gf_list_del(r->FromNode->sgprivate->interact->routes);
+			r->FromNode->sgprivate->interact->routes = NULL;
 		}
 	}
 	r->is_setup = 0;
@@ -83,6 +84,70 @@ GF_Err gf_sg_route_del_by_id(GF_SceneGraph *sg,u32 routeID)
 	return GF_OK;
 }
 
+void gf_sg_destroy_routes(GF_SceneGraph *sg)
+{
+	while (gf_list_count(sg->routes_to_destroy) ) {
+		GF_Route *r = (GF_Route *)gf_list_get(sg->routes_to_destroy, 0);
+		gf_list_rem(sg->routes_to_destroy, 0);
+		gf_sg_route_unqueue(sg, r);
+		if (r->name) free(r->name);
+		free(r);
+	}
+}
+
+
+void gf_sg_route_queue(GF_SceneGraph *sg, GF_Route *r)
+{
+	u32 now;
+	if (!sg) return;
+
+	/*get the top level scene (that's the only reliable one regarding simulatioin tick)*/
+	while (sg->parent_scene) sg = sg->parent_scene;
+	/*a single route may not be activated more than once in a simulation tick*/
+	now = 1 + sg->simulation_tick;
+	if (r->lastActivateTime >= now) return;
+	r->lastActivateTime = now;
+	gf_list_add(sg->routes_to_activate, r);
+}
+
+/*activate all routes in the order they where triggered*/
+GF_EXPORT
+void gf_sg_activate_routes(GF_SceneGraph *sg)
+{
+	GF_Route *r;
+	GF_Node *targ;
+	if (!sg) return;
+
+	sg->simulation_tick++;
+
+	while (gf_list_count(sg->routes_to_activate)) {
+		r = (GF_Route *)gf_list_get(sg->routes_to_activate, 0);
+		gf_list_rem(sg->routes_to_activate, 0);
+		if (r) {
+			targ = r->ToNode;
+			if (gf_sg_route_activate(r)) {
+#ifdef GF_SELF_REPLACE_ENABLE
+				if (sg->graph_has_been_reset) {
+					sg->graph_has_been_reset = 0;
+					return;
+				}
+#endif
+				if (r->is_setup) gf_node_changed(targ, &r->ToField);
+			}
+		}
+	}
+	gf_sg_destroy_routes(sg);
+}
+
+void gf_sg_route_unqueue(GF_SceneGraph *sg, GF_Route *r)
+{
+	/*get the top level scene*/
+	while (sg->parent_scene) sg = sg->parent_scene;
+	/*remove route from queue list*/
+	gf_list_del_item(sg->routes_to_activate, r);
+}
+
+
 GF_Route *gf_sg_route_find(GF_SceneGraph *sg, u32 RouteID)
 {
 	GF_Route *r;
@@ -155,13 +220,14 @@ Bool gf_sg_route_activate(GF_Route *r)
 		/*special case when initing ISed routes on eventOuts: skip*/
 		if (r->IS_route) {
 			if (r->FromField.eventType == GF_SG_EVENT_OUT) return 0;
+			if (r->ToField.eventType == GF_SG_EVENT_OUT) return 0;
 		}
 	}
 #ifndef GPAC_DISABLE_LOG
 	if (r->IS_route) {
-		GF_LOG(GF_LOG_DEBUG, GF_LOG_COMPOSE, ("[VRML Event] executing %s.%s IS %s.%s\n", gf_node_get_name(r->FromNode), r->FromField.name, gf_node_get_name(r->ToNode), r->ToField.name));
+		GF_LOG(GF_LOG_DEBUG, GF_LOG_INTERACT, ("[VRML Event] executing %s.%s IS %s.%s\n", gf_node_get_name(r->FromNode), r->FromField.name, gf_node_get_name(r->ToNode), r->ToField.name));
 	} else {
-		GF_LOG(GF_LOG_DEBUG, GF_LOG_COMPOSE, ("[VRML Event] executing ROUTE %s.%s TO %s.%s\n", gf_node_get_name(r->FromNode), r->FromField.name, gf_node_get_name(r->ToNode), r->ToField.name));
+		GF_LOG(GF_LOG_DEBUG, GF_LOG_INTERACT, ("[VRML Event] executing ROUTE %s.%s TO %s.%s\n", gf_node_get_name(r->FromNode), r->FromField.name, gf_node_get_name(r->ToNode), r->ToField.name));
 	}
 #endif
 
@@ -243,13 +309,11 @@ void gf_node_event_out(GF_Node *node, u32 FieldIndex)
 	GF_Route *r;
 	if (!node) return;
 	
-	//this is not an ISed
-	if (!(node->sgprivate->flags & GF_NODE_IS_DEF) && !node->sgprivate->scenegraph->pOwningProto) return;
 	if (!node->sgprivate->interact) return;
 	
 	//search for routes to activate in the order they where declared
 	i=0;
-	while ((r = (GF_Route*)gf_list_enum(node->sgprivate->interact->events, &i))) {
+	while ((r = (GF_Route*)gf_list_enum(node->sgprivate->interact->routes, &i))) {
 		if (r->FromNode != node) continue;
 		if (r->FromField.fieldIndex != FieldIndex) continue;
 
@@ -279,7 +343,7 @@ void gf_node_event_out_str(GF_Node *node, const char *eventName)
 	
 	//search for routes to activate in the order they where declared
 	i=0;
-	while ((r = (GF_Route*)gf_list_enum(node->sgprivate->interact->events, &i))) {
+	while ((r = (GF_Route*)gf_list_enum(node->sgprivate->interact->routes, &i))) {
 		if (!r->is_setup) gf_sg_route_setup(r);
 		if (stricmp(r->FromField.name, eventName)) continue;
 
