@@ -32,6 +32,7 @@
 GF_Err gf_node_replace_child(GF_Node *node, GF_ChildNodeItem **container, s32 pos, GF_Node *newNode)
 {
 	GF_ChildNodeItem *child, *prev;
+	u32 tag;
 	u32 cur_pos = 0;
 
 	child = *container;
@@ -40,13 +41,17 @@ GF_Err gf_node_replace_child(GF_Node *node, GF_ChildNodeItem **container, s32 po
 		if ((pos<0) || (cur_pos!=(u32)pos)) {
 			prev = child;
 			child = child->next;
+			cur_pos++;
 			continue;
 		}
 		break;
 	}
+	tag = child->node->sgprivate->tag;
 	gf_node_unregister(child->node, node);
 	if (newNode) {
 		child->node = newNode;
+		if (tag==TAG_MPEG4_ColorTransform) 
+			node->sgprivate->flags |= GF_SG_VRML_COLOR_DIRTY;
 	} else {
 		if (prev) prev->next = child->next;
 		else *container = child->next;
@@ -55,68 +60,6 @@ GF_Err gf_node_replace_child(GF_Node *node, GF_ChildNodeItem **container, s32 po
 	return GF_OK;
 }
 
-void gf_sg_destroy_routes(GF_SceneGraph *sg)
-{
-	while (gf_list_count(sg->routes_to_destroy) ) {
-		GF_Route *r = (GF_Route *)gf_list_get(sg->routes_to_destroy, 0);
-		gf_list_rem(sg->routes_to_destroy, 0);
-		gf_sg_route_unqueue(sg, r);
-		if (r->name) free(r->name);
-		free(r);
-	}
-}
-
-
-void gf_sg_route_queue(GF_SceneGraph *sg, GF_Route *r)
-{
-	u32 now;
-	if (!sg) return;
-
-	/*get the top level scene (that's the only reliable one regarding simulatioin tick)*/
-	while (sg->parent_scene) sg = sg->parent_scene;
-	/*a single route may not be activated more than once in a simulation tick*/
-	now = 1 + sg->simulation_tick;
-	if (r->lastActivateTime >= now) return;
-	r->lastActivateTime = now;
-	gf_list_add(sg->routes_to_activate, r);
-}
-
-/*activate all routes in the order they where triggered*/
-GF_EXPORT
-void gf_sg_activate_routes(GF_SceneGraph *sg)
-{
-	GF_Route *r;
-	GF_Node *targ;
-	if (!sg) return;
-
-	sg->simulation_tick++;
-
-	while (gf_list_count(sg->routes_to_activate)) {
-		r = (GF_Route *)gf_list_get(sg->routes_to_activate, 0);
-		gf_list_rem(sg->routes_to_activate, 0);
-		if (r) {
-			targ = r->ToNode;
-			if (gf_sg_route_activate(r)) {
-#ifdef GF_SELF_REPLACE_ENABLE
-				if (sg->graph_has_been_reset) {
-					sg->graph_has_been_reset = 0;
-					return;
-				}
-#endif
-				if (r->is_setup) gf_node_changed(targ, &r->ToField);
-			}
-		}
-	}
-	gf_sg_destroy_routes(sg);
-}
-
-void gf_sg_route_unqueue(GF_SceneGraph *sg, GF_Route *r)
-{
-	/*get the top level scene*/
-	while (sg->parent_scene) sg = sg->parent_scene;
-	/*remove route from queue list*/
-	gf_list_del_item(sg->routes_to_activate, r);
-}
 
 static void Node_on_add_children(GF_Node *node)
 {
@@ -725,28 +668,7 @@ void gf_sg_vrml_field_pointer_del(void *field, u32 FieldType)
 
 Bool gf_sg_vrml_is_sf_field(u32 FieldType)
 {
-	switch (FieldType) {
-	case GF_SG_VRML_SFBOOL:
-	case GF_SG_VRML_SFFLOAT:
-	case GF_SG_VRML_SFDOUBLE:
-	case GF_SG_VRML_SFTIME:
-	case GF_SG_VRML_SFINT32:
-	case GF_SG_VRML_SFSTRING:
-	case GF_SG_VRML_SFVEC3F:
-	case GF_SG_VRML_SFVEC3D:
-	case GF_SG_VRML_SFVEC2F:
-	case GF_SG_VRML_SFVEC2D:
-	case GF_SG_VRML_SFCOLOR:
-	case GF_SG_VRML_SFCOLORRGBA:
-	case GF_SG_VRML_SFROTATION:
-	case GF_SG_VRML_SFIMAGE:
-	case GF_SG_VRML_SFNODE:
-	case GF_SG_VRML_SFURL:
-	case GF_SG_VRML_SFCOMMANDBUFFER:
-		return 1;
-	default:
-		return 0;
-	}
+	return (FieldType<GF_SG_VRML_FIRST_MF);
 }
 
 /*********************************************************************
@@ -1284,16 +1206,24 @@ void gf_sg_vrml_field_copy(void *dest, void *orig, u32 field_type)
 		break;
 
 
-	//MFFields
+	//simple MFFields, do a memcpy
 	case GF_SG_VRML_MFBOOL:
 	case GF_SG_VRML_MFFLOAT:
 	case GF_SG_VRML_MFTIME:
 	case GF_SG_VRML_MFINT32:
-	case GF_SG_VRML_MFSTRING:
 	case GF_SG_VRML_MFVEC3F:
 	case GF_SG_VRML_MFVEC2F:
 	case GF_SG_VRML_MFCOLOR:
 	case GF_SG_VRML_MFROTATION:
+		size = gf_sg_vrml_get_sf_size(field_type) * ((GenMFField *)orig)->count;
+		if (((GenMFField *)orig)->count != ((GenMFField *)dest)->count) {
+			((GenMFField *)dest)->array = realloc(((GenMFField *)dest)->array, size);
+			((GenMFField *)dest)->count = ((GenMFField *)orig)->count;
+		}
+		memcpy(((GenMFField *)dest)->array, ((GenMFField *)orig)->array, size);
+		break;
+	//complex MFFields
+	case GF_SG_VRML_MFSTRING:
 	case GF_SG_VRML_MFIMAGE:
 	case GF_SG_VRML_MFURL:
 	case GF_SG_VRML_MFSCRIPT:
@@ -1425,18 +1355,12 @@ SFColorRGBA gf_sg_sfcolor_to_rgba(SFColor val)
 u32 gf_node_get_num_fields_in_mode(GF_Node *Node, u8 IndexMode)
 {
 	assert(Node);
-#ifdef GF_NODE_USE_POINTERS
-	return Node->sgprivate->get_field_count(Node, IndexMode);
-#else
-
 	if (Node->sgprivate->tag == TAG_ProtoNode) return gf_sg_proto_get_num_fields(Node, IndexMode);
 	else if ((Node->sgprivate->tag == TAG_MPEG4_Script) || (Node->sgprivate->tag == TAG_X3D_Script) )
 		return gf_sg_script_get_num_fields(Node, IndexMode);
 	else if (Node->sgprivate->tag <= GF_NODE_RANGE_LAST_MPEG4) return gf_sg_mpeg4_node_get_field_count(Node, IndexMode);
 	else if (Node->sgprivate->tag <= GF_NODE_RANGE_LAST_X3D) return gf_sg_x3d_node_get_field_count(Node);
 	else return 0;
-
-#endif
 }
 
 
