@@ -24,10 +24,12 @@
 
 
 #include <gpac/internal/terminal_dev.h>
+#include <gpac/internal/compositor_dev.h>
 #include <gpac/sync_layer.h>
 #include <gpac/constants.h>
 
 #include "media_memory.h"
+#include "media_control.h"
 
 static void ch_buffer_off(GF_Channel *ch)
 {
@@ -42,6 +44,10 @@ static void ch_buffer_off(GF_Channel *ch)
 
 static void ch_buffer_on(GF_Channel *ch)
 {
+	/*don't buffer on an already running clock*/
+	if (ch->clock->no_time_ctrl && ch->clock->clock_init && (ch->esd->ESID!=ch->clock->clockID) ) return;
+	if (ch->dispatch_after_db) return;
+
 	/*just in case*/
 	if (!ch->BufferOn) {
 		ch->BufferOn = 1;
@@ -61,6 +67,7 @@ static void Channel_Reset(GF_Channel *ch, Bool for_start)
 	ch->stream_state = 1;
 	ch->IsRap = 0;
 	ch->IsEndOfStream = 0;
+	ch->skip_carousel_au = 0;
 
 	if (for_start) {
 		gf_es_lock(ch, 0);
@@ -98,8 +105,7 @@ GF_Channel *gf_es_new(GF_ESD *esd)
 	GF_SAFEALLOC(tmp, GF_Channel);
 	if (!tmp) return NULL;
 
-	tmp->mx = gf_mx_new();
-
+	tmp->mx = gf_mx_new("Channel");
 	tmp->esd = esd;
 	tmp->chan_id = (u32) tmp;
 	tmp->es_state = GF_ESM_ES_SETUP;
@@ -193,9 +199,11 @@ GF_Err gf_es_start(GF_Channel *ch)
 	default:
 		break;
 	}
+	GF_LOG(GF_LOG_INFO, GF_LOG_SYNC, ("[SyncLayer] Starting ES %d\n", ch->esd->ESID));
 
 	/*reset clock if we own it*/
-	if (gf_es_owns_clock(ch)) gf_clock_reset(ch->clock);
+	if (gf_es_owns_clock(ch) && !ch->clock->no_time_ctrl) 
+		gf_clock_reset(ch->clock);
 
 	/*reset channel*/
 	Channel_Reset(ch, 1);
@@ -237,7 +245,8 @@ void Channel_WaitRAP(GF_Channel *ch)
 	ch->pck_sn = 0;
 
 	/*if using RAP signal and codec not resilient, wait for rap. If RAP isn't signaled DON'T wait for it :)*/
-	if (!ch->codec_resilient) ch->stream_state = 2;
+	if (!ch->codec_resilient) 
+		ch->stream_state = 2;
 	if (ch->buffer) free(ch->buffer);
 	ch->buffer = NULL;
 	ch->AULength = 0;
@@ -289,7 +298,7 @@ static Bool Channel_NeedsBuffering(GF_Channel *ch, u32 ForRebuffering)
 			gf_term_message(ch->odm->term, ch->service->url, "Data timeout - aborting buffering", GF_OK); 
 			ch->MinBuffer = ch->MaxBuffer = 0;
 			ch->au_duration = 0;
-			gf_is_buffering_info(ch->odm->parentscene ? ch->odm->parentscene : ch->odm->subscene);
+			gf_inline_buffering_info(ch->odm->parentscene ? ch->odm->parentscene : ch->odm->subscene);
 			return 0;
 		} else {
 			now = ch->odm->term->net_data_timeout + ch->last_au_time - now;
@@ -322,11 +331,16 @@ static Bool Channel_NeedsBuffering(GF_Channel *ch, u32 ForRebuffering)
 
 static void Channel_UpdateBuffering(GF_Channel *ch, Bool update_info)
 {
-	if (update_info && ch->MaxBuffer) gf_is_buffering_info(ch->odm->parentscene ? ch->odm->parentscene : ch->odm->subscene);
+	if (update_info && ch->MaxBuffer) gf_inline_buffering_info(ch->odm->parentscene ? ch->odm->parentscene : ch->odm->subscene);
+
+	gf_term_service_media_event(ch->odm, GF_EVENT_MEDIA_DATA_PROGRESS);
+	
 	if (!Channel_NeedsBuffering(ch, 0)) {
 		ch_buffer_off(ch);
-		if (ch->MaxBuffer) gf_is_buffering_info(ch->odm->parentscene ? ch->odm->parentscene : ch->odm->subscene);
+		if (ch->MaxBuffer && update_info) gf_inline_buffering_info(ch->odm->parentscene ? ch->odm->parentscene : ch->odm->subscene);
 		if (ch->clock->no_time_ctrl) ch->clock->no_time_ctrl = 2;
+
+		gf_term_service_media_event(ch->odm, GF_EVENT_MEDIA_PLAYABLE);
 	}
 }
 
@@ -364,7 +378,9 @@ static void Channel_UpdateBufferTime(GF_Channel *ch)
 /*dispatch the AU in the DB*/
 static void Channel_DispatchAU(GF_Channel *ch, u32 duration)
 {
+	u32 time;
 	GF_DBUnit *au;
+
 	if (!ch->buffer || !ch->len) {
 		if (ch->buffer) {
 			free(ch->buffer);
@@ -476,22 +492,65 @@ static void Channel_DispatchAU(GF_Channel *ch, u32 duration)
 
 	GF_LOG(GF_LOG_DEBUG, GF_LOG_SYNC, ("[SyncLayer] ES%d - Dispatch AU CTS %d time %d Buffer %d Nb AUs %d\n", ch->esd->ESID, au->CTS, gf_clock_time(ch->clock), ch->BufferTime, ch->AU_Count));
 
-	if (ch->BufferOn) {
-		ch->last_au_time = gf_term_get_time(ch->odm->term);
-		Channel_UpdateBuffering(ch, 1);
-	}
-	/*little opt: if this is an OD AU, try to setup the object if needed */
-	if (ch->esd->decoderConfig->streamType==GF_STREAM_OD) {
-		gf_term_lock_net(ch->odm->term, 1);
-		gf_codec_process(ch->odm->subscene->od_codec, 100);
-		gf_term_lock_net(ch->odm->term, 0);
-	}
-	gf_es_lock(ch, 0);
+	/*little optimisation: if direct dispatching is possible, try to decode the AU
+	we must lock the media scheduler to avoid deadlocks with other codecs accessing the scene or 
+	media resources*/
+	if (ch->dispatch_after_db) {
+		u32 retry = 100;
+		u32 current_frame;
+		GF_Terminal *term = ch->odm->term;
+		ch_buffer_off(ch);
+		if (gf_mx_try_lock(term->mm_mx)) {
+			switch (ch->esd->decoderConfig->streamType) {
+			case GF_STREAM_OD:
+				gf_codec_process(ch->odm->subscene->od_codec, 100);
+				break;
+			case GF_STREAM_SCENE:
+				if (ch->odm->codec) 
+					gf_codec_process(ch->odm->codec, 100);
+				else
+					gf_codec_process(ch->odm->subscene->scene_codec, 100);
+				break;
+			}
+			gf_mx_v(term->mm_mx);
+		}
 
-	return;
+		current_frame = term->compositor->frame_number;
+		/*wait for initial setup to complete before giving back the hand to the caller service*/
+		while (retry) {
+			/*Scene bootstrap: if the scene is attached, wait for first frame to complete so that initial PLAY on
+			objects can be evaluated*/
+			if (term->compositor->scene && (term->compositor->frame_number==current_frame) ) {
+				retry--;
+				gf_sleep(1);
+				continue;
+			}
+			/*Media bootstrap: wait for all pending requests on media objects are processed*/
+			if (gf_list_count(term->media_queue)) {
+				retry--;
+				gf_sleep(1);
+				continue;
+			}
+			break;
+		}
+	}
+
+	time = gf_term_get_time(ch->odm->term);
+	if (ch->BufferOn) {
+		ch->last_au_time = time;
+		Channel_UpdateBuffering(ch, 1);
+	} else {
+		/*trigger the data progress every 500 ms*/
+		if (ch->last_au_time + 500 > time) {
+			gf_term_service_media_event(ch->odm, GF_EVENT_MEDIA_DATA_PROGRESS);
+			ch->last_au_time = time;
+		}
+	}
+
+	gf_es_lock(ch, 0);
 }
 
-void Channel_RecieveSkipSL(GF_ClientService *serv, GF_Channel *ch, char *StreamBuf, u32 StreamLength)
+void Channel_ReceiveSkipSL(GF_ClientService *serv, GF_Channel *ch, char *StreamBuf, u32 StreamLength)
 {
 	GF_DBUnit *au;
 	if (!StreamLength) return;
@@ -547,11 +606,22 @@ void gf_es_receive_sl_packet(GF_ClientService *serv, GF_Channel *ch, char *Strea
 	Bool EndAU, NewAU;
 	char *payload;
 
-	if (ch->skip_sl) {
-		Channel_RecieveSkipSL(serv, ch, StreamBuf, StreamLength);
+	if (ch->bypass_sl_and_db) {
+		GF_SceneDecoder *sdec;
+		ch->IsClockInit = 1;
+		if (ch->odm->subscene) {
+			sdec = (GF_SceneDecoder *)ch->odm->subscene->scene_codec->decio;
+		} else {
+			sdec = (GF_SceneDecoder *)ch->odm->codec->decio;
+		}
+		gf_mx_p(ch->mx);
+		sdec->ProcessData(sdec, StreamBuf, StreamLength, ch->esd->ESID, 0, 0);
+		gf_mx_v(ch->mx);
 		return;
 	}
 
+	if (ch->es_state != GF_ESM_ES_RUNNING) return;
+
 	/*physical SL-PDU - depacketize*/
 	if (!header) {
 		if (!StreamLength) return;
@@ -563,6 +633,11 @@ void gf_es_receive_sl_packet(GF_ClientService *serv, GF_Channel *ch, char *Strea
 	}
 	payload = StreamBuf + SLHdrLen;
 
+	if (ch->skip_sl) {
+		Channel_ReceiveSkipSL(serv, ch, payload, StreamLength);
+		return;
+	}
+
 	/*check state*/
 	if (!ch->codec_resilient && (reception_status==GF_CORRUPTED_DATA)) {
 		Channel_WaitRAP(ch);
@@ -610,17 +685,19 @@ void gf_es_receive_sl_packet(GF_ClientService *serv, GF_Channel *ch, char *Strea
 		ch->pck_sn = hdr.packetSequenceNumber;
 	}
 
-	/*if idle or empty, skip the packet*/
-	if (hdr.idleFlag || (hdr.paddingFlag && !hdr.paddingBits)) {
-		GF_LOG(GF_LOG_DEBUG, GF_LOG_SYNC, ("[SyncLayer] ES%d: Idle or empty packet - skipping\n", ch->esd->ESID));
+	/*if empty, skip the packet*/
+	if (hdr.paddingFlag && !hdr.paddingBits) {
+		GF_LOG(GF_LOG_DEBUG, GF_LOG_SYNC, ("[SyncLayer] ES%d: Empty packet - skipping\n", ch->esd->ESID));
 		return;
 	}
+	/*IDLE stream shall be processed*/
 
 
 	NewAU = 0;
 	if (hdr.accessUnitStartFlag) {
 		NewAU = 1;
 		ch->NextIsAUStart = 0;
+		ch->skip_carousel_au = 0;
 
 		/*if we have a pending AU, add it*/
 		if (ch->buffer) {
@@ -674,32 +751,21 @@ void gf_es_receive_sl_packet(GF_ClientService *serv, GF_Channel *ch, char *Strea
 			}
 		}
 
-		if (!ch->IsClockInit) {
-			/*first data received on channel inits the clock, regardless of:
-				- wether channel owns the clock or not
-				- wether data can be processed or not (carousel)
-			We then offset other streams' timeline by the time ellapsed since clock initialization
-			*/
-#if 1
+		if (!ch->IsClockInit && (hdr.compositionTimeStampFlag || !ch->esd->slConfig->useTimestampsFlag) ) {
+			/*the first data received inits the clock - this is needed to handle clock dependencies on non-initialized 
+			streams (eg, bifs/od depends on audio/video clock)*/
 			if (!ch->clock->clock_init) {
 				gf_clock_set_time(ch->clock, ch->CTS);
-				GF_LOG(GF_LOG_INFO, GF_LOG_SYNC, ("[SyncLayer] ES%d: initializing clock at STB %d - AU DTS %d\n", ch->esd->ESID, gf_term_get_time(ch->odm->term), ch->DTS));
-			} else if (ch->clock->no_time_ctrl && 0) {
-				s32 offset = gf_term_get_time(ch->odm->term);
-				offset -= ch->clock->StartTime;
-				if (offset>0) {
-					ch->ts_offset += offset;
-					ch->DTS += offset;
-					ch->CTS += offset;
-				}
-			}
-			ch->IsClockInit = 1;
-#else
-			if (!ch->clock->clock_init && gf_es_owns_clock(ch)) {
+				GF_LOG(GF_LOG_INFO, GF_LOG_SYNC, ("[SyncLayer] ES%d: Forcing clock initialization at STB %d - AU DTS %d\n", ch->esd->ESID, gf_term_get_time(ch->odm->term), ch->DTS));
+			} 
+			/*channel is the OCR, force a re-init of the clock since we cannot assume the AU used to init the clock was
+			not sent ahead of time*/
+			else if (gf_es_owns_clock(ch)) {
+				ch->clock->clock_init = 0;
 				gf_clock_set_time(ch->clock, ch->DTS);
+				GF_LOG(GF_LOG_INFO, GF_LOG_SYNC, ("[SyncLayer] ES%d: initializing clock at STB %d - AU DTS %d - %d buffering\n", ch->esd->ESID, gf_term_get_time(ch->odm->term), ch->DTS, ch->clock->Buffering));
 			}
 			if (ch->clock->clock_init) ch->IsClockInit = 1;
-#endif
 		}
 		/*if the AU Length is carried in SL, get its size*/
 		if (ch->esd->slConfig->AULength > 0) {
@@ -724,6 +790,7 @@ void gf_es_receive_sl_packet(GF_ClientService *serv, GF_Channel *ch, char *Strea
 						ch->stream_state = 0;
 					} 
 					else {
+						ch->skip_carousel_au = 1;
 						GF_LOG(GF_LOG_INFO, GF_LOG_SYNC, ("[SyncLayer] ES%d: RAP Carousel found - skipping\n", ch->esd->ESID));
 						return;
 					}
@@ -736,6 +803,7 @@ void gf_es_receive_sl_packet(GF_ClientService *serv, GF_Channel *ch, char *Strea
 			} 
 			/*regular AU but waiting for RAP*/
 			else if (ch->stream_state) {
+				ch->skip_carousel_au = 1;
 				GF_LOG(GF_LOG_INFO, GF_LOG_SYNC, ("[SyncLayer] ES%d: Waiting for RAP Carousel - skipping\n", ch->esd->ESID));
 				return;
 			} else {
@@ -757,13 +825,22 @@ void gf_es_receive_sl_packet(GF_ClientService *serv, GF_Channel *ch, char *Strea
 
 	/*update the RAP marker on a packet base (to cope with AVC/H264 NALU->AU reconstruction)*/
 	if (hdr.randomAccessPointFlag) ch->IsRap = 1;
+	/* we need to skip all the packets of the current AU in the carousel scenario */
+	if (ch->skip_carousel_au == 1) return;
 
-	/*check OCR*/
-	if (hdr.OCRflag) {
-		s64 OCR_TS = (s64) (((s64) hdr.objectClockReference) * ch->ocr_scale);
-		gf_clock_set_time(ch->clock, (u32) OCR_TS);
-		ch->IsClockInit = 1;
+	/*we ignore OCRs for the moment*/
+#if 0
+	/*Apply OCR*/
+	if (!ch->BufferOn && ch->IsClockInit && hdr.OCRflag) {
+		u32 OCR_TS = (u32) (s64) (((s64) hdr.objectClockReference) * ch->ocr_scale);
+		u32 ck = gf_clock_time(ch->clock);
+		OCR_TS += ch->BufferTime;
+		fprintf(stdout, "OCR %d OTB %d Diff %d Buffer %d (%d AUs) \n", OCR_TS, ck, OCR_TS - ck, ch->BufferTime, ch->AU_Count);
+
+		GF_LOG(GF_LOG_DEBUG, GF_LOG_SYNC, ("[SyncLayer] ES%d: At OTB %d adjusting OCR to %d\n", ch->esd->ESID, gf_clock_real_time(ch->clock), OCR_TS));
+		gf_clock_set_time(ch->clock, OCR_TS);
 	}
+#endif
 
 	/*get AU end state*/	
 	OldLength = ch->buffer ? ch->len : 0;
@@ -783,7 +860,8 @@ void gf_es_receive_sl_packet(GF_ClientService *serv, GF_Channel *ch, char *Strea
 		if (ch->esd->slConfig->useAccessUnitStartFlag) {
 			GF_LOG(GF_LOG_ERROR, GF_LOG_SYNC, ("[SyncLayer] ES%d: missed begin of AU\n", ch->esd->ESID));
 		}
-		return;
+		if (ch->codec_resilient) NewAU = 1;
+		else return;
 	}
 
 	/*Write the Packet payload to the buffer*/
@@ -803,14 +881,9 @@ void gf_es_receive_sl_packet(GF_ClientService *serv, GF_Channel *ch, char *Strea
 		ch->len = 0;
 	}
 	if (!ch->esd->slConfig->usePaddingFlag) hdr.paddingFlag = 0;
-
-	/*if no bitstream, we missed the AU Start packet. Unusable ...*/
-	if (!ch->buffer) {
-		GF_LOG(GF_LOG_ERROR, GF_LOG_SYNC, ("[SyncLayer] ES%d: Empty buffer - missed begining of AU\n", ch->esd->ESID));
-		return;
-	}
 	
 	if (ch->ipmp_tool) {
+		GF_Err e;
 		GF_IPMPEvent evt;
 		memset(&evt, 0, sizeof(evt));
 		evt.event_type=GF_IPMP_TOOL_PROCESS_DATA;
@@ -819,7 +892,23 @@ void gf_es_receive_sl_packet(GF_ClientService *serv, GF_Channel *ch, char *Strea
 		evt.data_size = StreamLength;
 		evt.is_encrypted = hdr.isma_encrypted;
 		evt.isma_BSO = hdr.isma_BSO;
-		ch->ipmp_tool->process(ch->ipmp_tool, &evt);
+		e = ch->ipmp_tool->process(ch->ipmp_tool, &evt);
+
+		/*we discard undecrypted AU*/
+		if (e) {
+			if (e==GF_EOS) {
+				gf_es_on_eos(ch);
+				/*restart*/
+				if (evt.restart_requested) {
+					if (ch->odm->parentscene->is_dynamic_scene) {
+						gf_inline_restart_dynamic(ch->odm->parentscene, 0);
+					} else {
+						MC_Restart(ch->odm);
+					}
+				}
+			}
+			return;
+		}
 	}
 
 	if (hdr.paddingFlag && !EndAU) {	
@@ -905,7 +994,20 @@ GF_DBUnit *gf_es_get_au(GF_Channel *ch)
 			evt.isma_BSO = slh.isma_BSO;
 			evt.channel = ch;
 			e = ch->ipmp_tool->process(ch->ipmp_tool, &evt);
-			if (e && 0) {
+
+			/*we discard undecrypted AU*/
+			if (e) {
+				if (e==GF_EOS) {
+					gf_es_on_eos(ch);
+					/*restart*/
+					if (evt.restart_requested) {
+						if (ch->odm->parentscene->is_dynamic_scene) {
+							gf_inline_restart_dynamic(ch->odm->parentscene, 0);
+						} else {
+							MC_Restart(ch->odm);
+						}
+					}
+				}
 				gf_term_channel_release_sl_packet(ch->service, ch);
 				return NULL;
 			}
@@ -929,14 +1031,21 @@ void gf_es_init_dummy(GF_Channel *ch)
 	GF_SLHeader slh;
 	Bool comp, is_new_data;
 	GF_Err e, state;
-	if (!ch->is_pulling) return;
 	/*pull from stream - resume clock if needed*/
 	ch_buffer_off(ch);
 
-	e = gf_term_channel_get_sl_packet(ch->service, ch, (char **) &ch->AU_buffer_pull->data, &ch->AU_buffer_pull->dataLength, &slh, &comp, &state, &is_new_data);
-	if (e) state = e;
-	if ((state==GF_OK) && is_new_data) gf_es_receive_sl_packet(ch->service, ch, NULL, 0, &slh, GF_OK);
-	gf_term_channel_release_sl_packet(ch->service, ch);
+	ch->ts_res = 1000;
+	if (ch->is_pulling) {
+		e = gf_term_channel_get_sl_packet(ch->service, ch, (char **) &ch->AU_buffer_pull->data, &ch->AU_buffer_pull->dataLength, &slh, &comp, &state, &is_new_data);
+		if (e) state = e;
+		if ((state==GF_OK) && is_new_data) gf_es_receive_sl_packet(ch->service, ch, NULL, 0, &slh, GF_OK);
+		gf_term_channel_release_sl_packet(ch->service, ch);
+	} else {
+		memset(&slh, 0, sizeof(GF_SLHeader));
+		slh.accessUnitStartFlag = 1;
+		slh.compositionTimeStampFlag = 1;
+		gf_es_receive_sl_packet(ch->service, ch, NULL, 0, &slh, GF_OK);
+	}
 }
 
 void gf_es_drop_au(GF_Channel *ch)
@@ -944,18 +1053,21 @@ void gf_es_drop_au(GF_Channel *ch)
 	GF_DBUnit *au;
 
 	if (ch->is_pulling) {
-		assert(ch->AU_buffer_pull);
-		gf_term_channel_release_sl_packet(ch->service, ch);
-		ch->AU_buffer_pull->data = NULL;
-		ch->AU_buffer_pull->dataLength = 0;
+		if (ch->AU_buffer_pull) {
+			gf_term_channel_release_sl_packet(ch->service, ch);
+			ch->AU_buffer_pull->data = NULL;
+			ch->AU_buffer_pull->dataLength = 0;
+		}
 		ch->first_au_fetched = 1;
 		return;
 	}
 
-	if (!ch->AU_buffer_first) return;
-
 	/*lock the channel before touching the queue*/
 	gf_es_lock(ch, 1);
+	if (!ch->AU_buffer_first) {
+		gf_es_lock(ch, 0);
+		return;
+	}
 	ch->first_au_fetched = 1;
 
 	au = ch->AU_buffer_first;
@@ -974,6 +1086,7 @@ void gf_es_drop_au(GF_Channel *ch)
 	/*if we get under our limit, rebuffer EXCEPT WHEN EOS is signaled*/
 	if (!ch->IsEndOfStream && Channel_NeedsBuffering(ch, 1)) {
 		ch_buffer_on(ch);
+		gf_term_service_media_event(ch->odm, GF_EVENT_MEDIA_NOT_PLAYABLE);
 	}
 
 	/*unlock the channel*/
@@ -990,6 +1103,35 @@ void gf_es_lock(GF_Channel *ch, u32 LockIt)
 	}
 }
 
+/*refresh all ODs when an non-interactive stream is found*/
+static void refresh_non_interactive_clocks(GF_ObjectManager *odm)
+{
+	u32 i, j;
+	GF_Channel *ch;
+	GF_ObjectManager *test_od;
+	GF_InlineScene *in_scene;
+
+	/*check for inline*/
+	in_scene = odm->subscene ? odm->subscene : odm->parentscene;
+	i=0;
+	while ((ch = (GF_Channel*)gf_list_enum(in_scene->root_od->channels, &i)) ) {
+		if (ch->clock->no_time_ctrl) {
+			in_scene->root_od->flags |= GF_ODM_NO_TIME_CTRL;
+			//if (ch->clock->use_ocr && ch->BufferOn) ch_buffer_off(ch);
+		}
+	}
+
+	i=0;
+	while ((test_od = (GF_ObjectManager *)gf_list_enum(in_scene->ODlist, &i)) ) {
+		j=0;
+		while ((ch = (GF_Channel*)gf_list_enum(test_od->channels, &j)) ) {
+			if (ch->clock->no_time_ctrl) {
+				test_od->flags |= GF_ODM_NO_TIME_CTRL;
+				//if (ch->clock->use_ocr && ch->BufferOn) ch_buffer_off(ch);
+			}
+		}
+	}
+}
 /*performs final setup upon connection confirm*/
 void gf_es_on_connect(GF_Channel *ch)
 {
@@ -1024,7 +1166,7 @@ void gf_es_on_connect(GF_Channel *ch)
 	if (gf_term_service_command(ch->service, &com)!=GF_OK) {
 		ch->clock->no_time_ctrl = 1;
 		ch->odm->flags |= GF_ODM_NO_TIME_CTRL;
-		gf_odm_refresh_uninteractives(ch->odm);
+		refresh_non_interactive_clocks(ch->odm);
 	}
 
 	/*signal channel state*/
@@ -1036,6 +1178,13 @@ void gf_es_on_connect(GF_Channel *ch)
 	/*remember channels connected on service*/
 	if (ch->esd->URLString) ch->service->nb_ch_users++;
 
+	/*turn off buffering for JPEG and PNG*/
+	switch (ch->esd->decoderConfig->objectTypeIndication) {
+	case 0x6C:
+	case 0x6D:
+		can_buffer = 0;
+		break;
+	}
 	/*buffer setup*/
 	ch->MinBuffer = ch->MaxBuffer = 0;
 	if (can_buffer) {
@@ -1056,6 +1205,20 @@ void gf_es_on_connect(GF_Channel *ch)
 			ch->MaxBuffer = com.buffer.max;
 		}
 	}
+	if (ch->esd->decoderConfig->streamType == GF_STREAM_PRIVATE_SCENE &&
+		ch->esd->decoderConfig->objectTypeIndication == GPAC_OTI_PRIVATE_SCENE_EPG) { 
+		ch->bypass_sl_and_db = 1;
+	}
+	if (ch->clock->no_time_ctrl) {
+		switch (ch->esd->decoderConfig->streamType) {
+		case GF_STREAM_AUDIO:
+		case GF_STREAM_VISUAL:
+			break;
+		default:
+			ch->dispatch_after_db = 1;
+			break;
+		}
+	}
 
 	/*get duration*/
 	com.command_type = GF_NET_CHAN_DURATION;
