@@ -29,7 +29,7 @@ GF_Clock *NewClock(GF_Terminal *term)
 	GF_Clock *tmp;
 	GF_SAFEALLOC(tmp, GF_Clock);
 	if (!tmp) return NULL;
-	tmp->mx = gf_mx_new();
+	tmp->mx = gf_mx_new("Clock");
 	tmp->term = term;
 	tmp->speed = FIX_ONE;
 	if (term->play_state) tmp->Paused = 1;
@@ -168,14 +168,15 @@ void gf_clock_set_time(GF_Clock *ck, u32 TS)
 	if (!ck->clock_init) {
 		ck->init_time = TS;
 		ck->clock_init = 1;
+		ck->drift = 0;
 		/*update starttime and pausetime even in pause mode*/
 		ck->PauseTime = ck->StartTime = gf_term_get_time(ck->term);
 		if (ck->term->play_state) ck->Paused ++;
 	}
 	/*TODO: test with pure OCR streams*/
-	if (ck->use_ocr) {
+	else if (ck->use_ocr) {
 		/*just update the drift - we could also apply a drift algo*/
-		u32 now = gf_clock_time(ck);
+		u32 now = gf_clock_real_time(ck);
 		s32 drift = now - (u32) TS;
 		ck->drift += drift;
 	}
@@ -194,7 +195,7 @@ void gf_clock_pause(GF_Clock *ck)
 void gf_clock_resume(GF_Clock *ck)
 {
 	gf_mx_p(ck->mx);
-	assert(ck->Paused);
+//	assert(ck->Paused);
 	ck->Paused -= 1;
 	if (!ck->Paused) 
 		ck->StartTime += gf_term_get_time(ck->term) - ck->PauseTime;
@@ -205,7 +206,8 @@ void gf_clock_resume(GF_Clock *ck)
 u32 gf_clock_real_time(GF_Clock *ck)
 {
 	u32 time;
-	if (!ck || !ck->clock_init) return ck->StartTime;
+	assert(ck);
+	if (!ck->clock_init) return ck->StartTime;
 	time = ck->Paused > 0 ? ck->PauseTime : gf_term_get_time(ck->term);
 #ifdef GPAC_FIXED_POINT
 	time = ck->discontinuity_time + ck->init_time + (time - ck->StartTime) * FIX2INT(100*ck->speed) / 100;
@@ -222,10 +224,16 @@ u32 gf_clock_time(GF_Clock *ck)
 	return time - ck->drift;
 }
 
+u32 gf_clock_ellapse_time(GF_Clock *ck)
+{
+	if (ck->no_time_ctrl) return gf_clock_time(ck) - ck->init_time;
+	return gf_clock_time(ck);
+}
+
 
 Bool gf_clock_is_started(GF_Clock *ck)
 {
-	if (!ck->StartTime || ck->Buffering || ck->Paused) return 0;
+	if (/*!ck->StartTime || */ck->Buffering || ck->Paused) return 0;
 	return 1;
 }
 
