@@ -25,16 +25,12 @@
 
 
 #include <gpac/internal/terminal_dev.h>
-#include <gpac/internal/renderer_dev.h>
+#include <gpac/internal/compositor_dev.h>
 #include <gpac/constants.h>
 #include "media_memory.h"
 #include "media_control.h"
 #include "input_sensor.h"
 
-/*update config of object*/
-void MO_UpdateCaps(GF_MediaObject *mo);
-
-
 #define TIME_CHECK		3
 
 
@@ -108,20 +104,26 @@ GF_Err gf_codec_add_channel(GF_Codec *codec, GF_Channel *ch)
 			if (!e && com.get_dsi.dsi) {
 				dsi = com.get_dsi.dsi;
 				dsiSize = com.get_dsi.dsi_len;
+
+				if (ch->esd->decoderConfig->decoderSpecificInfo->data) free(ch->esd->decoderConfig->decoderSpecificInfo->data);
+				ch->esd->decoderConfig->decoderSpecificInfo->data = com.get_dsi.dsi;
+				ch->esd->decoderConfig->decoderSpecificInfo->dataLength = com.get_dsi.dsi_len;
 			}
 		}
 
-		e = codec->decio->AttachStream(codec->decio, ch->esd->ESID, 
-					dsi, dsiSize, ch->esd->dependsOnESID, ch->esd->decoderConfig->objectTypeIndication, ch->esd->decoderConfig->upstream);
+		GF_LOG(GF_LOG_DEBUG, GF_LOG_CODEC, ("[Codec] Attaching stream %d to codec %s\n", ch->esd->ESID, codec->decio->module_name));
 
-		if (com.get_dsi.dsi) {
-			if (ch->esd->decoderConfig->decoderSpecificInfo->data) free(ch->esd->decoderConfig->decoderSpecificInfo->data);
-			ch->esd->decoderConfig->decoderSpecificInfo->data = com.get_dsi.dsi;
-			ch->esd->decoderConfig->decoderSpecificInfo->dataLength = com.get_dsi.dsi_len;
+		/*lock the channel before setup in case we are using direct_decode */
+		gf_mx_p(ch->mx);
+		e = codec->decio->AttachStream(codec->decio, ch->esd);
+		gf_mx_v(ch->mx);
+
+
+		if (e) {
+			GF_LOG(GF_LOG_DEBUG, GF_LOG_CODEC, ("[Codec] Attach Stream failed %s\n", gf_error_to_string(e) ));
+			return e;
 		}
 
-		if (e) return e;
-
 		/*ask codec for desired output capacity - note this may be 0 if stream is not yet configured*/
 		cap.CapCode = GF_CODEC_OUTPUT_SIZE;
 		gf_codec_get_capability(codec, &cap);
@@ -151,6 +153,7 @@ GF_Err gf_codec_add_channel(GF_Codec *codec, GF_Channel *ch)
 		}
 		if ((codec->type==GF_STREAM_AUDIO) && (max<2)) max = 2;
 
+		GF_LOG(GF_LOG_DEBUG, GF_LOG_CODEC, ("[ODM] Creating composition buffer for codec %s - %d units %d bytes each\n", codec->decio->module_name, max, CUsize));
 		/*setup CB*/
 		if (!codec->CB && max) {
 			codec->CB = gf_cm_new(CUsize, max);
@@ -158,10 +161,11 @@ GF_Err gf_codec_add_channel(GF_Codec *codec, GF_Channel *ch)
 			codec->CB->odm = codec->odm;
 		}
 
-		/*check re-ordering*/
+		/*check re-ordering - set by default on all codecs*/
+		codec->is_reordering = 1;
 		cap.CapCode = GF_CODEC_REORDER;
-		gf_codec_get_capability(codec, &cap);
-		if (cap.cap.valueInt) codec->is_reordering = 1;
+		if (gf_codec_get_capability(codec, &cap) == GF_OK);
+			codec->is_reordering = cap.cap.valueInt;
 
 		/*setup net channel config*/
 		if (ch->service) {
@@ -287,7 +291,8 @@ static void Decoder_GetNextAU(GF_Codec *codec, GF_Channel **activeChannel, GF_DB
 	}
 
 	/*FIXME - we're breaking sync (couple of frames delay)*/
-	if (*nextAU && codec->is_reordering) (*nextAU)->CTS = (*nextAU)->DTS;
+	if (*nextAU && codec->is_reordering) 
+		(*nextAU)->CTS = (*nextAU)->DTS;
 }
 
 
@@ -304,8 +309,8 @@ static GF_Err SystemCodec_Process(GF_Codec *codec, u32 TimeAvailable)
 	scene_locked = 0;
 	
 	/*for resync, if needed - the logic behind this is that there is no composition memory on sytems codecs so
-	"frame dropping" is done by preventing the renderer from redrawing after an update and decoding following AU
-	so that the renderer is always woken up once all late systems AUs are decoded. This flag is overriden when 
+	"frame dropping" is done by preventing the compositor from redrawing after an update and decoding following AU
+	so that the compositor is always woken up once all late systems AUs are decoded. This flag is overriden when 
 	seeking*/
 	check_next_unit = (codec->odm->term->flags & GF_TERM_SYSDEC_RESYNC) ? 1 : 0;
 	
@@ -318,9 +323,6 @@ check_unit:
 	/*fetch next AU in DTS order for this codec*/
 	Decoder_GetNextAU(codec, &ch, &AU);
 
-	/*get the object time*/
-	obj_time = gf_clock_time(codec->ck);
-
 	/*no active channel return*/
 	if (!AU || !ch) {
 		/*if the codec is in EOS state, move to STOP*/
@@ -342,13 +344,17 @@ check_unit:
 		goto exit;
 	}
 	
+	if (ch && ch->odm->media_ctrl && !ch->odm->media_ctrl->media_speed) 
+		goto exit;
+
+	/*get the object time*/
+	obj_time = gf_clock_time(codec->ck);
 
 	/*clock is not started*/
 //	if (ch->first_au_fetched && !gf_clock_is_started(ch->clock)) goto exit;
 
 	/*check timing based on the input channel and main FPS*/
-//	if ( (AU->DTS > obj_time + codec->odm->term->half_frame_duration) ) goto exit;
-	if ( (AU->DTS > obj_time ) ) goto exit;
+	if (AU->DTS > obj_time /*+ codec->odm->term->half_frame_duration*/) goto exit;
 
 	/*check seeking and update timing - do NOT use the base layer, since BIFS streams may depend on other 
 	streams not on the same clock*/
@@ -356,7 +362,7 @@ check_unit:
 		/*hack for RTSP streaming of systems streams, except InputSensor*/
 		if (!ch->is_pulling && (codec->type != GF_STREAM_INTERACT) && (AU->dataLength == codec->prev_au_size)) {
 			gf_es_drop_au(ch);
-			GF_LOG(GF_LOG_WARNING, GF_LOG_CODEC, ("[Decoder] Same MPEG-4 Systems AU detected - dropping\n"));
+			GF_LOG(GF_LOG_WARNING, GF_LOG_MEDIA, ("[SysDec] Same MPEG-4 Systems AU detected - dropping\n"));
 			goto check_unit;
 		}
 		/*seeking for systems is done by not releasing the graph until seek is done*/
@@ -376,20 +382,21 @@ check_unit:
 
 	/*lock scene*/
 	if (!scene_locked) {
-		gf_term_lock_renderer(codec->odm->term, 1);
+		if (!gf_mx_try_lock(codec->odm->term->compositor->mx)) return GF_OK;
+		//gf_term_lock_compositor(codec->odm->term, 1);
 		scene_locked = 1;
 		/*if terminal is paused, force step-mode: it won't hurt in regular pause/play and ensures proper frame dumping*/
-		if (codec->odm->term->play_state) codec->odm->term->renderer->step_mode=1;
+		if (codec->odm->term->play_state) codec->odm->term->compositor->step_mode=1;
 	}
 
 	/*current media time for system objects is the clock time, since the media is likely to have random 
 	updates in time*/
 	codec->odm->current_time = gf_clock_time(codec->ck);
 
+	GF_LOG(GF_LOG_DEBUG, GF_LOG_CODEC, ("[SysDec] Codec %s AU CTS %d channel %d OTB %d\n", sdec->module_name , AU->CTS, ch->esd->ESID, codec->odm->current_time));
 	now = gf_term_get_time(codec->odm->term);
 	e = sdec->ProcessData(sdec, AU->data, AU->dataLength, ch->esd->ESID, au_time, mm_level);
 	now = gf_term_get_time(codec->odm->term) - now;
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_CODEC, ("[SysDec] Codec %s Processing AU CTS %d\n", sdec->module_name , AU->CTS));
 
 	codec_update_stats(codec, AU->dataLength, now);
 	codec->prev_au_size = AU->dataLength;
@@ -398,25 +405,24 @@ check_unit:
 	gf_es_drop_au(ch);
 
 	if (e) {
-		ch->stream_state = 2;
+		if (e<0) ch->stream_state = 2;
 		goto exit;
 	}
-	/*OD acts as scene codec, regenerate scene*/
-	if (codec->flags & GF_ESM_CODEC_IS_SCENE_OD) gf_is_regenerate(codec->odm->subscene ? codec->odm->subscene : codec->odm->parentscene);
+
 	/*in broadcast mode, generate a scene if none is available*/
-	else if (codec->ck->no_time_ctrl) {
+	if (codec->ck->no_time_ctrl) {
 		GF_InlineScene *is = codec->odm->subscene ? codec->odm->subscene : codec->odm->parentscene;
 
 		/*static OD resources (embedded in ESD) in broadcast mode, reset time*/
 		if (codec->flags & GF_ESM_CODEC_IS_STATIC_OD) gf_clock_reset(codec->ck);
 		/*generate a temp scene if none is in place*/
-		if (is->graph_attached != 1 ) {
+		if (is->graph_attached != 1) {
 			Bool prev_dyn = is->is_dynamic_scene; 
 			is->is_dynamic_scene = 1;
-			gf_is_regenerate(is);
+			gf_inline_regenerate(is);
 			is->graph_attached = 2;
 			is->is_dynamic_scene = prev_dyn;
-			GF_LOG(GF_LOG_INFO, GF_LOG_CODEC, ("[Decoder] Got OD resources before scene - generating temporary scene\n"));
+			GF_LOG(GF_LOG_INFO, GF_LOG_MEDIA, ("[Decoder] Got OD resources before scene - generating temporary scene\n"));
 		}
 	}
 
@@ -424,7 +430,7 @@ check_unit:
 	if (check_next_unit) goto check_unit;
 	
 exit:
-	if (scene_locked) gf_term_lock_renderer(codec->odm->term, 0);
+	if (scene_locked) gf_term_lock_compositor(codec->odm->term, 0);
 	return e;
 }
 
@@ -453,20 +459,31 @@ static GF_Err PrivateScene_Process(GF_Codec *codec, u32 TimeAvailable)
 	resume_clock = 0;
 	/*init channel clock*/
 	if (!ch->IsClockInit) {
+		Bool started;
+		/*signal seek*/
+		if (!gf_mx_try_lock(codec->odm->term->compositor->mx)) return GF_OK;
+		//gf_term_lock_compositor(codec->odm->term, 1);
 		gf_es_init_dummy(ch);
-		if (!gf_clock_is_started(ch->clock)) return GF_OK;
+
+		sdec->ProcessData(sdec, NULL, 0, ch->esd->ESID, -1, GF_CODEC_LEVEL_NORMAL);
+		gf_term_lock_compositor(codec->odm->term, 0);
+		started = gf_clock_is_started(ch->clock);
 		/*let's be nice to the scene loader (that usually involves quite some parsing), pause clock while
 		parsing*/
 		gf_clock_pause(ch->clock);
 		codec->last_unit_dts = 0;
+		if (!started) return GF_OK;
 	}
 
 	codec->odm->current_time = codec->last_unit_cts = gf_clock_time(codec->ck);
 
 	/*lock scene*/
-	gf_term_lock_renderer(codec->odm->term, 1);
+	GF_LOG(GF_LOG_DEBUG, GF_LOG_CODEC, ("[PrivateDec] Codec %s Processing at %d\n", sdec->module_name , codec->odm->current_time));
+
+	//gf_term_lock_compositor(codec->odm->term, 1);
+	if (!gf_mx_try_lock(codec->odm->term->compositor->mx)) return GF_OK;
 	now = gf_term_get_time(codec->odm->term);
-	e = sdec->ProcessData(sdec, NULL, codec->odm->current_time, ch->esd->ESID, codec->odm->current_time, GF_CODEC_LEVEL_NORMAL);
+	e = sdec->ProcessData(sdec, NULL, 0, ch->esd->ESID, codec->odm->current_time, GF_CODEC_LEVEL_NORMAL);
 	now = gf_term_get_time(codec->odm->term) - now;
 	codec->last_unit_dts ++;
 	/*resume on error*/
@@ -481,7 +498,7 @@ static GF_Err PrivateScene_Process(GF_Codec *codec, u32 TimeAvailable)
 
 	codec_update_stats(codec, 0, now);
 
-	gf_term_lock_renderer(codec->odm->term, 0);
+	gf_term_lock_compositor(codec->odm->term, 0);
 
 	if (e==GF_EOS) {
 		/*first end of stream, evaluate duration*/
@@ -492,31 +509,27 @@ static GF_Err PrivateScene_Process(GF_Codec *codec, u32 TimeAvailable)
 	return e;
 }
 /*Get a pointer to the next CU buffer*/
-static GFINLINE GF_Err LockCompositionUnit(GF_Codec *dec, u32 CU_TS, char **outBuffer, u32 *availableSize)
+static GFINLINE GF_Err LockCompositionUnit(GF_Codec *dec, u32 CU_TS, GF_CMUnit **cu, u32 *cu_size)
 {
-	GF_CMUnit *cu;
-	*outBuffer = NULL;
-	*availableSize = 0;
-
 	if (!dec->CB) return GF_BAD_PARAM;
 	
-	cu = gf_cm_lock_input(dec->CB, CU_TS);
-	if (!cu ) return GF_OUT_OF_MEM;
-	
-	cu->TS = CU_TS;
-	*outBuffer = cu->data;
-	*availableSize = dec->CB->UnitSize;
+	*cu = gf_cm_lock_input(dec->CB, CU_TS, dec->is_reordering);
+	if (! *cu ) return GF_OUT_OF_MEM;
+	*cu_size = dec->CB->UnitSize;
 	return GF_OK;
 }
 
 
-static GFINLINE GF_Err UnlockCompositionUnit(GF_Codec *dec, u32 CTS, u32 NbBytes)
+static GFINLINE GF_Err UnlockCompositionUnit(GF_Codec *dec, GF_CMUnit *CU, u32 cu_size)
 {
 	/*temporal scalability disabling: if we already rendered this, no point getting further*/
-	if (CTS < dec->CB->LastRenderedTS) NbBytes = 0;
+	if (CU->TS < dec->CB->LastRenderedTS) {
+		GF_LOG(GF_LOG_INFO, GF_LOG_CODEC, ("[ODM] CU (TS %d) later than last frame drawn (TS %d) - droping\n", CU->TS, dec->CB->LastRenderedTS));
+		cu_size = 0;
+	} 
 
 	/*unlock the CB*/
-	gf_cm_unlock_input(dec->CB, CTS, NbBytes);
+	gf_cm_unlock_input(dec->CB, CU, cu_size, dec->is_reordering);
 	return GF_OK;
 }
 
@@ -526,11 +539,14 @@ static GF_Err ResizeCompositionBuffer(GF_Codec *dec, u32 NewSize)
 	if (!dec || !dec->CB) return GF_BAD_PARAM;
 	
 	/*update config*/
-	MO_UpdateCaps(dec->odm->mo);
+	gf_mo_update_caps(dec->odm->mo);
 
 	/*bytes per sec not available: either video or audio not configured*/
 	if (!dec->bytes_per_sec) {
-		if (NewSize && (NewSize != dec->CB->UnitSize) ) gf_cm_resize(dec->CB, NewSize);
+		if (NewSize && (NewSize != dec->CB->UnitSize) ) {
+			GF_LOG(GF_LOG_DEBUG, GF_LOG_CODEC, ("[ODM] Resizing composition buffer for codec %s - %d bytes per unit\n", dec->decio->module_name, NewSize));
+			gf_cm_resize(dec->CB, NewSize);
+		}
 	} 
 	/*audio: make sure we have enough data in CM to entirely fill the HW audio buffer...*/
 	else {
@@ -559,18 +575,18 @@ static GF_Err ResizeCompositionBuffer(GF_Codec *dec, u32 NewSize)
 		if (!dec->CB->Min) dec->CB->Min = 1;
 	}
 	if ((dec->type==GF_STREAM_VISUAL) && dec->odm->parentscene->is_dynamic_scene) {
-		gf_is_force_scene_size_video(dec->odm->parentscene, dec->odm->mo);
+		gf_inline_force_scene_size_video(dec->odm->parentscene, dec->odm->mo);
 	}
 	return GF_OK;
 }
 
 static GF_Err MediaCodec_Process(GF_Codec *codec, u32 TimeAvailable)
 {
+	GF_CMUnit *CU;
 	GF_DBUnit *AU;
-	GF_Channel *ch;
-	char *cu_buf;
-	u32 cu_buf_size, mmlevel;
-	u32 first, entryTime, now, obj_time;
+	GF_Channel *ch, *prev_ch;
+	u32 mmlevel, cts;
+	u32 first, entryTime, now, obj_time, unit_size;
 	GF_MediaDecoder *mdec = (GF_MediaDecoder*)codec->decio;
 	GF_Err e = GF_OK;
 
@@ -588,10 +604,10 @@ static GF_Err MediaCodec_Process(GF_Codec *codec, u32 TimeAvailable)
 		if (codec->Status == GF_ESM_CODEC_EOS) {
 			/*if codec is reordering, try to flush it*/
 			if (codec->is_reordering) {
-				if ( LockCompositionUnit(codec, codec->last_unit_cts+1, &cu_buf, &cu_buf_size) == GF_OUT_OF_MEM) 
+				if ( LockCompositionUnit(codec, codec->last_unit_cts+1, &CU, &unit_size) == GF_OUT_OF_MEM) 
 					return GF_OK;
-				e = mdec->ProcessData(mdec, NULL, 0, 0, cu_buf, &cu_buf_size, 0, 0);
-				if (e==GF_OK) e = UnlockCompositionUnit(codec, codec->last_unit_cts+1, cu_buf_size);
+				e = mdec->ProcessData(mdec, NULL, 0, 0, CU->data, &unit_size, 0, 0);
+				if (e==GF_OK) e = UnlockCompositionUnit(codec, CU, unit_size);
 			}
 			gf_term_stop_codec(codec);
 			if (codec->CB) gf_cm_set_eos(codec->CB);
@@ -619,7 +635,7 @@ static GF_Err MediaCodec_Process(GF_Codec *codec, u32 TimeAvailable)
 	/*try to refill the full buffer*/
 	first = 1;
 	while (codec->CB->Capacity > codec->CB->UnitCount) {
-		/*set media processing level*/
+	/*set media processing level*/
 		mmlevel = GF_CODEC_LEVEL_NORMAL;
 		/*SEEK: if the last frame had the same TS, we are seeking. Ask the codec to drop*/
 		if (!ch->skip_sl && codec->last_unit_cts && (codec->last_unit_cts == AU->CTS) && !ch->esd->dependsOnESID) {
@@ -629,7 +645,7 @@ static GF_Err MediaCodec_Process(GF_Codec *codec, u32 TimeAvailable)
 				gf_cm_rewind_input(codec->CB);
 				mmlevel = GF_CODEC_LEVEL_NORMAL;
 				/*force staying in step-mode*/
-				codec->odm->term->renderer->step_mode=1;
+				codec->odm->term->compositor->step_mode=1;
 			}
 		}
 		/*only perform drop in normal playback*/
@@ -638,7 +654,7 @@ static GF_Err MediaCodec_Process(GF_Codec *codec, u32 TimeAvailable)
 			 NOTE: the 100 ms safety gard is to avoid discarding audio*/
 			if (!ch->skip_sl && (AU->CTS + 100 < obj_time) ) {
 				mmlevel = GF_CODEC_LEVEL_DROP;
-				GF_LOG(GF_LOG_INFO, GF_LOG_CODEC, ("[Decoder] ODM%d: frame too late (%d vs %d) - using drop level\n", codec->odm->OD->objectDescriptorID, AU->CTS, obj_time));
+				GF_LOG(GF_LOG_INFO, GF_LOG_MEDIA, ("[Decoder] ODM%d: frame too late (%d vs %d) - using drop level\n", codec->odm->OD->objectDescriptorID, AU->CTS, obj_time));
 			}
 			/*we are late according to the media manager*/
 			else if (codec->PriorityBoost) {
@@ -670,68 +686,78 @@ static GF_Err MediaCodec_Process(GF_Codec *codec, u32 TimeAvailable)
 				AU->CTS = codec->last_unit_cts + ch->ts_offset + (u32) (codec->cur_video_frames * 1000 / codec->fps);
 			}
 		}
-
-		if ( LockCompositionUnit(codec, AU->CTS, &cu_buf, &cu_buf_size) == GF_OUT_OF_MEM) 
+		if ( LockCompositionUnit(codec, AU->CTS, &CU, &unit_size) == GF_OUT_OF_MEM) 
 			return GF_OK;
 
+scalable_retry:
+
 		now = gf_term_get_time(codec->odm->term);
 
-		e = mdec->ProcessData(mdec, AU->data, AU->dataLength, 
-			ch->esd->ESID, cu_buf, &cu_buf_size, AU->PaddingBits, mmlevel);
+		e = mdec->ProcessData(mdec, AU->data, AU->dataLength, ch->esd->ESID, CU->data, &unit_size, AU->PaddingBits, mmlevel);
 		now = gf_term_get_time(codec->odm->term) - now;
+		if (codec->Status == GF_ESM_CODEC_STOP) return GF_OK;
 
 		/*input is too small, resize composition memory*/
 		switch (e) {
 		case GF_BUFFER_TOO_SMALL:
 			/*release but no dispatch*/
-			UnlockCompositionUnit(codec, AU->CTS, 0);
-			if (ResizeCompositionBuffer(codec, cu_buf_size)==GF_OK) continue;
-			break;
-		case GF_OK:
+			UnlockCompositionUnit(codec, CU, 0);
+			ResizeCompositionBuffer(codec, unit_size);
+			continue;
+
+		/*this happens a lot when using non-MPEG-4 streams (ex: ffmpeg demuxer)*/
+		case GF_PACKED_FRAMES:
 			/*in seek don't dispatch any output*/
-			if (mmlevel	== GF_CODEC_LEVEL_SEEK) cu_buf_size = 0;
-			e = UnlockCompositionUnit(codec, AU->CTS, cu_buf_size);
+			if (mmlevel	== GF_CODEC_LEVEL_SEEK) 
+				unit_size = 0;
+			e = UnlockCompositionUnit(codec, CU, unit_size);
+
+			GF_LOG(GF_LOG_DEBUG, GF_LOG_RTI|GF_LOG_CODEC, ("[%s] ODM%d at %d decoded packed frame TS %d in %d ms\n", codec->decio->module_name, codec->odm->OD->objectDescriptorID, obj_time, AU->CTS, now));
+			if (ch->skip_sl) {
+				if (codec->bytes_per_sec) {
+					codec->cur_audio_bytes += unit_size;
+				} else if (codec->fps && unit_size) {
+					codec->cur_video_frames += 1;
+				}
+			} else {
+				u32 deltaTS = 0;
+				if (codec->bytes_per_sec) {
+					deltaTS = unit_size * 1000 / codec->bytes_per_sec;
+				} /*else if (0 && codec->fps && unit_size) {
+					deltaTS = (u32) (1000.0f / codec->fps);
+				} */else {
+					deltaTS = (AU->DTS - codec->last_unit_dts);
+				}
+				AU->CTS += deltaTS;
+			}
+			codec_update_stats(codec, 0, now);
+			continue;
+
+		/*for all cases below, don't release the composition buffer until we are sure we are not
+		processing a scalable stream*/
+		case GF_OK:
+			if (unit_size) {
+				GF_LOG(GF_LOG_DEBUG, GF_LOG_RTI|GF_LOG_CODEC, ("[%s] ODM%d at %d decoded frame TS %d in %d ms (DTS %d)\n", codec->decio->module_name, codec->odm->OD->objectDescriptorID, obj_time, AU->CTS, now, AU->DTS));
+			}
+			/*in seek don't dispatch any output*/
+			if (mmlevel	== GF_CODEC_LEVEL_SEEK) 
+				unit_size = 0;
+
 			codec_update_stats(codec, AU->dataLength, now);
 			if (ch->skip_sl) {
 				if (codec->bytes_per_sec) {
-					codec->cur_audio_bytes += cu_buf_size;
+					codec->cur_audio_bytes += unit_size;
 					while (codec->cur_audio_bytes>codec->bytes_per_sec) {
 						codec->cur_audio_bytes -= codec->bytes_per_sec;
 						codec->last_unit_cts += 1000;
 					}
-				} else if (codec->fps && cu_buf_size) {
+				} else if (codec->fps && unit_size) {
 					codec->cur_video_frames += 1;
 				}
 			}
-			GF_LOG(GF_LOG_DEBUG, GF_LOG_CODEC, ("[Decoder %s] ODM%d: decoded frame TS %d in %d ms\n", codec->decio->module_name, codec->odm->OD->objectDescriptorID, AU->CTS, now));
 			break;
-		/*this happens a lot when using non-MPEG-4 streams (ex: ffmpeg demuxer)*/
-		case GF_PACKED_FRAMES:
-			/*in seek don't dispatch any output*/
-			if (mmlevel	== GF_CODEC_LEVEL_SEEK) cu_buf_size = 0;
-			e = UnlockCompositionUnit(codec, AU->CTS, cu_buf_size);
-
-			if (ch->skip_sl) {
-				if (codec->bytes_per_sec) {
-					codec->cur_audio_bytes += cu_buf_size;
-				} else if (codec->fps && cu_buf_size) {
-					codec->cur_video_frames += 1;
-				}
-			} else {
-			  u32 deltaTS = 0;
-				if (codec->bytes_per_sec) {
-					deltaTS = cu_buf_size * 1000 / codec->bytes_per_sec;
-				} else if (codec->fps && cu_buf_size) {
-					deltaTS = (u32) (1000.0f / codec->fps);
-				}
-				//AU->DTS += deltaTS;
-				AU->CTS += deltaTS;
-			}
-			codec_update_stats(codec, 0, now);
-			GF_LOG(GF_LOG_DEBUG, GF_LOG_CODEC, ("[Decoder] ODM%d: decoded packed frame TS %d in %d ms\n", codec->odm->OD->objectDescriptorID, AU->CTS, now));
-			continue;
 		default:
-			UnlockCompositionUnit(codec, AU->CTS, 0);
+			unit_size = 0;
 			/*error - if the object is in intitial buffering resume it!!*/
 			gf_cm_abort_buffering(codec->CB);
 			break;
@@ -741,21 +767,34 @@ static GF_Err MediaCodec_Process(GF_Codec *codec, u32 TimeAvailable)
 		/*remember base layer timing*/
 		if (!ch->esd->dependsOnESID && !ch->skip_sl) codec->last_unit_cts = AU->CTS;
 
+
 drop:
+		/*store current CTS*/
+		cts = AU->CTS;
+		prev_ch = ch;
+
 		gf_es_drop_au(ch);
 		if (e) {
-			GF_LOG(GF_LOG_DEBUG, GF_LOG_CODEC, ("[Decoder %s] ODM%d: decoded error %s\n", codec->decio->module_name, codec->odm->OD->objectDescriptorID, gf_error_to_string(e) ));
+			UnlockCompositionUnit(codec, CU, unit_size);
+			GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("[Decoder %s] ODM%d: decoded error %s\n", codec->decio->module_name, codec->odm->OD->objectDescriptorID, gf_error_to_string(e) ));
 			return e;
 		}
 
+		Decoder_GetNextAU(codec, &ch, &AU);
+		/*same CTS: same output, likely scalable stream so don't release the CB*/
+		if (AU && (AU->CTS == cts) && (ch !=prev_ch) )
+			goto scalable_retry;
+
+		UnlockCompositionUnit(codec, CU, unit_size);
+		if (!ch || !AU) return GF_OK;
+
 		/*escape from decoding loop only if above critical limit - this is to avoid starvation on audio*/
-		if (codec->CB->UnitCount > codec->CB->Min) {
+		if (!ch->esd->dependsOnESID && (codec->CB->UnitCount > codec->CB->Min)) {
 			now = gf_term_get_time(codec->odm->term);
-			if (now - entryTime >= TimeAvailable - TIME_CHECK) {
+			if (0 && now - entryTime + TIME_CHECK >= TimeAvailable) {
 				return GF_OK;
 			}
 		}
-
 		Decoder_GetNextAU(codec, &ch, &AU);
 		if (!ch || !AU) return GF_OK;
 	}
@@ -862,7 +901,6 @@ static GF_Err Codec_LoadModule(GF_Codec *codec, GF_ESD *esd, u32 PL)
 	u32 cfg_size;
 	GF_Terminal *term = codec->odm->term;
 
-
 	if (esd->decoderConfig->decoderSpecificInfo) {
 		cfg = esd->decoderConfig->decoderSpecificInfo->data;
 		cfg_size = esd->decoderConfig->decoderSpecificInfo->dataLength;
@@ -951,6 +989,10 @@ GF_Err Codec_Load(GF_Codec *codec, GF_ESD *esd, u32 PL)
 		return GF_OK;
 
 	/*load decoder module*/
+	case GF_STREAM_VISUAL:
+	case GF_STREAM_AUDIO:
+		if (!esd->decoderConfig->objectTypeIndication) 
+			return GF_NON_COMPLIANT_BITSTREAM;
 	default:
 		return Codec_LoadModule(codec, esd, PL);
 	}
