@@ -28,11 +28,13 @@
 #include <gpac/constants.h>
 #include <gpac/internal/terminal_dev.h>
 #include "media_control.h"
-/*for inline scene rendering*/
-#include <gpac/renderer.h>
-
-void MO_UpdateCaps(GF_MediaObject *mo);
+#include <gpac/compositor.h>
+#include <gpac/nodes_x3d.h>
 
+/*SVG properties*/
+#ifndef GPAC_DISABLE_SVG
+#include <gpac/scenegraph_svg.h>
+#endif
 
 /*extern proto fetcher*/
 typedef struct
@@ -42,20 +44,38 @@ typedef struct
 } ProtoLink;
 
 GF_EXPORT
-Double gf_is_get_time(void *_is)
+Double gf_inline_get_time(void *_is)
 {
+	GF_InlineScene *is = (GF_InlineScene *)_is;
+#if 0
 	u32 ret;
 	GF_Clock *ck;
-	GF_InlineScene *is = (GF_InlineScene *)_is;
 	assert(is);
 	ck = is->scene_codec ? is->scene_codec->ck : is->dyn_ck;
 	if (!ck) return 0.0;
 	ret = gf_clock_time(ck);
 	if (is->root_od->media_stop_time && (is->root_od->media_stop_time<ret)) ret = (u32) is->root_od->media_stop_time;
 	return ret/1000.0;
+#else
+	return is->simulation_time;
+#endif
 }
 
-GF_InlineScene *gf_is_new(GF_InlineScene *parentScene)
+void gf_inline_sample_time(GF_InlineScene *is)
+{
+	u32 ret;
+	GF_Clock *ck;
+	ck = is->scene_codec ? is->scene_codec->ck : is->dyn_ck;
+	if (!ck) 
+		is->simulation_time = 0;
+	else {
+		ret = gf_clock_time(ck);
+		if (is->root_od->media_stop_time && (is->root_od->media_stop_time<ret)) ret = (u32) is->root_od->media_stop_time;
+		is->simulation_time = ((Double) ret) / 1000.0;
+	}
+}
+
+GF_InlineScene *gf_inline_new(GF_InlineScene *parentScene)
 {
 	GF_InlineScene *tmp;
 	GF_SAFEALLOC(tmp, GF_InlineScene);
@@ -75,13 +95,13 @@ GF_InlineScene *gf_is_new(GF_InlineScene *parentScene)
 
 	gf_sg_set_private(tmp->graph, tmp);
 	gf_sg_set_node_callback(tmp->graph, gf_term_node_callback);
-	gf_sg_set_scene_time_callback(tmp->graph, gf_is_get_time);
+	gf_sg_set_scene_time_callback(tmp->graph, gf_inline_get_time);
 
-	gf_sg_set_proto_loader(tmp->graph, gf_is_get_proto_lib);
+	gf_sg_set_proto_loader(tmp->graph, gf_inline_get_proto_lib);
 	return tmp;
 }
 
-void gf_is_del(GF_InlineScene *is)
+void gf_inline_del(GF_InlineScene *is)
 {
 	gf_list_del(is->ODlist);
 	gf_list_del(is->inline_nodes);
@@ -123,6 +143,7 @@ void gf_is_del(GF_InlineScene *is)
 		if (obj->odm) obj->odm->mo = NULL;
 		gf_list_rem(is->media_objects, 0);
 		gf_sg_vrml_mf_reset(&obj->URLs, GF_SG_VRML_MFURL);
+		gf_list_del(obj->nodes);
 		free(obj);
 	}
 	gf_list_del(is->media_objects);
@@ -130,34 +151,38 @@ void gf_is_del(GF_InlineScene *is)
 	if (is->audio_url.url) free(is->audio_url.url);
 	if (is->visual_url.url) free(is->visual_url.url);
 	if (is->text_url.url) free(is->text_url.url);
+	if (is->fragment_uri) free(is->fragment_uri);
+	if (is->redirect_xml_base) free(is->redirect_xml_base);
 	free(is);
 }
 
 GF_EXPORT
-GF_ObjectManager *gf_is_find_odm(GF_InlineScene *is, u16 OD_ID)
+GF_ObjectManager *gf_inline_find_odm(GF_InlineScene *is, u16 OD_ID)
 {
 	GF_ObjectManager *odm;
 	u32 i=0;
 	while ((odm = (GF_ObjectManager *)gf_list_enum(is->ODlist, &i))) {
-		if (odm->OD->objectDescriptorID == OD_ID) return odm;
+		if (odm->OD && odm->OD->objectDescriptorID == OD_ID) return odm;
 	}
 	return NULL;
 }
 
-void gf_is_disconnect(GF_InlineScene *is, Bool for_shutdown)
+GF_EXPORT
+void gf_inline_disconnect(GF_InlineScene *is, Bool for_shutdown)
 {
+	u32 i;
 	GF_MediaObject *obj;
 	GF_Node *root_node;
 	GF_ObjectManager *odm;
 	GF_SceneDecoder *dec = NULL;
 	if (is->scene_codec) dec = (GF_SceneDecoder *)is->scene_codec->decio;
 
-	gf_term_lock_renderer(is->root_od->term, 1);
+	gf_term_lock_compositor(is->root_od->term, 1);
 	
 	/*disconnect / kill all objects BEFORE reseting the scene graph since we have 
 	potentially registered Inline nodes of the graph with the sub-scene*/
 	if (!for_shutdown && is->static_media_ressources) {
-		u32 i=0;
+		i=0;
 		/*stop all objects but DON'T DESTROY THEM*/
 		while ((odm = (GF_ObjectManager *)gf_list_enum(is->ODlist, &i))) {
 			if (odm->state) gf_odm_disconnect(odm, 0);
@@ -166,6 +191,7 @@ void gf_is_disconnect(GF_InlineScene *is, Bool for_shutdown)
 		i=0;
 		while ((obj = (GF_MediaObject*)gf_list_enum(is->media_objects, &i))) {
 			gf_sg_vrml_mf_reset(&obj->URLs, GF_SG_VRML_MFURL);
+			gf_list_reset(obj->nodes);
 		}
 	} else {
 		while (gf_list_count(is->ODlist)) {
@@ -175,21 +201,33 @@ void gf_is_disconnect(GF_InlineScene *is, Bool for_shutdown)
 	}
 	
 	root_node = gf_sg_get_root_node(is->graph);
+	/*reset private stack of all inline nodes still registered*/
 	while (gf_list_count(is->inline_nodes)) {
 		GF_Node *n = (GF_Node *)gf_list_get(is->inline_nodes, 0);
 		gf_list_rem(is->inline_nodes, 0);
-		gf_node_set_private(n, NULL);
+		switch (gf_node_get_tag(n)) {
+		case TAG_MPEG4_Inline:
+		case TAG_X3D_Inline:
+			gf_node_set_private(n, NULL);
+			break;
+		}
+	}
+
+	/*remove all associated eventTargets - THIS ENEDS CLEANUP*/
+	i=0;
+	while ((obj = (GF_MediaObject *)gf_list_enum(is->ODlist, &i))) {
+		if (obj->nodes) gf_list_reset(obj->nodes);
 	}
 
 	if (is->graph_attached && (is->root_od->term->root_scene == is)) {
-		gf_sr_set_scene(is->root_od->term->renderer, NULL);
+		gf_sc_set_scene(is->root_od->term->compositor, NULL);
 	}
 	/*release the scene*/
 	if (dec && dec->ReleaseScene) dec->ReleaseScene(dec);
 	gf_sg_reset(is->graph);
 	is->graph_attached = 0;
 	
-	gf_term_lock_renderer(is->root_od->term, 0);
+	gf_term_lock_compositor(is->root_od->term, 0);
 
 
 	assert(!gf_list_count(is->extra_scenes) );
@@ -202,11 +240,12 @@ void gf_is_disconnect(GF_InlineScene *is, Bool for_shutdown)
 		gf_list_rem(is->media_objects, 0);
 		if (obj->odm) obj->odm->mo = NULL;
 		gf_sg_vrml_mf_reset(&obj->URLs, GF_SG_VRML_MFURL);
+		gf_list_del(obj->nodes);
 		free(obj);
 	}
 }
 
-static void IS_InsertObject(GF_InlineScene *is, GF_MediaObject *mo, Bool lock_timelines, GF_MediaObject *sync_ref)
+static void IS_InsertObject(GF_InlineScene *is, GF_MediaObject *mo, Bool lock_timelines, GF_MediaObject *sync_ref, Bool keep_fragment)
 {
 	GF_ObjectManager *root_od;
 	GF_ObjectManager *odm;
@@ -256,7 +295,7 @@ static void IS_InsertObject(GF_InlineScene *is, GF_MediaObject *mo, Bool lock_ti
 		esd->ESID = esd->OCRESID = 65534;
 		gf_list_add(odm->OD->ESDescriptors, esd);
 	} else {
-		if (mo->type==GF_MEDIA_OBJECT_SCENE) {
+		if (!keep_fragment) {
 			char *frag = strrchr(mo->URLs.vals[0].url, '#');
 			if (frag) frag[0] = 0;
 			odm->OD->URLString = strdup(mo->URLs.vals[0].url);
@@ -282,11 +321,12 @@ static void IS_ReinsertObject(GF_InlineScene *is, GF_MediaObject *mo)
 	for (i=0; i<mo->URLs.count-1; i++) mo->URLs.vals[i].url = mo->URLs.vals[i+1].url;
 	mo->URLs.vals[mo->URLs.count-1].url = NULL;
 	mo->URLs.count-=1;
-	IS_InsertObject(is, mo, 0, NULL);
+	/*FIXME - we should re-ananlyse whether the fragment is important or not ...*/
+	IS_InsertObject(is, mo, 0, NULL, 0);
 }
 
 
-void gf_is_remove_object(GF_InlineScene *is, GF_ObjectManager *odm, Bool for_shutdown)
+void gf_inline_remove_object(GF_InlineScene *is, GF_ObjectManager *odm, Bool for_shutdown)
 {
 	u32 i;
 	GF_MediaObject *obj;
@@ -327,6 +367,7 @@ void gf_is_remove_object(GF_InlineScene *is, GF_ObjectManager *odm, Bool for_shu
 				}
 				gf_list_rem(is->media_objects, i-1);
 				gf_sg_vrml_mf_reset(&obj->URLs, GF_SG_VRML_MFURL);
+				gf_list_del(obj->nodes);
 				free(obj);
 			} else if (!for_shutdown) {
 				/*if dynamic OD and more than 1 URLs resetup*/
@@ -395,7 +436,7 @@ u32 URL_GetODID(MFURL *url)
 
 
 //browse all channels and update buffering info
-void gf_is_buffering_info(GF_InlineScene *is)
+void gf_inline_buffering_info(GF_InlineScene *is)
 {
 	u32 i, j, max_buffer, cur_buffer;
 	GF_Channel *ch;
@@ -450,11 +491,11 @@ static Bool Inline_SetScene(M_Inline *root)
 	parent = (GF_InlineScene *)gf_sg_get_private(graph);
 	if (!parent) return 0;
 
-	mo = gf_is_get_media_object(parent, &root->url, GF_MEDIA_OBJECT_SCENE, 0);
+	mo = gf_inline_get_media_object_ex(parent, &root->url, GF_MEDIA_OBJECT_SCENE, 0, NULL, 0, (GF_Node*)root);
 	if (!mo || !mo->odm) return 0;
 
 	if (!mo->odm->subscene) {
-		gf_term_invalidate_renderer(parent->root_od->term);
+		gf_term_invalidate_compositor(parent->root_od->term);
 		return 0;
 	}
 	/*assign inline scene as private stack of inline node, and remember inline node for event propagation*/
@@ -465,16 +506,13 @@ static Bool Inline_SetScene(M_Inline *root)
 	return 1;
 }
 
-Bool gf_mo_is_same_url(GF_MediaObject *obj, MFURL *an_url)
+static Bool gf_mo_is_same_url_ex(GF_MediaObject *obj, MFURL *an_url, Bool *keep_fragment, u32 obj_hint_type)
 {
 	Bool include_sub_url = 0;
 	u32 i;
 	char szURL1[GF_MAX_PATH], szURL2[GF_MAX_PATH], *ext;
 
-	/*don't analyse audio/video to locate segments or viewports*/
-	if (obj->type==GF_MEDIA_OBJECT_AUDIO) include_sub_url = 1;
-	if (obj->type==GF_MEDIA_OBJECT_VIDEO) include_sub_url = 1;
-
+	if (keep_fragment) *keep_fragment = 0;
 	if (obj->OD_ID==GF_ESM_DYNAMIC_OD_ID) {
 		if (!obj->URLs.count) {
 			if (!obj->odm) return 0;
@@ -486,6 +524,55 @@ Bool gf_mo_is_same_url(GF_MediaObject *obj, MFURL *an_url)
 		if (!obj->URLs.count) return 0;
 		strcpy(szURL1, obj->URLs.vals[0].url);
 	}
+
+	/*don't analyse audio/video to locate segments or viewports*/
+	if (obj->type==GF_MEDIA_OBJECT_AUDIO) 
+		include_sub_url = 1;
+	else if (obj->type==GF_MEDIA_OBJECT_VIDEO) 
+		include_sub_url = 1;
+	else if ((obj->type==GF_MEDIA_OBJECT_SCENE) && keep_fragment && obj->odm) {
+		GF_ClientService *ns;
+		u32 j;
+		/*for remoteODs/dynamic ODs, check if one of the running service cannot be used*/
+		for (i=0; i<an_url->count; i++) {
+			char *frag = strrchr(an_url->vals[i].url, '#');
+			j=0;
+			/*this is the same object (may need some refinement)*/
+			if (!stricmp(szURL1, an_url->vals[i].url)) return 1;
+
+			/*fragment is a media segment, same URL*/
+			if (frag ) {
+				Bool same_res = 0;
+				frag[0] = 0;
+				same_res = !strncmp(an_url->vals[i].url, szURL1, strlen(an_url->vals[i].url)) ? 1 : 0;
+				frag[0] = '#';
+
+				/*if we're talking about the same resource, check if the fragment can be matched*/
+				if (same_res) {
+					/*if the expected type is a segment (undefined media type) 
+					and the fragment is a media segment, same URL
+					*/
+					if (obj->odm->subscene && (gf_sg_find_node_by_name(obj->odm->subscene->graph, frag+1)!=NULL) )
+						return 1;
+				
+					/*if the expected type is a segment (undefined media type) 
+					and the fragment is a media segment, same URL
+					*/
+					if (!obj_hint_type && gf_odm_find_segment(obj->odm, frag+1))
+						return 1;
+				}
+			}
+
+			while ( (ns = (GF_ClientService*)gf_list_enum(obj->odm->term->net_services, &j)) ) {
+				/*sub-service of an existing service - don't touch any fragment*/
+				if (gf_term_service_can_handle_url(ns, an_url->vals[i].url)) {
+					*keep_fragment = 1;
+					return 0;
+				}
+			}
+		}
+	}
+
 	/*check on full URL without removing fragment IDs*/
 	if (include_sub_url) {
 		for (i=0; i<an_url->count; i++) {
@@ -496,6 +583,7 @@ Bool gf_mo_is_same_url(GF_MediaObject *obj, MFURL *an_url)
 	ext = strrchr(szURL1, '#');
 	if (ext) ext[0] = 0;
 	for (i=0; i<an_url->count; i++) {
+		if (!an_url->vals[i].url) return 0;
 		strcpy(szURL2, an_url->vals[i].url);
 		ext = strrchr(szURL2, '#');
 		if (ext) ext[0] = 0;
@@ -504,7 +592,12 @@ Bool gf_mo_is_same_url(GF_MediaObject *obj, MFURL *an_url)
 	return 0;
 }
 
-void gf_is_on_modified(GF_Node *node)
+Bool gf_mo_is_same_url(GF_MediaObject *obj, MFURL *an_url)
+{
+	return gf_mo_is_same_url_ex(obj, an_url, NULL, 0);
+}
+
+void gf_inline_on_modified(GF_Node *node)
 {
 	u32 ODID;
 	GF_MediaObject *mo;
@@ -528,7 +621,7 @@ void gf_is_on_modified(GF_Node *node)
 				mo->num_open --;
 				if (!mo->num_open) {
 					gf_odm_stop(pIS->root_od, 1);
-					gf_is_disconnect(pIS, 1);
+					gf_inline_disconnect(pIS, 1);
 					assert(gf_list_count(pIS->ODlist) == 0);
 				}
 			}
@@ -570,20 +663,20 @@ static void IS_CheckMediaRestart(GF_InlineScene *is)
 			}
 		} else {
 			/*trigger render until to watch for restart...*/
-			gf_term_invalidate_renderer(is->root_od->term);
+			gf_term_invalidate_compositor(is->root_od->term);
 		}
 	}
 }
 
-
-void gf_is_render(GF_Node *n, void *render_stack, Bool is_destroy)
+static void gf_inline_traverse(GF_Node *n, void *rs, Bool is_destroy)
 {
-	GF_Node *root;
 	GF_InlineScene *is = (GF_InlineScene *)gf_node_get_private(n);
 
 	if (is_destroy) {
 		GF_MediaObject *mo = (is && is->root_od) ? is->root_od->mo : NULL;
 
+		if (is) gf_list_del_item(is->inline_nodes, n);
+
 		if (!mo) return;
 		/*disconnect current inline if we're the last one using it (same as regular OD session leave/join)*/
 		if (mo->num_open) {
@@ -593,9 +686,17 @@ void gf_is_render(GF_Node *n, void *render_stack, Bool is_destroy)
 				OD framework is destroyed, destroy the associated resource*/
 				if (mo->OD_ID == GF_ESM_DYNAMIC_OD_ID) {
 					gf_odm_disconnect(is->root_od, 1);
+
+					/*get parent scene and remove MediaObject in case the ressource
+					gets re-requested later on*/
+					is = (GF_InlineScene *)gf_sg_get_private(gf_node_get_graph((GF_Node *) n) );
+					gf_list_del_item(is->media_objects, mo);
+					gf_sg_vrml_mf_reset(&mo->URLs, GF_SG_VRML_MFURL);
+					gf_list_del(mo->nodes);
+					free(mo);
 				} else {
 					gf_odm_stop(is->root_od, 1);
-					gf_is_disconnect(is, 1);
+					gf_inline_disconnect(is, 1);
 					assert(gf_list_count(is->ODlist) == 0);
 				}
 			}
@@ -606,11 +707,13 @@ void gf_is_render(GF_Node *n, void *render_stack, Bool is_destroy)
 
 	//if no private scene is associated	get the node parent graph, retrieve the IS and find the OD
 	if (!is) {
-		Inline_SetScene((M_Inline *) n);
+		M_Inline *inl = (M_Inline *)n;
+		Inline_SetScene(inl);
 		is = (GF_InlineScene *)gf_node_get_private(n);
 		if (!is) {
 			/*just like protos, we must invalidate parent graph until attached*/
-			gf_node_dirty_set(n, 0, 1);
+			if (inl->url.count)
+				gf_node_dirty_set(n, 0, 1);
 			return;
 		}
 	}
@@ -623,7 +726,7 @@ void gf_is_render(GF_Node *n, void *render_stack, Bool is_destroy)
 		/*special case: scene change*/
 		if (is->needs_restart==2) {
 			is->needs_restart = 0;
-			gf_is_on_modified(n);
+			gf_inline_on_modified(n);
 			return;
 		}
 
@@ -632,17 +735,17 @@ void gf_is_render(GF_Node *n, void *render_stack, Bool is_destroy)
 
 		if (is->is_dynamic_scene) {
 			if (is->root_od->media_ctrl) is->root_od->media_ctrl->current_seg = current_seg;
-			gf_is_restart_dynamic(is, 0);
+			gf_inline_restart_dynamic(is, 0);
 		} else {
 			/*we cannot use gf_mo_restart since it only sets the needs_restart for inline scenes. 
 			The rational is that gf_mo_restart can be called from the parent scene (OK) or from the scene itself, in 
-			which case shutting down the graph would crash the renderer. We therefore need two render passes to 
+			which case shutting down the graph would crash the compositor. We therefore need two render passes to 
 			safely restart an inline scene*/
 
 			/*1- stop main object from playing but don't disconnect channels*/
 			gf_odm_stop(is->root_od, 1);
 			/*2- close all ODs inside the scene and reset the graph*/
-			gf_is_disconnect(is, 0);
+			gf_inline_disconnect(is, 0);
 			if (is->root_od->media_ctrl) is->root_od->media_ctrl->current_seg = current_seg;
 			/*3- restart the scene*/
 			gf_odm_start(is->root_od);
@@ -660,34 +763,76 @@ void gf_is_render(GF_Node *n, void *render_stack, Bool is_destroy)
 	/*clear dirty flags for any sub-inlines, bitmaps or protos*/
 	gf_node_dirty_clear(n, 0);
 	
-	root = gf_sg_get_root_node(is->graph);
-	if (root) {
-		gf_sr_render_inline(is->root_od->term->renderer, n, root, render_stack);
+	gf_sc_traverse_subscene(is->root_od->term->compositor, n, is->graph, rs);
+}
+
+
+static void gf_is_resize_event(GF_InlineScene *is)
+{
+	/*fire resize event*/
+#ifndef GPAC_DISABLE_SVG
+	u32 i, count;
+	u32 w, h;
+	GF_DOM_Event evt;
+	memset(&evt, 0, sizeof(GF_DOM_Event));
+	w = h = 0;
+	gf_sg_get_scene_size_info(is->graph, &w, &h);
+	evt.type = GF_EVENT_RESIZE;
+	evt.screen_rect.width = INT2FIX(w);
+	evt.screen_rect.height = INT2FIX(h);
+	gf_dom_event_fire(gf_sg_get_root_node(is->graph), &evt);
+	
+	count=gf_list_count(is->inline_nodes);
+	for (i=0;i<count; i++) {
+		gf_dom_event_fire( gf_list_get(is->inline_nodes, i), &evt );
 	}
+#endif
 }
 
 GF_EXPORT
-void gf_is_attach_to_renderer(GF_InlineScene *is)
+void gf_inline_attach_to_compositor(GF_InlineScene *is)
 {
+	char *url;
 	if ((is->graph_attached==1) || (gf_sg_get_root_node(is->graph)==NULL) ) {
-		gf_term_invalidate_renderer(is->root_od->term);
+		gf_term_invalidate_compositor(is->root_od->term);
 		return;
 	}
 	is->graph_attached = 1;
-	/*main display scene, setup renderer*/
-	if (is->root_od->term->root_scene == is) {
-		gf_sr_set_scene(is->root_od->term->renderer, is->graph);
+
+	/*locate fragment IRI*/
+	if (!is->root_od || !is->root_od->net_service || !is->root_od->net_service->url) return;
+	if (is->fragment_uri) {
+		free(is->fragment_uri);
+		is->fragment_uri = NULL;
 	}
-	else {		
-		gf_term_invalidate_renderer(is->root_od->term);
+	url = strchr(is->root_od->net_service->url, '#');
+	if (url) is->fragment_uri = strdup(url+1);
+
+	/*main display scene, setup compositor*/
+	if (is->root_od->term->root_scene == is) {
+		gf_sc_set_scene(is->root_od->term->compositor, is->graph);
+	}
+	else {
+		u32 i, count=gf_list_count(is->inline_nodes);
+		for (i=0;i<count; i++) 
+			gf_node_dirty_parents( gf_list_get(is->inline_nodes, i) );
+		gf_term_invalidate_compositor(is->root_od->term);
+
+		if (is->root_od->parentscene->is_dynamic_scene) {
+			u32 w, h;
+			gf_sg_get_scene_size_info(is->graph, &w, &h);
+			gf_sc_set_size(is->root_od->term->compositor, w, h);
+		}
+		gf_is_resize_event(is);
 	}
 }
 
-static GF_MediaObject *IS_CheckExistingObject(GF_InlineScene *is, MFURL *urls)
+static GF_MediaObject *IS_CheckExistingObject(GF_InlineScene *is, MFURL *urls, u32 type)
 {
 	GF_MediaObject *obj;
 	u32 i = 0;
 	while ((obj = (GF_MediaObject *)gf_list_enum(is->media_objects, &i))) {
+		if (type && (type != obj->type)) continue;
 		if ((obj->OD_ID == GF_ESM_DYNAMIC_OD_ID) && gf_mo_is_same_url(obj, urls)) return obj;
 		else if ((obj->OD_ID != GF_ESM_DYNAMIC_OD_ID) && (obj->OD_ID == urls->vals[0].OD_ID)) return obj;
 	}
@@ -699,32 +844,42 @@ static GFINLINE Bool is_match_obj_type(u32 type, u32 hint_type)
 	if (!hint_type) return 1;
 	if (type==hint_type) return 1;
 	/*TEXT are used by animation stream*/
-	if ((type==GF_MEDIA_OBJECT_TEXT) && (hint_type==GF_MEDIA_OBJECT_BIFS)) return 1;
+	if ((type==GF_MEDIA_OBJECT_TEXT) && (hint_type==GF_MEDIA_OBJECT_UPDATES)) return 1;
 	return 0;
 }
 
-GF_MediaObject *gf_is_get_media_object_ex(GF_InlineScene *is, MFURL *url, u32 obj_type_hint, Bool lock_timelines, GF_MediaObject *sync_ref)
+GF_MediaObject *gf_inline_get_media_object_ex(GF_InlineScene *is, MFURL *url, u32 obj_type_hint, Bool lock_timelines, GF_MediaObject *sync_ref, Bool always_load_new, GF_Node *node)
 {
 	GF_MediaObject *obj;
+	Bool keep_fragment = 1;
 	u32 i, OD_ID;
 
 	OD_ID = URL_GetODID(url);
 	if (!OD_ID) return NULL;
 
-	obj = NULL;
-	i=0;
-	while ((obj = (GF_MediaObject *)gf_list_enum(is->media_objects, &i))) {
-		/*regular OD scheme*/
-		if (OD_ID != GF_ESM_DYNAMIC_OD_ID && (obj->OD_ID==OD_ID)) return obj;
-
-		/*dynamic OD scheme*/
-		if ((OD_ID == GF_ESM_DYNAMIC_OD_ID) && (obj->OD_ID==GF_ESM_DYNAMIC_OD_ID)
-			/*if object type unknown (media control, media sensor), return first obj matching URL
-			otherwise check types*/
-			&& is_match_obj_type(obj->type, obj_type_hint)
-			/*locate sub-url in given one (handles viewpoint/segments)*/
-			&& gf_mo_is_same_url(obj, url) 
-			) return obj;
+	if (!always_load_new) {
+		obj = NULL;
+		i=0;
+		while ((obj = (GF_MediaObject *)gf_list_enum(is->media_objects, &i))) {
+			if (
+				/*regular OD scheme*/
+				(OD_ID != GF_ESM_DYNAMIC_OD_ID && (obj->OD_ID==OD_ID)) 
+			||
+				/*dynamic OD scheme*/
+				((OD_ID == GF_ESM_DYNAMIC_OD_ID) && (obj->OD_ID==GF_ESM_DYNAMIC_OD_ID)
+					/*if object type unknown (media control, media sensor), return first obj matching URL
+					otherwise check types*/
+					&& is_match_obj_type(obj->type, obj_type_hint)
+					/*locate sub-url in given one and handle fragments (viewpoint/segments/...)*/
+					&& gf_mo_is_same_url_ex(obj, url, &keep_fragment, obj_type_hint) 
+				)
+			) {
+				
+				if (node && (gf_list_find(obj->nodes, node)<0))
+					gf_list_add(obj->nodes, node);
+				return obj;
+			}
+		}
 	}
 	/*we cannot create an OD manager at this point*/
 	if (obj_type_hint==GF_MEDIA_OBJECT_UNDEF) return NULL;
@@ -733,10 +888,21 @@ GF_MediaObject *gf_is_get_media_object_ex(GF_InlineScene *is, MFURL *url, u32 ob
 	obj = gf_mo_new();
 	obj->OD_ID = OD_ID;
 	obj->type = obj_type_hint;
+
+	/*register node with object*/
+	if (node)
+		gf_list_add(obj->nodes, node);
+	
+	/*if animation stream object, remember originating node
+		!! FIXME - this should be cleaned up !! 
+	*/
+	if (obj->type == GF_MEDIA_OBJECT_UPDATES)
+		obj->node_ptr = node;
+
 	gf_list_add(is->media_objects, obj);
 	if (OD_ID == GF_ESM_DYNAMIC_OD_ID) {
 		gf_sg_vrml_field_copy(&obj->URLs, url, GF_SG_VRML_MFURL);
-		IS_InsertObject(is, obj, lock_timelines, sync_ref);
+		IS_InsertObject(is, obj, lock_timelines, sync_ref, keep_fragment);
 		/*safety check!!!*/
 		if (gf_list_find(is->media_objects, obj)<0) 
 			return NULL;
@@ -744,13 +910,13 @@ GF_MediaObject *gf_is_get_media_object_ex(GF_InlineScene *is, MFURL *url, u32 ob
 	return obj;
 }
 
-GF_MediaObject *gf_is_get_media_object(GF_InlineScene *is, MFURL *url, u32 obj_type_hint, Bool lock_timelines)
+GF_MediaObject *gf_inline_get_media_object(GF_InlineScene *is, MFURL *url, u32 obj_type_hint, Bool lock_timelines)
 {
-	return gf_is_get_media_object_ex(is, url, obj_type_hint, lock_timelines, NULL);
+	return gf_inline_get_media_object_ex(is, url, obj_type_hint, lock_timelines, NULL, 0, NULL);
 }
 
 GF_EXPORT
-void gf_is_setup_object(GF_InlineScene *is, GF_ObjectManager *odm)
+void gf_inline_setup_object(GF_InlineScene *is, GF_ObjectManager *odm)
 {
 	GF_MediaObject *obj;
 	u32 i;
@@ -788,28 +954,31 @@ existing:
 	else if (odm->codec->type == GF_STREAM_VISUAL) odm->mo->type = GF_MEDIA_OBJECT_VIDEO;
 	else if (odm->codec->type == GF_STREAM_AUDIO) odm->mo->type = GF_MEDIA_OBJECT_AUDIO;
 	else if (odm->codec->type == GF_STREAM_TEXT) odm->mo->type = GF_MEDIA_OBJECT_TEXT;
-	else if (odm->codec->type == GF_STREAM_SCENE) odm->mo->type = GF_MEDIA_OBJECT_BIFS;
+	else if (odm->codec->type == GF_STREAM_SCENE) odm->mo->type = GF_MEDIA_OBJECT_UPDATES;
 	
 	/*update info*/
-	MO_UpdateCaps(odm->mo);
+	gf_mo_update_caps(odm->mo);
 	/*media object playback has already been requested by the scene, trigger media start*/
 	if (odm->mo->num_open && !odm->state) {
 		gf_odm_start(odm);
 		if (odm->mo->speed != FIX_ONE) gf_odm_set_speed(odm, odm->mo->speed);
 	}
+	if ((odm->mo->type==GF_MEDIA_OBJECT_VIDEO) && is->is_dynamic_scene) {
+		gf_inline_force_scene_size_video(is, odm->mo);
+	}
 	/*invalidate scene for all nodes using the OD*/
-	gf_term_invalidate_renderer(odm->term);
+	gf_term_invalidate_compositor(odm->term);
 }
 
-void gf_is_restart(GF_InlineScene *is)
+void gf_inline_restart(GF_InlineScene *is)
 {
 	is->needs_restart = 1;
-	gf_term_invalidate_renderer(is->root_od->term);
+	gf_term_invalidate_compositor(is->root_od->term);
 }
 
 
 GF_EXPORT
-void gf_is_set_duration(GF_InlineScene *is)
+void gf_inline_set_duration(GF_InlineScene *is)
 {
 	Double dur;
 	u32 i;
@@ -885,13 +1054,13 @@ void IS_LoadExternProto(GF_InlineScene *is, MFURL *url)
 	pl = (ProtoLink*)malloc(sizeof(ProtoLink));
 	pl->url = url;
 	gf_list_add(is->extern_protos, pl);
-	pl->mo = gf_is_get_media_object(is, url, GF_MEDIA_OBJECT_SCENE, 0);
+	pl->mo = gf_inline_get_media_object(is, url, GF_MEDIA_OBJECT_SCENE, 0);
 	/*this may already be destroyed*/
 	if (pl->mo) gf_mo_play(pl->mo, 0, -1, 0);
 }
 
 GF_EXPORT
-GF_SceneGraph *gf_is_get_proto_lib(void *_is, MFURL *lib_url)
+GF_SceneGraph *gf_inline_get_proto_lib(void *_is, MFURL *lib_url)
 {
 	ProtoLink *pl;
 	u32 i;
@@ -949,7 +1118,7 @@ Bool IS_IsProtoLibObject(GF_InlineScene *is, GF_ObjectManager *odm)
 }
 
 
-GF_MediaObject *gf_is_find_object(GF_InlineScene *is, u16 ODID, char *url)
+GF_MediaObject *gf_inline_find_object(GF_InlineScene *is, u16 ODID, char *url)
 {
 	u32 i;
 	GF_MediaObject *mo;
@@ -977,7 +1146,7 @@ const char *IS_GetSceneViewName(GF_InlineScene *is)
 }
 
 GF_EXPORT
-Bool gf_is_default_scene_viewpoint(GF_Node *node)
+Bool gf_inline_default_scene_viewpoint(GF_Node *node)
 {
 	const char *nname, *sname;
 	GF_SceneGraph *sg = gf_node_get_graph(node);
@@ -992,27 +1161,27 @@ Bool gf_is_default_scene_viewpoint(GF_Node *node)
 }
 
 GF_EXPORT
-void gf_is_register_extra_graph(GF_InlineScene *is, GF_SceneGraph *extra_scene, Bool do_remove)
+void gf_inline_register_extra_graph(GF_InlineScene *is, GF_SceneGraph *extra_scene, Bool do_remove)
 {
 	if (do_remove) {
 		if (gf_list_find(is->extra_scenes, extra_scene)<0) return;
 		gf_list_del_item(is->extra_scenes, extra_scene);
 		/*for root scene*/
 		if (is->root_od->term->root_scene == is) {
-			gf_sr_register_extra_graph(is->root_od->term->renderer, extra_scene, 1);
+			gf_sc_register_extra_graph(is->root_od->term->compositor, extra_scene, 1);
 		}
 	} else {
 		if (gf_list_find(is->extra_scenes, extra_scene)>=0) return;
 		gf_list_add(is->extra_scenes, extra_scene);
 		/*for root scene*/
 		if (is->root_od->term->root_scene == is) {
-			gf_sr_register_extra_graph(is->root_od->term->renderer, extra_scene, 0);
+			gf_sc_register_extra_graph(is->root_od->term->compositor, extra_scene, 0);
 		}
 	}
 }
 
 
-static void gf_is_get_video_size(GF_MediaObject *mo, u32 *w, u32 *h)
+static void gf_inline_get_video_size(GF_MediaObject *mo, u32 *w, u32 *h)
 {
 	u32 pixel_ar;
 	if (!gf_mo_get_visual_info(mo, w, h, NULL, &pixel_ar, NULL)) return;
@@ -1034,7 +1203,7 @@ static void IS_UpdateVideoPos(GF_InlineScene *is)
 
 	url.count = 1;
 	url.vals = &is->visual_url;
-	mo = IS_CheckExistingObject(is, &url);
+	mo = IS_CheckExistingObject(is, &url, GF_MEDIA_OBJECT_VIDEO);
 	if (!mo) return;
 	tr = (M_Transform2D *) gf_sg_find_node_by_name(is->graph, "DYN_TRANS");
 	if (!tr) return;
@@ -1042,14 +1211,14 @@ static void IS_UpdateVideoPos(GF_InlineScene *is)
 	gf_sg_get_scene_size_info(is->graph, &w, &h);
 	if (!w || !h) return;
 
-	gf_is_get_video_size(mo, &v_w, &v_h);
+	gf_inline_get_video_size(mo, &v_w, &v_h);
 	tr->translation.x = INT2FIX((s32) (w - v_w)) / 2;
 	tr->translation.y = INT2FIX((s32) (h - v_h)) / 2;
 	gf_node_dirty_set((GF_Node *)tr, 0, 0);
 
 	if (is->root_od->term->root_scene == is) {
-		//if (is->graph_attached) gf_sr_set_scene(is->root_od->term->renderer, NULL);
-		gf_sr_set_scene(is->root_od->term->renderer, is->graph);
+		//if (is->graph_attached) gf_sc_set_scene(is->root_od->term->compositor, NULL);
+		gf_sc_set_scene(is->root_od->term->compositor, is->graph);
 	}
 }
 
@@ -1071,19 +1240,19 @@ static Bool is_odm_url(SFURL *url, GF_ObjectManager *odm)
 	return !stricmp(url->url, odm->OD->URLString);
 }
 
-void gf_is_force_scene_size_video(GF_InlineScene *is, GF_MediaObject *mo)
+void gf_inline_force_scene_size_video(GF_InlineScene *is, GF_MediaObject *mo)
 {
 	u32 w, h;
-	gf_is_get_video_size(mo, &w, &h);
-	gf_is_force_scene_size(is, w, h);
+	gf_inline_get_video_size(mo, &w, &h);
+	gf_inline_force_scene_size(is, w, h);
 }
 
 
 /*regenerates the scene graph for dynamic scene.
 This will also try to reload any previously presented streams. Note that in the usual case the scene is generated
-just once when recieving the first OD AU (ressources are NOT destroyed when seeking), but since the network may need
+just once when receiving the first OD AU (ressources are NOT destroyed when seeking), but since the network may need
 to update the OD ressources, we still kake care of it*/
-void gf_is_regenerate(GF_InlineScene *is)
+void gf_inline_regenerate(GF_InlineScene *is)
 {
 	u32 i, nb_obj, w, h;
 	GF_Node *n1, *n2;
@@ -1093,17 +1262,21 @@ void gf_is_regenerate(GF_InlineScene *is)
 	M_AudioClip *ac;
 	M_MovieTexture *mt;
 	M_AnimationStream *as;
+	M_Inline *dims;
 
 	if (!is->is_dynamic_scene) return;
 
 	GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("[Inline] Regenerating scene graph for service %s\n", is->root_od->net_service->url));
 
+	gf_sc_lock(is->root_od->term->compositor, 1);
+
 	if (is->root_od->term->root_scene == is) 
-		gf_sr_set_scene(is->root_od->term->renderer, NULL);
+		gf_sc_set_scene(is->root_od->term->compositor, NULL);
 
 	gf_sg_reset(is->graph);
 	gf_sg_get_scene_size_info(is->graph, &w, &h);
 	gf_sg_set_scene_size_info(is->graph, w, h, 1);
+
 	n1 = is_create_node(is->graph, TAG_MPEG4_OrderedGroup, NULL);
 	gf_sg_set_root_node(is->graph, n1);
 	gf_node_register(n1, NULL);
@@ -1113,7 +1286,7 @@ void gf_is_regenerate(GF_InlineScene *is)
 	gf_node_register(n2, n1);
 
 	ac = (M_AudioClip *) is_create_node(is->graph, TAG_MPEG4_AudioClip, "DYN_AUDIO");
-	ac->startTime = gf_is_get_time(is);
+	ac->startTime = gf_inline_get_time(is);
 	((M_Sound2D *)n2)->source = (GF_Node *)ac;
 	gf_node_register((GF_Node *)ac, n2);
 
@@ -1162,7 +1335,7 @@ void gf_is_regenerate(GF_InlineScene *is)
 
 	/*note we create a movie texture even for images...*/
 	mt = (M_MovieTexture *) is_create_node(is->graph, TAG_MPEG4_MovieTexture, "DYN_VIDEO");
-	mt->startTime = gf_is_get_time(is);
+	mt->startTime = gf_inline_get_time(is);
 	((M_Appearance *)n2)->texture = (GF_Node *)mt;
 	gf_node_register((GF_Node *)mt, n2);
 
@@ -1176,7 +1349,7 @@ void gf_is_regenerate(GF_InlineScene *is)
 			sfu->OD_ID = is->visual_url.OD_ID;
 			if (is->visual_url.url) sfu->url = strdup(is->visual_url.url);
 			if (odm->mo) {
-				gf_is_get_video_size(odm->mo, &w, &h);
+				gf_inline_get_video_size(odm->mo, &w, &h);
 				gf_sg_set_scene_size_info(is->graph, w, h, 1);
 			}
 			first_odm = NULL;
@@ -1197,7 +1370,7 @@ void gf_is_regenerate(GF_InlineScene *is)
 		if (is->visual_url.url) sfu->url = strdup(is->visual_url.url);
 	
 		if (first_odm->mo) {
-			gf_is_get_video_size(first_odm->mo, &w, &h);
+			gf_inline_get_video_size(first_odm->mo, &w, &h);
 			gf_sg_set_scene_size_info(is->graph, w, h, 1);
 		}
 		nb_obj++;
@@ -1225,7 +1398,7 @@ void gf_is_regenerate(GF_InlineScene *is)
 			is->text_url.url = NULL;
 
 			gf_sg_vrml_mf_append(&as->url, GF_SG_VRML_MFURL, (void **) &sfu);
-			sfu->OD_ID = is->visual_url.OD_ID = odm->OD->objectDescriptorID;
+			sfu->OD_ID = is->text_url.OD_ID = odm->OD->objectDescriptorID;
 			if (odm->OD->objectDescriptorID == GF_ESM_DYNAMIC_OD_ID) {
 				sfu->url = strdup(odm->net_service->url);
 				is->text_url.url = strdup(odm->net_service->url);
@@ -1248,17 +1421,39 @@ void gf_is_regenerate(GF_InlineScene *is)
 		if (!is->dyn_ck) is->dyn_ck = first_odm->codec->ck;
 	}
 
+
+	/*3GPP DIMS streams controlled */
+	n1 = gf_sg_get_root_node(is->graph);
+	dims = (M_Inline *) is_create_node(is->graph, TAG_MPEG4_Inline, "DYN_SCENE");
+	gf_node_list_add_child( &((GF_ParentNode *)n1)->children, (GF_Node*)dims);
+	gf_node_register((GF_Node *)dims, n1);
+
+	i=0;
+	while ((odm = (GF_ObjectManager*)gf_list_enum(is->ODlist, &i))) {
+		if (!odm->subscene || !odm->subscene->scene_codec) continue;
+
+		gf_sg_vrml_mf_append(&dims->url, GF_SG_VRML_MFURL, (void **) &sfu);
+		sfu->OD_ID = odm->OD->objectDescriptorID;
+		if (odm->OD->objectDescriptorID == GF_ESM_DYNAMIC_OD_ID) {
+			sfu->url = strdup(odm->net_service->url);
+		}
+		if (!is->dyn_ck) is->dyn_ck = odm->subscene->scene_codec->ck;
+		break;
+	}
+
+	gf_sc_lock(is->root_od->term->compositor, 0);
+	
 	/*disconnect to force resize*/
 	if (is->root_od->term->root_scene == is) {
-		if (is->graph_attached) gf_sr_set_scene(is->root_od->term->renderer, NULL);
-		gf_sr_set_scene(is->root_od->term->renderer, is->graph);
+		if (is->graph_attached) gf_sc_set_scene(is->root_od->term->compositor, NULL);
+		gf_sc_set_scene(is->root_od->term->compositor, is->graph);
 		is->graph_attached = 1;
 		evt.type = GF_EVENT_STREAMLIST;
 		GF_USER_SENDEVENT(is->root_od->term->user,&evt);
 		IS_UpdateVideoPos(is);
 	} else {
 		is->graph_attached = 1;
-		gf_term_invalidate_renderer(is->root_od->term);
+		gf_term_invalidate_compositor(is->root_od->term);
 	}
 }
 
@@ -1279,8 +1474,9 @@ static Bool check_odm_deactivate(SFURL *url, GF_ObjectManager *odm, GF_Node *n)
 	return 1;
 }
 
-void gf_is_select_object(GF_InlineScene *is, GF_ObjectManager *odm)
+void gf_inline_select_object(GF_InlineScene *is, GF_ObjectManager *odm)
 {
+	char *url;
 	if (!is->is_dynamic_scene || !is->graph_attached || !odm) return;
 	
 	if (!odm->codec) return;
@@ -1299,12 +1495,16 @@ void gf_is_select_object(GF_InlineScene *is, GF_ObjectManager *odm)
 		is->audio_url.OD_ID = odm->OD->objectDescriptorID;
 		if (!ac->url.count) gf_sg_vrml_mf_alloc(&ac->url, GF_SG_VRML_MFURL, 1);
 		ac->url.vals[0].OD_ID = odm->OD->objectDescriptorID;
-		if (ac->url.vals[0].url) free(ac->url.vals[0].url);
-		if (odm->OD->URLString) {
-			is->audio_url.url = strdup(odm->OD->URLString);
-			ac->url.vals[0].url = strdup(odm->OD->URLString);
+		if (ac->url.vals[0].url) {
+			free(ac->url.vals[0].url);
+			ac->url.vals[0].url = NULL; 
 		}
-		ac->startTime = gf_is_get_time(is);
+		url = odm->mo->URLs.count ? odm->mo->URLs.vals[0].url : NULL;
+		if (url) {
+			is->audio_url.url = strdup(url);
+			ac->url.vals[0].url = strdup(url);
+		}
+		ac->startTime = gf_inline_get_time(is);
 		gf_node_changed((GF_Node *)ac, NULL);
 		return;
 	}
@@ -1318,13 +1518,14 @@ void gf_is_select_object(GF_InlineScene *is, GF_ObjectManager *odm)
 		if (!mt->url.count) gf_sg_vrml_mf_alloc(&mt->url, GF_SG_VRML_MFURL, 1);
 		mt->url.vals[0].OD_ID = odm->OD->objectDescriptorID;
 		if (mt->url.vals[0].url) free(mt->url.vals[0].url);
-		if (odm->OD->URLString) {
-			is->visual_url.url = strdup(odm->OD->URLString);
-			mt->url.vals[0].url = strdup(odm->OD->URLString);
+		url = odm->mo->URLs.count ? odm->mo->URLs.vals[0].url : NULL;
+		if (url) {
+			is->visual_url.url = strdup(url);
+			mt->url.vals[0].url = strdup(url);
 		}
-		mt->startTime = gf_is_get_time(is);
+		mt->startTime = gf_inline_get_time(is);
 		gf_node_changed((GF_Node *)mt, NULL);
-		if (odm->mo) gf_is_force_scene_size_video(is, odm->mo);
+		if (odm->mo) gf_inline_force_scene_size_video(is, odm->mo);
 		return;
 	}
 
@@ -1338,11 +1539,12 @@ void gf_is_select_object(GF_InlineScene *is, GF_ObjectManager *odm)
 		if (!as->url.count) gf_sg_vrml_mf_alloc(&as->url, GF_SG_VRML_MFURL, 1);
 		as->url.vals[0].OD_ID = odm->OD->objectDescriptorID;
 		if (as->url.vals[0].url) free(as->url.vals[0].url);
-		if (odm->OD->URLString) {
-			is->text_url.url = strdup(odm->OD->URLString);
-			as->url.vals[0].url = strdup(odm->OD->URLString);
+		url = odm->mo->URLs.count ? odm->mo->URLs.vals[0].url : NULL;
+		if (url) {
+			is->text_url.url = strdup(url);
+			as->url.vals[0].url = strdup(url);
 		}
-		as->startTime = gf_is_get_time(is);
+		as->startTime = gf_inline_get_time(is);
 		gf_node_changed((GF_Node *)as, NULL);
 		return;
 	}
@@ -1350,18 +1552,21 @@ void gf_is_select_object(GF_InlineScene *is, GF_ObjectManager *odm)
 
 
 GF_EXPORT
-void gf_is_force_scene_size(GF_InlineScene *is, u32 width, u32 height)
+void gf_inline_force_scene_size(GF_InlineScene *is, u32 width, u32 height)
 {
 	/*for now only allowed when no scene info*/
 	if (!is->is_dynamic_scene) return;
 	gf_sg_set_scene_size_info(is->graph, width, height, gf_sg_use_pixel_metrics(is->graph));
-	if (is->root_od->term->root_scene != is) return;
-	gf_sr_set_scene(is->root_od->term->renderer, is->graph);
+	
+	if (is->root_od->term->root_scene == is) 
+		gf_sc_set_scene(is->root_od->term->compositor, is->graph);
+
+	gf_is_resize_event(is);
 
 	IS_UpdateVideoPos(is);
 }
 
-void gf_is_restart_dynamic(GF_InlineScene *is, u64 from_time)
+void gf_inline_restart_dynamic(GF_InlineScene *is, u64 from_time)
 {
 	u32 i;
 	GF_List *to_restart;
@@ -1394,15 +1599,15 @@ void gf_is_restart_dynamic(GF_InlineScene *is, u64 from_time)
 		M_MovieTexture *mt = (M_MovieTexture *) gf_sg_find_node_by_name(is->graph, "DYN_VIDEO");
 		M_AnimationStream *as = (M_AnimationStream *) gf_sg_find_node_by_name(is->graph, "DYN_TEXT");
 		if (ac) {
-			ac->startTime = gf_is_get_time(is);
+			ac->startTime = gf_inline_get_time(is);
 			gf_node_changed((GF_Node *)ac, NULL);
 		}
 		if (mt) {
-			mt->startTime = gf_is_get_time(is);
+			mt->startTime = gf_inline_get_time(is);
 			gf_node_changed((GF_Node *)mt, NULL);
 		}
 		if (as) {
-			as->startTime = gf_is_get_time(is);
+			as->startTime = gf_inline_get_time(is);
 			gf_node_changed((GF_Node *)as, NULL);
 		}
 	}
@@ -1410,7 +1615,7 @@ void gf_is_restart_dynamic(GF_InlineScene *is, u64 from_time)
 
 
 GF_EXPORT
-Bool gf_is_process_anchor(GF_Node *caller, GF_Event *evt)
+Bool gf_inline_process_anchor(GF_Node *caller, GF_Event *evt)
 {
 	u32 i;
 	GF_Terminal *term;
@@ -1430,30 +1635,85 @@ Bool gf_is_process_anchor(GF_Node *caller, GF_Event *evt)
 	/*FIXME this is too restrictive, we assume the navigate URL is really a presentation one...*/
 	i=0;
 	while ((inl = (M_Inline*)gf_list_enum(is->inline_nodes, &i))) {
-		gf_sg_vrml_mf_reset(&inl->url, GF_SG_VRML_MFURL);
-		gf_sg_vrml_mf_alloc(&inl->url, GF_SG_VRML_MFURL, 1);
-		inl->url.vals[0].url = strdup(evt->navigate.to_url ? evt->navigate.to_url : "");
-		/*signal URL change but don't destroy inline scene now since we got this event from inside the scene, 
-		this could crash renderers*/
-		is->needs_restart = 2;
+		switch (gf_node_get_tag((GF_Node *)inl)) {
+		case TAG_MPEG4_Inline:
+		case TAG_X3D_Inline:
+			gf_sg_vrml_mf_reset(&inl->url, GF_SG_VRML_MFURL);
+			gf_sg_vrml_mf_alloc(&inl->url, GF_SG_VRML_MFURL, 1);
+			inl->url.vals[0].url = strdup(evt->navigate.to_url ? evt->navigate.to_url : "");
+			/*signal URL change but don't destroy inline scene now since we got this event from inside the scene, 
+			this could crash compositors*/
+			is->needs_restart = 2;
+			break;
+		}
 	}
 	return 1;
 }
 
 GF_EXPORT
-GF_Renderer *gf_sr_get_renderer(GF_Node *node)
+GF_Compositor *gf_sc_get_compositor(GF_Node *node)
 {
 	GF_InlineScene *is;
 	GF_SceneGraph *sg = gf_node_get_graph(node);
 	if (!sg) return NULL;
 	is = (GF_InlineScene *)gf_sg_get_private(sg);
 	if (!is) return NULL;
-	return is->root_od->term->renderer;
+	return is->root_od->term->compositor;
 }
 
+const char *gf_inline_get_fragment_uri(GF_Node *node)
+{
+	GF_SceneGraph *sg = gf_node_get_graph(node);
+	GF_InlineScene *is = sg ? (GF_InlineScene *) gf_sg_get_private(sg) : NULL;
+	if (!is) return NULL;
+	return is->fragment_uri;
+}
+void gf_inline_set_fragment_uri(GF_Node *node, const char *uri)
+{
+	GF_SceneGraph *sg = gf_node_get_graph(node);
+	GF_InlineScene *is = sg ? (GF_InlineScene *) gf_sg_get_private(sg) : NULL;
+	if (!is) return;
+	if (is->fragment_uri) {
+		free(is->fragment_uri);
+		is->fragment_uri = NULL;
+	}
+	if (uri) is->fragment_uri = strdup(uri);
+}
+
+GF_Node *gf_inline_get_subscene_root(GF_Node *node)
+{
+	GF_InlineScene *is;
+	if (!node) return NULL;
+	switch (gf_node_get_tag(node)) {
+	case TAG_MPEG4_Inline: case TAG_X3D_Inline: 
+		break;
+	default:
+		return NULL;
+	}
+	is = (GF_InlineScene *)gf_node_get_private(node);
+	if (!is) return NULL;
+	return gf_sg_get_root_node(is->graph);
+}
+
+GF_Node *gf_inline_get_parent_node(GF_Node *node, u32 idx)
+{
+	GF_InlineScene *is;
+	if (!node) return NULL;
+	switch (gf_node_get_tag(node)) {
+	case TAG_MPEG4_Inline: case TAG_X3D_Inline: 
+		is = (GF_InlineScene *)gf_node_get_private(node);
+		break;
+	default:
+		is = (GF_InlineScene *)gf_sg_get_private(gf_node_get_graph(node));
+		break;
+	}
+	if (!is) return NULL;
+	return (GF_Node *) gf_list_get(is->inline_nodes, idx);
+}
 
 void InitInline(GF_InlineScene *is, GF_Node *node)
 {
-	gf_node_set_callback_function(node, gf_is_render);
+	gf_node_set_callback_function(node, gf_inline_traverse);
 }
 
+
