@@ -26,8 +26,7 @@
 #include <gpac/utf.h>
 #include <gpac/nodes_x3d.h>
 #include <gpac/constants.h>
-/*for screen to BIFS coordinate mapping (MouseSensor) */
-#include <gpac/renderer.h>
+#include <gpac/compositor.h>
 
 #include "input_sensor.h"
 
@@ -72,13 +71,7 @@ static void add_field(ISPriv *priv, u32 fieldType, const char *fieldName)
 	gf_list_add(priv->ddf, field);
 }
 
-static GF_Err IS_AttachStream(GF_BaseDecoder *plug, 
-									 u16 ES_ID, 
-									 char *decSpecInfo, 
-									 u32 decSpecInfoSize, 
-									 u16 DependsOnES_ID,
-									 u32 objectTypeIndication, 
-									 Bool Upstream)
+static GF_Err IS_AttachStream(GF_BaseDecoder *plug, GF_ESD *esd)
 {
 	GF_BitStream *bs;
 	u32 len, size, i;
@@ -86,14 +79,14 @@ static GF_Err IS_AttachStream(GF_BaseDecoder *plug,
 	u16 termSeq[20];
 
 	ISPriv *is = (ISPriv *)plug->privateStack;
-	if (Upstream) return GF_NOT_SUPPORTED;
-	if (!decSpecInfo) return GF_NON_COMPLIANT_BITSTREAM;
+	if (esd->decoderConfig->upstream) return GF_NOT_SUPPORTED;
+	if (!esd->decoderConfig->decoderSpecificInfo || !esd->decoderConfig->decoderSpecificInfo->dataLength) return GF_NON_COMPLIANT_BITSTREAM;
 
 	/*no more than one UI stream per object*/
 	if (is->ES_ID) return GF_NOT_SUPPORTED;
-	is->ES_ID = ES_ID;
+	is->ES_ID = esd->ESID;
 	/*parse config*/
-	bs = gf_bs_new(decSpecInfo, decSpecInfoSize, GF_BITSTREAM_READ);
+	bs = gf_bs_new(esd->decoderConfig->decoderSpecificInfo->data, esd->decoderConfig->decoderSpecificInfo->dataLength, GF_BITSTREAM_READ);
 	len = gf_bs_read_int(bs, 8);
 	for (i=0; i<len; i++) {
 		devName[i] = gf_bs_read_int(bs, 8);
@@ -121,9 +114,9 @@ static GF_Err IS_AttachStream(GF_BaseDecoder *plug,
 		is->delChar = '\b';
 
 		/*get escape chars if any specified*/
-		if (size<decSpecInfoSize) {
-			const char *src = decSpecInfo + size;
-			gf_utf8_mbstowcs(termSeq, decSpecInfoSize - size, &src);
+		if (size<esd->decoderConfig->decoderSpecificInfo->dataLength) {
+			const char *src = esd->decoderConfig->decoderSpecificInfo->data + size;
+			gf_utf8_mbstowcs(termSeq, esd->decoderConfig->decoderSpecificInfo->dataLength - size, &src);
 			is->termChar = termSeq[0];
 			is->delChar = termSeq[1];
 		}
@@ -327,6 +320,8 @@ static GF_Err IS_ProcessData(GF_SceneDecoder *plug, char *inBuffer, u32 inBuffer
 		}
 	}
 
+	gf_term_lock_compositor(priv->scene->root_od->term, 1);
+
 	/*apply it*/
 	i=0;
 	while ((st = (ISStack*)gf_list_enum(priv->is_nodes, &i))) {
@@ -335,7 +330,7 @@ static GF_Err IS_ProcessData(GF_SceneDecoder *plug, char *inBuffer, u32 inBuffer
 		if (!st->is->enabled) continue;
 
 		count = gf_list_count(st->is->buffer.commandList);
-		scene_time = gf_is_get_time(priv->scene);
+		scene_time = gf_inline_get_time(priv->scene);
 		for (j=0; j<count; j++) {
 			GF_Command *com = (GF_Command *)gf_list_get(st->is->buffer.commandList, j);
 			GF_FieldInfo *field = (GF_FieldInfo *)gf_list_get(priv->ddf, j);
@@ -346,6 +341,7 @@ static GF_Err IS_ProcessData(GF_SceneDecoder *plug, char *inBuffer, u32 inBuffer
 			}
 		}
 	}
+	gf_term_lock_compositor(priv->scene->root_od->term, 0);
 	return e;
 }
 
@@ -397,7 +393,7 @@ GF_BaseDecoder *NewISCodec(u32 PL)
 	GF_REGISTER_MODULE_INTERFACE(tmp, GF_SCENE_DECODER_INTERFACE, "GPAC InputSensor Decoder", "gpac distribution")
 
 #if GPAC_HTK_DEMO
-	priv->th = gf_th_new();
+	priv->th = gf_th_new("HTKDecoder");
 #endif
 	return (GF_BaseDecoder *) tmp;
 }
@@ -405,11 +401,14 @@ GF_BaseDecoder *NewISCodec(u32 PL)
 /*
 				input sensor node handling
 */
-static void IS_Unregister(ISStack *st)
+static void IS_Unregister(GF_Node *node, ISStack *st)
 {
 	u32 i;
 	GF_ObjectManager *odm;
 	ISPriv *is_dec;
+
+	gf_mo_unregister(node, st->mo);
+	
 	odm = st->mo->odm;
 	if (!odm) return;
 
@@ -450,23 +449,23 @@ static void IS_Register(GF_Node *n)
 	/*start stream*/
 	gf_mo_play(st->mo, 0, -1, 0);
 
-	gf_term_rem_render_node(odm->term, n);
+	gf_term_unqueue_node_traverse(odm->term, n);
 }
 
-static void RenderInputSensor(GF_Node *node, void *rs, Bool is_destroy)
+static void TraverseInputSensor(GF_Node *node, void *rs, Bool is_destroy)
 {
 	ISStack *st = (ISStack*)gf_node_get_private(node);
 	M_InputSensor *is = (M_InputSensor *)node;
 
 	if (is_destroy) {
 		GF_InlineScene *is;
-		if (st->registered) IS_Unregister(st);
+		if (st->registered) IS_Unregister(node, st);
 		is = (GF_InlineScene*)gf_sg_get_private(gf_node_get_graph(node));
-		gf_term_rem_render_node(is->root_od->term, node);
+		gf_term_unqueue_node_traverse(is->root_od->term, node);
 		free(st);
 	} else {
 		/*get decoder object */
-		if (!st->mo) st->mo = gf_mo_find(node, &is->url, 0);
+		if (!st->mo) st->mo = gf_mo_register(node, &is->url, 0);
 		/*register with decoder*/
 		if (st->mo && !st->registered) IS_Register(node);
 	}
@@ -479,8 +478,8 @@ void InitInputSensor(GF_InlineScene *is, GF_Node *node)
 	GF_SAFEALLOC(stack, ISStack);
 	stack->is = (M_InputSensor *) node;
 	gf_node_set_private(node, stack);
-	gf_node_set_callback_function(node, RenderInputSensor);
-	gf_term_add_render_node(is->root_od->term, node);
+	gf_node_set_callback_function(node, TraverseInputSensor);
+	gf_term_queue_node_traverse(is->root_od->term, node);
 }
 
 /*check only URL changes*/
@@ -493,10 +492,10 @@ void InputSensorModified(GF_Node *node)
 	GF_MediaObject *mo;
 	ISStack *st = (ISStack *)gf_node_get_private(node);
 
-	mo = gf_mo_find(node, &st->is->url, 0);
+	mo = gf_mo_register(node, &st->is->url, 0);
 	if ((mo!=st->mo) || !st->registered){
 		if (mo!=st->mo) {
-			if (st->mo) IS_Unregister(st);
+			if (st->mo) IS_Unregister(node, st);
 			st->mo = mo;
 		}
 		if (st->is->enabled) 
@@ -504,8 +503,7 @@ void InputSensorModified(GF_Node *node)
 		else
 			return;
 	} else if (!st->is->enabled) {
-		IS_Unregister(st);
-		st->mo = NULL;
+		IS_Unregister(node, st);
 		return;
 	}
 
@@ -562,7 +560,7 @@ void gf_term_mouse_input(GF_Terminal *term, GF_EventMouse *event)
 	}
 
 	/*get BIFS coordinates*/
-	gf_sr_map_point(term->renderer, X, Y, &bX, &bY);
+	gf_sc_map_point(term->compositor, X, Y, &bX, &bY);
 
 	bs = gf_bs_new(NULL, 0, GF_BITSTREAM_WRITE);
 
