@@ -64,14 +64,18 @@ void MC_Restart(GF_ObjectManager *odm)
 
 		/*this is inline restart - only possible through media control*/
 		if (odm->subscene && odm->subscene->root_od==ctrl->stream->odm) {
-			gf_is_restart(odm->subscene);
+			gf_inline_restart(odm->subscene);
 			return;
 		}
 	}
 
 	/*if clock is main scene clock do nothing*/
 	scene_ck = gf_odm_get_media_clock(odm->parentscene->root_od);
-	if (gf_odm_shares_clock(odm, scene_ck)) return;
+	if (gf_odm_shares_clock(odm, scene_ck)) {
+		if (odm->parentscene->is_dynamic_scene)
+			gf_inline_restart_dynamic(odm->parentscene, 0);
+		return;
+	}
 
 	/*otherwise locate all objects sharing the clock*/
 	ck = gf_odm_get_media_clock(odm);
@@ -134,7 +138,7 @@ void MC_Resume(GF_ObjectManager *odm)
 
 	i=0;
 	while ((ctrl_od = (GF_ObjectManager*)gf_list_enum(in_scene->ODlist, &i))) {
-		if (!gf_odm_shares_clock(ctrl_od, ck)) continue;
+		if (!odm->subscene && !gf_odm_shares_clock(ctrl_od, ck)) continue;
 		gf_odm_resume(ctrl_od);
 	}
 }
@@ -165,7 +169,7 @@ void MC_Pause(GF_ObjectManager *odm)
 
 	i=0;
 	while ((ctrl_od = (GF_ObjectManager*)gf_list_enum(in_scene->ODlist, &i))) {
-		if (!gf_odm_shares_clock(ctrl_od, ck)) continue;
+		if (!odm->subscene && !gf_odm_shares_clock(ctrl_od, ck)) continue;
 		gf_odm_pause(ctrl_od);
 	}
 }
@@ -259,6 +263,8 @@ void RenderMediaControl(GF_Node *node, void *rs, Bool is_destroy)
 			odm = stack->stream->odm;
 			ODM_RemoveMediaControl(odm, stack);
 		}
+		/*also removes the association ck<->MC if the object has been destroyed before the node*/
+		if (stack->ck) stack->ck->mc = NULL;
 
 		gf_list_del(stack->seg);
 		gf_sg_vrml_mf_reset(&stack->url, GF_SG_VRML_MFURL);
@@ -278,7 +284,7 @@ void RenderMediaControl(GF_Node *node, void *rs, Bool is_destroy)
 			gf_sg_vrml_mf_reset(&stack->url, GF_SG_VRML_MFURL);
 
 			prev = stack->stream;
-			stack->stream = gf_is_get_media_object(stack->parent, &stack->control->url, GF_MEDIA_OBJECT_UNDEF, 0);
+			stack->stream = gf_inline_get_media_object(stack->parent, &stack->control->url, GF_MEDIA_OBJECT_UNDEF, 0);
 			if (stack->stream) {
 				if (!stack->stream->odm) return;
 				/*MediaControl on inline: if dynamic scene, make sure it is connected before attaching...*/
@@ -297,7 +303,7 @@ void RenderMediaControl(GF_Node *node, void *rs, Bool is_destroy)
 
 				stack->current_seg = 0;
 				shall_restart = need_restart = 1;
-
+				stack->ck = gf_odm_get_media_clock(stack->stream->odm);
 			}
 			/*control has been removed and we were paused, resume*/
 			else if (stack->paused) {
@@ -311,11 +317,12 @@ void RenderMediaControl(GF_Node *node, void *rs, Bool is_destroy)
 			}
 		}
 	} else {
-		stack->stream = gf_is_get_media_object(stack->parent, &stack->control->url, GF_MEDIA_OBJECT_UNDEF, 0);
+		stack->stream = gf_inline_get_media_object(stack->parent, &stack->control->url, GF_MEDIA_OBJECT_UNDEF, 0);
 		if (!stack->stream || !stack->stream->odm) {
-			if (stack->control->url.count) gf_term_invalidate_renderer(stack->parent->root_od->term);
+			if (stack->control->url.count) gf_term_invalidate_compositor(stack->parent->root_od->term);
 			return;
 		}
+		stack->ck = gf_odm_get_media_clock(stack->stream->odm);
 		gf_sg_vrml_field_copy(&stack->url, &stack->control->url, GF_SG_VRML_MFURL);
 		ODM_SetMediaControl((GF_ObjectManager *) stack->stream->odm, stack);
 
@@ -371,6 +378,10 @@ void RenderMediaControl(GF_Node *node, void *rs, Bool is_destroy)
 			if (!shall_restart) MC_SetSpeed(odm, stack->control->mediaSpeed);
 			need_restart += shall_restart;
 		}
+		/*init state was paused*/
+		else if (!stack->media_speed) {
+			need_restart ++;
+		}
 		stack->media_speed = stack->control->mediaSpeed;
 	}
 	/*check start/stop changes*/
@@ -412,12 +423,21 @@ void MC_Modified(GF_Node *node)
 {
 	MediaControlStack *stack =(MediaControlStack *) gf_node_get_private(node);
 	if (!stack) return;
-	if (!stack->changed) {
+	if (stack->changed!=2) {
+		/*check URL*/
 		if (MC_URLChanged(&stack->url, &stack->control->url)) stack->changed = 2;
-		else if (stack->media_start != stack->control->mediaStartTime) stack->changed = 2;
-		else stack->changed = 1;
+		/*check speed (play/pause)*/
+		else if (stack->media_speed != stack->control->mediaSpeed) 
+			stack->changed = 1;
+		/*check mediaStartTime (seek)*/
+		else if (stack->media_start != stack->control->mediaStartTime) {
+			/*do not reevaluate if mediaStartTime is reset to -1 (current time)*/
+			if (stack->control->mediaStartTime!=-1.0)
+				stack->changed = 2;
+		}
+//		else stack->changed = 1;
 	}
 
 	/*invalidate scene, we recompute MC state in render*/
-	gf_term_invalidate_renderer(stack->parent->root_od->term);
+	gf_term_invalidate_compositor(stack->parent->root_od->term);
 }
