@@ -46,6 +46,9 @@ typedef struct _media_control
 	GF_InlineScene *parent;
 	/*stream owner*/
 	GF_MediaObject *stream;
+	/*stream owner's clock*/
+	GF_Clock *ck;
+
 	Bool changed;
 	Bool is_init;
 	Bool paused;
@@ -75,6 +78,8 @@ Bool ODM_SwitchMediaControl(GF_ObjectManager *odm, struct _media_control *ctrl);
 
 /*restart object and takes care of media control/clock dependencies*/
 void MC_Restart(GF_ObjectManager *odm);
+void MC_Pause(GF_ObjectManager *odm);
+void MC_Resume(GF_ObjectManager *odm);
 
 Bool MC_URLChanged(MFURL *old_url, MFURL *new_url);
 
