@@ -26,8 +26,7 @@
 
 #include <gpac/internal/terminal_dev.h>
 #include "media_memory.h"
-/*in case the media manager is also responsible for visual rendering*/
-#include <gpac/renderer.h>
+#include <gpac/internal/compositor_dev.h>
 
 
 u32 MM_Loop(void *par);
@@ -54,7 +53,7 @@ typedef struct
 
 GF_Err gf_term_init_scheduler(GF_Terminal *term, u32 threading_mode)
 {
-	term->mm_mx = gf_mx_new();
+	term->mm_mx = gf_mx_new("MediaManager");
 	term->codecs = gf_list_new();
 
 	term->frame_duration = 33;
@@ -69,7 +68,7 @@ GF_Err gf_term_init_scheduler(GF_Terminal *term, u32 threading_mode)
 
 	if (term->user->init_flags & GF_TERM_NO_VISUAL_THREAD) return GF_OK;
 
-	term->mm_thread = gf_th_new();
+	term->mm_thread = gf_th_new("MediaManager");
 	term->flags |= GF_TERM_RUNNING;
 	term->priority = GF_THREAD_PRIORITY_NORMAL;
 	gf_th_run(term->mm_thread, MM_Loop, term);
@@ -81,7 +80,7 @@ void gf_term_stop_scheduler(GF_Terminal *term)
 	if (term->mm_thread) {
 		term->flags &= ~GF_TERM_RUNNING;
 		while (!(term->flags & GF_TERM_DEAD) ) 
-			gf_sleep(0);
+			gf_sleep(2);
 
 		assert(! gf_list_count(term->codecs));
 		gf_th_del(term->mm_thread);
@@ -104,14 +103,18 @@ static CodecEntry *mm_get_codec(GF_List *list, GF_Codec *codec)
 void gf_term_add_codec(GF_Terminal *term, GF_Codec *codec)
 {
 	u32 i, count;
+	Bool locked;
 	Bool threaded;
 	CodecEntry *cd;
 	CodecEntry *ptr, *next;
 	GF_CodecCapability cap;
 	assert(codec);
 
-	/*we need REAL exclusive access when adding a dec*/
-	gf_mx_p(term->mm_mx);
+	GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("[Terminal] Registering codec %s\n", codec->decio ? codec->decio->module_name : "Unknown"));
+
+	/*caution: the mutex can be grabbed by a decoder waiting for a mutex owned by the calling thread
+	this happens when several scene codecs are running concurently and triggering play/pause on media*/
+	locked = gf_mx_try_lock(term->mm_mx);
 
 	cd = mm_get_codec(term->codecs, codec);
 	if (cd) goto exit;
@@ -132,8 +135,8 @@ void gf_term_add_codec(GF_Terminal *term, GF_Codec *codec)
 	}
 	
 	if (threaded) {
-		cd->thread = gf_th_new();
-		cd->mx = gf_mx_new();
+		cd->thread = gf_th_new(cd->dec->decio->module_name);
+		cd->mx = gf_mx_new(cd->dec->decio->module_name);
 		cd->flags |= GF_MM_CE_THREADED;
 		gf_list_add(term->codecs, cd);
 		goto exit;
@@ -188,17 +191,20 @@ void gf_term_add_codec(GF_Terminal *term, GF_Codec *codec)
 	gf_list_add(term->codecs, cd);
 
 exit:
-	gf_mx_v(term->mm_mx);
+	if (locked) gf_mx_v(term->mm_mx);
 	return;
 }
 
 void gf_term_remove_codec(GF_Terminal *term, GF_Codec *codec)
 {
 	u32 i;
+	Bool locked;
 	CodecEntry *ce;
 
-	/*we need REAL exclusive access when removing a dec*/
-	gf_mx_p(term->mm_mx);
+	GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("[Terminal] Unregistering codec %s\n", codec->decio ? codec->decio->module_name : "Unknown"));
+
+	/*cf note above*/
+	locked = gf_mx_try_lock(term->mm_mx);
 
 	i=0;
 	while ((ce = (CodecEntry*)gf_list_enum(term->codecs, &i))) {
@@ -217,38 +223,57 @@ void gf_term_remove_codec(GF_Terminal *term, GF_Codec *codec)
 		gf_list_rem(term->codecs, i-1);
 		break;
 	}
-	gf_mx_v(term->mm_mx);
+	if (locked) gf_mx_v(term->mm_mx);
 	return;
 }
 
-static u32 MM_SimulationStep(GF_Terminal *term, u32 *last_dec)
+Bool gf_term_find_codec(GF_Terminal *term, GF_Codec *codec)
+{
+	CodecEntry *ce;
+	u32 i=0;
+	while ((ce = (CodecEntry*)gf_list_enum(term->codecs, &i))) {
+		if (ce->dec == codec) return 1;
+	}
+	return 0;
+}
+
+static u32 MM_SimulationStep(GF_Terminal *term)
 {
 	CodecEntry *ce;
 	GF_Err e;
-	u32 count, current_dec, remain;
+	u32 count, remain;
 	u32 time_taken, time_slice, time_left;
-
-	current_dec = last_dec ? *last_dec : 0;
 	
+#ifndef GF_DISABLE_LOG
+	term->compositor->networks_time = gf_sys_clock();
+#endif
+
 	gf_term_handle_services(term);
 
+#ifndef GF_DISABLE_LOG
+	term->compositor->networks_time = gf_sys_clock() - term->compositor->networks_time;
+#endif
+
+#ifndef GF_DISABLE_LOG
+	term->compositor->decoders_time = gf_sys_clock();
+#endif
 	gf_mx_p(term->mm_mx);
 
 	count = gf_list_count(term->codecs);
 	time_left = term->frame_duration;
 
-	if (current_dec >= count) current_dec = 0;
+	if (term->last_codec >= count) term->last_codec = 0;
 	remain = count;
 
 	/*this is ultra basic a nice scheduling system would be much better*/
 	while (remain) {
-		ce = (CodecEntry*)gf_list_get(term->codecs, current_dec);
+		ce = (CodecEntry*)gf_list_get(term->codecs, term->last_codec);
 		if (!ce) break;
 
 		if (!(ce->flags & GF_MM_CE_RUNNING) || (ce->flags & GF_MM_CE_THREADED) ) {
 			remain--;
 			if (!remain) break;
-			current_dec = (current_dec + 1) % count;
+			term->last_codec = (term->last_codec + 1) % count;
 			continue;
 		}
 		time_slice = ce->dec->Priority * time_left / term->cumulated_priority;
@@ -259,7 +284,7 @@ static u32 MM_SimulationStep(GF_Terminal *term, u32 *last_dec)
 
 		/*avoid signaling errors too often...*/
 #ifndef GPAC_DISABLE_LOG
-		if (e) GF_LOG(GF_LOG_ERROR, GF_LOG_MEDIA, ("[ODM%d] Decoding Error %s\n", ce->dec->odm->OD->objectDescriptorID, gf_error_to_string(e) ));
+		if (e) GF_LOG(GF_LOG_WARNING, GF_LOG_MEDIA, ("[ODM%d] Decoding Error %s\n", ce->dec->odm->OD->objectDescriptorID, gf_error_to_string(e) ));
 #endif
 
 		time_taken = gf_sys_clock() - time_taken;
@@ -269,7 +294,7 @@ static u32 MM_SimulationStep(GF_Terminal *term, u32 *last_dec)
 		remain -= 1;
 		if (!remain) break;
 
-		current_dec = (current_dec + 1) % count;
+		term->last_codec = (term->last_codec + 1) % count;
 
 		if (time_left > time_taken) {
 			time_left -= time_taken;
@@ -278,33 +303,38 @@ static u32 MM_SimulationStep(GF_Terminal *term, u32 *last_dec)
 		}
 	}
 	gf_mx_v(term->mm_mx);
+#ifndef GF_DISABLE_LOG
+	term->compositor->decoders_time = gf_sys_clock() - term->compositor->decoders_time;
+#endif
 
-	if (term->flags & GF_TERM_RENDER_FRAME) {
+	if (term->flags & GF_TERM_DRAW_FRAME) {
 		time_taken = gf_sys_clock();
-		gf_sr_render_frame(term->renderer);
+		gf_sc_draw_frame(term->compositor);
 		time_taken = gf_sys_clock() - time_taken;
-
 		if (time_left>time_taken) 
 			time_left -= time_taken;
 		else
 			time_left = 0;
+	} /*else {
+		GF_LOG(GF_LOG_DEBUG, GF_LOG_RTI, ("(RTI] Terminal Cycle Log\t%d\t%d\t%d\n", term->compositor->networks_time, term->compositor->decoders_time, 0));
+	}*/
+	if (!(term->user->init_flags & GF_TERM_NO_REGULATION)) {
+		gf_sleep(time_left);
 	}
-	if (!(term->user->init_flags & GF_TERM_NO_REGULATION)) gf_sleep(time_left);
 //	GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("[Terminal] Simulation step done in %d / %d ms\n", term->frame_duration-time_left, term->frame_duration));
 	return time_left;
 }
 
 u32 MM_Loop(void *par)
 {
-	u32 current_dec;
 	GF_Terminal *term = (GF_Terminal *) par;
 
 	gf_th_set_priority(term->mm_thread, term->priority);
 	GF_LOG(GF_LOG_DEBUG, GF_LOG_CORE, ("[MediaManager] Entering thread ID %d\n", gf_th_id() ));
+//	GF_LOG(GF_LOG_DEBUG, GF_LOG_RTI, ("(RTI] Terminal Cycle Log\tServices\tDecoders\tCompositor\tSleep\n"));
 
-	current_dec = 0;
 	while (term->flags & GF_TERM_RUNNING) {
-		MM_SimulationStep(term, &current_dec);
+		MM_SimulationStep(term);
 	}
 	term->flags |= GF_TERM_DEAD;
 	return 0;
@@ -347,7 +377,7 @@ u32 RunSingleDec(void *ptr)
 }
 
 /*NOTE: when starting/stoping a decoder we only lock the decoder mutex, NOT the media manager. This
-avoids deadlocking in case a system codec waits for the scene graph and the renderer requests 
+avoids deadlocking in case a system codec waits for the scene graph and the compositor requests 
 a stop/start on a media*/
 void gf_term_start_codec(GF_Codec *codec)
 {
@@ -391,13 +421,24 @@ void gf_term_start_codec(GF_Codec *codec)
 
 void gf_term_stop_codec(GF_Codec *codec)
 {
+	Bool locked = 0;
 	CodecEntry *ce;
 	GF_Terminal *term = codec->odm->term;
 	ce = mm_get_codec(term->codecs, codec);
 	if (!ce) return;
 
 	if (ce->mx) gf_mx_p(ce->mx);
-	else gf_mx_p(term->mm_mx);
+	/*We must make sure:
+		1- media codecs are synchrounously stop otherwise we could destroy the composition memory while 
+	the codec writes to it
+		2- prevent deadlock for other codecs waiting for the scene graph
+	*/
+	else if (codec->CB) {
+		locked = 1;
+		gf_mx_p(term->mm_mx);
+	} else {
+		locked = gf_mx_try_lock(term->mm_mx);
+	}
 	
 	if (codec->decio && codec->odm->mo && (codec->odm->mo->flags & GF_MO_DISPLAY_REMOVE) ) {
 		GF_CodecCapability cap;
@@ -417,7 +458,8 @@ void gf_term_stop_codec(GF_Codec *codec)
 	}
 
 	if (ce->mx) gf_mx_v(ce->mx);
-	else gf_mx_v(term->mm_mx);
+	/*cf note above*/
+	else if (locked) gf_mx_v(term->mm_mx);
 }
 
 void gf_term_set_threading(GF_Terminal *term, u32 mode)
@@ -478,8 +520,8 @@ void gf_term_set_threading(GF_Terminal *term, u32 mode)
 
 		if (thread_it) {
 			ce->flags |= GF_MM_CE_THREADED;
-			ce->thread = gf_th_new();
-			ce->mx = gf_mx_new();
+			ce->thread = gf_th_new(ce->dec->decio->module_name);
+			ce->mx = gf_mx_new(ce->dec->decio->module_name);
 		}
 
 		if (restart_it) {
@@ -515,9 +557,8 @@ void gf_term_set_priority(GF_Terminal *term, s32 Priority)
 GF_EXPORT
 GF_Err gf_term_process_step(GF_Terminal *term)
 {
-	if (!(term->flags & GF_TERM_RENDER_FRAME)) return GF_BAD_PARAM;
-
-	MM_SimulationStep(term, NULL);
+	if (!(term->flags & GF_TERM_DRAW_FRAME)) return GF_BAD_PARAM;
+	MM_SimulationStep(term);
 	return GF_OK;
 }
 
@@ -526,7 +567,7 @@ GF_Err gf_term_process_flush(GF_Terminal *term)
 {
 	u32 i;
 	CodecEntry *ce;
-	if (!(term->flags & GF_TERM_RENDER_FRAME) ) return GF_BAD_PARAM;
+	if (!(term->flags & GF_TERM_DRAW_FRAME) ) return GF_BAD_PARAM;
 
 	/*update till frame mature*/
 	while (1) {
@@ -539,7 +580,7 @@ GF_Err gf_term_process_flush(GF_Terminal *term)
 		}
 		gf_mx_v(term->mm_mx);
 
-		if (!gf_sr_render_frame(term->renderer))
+		if (!gf_sc_draw_frame(term->compositor))
 			break;
 
 		if (! (term->user->init_flags & GF_TERM_NO_REGULATION))
