@@ -90,6 +90,7 @@ GF_CompositionMemory *gf_cm_new(u32 UnitSize, u32 capacity)
 		}
 		cu->dataLength = 0;
 		cu->data = UnitSize ? (char*)malloc(sizeof(char)*UnitSize) : NULL;
+		if (cu->data) memset(cu->data, 0, sizeof(char)*UnitSize);
 		prev = cu;
 		capacity --;
 		i++;
@@ -131,84 +132,92 @@ void gf_cm_rewind_input(GF_CompositionMemory *cb)
 }
 
 /*access to the input buffer - return NULL if no input is available (buffer full)*/
-GF_CMUnit *gf_cm_lock_input(GF_CompositionMemory *cb, u32 TS)
+GF_CMUnit *gf_cm_lock_input(GF_CompositionMemory *cb, u32 TS, Bool codec_reordering)
 {
-#if NO_TEMPORAL_SCALABLE
-	/*there is still something in the input buffer*/
-	if (cb->input->dataLength) {
-		if (cb->input->TS==TS) return cb->input;
-		return NULL;
-	}
-	return cb->input;
-
-#else	
+#if !NO_TEMPORAL_SCALABLE
 	GF_CMUnit *cu;
+	if (codec_reordering) {
+#endif
+		/*there is still something in the input buffer*/
+		if (cb->input->dataLength) {
+			if (cb->input->TS==TS) 
+				return cb->input;
+			return NULL;
+		}
+		cb->input->TS = TS;
+		return cb->input;
+
+#if !NO_TEMPORAL_SCALABLE
+	}
 
 	/*spatial scalable, go backward to fetch same TS*/
 	cu = cb->input;
 	while (1) {
-		if (cu->TS == TS) return cu;
+		if (cu->TS == TS) 
+			return cu;
 		cu = cu->prev;
 		if (cu == cb->input) break;
 	}
 	/*temporal scalable (find empty slot)*/
 	cu = cb->input;
 	while (1) {
-		if (!cu->dataLength) return cu;
+		if (!cu->dataLength) {
+			assert(!cu->TS || (cb->Capacity==1));
+			cu->TS = TS;
+			return cu;
+		}
 		cu = cu->next;
 		if (cu == cb->input) return NULL;
 	}
 	return NULL;
 #endif
-
 }
 
-/*re-orders units in case of temporal scalability. Blocking call as it may change the output unit too*/
-static GF_CMUnit *LocateAndOrderUnit(GF_CompositionMemory *cb, u32 TS)
+#if 0
+static void check_temporal(GF_CompositionMemory *cb)
 {
-	GF_CMUnit *unit;
-#if NO_TEMPORAL_SCALABLE
-	unit = cb->input;
-	cb->input = cb->input->next;
-	return unit;
-
-#else
 	GF_CMUnit *cu;
-
-	/*lock the buffer since we may move pointers*/
-	gf_cm_lock(cb, 1);
-	unit = NULL;
-
-	/*1- locate cu*/
-	cu = cb->input;
+	cu = cb->output;
 	while (1) {
-		if (cu->TS == TS) break;
+		if (cu->next==cb->output) break;
+		assert(!cu->next->dataLength || (cu->TS < cu->next->TS));
+		assert(!cu(>TS || (cu->TS >= cb->LastRenderedTS));
 		cu = cu->next;
-		/*done (should never happen)*/
-		if (cu == cb->input) goto exit;
 	}
-	unit = cu;
+}
+#endif
 
-	/*2- spatial scalability or input is our unit, no reordering*/
-	if (cu->dataLength || (cb->input->TS == TS) ) goto exit;
+/*re-orders units in case of temporal scalability. Blocking call as it may change the output unit too*/
+static GF_CMUnit *gf_cm_reorder_unit(GF_CompositionMemory *cb, GF_CMUnit *unit, Bool codec_reordering)
+{
+	GF_CMUnit *cu;
+#if NO_TEMPORAL_SCALABLE
+	cu = cb->input;
+	cb->input = cb->input->next;
+	return cu;
+#else
+	/*lock the buffer since we may move pointers*/
+	gf_odm_lock(cb->odm, 1);
 
-	/*3- unit is next on time axis (no temporal scalability)*/
-	if (!cb->input->dataLength || cb->input->TS < TS) {
+	/*1- unit is next on time axis (no temporal scalability)*/
+	if (!cb->input->dataLength || cb->input->TS < unit->TS) {
 
-		/*remove unit from the list*/
-		unit->prev->next = unit->next;
-		unit->next->prev = unit->prev;
+		if (unit != cb->input) {
+			/*remove unit from the list*/
+			unit->prev->next = unit->next;
+			unit->next->prev = unit->prev;
 
-		/*insert unit after input*/
-		unit->prev = cb->input;
-		unit->next = cb->input->next;
-		unit->next->prev = unit;
-		unit->prev->next = unit;
+			/*insert unit after input*/
+			unit->prev = cb->input;
+			unit->next = cb->input->next;
+			unit->next->prev = unit;
+			unit->prev->next = unit;
+		}
 		/*set input to our unit*/
 		cb->input = unit;
 		goto exit;
 	}
-	/*4- unit is before our last delivered input - temporal scalability*/
+	/*2- unit is before our last delivered input - temporal scalability*/
 	cu = cb->input;
 	while (1) {
 
@@ -233,7 +242,8 @@ static GF_CMUnit *LocateAndOrderUnit(GF_CompositionMemory *cb, u32 TS)
 		/*go on*/
 		cu = cu->prev;
 		/*done (should never happen)*/
-		if (cu == cb->input) goto exit;
+		if (cu == cb->input) 
+			goto exit;
 	}
 	
 	/*remove unit from the list*/
@@ -245,6 +255,7 @@ static GF_CMUnit *LocateAndOrderUnit(GF_CompositionMemory *cb, u32 TS)
 	unit->prev = cu->prev;
 	unit->next->prev = unit;
 	unit->prev->next = unit;
+	GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("Swapping CU buffer\n"));
 	
 exit:
 
@@ -262,30 +273,41 @@ exit:
 		else if (cb->output->TS > unit->TS) {
 			cb->output = unit;
 		}
+
 	}
 
+	//check_temporal(cb);
 	/*unlock the buffer*/
-	gf_cm_lock(cb, 0);
+	gf_odm_lock(cb->odm, 0);
 	return unit;
-
 #endif
 }
 
-void gf_cm_unlock_input(GF_CompositionMemory *cb, u32 TS, u32 NbBytes)
+void gf_cm_unlock_input(GF_CompositionMemory *cb, GF_CMUnit *cu, u32 cu_size, Bool codec_reordering)
 {
-	GF_CMUnit *cu;
-
 	/*nothing dispatched, ignore*/
-	if (!NbBytes) return;
+	if (!cu_size) {
+		cu->dataLength = 0;
+		cu->TS = 0;
+		return;
+	}
 	gf_odm_lock(cb->odm, 1);
 
-	/*insert/swap this CU*/
-	cu = LocateAndOrderUnit(cb, TS);
+	if (cu->TS < cb->input->TS)
+		cu = cu;
+
+
+	if (codec_reordering) {
+		cb->input = cb->input->next;
+	} else {
+		cu = gf_cm_reorder_unit(cb, cu, codec_reordering);
+		assert(cu);
+	}
 
 	if (cu) {
-		/*if the CU already has data, this is spatial scalability so same num buffers*/
+		/*FIXME - if the CU already has data, this is spatial scalability so same num buffers*/
 		if (!cu->dataLength) cb->UnitCount += 1;
-		cu->dataLength = NbBytes;
+		cu->dataLength = cu_size;
 		cu->RenderedLength = 0;
 
 		/*turn off buffering - this must be done now rather than when fetching first output frame since we're not 
@@ -297,9 +319,9 @@ void gf_cm_unlock_input(GF_CompositionMemory *cb, u32 TS, u32 NbBytes)
 			GF_LOG(GF_LOG_DEBUG, GF_LOG_SYNC, ("[SyncLayer] ODM%d: buffering off at %d (nb buffering on clock: %d)\n", cb->odm->OD->objectDescriptorID, gf_term_get_time(cb->odm->term), cb->odm->codec->ck->Buffering));
 		} 
 
-		/*since a new CU is here notify the renderer*/
+		/*since a new CU is here notify the compositor*/
 		if ((cb->odm->codec->type==GF_STREAM_VISUAL) && cb->odm->mo && cb->odm->mo->num_open) {
-			gf_term_invalidate_renderer(cb->odm->term);
+			gf_term_invalidate_compositor(cb->odm->term);
 		}
 	}
 	gf_odm_lock(cb->odm, 0);
@@ -410,12 +432,15 @@ GF_CMUnit *gf_cm_get_output(GF_CompositionMemory *cb)
 		/*only visual buffers deliver data when paused*/
 		if (cb->odm->codec->type != GF_STREAM_VISUAL) goto exit;
 		break;
+	case CB_BUFFER_DONE:
+		cb->Status = CB_PLAY;
+		break;
 	}
 
 	/*no output*/
 	if (!cb->output->dataLength) {
 		if ((cb->Status != CB_STOP) && cb->HasSeenEOS && (cb->odm && cb->odm->codec)) {
-			GF_LOG(GF_LOG_DEBUG, GF_LOG_COMPOSE, ("[ODM%d] Switching composition memory to stop state - time %d\n", cb->odm->OD->objectDescriptorID, (u32) cb->odm->media_stop_time));
+			GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("[ODM%d] Switching composition memory to stop state - time %d\n", cb->odm->OD->objectDescriptorID, (u32) cb->odm->media_stop_time));
 
 			cb->Status = CB_STOP;
 			cb->odm->current_time = (u32) cb->odm->media_stop_time;
@@ -431,7 +456,7 @@ GF_CMUnit *gf_cm_get_output(GF_CompositionMemory *cb)
 
 		/*handle visual object - EOS if no more data (we keep the last CU for rendering, so check next one)*/
 		if (cb->HasSeenEOS && (!cb->output->next->dataLength || (cb->Capacity==1))) {
-			GF_LOG(GF_LOG_DEBUG, GF_LOG_COMPOSE, ("[ODM%d] Switching composition memory to stop state - time %d\n", cb->odm->OD->objectDescriptorID, (u32) cb->odm->media_stop_time));
+			GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("[ODM%d] Switching composition memory to stop state - time %d\n", cb->odm->OD->objectDescriptorID, (u32) cb->odm->media_stop_time));
 			cb->Status = CB_STOP;
 			cb->odm->current_time = (u32) cb->odm->media_stop_time;
 			/*force update of media time*/
@@ -440,6 +465,8 @@ GF_CMUnit *gf_cm_get_output(GF_CompositionMemory *cb)
 	}
 	out = cb->output;
 
+	assert(out->TS >= cb->LastRenderedTS);
+
 exit:
 	return out;
 }
@@ -454,7 +481,6 @@ void gf_cm_drop_output(GF_CompositionMemory *cb)
 	cb->output->RenderedLength = 0;
 	cb->LastRenderedTS = cb->output->TS;
 
-
 	/*on visual streams, always keep the last AU*/
 	if (cb->output->dataLength && (cb->odm->codec->type == GF_STREAM_VISUAL) ) {
 		if ( !cb->output->next->dataLength || (cb->Capacity == 1) )  {
@@ -464,6 +490,7 @@ void gf_cm_drop_output(GF_CompositionMemory *cb)
 	
 	/*reset the output*/
 	cb->output->dataLength = 0;
+	cb->output->TS = 0;
 	cb->output = cb->output->next;
 	cb->UnitCount -= 1;
 
@@ -521,7 +548,7 @@ void gf_cm_set_eos(GF_CompositionMemory *cb)
 		GF_LOG(GF_LOG_DEBUG, GF_LOG_SYNC, ("[SyncLayer] ODM%d: buffering off at %d (nb buffering on clock: %d)\n", cb->odm->OD->objectDescriptorID, gf_term_get_time(cb->odm->term), cb->odm->codec->ck->Buffering));
 	}
 	cb->HasSeenEOS = 1;
-	gf_term_invalidate_renderer(cb->odm->term);
+	gf_term_invalidate_compositor(cb->odm->term);
 	gf_odm_lock(cb->odm, 0);
 }
 
