@@ -24,14 +24,12 @@
 
 
 #include <gpac/internal/terminal_dev.h>
-#include <gpac/internal/renderer_dev.h>
+#include <gpac/internal/compositor_dev.h>
 #include <gpac/internal/scenegraph_dev.h>
 #include <gpac/nodes_x3d.h>
 #include "media_memory.h"
 #include "media_control.h"
-#include <gpac/nodes_svg_sa.h>
-#include <gpac/nodes_svg_sani.h>
-#include <gpac/nodes_svg_da.h>
+#include <gpac/nodes_svg.h>
 
 
 static GF_MediaObject *get_sync_reference(GF_InlineScene *is, XMLRI *iri, u32 o_type, GF_Node *orig_ref, Bool *post_pone)
@@ -52,23 +50,27 @@ static GF_MediaObject *get_sync_reference(GF_InlineScene *is, XMLRI *iri, u32 o_
 		else ref = gf_sg_find_node_by_name(is->graph, iri->string);
 
 		if (ref) {
+#ifndef GPAC_DISABLE_SVG
 			GF_FieldInfo info;
+#endif
 			/*safety check, break cyclic references*/
 			if (ref==orig_ref) return NULL;
 
 			switch (ref->sgprivate->tag) {
+#ifndef GPAC_DISABLE_SVG
 			case TAG_SVG_audio:
 				o_type = GF_MEDIA_OBJECT_AUDIO; 
-				if (gf_svg_get_attribute_by_tag(ref, TAG_SVG_ATT_xlink_href, 0, 0, &info)==GF_OK) {
+				if (gf_node_get_attribute_by_tag(ref, TAG_XLINK_ATT_href, 0, 0, &info)==GF_OK) {
 					return get_sync_reference(is, info.far_ptr, o_type, orig_ref ? orig_ref : ref, post_pone);
 				}
 				return NULL;
 			case TAG_SVG_video:
 				o_type = GF_MEDIA_OBJECT_VIDEO; 
-				if (gf_svg_get_attribute_by_tag(ref, TAG_SVG_ATT_xlink_href, 0, 0, &info)==GF_OK) {
+				if (gf_node_get_attribute_by_tag(ref, TAG_XLINK_ATT_href, 0, 0, &info)==GF_OK) {
 					return get_sync_reference(is, info.far_ptr, o_type, orig_ref ? orig_ref : ref, post_pone);
 				}
 				return NULL;
+#endif
 			default:
 				return NULL;
 			}
@@ -80,17 +82,19 @@ static GF_MediaObject *get_sync_reference(GF_InlineScene *is, XMLRI *iri, u32 o_
 	mfurl.vals[0].OD_ID = stream_id;
 	mfurl.vals[0].url = iri->string;
 
-	res = gf_is_get_media_object(is, &mfurl, o_type, 0);
+	res = gf_inline_get_media_object(is, &mfurl, o_type, 0);
 	if (!res) *post_pone = 1;
 	return res;
 }
 
 GF_EXPORT
-GF_MediaObject *gf_mo_find(GF_Node *node, MFURL *url, Bool lock_timelines)
+GF_MediaObject *gf_mo_register(GF_Node *node, MFURL *url, Bool lock_timelines)
 {
 	u32 obj_type;
+#ifndef GPAC_DISABLE_SVG
 	Bool post_pone;
 	GF_FieldInfo info;
+#endif
 	GF_InlineScene *is;
 	GF_MediaObject *res, *syncRef;
 	GF_SceneGraph *sg = gf_node_get_graph(node);
@@ -104,7 +108,10 @@ GF_MediaObject *gf_mo_find(GF_Node *node, MFURL *url, Bool lock_timelines)
 	switch (gf_node_get_tag(node)) {
 	/*MPEG4 only*/
 	case TAG_MPEG4_AudioSource: obj_type = GF_MEDIA_OBJECT_AUDIO; break;
-	case TAG_MPEG4_AnimationStream: obj_type = GF_MEDIA_OBJECT_BIFS; break;
+	case TAG_MPEG4_AnimationStream: 
+		obj_type = GF_MEDIA_OBJECT_UPDATES; 
+		break;
+
 	case TAG_MPEG4_InputSensor: obj_type = GF_MEDIA_OBJECT_INTERACT; break;
 
 	/*MPEG4/X3D*/
@@ -116,19 +123,18 @@ GF_MediaObject *gf_mo_find(GF_Node *node, MFURL *url, Bool lock_timelines)
 	case TAG_MPEG4_Inline: case TAG_X3D_Inline: obj_type = GF_MEDIA_OBJECT_SCENE; break;
 	
 	/*SVG*/
-#ifdef GPAC_ENABLE_SVG_SA
-	case TAG_SVG_SA_audio: obj_type = GF_MEDIA_OBJECT_AUDIO; break;
-	case TAG_SVG_SA_image: obj_type = GF_MEDIA_OBJECT_VIDEO; break;
-	case TAG_SVG_SA_video: obj_type = GF_MEDIA_OBJECT_VIDEO; break;
-#endif
-#ifdef GPAC_ENABLE_SVG_SANI
-	case TAG_SVG_SANI_audio: obj_type = GF_MEDIA_OBJECT_AUDIO; break;
-	case TAG_SVG_SANI_image: obj_type = GF_MEDIA_OBJECT_VIDEO; break;
-	case TAG_SVG_SANI_video: obj_type = GF_MEDIA_OBJECT_VIDEO; break;
-#endif
+#ifndef GPAC_DISABLE_SVG
 	case TAG_SVG_audio: 
 		obj_type = GF_MEDIA_OBJECT_AUDIO; 
-		if (gf_svg_get_attribute_by_tag(node, TAG_SVG_ATT_syncReference, 0, 0, &info)==GF_OK) {
+		if (gf_node_get_attribute_by_tag(node, TAG_SVG_ATT_syncReference, 0, 0, &info)==GF_OK) {
+			syncRef = get_sync_reference(is, info.far_ptr, GF_MEDIA_OBJECT_UNDEF, node, &post_pone);
+			/*syncRef is specified but doesn't exist yet, post-pone*/
+			if (post_pone) return NULL;
+		}
+		break;
+	case TAG_SVG_video: 
+		obj_type = GF_MEDIA_OBJECT_VIDEO; 
+		if (gf_node_get_attribute_by_tag(node, TAG_SVG_ATT_syncReference, 0, 0, &info)==GF_OK) {
 			syncRef = get_sync_reference(is, info.far_ptr, GF_MEDIA_OBJECT_UNDEF, node, &post_pone);
 			/*syncRef is specified but doesn't exist yet, post-pone*/
 			if (post_pone) return NULL;
@@ -137,22 +143,29 @@ GF_MediaObject *gf_mo_find(GF_Node *node, MFURL *url, Bool lock_timelines)
 	case TAG_SVG_image: 
 		obj_type = GF_MEDIA_OBJECT_VIDEO; 
 		break;
-	case TAG_SVG_video: 
-		obj_type = GF_MEDIA_OBJECT_VIDEO; 
-		if (gf_svg_get_attribute_by_tag(node, TAG_SVG_ATT_syncReference, 0, 0, &info)==GF_OK) {
-			syncRef = get_sync_reference(is, info.far_ptr, GF_MEDIA_OBJECT_UNDEF, node, &post_pone);
-			/*syncRef is specified but doesn't exist yet, post-pone*/
-			if (post_pone) return NULL;
-		}
-		break;
+#endif
 
 	default: obj_type = GF_MEDIA_OBJECT_UNDEF; break;
 	}
-	res = gf_is_get_media_object_ex(is, url, obj_type, lock_timelines, syncRef);
 
+	/*move to primary resource handler*/
+	while (is->secondary_resource && is->root_od->parentscene)
+		is = is->root_od->parentscene;
+
+	res = gf_inline_get_media_object_ex(is, url, obj_type, lock_timelines, syncRef, 0, node);
+
+	if (res) {
+	}
 	return res;
 }
 
+void gf_mo_unregister(GF_Node *node, GF_MediaObject *mo)
+{
+	if (mo && node) {
+		gf_list_del_item(mo->nodes, node);
+	}
+}
+
 GF_MediaObject *gf_mo_new()
 {
 	GF_MediaObject *mo;
@@ -161,6 +174,7 @@ GF_MediaObject *gf_mo_new()
 	mo->speed = FIX_ONE;
 	mo->URLs.count = 0;
 	mo->URLs.vals = NULL;
+	mo->nodes = gf_list_new();
 	return mo;
 }
 
@@ -200,6 +214,26 @@ Bool gf_mo_get_visual_info(GF_MediaObject *mo, u32 *width, u32 *height, u32 *str
 		*pixel_ar = cap.cap.valueInt;
 		if (! (*pixel_ar & 0x0000FFFF)) *pixel_ar = 0;
 		if (! (*pixel_ar & 0xFFFF0000)) *pixel_ar = 0;
+
+		/**/
+		if (! *pixel_ar) {
+			GF_Channel *ch;
+			GF_NetworkCommand com;
+			com.base.command_type = GF_NET_CHAN_GET_PIXEL_AR;
+			ch = gf_list_get(mo->odm->channels, 0);
+			if (!ch) return 0;
+
+			com.base.on_channel = ch;
+			com.par.hSpacing = com.par.vSpacing = 0;
+			if (gf_term_service_command(ch->service, &com) == GF_OK) {
+				if ((com.par.hSpacing>65535) || (com.par.vSpacing>65535)) {
+					com.par.hSpacing>>=16;
+					com.par.vSpacing>>=16;
+				}
+				if (com.par.hSpacing|| com.par.vSpacing)
+					*pixel_ar = (com.par.hSpacing<<16) | com.par.vSpacing;
+			}
+		}
 	}
 	return 1;
 }
@@ -233,7 +267,7 @@ Bool gf_mo_get_audio_info(GF_MediaObject *mo, u32 *sample_rate, u32 *bits_per_sa
 	return 1;
 }
 
-void MO_UpdateCaps(GF_MediaObject *mo)
+void gf_mo_update_caps(GF_MediaObject *mo)
 {
 	GF_CodecCapability cap;
 
@@ -246,6 +280,7 @@ void MO_UpdateCaps(GF_MediaObject *mo)
 	}
 	else if (mo->type == GF_MEDIA_OBJECT_AUDIO) {
 		u32 sr, nb_ch, bps;
+		sr = nb_ch = bps = 0;
 		gf_mo_get_audio_info(mo, &sr, &bps, &nb_ch, NULL);
 		mo->odm->codec->bytes_per_sec = sr * nb_ch * bps / 8;
 	}
@@ -348,7 +383,7 @@ void gf_mo_release_data(GF_MediaObject *mo, u32 nb_bytes, s32 forceDrop)
 	u32 obj_time;
 	if (!gf_odm_lock_mo(mo)) return;
 
-	if (!mo->nb_fetch) {
+	if (!mo->nb_fetch || !mo->odm->codec) {
 		gf_odm_lock(mo->odm, 0);
 		return;
 	}
@@ -464,6 +499,9 @@ void gf_mo_play(GF_MediaObject *mo, Double clipBegin, Double clipEnd, Bool can_l
 		if (is_restart) {
 			MC_Restart(mo->odm);
 		} else {
+			if (mo->odm->subscene && mo->odm->subscene->is_dynamic_scene)
+				mo->odm->flags |= GF_ODM_REGENERATE_SCENE;
+
 			gf_odm_start(mo->odm);
 		}
 	} else if (mo->odm) {
@@ -493,16 +531,6 @@ void gf_mo_stop(GF_MediaObject *mo)
 		mo->odm->media_start_time = (u64)-1;
 
 		gf_mx_v(mo->odm->term->net_mx);
-		if (mo->odm->codec && mo->odm->codec->CB) {
-			GF_CompositionMemory *cb = mo->odm->codec->CB;
-			GF_CMUnit *out = cb->output->next;
-			while (out != cb->output) {
-				out->dataLength = 0;
-				out->TS = 0;
-				out = out->next;
-			}
-		}
-
 	} else {
 		if (!mo->num_to_restart) {
 			mo->num_restart = mo->num_to_restart = mo->num_open + 1;
@@ -550,6 +578,21 @@ Bool gf_mo_url_changed(GF_MediaObject *mo, MFURL *url)
 	return ret;
 }
 
+GF_EXPORT
+void gf_mo_pause(GF_MediaObject *mo)
+{
+	if (!mo || !mo->num_open || !mo->odm) return;
+
+	MC_Pause(mo->odm);
+}
+
+GF_EXPORT
+void gf_mo_resume(GF_MediaObject *mo)
+{
+	if (!mo || !mo->num_open || !mo->odm) return;
+
+	MC_Resume(mo->odm);
+}
 
 GF_EXPORT
 void gf_mo_set_speed(GF_MediaObject *mo, Fixed speed)
@@ -710,9 +753,11 @@ u32 gf_mo_get_last_frame_time(GF_MediaObject *mo)
 	return mo ? mo->timestamp : 0;
 }
 
+
 GF_EXPORT
 Bool gf_mo_has_audio(GF_MediaObject *mo)
 {
+	char *sub_url, *ext;
 	u32 i;
 	GF_NetworkCommand com;
 	GF_ClientService *ns;
@@ -722,15 +767,32 @@ Bool gf_mo_has_audio(GF_MediaObject *mo)
 
 	ns = mo->odm->net_service;
 	is = mo->odm->parentscene;
+	sub_url = strchr(ns->url, '#');
 	for (i=0; i<gf_list_count(is->ODlist); i++) {
 		GF_ObjectManager *odm = gf_list_get(is->ODlist, i);
 		if (odm->net_service != ns) continue;
 		if (!odm->mo) continue;
+
+		if (sub_url) {
+			ext = odm->mo->URLs.count ? odm->mo->URLs.vals[0].url : NULL;
+			if (ext) ext = strchr(ext, '#');
+			if (!ext || strcmp(sub_url, ext)) continue;
+		} 
 		/*there is already an audio object in this service, do not recreate one*/
 		if (odm->mo->type == GF_MEDIA_OBJECT_AUDIO) return 0;
 	}
 	memset(&com, 0, sizeof(GF_NetworkCommand) );
 	com.command_type = GF_NET_SERVICE_HAS_AUDIO;
+	com.audio.base_url = mo->URLs.count ? mo->URLs.vals[0].url : NULL;
+	if (!com.audio.base_url) com.audio.base_url = ns->url;
 	if (gf_term_service_command(ns, &com) == GF_OK) return 1;
 	return 0;
 }
+
+GF_EXPORT
+GF_SceneGraph *gf_mo_get_scenegraph(GF_MediaObject *mo)
+{
+	if (!mo || !mo->odm || !mo->odm->subscene) return NULL;
+	return mo->odm->subscene->graph;
+}
+
