@@ -42,7 +42,7 @@ void RenderMediaSensor(GF_Node *node, void *rs, Bool is_destroy)
 		return;
 	}
 
-	if (!st->stream) st->stream = gf_mo_find(node, &st->sensor->url, 0);
+	if (!st->stream) st->stream = gf_mo_register(node, &st->sensor->url, 0);
 	if (!st->stream || !st->stream->odm) return;
 
 	if (!st->is_init) {
@@ -63,7 +63,7 @@ void RenderMediaSensor(GF_Node *node, void *rs, Bool is_destroy)
 		/*dynamic scene*/
 		else ck = st->stream->odm->subscene->dyn_ck;
 		/*since audio may be used alone through an inline scene, we need to refresh the graph*/
-		if (st->stream->odm->state) gf_term_invalidate_renderer(st->stream->odm->term);
+		if (st->stream->odm->state) gf_term_invalidate_compositor(st->stream->odm->term);
 	}
 	/*check anim streams*/
 	else if (st->stream->odm->codec && (st->stream->odm->codec->type==GF_STREAM_SCENE)) ck = st->stream->odm->codec->ck;
@@ -101,9 +101,10 @@ void MS_Modified(GF_Node *node)
 	if (st->stream && st->stream->odm) 
 		gf_list_del_item(st->stream->odm->ms_stack, st);
 
-	st->stream = gf_mo_find(node, &st->sensor->url, 0);
+	gf_mo_unregister(node, st->stream);
+	st->stream = gf_mo_register(node, &st->sensor->url, 0);
 	st->is_init = 0;
-	gf_term_invalidate_renderer(st->parent->root_od->term);
+	gf_term_invalidate_compositor(st->parent->root_od->term);
 }
 
 void MS_UpdateTiming(GF_ObjectManager *odm, Bool is_eos)
@@ -131,7 +132,11 @@ void MS_UpdateTiming(GF_ObjectManager *odm, Bool is_eos)
 				} else {
 					media_sens->sensor->mediaDuration = (Double) (s64)odm->duration;
 				}
-				media_sens->sensor->mediaDuration /= 1000;
+				if (media_sens->sensor->mediaDuration)
+					media_sens->sensor->mediaDuration /= 1000;
+				else
+					media_sens->sensor->mediaDuration = -FIX_ONE;
+
 				gf_node_event_out_str((GF_Node *) media_sens->sensor, "mediaDuration");
 			}
 			if (media_sens->sensor->isActive && (media_sens->sensor->mediaCurrentTime != time)) {
@@ -145,7 +150,7 @@ void MS_UpdateTiming(GF_ObjectManager *odm, Bool is_eos)
 					media_sens->sensor->isActive = 0;
 					gf_node_event_out_str((GF_Node *) media_sens->sensor, "isActive");
 
-					GF_LOG(GF_LOG_DEBUG, GF_LOG_COMPOSE, ("[ODM%d] Deactivating media sensor\n", odm->OD->objectDescriptorID));
+					GF_LOG(GF_LOG_DEBUG, GF_LOG_INTERACT, ("[ODM%d] Deactivating media sensor\n", odm->OD->objectDescriptorID));
 				}
 			}
 			if (is_eos && media_sens->sensor->isActive) {
@@ -164,7 +169,7 @@ void MS_UpdateTiming(GF_ObjectManager *odm, Bool is_eos)
 					media_sens->sensor->isActive = 0;
 					gf_node_event_out_str((GF_Node *) media_sens->sensor, "isActive");
 
-					GF_LOG(GF_LOG_DEBUG, GF_LOG_COMPOSE, ("[ODM%d] Deactivating media sensor at time %g - segment %s\n", odm->OD->objectDescriptorID, time, desc->SegmentName));
+					GF_LOG(GF_LOG_DEBUG, GF_LOG_INTERACT, ("[ODM%d] Deactivating media sensor at time %g - segment %s\n", odm->OD->objectDescriptorID, time, desc->SegmentName));
 				}
 				break;
 			}
@@ -193,7 +198,7 @@ void MS_UpdateTiming(GF_ObjectManager *odm, Bool is_eos)
 				media_sens->sensor->streamObjectStartTime = desc->startTime;
 				gf_node_event_out_str((GF_Node *) media_sens->sensor, "streamObjectStartTime");
 
-				GF_LOG(GF_LOG_DEBUG, GF_LOG_COMPOSE, ("[ODM%d] Activating media sensor time %g - segment %s\n", odm->OD->objectDescriptorID, time, desc->SegmentName));
+				GF_LOG(GF_LOG_DEBUG, GF_LOG_INTERACT, ("[ODM%d] Activating media sensor time %g - segment %s\n", odm->OD->objectDescriptorID, time, desc->SegmentName));
 			}
 			/*set media time - relative to segment start time*/
 			time -= desc->startTime;
@@ -209,7 +214,7 @@ void MS_UpdateTiming(GF_ObjectManager *odm, Bool is_eos)
 				media_sens->sensor->isActive = 0;
 				gf_node_event_out_str((GF_Node *) media_sens->sensor, "isActive");
 				media_sens->active_seg = count;
-				GF_LOG(GF_LOG_DEBUG, GF_LOG_COMPOSE, ("[ODM%d] Deactivating media sensor at time %g: no more segments\n", odm->OD->objectDescriptorID, time));
+				GF_LOG(GF_LOG_DEBUG, GF_LOG_INTERACT, ("[ODM%d] Deactivating media sensor at time %g: no more segments\n", odm->OD->objectDescriptorID, time));
 			}
 		}
 	}
@@ -221,7 +226,7 @@ void MS_Stop(MediaSensorStack *st)
 		st->sensor->isActive = 0;
 		gf_node_event_out_str((GF_Node *) st->sensor, "isActive");
 
-		GF_LOG(GF_LOG_DEBUG, GF_LOG_COMPOSE, ("[ODM%d] Deactivating media sensor\n", st->stream->odm->OD->objectDescriptorID));
+		GF_LOG(GF_LOG_DEBUG, GF_LOG_INTERACT, ("[ODM%d] Deactivating media sensor\n", st->stream->odm->OD->objectDescriptorID));
 	}
 	st->active_seg = 0;
 }
