@@ -46,20 +46,16 @@ static void term_on_message(void *user_priv, GF_ClientService *service, GF_Err e
 	if (error==GF_IP_UDP_TIMEOUT) {
 		const char *sOpt = gf_cfg_get_key(term->user->config, "Network", "AutoReconfigUDP");
 		if (sOpt && !stricmp(sOpt, "yes")) {
-			sOpt = gf_cfg_get_key(term->user->config, "Network", "UDPNotAvailable");
-			/*if option is already set don't bother try reconfig*/
-			if (!sOpt || stricmp(sOpt, "yes")) {
-				char szMsg[1024];
-				sprintf(szMsg, "!! UDP down (%s) - Retrying with TCP !!\n", message);
-				gf_term_message(term, service->url, szMsg, GF_OK);
+			char szMsg[1024];
+			sprintf(szMsg, "!! UDP down (%s) - Retrying with TCP !!\n", message);
+			gf_term_message(term, service->url, szMsg, GF_OK);
 
-				/*reload scene*/
-				if (term->reload_url) free(term->reload_url);
-				term->reload_state = 1;
-				term->reload_url = strdup(term->root_scene->root_od->net_service->url);
-				gf_cfg_set_key(term->user->config, "Network", "UDPNotAvailable", "yes");
-				return;
-			}
+			/*reload scene*/
+			if (term->reload_url) free(term->reload_url);
+			term->reload_state = 1;
+			term->reload_url = strdup(term->root_scene->root_od->net_service->url);
+			gf_cfg_set_key(term->user->config, "Network", "UDPNotAvailable", "yes");
+			return;
 		}
 	}
 	gf_term_message(term, service->url, message, error);
@@ -71,6 +67,8 @@ static void term_on_connect(void *user_priv, GF_ClientService *service, LPNETCHA
 	GF_ObjectManager *root;
 	GET_TERM();
 
+	GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("[Terminal] Connection ACK received from %s (channel %d) - %s\n", service->url, (u32) netch, gf_error_to_string(err) ));
+
 	root = service->owner;
 	if (root && (root->net_service != service)) {
 		gf_term_message(term, service->url, "Incomaptible module type", GF_SERVICE_ERROR);
@@ -78,11 +76,14 @@ static void term_on_connect(void *user_priv, GF_ClientService *service, LPNETCHA
 	}
 	/*this is service connection*/
 	if (!netch) {
+		gf_term_service_media_event(service->owner, GF_EVENT_MEDIA_END_SESSION_SETUP);
 		if (err) {
 			char msg[5000];
 			sprintf(msg, "Cannot open %s", service->url);
 			gf_term_message(term, service->url, msg, err);
 
+			gf_term_service_media_event(service->owner, GF_EVENT_MEDIA_ERROR);
+
 			/*destroy service only if attached*/
 			if (root) {
 				gf_term_lock_net(term, 1);
@@ -105,7 +106,7 @@ static void term_on_connect(void *user_priv, GF_ClientService *service, LPNETCHA
 					1- first remove from parent scene without destroying object, this will trigger a re-setup
 					if other URLs are present
 					2- then destroy object*/
-					gf_is_remove_object(root->parentscene, root, 0);
+					gf_inline_remove_object(root->parentscene, root, 0);
 					gf_odm_disconnect(root, 1);
 				}
 				return;
@@ -135,12 +136,12 @@ static void term_on_connect(void *user_priv, GF_ClientService *service, LPNETCHA
 				GF_ObjectManager *odm = (GF_ObjectManager*)gf_list_get(ODs, 0);
 				gf_list_rem(ODs, 0);
 				/*force re-setup*/
-				gf_is_setup_object(odm->parentscene, odm);
+				gf_inline_setup_object(odm->parentscene, odm);
 			}
 			gf_list_del(ODs);
 		} else {
 			/*setup od*/
-			gf_odm_setup_entry_point(root, NULL);
+			gf_odm_setup_entry_point(root, service->url);
 		}
 		/*load cache if requested*/
 		if (!err && term->enable_cache) {
@@ -214,13 +215,14 @@ static void term_on_disconnect(void *user_priv, GF_ClientService *service, LPNET
 	ch->es_state = GF_ESM_ES_DISCONNECTED;
 }
 
-static void term_on_slp_recieved(void *user_priv, GF_ClientService *service, LPNETCHANNEL netch, char *data, u32 data_size, GF_SLHeader *hdr, GF_Err reception_status)
+static void term_on_slp_received(void *user_priv, GF_ClientService *service, LPNETCHANNEL netch, char *data, u32 data_size, GF_SLHeader *hdr, GF_Err reception_status)
 {
 	GF_Channel *ch;
 	GET_TERM();
 
 	ch = gf_term_get_channel(service, netch);
-	if (!ch) return;
+	if (!ch) 
+		return;
 	
 	if (reception_status==GF_EOS) {
 		gf_es_on_eos(ch);
@@ -230,8 +232,22 @@ static void term_on_slp_recieved(void *user_priv, GF_ClientService *service, LPN
 	gf_es_receive_sl_packet(service, ch, data, data_size, hdr, reception_status);
 }
 
+static Bool is_same_od(GF_ObjectDescriptor *od1, GF_ObjectDescriptor *od2)
+{
+	GF_ESD *esd1, *esd2;
+	if (gf_list_count(od1->ESDescriptors) != gf_list_count(od2->ESDescriptors)) return 0;
+	esd1 = gf_list_get(od1->ESDescriptors, 0);
+	if (!esd1) return 0;
+	esd2 = gf_list_get(od2->ESDescriptors, 0);
+	if (!esd2) return 0;
+	if (esd1->ESID==esd2->ESID) return 1;
+	return 0;
+}
+
 static void term_on_media_add(void *user_priv, GF_ClientService *service, GF_Descriptor *media_desc, Bool no_scene_check)
 {
+	u32 i;
+	GF_MediaObject *the_mo;
 	GF_InlineScene *is;
 	GF_ObjectManager *odm, *root;
 	GF_ObjectDescriptor *od;
@@ -240,9 +256,9 @@ static void term_on_media_add(void *user_priv, GF_ClientService *service, GF_Des
 	root = service->owner;
 	is = root->subscene ? root->subscene : root->parentscene;
 
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_SERVICE, ("[Service %s] %s\n", service->url, media_desc ? "Adding new media object" : "Regenerating scene graph"));
+	GF_LOG(GF_LOG_DEBUG, GF_LOG_NETWORK, ("[Service %s] %s\n", service->url, media_desc ? "Adding new media object" : "Regenerating scene graph"));
 	if (!media_desc) {
-		if (!no_scene_check) gf_is_regenerate(is);
+		if (!no_scene_check) gf_inline_regenerate(is);
 		return;
 	}
 
@@ -259,21 +275,93 @@ static void term_on_media_add(void *user_priv, GF_ClientService *service, GF_Des
 	}
 
 	gf_term_lock_net(term, 1);
-	odm = gf_is_find_odm(is, od->objectDescriptorID);
-	/*remove the old OD*/
-	if (odm) gf_odm_disconnect(odm, 1);
-	odm = gf_odm_new();
+	/*object declared this way are not part of an OD stream and are considered as dynamic*/
+//	od->objectDescriptorID = GF_ESM_DYNAMIC_OD_ID;
+
+	/*check if we have a mediaObject in the scene not attached and matching this object*/
+	the_mo = NULL;
+	odm = NULL;
+	for (i=0; i<gf_list_count(is->media_objects); i++) {
+		char *frag, *ext;
+		GF_ESD *esd;
+		GF_MediaObject *mo = gf_list_get(is->media_objects, i);
+		if (!mo->odm) continue;
+		/*already assigned object - this may happen since the compositor has no control on when objects are declared by the service, 
+		therefore opening file#video and file#audio may result in the objects being declared twice if the service doesn't
+		keep track of declared objects*/
+		if (mo->odm->OD) {
+			if (is_same_od(mo->odm->OD, od)) {
+				/*reassign OD ID*/
+				mo->OD_ID = od->objectDescriptorID;
+				gf_odf_desc_del(media_desc);
+				gf_term_lock_net(term, 0);
+				return;
+			}
+			continue;
+		}
+		if (mo->OD_ID != GF_ESM_DYNAMIC_OD_ID) {
+			if (mo->OD_ID == od->objectDescriptorID) {
+				the_mo = mo;
+				odm = mo->odm;
+				break;
+			}
+			continue;
+		}
+		if (!mo->URLs.count || !mo->URLs.vals[0].url) continue;
+
+		frag = NULL;
+		ext = strrchr(mo->URLs.vals[0].url, '#');
+		if (ext) {
+			frag = strchr(ext, '=');
+			ext[0] = 0;
+		}
+		if (!strstr(service->url, mo->URLs.vals[0].url)) {
+			if (ext) ext[0] = '#';
+			continue;
+		}
+		if (ext) ext[0] = '#';
+
+		esd = gf_list_get(od->ESDescriptors, 0);
+		/*match type*/
+		switch (esd->decoderConfig->streamType) {
+		case GF_STREAM_VISUAL:
+			if (mo->type != GF_MEDIA_OBJECT_VIDEO) continue;
+			break;
+		case GF_STREAM_AUDIO:
+			if (mo->type != GF_MEDIA_OBJECT_AUDIO) continue;
+			break;
+		default:
+			continue;
+		}
+		if (frag) {
+			u32 frag_id = 0;
+			u32 ID = od->objectDescriptorID;
+			if (ID==GF_ESM_DYNAMIC_OD_ID) ID = esd->ESID;
+			frag++;
+			frag_id = atoi(frag);
+			if (ID!=frag_id) continue;
+		}
+		the_mo = mo;
+		odm = mo->odm;
+		break;
+	}
+
+	if (!odm) {
+		odm = gf_odm_new();
+		odm->term = term;
+		odm->parentscene = is;
+		gf_list_add(is->ODlist, odm);
+	}
 	odm->OD = od;
-	odm->term = term;
-	odm->parentscene = is;
+	odm->mo = the_mo;
+	if (the_mo) the_mo->OD_ID = od->objectDescriptorID;
 	odm->flags |= GF_ODM_NOT_IN_OD_STREAM;
-	gf_list_add(is->ODlist, odm);
 	gf_term_lock_net(term, 0);
 
 	gf_odm_setup_object(odm, service);
 
 	/*OD inserted by service: resetup scene*/
-	if (!no_scene_check && is->is_dynamic_scene) gf_is_regenerate(is);
+	if (!no_scene_check && is->is_dynamic_scene) gf_inline_regenerate(is);
 }
 
 static void term_on_command(void *user_priv, GF_ClientService *service, GF_NetworkCommand *com, GF_Err response)
@@ -287,12 +375,25 @@ static void term_on_command(void *user_priv, GF_ClientService *service, GF_Netwo
 		GF_ObjectManager *odm;
 		com->buffer.max = 0;
 		com->buffer.min = com->buffer.occupancy = (u32) -1;
-		if (!service->owner) return;
+		if (!service->owner) {
+			com->buffer.occupancy = 0;
+			return;
+		}
 		
+		/*browse all channels in the scene, running on this service, and get buffer info*/
 		od_list = NULL;
-		if (service->owner->parentscene) od_list = service->owner->parentscene->ODlist;
-		else if (service->owner->subscene) od_list = service->owner->subscene->ODlist;
-		if (!od_list) return;
+		if (service->owner->parentscene) {
+			od_list = service->owner->parentscene->ODlist;
+		} else if (service->owner->subscene) {
+			od_list = service->owner->subscene->ODlist;
+		}
+		if (!od_list) {
+			com->buffer.occupancy = 0;
+			return;
+		}
+		/*get exclusive access to media scheduler, to make sure ODs are not being
+		manipulated*/
+		gf_mx_p(term->mm_mx);
 		i=0;
 		while ((odm = (GF_ObjectManager*)gf_list_enum(od_list, &i))) {
 			u32 j, count;
@@ -300,15 +401,15 @@ static void term_on_command(void *user_priv, GF_ClientService *service, GF_Netwo
 			for (j=0; j<count; j++) {
 				GF_Channel *ch = (GF_Channel *)gf_list_get(odm->channels, j);
 				if (ch->service != service) continue;
-				if (ch->IsEndOfStream) continue;
-				if (ch->clock->Buffering) continue;
 				if (ch->es_state != GF_ESM_ES_RUNNING) continue;
+				if (!ch->MaxBuffer || ch->dispatch_after_db || ch->bypass_sl_and_db || ch->IsEndOfStream) continue;
 				if (ch->MaxBuffer>com->buffer.max) com->buffer.max = ch->MaxBuffer;
 				if (ch->MinBuffer<com->buffer.min) com->buffer.min = ch->MinBuffer;
 				if ((ch->AU_Count > 2)  && ((u32) ch->BufferTime<com->buffer.occupancy))
 					com->buffer.occupancy = ch->BufferTime;
 			}
 		}
+		gf_mx_v(term->mm_mx);
 		if (com->buffer.occupancy==(u32) -1) com->buffer.occupancy = 0;
 		return;
 	}
@@ -396,6 +497,8 @@ static void fetch_mime_io(void *dnld, GF_NETIO_Parameter *parameter)
 	/*only get the content type*/
 	if (parameter->msg_type==GF_NETIO_GET_METHOD) parameter->name = "HEAD";
 #endif
+
+
 }
 
 static char *get_mime_type(GF_Terminal *term, const char *url, GF_Err *ret_code)
@@ -419,9 +522,9 @@ static char *get_mime_type(GF_Terminal *term, const char *url, GF_Err *ret_code)
 }
 
 
-static Bool check_extension(char *szExtList, char *szExt)
+static Bool check_extension(const char *szExtList, char *szExt)
 {
-	char szExt2[50];
+	char szExt2[500];
 	if (szExtList[0] != '"') return 0;
 	szExtList += 1;
 
@@ -446,7 +549,7 @@ static GF_InputService *gf_term_can_handle_service(GF_Terminal *term, const char
 	u32 i;
 	GF_Err e;
 	char *sURL, *ext, *mime_type;
-	char szExt[50];
+	char szExt[500];
 	GF_InputService *ifce;
 
 	GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("[Terminal] Looking for plugin for URL %s\n", url));
@@ -460,13 +563,10 @@ static GF_InputService *gf_term_can_handle_service(GF_Terminal *term, const char
 	if (parent_url) sURL = gf_url_concatenate(parent_url, url);
 
 	/*path absolute*/
-	if (!sURL) {
-		char *tmp = (char *) url;
-		if (!strnicmp(url, "file:///", 8)) tmp += 8;
-		else if (!strnicmp(url, "file://", 7)) tmp += 7;
-		if ((tmp[0]=='/') && (tmp[2]==':')) tmp += 1;
-		sURL = strdup(tmp);
-	}
+	if (!sURL) sURL = strdup(url);
+
+	if (gf_url_is_local(sURL)) 
+		gf_url_to_fs_path(sURL);
 
 	if (no_mime_check) {
 		mime_type = NULL;
@@ -483,7 +583,12 @@ static GF_InputService *gf_term_can_handle_service(GF_Terminal *term, const char
 		}
 	}
 	
-	if (mime_type && (!stricmp(mime_type, "text/plain") || !stricmp(mime_type, "video/quicktime")) ) {
+	if (mime_type && 
+		(!stricmp(mime_type, "text/plain") 
+			|| !stricmp(mime_type, "video/quicktime")
+			|| !stricmp(mime_type, "application/octet-stream")
+		) 
+	) {
 		free(mime_type);
 		mime_type = NULL;
 	}
@@ -505,7 +610,17 @@ static GF_InputService *gf_term_can_handle_service(GF_Terminal *term, const char
 		}
 	}
 
-	ext = strrchr(sURL, '.');
+	
+	ext = strchr(sURL, '#');
+	if (ext) {
+		char *anext;
+		ext[0] = 0;
+		anext = strrchr(sURL, '.');
+		ext[0] = '#';
+		ext = anext;
+	} else {
+		ext = strrchr(sURL, '.');
+	}
 	if (ext && !stricmp(ext, ".gz")) {
 		char *anext;
 		ext[0] = 0;
@@ -534,7 +649,7 @@ static GF_InputService *gf_term_can_handle_service(GF_Terminal *term, const char
 			if (!sMime) continue;
 			sKey = gf_cfg_get_key(term->user->config, "MimeTypes", sMime);
 			if (!sKey) continue;
-			if (!check_extension((char *)sKey, szExt)) continue;
+			if (!check_extension(sKey, szExt)) continue;
 			sPlug = strrchr(sKey, '"');
 			if (!sPlug) continue;	/*bad format entry*/
 			sPlug += 2;
@@ -657,7 +772,7 @@ GF_EXPORT
 void gf_term_on_sl_packet(GF_ClientService *service, LPNETCHANNEL ns, char *data, u32 data_size, GF_SLHeader *hdr, GF_Err reception_status)
 {
 	assert(service);
-	term_on_slp_recieved(service->term, service, ns, data, data_size, hdr, reception_status);
+	term_on_slp_received(service->term, service, ns, data, data_size, hdr, reception_status);
 }
 
 GF_EXPORT
@@ -682,6 +797,7 @@ void NM_DeleteService(GF_ClientService *ns)
 
 	assert(!ns->nb_odm_users);
 	assert(!ns->nb_ch_users);
+	assert(!ns->owner);
 
 	/*delete all the clocks*/
 	while (gf_list_count(ns->Clocks)) {
@@ -745,6 +861,8 @@ void gf_term_download_update_stats(GF_DownloadSession * sess)
 	const char *szURI;
 	u32 total_size, bytes_done, net_status, bytes_per_sec;
 	
+	if (!sess) return;
+
 	gf_dm_sess_get_stats(sess, NULL, &szURI, &total_size, &bytes_done, &bytes_per_sec, &net_status);
 	serv = (GF_ClientService *)gf_dm_sess_get_private(sess);
 	switch (net_status) {
@@ -768,6 +886,7 @@ void gf_term_download_update_stats(GF_DownloadSession * sess)
 			evt.progress.done = bytes_done;
 			GF_USER_SENDEVENT(serv->term->user, &evt);
 		}
+		GF_LOG(GF_LOG_DEBUG, GF_LOG_NETWORK, ("[HTTP] %s received %d / %d\n", szURI, bytes_done, total_size));
 		break;
 	}
 }
@@ -805,7 +924,7 @@ GF_EXPORT
 Bool gf_term_check_extension(GF_InputService *ifce, const char *mimeType, const char *extList, const char *description, const char *fileExt)
 {
 	const char *szExtList;
-	char *ext, szExt[100];
+	char *ext, szExt[500];
 	if (!ifce || !mimeType || !extList || !description || !fileExt) return 0;
 	/*this is a URL*/
 	if ( (strlen(fileExt)>20) || strchr(fileExt, '/')) return 0;
