@@ -32,13 +32,7 @@
 /*WorldInfo node*/
 #include <gpac/nodes_mpeg4.h>
 /*title node*/
-#include <gpac/nodes_svg_da.h>
-#ifdef GPAC_ENABLE_SVG_SA
-#include <gpac/nodes_svg_sa.h>
-#endif
-#ifdef GPAC_ENABLE_SVG_SANI
-#include <gpac/nodes_svg_sani.h>
-#endif
+#include <gpac/nodes_svg.h>
 
 
 static Bool check_in_scene(GF_InlineScene *scene, GF_ObjectManager *odm)
@@ -109,7 +103,7 @@ void gf_term_select_object(GF_Terminal *term, GF_ObjectManager *odm)
 	if (!term || !odm) return;
 	if (!gf_term_check_odm(term, odm)) return;
 
-	gf_is_select_object(term->root_scene, odm);
+	gf_inline_select_object(term->root_scene, odm);
 }
 
 
@@ -125,7 +119,9 @@ static void get_codec_stats(GF_Codec *dec, ODInfo *info)
 
 GF_Err gf_term_get_object_info(GF_Terminal *term, GF_ObjectManager *odm, ODInfo *info)
 {
-	if (!term || !odm || !info) return GF_BAD_PARAM;
+	GF_Channel *ch;
+
+	if (!term || !odm || !odm->OD || !info) return GF_BAD_PARAM;
 	if (!gf_term_check_odm(term, odm)) return GF_BAD_PARAM;
 
 	memset(info, 0, sizeof(ODInfo));
@@ -139,8 +135,10 @@ GF_Err gf_term_get_object_info(GF_Terminal *term, GF_ObjectManager *odm, ODInfo
 		info->current_time /= 1000;
 		info->nb_droped = odm->codec->nb_droped;
 	} else if (odm->subscene && odm->subscene->scene_codec) {
-		info->current_time = gf_clock_time(odm->subscene->scene_codec->ck);
-		info->current_time /= 1000;
+		if (odm->subscene->scene_codec->ck) {
+			info->current_time = gf_clock_time(odm->subscene->scene_codec->ck);
+			info->current_time /= 1000;
+		}
 		info->duration = (Double) (s64)odm->subscene->duration;
 		info->duration /= 1000;
 		info->nb_droped = odm->subscene->scene_codec->nb_droped;
@@ -164,7 +162,6 @@ GF_Err gf_term_get_object_info(GF_Terminal *term, GF_ObjectManager *odm, ODInfo
 		if (!ck) {
 			info->status = 4;
 		} else {
-			GF_Channel *ch;
 			info->status = gf_clock_is_started(ck) ? 1 : 2;
 			info->clock_drift = ck->drift;
 
@@ -178,6 +175,7 @@ GF_Err gf_term_get_object_info(GF_Terminal *term, GF_ObjectManager *odm, ODInfo
 					buf += ch->BufferTime;
 				}
 				if (ch->is_protected) info->protection = ch->ipmp_tool ? 1 : 2;
+
 			}
 			if (buf) info->buffer = (s32) buf;
 		}
@@ -241,6 +239,8 @@ GF_Err gf_term_get_object_info(GF_Terminal *term, GF_ObjectManager *odm, ODInfo
 	if (odm->subscene && odm->subscene->scene_codec) get_codec_stats(odm->subscene->scene_codec, info);
 	else if (odm->codec) get_codec_stats(odm->codec, info);
 
+	ch = (GF_Channel*)gf_list_get(odm->channels, 0);
+	if (ch && ch->esd->langDesc) info->lang = ch->esd->langDesc->langCode;
 	return GF_OK;
 }
 
@@ -309,16 +309,6 @@ const char *gf_term_get_world_info(GF_Terminal *term, GF_ObjectManager *scene_od
 	}
 	if (!info) return NULL;
 	
-#ifdef GPAC_ENABLE_SVG_SA
-	if (gf_node_get_tag(info) == TAG_SVG_SA_title) {
-		return ((SVG_SA_titleElement *) info)->textContent;
-	} else 
-#endif
-#ifdef GPAC_ENABLE_SVG_SANI
-	if (gf_node_get_tag(info) == TAG_SVG_SANI_title) {
-		return ((SVG_SANI_titleElement *) info)->textContent;
-	} else 
-#endif
 	if (gf_node_get_tag(info) == TAG_SVG_title) {
 		/*FIXME*/
 		//return ((SVG_titleElement *) info)->textContent;
