@@ -48,16 +48,22 @@ GF_ObjectManager *gf_odm_new()
 	tmp->Visual_PL = (u8) -1;
 	tmp->ms_stack = gf_list_new();
 	tmp->mc_stack = gf_list_new();
-	tmp->mx = gf_mx_new();
+	tmp->mx = gf_mx_new("ODM");
 	return tmp;
 }
 
 void gf_odm_del(GF_ObjectManager *odm)
 {
+	Bool lock;
 	u32 i;
 	MediaSensorStack *media_sens;
 
-	gf_mx_p(odm->mx);
+	/*make sure we are not in the media queue*/
+	gf_mx_p(odm->term->net_mx);
+	gf_list_del_item(odm->term->media_queue, odm);
+	gf_mx_v(odm->term->net_mx);
+
+	lock = gf_mx_try_lock(odm->mx);
 	i=0;
 	while ((media_sens = (MediaSensorStack *)gf_list_enum(odm->ms_stack, &i))) {
 		MS_Stop(media_sens);
@@ -66,12 +72,13 @@ void gf_odm_del(GF_ObjectManager *odm)
 	}
 	if (odm->mo) odm->mo->odm = NULL;
 
+
 	gf_list_del(odm->channels);
 	gf_list_del(odm->ms_stack);
 	gf_list_del(odm->mc_stack);
 	gf_odf_desc_del((GF_Descriptor *)odm->OD);
 	assert (!odm->net_service);
-	gf_mx_v(odm->mx);
+	if (lock) gf_mx_v(odm->mx);
 	gf_mx_del(odm->mx);
 	free(odm);
 }
@@ -103,7 +110,7 @@ void gf_odm_disconnect(GF_ObjectManager *odm, Bool do_remove)
 	gf_odm_stop(odm, 1);
 
 	/*disconnect sub-scene*/
-	if (odm->subscene) gf_is_disconnect(odm->subscene, do_remove);
+	if (odm->subscene) gf_inline_disconnect(odm->subscene, do_remove);
 
 	/*no destroy*/
 	if (!do_remove) return;
@@ -141,34 +148,35 @@ void gf_odm_disconnect(GF_ObjectManager *odm, Bool do_remove)
 		odm->oci_codec = NULL;
 	}
 
-	/*then detach from network service*/
+	/*detach from network service */
 	if (odm->net_service) {
-		if (odm->net_service->owner == odm) {
-			if (odm->net_service->nb_odm_users) odm->net_service->nb_odm_users--;
+		GF_ClientService *ns = odm->net_service;
+		if (ns->owner == odm) {
+			if (ns->nb_odm_users) ns->nb_odm_users--;
 			/*detach it!!*/
-			odm->net_service->owner = NULL;
+			ns->owner = NULL;
 			/*try to assign a new root in case this is not scene shutdown*/
-			if (odm->net_service->nb_odm_users && odm->parentscene) {
+			if (ns->nb_odm_users && odm->parentscene) {
 				GF_ObjectManager *new_root;
 				u32 i = 0;
 				while ((new_root = (GF_ObjectManager *)gf_list_enum(odm->parentscene->ODlist, &i)) ) {
 					if (new_root == odm) continue;
-					if (new_root->net_service != odm->net_service) continue;
-					new_root->net_service->owner = new_root;
+					if (new_root->net_service != ns) continue;
+					ns->owner = new_root;
 					break;
 				}
 			}
 		}
-		if (!odm->net_service->nb_odm_users) gf_term_close_services(odm->term, odm->net_service);
 		odm->net_service = NULL;
+		if (!ns->nb_odm_users) gf_term_close_services(odm->term, ns);
 	}
 
 	gf_odm_lock(odm, 0);
 
 	/*delete from the parent scene.*/
 	if (odm->parentscene) {
-		gf_is_remove_object(odm->parentscene, odm, do_remove);
-		if (odm->subscene) gf_is_del(odm->subscene);
+		gf_inline_remove_object(odm->parentscene, odm, do_remove);
+		if (odm->subscene) gf_inline_del(odm->subscene);
 		gf_odm_del(odm);
 		return;
 	}
@@ -177,7 +185,7 @@ void gf_odm_disconnect(GF_ObjectManager *odm, Bool do_remove)
 	if (odm->term->root_scene) {
 		GF_Event evt;
 		assert(odm->term->root_scene == odm->subscene);
-		gf_is_del(odm->subscene);
+		gf_inline_del(odm->subscene);
 		/*reset main pointer*/
 		odm->term->root_scene = NULL;
 
@@ -200,9 +208,12 @@ void gf_odm_setup_entry_point(GF_ObjectManager *odm, const char *service_sub_url
 	GF_Descriptor *desc;
 	GF_IPMP_ToolList *toolList;
 
-	assert(odm->OD==NULL);
+//	assert(odm->OD==NULL);
+
+	term = odm->term;
+
+	GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("[ODM] Setting up root object for %s\n", odm->net_service->url));
 
-	odm->net_service->nb_odm_users++;
 	if (odm->subscene) od_type = GF_MEDIA_OBJECT_SCENE;
 	else if (odm->mo) {
 		od_type = odm->mo->type;
@@ -215,26 +226,29 @@ void gf_odm_setup_entry_point(GF_ObjectManager *odm, const char *service_sub_url
 	/*for remote ODs, get expected OD type in case the service needs to generate the IOD on the fly*/
 	if (odm->parentscene && odm->OD && odm->OD->URLString) {
 		GF_MediaObject *mo;
-		mo = gf_is_find_object(odm->parentscene, odm->OD->objectDescriptorID, odm->OD->URLString);
+		mo = gf_inline_find_object(odm->parentscene, odm->OD->objectDescriptorID, odm->OD->URLString);
 		if (mo) od_type = mo->type;
 		ext = strchr(odm->OD->URLString, '#');
 		if (ext) sub_url = ext;
 	}
 
 	desc = odm->net_service->ifce->GetServiceDescriptor(odm->net_service->ifce, od_type, sub_url); 
+	if (odm->OD) return;
 
-	/*create empty service descriptor, this will automatically create a dynamic scene*/
 	if (!desc) {
+		/*if desc is NULL for a media, the media will be declared later by the service (gf_term_media_add)*/
 		if (od_type != GF_MEDIA_OBJECT_SCENE)
 			return;
+		/*create empty service descriptor, this will automatically create a dynamic scene*/
 		desc = gf_odf_desc_new(GF_ODF_OD_TAG);
 	}
+	odm->net_service->nb_odm_users++;
 
 	if (!gf_list_count( ((GF_ObjectDescriptor*)desc)->ESDescriptors)) {
 		/*new subscene*/
 		if (!odm->subscene) {
 			assert(odm->parentscene);
-			odm->subscene = gf_is_new(odm->parentscene);
+			odm->subscene = gf_inline_new(odm->parentscene);
 			odm->subscene->root_od = odm;
 		}
 	}
@@ -287,8 +301,6 @@ void gf_odm_setup_entry_point(GF_ObjectManager *odm, const char *service_sub_url
 		}
 	}
 
-	/*keep track of term since the setup may fail and the OD may be destroyed*/
-	term = odm->term;
 	gf_term_lock_net(term, 1);
 	gf_odm_setup_object(odm, odm->net_service);
 	gf_term_lock_net(term, 0);
@@ -369,7 +381,7 @@ static void ODM_SelectAlternateStream(GF_ObjectManager *odm, u32 lang_code, u8 s
 
 
 /*Validate the streams in this OD, and check if we have to setup an inline scene*/
-GF_Err ODM_ValidateOD(GF_ObjectManager *odm, Bool *hasInline, Bool *externalClock)
+GF_Err ODM_ValidateOD(GF_ObjectManager *odm, Bool *hasInline)
 {
 	u32 i;
 	u16 es_id;
@@ -381,15 +393,10 @@ GF_Err ODM_ValidateOD(GF_ObjectManager *odm, Bool *hasInline, Bool *externalCloc
 	prev_st = 0;
 
 	*hasInline = 0;
-	*externalClock = 0;
 
 	/*step 1: validate OD*/
 	i=0;
 	while ((esd = (GF_ESD *)gf_list_enum(odm->OD->ESDescriptors, &i))) {
-		/*check external clock refs*/
-		if (esd->OCRESID && (esd->OCRESID!=esd->ESID) && (od_get_esd(odm->OD, esd->OCRESID) == NULL) ) {
-			*externalClock = 1;
-		}
 		switch (esd->decoderConfig->streamType) {
 		case GF_STREAM_OD: nb_od++; break;
 		case GF_STREAM_OCR: nb_ocr++; break;
@@ -424,6 +431,7 @@ GF_Err ODM_ValidateOD(GF_ObjectManager *odm, Bool *hasInline, Bool *externalCloc
 		sOpt = "und";
 	}
 	lang = (sOpt[0]<<16) | (sOpt[1]<<8) | sOpt[2];
+#if 0
 	if (gf_list_count(odm->OD->ESDescriptors)>1) {
 		ODM_SelectAlternateStream(odm, lang, GF_STREAM_SCENE);
 		ODM_SelectAlternateStream(odm, lang, GF_STREAM_OD);
@@ -433,7 +441,7 @@ GF_Err ODM_ValidateOD(GF_ObjectManager *odm, Bool *hasInline, Bool *externalCloc
 		ODM_SelectAlternateStream(odm, lang, GF_STREAM_INTERACT);
 		ODM_SelectAlternateStream(odm, lang, GF_STREAM_TEXT);
 	}
-	
+#endif
 	/*no scene, OK*/
 	if (!nb_scene) return GF_OK;
 
@@ -482,7 +490,7 @@ case, the GF_InlineScene pointer will be set by the OD codec.*/
 GF_EXPORT
 void gf_odm_setup_object(GF_ObjectManager *odm, GF_ClientService *serv)
 {
-	Bool hasInline, externalClock;
+	Bool hasInline;
 	u32 i, numOK;
 	GF_Err e;
 	GF_ESD *esd;
@@ -501,6 +509,16 @@ void gf_odm_setup_object(GF_ObjectManager *odm, GF_ClientService *serv)
 		gf_odf_desc_del((GF_Descriptor *)odm->OD);
 		odm->OD = NULL;
 		odm->net_service = NULL;
+		GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("[Terminal] Object redirection to %s\n", url));
+		
+		/*if object is a scene, create the inline before connecting the object.
+			This is needed in irder to register the nodes using the resource for event
+			propagation (stored at the inline level)
+		*/
+		if (odm->mo && (odm->mo->type==GF_MEDIA_OBJECT_SCENE)) {
+			odm->subscene = gf_inline_new(odm->parentscene);
+			odm->subscene->root_od = odm;
+		}
 		gf_term_connect_object(odm->term, odm, url, parent);
 		free(url);
 		return;
@@ -516,36 +534,32 @@ void gf_odm_setup_object(GF_ObjectManager *odm, GF_ClientService *serv)
 	syncRef = (GF_MediaObject*)odm->ocr_codec;
 	odm->ocr_codec = NULL;
 
-	e = ODM_ValidateOD(odm, &hasInline, &externalClock);
+	e = ODM_ValidateOD(odm, &hasInline);
 	if (e) {
 		gf_term_message(odm->term, odm->net_service->url, "MPEG-4 Service Error", e);
 		gf_odm_disconnect(odm, 1);
 		return;
 	}
 
+	if (odm->mo && (odm->mo->type == GF_MEDIA_OBJECT_UPDATES)) {
+		hasInline = 0;
+	}
+
 	/*if there is a BIFS stream in the OD, we need an GF_InlineScene (except if we already 
 	have one, which means this is the first IOD)*/
 	if (hasInline && !odm->subscene) {
-		odm->subscene = gf_is_new(odm->parentscene);
+		odm->subscene = gf_inline_new(odm->parentscene);
 		odm->subscene->root_od = odm;
 	}
 
-	/*this is an inline OD using clocks in its subnamespace - this is NOT supported by gpac since it breaks
-	all buffering logics (and may not be compliant, spec is unclear here) -> force single clock for these 
-	streams (many ISMA files use this...)*/
-	if (hasInline && externalClock) {
-		GF_ESD *esd = (GF_ESD *)gf_list_get(odm->OD->ESDescriptors, 0);
-		odm->subscene->force_sub_clock_id = esd->ESID;
-	}
-
 	numOK = odm->pending_channels = 0;
-
 	/*empty IOD, use a dynamic scene*/
 	if (!gf_list_count(odm->OD->ESDescriptors) && odm->subscene) {
 		GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("[ODM%d] No streams in object - taking over scene graph generation\n",odm->OD->objectDescriptorID));
 		assert(odm->subscene->root_od==odm);
 		odm->subscene->is_dynamic_scene = 1;
 	} else {
+		GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("[Terminal] Setting up object streams\n"));
 		/*avoid channels PLAY request when confirming connection (sync network service)*/
 		odm->state = GF_ODM_STATE_IN_SETUP;
 
@@ -575,10 +589,13 @@ void gf_odm_setup_object(GF_ObjectManager *odm, GF_ClientService *serv)
 	
 	/*setup mediaobject info except for top-level OD*/
 	if (odm->parentscene) {
-		gf_is_setup_object(odm->parentscene, odm);
+		gf_inline_setup_object(odm->parentscene, odm);
 	} else {
 		/*othewise send a connect ack for top level*/
 		GF_Event evt;
+
+		GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("[ODM] Root object connected !\n", odm->net_service->url));
+		
 		evt.type = GF_EVENT_CONNECT;
 		evt.connect.is_connected = 1;
 		GF_USER_SENDEVENT(odm->term->user, &evt);
@@ -595,7 +612,7 @@ void gf_odm_setup_object(GF_ObjectManager *odm, GF_ClientService *serv)
 	if (odm->term->root_scene->is_dynamic_scene && (odm->OD->objectDescriptorID==GF_ESM_DYNAMIC_OD_ID) && (odm->flags & GF_ODM_REMOTE_OD)) {
 		GF_Event evt;
 		if (odm->OD_PL) {
-			gf_is_select_object(odm->term->root_scene, odm);
+			gf_inline_select_object(odm->term->root_scene, odm);
 			odm->OD_PL = 0;
 		}
 		evt.type = GF_EVENT_STREAMLIST;
@@ -603,46 +620,14 @@ void gf_odm_setup_object(GF_ObjectManager *odm, GF_ClientService *serv)
 	}
 }
 
-/*refresh all ODs when an non-interactive stream is found*/
-void gf_odm_refresh_uninteractives(GF_ObjectManager *odm)
-{
-	u32 i, j;
-	GF_Channel *ch;
-	GF_ObjectManager *test_od;
-	GF_InlineScene *in_scene;
 
-	/*check for inline*/
-	in_scene = odm->parentscene;
-	if (odm->subscene) {
-		assert(odm->subscene->root_od == odm);
-		in_scene = odm->subscene;
-		i=0;
-		while ((ch = (GF_Channel*)gf_list_enum(odm->channels, &i)) ) {
-			if (ch->clock->no_time_ctrl) {
-				odm->flags |= GF_ODM_NO_TIME_CTRL;
-				break;
-			}
-		}
-	}
-
-	i=0;
-	while ((test_od = (GF_ObjectManager *)gf_list_enum(in_scene->ODlist, &i)) ) {
-		if (odm==test_od) continue;
-		j=0;
-		while ((ch = (GF_Channel*)gf_list_enum(test_od->channels, &j)) ) {
-			if (ch->clock->no_time_ctrl) {
-				test_od->flags |= GF_ODM_NO_TIME_CTRL;
-				break;
-			}
-		}
-	}
-}
 
 void ODM_CheckChannelService(GF_Channel *ch)
 {
 	if (ch->service == ch->odm->net_service) return;
 	/*if the stream has created a service check if close is needed or not*/
-	if (ch->esd->URLString && !ch->service->nb_ch_users) gf_term_close_services(ch->odm->term, ch->service);
+	if (ch->esd->URLString && !ch->service->nb_ch_users) 
+		gf_term_close_services(ch->odm->term, ch->service);
 }
 
 /*setup channel, clock and query caps*/
@@ -740,6 +725,8 @@ clock_setup:
 	ch->clock = ck;
 	ch->service = serv;
 
+	GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("[ODM] Creating codec for stream %d\n", ch->esd->ESID));
+
 	/*setup the decoder for this stream or find the existing one.*/
 	e = GF_OK;
 	dec = NULL;
@@ -753,6 +740,7 @@ clock_setup:
 		/*OD codec acts as main scene codec when used to generate scene graph*/
 		if (! odm->subscene->od_codec) {
 			odm->subscene->od_codec = gf_codec_new(odm, esd, odm->OD_PL, &e);
+			if (e) return e;
 			gf_term_add_codec(odm->term, odm->subscene->od_codec);
 		} 
 		dec = odm->subscene->od_codec;
@@ -856,7 +844,19 @@ clock_setup:
 	if (dec->decio && (dec->decio->InterfaceType==GF_SCENE_DECODER_INTERFACE) ) {
 		GF_SceneDecoder *sdec = (GF_SceneDecoder *) dec->decio;
 		is = odm->subscene ? odm->subscene : odm->parentscene;
-		if (sdec->AttachScene) sdec->AttachScene(sdec, is, (is->scene_codec==dec) ? 1: 0);
+		if (sdec->AttachScene) {
+			/*if a node asked for this media object, use the scene graph of the node (AnimationStream in PROTO)*/
+			if (odm->mo && odm->mo->node_ptr) {
+				GF_SceneGraph *sg = is->graph;
+				/*FIXME - this MUST be cleaned up*/
+				is->graph = gf_node_get_graph((GF_Node*)odm->mo->node_ptr);
+				sdec->AttachScene(sdec, is, (is->scene_codec==dec) ? 1: 0);
+				is->graph = sg;
+				odm->mo->node_ptr = NULL;
+			} else {
+				sdec->AttachScene(sdec, is, (is->scene_codec==dec) ? 1: 0);
+			}
+		}
 	}
 
 	ch->es_state = GF_ESM_ES_SETUP;
@@ -884,7 +884,7 @@ clock_setup:
 	if (esd->URLString) {
 		GF_ChannelSetup *cs;
 		/*here we have a pb with the MPEG4 model: streams are supposed to be attachable as soon as the OD 
-		update is recieved, but this is not true with ESD URLs, where service setup may take some time (file
+		update is received, but this is not true with ESD URLs, where service setup may take some time (file
 		downloading, authentification, etc...). We therefore need to wait for the service connect response before 
 		setting up the channel...*/
 		cs = (GF_ChannelSetup*)malloc(sizeof(GF_ChannelSetup));
@@ -975,7 +975,10 @@ GF_Err gf_odm_post_es_setup(GF_Channel *ch, GF_Codec *dec, GF_Err had_err)
 			gf_list_rem(ch->odm->channels, 0);
 			/*disconnect*/
 			ch->service->ifce->DisconnectChannel(ch->service->ifce, ch); 
-			if (ch->esd->URLString) ch->service->nb_ch_users--;
+			if (ch->esd->URLString) {
+				assert(ch->service->nb_ch_users);
+				ch->service->nb_ch_users--;
+			}
 			goto err_exit;
 		}
 	}
@@ -1052,7 +1055,10 @@ void ODM_DeleteChannel(GF_ObjectManager *odm, GF_Channel *ch)
 	
 	if (ch->service) {
 		ch->service->ifce->DisconnectChannel(ch->service->ifce, ch); 
-		if (ch->esd->URLString) ch->service->nb_ch_users--;
+		if (ch->esd->URLString) {
+			assert(ch->service->nb_ch_users);
+			ch->service->nb_ch_users--;
+		}
 		ODM_CheckChannelService(ch);
 	}
 
@@ -1081,10 +1087,10 @@ esd_found:
 		if (ch->esd->ESID == ES_ID) break;
 		ch = NULL;
 	}
-	/*destroy ESD*/
-	gf_odf_desc_del((GF_Descriptor *) esd);
 	/*remove channel*/
 	if (ch) ODM_DeleteChannel(odm, ch);
+	/*destroy ESD*/
+	gf_odf_desc_del((GF_Descriptor *) esd);
 }
 
 /*this is the tricky part: make sure the net is locked before doing anything since an async service 
@@ -1094,7 +1100,7 @@ void gf_odm_start(GF_ObjectManager *odm)
 	gf_term_lock_net(odm->term, 1);
 
 	/*only if not open & ready (not waiting for ACK on channel setup)*/
-	if (!odm->state && !odm->pending_channels) {
+	if (!odm->state && !odm->pending_channels && odm->OD) {
 		GF_Channel *ch;
 		u32 i = 0;
 		odm->state = GF_ODM_STATE_PLAY;
@@ -1154,7 +1160,6 @@ void gf_odm_play(GF_ObjectManager *odm)
 	while ( (ch = (GF_Channel*)gf_list_enum(odm->channels, &i)) ) {
 		Double ck_time;
 
-
 		if (ch->ipmp_tool) {
 			GF_IPMPEvent evt;
 			GF_Err e;
@@ -1258,6 +1263,8 @@ void gf_odm_play(GF_ObjectManager *odm)
 		return;
 	}
 
+	gf_term_service_media_event(odm, GF_EVENT_MEDIA_DATA_REQUEST);
+
 
 	/*start codecs last (otherwise we end up pulling data from channels not yet connected->pbs when seeking)*/
 	if (odm->codec) {
@@ -1270,6 +1277,11 @@ void gf_odm_play(GF_ObjectManager *odm)
 	} else if (odm->subscene) {
 		if (odm->subscene->scene_codec) gf_term_start_codec(odm->subscene->scene_codec);
 		if (!skip_od_st && odm->subscene->od_codec) gf_term_start_codec(odm->subscene->od_codec);
+
+		if (odm->flags & GF_ODM_REGENERATE_SCENE) {
+			odm->flags &= ~GF_ODM_REGENERATE_SCENE;
+			gf_inline_regenerate(odm->subscene);
+		}
 	}
 	if (odm->ocr_codec) gf_term_start_codec(odm->ocr_codec);
 	if (odm->oci_codec) gf_term_start_codec(odm->oci_codec);
@@ -1349,6 +1361,7 @@ void gf_odm_stop(GF_ObjectManager *odm, Bool force_close)
 			GF_LOG(GF_LOG_INFO, GF_LOG_MEDIA, ("[ODM%d] CH %d At OTB %d requesting STOP\n", odm->OD->objectDescriptorID, ch->esd->ESID, gf_clock_time(ch->clock)));
 		}
 	}
+	gf_term_service_media_event(odm, GF_EVENT_MEDIA_STOP);
 
 	/*stop channels*/
 	i=0;
@@ -1378,6 +1391,8 @@ void gf_odm_stop(GF_ObjectManager *odm, Bool force_close)
 void gf_odm_on_eos(GF_ObjectManager *odm, GF_Channel *on_channel)
 {
 	if (gf_odm_check_segment_switch(odm)) return;
+
+	gf_term_service_media_event(odm, GF_EVENT_MEDIA_END_OF_DATA);
 	
 	if (odm->codec && (on_channel->esd->decoderConfig->streamType==odm->codec->type)) {
 		gf_codec_set_status(odm->codec, GF_ESM_CODEC_EOS);
@@ -1421,7 +1436,7 @@ void gf_odm_set_duration(GF_ObjectManager *odm, GF_Channel *ch, u64 stream_durat
 	}
 
 	/*update scene duration*/
-	gf_is_set_duration(odm->subscene ? odm->subscene : (odm->parentscene ? odm->parentscene : odm->term->root_scene));
+	gf_inline_set_duration(odm->subscene ? odm->subscene : (odm->parentscene ? odm->parentscene : odm->term->root_scene));
 }
 
 
