@@ -125,7 +125,7 @@ static void gf_yuv_load_lines(unsigned char *dst, s32 dststride, unsigned char *
 
 
 
-static void gf_yuva_load_lines(unsigned char *dst, s32 dststride, unsigned char *y_src, unsigned char *v_src, unsigned char * u_src, unsigned char *a_src,
+static void gf_yuva_load_lines(unsigned char *dst, s32 dststride, unsigned char *y_src, unsigned char *u_src, unsigned char *v_src, unsigned char *a_src,
 				 s32 y_stride, s32 uv_stride, s32 width)
 {
 	u32 x, hw;
@@ -542,6 +542,19 @@ static void load_line_bgr_24(u8 *src_bits, u32 x_offset, u32 y_offset, u32 y_pit
 }
 
 static void load_line_rgb_32(u8 *src_bits, u32 x_offset, u32 y_offset, u32 y_pitch, u32 width, u8 *dst_bits)
+{
+	u32 i;
+	src_bits += x_offset*4 + y_offset*y_pitch;
+	for (i=0; i<width; i++) {
+		dst_bits[0] = *src_bits++;
+		dst_bits[1] = *src_bits++;
+		dst_bits[2] = *src_bits++;
+		dst_bits[3] = *src_bits++;
+		dst_bits += 4;
+	}
+}
+
+static void load_line_argb(u8 *src_bits, u32 x_offset, u32 y_offset, u32 y_pitch, u32 width, u8 *dst_bits)
 {
 	u32 i;
 	src_bits += x_offset*4 + y_offset*y_pitch;
@@ -590,12 +603,13 @@ static void load_line_yuva(char *src_bits, u32 x_offset, u32 y_offset, u32 y_pit
 static void gf_cmx_apply_argb(GF_ColorMatrix *_this, u8 *a_, u8 *r_, u8 *g_, u8 *b_);
 
 GF_EXPORT
-GF_Err gf_stretch_bits(GF_VideoSurface *dst, GF_VideoSurface *src, GF_Window *dst_wnd, GF_Window *src_wnd, s32 dst_x_pitch, u8 alpha, Bool flip, u32 *col_key, GF_ColorMatrix *cmat)
+GF_Err gf_stretch_bits(GF_VideoSurface *dst, GF_VideoSurface *src, GF_Window *dst_wnd, GF_Window *src_wnd, s32 dst_x_pitch, u8 alpha, Bool flip, GF_ColorKey *key, GF_ColorMatrix *cmat)
 {
 	u8 *tmp, *rows;
-	u8 ka, kr, kg, kb;
+	u8 ka, kr, kg, kb, kl, kh;
 	s32 src_row;
 	u32 i, yuv_type = 0;
+	Bool yuv_init = 0;
 	Bool has_alpha = (alpha!=0xFF) ? 1 : 0;
 	u32 dst_bpp, dst_w_size;
 	s32 pos_y, inc_y, inc_x, prev_row, x_off;
@@ -605,7 +619,7 @@ GF_Err gf_stretch_bits(GF_VideoSurface *dst, GF_VideoSurface *src, GF_Window *ds
 	load_line_proto load_line = NULL;
 
 	if (cmat && (cmat->m[15] || cmat->m[16] || cmat->m[17] || (cmat->m[18]!=FIX_ONE) || cmat->m[19] )) has_alpha = 1;
-	else if (col_key) has_alpha = GF_COL_A(*col_key) ? 1 : 0;
+	else if (key && (key->alpha<0xFF)) has_alpha = 1;
 
 	switch (src->pixel_format) {
 	case GF_PIXEL_RGB_555:
@@ -622,11 +636,13 @@ GF_Err gf_stretch_bits(GF_VideoSurface *dst, GF_VideoSurface *src, GF_Window *ds
 		break;
 	case GF_PIXEL_ARGB:
 		has_alpha = 1;
-	case GF_PIXEL_RGB_32:
-		load_line = load_line_rgb_32;
+		load_line = load_line_argb;
 		break;
 	case GF_PIXEL_RGBA:
 		has_alpha = 1;
+	case GF_PIXEL_RGB_32:
+		load_line = load_line_rgb_32;
+		break;
 	case GF_PIXEL_BGR_32:
 		load_line = load_line_bgr_32;
 		break;
@@ -637,6 +653,7 @@ GF_Err gf_stretch_bits(GF_VideoSurface *dst, GF_VideoSurface *src, GF_Window *ds
 		yuv_type = 1;
 		break;
 	case GF_PIXEL_YUVA:
+		has_alpha = 1;
 		yuv_type = 2;
 		break;
 	default:
@@ -689,6 +706,8 @@ GF_Err gf_stretch_bits(GF_VideoSurface *dst, GF_VideoSurface *src, GF_Window *ds
 	dst_w = dst_wnd ? dst_wnd->w : dst->width;
 	dst_h = dst_wnd ? dst_wnd->h : dst->height;
 
+	if (yuv_type && (src_w%2)) src_w++;
+
 	tmp = (u8 *) malloc(sizeof(u8) * src_w * (yuv_type ? 8 : 4) );
 	rows = tmp;
 
@@ -717,12 +736,15 @@ GF_Err gf_stretch_bits(GF_VideoSurface *dst, GF_VideoSurface *src, GF_Window *ds
 	/*for 2 and 4 bytes colors, precompute pitch for u16 and u32 type casting*/
 	if ((dst_bpp==2) || (dst_bpp==4) ) dst_x_pitch /= (s32) dst_bpp;
 
-	ka = kr = kg = kb = 0;
-	if (col_key) {
-		ka = GF_COL_A(*col_key);
-		kr = GF_COL_R(*col_key);
-		kg = GF_COL_G(*col_key);
-		kb = GF_COL_B(*col_key);
+	kl = kh = ka = kr = kg = kb = 0;
+	if (key) {
+		ka = key->alpha;
+		kr = key->r;
+		kg = key->g;
+		kb = key->b;
+		kl = key->low;
+		kh = key->high;
+		if (kh==kl) kh++;
 	}
 
 	while (dst_h) {
@@ -734,7 +756,18 @@ GF_Err gf_stretch_bits(GF_VideoSurface *dst, GF_VideoSurface *src, GF_Window *ds
 		if (prev_row != src_row) {
 			u32 the_row = src_row - 1;
 			if (yuv_type) {
-				if ( (src_row - 1) % 2) {
+				if ( the_row % 2) {
+					if (!yuv_init) {
+						yuv_init = 1;
+						the_row --;
+						if (flip) the_row = src->height-2 - the_row;
+						if (yuv_type==1) {
+							load_line_yv12(src->video_buffer, x_off, the_row, src->pitch, src_w, src->height, tmp);
+						} else {
+							load_line_yuva(src->video_buffer, x_off, the_row, src->pitch, src_w, src->height, tmp);
+						}
+						the_row = src_row - 1;
+					}
 					rows = flip ? tmp : tmp + src_w * 4;
 				} else {
 					if (flip) the_row = src->height-2 - the_row;
@@ -743,6 +776,7 @@ GF_Err gf_stretch_bits(GF_VideoSurface *dst, GF_VideoSurface *src, GF_Window *ds
 					} else {
 						load_line_yuva(src->video_buffer, x_off, the_row, src->pitch, src_w, src->height, tmp);
 					}
+					yuv_init = 1;
 					rows = flip ? tmp + src_w * 4 : tmp;
 						
 					if (cmat) {
@@ -751,11 +785,21 @@ GF_Err gf_stretch_bits(GF_VideoSurface *dst, GF_VideoSurface *src, GF_Window *ds
 							gf_cmx_apply_argb(cmat, &tmp[idx+3], &tmp[idx], &tmp[idx+1], &tmp[idx+2]);
 						}
 					}
-					if (col_key) {
+					if (key) {
 						for (i=0; i<2*src_w; i++) {
 							u32 idx = 4*i; 
-							if ( (tmp[idx]==kr) && (tmp[idx+1]==kg) && (tmp[idx+2]==kb)) 
-								tmp[idx+3] = ka;
+							s32 thres, v;
+							v = tmp[idx]-kr; thres = ABS(v);
+							v = tmp[idx+1]-kg; thres += ABS(v);
+							v = tmp[idx+2]-kb; thres += ABS(v);
+							thres/=3;
+#if 0
+							if (thres < kl) tmp[idx+3] = 0;
+							else if (thres <= kh) tmp[idx+3] = (thres-kl)*ka / (kh-kl);
+#else
+							if (thres < kh) tmp[idx+3] = 0;
+#endif
+							else tmp[idx+3] = ka;
 						}
 					}
 				}
@@ -769,10 +813,15 @@ GF_Err gf_stretch_bits(GF_VideoSurface *dst, GF_VideoSurface *src, GF_Window *ds
 						gf_cmx_apply_argb(cmat, &tmp[idx+3], &tmp[idx], &tmp[idx+1], &tmp[idx+2]);
 					}
 				}
-				if (col_key) {
+				if (key) {
 					for (i=0; i<src_w; i++) {
 						u32 idx = 4*i; 
-						if ( (tmp[idx]==kr) && (tmp[idx+1]==kg)  && (tmp[idx+2]==kb)) tmp[idx+3] = ka;
+						s32 thres;
+						thres = (255 + (tmp[idx]-kr)) % 255;
+						thres += (255 + (tmp[idx+1]-kg)) % 255;
+						thres += (255 + (tmp[idx+2]-kb)) % 255;
+						if (thres > 3*0xFE) tmp[idx+3] = ka;
+//						if ( (tmp[idx]==kr) && (tmp[idx+1]==kg) && (tmp[idx+2]==kb)) tmp[idx+3] = ka;
 					}
 				}
 			}
