@@ -25,7 +25,6 @@
 #include <gpac/module.h>
 #include <gpac/list.h>
 
-
 #define MAX_SECTION_NAME		500
 #define MAX_INI_LINE			2046
 
@@ -76,10 +75,8 @@ GF_Config *gf_cfg_new(const char *filePath, const char* file_name)
 	tmp = (GF_Config *)malloc(sizeof(GF_Config));
 	memset((void *)tmp, 0, sizeof(GF_Config));
 
-	tmp->filePath = (char *)malloc(sizeof(char) * (strlen(filePath)+1));
-	strcpy(tmp->filePath, filePath ? filePath : "");
-	tmp->fileName = (char *)malloc(sizeof(char) * (strlen(fileName)+1));
-	strcpy(tmp->fileName, fileName);
+	tmp->filePath = strdup(filePath);
+	tmp->fileName = strdup(fileName);
 	tmp->sections = gf_list_new();
 
 	//load the file
@@ -89,12 +86,16 @@ GF_Config *gf_cfg_new(const char *filePath, const char* file_name)
 		ret = fgets(line, MAX_INI_LINE, file);
 
 		if (!ret) continue;
-		if (!strlen(line)) continue;
-		if (line[0] == '#') continue;
 
 		//get rid of the end of line stuff
-		while ((strlen(line) > 0) && ((line[strlen(line)-1] == '\n') || (line[strlen(line)-1] == '\r')) )
-			line[strlen(line)-1] = 0;
+		while (1) {
+			u32 len = strlen(line);
+			if (!len) break;
+			if ((line[len-1] != '\n') && (line[len-1] != '\r')) break;
+			line[len-1] = 0;
+		}
+		if (!strlen(line)) continue;
+		if (line[0] == '#') continue;
 
 		
 		//new section
@@ -140,20 +141,22 @@ static void DelSection(IniSection *ptr)
 {
 	IniKey *k;
 	if (!ptr) return;
-
-	while (gf_list_count(ptr->keys)) {
-		k = (IniKey *) gf_list_get(ptr->keys, 0);
-		if (k->value) free(k->value);
-		if (k->name) free(k->name);
-		free(k);
-		gf_list_rem(ptr->keys, 0);
+	if (ptr->keys) {
+		while (gf_list_count(ptr->keys)) {
+			k = (IniKey *) gf_list_get(ptr->keys, 0);
+			if (k->value) free(k->value);
+			if (k->name) free(k->name);
+			free(k);
+			gf_list_rem(ptr->keys, 0);
+		}
+		gf_list_del(ptr->keys);
 	}
-	gf_list_del(ptr->keys);
 	free(ptr);
 }
 
 
-static GF_Err WriteIniFile(GF_Config *iniFile)
+GF_EXPORT 
+GF_Err gf_cfg_save(GF_Config *iniFile)
 {
 	u32 i, j;
 	IniSection *sec;
@@ -185,7 +188,7 @@ void gf_cfg_del(GF_Config *iniFile)
 	IniSection *p;
 	if (!iniFile) return;
 
-	WriteIniFile(iniFile);
+	gf_cfg_save(iniFile);
 	while (gf_list_count(iniFile->sections)) {
 		p = (IniSection *) gf_list_get(iniFile->sections, 0);
 		DelSection(p);
@@ -311,6 +314,23 @@ const char *gf_cfg_get_key_name(GF_Config *iniFile, const char *secName, u32 key
 	return NULL;
 }
 
+GF_EXPORT
+void gf_cfg_del_section(GF_Config *iniFile, const char *secName)
+{
+	u32 i;
+	IniSection *p;
+	if (!iniFile) return;
+
+	i = 0;
+	while ((p = gf_list_enum(iniFile->sections, &i))) {
+		if (!strcmp(secName, p->section_name)) {
+			DelSection(p);
+			gf_list_rem(iniFile->sections, i-1);
+			return;
+		}
+	}
+}
+
 GF_EXPORT
 GF_Err gf_cfg_insert_key(GF_Config *iniFile, const char *secName, const char *keyName, const char *keyValue, u32 index)
 {
