@@ -30,10 +30,7 @@
 #include <gpac/thread.h>
 #include <gpac/list.h>
 #include <gpac/base_coding.h>
-
-#ifdef WIN32
-//#define GPAC_HAS_SSL
-#endif
+#include <gpac/crypt.h>
 
 
 #ifdef GPAC_HAS_SSL
@@ -46,7 +43,7 @@
 
 static void gf_dm_connect(GF_DownloadSession *sess);
 
-#define GF_DOWNLOAD_AGENT_NAME		"GPAC/" GPAC_VERSION
+#define GF_DOWNLOAD_AGENT_NAME		"GPAC/" GPAC_FULL_VERSION
 #define GF_DOWNLOAD_BUFFER_SIZE		8192
 
 /*internal flags*/
@@ -232,36 +229,50 @@ static Bool gf_dm_can_handle_url(GF_DownloadManager *dm, const char *url)
 
 void gf_dm_configure_cache(GF_DownloadSession *sess)
 {
-	u32 i, last_sep;
-	char cache_name[GF_MAX_PATH], tmp[GF_MAX_PATH];
+	u32 i, len;
+	char *tmp, *ext;
+	u8 hash[20];
 	const char *opt;
-	char *doubledots;
 
 	if (!sess->dm->cache_directory) return;
 	if (sess->flags & GF_NETIO_SESSION_NOT_CACHED) return;
 
-	strcpy(cache_name, sess->dm->cache_directory);
+	len = strlen(sess->server_name) + strlen(sess->remote_path) + 1;
+	if (len<50) len = 50;
+	tmp = malloc(sizeof(char) * len);
+	tmp[0] = 0;
 
+	/*generate hash of the full url*/
 	strcpy(tmp, sess->server_name);
-	while ( (doubledots = strchr(tmp, ':')) )
-		*doubledots = '_';
 	strcat(tmp, sess->remote_path);
-	last_sep = 0;
-
-	for (i=0; i<strlen(tmp); i++) {
-		if (tmp[i] == '/') tmp[i] = '_';
-		else if (tmp[i] == '.') {
-			tmp[i] = '_';
-			last_sep = i;
-		}
+	gf_sha1_csum(tmp, strlen(tmp), hash);
+	tmp[0] = 0;
+	for (i=0; i<20; i++) {
+		char t[3];
+		t[2] = 0;
+		sprintf(t, "%02X", hash[i]);
+		strcat(tmp, t);
 	}
-	if (last_sep) tmp[last_sep] = '.';
-	strcat(cache_name, tmp);
+
+	len += strlen(sess->dm->cache_directory) + 6;
+	sess->cache_name = malloc(sizeof(char)*len);
+	sess->cache_name[0] = 0;
+
+	strcpy(sess->cache_name, sess->dm->cache_directory);
+	strcat(sess->cache_name, tmp);
+
+	/*try to locate an extension*/
+	strcpy(tmp, sess->remote_path);
+	ext = strchr(tmp, '?');
+	if (ext) ext[0] = 0;
+	ext = strchr(tmp, '.');
+	if (ext && (strlen(ext)<6) ) strcat(sess->cache_name, ext);
+	free(tmp);
 
 	/*first try, check cached file*/
 	if (!sess->cache_start_size) {
 		/*if file present figure out how much of the file is downloaded - we assume 2^31 byte file max*/
-		FILE *the_cache = fopen(cache_name, "rb");
+		FILE *the_cache = fopen(sess->cache_name, "rb");
 		if (the_cache) {
 			fseek(the_cache, 0, SEEK_END);
 			sess->cache_start_size = ftell(the_cache);
@@ -272,7 +283,7 @@ void gf_dm_configure_cache(GF_DownloadSession *sess)
 	else {
 		sess->cache_start_size = 0;
 	}
-	sess->cache_name = strdup(cache_name);
+	GF_LOG(GF_LOG_INFO, GF_LOG_NETWORK, ("[HTTP] Cache setup to %s\n", sess->cache_name));
 
 	/*are we using existing cached files ?*/
 	opt = gf_cfg_get_key(sess->dm->cfg, "Downloader", "RestartFiles");
@@ -364,7 +375,7 @@ GF_Err gf_dm_sess_last_error(GF_DownloadSession *sess)
 
 static GF_Err gf_dm_setup_from_url(GF_DownloadSession *sess, char *url)
 {
-	char *tmp, tmp_url[GF_MAX_PATH];
+	char *tmp, *tmp_url;
 	const char *opt;
 	if (!strnicmp(url, "http://", 7)) {
 		url += 7;
@@ -405,10 +416,10 @@ static GF_Err gf_dm_setup_from_url(GF_DownloadSession *sess, char *url)
 	sess->remote_path = strdup(tmp ? tmp : "/");
 	if (tmp) {
 		tmp[0] = 0;
-		strcpy(tmp_url, url);
+		tmp_url = strdup(url);
 		tmp[0] = '/';
 	} else {
-		strcpy(tmp_url, url);
+		tmp_url = strdup(url);
 	}
 	
 	tmp = strrchr(tmp_url, ':');
@@ -434,6 +445,7 @@ static GF_Err gf_dm_setup_from_url(GF_DownloadSession *sess, char *url)
 	} else {
 		sess->server_name = strdup(tmp_url);
 	}
+	free(tmp_url);
 
 	/*setup BW limiter*/
 	sess->limit_data_rate = 0;
@@ -515,8 +527,8 @@ GF_DownloadSession *gf_dm_sess_new(GF_DownloadManager *dm, char *url, u32 dl_fla
 		return NULL;
 	}
 	if (!(sess->flags & GF_NETIO_SESSION_NOT_THREADED) ) {
-		sess->th = gf_th_new();
-		sess->mx = gf_mx_new();
+		sess->th = gf_th_new(url);
+		sess->mx = gf_mx_new(url);
 		gf_th_run(sess->th, gf_dm_session_thread, sess);
 	}
 	sess->num_retry = SESSION_RETRY_COUNT;
@@ -579,10 +591,11 @@ static void gf_dm_connect(GF_DownloadSession *sess)
 {
 	GF_Err e;
 	u16 proxy_port = 0;
-	const char *proxy;
+	const char *proxy, *ip;
 	if (!sess->sock) {
-		//sess->num_retry = 40;
+		sess->num_retry = 40;
 		sess->sock = gf_sk_new(GF_SOCK_TYPE_TCP);
+		//gf_sk_set_block_mode(sess->sock, 1);
 	}
 
 	/*connect*/
@@ -600,11 +613,22 @@ static void gf_dm_connect(GF_DownloadSession *sess)
 		proxy = NULL;
 	}
 
-	if (proxy) {
-		e = gf_sk_connect(sess->sock, (char *) proxy, proxy_port);
+
+	ip = gf_cfg_get_key(sess->dm->cfg, "Network", "MobileIPEnabled");
+	if (ip && !strcmp(ip, "yes")) {
+		ip = gf_cfg_get_key(sess->dm->cfg, "Network", "MobileIP");
 	} else {
-		e = gf_sk_connect(sess->sock, sess->server_name, sess->port);
+		ip = NULL;
 	}
+
+	if (!proxy) {
+		proxy = sess->server_name;
+		proxy_port = sess->port;
+	}
+	GF_LOG(GF_LOG_INFO, GF_LOG_NETWORK, ("[HTTP] Connecting to %s:%d\n", proxy, proxy_port));
+
+	e = gf_sk_connect(sess->sock, (char *) proxy, proxy_port, (char *)ip);
+
 	/*retry*/
 	if ((e == GF_IP_SOCK_WOULD_BLOCK) && sess->num_retry) {
 		sess->status = GF_NETIO_SETUP;
@@ -622,7 +646,8 @@ static void gf_dm_connect(GF_DownloadSession *sess)
 	
 	sess->status = GF_NETIO_CONNECTED;
 	gf_dm_sess_notify_state(sess, GF_NETIO_CONNECTED, GF_OK);
-	gf_sk_set_block_mode(sess->sock, 1);
+	//gf_sk_set_block_mode(sess->sock, 1);
+	gf_sk_set_buffer_size(sess->sock, 0, GF_DOWNLOAD_BUFFER_SIZE);
 	gf_dm_configure_cache(sess);
 
 #ifdef GPAC_HAS_SSL
@@ -675,6 +700,8 @@ const char *gf_dm_sess_mime_type(GF_DownloadSession *sess)
 		/*setup download*/
 		case GF_NETIO_SETUP:
 			gf_dm_connect(sess);
+			if (sess->status == GF_NETIO_SETUP)
+				gf_sleep(200);
 			break;
 		case GF_NETIO_WAIT_FOR_REPLY:
 			gf_sleep(20);
@@ -749,7 +776,7 @@ void gf_dm_del(GF_DownloadManager *dm)
 }
 
 
-static GFINLINE void gf_dm_data_recieved(GF_DownloadSession *sess, char *data, u32 nbBytes)
+static GFINLINE void gf_dm_data_received(GF_DownloadSession *sess, char *data, u32 nbBytes)
 {
 	GF_NETIO_Parameter par;
 	u32 runtime, rcv;
@@ -857,7 +884,8 @@ GF_EXPORT
 GF_Err gf_dm_sess_fetch_data(GF_DownloadSession *sess, char *buffer, u32 buffer_size, u32 *read_size)
 {
 	GF_Err e;
-	if (sess->cache || !buffer || !buffer_size) return GF_BAD_PARAM;
+//	if (sess->cache || !buffer || !buffer_size) return GF_BAD_PARAM;
+	if (/*sess->cache || */ !buffer || !buffer_size) return GF_BAD_PARAM;
 	if (sess->th) return GF_BAD_PARAM;
 	if (sess->status == GF_NETIO_DISCONNECTED) return GF_EOS;
 	if (sess->status > GF_NETIO_DATA_TRANSFERED) return GF_BAD_PARAM;
@@ -870,21 +898,29 @@ GF_Err gf_dm_sess_fetch_data(GF_DownloadSession *sess, char *buffer, u32 buffer_
 		return GF_OK;
 	} else if (sess->status < GF_NETIO_DATA_EXCHANGE) {
 		sess->do_requests(sess);
+		if (sess->status > GF_NETIO_DATA_TRANSFERED) return GF_SERVICE_ERROR;
 		return GF_OK;
 	}
 	/*we're running but we had data previously*/
 	if (sess->init_data) {
-		memcpy(buffer, sess->init_data, sizeof(char)*sess->init_data_size);
-		*read_size = sess->init_data_size;
-		free(sess->init_data);
-		sess->init_data = NULL;
-		sess->init_data_size = 0;
+		if (sess->init_data_size<=buffer_size) {
+			memcpy(buffer, sess->init_data, sizeof(char)*sess->init_data_size);
+			*read_size = sess->init_data_size;
+			free(sess->init_data);
+			sess->init_data = NULL;
+			sess->init_data_size = 0;
+		} else {
+			memcpy(buffer, sess->init_data, sizeof(char)*buffer_size);
+			*read_size = buffer_size;
+			sess->init_data_size -= buffer_size;
+			memcpy(sess->init_data, sess->init_data+buffer_size, sizeof(char)*sess->init_data_size);
+		}
 		return GF_OK;
 	}
 
 	e = gf_dm_read_data(sess, buffer, buffer_size, read_size);
 	if (e) return e;
-	gf_dm_data_recieved(sess, buffer, *read_size);
+	gf_dm_data_received(sess, buffer, *read_size);
 	return GF_OK;
 }
 
@@ -968,8 +1004,10 @@ void http_do_requests(GF_DownloadSession *sess)
 		char range_buf[1024];
 		char pass_buf[1024];
 		const char *user_agent;
+		const char *user_profile;
+		const char *param_string;
 		u32 size;
-		Bool has_accept, has_connection, has_range, has_agent;
+		Bool has_accept, has_connection, has_range, has_agent, send_profile;
 
 		/*setup authentification*/
 		strcpy(pass_buf, "");
@@ -1007,6 +1045,7 @@ void http_do_requests(GF_DownloadSession *sess)
 		user_agent = gf_cfg_get_key(sess->dm->cfg, "Downloader", "UserAgent");
 		if (!user_agent) user_agent = GF_DOWNLOAD_AGENT_NAME;
 
+		
 		par.error = 0;
 		par.msg_type = GF_NETIO_GET_METHOD;
 		par.name = NULL;
@@ -1020,8 +1059,19 @@ void http_do_requests(GF_DownloadSession *sess)
 			sess->http_read_type = 0;
 		}
 
-		sprintf(sHTTP, "%s %s HTTP/1.0\r\nHost: %s\r\n" ,
-			par.name ? par.name : "GET", sess->remote_path, sess->server_name);
+		param_string = gf_cfg_get_key(sess->dm->cfg, "Downloader", "ParamString");
+		if (param_string) {
+			if (strchr(sess->remote_path, '?')) {
+				sprintf(sHTTP, "%s %s&%s HTTP/1.0\r\nHost: %s\r\n" ,
+					par.name ? par.name : "GET", sess->remote_path, param_string, sess->server_name);
+			} else {
+				sprintf(sHTTP, "%s %s?%s HTTP/1.0\r\nHost: %s\r\n" ,
+					par.name ? par.name : "GET", sess->remote_path, param_string, sess->server_name);
+			}
+		} else {
+			sprintf(sHTTP, "%s %s HTTP/1.0\r\nHost: %s\r\n" ,
+				par.name ? par.name : "GET", sess->remote_path, sess->server_name);
+		}
 
 		/*signal we support title streaming*/
 		if (!strcmp(sess->remote_path, "/")) strcat(sHTTP, "icy-metadata:1\r\n");
@@ -1059,26 +1109,82 @@ void http_do_requests(GF_DownloadSession *sess)
 			strcat(sHTTP, "\r\n");
 		}
 		if (sess->flags & GF_DOWNLOAD_IS_ICY) strcat(sHTTP, "Icy-Metadata: 1\r\n");
-		
+
 		par.msg_type = GF_NETIO_GET_CONTENT;
 		par.data = NULL;
 		par.size = 0;
-		gf_dm_sess_user_io(sess, &par);
-		if (par.data && par.size) {
-			sprintf(range_buf, "Content-Length: %d\r\n", par.size);
-			strcat(sHTTP, range_buf);
+
+		/*check if we have personalization info*/
+		send_profile = 0;
+		user_profile = gf_cfg_get_key(sess->dm->cfg, "Downloader", "UserProfileID");
+		if (user_profile) {
+			strcat(sHTTP, "X-UserProfileID: ");
+			strcat(sHTTP, user_profile);
+			strcat(sHTTP, "\r\n");
+		} else {
+			user_profile = gf_cfg_get_key(sess->dm->cfg, "Downloader", "UserProfile");
+			if (user_profile) {
+				FILE *profile = fopen(user_profile, "rt");
+				if (profile) {
+					fseek(profile, 0, SEEK_END);
+					par.size = ftell(profile);
+					fclose(profile);
+					sprintf(range_buf, "Content-Length: %d\r\n", par.size);
+					strcat(sHTTP, range_buf);
+					strcat(sHTTP, "Content-Type: text/xml\r\n");
+					send_profile = 1;
+				}
+			}
+		}
+
+
+		if (!send_profile) {
+			gf_dm_sess_user_io(sess, &par);
+			if (par.data && par.size) {
+				sprintf(range_buf, "Content-Length: %d\r\n", par.size);
+				strcat(sHTTP, range_buf);
+			}
 		}
 		strcat(sHTTP, "\r\n");
 
+		if (send_profile || par.data) {
+			u32 len = strlen(sHTTP);
+			char *tmp_buf = malloc(sizeof(char)*(len+par.size));
+			strcpy(tmp_buf, sHTTP);
+			if (par.data) {
+				memcpy(tmp_buf+len, par.data, par.size);
+			} else {
+				FILE *profile;
+				user_profile = gf_cfg_get_key(sess->dm->cfg, "Downloader", "UserProfile");
+				assert (user_profile);
+				profile = fopen(user_profile, "rt");
+				fread(tmp_buf+len, 1, par.size, profile);
+				fclose(profile);
+			}
+
 #ifdef GPAC_HAS_SSL
-		if (sess->ssl) {
-			e = GF_IP_NETWORK_FAILURE;
-			if (!SSL_write(sess->ssl, sHTTP, strlen(sHTTP))) e = GF_OK;
-		} else 
+			if (sess->ssl) {
+				e = GF_IP_NETWORK_FAILURE;
+				if (!SSL_write(sess->ssl, tmp_buf, len+par.size)) e = GF_OK;
+			} else 
 #endif
-			e = gf_sk_send(sess->sock, sHTTP, strlen(sHTTP));
+				e = gf_sk_send(sess->sock, tmp_buf, len+par.size);
+
+			GF_LOG(GF_LOG_DEBUG, GF_LOG_NETWORK, ("[HTTP] %s\n\n", tmp_buf));
+			free(tmp_buf);
+		} else {
+
+#ifdef GPAC_HAS_SSL
+			if (sess->ssl) {
+				e = GF_IP_NETWORK_FAILURE;
+				if (!SSL_write(sess->ssl, sHTTP, strlen(sHTTP))) e = GF_OK;
+			} else 
+#endif
+				e = gf_sk_send(sess->sock, sHTTP, strlen(sHTTP));
+
+			GF_LOG(GF_LOG_DEBUG, GF_LOG_NETWORK, ("[HTTP] %s\n\n", sHTTP));
+		}
 
-		GF_LOG(GF_LOG_DEBUG, GF_LOG_NETWORK, ("[HTTP] %s\n\n", sHTTP));
 		if (e) {
 			sess->status = GF_NETIO_STATE_ERROR;
 			sess->last_error = e;
@@ -1086,26 +1192,6 @@ void http_do_requests(GF_DownloadSession *sess)
 			return;
 		}
 
-		if (par.size && par.data) {
-			u32 done = 0;
-			while (done<par.size) {
-			
-#ifdef GPAC_HAS_SSL
-				if (sess->ssl) {
-					e = GF_IP_NETWORK_FAILURE;
-					if (!SSL_write(sess->ssl, par.data+done, par.size-done)) e = GF_OK;
-				} else 
-#endif
-					e = gf_sk_send(sess->sock, par.data+done, par.size-done);
-
-				if (e) {
-					sess->status = GF_NETIO_STATE_ERROR;
-					sess->last_error = e;
-					gf_dm_sess_notify_state(sess, GF_NETIO_STATE_ERROR, e);
-					return;
-				}
-			}
-		}
 		sess->status = GF_NETIO_WAIT_FOR_REPLY;
 		gf_dm_sess_notify_state(sess, GF_NETIO_WAIT_FOR_REPLY, GF_OK);
 		return;
@@ -1168,7 +1254,7 @@ void http_do_requests(GF_DownloadSession *sess)
 		is_ice = 0;
 		if (!strncmp("ICY", comp, 4)) {
 			is_ice = 1;
-			/*be prepared not to recieve any mime type from ShoutCast servers*/
+			/*be prepared not to receive any mime type from ShoutCast servers*/
 			sess->mime_type = strdup("audio/mpeg");
 		} else if ((strncmp("HTTP", comp, 4) != 0)) {
 			e = GF_REMOTE_SERVICE_ERROR;
@@ -1186,7 +1272,7 @@ void http_do_requests(GF_DownloadSession *sess)
 		//parse header
 		while (1) {
 			char *sep, *hdr_sep;
-			if ( (u32) LinePos + 4 > BodyStart) break;
+			if ( (s32) LinePos + 4 > BodyStart) break;
 			LinePos = gf_token_get_line(sHTTP, LinePos , bytesRead, buf, 1024);
 			if (LinePos < 0) break;
 
@@ -1206,6 +1292,7 @@ void http_do_requests(GF_DownloadSession *sess)
 			par.msg_type = GF_NETIO_PARSE_HEADER;
 			par.name = hdr;
 			par.value = hdr_val;
+			GF_LOG(GF_LOG_INFO, GF_LOG_NETWORK, ("[HTTP] Processing header %s: %s\n", hdr, hdr_val));
 			gf_dm_sess_user_io(sess, &par);
 
 			if (!stricmp(hdr, "Content-Length") ) ContentLength = (u32) atoi(hdr_val);
@@ -1246,6 +1333,11 @@ void http_do_requests(GF_DownloadSession *sess)
 				sess->icy_metaint = atoi(hdr_val);
 			else if (!stricmp(hdr, "ice") || !stricmp(hdr, "icy") ) 
 				is_ice = 1;
+			else if (!stricmp(hdr, "X-UserProfileID") ) 
+				gf_cfg_set_key(sess->dm->cfg, "Downloader", "UserProfileID", hdr_val);
+/*			else if (!stricmp(hdr, "Connection") ) 
+				if (strstr(hdr_val, "close")) sess->http_read_type = 1; */
+
 
 			if (sep) sep[0]=':';
 			if (hdr_sep) hdr_sep[0] = '\r';
@@ -1331,6 +1423,8 @@ void http_do_requests(GF_DownloadSession *sess)
 
 		if (!ContentLength && sess->mime_type && strstr(sess->mime_type, "ogg")) is_ice = 1;
 
+		GF_LOG(GF_LOG_INFO, GF_LOG_NETWORK, ("[HTTP] Connected to %s - error %s\n", sess->server_name, gf_error_to_string(e) ));
+
 		/*some servers may reply without content length, but we MUST have it*/
 //		if (!is_ice && !ContentLength) e = GF_REMOTE_SERVICE_ERROR;
 		if (e) goto exit;
@@ -1344,7 +1438,6 @@ void http_do_requests(GF_DownloadSession *sess)
 			}
 		}
 
-
 		/*done*/
 		if (sess->cache_start_size 
 			&& ( (total_size && sess->cache_start_size >= total_size) || (sess->cache_start_size == ContentLength)) ) {
@@ -1365,7 +1458,7 @@ void http_do_requests(GF_DownloadSession *sess)
 			sess->status = GF_NETIO_DISCONNECTED;
 			sess->http_read_type = 0;
 		}
-		/*no range header, Accep-Ranges deny or dumb server : restart*/
+		/*no range header, Accept-Ranges deny or dumb server : restart*/
 		else if (!range || !first_byte || (first_byte != sess->cache_start_size) ) {
 			sess->cache_start_size = sess->bytes_done = 0;
 			sess->total_size = ContentLength;
@@ -1397,18 +1490,19 @@ void http_do_requests(GF_DownloadSession *sess)
 
 
 		//we may have existing data in this buffer ...
-		if (!e && (BodyStart < (u32) bytesRead)) {
-			gf_dm_data_recieved(sess, sHTTP + BodyStart, bytesRead - BodyStart);
+		if (!e && (BodyStart < (s32) bytesRead)) {
+			gf_dm_data_received(sess, sHTTP + BodyStart, bytesRead - BodyStart);
 			/*store data if no callbacks or cache*/
-			if (sess->flags & GF_NETIO_SESSION_NOT_CACHED) {
+//			if (sess->flags & GF_NETIO_SESSION_NOT_CACHED) {
 				if (sess->init_data) free(sess->init_data);
 				sess->init_data_size = bytesRead - BodyStart;
 				sess->init_data = (char *) malloc(sizeof(char) * sess->init_data_size);
 				memcpy(sess->init_data, sHTTP+BodyStart, sess->init_data_size);
-			}
+//			}
 		}
 exit:
 		if (e) {
+			GF_LOG(GF_LOG_ERROR, GF_LOG_NETWORK, ("[HTTP] Error parsing reply: %s\n", gf_error_to_string(e) ));
 			gf_dm_disconnect(sess);
 			sess->status = GF_NETIO_STATE_ERROR;
 			sess->last_error = e;
@@ -1430,8 +1524,7 @@ exit:
 		}
 #endif
 		e = gf_dm_read_data(sess, sHTTP, GF_DOWNLOAD_BUFFER_SIZE, &size);
-		if (!size || e == GF_IP_NETWORK_EMPTY) {
-		
+		if (!size || e == GF_IP_NETWORK_EMPTY) {	
 			if (!sess->total_size && (gf_sys_clock() - sess->window_start > 1000)) {
 				sess->total_size = sess->bytes_done;
 				gf_dm_sess_notify_state(sess, GF_NETIO_DATA_TRANSFERED, GF_OK);
@@ -1445,7 +1538,7 @@ exit:
 			gf_dm_sess_notify_state(sess, sess->status, e);
 			return;
 		}
-		gf_dm_data_recieved(sess, sHTTP, size);
+		gf_dm_data_received(sess, sHTTP, size);
 		/*socket empty*/
 		if (size < GF_DOWNLOAD_BUFFER_SIZE) return;
 	}
