@@ -23,7 +23,7 @@
  */
 
 #include "module_wrap.h"
-#include <gpac/config.h>
+#include <gpac/config_file.h>
 
 GF_EXPORT
 GF_ModuleManager *gf_modules_new(const char *directory, GF_Config *config)
@@ -52,7 +52,7 @@ GF_EXPORT
 void gf_modules_del(GF_ModuleManager *pm)
 {
 	ModuleInstance *inst;
-
+	if (!pm) return;
 	/*unload all modules*/
 	while (gf_list_count(pm->plug_list)) {
 		inst = (ModuleInstance *) gf_list_get(pm->plug_list, 0);
@@ -176,6 +176,13 @@ GF_Err gf_modules_set_option(GF_BaseInterface *ifce, const char *secName, const
 	return gf_cfg_set_key(cfg, secName, keyName, keyValue);
 }
 
+GF_EXPORT
+GF_Config *gf_modules_get_config(GF_BaseInterface *ifce)
+{
+	if (!ifce || !ifce->HPLUG) return NULL;
+	return ((ModuleInstance *)ifce->HPLUG)->plugman->cfg;
+}
+
 GF_EXPORT
 const char *gf_modules_get_file_name(GF_ModuleManager *pm, u32 i)
 {
