@@ -29,6 +29,10 @@
 #include <winsock.h>
 #include <tlhelp32.h>
 
+#if !defined(__GNUC__)
+#pragma comment(lib, "toolhelp")
+#endif
+
 #elif defined(WIN32)
 
 #include <time.h>
@@ -37,6 +41,10 @@
 #include <windows.h>
 #include <tlhelp32.h>
 
+#if !defined(__GNUC__)
+#pragma comment(lib, "winmm")
+#endif
+
 #else
 
 #include <time.h>
@@ -155,10 +163,15 @@ s32 gettimeofday(struct timeval *tp, void *tz)
 
 #ifdef _WIN32_WCE
 
-void CE_Assert(u32 valid)
+void CE_Assert(u32 valid, char *file, u32 line)
 {
 	if (!valid) {
-		MessageBox(NULL, _T("ASSERT FAILED"), _T("Fatal Error"), MB_OK);
+		char szBuf[2048];
+		u16 wcBuf[2048];
+		sprintf(szBuf, "File %s : line %d", file, line);
+		CE_CharToWide(szBuf, wcBuf);
+		MessageBox(NULL, wcBuf, _T("GPAC Assertion Failure"), MB_OK);
+		exit(EXIT_FAILURE);
 	}
 }
 
@@ -224,7 +237,14 @@ FILE *gf_temp_file_new()
 
 	return NULL;
 #elif defined(WIN32)
-	return tmpfile();
+	char tmp[MAX_PATH], t_file[100];
+	FILE *res = tmpfile();
+	if (res) return res;
+	/*tmpfile() may fail under vista ...*/
+	if (!GetEnvironmentVariable("TEMP",tmp,MAX_PATH)) return NULL;
+	sprintf(t_file, "\\gpac_%08x.tmp", (u32) tmp);
+	strcat(tmp, t_file);
+	return fopen(tmp, "w+b");
 #else
 	return tmpfile(); 
 #endif
@@ -274,15 +294,15 @@ void gf_get_user_name(char *buf, u32 buf_size)
 /*enumerate directories*/
 GF_Err gf_enum_directory(const char *dir, Bool enum_directory, gf_enum_dir_item enum_dir_fct, void *cbck, const char *filter)
 {
-	unsigned char item_path[GF_MAX_PATH];
+	char item_path[GF_MAX_PATH];
 
 #if defined(_WIN32_WCE)
-	unsigned char _path[GF_MAX_PATH];
+	char _path[GF_MAX_PATH];
 	unsigned short path[GF_MAX_PATH];
 	unsigned short w_filter[GF_MAX_PATH];
-	unsigned char file[GF_MAX_PATH];
+	char file[GF_MAX_PATH];
 #else
-	unsigned char path[GF_MAX_PATH], *file;
+	char path[GF_MAX_PATH], *file;
 #endif
 
 #ifdef WIN32
@@ -450,9 +470,9 @@ u64 gf_f64_tell(FILE *fp)
 		return (u64) -1;
 	else
 		return ((u64) pos);
-#elif defined(CONFIG_LINUX)
+#elif defined(GPAC_CONFIG_LINUX)
 	return (u64) ftello64(fp);
-#elif defined(CONFIG_FREEBSD)
+#elif (defined(GPAC_CONFIG_FREEBSD) || defined(GPAC_CONFIG_DARWIN))
 	return (u64) ftell(fp);
 #else
 	return (u64) ftell(fp);
@@ -474,9 +494,9 @@ u64 gf_f64_seek(FILE *fp, s64 offset, s32 whence)
 	else if (whence == SEEK_SET)
 		pos = (fpos_t) offset;
 	return fsetpos(fp, &pos);
-#elif defined(CONFIG_LINUX)
+#elif defined(GPAC_CONFIG_LINUX)
 	return fseeko64(fp, (off64_t) offset, whence);
-#elif defined(CONFIG_FREEBSD)
+#elif (defined(GPAC_CONFIG_FREEBSD) || defined(GPAC_CONFIG_DARWIN))
 	return fseek(fp, offset, whence);
 #else
 	return fseek(fp, (s32) offset, whence);
@@ -487,9 +507,9 @@ FILE *gf_f64_open(const char *file_name, const char *mode)
 {
 #if defined(WIN32)
 	return fopen(file_name, mode);
-#elif defined(CONFIG_LINUX)
+#elif defined(GPAC_CONFIG_LINUX)
 	return fopen64(file_name, mode);
-#elif defined(CONFIG_FREEBSD)
+#elif (defined(GPAC_CONFIG_FREEBSD) || defined(GPAC_CONFIG_DARWIN))
 	return fopen(file_name, mode);
 #else
 	return fopen(file_name, mode);
@@ -835,7 +855,6 @@ Bool gf_sys_get_rti(u32 refresh_time_ms, GF_SystemRTInfo *rti, u32 flags)
 		rti->physical_memory = ms.dwTotalPhys;
 		rti->physical_memory_avail = ms.dwAvailPhys;
 		rti->gpac_memory = (u64) gpac_allocated_memory;
-//		fprintf(stdout, "Total memory (nbblocs*4+memory) %d\n", 4*gpac_nb_alloc_blocs+gpac_allocated_memory);
 		return 1;
 	}
 
@@ -847,24 +866,25 @@ Bool gf_sys_get_rti(u32 refresh_time_ms, GF_SystemRTInfo *rti, u32 flags)
 	orig_perm = GetCurrentPermissions();
 	SetProcPermissions(0xFFFFFFFF);
 	hSnapShot = CreateToolhelp32Snapshot(TH32CS_SNAPTHREAD, 0); 
-	if (!hSnapShot) return 0;
-	tentry.dwSize = sizeof(THREADENTRY32); 
-	the_rti.thread_count = 0;
-	/*note we always act as if GF_RTI_ALL_PROCESSES_TIMES flag is set, since there is no other way
-	to enumerate threads from a process, and GetProcessTimes doesn't exist on CE*/
-	if (Thread32First(hSnapShot, &tentry)) {
-		do {
-			/*get thread times*/
-			if (GetThreadTimes( (HANDLE) tentry.th32ThreadID, (FILETIME *) &creation, (FILETIME *) &exit, (FILETIME *) &kernel, (FILETIME *) &user)) {
-				total_cpu_time += user + kernel;
-				if (tentry.th32OwnerProcessID==the_rti.pid) {
-					process_cpu_time += user + kernel;
-					the_rti.thread_count ++;
+	if (hSnapShot) {
+		tentry.dwSize = sizeof(THREADENTRY32); 
+		the_rti.thread_count = 0;
+		/*note we always act as if GF_RTI_ALL_PROCESSES_TIMES flag is set, since there is no other way
+		to enumerate threads from a process, and GetProcessTimes doesn't exist on CE*/
+		if (Thread32First(hSnapShot, &tentry)) {
+			do {
+				/*get thread times*/
+				if (GetThreadTimes( (HANDLE) tentry.th32ThreadID, (FILETIME *) &creation, (FILETIME *) &exit, (FILETIME *) &kernel, (FILETIME *) &user)) {
+					total_cpu_time += user + kernel;
+					if (tentry.th32OwnerProcessID==the_rti.pid) {
+						process_cpu_time += user + kernel;
+						the_rti.thread_count ++;
+					}
 				}
-			}
-		} while (Thread32Next(hSnapShot, &tentry));
+			} while (Thread32Next(hSnapShot, &tentry));
+		}
+		CloseToolhelp32Snapshot(hSnapShot); 
 	}
-	CloseToolhelp32Snapshot(hSnapShot); 
 
 	if (flags & GF_RTI_PROCESS_MEMORY) {
 		HEAPLIST32 hlentry;
@@ -1038,6 +1058,8 @@ Bool gf_sys_get_rti(u32 refresh_time_ms, GF_SystemRTInfo *rti, u32 flags)
 	last_proc_k_u_time = proc_k_u_time;
 #endif
 
+	if (!the_rti.gpac_memory) the_rti.gpac_memory = the_rti.process_memory;
+
 	memcpy(rti, &the_rti, sizeof(GF_SystemRTInfo));
 	return 1;
 }
@@ -1204,15 +1226,14 @@ Bool gf_sys_get_rti(u32 refresh_time_ms, GF_SystemRTInfo *rti, u32 flags)
 #endif
 
 
-Bool gf_sys_get_battery_state(Bool *onBattery, u32 *state, u32*level) 
+Bool gf_sys_get_battery_state(Bool *onBattery, u32 *onCharge, u32*level) 
 {
 #if defined(WIN32) && !defined(_WIN32_WCE)
 	SYSTEM_POWER_STATUS sps;
-	if (!onBattery || !state || !level) return 0;
 	GetSystemPowerStatus(&sps);
-	*onBattery = sps.ACLineStatus;
-	*state = sps.BatteryFlag;
-	*level = sps.BatteryLifePercent;
+	if (onBattery) *onBattery = sps.ACLineStatus;
+	if (onCharge) *onCharge = (sps.BatteryFlag & BATTERY_FLAG_CHARGING) ? 1 : 0;
+	if (level) *level = sps.BatteryLifePercent;
 #endif
 	return 1;
 }
