@@ -27,21 +27,32 @@
 
 #ifdef _WIN32_WCE
 #include <winsock.h>
+
+#if !defined(__GNUC__)
+#pragma comment(lib, "winsock")
+#endif
+
 #else
+
 #include <sys/timeb.h>
 #include <winsock2.h>
 #include <ws2tcpip.h>
+
+#if !defined(__GNUC__)
+#pragma comment(lib, "ws2_32")
+#endif
+
 #endif
 
 #include <windows.h>
 
 #if !defined(__GNUC__)
 
-#if 0 && defined(IPV6_MULTICAST_IF)
-#define GPAC_IPV6 1
+#if defined(IPV6_MULTICAST_IF___dis)
+#define GPAC_HAS_IPV6 1
 #pragma message("Using WinSock IPV6")
 #else
-#undef GPAC_IPV6
+#undef GPAC_HAS_IPV6
 #pragma message("Using WinSock IPV4")
 #endif
 
@@ -82,10 +93,21 @@ static int wsa_init = 0;
 #include <netinet/in.h>
 #include <netinet/tcp.h>
 #include <sys/socket.h>
+#include <sys/types.h>
 #include <arpa/inet.h>
 
 #include <gpac/network.h>
 
+/*not defined on solaris*/
+#if !defined(INADDR_NONE)
+# if (defined(sun) && defined(__SVR4))
+#  define INADDR_NONE -1
+# else
+#  define INADDR_NONE ((unsigned long)-1)
+# endif
+#endif
+
+
 #define INVALID_SOCKET -1
 #define SOCKET_ERROR -1
 #define LASTSOCKERROR errno
@@ -104,7 +126,7 @@ typedef s32 SOCKET;
 
 #define SOCK_MICROSEC_WAIT	500
 
-#ifdef GPAC_IPV6
+#ifdef GPAC_HAS_IPV6
 static u32 ipv6_check_state = 0;
 #endif
 
@@ -117,7 +139,8 @@ enum
 	GF_SOCK_IS_MULTICAST = 1<<12,
 	GF_SOCK_IS_LISTENING = 1<<13,
 	/*socket is bound to a specific dest (server) or source (client) */
-	GF_SOCK_HAS_PEER = 1<<14
+	GF_SOCK_HAS_PEER = 1<<14,
+	GF_SOCK_IS_MIP = 1<<15
 };
 
 struct __tag_socket
@@ -125,7 +148,7 @@ struct __tag_socket
 	u32 flags;
 	SOCKET socket;
 	/*destination address for sendto/recvfrom*/
-#ifdef GPAC_IPV6
+#ifdef GPAC_HAS_IPV6
 	struct sockaddr_storage dest_addr;
 #else
 	struct sockaddr_in dest_addr;
@@ -133,10 +156,32 @@ struct __tag_socket
 	u32 dest_addr_len;
 };
 
+
+
 /*
-		Some NTP tools
+	MobileIP tools
 */
 
+static gf_net_mobileip_ctrl_cbk mobip_cbk = NULL;
+static const char *MobileIPAdd = NULL;
+
+GF_EXPORT
+void gf_net_mobileip_set_callback(gf_net_mobileip_ctrl_cbk _mobip_cbk, const char *mip)
+{
+	mobip_cbk = _mobip_cbk;
+	MobileIPAdd = _mobip_cbk ? mip : NULL;
+}
+
+static GF_Err gf_net_mobileip_ctrl(Bool start)
+{
+	if (mobip_cbk) return mobip_cbk(start);
+	return GF_NOT_SUPPORTED;
+}
+
+
+/*
+		NTP tools
+*/
 void gf_net_get_ntp(u32 *sec, u32 *frac)
 {
 	struct timeval now;
@@ -150,7 +195,7 @@ void gf_net_get_ntp(u32 *sec, u32 *frac)
 
 u32 gf_net_has_ipv6()
 {
-#ifdef GPAC_IPV6
+#ifdef GPAC_HAS_IPV6
 	if (!ipv6_check_state) {
 		SOCKET s;
 #ifdef WIN32
@@ -188,7 +233,7 @@ Bool gf_net_is_ipv6(char *address)
 	return sep ? 1 : 0;
 }
 
-#ifdef GPAC_IPV6
+#ifdef GPAC_HAS_IPV6
 static struct addrinfo *gf_sk_get_ipv6_addr(char *PeerName, u16 PortNumber, int family, int flags, int sock_type)
 {
 	struct	addrinfo *res=NULL;
@@ -245,7 +290,7 @@ GF_Err gf_sk_get_host_name(char *buffer)
 
 GF_Err gf_sk_get_local_ip(GF_Socket *sock, char *buffer)
 {
-#ifdef GPAC_IPV6
+#ifdef GPAC_HAS_IPV6
 	char clienthost[NI_MAXHOST];
 	if (sock->flags & GF_SOCK_HAS_PEER) {
 		if (getnameinfo((struct sockaddr *)&sock->dest_addr, sock->dest_addr_len, clienthost, sizeof(clienthost), NULL, 0, NI_NUMERICHOST)) 
@@ -275,6 +320,7 @@ GF_Err gf_sk_get_local_ip(GF_Socket *sock, char *buffer)
 	return GF_OK;
 }
 
+
 GF_Socket *gf_sk_new(u32 SocketType)
 {
 	GF_Socket *tmp;
@@ -290,7 +336,7 @@ GF_Socket *gf_sk_new(u32 SocketType)
 	if (!tmp) return NULL;
 	if (SocketType == GF_SOCK_TYPE_TCP) tmp->flags |= GF_SOCK_IS_TCP;
 
-#ifdef GPAC_IPV6
+#ifdef GPAC_HAS_IPV6
 	memset(&tmp->dest_addr, 0, sizeof(struct sockaddr_storage));
 #else
 	memset(&tmp->dest_addr, 0, sizeof(struct sockaddr_in));
@@ -319,13 +365,17 @@ GF_Err gf_sk_set_block_mode(GF_Socket *sock, u32 NonBlockingOn)
 {
 	s32 res;
 #ifdef WIN32
-	u_long val = NonBlockingOn;
-	res = ioctlsocket(sock->socket, FIONBIO, &val);
-	if (res) return GF_SERVICE_ERROR;
+	long val = NonBlockingOn;
+	if (sock->socket) {
+		res = ioctlsocket(sock->socket, FIONBIO, &val);
+		if (res) return GF_SERVICE_ERROR;
+	}
 #else
 	s32 flag = fcntl(sock->socket, F_GETFL, 0);
-	res = fcntl(sock->socket, F_SETFL, flag | O_NONBLOCK);
-	if (res) return GF_SERVICE_ERROR;
+	if (sock->socket) {
+		res = fcntl(sock->socket, F_SETFL, flag | O_NONBLOCK);
+		if (res) return GF_SERVICE_ERROR;
+	}
 #endif
 	if (NonBlockingOn) {
 		sock->flags |= GF_SOCK_NON_BLOCKING;
@@ -341,7 +391,7 @@ static void gf_sk_free(GF_Socket *sock)
 	/*leave multicast*/
 	if (sock->socket && (sock->flags & GF_SOCK_IS_MULTICAST) ) {
 		struct ip_mreq mreq;
-#ifdef GPAC_IPV6
+#ifdef GPAC_HAS_IPV6
 		struct sockaddr *addr = (struct sockaddr *)&sock->dest_addr;
 		if (addr->sa_family==AF_INET6) {
 		    struct ipv6_mreq mreq6; 
@@ -361,6 +411,12 @@ static void gf_sk_free(GF_Socket *sock)
 	}
 	if (sock->socket) closesocket(sock->socket);
 	sock->socket = (SOCKET) 0L;
+
+	/*if MobileIP socket, unregister it*/
+	if (sock->flags & GF_SOCK_IS_MIP) {
+		sock->flags &= ~GF_SOCK_IS_MIP;
+		gf_net_mobileip_ctrl(0);
+	}
 }
 
 void gf_sk_del(GF_Socket *sock)
@@ -387,18 +443,36 @@ s32 gf_sk_get_handle(GF_Socket *sock)
 
 
 //connects a socket to a remote peer on a given port
-GF_Err gf_sk_connect(GF_Socket *sock, char *PeerName, u16 PortNumber)
+GF_Err gf_sk_connect(GF_Socket *sock, char *PeerName, u16 PortNumber, char *local_ip)
 {
 	s32 ret;
-#ifdef GPAC_IPV6
+#ifdef GPAC_HAS_IPV6
 	u32 type = (sock->flags & GF_SOCK_IS_TCP) ? SOCK_STREAM : SOCK_DGRAM;
-	struct addrinfo *res, *aip;
+	struct addrinfo *res, *aip, *lip;
 
 	gf_sk_free(sock);
 
 	res = gf_sk_get_ipv6_addr(PeerName, PortNumber, AF_UNSPEC, AI_PASSIVE, type);
 	if (!res) return GF_IP_CONNECTION_FAILURE;
 
+	/*turn on MobileIP*/
+	if (local_ip && MobileIPAdd && !strcmp(MobileIPAdd, local_ip) ) {
+		if (gf_net_mobileip_ctrl(1)==GF_OK) {
+			sock->flags |= GF_SOCK_IS_MIP;
+		} else {
+			local_ip = NULL;
+		}
+	}
+
+	lip = NULL;
+	if (local_ip) {
+		lip = gf_sk_get_ipv6_addr(local_ip, PortNumber, AF_UNSPEC, AI_PASSIVE, type);
+		if (!lip && local_ip) {
+			lip = gf_sk_get_ipv6_addr(NULL, PortNumber, AF_UNSPEC, AI_PASSIVE, type);
+			local_ip = NULL;
+		}
+	}
+
 	/*for all interfaces*/
 	for (aip=res; aip!=NULL; aip=aip->ai_next) {
 		if (type != (u32) aip->ai_socktype) continue;
@@ -411,6 +485,15 @@ GF_Err gf_sk_connect(GF_Socket *sock, char *PeerName, u16 PortNumber)
 		if (aip->ai_family==PF_INET6) sock->flags |= GF_SOCK_IS_IPV6;
 		else sock->flags &= ~GF_SOCK_IS_IPV6;
 
+		if (lip) {
+			ret = bind(sock->socket, lip->ai_addr, lip->ai_addrlen);
+			if (ret == SOCKET_ERROR) {
+				closesocket(sock->socket);
+				sock->socket = (SOCKET)NULL;
+				continue;
+			}
+		}
+
 		ret = connect(sock->socket, aip->ai_addr, aip->ai_addrlen);
 		if (ret == SOCKET_ERROR) {
 			closesocket(sock->socket);
@@ -421,16 +504,26 @@ GF_Err gf_sk_connect(GF_Socket *sock, char *PeerName, u16 PortNumber)
 		memcpy(&sock->dest_addr, aip->ai_addr, aip->ai_addrlen);
 		sock->dest_addr_len = aip->ai_addrlen;
 		freeaddrinfo(res);
+		if (lip) freeaddrinfo(lip);
 		return GF_OK;
 	}
 	freeaddrinfo(res);
+	if (lip) freeaddrinfo(lip);
 	return GF_IP_CONNECTION_FAILURE;
 
 #else	
 	struct hostent *Host;
 	
-	if (!sock->socket) 
+	if (local_ip) {
+		/*this will turn on MobileIP if needed*/
+		GF_Err e = gf_sk_bind(sock, local_ip, PortNumber, PeerName, PortNumber, GF_SOCK_REUSE_PORT);
+		if (e) return e;
+	}
+	if (!sock->socket) {
 		sock->socket = socket(AF_INET, (sock->flags & GF_SOCK_IS_TCP) ? SOCK_STREAM : SOCK_DGRAM, 0);
+		if (sock->flags & GF_SOCK_NON_BLOCKING)		
+			gf_sk_set_block_mode(sock, 1);
+	}
 
 	/*setup the address*/
 	sock->dest_addr.sin_family = AF_INET;
@@ -454,9 +547,21 @@ GF_Err gf_sk_connect(GF_Socket *sock, char *PeerName, u16 PortNumber)
 	if (sock->flags & GF_SOCK_IS_TCP) {
 		ret = connect(sock->socket, (struct sockaddr *) &sock->dest_addr, sizeof(struct sockaddr));
 		if (ret == SOCKET_ERROR) {
-			switch (LASTSOCKERROR) {
+			u32 res = LASTSOCKERROR;
+			GF_LOG(GF_LOG_CORE, GF_LOG_ERROR, ("[Core] Couldn't connect socket - last sock error %d\n", res));
+			switch (res) {
 			case EAGAIN: return GF_IP_SOCK_WOULD_BLOCK;
+#ifdef WIN32
+			case WSAEINVAL: 
+				if (sock->flags & GF_SOCK_NON_BLOCKING) 
+					return GF_IP_SOCK_WOULD_BLOCK;
+#endif
 			case EISCONN: return GF_OK;
+			case ENOTCONN: return GF_IP_CONNECTION_FAILURE;
+			case ECONNRESET: return GF_IP_CONNECTION_FAILURE;
+			case EMSGSIZE: return GF_IP_CONNECTION_FAILURE;
+			case ECONNABORTED: return GF_IP_CONNECTION_FAILURE;
+			case ENETDOWN: return GF_IP_CONNECTION_FAILURE;
 			default: return GF_IP_CONNECTION_FAILURE;
 			}
 		}
@@ -468,13 +573,14 @@ GF_Err gf_sk_connect(GF_Socket *sock, char *PeerName, u16 PortNumber)
 
 //binds the given socket to the specified port. If ReUse is true
 //this will enable reuse of ports on a single machine
-GF_Err gf_sk_bind(GF_Socket *sock, u16 port, char *peer_name, u16 peer_port, u32 options)
+GF_Err gf_sk_bind(GF_Socket *sock, char *local_ip, u16 port, char *peer_name, u16 peer_port, u32 options)
 {
-#ifdef GPAC_IPV6
+#ifdef GPAC_HAS_IPV6
 	struct addrinfo *res, *aip;
 	int af;
 	u32 type;
 #else
+	u32 ip_add;
 	size_t addrlen;
 	struct sockaddr_in LocalAdd;
 	struct hostent *Host;
@@ -485,7 +591,7 @@ GF_Err gf_sk_bind(GF_Socket *sock, u16 port, char *peer_name, u16 peer_port, u32
 
 	if (!sock || sock->socket) return GF_BAD_PARAM;
 
-#ifdef GPAC_IPV6
+#ifdef GPAC_HAS_IPV6
 	type = (sock->flags & GF_SOCK_IS_TCP) ? SOCK_STREAM : SOCK_DGRAM;
 	af = (options & GF_SOCK_FORCE_IPV6) ? PF_INET6 : PF_UNSPEC;
 	if (!gf_net_has_ipv6()) af = PF_INET;
@@ -503,8 +609,23 @@ GF_Err gf_sk_bind(GF_Socket *sock, u16 port, char *peer_name, u16 peer_port, u32
 		freeaddrinfo(res);
 	}
 	
-	res = gf_sk_get_ipv6_addr(NULL, port, af, AI_PASSIVE, type);
-	if (!res) return GF_IP_CONNECTION_FAILURE;
+	/*turn on MobileIP*/
+	if (local_ip && MobileIPAdd && !strcmp(MobileIPAdd, local_ip) ) {
+		if (gf_net_mobileip_ctrl(1)==GF_OK) {
+			sock->flags |= GF_SOCK_IS_MIP;
+		} else {
+			res = gf_sk_get_ipv6_addr(NULL, port, af, AI_PASSIVE, type);
+			local_ip = NULL;
+		}
+	}
+	res = gf_sk_get_ipv6_addr(local_ip, port, af, AI_PASSIVE, type);
+	if (!res) {
+		if (local_ip) {
+			res = gf_sk_get_ipv6_addr(NULL, port, af, AI_PASSIVE, type);
+			local_ip = NULL;
+		}
+		if (!res) return GF_IP_CONNECTION_FAILURE;
+	}
 
 	/*for all interfaces*/
 	for (aip=res; aip!=NULL; aip=aip->ai_next) {
@@ -564,10 +685,26 @@ GF_Err gf_sk_bind(GF_Socket *sock, u16 port, char *peer_name, u16 peer_port, u32
 		GF_LOG(GF_LOG_ERROR, GF_LOG_CORE, ("[socket] cannot resolve localhost name - socket error %x\n", LASTSOCKERROR));
 		return GF_IP_ADDRESS_NOT_FOUND;
 	}
+
+	/*turn on MobileIP*/
+	if (local_ip && MobileIPAdd && !strcmp(MobileIPAdd, local_ip) ) {
+		if (gf_net_mobileip_ctrl(1)==GF_OK) {
+			sock->flags |= GF_SOCK_IS_MIP;
+		} else {
+			local_ip = NULL;
+		}
+	}
+
 	/*setup the address*/
+	ip_add = 0;
+	if (local_ip) ip_add = inet_addr(local_ip);
+	if (!ip_add) {
+		ip_add = htonl(INADDR_ANY);
+		local_ip = NULL;
+	}
 	memcpy((char *) &LocalAdd.sin_addr, Host->h_addr_list[0], sizeof(LocalAdd.sin_addr));
 	LocalAdd.sin_family = AF_INET;
-	LocalAdd.sin_addr.s_addr = INADDR_ANY;
+	LocalAdd.sin_addr.s_addr = ip_add;
 	LocalAdd.sin_port = htons(port);
 	addrlen = sizeof(struct sockaddr_in);
 	if (options & GF_SOCK_REUSE_PORT) {
@@ -672,7 +809,7 @@ GF_Err gf_sk_send(GF_Socket *sock, char *buffer, u32 length)
 
 u32 gf_sk_is_multicast_address(char *multi_IPAdd)
 {
-#ifdef GPAC_IPV6
+#ifdef GPAC_HAS_IPV6
 	u32 val;
  	char *sep;
 	struct addrinfo *res;
@@ -704,12 +841,12 @@ GF_Err gf_sk_setup_multicast(GF_Socket *sock, char *multi_IPAdd, u16 MultiPortNu
 	u32 flag; 
 	struct ip_mreq M_req;
 	u32 optval;
-#ifdef GPAC_IPV6
+#ifdef GPAC_HAS_IPV6
 	struct sockaddr *addr;
 	struct addrinfo *res, *aip;
 	u32 type;
 #else
-	u_long local_add_id;
+	unsigned long local_add_id;
 #endif
 
 	if (!sock || sock->socket) return GF_BAD_PARAM;
@@ -719,7 +856,17 @@ GF_Err gf_sk_setup_multicast(GF_Socket *sock, char *multi_IPAdd, u16 MultiPortNu
 	/*check the address*/
 	if (!gf_sk_is_multicast_address(multi_IPAdd)) return GF_BAD_PARAM;
 
-#ifdef GPAC_IPV6
+	/*turn on MobileIP*/
+	if (local_interface_ip && MobileIPAdd && !strcmp(MobileIPAdd, local_interface_ip) ) {
+		if (gf_net_mobileip_ctrl(1)==GF_OK) {
+			sock->flags |= GF_SOCK_IS_MIP;
+		} else {
+			local_interface_ip = NULL;
+		}
+	}
+
+
+#ifdef GPAC_HAS_IPV6
 	type = (sock->flags & GF_SOCK_IS_TCP) ? SOCK_STREAM : SOCK_DGRAM;
 	res = gf_sk_get_ipv6_addr(local_interface_ip, MultiPortNumber, AF_UNSPEC, AI_PASSIVE, type);
 	if (!res) {
@@ -888,14 +1035,8 @@ GF_Err gf_sk_receive(GF_Socket *sock, char *buffer, u32 length, u32 startFrom, u
 	e = GF_OK;
 
 	*BytesRead = 0;
-	if (!sock->socket) {
-		assert(0);
-		return GF_BAD_PARAM;
-	}
-	if (startFrom >= length) {
-		assert(0);
-		return GF_IO_ERR;
-	}
+	if (!sock->socket) return GF_BAD_PARAM;
+	if (startFrom >= length) return GF_IO_ERR;
 
 #ifndef __SYMBIAN32__
 	//can we read?
@@ -981,7 +1122,7 @@ GF_Err gf_sk_accept(GF_Socket *sock, GF_Socket **newConnection)
 	timeout.tv_usec = SOCK_MICROSEC_WAIT;
 
 	res = 0;
-	ready = select(sock->socket, &Group, NULL, NULL, &timeout);
+	ready = select(sock->socket+1, &Group, NULL, NULL, &timeout);
 	if (ready == SOCKET_ERROR) {
 		switch (LASTSOCKERROR) {
 		case EAGAIN:
@@ -993,7 +1134,7 @@ GF_Err gf_sk_accept(GF_Socket *sock, GF_Socket **newConnection)
 	if (!ready || !FD_ISSET(sock->socket, &Group)) return GF_IP_NETWORK_EMPTY;
 #endif
 
-#ifdef GPAC_IPV6
+#ifdef GPAC_HAS_IPV6
 	client_address_size = sizeof(struct sockaddr_in6);
 #else
 	client_address_size = sizeof(struct sockaddr_in);
@@ -1014,20 +1155,25 @@ GF_Err gf_sk_accept(GF_Socket *sock, GF_Socket **newConnection)
 	(*newConnection) = (GF_Socket *) malloc(sizeof(GF_Socket));
 	(*newConnection)->socket = sk;
 	(*newConnection)->flags = sock->flags & ~GF_SOCK_IS_LISTENING;
-#ifdef GPAC_IPV6
+#ifdef GPAC_HAS_IPV6
 	memcpy( &(*newConnection)->dest_addr, &sock->dest_addr, client_address_size);
 	memset(&sock->dest_addr, 0, sizeof(struct sockaddr_in6));
 #else
 	memcpy( &(*newConnection)->dest_addr, &sock->dest_addr, client_address_size);
 	memset(&sock->dest_addr, 0, sizeof(struct sockaddr_in));
 #endif
+
+#if defined(WIN32) || defined(_WIN32_WCE)
+	wsa_init++;
+#endif
+
 	(*newConnection)->dest_addr_len = client_address_size;
 	return GF_OK;
 }
 
 GF_Err gf_sk_get_local_info(GF_Socket *sock, u16 *Port, u32 *Familly)
 {
-#ifdef GPAC_IPV6
+#ifdef GPAC_HAS_IPV6
 	struct sockaddr_in6 the_add;
 #else
 	struct sockaddr_in the_add;
@@ -1037,7 +1183,7 @@ GF_Err gf_sk_get_local_info(GF_Socket *sock, u16 *Port, u32 *Familly)
 	if (!sock || !sock->socket) return GF_BAD_PARAM;
 
 	if (Port) {
-#ifdef GPAC_IPV6
+#ifdef GPAC_HAS_IPV6
 		size = sizeof(struct sockaddr_in6);
 		if (getsockname(sock->socket, (struct sockaddr *) &the_add, &size) == SOCKET_ERROR) return GF_IP_NETWORK_FAILURE;
 		*Port = (u32) ntohs(the_add.sin6_port);
@@ -1077,7 +1223,7 @@ GF_Err gf_sk_server_mode(GF_Socket *sock, Bool serverOn)
 
 GF_Err gf_sk_get_remote_address(GF_Socket *sock, char *buf)
 {
-#ifdef GPAC_IPV6
+#ifdef GPAC_HAS_IPV6
 	char clienthost[NI_MAXHOST];
  	struct sockaddr_in6 * addrptr = (struct sockaddr_in6 *)(&sock->dest_addr_len);
 	if (!sock || sock->socket) return GF_BAD_PARAM;
@@ -1098,7 +1244,7 @@ GF_Err gf_sk_get_remote_address(GF_Socket *sock, char *buf)
 GF_Err gf_sk_send_to(GF_Socket *sock, char *buffer, u32 length, char *remoteHost, u16 remotePort)
 {
 	u32 Count, Res, remote_add_len;
-#ifdef GPAC_IPV6
+#ifdef GPAC_HAS_IPV6
 	struct sockaddr_storage remote_add;
 #else
 	struct sockaddr_in remote_add;
@@ -1135,7 +1281,7 @@ GF_Err gf_sk_send_to(GF_Socket *sock, char *buffer, u32 length, char *remoteHost
 
 
 	/*setup the address*/
-#ifdef GPAC_IPV6
+#ifdef GPAC_HAS_IPV6
 	remote_add.ss_family = AF_INET6;
 	//if a remote host is specified, use it. Otherwise use the default host
 	if (remoteHost) {
@@ -1298,93 +1444,3 @@ GF_Err gf_sk_send_wait(GF_Socket *sock, char *buffer, u32 length, u32 Second )
 	return GF_OK;
 }
 
-
-#if 0
-
-//Socket Group for select(). The group is a collection of sockets ready for reading / writing
-typedef struct __tag_sock_group
-{
-	//the max time value before a select returns
-	struct timeval timeout;
-	fd_set ReadGroup;
-	fd_set WriteGroup;
-} GF_SocketGroup;
-
-
-#define GF_SOCK_GROUP_READ 0
-#define GF_SOCK_GROUP_WRITE 1
-
-GF_SocketGroup *NewSockGroup()
-{	
-	GF_SocketGroup *tmp = (GF_SocketGroup*)malloc(sizeof(GF_SocketGroup));
-	if (!tmp) return NULL;
-	FD_ZERO(&tmp->ReadGroup);
-	FD_ZERO(&tmp->WriteGroup);
-	return tmp;
-}
-
-void SKG_Delete(GF_SocketGroup *group)
-{
-	free(group);
-}
-
-void SKG_SetWatchTime(GF_SocketGroup *group, u32 DelayInS, u32 DelayInMicroS)
-{
-	group->timeout.tv_sec = DelayInS;
-	group->timeout.tv_usec = DelayInMicroS;
-}
-
-void SKG_AddSocket(GF_SocketGroup *group, GF_Socket *sock, u32 GroupType)
-{
-
-	switch (GroupType) {
-	case GF_SOCK_GROUP_READ:
-		FD_SET(sock->socket, &group->ReadGroup);
-		return;
-	case GF_SOCK_GROUP_WRITE:
-		FD_SET(sock->socket, &group->WriteGroup);
-		return;
-	default:
-		return;
-	}
-}
-
-void SKG_RemoveSocket(GF_SocketGroup *group, GF_Socket *sock, u32 GroupType)
-{
-	switch (GroupType) {
-	case GF_SOCK_GROUP_READ:
-		FD_CLR(sock->socket, &group->ReadGroup);
-		return;
-	case GF_SOCK_GROUP_WRITE:
-		FD_CLR(sock->socket, &group->WriteGroup);
-		return;
-	default:
-		return;
-	}
-}
-
-
-Bool SKG_IsSocketIN(GF_SocketGroup *group, GF_Socket *sock, u32 GroupType)
-{
-
-	switch (GroupType) {
-	case GF_SOCK_GROUP_READ:
-		if (FD_ISSET(sock->socket, &group->ReadGroup)) return 1;
-		return 0;
-	case GF_SOCK_GROUP_WRITE:
-		if (FD_ISSET(sock->socket, &group->WriteGroup)) return 1;
-		return 0;
-	default:
-		return 0;
-	}
-}
-
-u32 SKG_Select(GF_SocketGroup *group)
-{
-	u32 ready, rien = 0;
-	ready = select(rien, &group->ReadGroup, &group->WriteGroup, NULL, &group->timeout);
-	if (ready == SOCKET_ERROR) return 0;
-	return ready;
-}
-
-#endif
