@@ -83,11 +83,9 @@ void gf_path_del(GF_Path *gp)
 	free(gp);
 }
 
-#define PATH_POINT_ALLOC_STEP	10
-
-#define GF_2D_REALLOC_POINT(_gp, _nb)	\
-	if (_gp->n_alloc_points < _gp->n_points+_nb+1) {	\
-		_gp->n_alloc_points = _gp->n_points+_nb+1;	\
+#define GF_2D_REALLOC(_gp)	\
+	if (_gp->n_alloc_points < _gp->n_points+3) {	\
+		_gp->n_alloc_points = (_gp->n_alloc_points<5) ? 10 : (_gp->n_alloc_points*3/2);	\
 		_gp->points = (GF_Point2D *)realloc(_gp->points, sizeof(GF_Point2D)*(_gp->n_alloc_points));	\
 		_gp->tags = (u8 *) realloc(_gp->tags, sizeof(u8)*(_gp->n_alloc_points));	\
 	}	\
@@ -97,16 +95,18 @@ GF_EXPORT
 GF_Err gf_path_add_move_to(GF_Path *gp, Fixed x, Fixed y)
 {
 	if (!gp) return GF_BAD_PARAM;
-	if (gp->n_points && (gp->points[gp->n_points-1].x == x) && (gp->points[gp->n_points-1].y == y)) return GF_OK;
+
+#if 0
 	/*skip empty paths*/
 	if ((gp->n_contours>=2) && (gp->contours[gp->n_contours-2]+1==gp->contours[gp->n_contours-1])) {
 		gp->points[gp->n_points].x = x;
 		gp->points[gp->n_points].y = y;
 		return GF_OK;
 	}
+#endif
 
 	gp->contours = (u32 *) realloc(gp->contours, sizeof(u32)*(gp->n_contours+1));
-	GF_2D_REALLOC_POINT(gp, 1)
+	GF_2D_REALLOC(gp)
 
 	gp->points[gp->n_points].x = x;
 	gp->points[gp->n_points].y = y;
@@ -129,7 +129,7 @@ GF_Err gf_path_add_line_to(GF_Path *gp, Fixed x, Fixed y)
 	if (!gp || !gp->n_contours) return GF_BAD_PARAM;
 	/*we allow line to same point as move (seen in SVG sequences) - striking will make a point*/
 
-	GF_2D_REALLOC_POINT(gp, 1)
+	GF_2D_REALLOC(gp)
 	gp->points[gp->n_points].x = x;
 	gp->points[gp->n_points].y = y;
 	gp->tags[gp->n_points] = 1;
@@ -156,7 +156,7 @@ GF_Err gf_path_close(GF_Path *gp)
 	end.x -= start.x;
 	end.y -= start.y;
 	diff = gf_mulfix(end.x, end.x) + gf_mulfix(end.y, end.y);
-	if (900*diff > FIX_ONE) {
+	if (ABS(diff) > FIX_ONE/1000) {
 		GF_Err e = gf_path_add_line_to(gp, start.x, start.y);
 		if (e) return e;
 	}
@@ -168,7 +168,7 @@ GF_EXPORT
 GF_Err gf_path_add_cubic_to(GF_Path *gp, Fixed c1_x, Fixed c1_y, Fixed c2_x, Fixed c2_y, Fixed x, Fixed y)
 {
 	if (!gp || !gp->n_contours) return GF_BAD_PARAM;
-	GF_2D_REALLOC_POINT(gp, 3)
+	GF_2D_REALLOC(gp)
 	gp->points[gp->n_points].x = c1_x;
 	gp->points[gp->n_points].y = c1_y;
 	gp->tags[gp->n_points] = GF_PATH_CURVE_CUBIC;
@@ -199,7 +199,7 @@ GF_EXPORT
 GF_Err gf_path_add_quadratic_to(GF_Path *gp, Fixed c_x, Fixed c_y, Fixed x, Fixed y)
 {
 	if (!gp || !gp->n_contours) return GF_BAD_PARAM;
-	GF_2D_REALLOC_POINT(gp, 2)
+	GF_2D_REALLOC(gp)
 	gp->points[gp->n_points].x = c_x;
 	gp->points[gp->n_points].y = c_y;
 	gp->tags[gp->n_points] = GF_PATH_CURVE_CONIC;
@@ -259,10 +259,10 @@ GF_Err gf_path_add_ellipse(GF_Path *gp, Fixed cx, Fixed cy, Fixed a_axis, Fixed
 	u32 i;
 	a_axis /= 2;
 	b_axis /= 2;
-	e = gf_path_add_move_to(gp, cx, cy+b_axis);
+	e = gf_path_add_move_to(gp, cx+a_axis, cy);
 	if (e) return e;
 	for (i=1; i<GF_2D_DEFAULT_RES; i++) {
-		cur = GF_PI2 + GF_2PI*i/GF_2D_DEFAULT_RES;
+		cur = GF_2PI*i/GF_2D_DEFAULT_RES;
 		_vx = gf_mulfix(a_axis, gf_cos(cur) );
 		_vy = gf_mulfix(b_axis, gf_sin(cur) );
 		e = gf_path_add_line_to(gp, _vx + cx, _vy + cy);
@@ -271,6 +271,35 @@ GF_Err gf_path_add_ellipse(GF_Path *gp, Fixed cx, Fixed cy, Fixed a_axis, Fixed
 	return gf_path_close(gp);
 }
 
+GF_Err gf_path_add_subpath(GF_Path *gp, GF_Path *src, GF_Matrix2D *mx)
+{
+	u32 i;
+	if (!src) return GF_OK;
+	gp->contours = realloc(gp->contours, sizeof(u32) * (gp->n_contours + src->n_contours));
+	if (!gp->contours) return GF_OUT_OF_MEM;
+	for (i=0; i<src->n_contours; i++) {
+		gp->contours[i+gp->n_contours] = src->contours[i] + gp->n_points;
+	}
+	gp->n_contours += src->n_contours;
+	gp->n_alloc_points += src->n_alloc_points;
+	gp->points = realloc(gp->points, sizeof(GF_Point2D)*gp->n_alloc_points);
+	if (!gp->points) return GF_OUT_OF_MEM;
+	gp->tags = realloc(gp->tags, sizeof(u8)*gp->n_alloc_points);
+	if (!gp->tags) return GF_OUT_OF_MEM;
+	memcpy(gp->points + gp->n_points, src->points, sizeof(GF_Point2D)*src->n_points);
+	if (mx) {
+		for (i=0;i<src->n_points; i++) {
+			gf_mx2d_apply_coords(mx, &gp->points[i+gp->n_points].x, &gp->points[i+gp->n_points].y);
+		}
+	}
+	memcpy(gp->tags + gp->n_points, src->tags, sizeof(u8)*src->n_points);
+	gp->n_points += src->n_points;
+	gf_rect_union(&gp->bbox, &src->bbox);
+	if (!(src->flags & GF_PATH_FLATTENED)) gp->flags &= ~GF_PATH_FLATTENED;
+	if (src->flags & GF_PATH_BBOX_DIRTY) gp->flags |= GF_PATH_BBOX_DIRTY;
+	return GF_OK;
+}
+
 /*generic N-bezier*/
 static void NBezier(GF_Point2D *pts, s32 n, Double mu, GF_Point2D *pt_out)
 {
@@ -392,6 +421,151 @@ GF_Err gf_path_add_arc_to(GF_Path *gp, Fixed end_x, Fixed end_y, Fixed fa_x, Fix
 	return GF_OK;
 }
 
+
+
+GF_EXPORT
+GF_Err gf_path_add_svg_arc_to(GF_Path *gp, Fixed end_x, Fixed end_y, Fixed r_x, Fixed r_y, Fixed x_axis_rotation, Bool large_arc_flag, Bool sweep_flag)
+{
+	Fixed start_x, start_y;
+	Fixed xmid,ymid;
+	Fixed xmidp,ymidp;
+	Fixed xmidpsq,ymidpsq;
+	Fixed phi, cos_phi, sin_phi;
+	Fixed c_x, c_y;
+	Fixed cxp, cyp;
+	Fixed scale;
+	Fixed rxsq, rysq;
+	Fixed start_angle, sweep_angle;
+	Fixed radius_scale;
+	Fixed vx, vy, normv;
+	Fixed ux, uy, normu;
+	Fixed sign;
+	u32 i, num_steps;
+
+	if (!gp->n_points) return GF_BAD_PARAM;
+
+	if (!r_x || !r_y) {
+		gf_path_add_line_to(gp, end_x, end_y);
+		return GF_OK;
+	}
+
+	if (r_x < 0) r_x = -r_x;
+	if (r_y < 0) r_y = -r_y;
+
+	start_x = gp->points[gp->n_points-1].x;
+	start_y = gp->points[gp->n_points-1].y;
+
+	phi = gf_mulfix(gf_divfix(x_axis_rotation, 180), GF_PI);
+	cos_phi = gf_cos(phi);
+	sin_phi = gf_sin(phi);
+	xmid = (start_x - end_x)/2;
+	ymid = (start_y - end_y)/2;
+	if (!xmid && !ymid) {
+		gf_path_add_line_to(gp, end_x, end_y);
+		return GF_OK;
+	}
+
+	xmidp = gf_mulfix(cos_phi, xmid) + gf_mulfix(sin_phi, ymid);
+	ymidp = gf_mulfix(-sin_phi, xmid) + gf_mulfix(cos_phi, ymid);
+	xmidpsq = gf_mulfix(xmidp, xmidp);
+	ymidpsq = gf_mulfix(ymidp, ymidp);
+
+	rxsq = gf_mulfix(r_x, r_x);
+	rysq = gf_mulfix(r_y, r_y);
+	assert(rxsq && rxsq);
+
+	radius_scale = gf_divfix(xmidpsq, rxsq) + gf_divfix(ymidpsq, rysq);
+	if (radius_scale > FIX_ONE) {
+		r_x = gf_mulfix(gf_sqrt(radius_scale), r_x);
+		r_y = gf_mulfix(gf_sqrt(radius_scale), r_y);
+		rxsq = gf_mulfix(r_x, r_x);
+		rysq = gf_mulfix(r_y, r_y);
+	} 
+
+#if 0
+	/* Old code with overflow problems in fixed point, 
+	   sign was sometimes negative (cf tango SVG icons appointment-new.svg)*/
+
+	sign = gf_mulfix(rxsq,ymidpsq) + gf_mulfix(rysq, xmidpsq);
+	scale = FIX_ONE;
+	/*FIXME - what if scale is 0 ??*/
+	if (sign) scale = gf_divfix(
+					(gf_mulfix(rxsq,rysq) - gf_mulfix(rxsq, ymidpsq) - gf_mulfix(rysq,xmidpsq)),
+					sign
+			);
+#else
+	/* New code: the sign variable computation is split into simpler cases and 
+	   the expression is divided by rxsq to reduce the range */	   
+	if ((rxsq == 0 || ymidpsq ==0) && (rysq == 0 || xmidpsq == 0)) {
+		scale = FIX_ONE;
+	} else if (rxsq == 0 || ymidpsq ==0) {
+		scale = gf_divfix(rxsq,xmidpsq) - FIX_ONE;
+	} else if (rysq == 0 || xmidpsq == 0) {
+		scale = gf_divfix(rysq,ymidpsq) - FIX_ONE;
+	} else {
+		Fixed tmp;
+		tmp = gf_mulfix(gf_divfix(rysq, rxsq), xmidpsq);
+		sign = ymidpsq + tmp;
+		scale = gf_divfix((rysq - ymidpsq - tmp),sign);
+	}
+#endif
+	/* precision problem may lead to negative value around zero, we need to take care of it before sqrt */
+	scale = gf_sqrt(ABS(scale));
+
+	cxp = gf_mulfix(scale, gf_divfix(gf_mulfix(r_x, ymidp),r_y));
+	cyp = gf_mulfix(scale, -gf_divfix(gf_mulfix(r_y, xmidp),r_x));
+	cxp = (large_arc_flag == sweep_flag ? - cxp : cxp);
+	cyp = (large_arc_flag == sweep_flag ? - cyp : cyp);
+
+	c_x = gf_mulfix(cos_phi, cxp) - gf_mulfix(sin_phi, cyp) + (start_x + end_x)/2;
+	c_y = gf_mulfix(sin_phi, cxp) + gf_mulfix(cos_phi, cyp) + (start_y + end_y)/2;
+
+	ux = FIX_ONE;
+	uy = 0;
+	normu = FIX_ONE;
+	
+	vx = gf_divfix(xmidp-cxp,r_x);
+	vy = gf_divfix(ymidp-cyp,r_y);
+	normv = gf_sqrt(gf_mulfix(vx, vx) + gf_mulfix(vy,vy));
+
+	sign = vy;
+	start_angle = gf_acos(gf_divfix(vx,normv));
+	start_angle = (sign > 0 ? start_angle: -start_angle);
+
+	ux = vx;
+	uy = vy;
+	normu = normv;
+	
+	vx = gf_divfix(-xmidp-cxp,r_x);
+	vy = gf_divfix(-ymidp-cyp,r_y);
+	normu = gf_sqrt(gf_mulfix(ux, ux) + gf_mulfix(uy,uy));
+	
+	sign = gf_mulfix(ux, vy) - gf_mulfix(uy, vx);
+	sweep_angle = gf_divfix( gf_mulfix(ux,vx) + gf_mulfix(uy, vy), gf_mulfix(normu, normv) );
+	/*numerical stability safety*/
+	sweep_angle = MAX(-FIX_ONE, MIN(sweep_angle, FIX_ONE));
+	sweep_angle = gf_acos(sweep_angle);
+	sweep_angle = (sign > 0 ? sweep_angle: -sweep_angle);
+	if (sweep_flag == 0) {
+		if (sweep_angle > 0) sweep_angle -= GF_2PI; 
+	} else {
+		if (sweep_angle < 0) sweep_angle += GF_2PI; 
+	}
+
+	num_steps = GF_2D_DEFAULT_RES/2;
+	for (i=1; i<=num_steps; i++) {
+		Fixed _vx, _vy;
+		Fixed _vxp, _vyp;
+		Fixed angle = start_angle + sweep_angle*i/num_steps;
+		_vx = gf_mulfix(r_x, gf_cos(angle));
+		_vy = gf_mulfix(r_y, gf_sin(angle));
+		_vxp = gf_mulfix(cos_phi, _vx) - gf_mulfix(sin_phi, _vy) + c_x;
+		_vyp = gf_mulfix(sin_phi, _vx) + gf_mulfix(cos_phi, _vy) + c_y;
+		gf_path_add_line_to(gp, _vxp, _vyp);
+	}
+	return GF_OK;
+}
+
 GF_EXPORT
 GF_Err gf_path_add_arc(GF_Path *gp, Fixed radius, Fixed start_angle, Fixed end_angle, u32 close_type)
 {
@@ -461,6 +635,7 @@ GF_Err gf_path_get_control_bounds(GF_Path *gp, GF_Rect *rc)
 	rc->width = xMax - xMin;
 	rc->height = yMax - yMin;
 
+#if 0
 	/*take care of straight line path by adding a default width if height and vice-versa*/
 	if (rc->height && !rc->width) {
 		rc->width = 2*FIX_ONE;
@@ -470,6 +645,7 @@ GF_Err gf_path_get_control_bounds(GF_Path *gp, GF_Rect *rc)
 		rc->height = 2*FIX_ONE;
 		rc->y += FIX_ONE;
 	}
+#endif
 	return GF_OK;
 }
 
@@ -527,6 +703,11 @@ static void gf_cubic_check(Fixed p1, Fixed p2, Fixed p3, Fixed p4, Fixed *min, F
 		Fixed y3 = arc[2];
 		Fixed y4 = arc[3];
 		
+		if (ABS(y1)<FIX_EPSILON) arc[0] = y1 = 0;
+		if (ABS(y2)<FIX_EPSILON) arc[1] = y2 = 0;
+		if (ABS(y3)<FIX_EPSILON) arc[2] = y3 = 0;
+		if (ABS(y4)<FIX_EPSILON) arc[3] = y4 = 0;
+
 		if ( y1 == y4 ) {
 			/* flat */
 			if ((y1 == y2) && (y1 == y3)) goto Test;
@@ -687,7 +868,7 @@ static GF_Err gf_subdivide_cubic(GF_Path *gp, Fixed x0, Fixed y0, Fixed x1, Fixe
 	pt.y = y1 - y0;
 	z1_0 = gf_v2d_len(&pt);
 
-	if ((z3_0 < FIX_ONE/100) && (z1_0 < FIX_ONE/100)) 
+	if ((z3_0*100 < FIX_ONE) && (z1_0*100 < FIX_ONE)) 
 		goto nosubdivide;
 
 	/* perp is distance from line, multiplied by dist z0-z3 */
@@ -730,6 +911,11 @@ subdivide:
 	yb2 = (y2 + y3) / 2;
 	x_m = (xa2 + xb1) / 2;
 	y_m = (ya2 + yb1) / 2;
+	/*safeguard for numerical stability*/
+	if ( (ABS(x_m-x0) < FIX_EPSILON) && (ABS(y_m-y0) < FIX_EPSILON))
+		return gf_path_add_line_to(gp, x3, y3);
+	if ( (ABS(x3-x_m) < FIX_EPSILON) && (ABS(y3-y_m) < FIX_EPSILON))
+		return gf_path_add_line_to(gp, x3, y3);
 
 	e = gf_subdivide_cubic(gp, x0, y0, xa1, ya1, xa2, ya2, x_m, y_m, fineness);
 	if (e) return e;
