@@ -1194,6 +1194,7 @@ static s32 FT_Stroker_ParseOutline(FT_Stroker *stroker, GF_Path*  outline)
 	first = 0;
 
 	for ( n = 0; n < outline->n_contours; n++ ) {
+	  s32 closed_subpath;
 	  s32 last;  /* index of last point in contour */
 	  
 	  last  = outline->contours[n];
@@ -1231,11 +1232,19 @@ static s32 FT_Stroker_ParseOutline(FT_Stroker *stroker, GF_Path*  outline)
 		  point--;
 		  tags--;
 	  }
+	  closed_subpath = (outline->tags[outline->contours[n]]==GF_PATH_CLOSE) ? 1 : 0;
 
 	  error = FT_Stroker_BeginSubPath(stroker, &v_start);
 	  if ( error )
 		  goto Exit;
 
+	  /*subpath is a single point, force a lineTo to start for the stroker to compute lineCap*/
+	  if (point==limit) {
+		  error = FT_Stroker_LineTo(stroker, &v_start, 1);
+		  closed_subpath = 0;
+		  goto Close;
+	  }
+
 	  while ( point < limit ) {
 		  point++;
 		  tags++;
@@ -1328,7 +1337,7 @@ static s32 FT_Stroker_ParseOutline(FT_Stroker *stroker, GF_Path*  outline)
 Close:
 	  if ( error ) goto Exit;
 
-	  error = FT_Stroker_EndSubPath(stroker, (outline->tags[outline->contours[n]]==GF_PATH_CLOSE) ? 1 : 0);
+	  error = FT_Stroker_EndSubPath(stroker, closed_subpath);
 	  if ( error )
 		  goto Exit;
 
@@ -1392,12 +1401,12 @@ static Fixed gf_path_get_dash(GF_PenSettings *pen, u32 dash_slot, u32 *next_slot
 		*next_slot = (dash_slot + 1) % 6;
 		return ret * pen->width;
 	case GF_DASH_STYLE_CUSTOM:
-	case GF_DASH_STYLE_CUSTOM_ABS:
+	case GF_DASH_STYLE_SVG:
 		if (!pen->dash_set || !pen->dash_set->num_dash) return 0;
 		if (dash_slot>=pen->dash_set->num_dash) dash_slot = 0;
 		ret = pen->dash_set->dashes[dash_slot];
 		*next_slot = (1 + dash_slot) % pen->dash_set->num_dash;
-		if (pen->dash==GF_DASH_STYLE_CUSTOM_ABS) return ret;
+		if (pen->dash==GF_DASH_STYLE_SVG) return ret;
 		/*custom dashes are of type Fixed !!*/
 		return gf_mulfix(ret, pen->width);
 
@@ -1452,6 +1461,7 @@ static GF_Err evg_dash_subpath(GF_Path *dashed, GF_Point2D *pts, u32 nb_pts, GF_
 	Fixed totaldist;
 	Fixed dash;
 	Fixed dist;
+	Fixed dash_dist;
 	s32 offsetinit;
 	u32 next_offset;
 	s32 toggleinit;
@@ -1468,6 +1478,7 @@ static GF_Err evg_dash_subpath(GF_Path *dashed, GF_Point2D *pts, u32 nb_pts, GF_
 	/* initial values */
 	toggleinit = 1;
 	offsetinit = 0;
+	dash_dist = 0;
 
 	dash = gf_path_get_dash(pen, offsetinit, &next_offset);
 	if (length_scale) dash = gf_mulfix(dash, length_scale);
@@ -1477,6 +1488,40 @@ static GF_Err evg_dash_subpath(GF_Path *dashed, GF_Point2D *pts, u32 nb_pts, GF_
 	start_ind = 0;
 	dist = 0;
 
+	/*SVG dashing is different from BIFS one*/
+	if (pen->dash==GF_DASH_STYLE_SVG) {
+		while (pen->dash_offset>0) {
+			if (pen->dash_offset - dash < 0) {
+				dash -= pen->dash_offset;
+				pen->dash_offset = 0;
+				break;
+			}
+			pen->dash_offset -= dash;
+			offsetinit = next_offset;
+			toggleinit = !toggleinit;
+			dash = gf_path_get_dash(pen, offsetinit, &next_offset);
+			if (length_scale) dash = gf_mulfix(dash, length_scale);
+		}
+		if (pen->dash_offset<0) {
+			offsetinit = pen->dash_set->num_dash-1;
+			dash = gf_path_get_dash(pen, offsetinit, &next_offset);
+			if (length_scale) dash = gf_mulfix(dash, length_scale);
+			while (pen->dash_offset<0) {
+				toggleinit = !toggleinit;
+				if (pen->dash_offset + dash > 0) {
+					dash_dist = dash+pen->dash_offset;
+					pen->dash_offset = 0;
+					break;
+				}
+				pen->dash_offset += dash;
+				if (offsetinit) offsetinit --;
+				else offsetinit = pen->dash_set->num_dash-1;
+				dash = gf_path_get_dash(pen, offsetinit, &next_offset);
+				if (length_scale) dash = gf_mulfix(dash, length_scale);
+			}
+		}
+	}
+
 	/* calculate line lengths and update offset*/
 	totaldist = 0;
 	for (i = 0; i < nb_pts - 1; i++) {
@@ -1484,11 +1529,12 @@ static GF_Err evg_dash_subpath(GF_Path *dashed, GF_Point2D *pts, u32 nb_pts, GF_
 		diff.x = pts[i+1].x - pts[i].x;
 		diff.y = pts[i+1].y - pts[i].y;
 		dists[i] = gf_v2d_len(&diff);
-		
+
 		if (pen->dash_offset > dists[i]) {
 			pen->dash_offset -= dists[i];
 			dists[i] = 0;
-		} else if (pen->dash_offset) {
+		} 
+		else if (pen->dash_offset) {
 			Fixed a, x, y, dx, dy;
 
 			a = gf_divfix(pen->dash_offset, dists[i]);
@@ -1506,6 +1552,8 @@ static GF_Err evg_dash_subpath(GF_Path *dashed, GF_Point2D *pts, u32 nb_pts, GF_
 			totaldist += dists[i];
 		}
 	}
+	dash -= dash_dist;
+	dash_dist = 0;
 
 	/* subpath fits within first dash and no offset*/
 	if (!dist && totaldist <= dash) {
@@ -1637,8 +1685,9 @@ GF_Path *gf_path_get_outline(GF_Path *path, GF_PenSettings pen)
 	GF_Path *outline;
 	GF_Path *dashed;
 	GF_Path *scaled;
-
 	FT_Stroker stroker;
+	if (!path || !pen.width) return NULL;
+
 	memset(&stroker, 0, sizeof(stroker));
     stroker.borders[0].start = -1;
     stroker.borders[1].start = -1;
@@ -1647,10 +1696,6 @@ GF_Path *gf_path_get_outline(GF_Path *path, GF_PenSettings pen)
 	stroker.miter_limit = pen.miterLimit;
 	stroker.radius = pen.width/2;
 
-	/*security: some SVG paths use a single MoveTo for points drawing but the stroker needs at least 2 points*/
-	if (path->n_points==1) 
-		gf_path_add_line_to(path, path->points[0].x, path->points[0].y);
-
 	gf_path_flatten(path);
 
 	scaled = NULL;
