@@ -48,7 +48,7 @@
 
 //  Exported Functions (DLL entry point)
 #ifndef EKA2 // for EKA1 only
-EXPORT_C TInt E32Dll(TDllReason /*aReason*/)
+GLDEF_C TInt E32Dll(TDllReason /*aReason*/)
 // Called when the DLL is loaded and unloaded. Note: have to define
 // epoccalldllentrypoints in MMP file to get this called in THUMB.
 {
@@ -77,7 +77,11 @@ void gf_sleep(u32 ms)
 {
 	TTimeIntervalMicroSeconds32 inter;
 	inter = (TInt) (1000*ms);
+#ifdef __SERIES60_3X__
 	User::AfterHighRes(inter); 
+#else
+	User::After(inter); 
+#endif
 
 #if 0
 	TInt error;
@@ -263,40 +267,32 @@ struct __tag_thread
 	/* the thread procedure */
 	u32 (*Run)(void *param);
 	void *args;
+#ifndef GPAC_DISABLE_LOG
+	char *log_name;
+#endif
 	/* lock for signal */
 	GF_Semaphore *_signal;
 };
 
 GF_EXPORT
-GF_Thread *gf_th_new()
+GF_Thread *gf_th_new(const char *name)
 {
 	GF_Thread *tmp = (GF_Thread *) malloc(sizeof(GF_Thread));
 	memset((void *)tmp, 0, sizeof(GF_Thread));
 	tmp->status = GF_THREAD_STATUS_STOP;
+#ifndef GPAC_DISABLE_LOG
+	if (name) {
+		tmp->log_name = strdup(name);
+	} else {
+		char szN[20];
+		sprintf(szN, "0x%08x", (u32) tmp);
+		tmp->log_name = strdup(szN);
+	}
+#endif
+
 	return tmp;
 }
 
-#if 0
-static void *RunThread(void *ptr)
-{
-	u32 ret = 0;
-	GF_Thread *t = (GF_Thread *)ptr;
-
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_CORE, ("[Core] Thread 0x%08x start\n", t->threadH->Handle() ));
-	/* Signal the caller */
-	t->status = GF_THREAD_STATUS_RUN;
-	gf_sema_notify(t->_signal, 1);
-	/* Run our thread */
-	ret = t->Run(t->args);
-
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_CORE, ("[Core] Thread 0x%08x exit\n", t->threadH->Handle() ));
-	t->status = GF_THREAD_STATUS_DEAD;
-	t->Run = NULL;
-	t->threadH->Close();
-	t->threadH = NULL;
-	return (void *)ret;
-}
-#else
 static void *RunThread(void *ptr)
 {
 	TInt err;
@@ -304,21 +300,21 @@ static void *RunThread(void *ptr)
 	CTrapCleanup * cleanup = NULL;
 	GF_Thread *t = (GF_Thread *)ptr;
 
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_CORE, ("[Core] Thread 0x%08x start\n", t->threadH->Handle() ));
 
-
-#if 1
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_CORE, ("[core] Creating new ActiveScheduler for thread\n"));
 	CActiveScheduler * scheduler = new CActiveScheduler();
 	if (scheduler == NULL) {
 		t->status = GF_THREAD_STATUS_DEAD;
 		gf_sema_notify(t->_signal, 1);
 		goto exit;
 	} 
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_CORE, ("[core] Installing ActiveScheduler for thread\n"));
+#ifndef GPAC_DISABLE_LOG
+	GF_LOG(GF_LOG_DEBUG, GF_LOG_CORE, ("[Thread %s] Installing ActiveScheduler\n", t->log_name));
+#endif
 	CActiveScheduler::Install(scheduler);
+
+#ifndef GPAC_DISABLE_LOG
+	GF_LOG(GF_LOG_DEBUG, GF_LOG_CORE, ("[Thread %s] Creating cleanup trap\n", t->log_name));
 #endif
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_CORE, ("[core] Creating cleanup trap for thread\n"));
 	cleanup = CTrapCleanup::New();
 	if( cleanup == NULL ) {
 		t->status = GF_THREAD_STATUS_DEAD;
@@ -327,32 +323,31 @@ static void *RunThread(void *ptr)
 		goto exit;
 	}
 #if 0
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_CORE, ("[core] Starting scheduler for thread\n"));
 	CActiveScheduler::Start();
 #endif
 	/* OK , signal the caller */
 	t->status = GF_THREAD_STATUS_RUN;
 	gf_sema_notify(t->_signal, 1);
 	/* Run our thread */
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_CORE, ("[core] Executing thread body\n"));
+#ifndef GPAC_DISABLE_LOG
+	GF_LOG(GF_LOG_DEBUG, GF_LOG_CORE, ("[Thread %s] Entering thread proc\n", t->log_name));
+#endif
 	TRAP(err, ret=t->Run(t->args) );
 	//ret = t->Run(t->args);
 
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_CORE, ("[Core] destroying current ActiveScheduler\n"));
 	delete CActiveScheduler::Current();
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_CORE, ("[Core] destroying trap cleanup\n"));
 	delete cleanup;
 
 exit:
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_CORE, ("[Core] Thread 0x%08x exit\n", t->threadH->Handle() ));
-
+#ifndef GPAC_DISABLE_LOG
+	GF_LOG(GF_LOG_DEBUG, GF_LOG_CORE, ("[Thread %s] Exit\n", t->log_name ));
+#endif
 	t->status = GF_THREAD_STATUS_DEAD;
 	t->Run = NULL;
 	t->threadH->Close();
 	t->threadH = NULL;
 	return (void *)ret;
 }
-#endif
 
 GF_EXPORT
 GF_Err gf_th_run(GF_Thread *t, u32 (*Run)(void *param), void *param)
@@ -416,6 +411,9 @@ GF_EXPORT
 void gf_th_del(GF_Thread *t)
 {
 	Thread_Stop(t, 0);
+#ifndef GPAC_DISABLE_LOG
+	free(t->log_name);
+#endif
 	free(t);
 }
 
@@ -457,11 +455,14 @@ struct __tag_mutex
 	/* We filter recursive calls (1 thread calling Lock several times in a row only locks
 	ONCE the mutex. Holder is the current ThreadID of the mutex holder*/
 	u32 Holder, HolderCount;
+#ifndef GPAC_DISABLE_LOG
+	char *log_name;
+#endif
 };
 
 
 GF_EXPORT
-GF_Mutex *gf_mx_new()
+GF_Mutex *gf_mx_new(const char *name)
 {
 	GF_Mutex *tmp = (GF_Mutex *)malloc(sizeof(GF_Mutex));
 	if (!tmp) return NULL;
@@ -472,6 +473,15 @@ GF_Mutex *gf_mx_new()
 		free(tmp);
 		return NULL;
 	}
+#ifndef GPAC_DISABLE_LOG
+	if (name) {
+		tmp->log_name = strdup(name);
+	} else {
+		char szN[20];
+		sprintf(szN, "0x%08x", (u32) tmp);
+		tmp->log_name = strdup(szN);
+	}
+#endif
 	return tmp;
 }
 
@@ -637,7 +647,9 @@ Bool gf_sys_get_rti(u32 refresh_time_ms, GF_SystemRTInfo *rti, u32 flags)
 {
 	TInt ram, ram_free;
 	u32 now, time;
+#ifdef __SERIES60_3X__
 	TModuleMemoryInfo mi;
+#endif
 	TTimeIntervalMicroSeconds tims;
 	RProcess cur_process;
 	RThread cur_th;
@@ -648,7 +660,11 @@ Bool gf_sys_get_rti(u32 refresh_time_ms, GF_SystemRTInfo *rti, u32 flags)
 		if (cur_th.GetCpuTime(tims) != KErrNone) {
 			return 0;
 		}
+#ifdef __SERIES60_3X__
 		rti->process_cpu_time = (u32) (tims.Int64() / 1000);
+#else
+		rti->process_cpu_time = (u32) ( TInt64(tims.Int64() / 1000).GetTInt() );
+#endif
 		return 0;
 	}
 	if (rti->sampling_instant + refresh_time_ms > now) return 0;
@@ -658,15 +674,20 @@ Bool gf_sys_get_rti(u32 refresh_time_ms, GF_SystemRTInfo *rti, u32 flags)
 	if (cur_th.Process(cur_process) != KErrNone) {
 		return 0;
 	}
+#ifdef __SERIES60_3X__
 	if (cur_process.GetMemoryInfo(mi) != KErrNone) {
 		return 0;
 	}
 	rti->process_memory = mi.iCodeSize + mi.iConstDataSize + mi.iInitialisedDataSize + mi.iUninitialisedDataSize;
-
+#endif
 	if (cur_th.GetCpuTime(tims) != KErrNone) {
 		return 0;
 	}
+#ifdef __SERIES60_3X__
 	time = (u32) (tims.Int64() / 1000);
+#else
+	time = (u32) ( TInt64(tims.Int64() / 1000).GetTInt() );
+#endif
 	rti->process_cpu_time_diff = time - rti->process_cpu_time;
 	rti->process_cpu_time = time;
 	rti->process_cpu_usage = 100*rti->process_cpu_time_diff / rti->sampling_period_duration;
