@@ -109,6 +109,10 @@ char *gf_url_concatenate(const char *parentName, const char *pathName)
 	pathSepCount = 0;
 	name = NULL;
 	if (pathName[0] == '.') {
+		if (!strcmp(pathName, "..")) {
+			pathSepCount = 1;
+			name = "";
+		}
 		for (i = 0; i< strlen(pathName) - 2; i++) {
 			/*current dir*/
 			if ( (pathName[i] == '.') 
@@ -122,6 +126,7 @@ char *gf_url_concatenate(const char *parentName, const char *pathName)
 				) {
 				pathSepCount ++;
 				i+=2;
+				name = (char *) &pathName[i+1];
 			} else {
 				name = (char *) &pathName[i];
 				break;
@@ -152,10 +157,11 @@ char *gf_url_concatenate(const char *parentName, const char *pathName)
 	sprintf(outPath, "%s%c%s", tmp, psep, name);
 
 	/*cleanup paths sep for win32*/
-	if ((prot_type == GF_URL_TYPE_FILE) && (GF_PATH_SEPARATOR != '/')) {
+//	if ((prot_type == GF_URL_TYPE_FILE) && (GF_PATH_SEPARATOR != '/')) {
 		for (i = 0; i<strlen(outPath); i++) 
-			if (outPath[i]=='/') outPath[i] = GF_PATH_SEPARATOR;
-	}
+//			if (outPath[i]=='/') outPath[i] = GF_PATH_SEPARATOR;
+			if (outPath[i]=='\\') outPath[i] = '/';
+//	}
 
 check_spaces:
 	while (1) {
@@ -166,3 +172,24 @@ check_spaces:
 	}
 	return outPath;
 }
+
+GF_EXPORT
+void gf_url_to_fs_path(char *sURL)
+{
+	if (!strnicmp(sURL, "file://", 7)) {
+		/*file:///C:\ scheme*/
+		if ((sURL[7]=='/') && (sURL[9]==':')) {
+			memmove(sURL, sURL+8, strlen(sURL)-7);
+		} else {
+			memmove(sURL, sURL+7, strlen(sURL)-6);
+		}
+	}
+
+	while (1) {
+		char *sep = strstr(sURL, "%20");
+		if (!sep) break;
+		sep[0] = ' ';
+		memmove(sep+1, sep+3, strlen(sep)-2);
+	}
+}
+
