@@ -29,6 +29,11 @@
 /*since 0.2.2, we use zlib for xmt/x3d reading to handle gz files*/
 #include <zlib.h>
 
+#if (defined(WIN32) || defined(_WIN32_WCE)) && !defined(__GNUC__)
+#pragma comment(lib, "zlib")
+#endif
+
+
 static GF_Err gf_xml_sax_parse_intern(GF_SAXParser *parser, char *current);
 
 static char *xml_translate_xml_string(char *str)
@@ -40,7 +45,7 @@ static char *xml_translate_xml_string(char *str)
 	size = 500;
 	i = j = 0;
 	while (str[i]) {
-		if (j >= size) {
+		if (j+20 >= size) {
 			size += 500;
 			value = (char *)realloc(value, sizeof(char)*size);
 		}
@@ -48,14 +53,19 @@ static char *xml_translate_xml_string(char *str)
 			if (str[i+1]=='#') {
 				char szChar[20], *end;
 				u16 wchar[2];
+				u32 val;
 				const unsigned short *srcp;
 				strncpy(szChar, str+i, 10);
 				end = strchr(szChar, ';');
-				assert(end);
+				if (!end) break;
 				end[1] = 0;
 				i+=strlen(szChar);
 				wchar[1] = 0;
-				sscanf(szChar, "&#%hd;", &wchar[0]);
+				if (szChar[2]=='x')
+					sscanf(szChar, "&#x%x;", &val);
+				else
+					sscanf(szChar, "&#%d;", &val);
+				wchar[0] = val;
 				srcp = wchar;
 				j += gf_utf8_wcstombs(&value[j], 20, &srcp);
 			}
@@ -119,6 +129,10 @@ typedef struct
 	Bool has_entities;
 } GF_XMLSaxAttribute;
 
+
+//#define NO_GZIP
+
+
 struct _tag_sax_parser
 {
 	/*0: UTF-8, 1: UTF-16 BE, 2: UTF-16 LE. String input is always converted back to utf8*/
@@ -130,7 +144,11 @@ struct _tag_sax_parser
 	u32 node_depth;
 
 	/*gz input file*/
+#ifdef NO_GZIP
+	FILE *f_in;
+#else
 	gzFile gz_in;
+#endif
 	/*current line , file size and pos for user notif*/
 	u32 line, file_size, file_pos;
 
@@ -186,7 +204,13 @@ static void xml_sax_node_end(GF_SAXParser *parser, Bool had_children)
 {
 	char *name, *sep, c;
 
-	assert(parser->elt_name_start && parser->elt_name_end && parser->node_depth);
+	assert(parser->elt_name_start);
+	assert(parser->elt_name_end);
+	if (!parser->node_depth) {
+		parser->sax_state = SAX_STATE_SYNTAX_ERROR;
+		sprintf(parser->err_msg, "Markup error");
+		return;
+	}
 	c = parser->buffer[parser->elt_name_end - 1];
 	parser->buffer[parser->elt_name_end - 1] = 0;
 	name = parser->buffer + parser->elt_name_start - 1;
@@ -291,6 +315,8 @@ static Bool xml_sax_parse_attribute(GF_SAXParser *parser)
 						/*done parsing attr AND elements*/
 						if (!parser->init_state) {
 							xml_sax_node_start(parser);
+							/*move to SAX_STATE_TEXT_CONTENT to force text flush*/
+							parser->sax_state = SAX_STATE_TEXT_CONTENT;
 							xml_sax_node_end(parser, 0);
 						} else {
 							parser->nb_attrs = 0;
@@ -423,11 +449,15 @@ static Bool xml_sax_parse_attribute(GF_SAXParser *parser)
 			if (parser->current_pos == parser->line_size) return 1;
 		} 
 
+att_retry:
+
 		assert(parser->att_sep);
 		sep = strchr(parser->buffer + parser->current_pos, parser->att_sep);
-		if (!sep) {
-			//fprintf(stdout, "XML: sep not found, waiting for more data\n");
-			return 1;
+		if (!sep || !sep[1]) return 1;
+
+		if (!parser->init_state && (strchr(" />\n\t\r", sep[1])==NULL)) {
+			parser->current_pos = sep - parser->buffer + 1;
+			goto att_retry;
 		}
 
 		parser->current_pos = sep - parser->buffer;
@@ -453,15 +483,19 @@ typedef struct
 {
 	char *name;
 	char *value;
+	u32 namelen;
 	u8 sep;
 } XML_Entity;
 
 static void xml_sax_flush_text(GF_SAXParser *parser)
 {
-	u32 offset;
 	char *text, c;
 	if (!parser->text_start || parser->init_state || !parser->sax_text_content) return;
 
+	/* This optimization should be done at the application level
+	   generic XML parsing should not try to remove any character !!*/
+#if 0
+	u32 offset;
 	offset = 0;
 	while (parser->text_start+offset<parser->text_end) {
 		c = parser->buffer[parser->text_start-1+offset];
@@ -493,6 +527,8 @@ static void xml_sax_flush_text(GF_SAXParser *parser)
 		}
 	}
 	parser->text_end-=offset;
+#endif
+
 	assert(parser->text_start < parser->text_end);
 
 	c = parser->buffer[parser->text_end-1];
@@ -606,6 +642,7 @@ static void xml_sax_parse_entity(GF_SAXParser *parser)
 			szName[i] = 0;
 			GF_SAFEALLOC(ent, XML_Entity);
 			ent->name = strdup(szName);
+			ent->namelen = strlen(ent->name);
 			ent->sep = c;
 			parser->current_pos += 1+i;
 			assert(parser->current_pos < parser->line_size);
@@ -644,7 +681,7 @@ static void xml_sax_cdata(GF_SAXParser *parser)
 		xml_sax_store_text(parser, size);
 		xml_sax_flush_text(parser);
 		parser->current_pos += 3;
-		assert(parser->current_pos < parser->line_size);
+		assert(parser->current_pos <= parser->line_size);
 		parser->sax_state = SAX_STATE_TEXT_CONTENT;
 	}
 }
@@ -769,6 +806,7 @@ restart:
 			parser->elt_start_pos = parser->file_pos + parser->current_pos;
 
 			if (!strncmp(elt, "!--", 3)) { 
+				xml_sax_flush_text(parser);
 				parser->sax_state = SAX_STATE_COMMENT;
 				if (i>3) parser->current_pos -= (i-3);
 			}
@@ -849,7 +887,11 @@ static GF_Err xml_sax_append_string(GF_SAXParser *parser, char *string)
 	u32 size = parser->line_size;
 	u32 nl_size = strlen(string);
 	
-	if (parser->alloc_size < size+nl_size+1) {
+	if (!nl_size) return GF_OK;
+
+	if ( (parser->alloc_size < size+nl_size+1) 
+		|| (parser->alloc_size / 2 ) > size+nl_size+1) 
+	{
 		parser->buffer = realloc(parser->buffer, sizeof(char) * (size+nl_size+1) );
 		if (!parser->buffer ) return GF_OUT_OF_MEM;
 		parser->alloc_size = size+nl_size+1;
@@ -860,57 +902,102 @@ static GF_Err xml_sax_append_string(GF_SAXParser *parser, char *string)
 	return GF_OK;
 }
 
-static GF_Err gf_xml_sax_parse_intern(GF_SAXParser *parser, char *current)
+static XML_Entity *gf_xml_locate_entity(GF_SAXParser *parser, char *ent_start, Bool *needs_text)
 {
 	u32 i, count;
+	u32 len = strlen(ent_start);
+
+	*needs_text = 0;
+	count = gf_list_count(parser->entities);
+
+	for (i=0; i<count; i++) {
+		XML_Entity *ent = (XML_Entity *)gf_list_get(parser->entities, i);
+		if (len < ent->namelen + 1) {
+			*needs_text = 1;
+			return NULL;
+		}
+		if (!strncmp(ent->name, ent_start, ent->namelen) && (ent_start[ent->namelen]==';')) {
+			return ent;
+		}
+	}
+	return NULL;
+}
+
+
+static GF_Err gf_xml_sax_parse_intern(GF_SAXParser *parser, char *current)
+{
+	u32 count;
 	/*solve entities*/
 	count = gf_list_count(parser->entities);
 	while (count) {
-		char *entityEnd, szName[200];
+		char *entityEnd;
 		XML_Entity *ent;
 		char *entityStart = strstr(current, "&");
+		Bool needs_text;
+		u32 line_num;
 
+		/*if in entity, the start of the entity is in the buffer !!*/
 		if (parser->in_entity) {
+			u32 len;
+			char *name;
 			entityEnd = strstr(current, ";");
 			if (!entityEnd) return xml_sax_append_string(parser, current);
 			entityStart = strrchr(parser->buffer, '&');
 
-			strcpy(szName, entityStart+1);
-			entityStart[0] = 0;
 			entityEnd[0] = 0;
-			strcat(szName, current);
-			entityEnd[0] = ';';
+			len = strlen(entityStart) + strlen(current) + 1;
+			name = malloc(sizeof(char)*len);
+			sprintf(name, "%s%s;", entityStart+1, current);
+
+			ent = gf_xml_locate_entity(parser, name, &needs_text);
+			free(name);
+
+			if (!ent && !needs_text) {
+				xml_sax_append_string(parser, current);
+				xml_sax_parse(parser, 1);
+				entityEnd[0] = ';';
+				current = entityEnd;
+				continue;
+			}
+			assert(ent);
+			/*truncate input buffer*/
+			parser->line_size -= strlen(entityStart);
+			entityStart[0] = 0;
+
 			parser->in_entity = 0;
+			entityEnd[0] = ';';
 			current = entityEnd+1;
 		} else {
 			if (!entityStart) break;
-			entityEnd = strstr(entityStart, ";");
 
+			ent = gf_xml_locate_entity(parser, entityStart+1, &needs_text);
+
+			/*store current string before entity start*/
 			entityStart[0] = 0;
 			xml_sax_append_string(parser, current);
 			xml_sax_parse(parser, 1);
 			entityStart[0] = '&';
 
-			if (!entityEnd) {
+			/*this is not an entitiy*/
+			if (!ent && !needs_text) {
+				xml_sax_append_string(parser, "&");
+				current = entityStart+1;
+				continue;
+			}
+
+			if (!ent) {
 				parser->in_entity = 1;
 				/*store entity start*/
 				return xml_sax_append_string(parser, entityStart);
 			}
-			strncpy(szName, entityStart+1, entityEnd - entityStart - 1);
-			szName[entityEnd - entityStart - 1] = 0;
-			current = entityEnd + 1;
+			current = entityStart + ent->namelen + 2;
 		}
+		/*append entity*/
+		line_num = parser->line;
+		xml_sax_append_string(parser, ent->value);
+		xml_sax_parse(parser, 1);
+		parser->line = line_num;
 
-		for (i=0; i<count; i++) {
-			ent = (XML_Entity *)gf_list_get(parser->entities, i);
-			if (!strcmp(ent->name, szName)) {
-				u32 line_num = parser->line;
-				xml_sax_append_string(parser, ent->value);
-				xml_sax_parse(parser, 1);
-				parser->line = line_num;
-				break;
-			}
-		}
 	}
 	xml_sax_append_string(parser, current);
 	return xml_sax_parse(parser, 0);
@@ -951,7 +1038,12 @@ GF_EXPORT
 GF_Err gf_xml_sax_init(GF_SAXParser *parser, unsigned char *BOM)
 {
 	u32 offset;
-	if (!BOM) parser->unicode_type = 0;
+	if (!BOM) {
+		parser->unicode_type = 0;
+		parser->sax_state = SAX_STATE_ELEMENT;
+		return GF_OK;
+	}
+
 	if (parser->unicode_type >= 0) return gf_xml_sax_parse(parser, BOM);
 
 	offset = 0;
@@ -1001,27 +1093,44 @@ static GF_Err xml_sax_read_file(GF_SAXParser *parser)
 {
 	GF_Err e = GF_EOS;
 	unsigned char szLine[XML_INPUT_SIZE+2];
+
+#ifdef NO_GZIP
+	if (!parser->f_in) return GF_BAD_PARAM;
+#else
 	if (!parser->gz_in) return GF_BAD_PARAM;
+#endif
 
-	parser->file_pos = 0;
 
-	while (!gzeof(parser->gz_in) && !parser->suspended) {
-		u32 read = gzread(parser->gz_in, szLine, XML_INPUT_SIZE);
-		if (!read) break;
+	while (!parser->suspended) {
+#ifdef NO_GZIP
+		s32 read = fread(szLine, 1, XML_INPUT_SIZE, parser->f_in);
+#else
+		s32 read = gzread(parser->gz_in, szLine, XML_INPUT_SIZE);
+#endif
+		if ((read<=0) && !parser->node_depth) break;
 		szLine[read] = 0;
-		szLine[read+1] = 0;
+		szLine[read+1] = 0;		
 		e = gf_xml_sax_parse(parser, szLine);
 		if (e) break;
-
 		if (parser->file_pos > parser->file_size) parser->file_size = parser->file_pos + 1;
 		if (parser->on_progress) parser->on_progress(parser->sax_cbck, parser->file_pos, parser->file_size);
 	}
+	
+#ifdef NO_GZIP
+	if (feof(parser->f_in)) {
+#else
 	if (gzeof(parser->gz_in)) {
+#endif
 		if (!e) e = GF_EOS;
 		if (parser->on_progress) parser->on_progress(parser->sax_cbck, parser->file_size, parser->file_size);
 
+#ifdef NO_GZIP
+		fclose(parser->f_in);
+		parser->f_in = NULL;
+#else
 		gzclose(parser->gz_in);
 		parser->gz_in = 0;
+#endif
 	}
 	return e;
 }
@@ -1031,7 +1140,9 @@ GF_Err gf_xml_sax_parse_file(GF_SAXParser *parser, const char *fileName, gf_xml_
 {
 	FILE *test;
 	GF_Err e;
+#ifndef NO_GZIP
 	gzFile gzInput;
+#endif
 	unsigned char szLine[6];
 
 	/*check file exists and gets its size (zlib doesn't support SEEK_END)*/
@@ -1043,22 +1154,33 @@ GF_Err gf_xml_sax_parse_file(GF_SAXParser *parser, const char *fileName, gf_xml_
 
 	parser->on_progress = OnProgress;
 
+#ifdef NO_GZIP
+	parser->f_in = fopen(fileName, "rt");
+	fread(szLine, 1, 4, parser->f_in);
+#else
 	gzInput = gzopen(fileName, "rb");
 	if (!gzInput) return GF_IO_ERR;
 	parser->gz_in = gzInput;
 	/*init SAX parser (unicode setup)*/
 	gzread(gzInput, szLine, 4);
+#endif
 	szLine[4] = szLine[5] = 0;
 	e = gf_xml_sax_init(parser, szLine);
 	if (e) return e;
+	parser->file_pos = 4;
 	return xml_sax_read_file(parser);
 }
 
 GF_EXPORT
 Bool gf_xml_sax_binary_file(GF_SAXParser *parser)
 {
-	if (!parser || !parser->gz_in) return 0;
+	if (!parser) return 0;
+#ifdef NO_GZIP
+	return 0;
+#else
+	if (!parser->gz_in) return 0;
 	return (((z_stream*)parser->gz_in)->data_type==Z_BINARY) ? 1 : 0;
+#endif
 }
 
 GF_EXPORT
@@ -1084,7 +1206,11 @@ void gf_xml_sax_del(GF_SAXParser *parser)
 {
 	xml_sax_reset(parser);
 	gf_list_del(parser->entities);
+#ifdef NO_GZIP
+	if (parser->f_in) fclose(parser->f_in);
+#else
 	if (parser->gz_in) gzclose(parser->gz_in);
+#endif
 	free(parser);
 }
 
@@ -1093,7 +1219,11 @@ GF_Err gf_xml_sax_suspend(GF_SAXParser *parser, Bool do_suspend)
 {
 	parser->suspended = do_suspend;
 	if (!do_suspend) {
+#ifdef NO_GZIP
+		if (parser->f_in) return xml_sax_read_file(parser);
+#else
 		if (parser->gz_in) return xml_sax_read_file(parser);
+#endif
 		return xml_sax_parse(parser, 0);
 	}
 	return GF_OK;
@@ -1103,49 +1233,88 @@ GF_EXPORT
 u32 gf_xml_sax_get_line(GF_SAXParser *parser) { return parser->line + 1 ; }
 
 GF_EXPORT
-u32 gf_xml_sax_get_file_size(GF_SAXParser *parser) { return parser->gz_in ? parser->file_size : 0; }
+u32 gf_xml_sax_get_file_size(GF_SAXParser *parser) 
+{ 
+#ifdef NO_GZIP
+	return parser->f_in ? parser->file_size : 0; 
+#else
+	return parser->gz_in ? parser->file_size : 0; 
+#endif
+}
 
 GF_EXPORT
-u32 gf_xml_sax_get_file_pos(GF_SAXParser *parser) { return parser->gz_in ? parser->file_pos : 0; }
+u32 gf_xml_sax_get_file_pos(GF_SAXParser *parser) 
+{
+#ifdef NO_GZIP
+	return parser->f_in ? parser->file_pos : 0; 
+#else
+	return parser->gz_in ? parser->file_pos : 0; 
+#endif
+}
 
 GF_EXPORT
 char *gf_xml_sax_peek_node(GF_SAXParser *parser, char *att_name, char *att_value, char *substitute, char *get_attr, char *end_pattern, Bool *is_substitute)
 {
-	u32 state, att_len;
+	u32 state, att_len, alloc_size;
 	z_off_t pos;
 	char szLine1[XML_INPUT_SIZE+2], szLine2[XML_INPUT_SIZE+2], *szLine, *cur_line, *sep, *start, first_c, *result;
+#ifdef NO_GZIP
+	if (!parser->f_in) return NULL;
+#else
 	if (!parser->gz_in) return NULL;
+#endif
+
+#define CPYCAT_ALLOC(__str, __is_copy) if ( strlen(__str) + (__is_copy ? 0 : strlen(szLine))>=alloc_size) {\
+								alloc_size = 1+strlen(__str);	\
+								if (!__is_copy) alloc_size += strlen(szLine); \
+								szLine = realloc(szLine, alloc_size);	\
+							}\
+							if (__is_copy) strcpy(szLine, __str);	\
+							else strcat(szLine, __str); \
 
 	result = NULL;
 
 	szLine1[0] = szLine2[0] = 0;
+#ifdef NO_GZIP
+	pos = ftell(parser->f_in);
+#else
 	pos = gztell(parser->gz_in);
+#endif
 	att_len = strlen(parser->buffer + parser->att_name_start);
 	if (att_len<2*XML_INPUT_SIZE) att_len = 2*XML_INPUT_SIZE;
-	szLine = (char *) malloc(sizeof(char)*att_len);
+	alloc_size = att_len;
+	szLine = (char *) malloc(sizeof(char)*alloc_size);
 	strcpy(szLine, parser->buffer + parser->att_name_start);
 	cur_line = szLine;
 	att_len = strlen(att_value);
 	state = 0;
 	goto retry;
 
+#ifdef NO_GZIP
+	while (!feof(parser->f_in)) {
+#else
 	while (!gzeof(parser->gz_in)) {
+#endif
 		u32 read;
 		if (cur_line == szLine2) {
 			cur_line = szLine1;
 		} else {
 			cur_line = szLine2;
 		}
+#ifdef NO_GZIP
+		read = fread(cur_line, 1, XML_INPUT_SIZE, parser->f_in);
+#else
 		read = gzread(parser->gz_in, cur_line, XML_INPUT_SIZE);
+#endif
 		cur_line[read] = cur_line[read+1] = 0;
 
-		strcat(szLine, cur_line);
+		CPYCAT_ALLOC(cur_line, 0);
 retry:
 		if (state == 2) goto fetch_attr;
 		sep = strstr(szLine, att_name);
 		if (!sep && !state) {
 			state = 0;
-			strcpy(szLine, cur_line);
+			CPYCAT_ALLOC(cur_line, 1);
 			if (end_pattern && strstr(szLine, end_pattern)) goto exit;
 			continue;
 		}
@@ -1157,17 +1326,20 @@ retry:
 			start = strrchr(szLine, '<');
 			if (!start) goto exit;
 			sep[0] = first_c;
-			strcpy(szLine, start);
+			CPYCAT_ALLOC(start, 1);
 			sep = strstr(szLine, att_name);
 		}
 		sep = strchr(sep, '=');
 		if (!sep) {
 			state = 0;
-			strcpy(szLine, cur_line);
+			CPYCAT_ALLOC(cur_line, 1);
 			continue;
 		}
-		while ( (sep[0] != '\"') && (sep[0] != '\"') ) sep++;
+		while (sep[0] && (sep[0] != '\"') ) sep++;
+		if (!sep[0]) continue;
 		sep++;
+		while (sep[0] && strchr(" \n\r\t", sep[0]) ) sep++;
+		if (!sep[0]) continue;
 
 		/*found*/
 		if (!strncmp(sep, att_value, att_len)) {
@@ -1188,7 +1360,7 @@ retry:
 fetch_attr:
 			sep = strstr(szLine + 1, get_attr);
 			if (!sep) {
-				strcpy(szLine, cur_line);
+				CPYCAT_ALLOC(cur_line, 1);
 				continue;
 			}
 			sep += strlen(get_attr);
@@ -1202,13 +1374,17 @@ fetch_attr:
 			goto exit;
 		}
 		state = 0;
-		strcpy(szLine, sep);
+		CPYCAT_ALLOC(sep, 1);
 		goto retry;
 	}
 exit:
 	free(szLine);
+#ifdef NO_GZIP
+	fseek(parser->f_in, pos, SEEK_SET);
+#else
 	gzrewind(parser->gz_in);
 	gzseek(parser->gz_in, pos, SEEK_SET);
+#endif
 	return result;
 }
 
@@ -1270,7 +1446,8 @@ struct _tag_dom_parser
 };
 
 
-static void gf_xml_dom_node_del(GF_XMLNode *node)
+GF_EXPORT
+void gf_xml_dom_node_del(GF_XMLNode *node)
 {
 	if (node->attributes) {
 		while (gf_list_count(node->attributes)) {
@@ -1393,6 +1570,14 @@ void gf_xml_dom_del(GF_DOMParser *parser)
 	free(parser);
 }
 
+GF_EXPORT
+GF_XMLNode *gf_xml_dom_detach_root(GF_DOMParser *parser)
+{
+	GF_XMLNode *root = parser->root;
+	parser->root = NULL;
+	return root;
+}
+
 static void dom_on_progress(void *cbck, u32 done, u32 tot)
 {
 	GF_DOMParser *dom = (GF_DOMParser *)cbck;
@@ -1429,3 +1614,82 @@ u32 gf_xml_dom_get_line(GF_DOMParser *parser)
 	return gf_xml_sax_get_line(parser->parser);
 }
 
+
+static void gf_xml_dom_node_serialize(GF_XMLNode *node, Bool content_only, char **str, u32 *alloc_size, u32 *size)
+{
+	u32 i, count, vlen;
+	char *name;
+	
+#define SET_STRING(v)	\
+	vlen = strlen(v);	\
+	if (vlen+ (*size) >= (*alloc_size)) {	\
+		(*alloc_size) += 1024;	\
+		(*str) = realloc((*str), (*alloc_size));	\
+		(*str)[(*size)] = 0;	\
+	}	\
+	strcat((*str), v);	\
+	*size += vlen;	\
+
+	switch (node->type) {
+	case GF_XML_CDATA_TYPE:
+		SET_STRING("![CDATA[");
+		SET_STRING(node->name);
+		SET_STRING("]]>");
+		return;
+	case GF_XML_TEXT_TYPE:
+		name = node->name;
+		if ((name[0]=='\r') && (name[1]=='\n'))
+			name++;
+		SET_STRING(name);
+		return;
+	}
+
+	if (!content_only) {
+		SET_STRING("<");
+		if (node->ns) {
+			SET_STRING(node->ns);
+			SET_STRING(":");
+		}
+		SET_STRING(node->name);
+		SET_STRING(" ");
+		count = gf_list_count(node->attributes);
+		for (i=0; i<count; i++) {
+			GF_XMLAttribute *att = gf_list_get(node->attributes, i);
+			SET_STRING(att->name);
+			SET_STRING("=\"");
+			SET_STRING(att->value);
+			SET_STRING("\" ");
+		}
+	
+		if (!gf_list_count(node->content)) {
+			SET_STRING("/>");
+			return;
+		}
+		SET_STRING(">");
+	}
+
+	count = gf_list_count(node->content);
+	for (i=0; i<count; i++) {
+		GF_XMLNode *child = gf_list_get(node->content, i);
+		gf_xml_dom_node_serialize(child, 0, str, alloc_size, size);
+	}
+	if (!content_only) {
+		SET_STRING("</");
+		if (node->ns) {
+			SET_STRING(node->ns);
+			SET_STRING(":");
+		}
+		SET_STRING(node->name);
+		SET_STRING(">");
+	}
+}
+
+GF_EXPORT
+char *gf_xml_dom_serialize(GF_XMLNode *node, Bool content_only)
+{
+	u32 alloc_size = 0;
+	u32 size = 0;
+	char *str = NULL;
+	gf_xml_dom_node_serialize(node, content_only, &str, &alloc_size, &size);
+	return str;
+}
